Package: EpiModel
Version: 2.6.1
Date: 2026-05-12
Title: Mathematical Modeling of Infectious Disease Dynamics
Description: Tools for simulating mathematical models of infectious disease dynamics.
    Epidemic model classes include deterministic compartmental models, stochastic
    individual-contact models, and stochastic network models. Network models use the
    robust statistical methods of exponential-family random graph models (ERGMs)
    from the Statnet suite of software packages in R. Standard templates for epidemic
    modeling include SI, SIR, and SIS disease types. EpiModel features an API for
    extending these templates to address novel scientific research aims. Full
    methods for EpiModel are detailed in Jenness et al. (2018, <doi:10.18637/jss.v084.i08>).
Maintainer: Samuel Jenness <samuel.m.jenness@emory.edu>
Authors@R: c(person("Samuel", "Jenness",
    role=c("cre","aut"), email="samuel.m.jenness@emory.edu"),
    person("Steven M.","Goodreau",
    role=c("aut"), email="goodreau@uw.edu"),
    person("Martina", "Morris",
    role=c("aut"), email="morrism@uw.edu"),
    person("Adrien", "Le Guillou",
    role=c("aut"), email="contact@aleguillou.org"),
    person("Chad", "Klumb",
    role=c("aut"), email="cklumb@gmail.com"),
    person("Skye", "Bender-deMoll",
    role=c("ctb"), email="skyebend@uw.edu"))
License: GPL-3
URL: https://www.epimodel.org/, https://epimodel.github.io/EpiModel/
BugReports: https://github.com/EpiModel/EpiModel/issues/
Depends: R (>= 4.1), deSolve (>= 1.21), networkDynamic (>= 0.11.3),
        tergm (>= 4.2.1), statnet.common (>= 4.10.0)
Imports: future, future.apply, graphics, grDevices, stats, utils,
        collections, ergm (>= 4.7.1), network (>= 1.18.1),
        RColorBrewer, ape, lazyeval, ggplot2, tibble, methods, rlang,
        dplyr, coda, networkLite (>= 1.0.5)
Suggests: bslib (>= 0.6.0), DT, ergm.ego (>= 1.1.0), egor, knitr, ndtv,
        plotly (>= 4.10.0), rmarkdown, shiny, testthat, progressr,
        tidyr
VignetteBuilder: knitr
LinkingTo: Rcpp, ergm
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-12 20:22:31 UTC; sjennes
Author: Samuel Jenness [cre, aut],
  Steven M. Goodreau [aut],
  Martina Morris [aut],
  Adrien Le Guillou [aut],
  Chad Klumb [aut],
  Skye Bender-deMoll [ctb]
Repository: CRAN
Date/Publication: 2026-05-13 03:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-13 05:26:07 UTC; unix
Archs: EpiModel.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,dcm)
S3method(as.data.frame,icm)
S3method(as.data.frame,netdx)
S3method(as.data.frame,netsim)
S3method(as.network,transmat)
S3method(as_cumulative_edgelist,networkDynamic)
S3method(comp_plot,dcm)
S3method(comp_plot,icm)
S3method(comp_plot,netsim)
S3method(get_network,netdx)
S3method(get_network,netsim)
S3method(get_network,netsim_dat)
S3method(merge,icm)
S3method(merge,netsim)
S3method(plot,dcm)
S3method(plot,epi.data.frame)
S3method(plot,icm)
S3method(plot,netdx)
S3method(plot,netsim)
S3method(plot,transmat)
S3method(print,control.dcm)
S3method(print,control.icm)
S3method(print,control.net)
S3method(print,dcm)
S3method(print,disscoef)
S3method(print,icm)
S3method(print,init.dcm)
S3method(print,init.icm)
S3method(print,init.net)
S3method(print,netdx)
S3method(print,netest)
S3method(print,netsim)
S3method(print,param.dcm)
S3method(print,param.icm)
S3method(print,param.net)
S3method(set_network,netsim_dat)
S3method(summary,dcm)
S3method(summary,icm)
S3method(summary,netest)
S3method(summary,netsim)
S3method(truncate_sim,dcm)
S3method(truncate_sim,icm)
S3method(truncate_sim,netsim)
export(add_attr)
export(add_control)
export(add_epi)
export(add_init)
export(add_param)
export(add_vertices)
export(append_attr)
export(append_core_attr)
export(apportion_lr)
export(arrivals.2g.net)
export(arrivals.icm)
export(arrivals.icm.bip)
export(arrivals.net)
export(arrive_nodes)
export(as.epi.data.frame)
export(as.phylo.transmat)
export(as_cumulative_edgelist)
export(as_tibble_edgelist)
export(auto_update_attr)
export(brewer_ramp)
export(check_degdist_bal)
export(color_tea)
export(comp_plot)
export(control.dcm)
export(control.icm)
export(control.net)
export(copy_datattr_to_nwattr)
export(copy_nwattr_to_datattr)
export(create_dat_object)
export(create_scenario_list)
export(crosscheck.dcm)
export(crosscheck.icm)
export(crosscheck.net)
export(dcm)
export(dedup_cumulative_edgelist)
export(delete_attr)
export(delete_edges)
export(delete_vertices)
export(depart_nodes)
export(departures.2g.net)
export(departures.icm)
export(departures.icm.bip)
export(departures.net)
export(discord_edgelist)
export(dissolution_coefs)
export(edgelist_censor)
export(edges_correct)
export(epiweb)
export(generate_random_params)
export(geom_bands)
export(get_args)
export(get_attr)
export(get_attr_history)
export(get_attr_list)
export(get_attr_prop)
export(get_backward_reachable)
export(get_control)
export(get_control_list)
export(get_core_attributes)
export(get_cumulative_degree)
export(get_cumulative_edgelist)
export(get_cumulative_edgelists_df)
export(get_current_timestep)
export(get_degree)
export(get_discordant_edgelist)
export(get_edgelist)
export(get_edgelists_df)
export(get_epi)
export(get_epi_list)
export(get_formula_term_attr)
export(get_forward_reachable)
export(get_init)
export(get_init_list)
export(get_network)
export(get_network_attributes)
export(get_network_control)
export(get_network_term_attr)
export(get_nwparam)
export(get_nwstats)
export(get_param)
export(get_param_list)
export(get_param_set)
export(get_partners)
export(get_posit_ids)
export(get_sims)
export(get_transmat)
export(get_unique_ids)
export(get_vertex_attribute)
export(icm)
export(idgroup)
export(increment_timestep)
export(infection.2g.net)
export(infection.icm)
export(infection.icm.bip)
export(infection.net)
export(init.dcm)
export(init.icm)
export(init.net)
export(init_nets)
export(init_status.icm)
export(init_status.net)
export(initialize.icm)
export(initialize.net)
export(is.transmat)
export(is_active_posit_ids)
export(is_active_unique_ids)
export(make_restart_point)
export(mcat)
export(mod_SIR_1g_cl)
export(mod_SIR_1g_op)
export(mod_SIR_2g_cl)
export(mod_SIR_2g_op)
export(mod_SIS_1g_cl)
export(mod_SIS_1g_op)
export(mod_SIS_2g_cl)
export(mod_SIS_2g_op)
export(mod_SI_1g_cl)
export(mod_SI_1g_op)
export(mod_SI_2g_cl)
export(mod_SI_2g_op)
export(multilayer)
export(mutate_epi)
export(netdx)
export(netest)
export(netsim)
export(network_initialize)
export(nwupdate.net)
export(overwrite_attrs)
export(padded_vector)
export(param.dcm)
export(param.icm)
export(param.net)
export(param.net_from_table)
export(param.net_to_table)
export(param_random)
export(prevalence.icm)
export(prevalence.icm.bip)
export(prevalence.net)
export(process_out.net)
export(record_attr_history)
export(record_raw_object)
export(recovery.2g.net)
export(recovery.icm)
export(recovery.icm.bip)
export(recovery.net)
export(remove_node_attr)
export(resim_nets)
export(saveout.dcm)
export(saveout.icm)
export(saveout.net)
export(set_attr)
export(set_control)
export(set_current_timestep)
export(set_epi)
export(set_init)
export(set_network)
export(set_param)
export(set_transmat)
export(set_vertex_attribute)
export(sim_nets_t1)
export(simulate_dat)
export(ssample)
export(summary_nets)
export(test_icm)
export(test_net)
export(trim_netest)
export(truncate_sim)
export(update_cumulative_edgelist)
export(update_dissolution)
export(update_params)
export(use_scenario)
export(verbose.dcm)
export(verbose.icm)
export(verbose.net)
import(ergm)
import(ggplot2)
import(network)
import(networkDynamic)
import(networkLite)
import(tergm)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(ape,as.phylo)
importFrom(ape,collapse.singles)
importFrom(coda,effectiveSize)
importFrom(deSolve,dede)
importFrom(deSolve,ode)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,boxplot)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(lazyeval,lazy_dots)
importFrom(lazyeval,lazy_eval)
importFrom(methods,is)
importFrom(rlang,.data)
importFrom(statnet.common,.Deprecate_once)
importFrom(statnet.common,NVL)
importFrom(statnet.common,check.control.class)
importFrom(statnet.common,paste.and)
importFrom(statnet.common,set.control.class)
importFrom(statnet.common,trim_env)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,dgeom)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rgeom)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,simulate)
importFrom(stats,supsmu)
importFrom(stats,terms.formula)
importFrom(stats,update)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,tail)
useDynLib(EpiModel, .registration = TRUE)

##
## UI File for epiicm Shiny Application
##
## Run local: epiweb(class = "icm")
##

library(shiny)
library(bslib)
library(DT)
library(plotly)
library(EpiModel)

# -- Theme --
app_theme <- bs_theme(
  version = 5,
  bootswatch = "flatly",
  base_font = font_google("Atkinson Hyperlegible"),
  heading_font = font_google("Atkinson Hyperlegible"),
  "primary" = "#2C3E50",
  "success" = "#18BC9C",
  "info" = "#3498DB",
  "danger" = "#E74C3C"
)

# -- UI --
page_sidebar(
  title = div(
    class = "d-flex justify-content-between align-items-center w-100",
    span("EpiModel: Stochastic Individual Contact Models"),
    actionButton("runMod", "Run Model",
                 class = "btn-light btn-sm",
                 icon = icon("play"))
  ),
  theme = app_theme,

  # ===== Sidebar =====
  sidebar = sidebar(
    width = 320,
    open = "always",

    # Parameter accordion
    accordion(
      id = "params_accordion",
      open = c("Disease Type or Scenario", "Population & Time",
               "Epidemic Parameters"),

      # -- Disease Type or Scenario --
      accordion_panel(
        "Disease Type or Scenario",
        selectInput("modtype", "Disease Type",
                    choices = c("SI", "SIR", "SIS")),
        selectInput("preset", "Scenario Preset",
                    choices = c("Custom",
                                "Flu-like (SIR)",
                                "STI-like (SIS)",
                                "Measles-like (SIR)"))
      ),

      # -- Population & Time --
      accordion_panel(
        "Population & Time",
        numericInput("s.num", "Number Susceptible",
                     value = 500, min = 1, step = 50),
        numericInput("i.num", "Number Infected",
                     value = 1, min = 0, step = 1),
        conditionalPanel(
          "input.modtype == 'SIR'",
          numericInput("r.num", "Number Recovered",
                       value = 0, min = 0, step = 1)
        ),
        numericInput("nsteps", "Time Steps",
                     value = 300, min = 10, step = 50),
        numericInput("nsims", "Number of Simulations",
                     value = 5, min = 1, max = 50, step = 1),
        helpText("More simulations capture stochastic variability",
                 "but take longer to run.")
      ),

      # -- Epidemic Parameters --
      accordion_panel(
        "Epidemic Parameters",
        sliderInput("inf.prob", "Transmission Probability per Act",
                    min = 0, max = 1, value = 0.1, step = 0.01),
        sliderInput("act.rate", "Act Rate",
                    min = 0, max = 20, value = 0.5, step = 0.1),
        conditionalPanel(
          "input.modtype != 'SI'",
          sliderInput("rec.rate", "Recovery Rate",
                      min = 0, max = 1, value = 0.05, step = 0.01)
        )
      ),

      # -- Intervention --
      accordion_panel(
        "Intervention",
        checkboxInput("enable_intervention", "Enable Intervention",
                      value = FALSE),
        conditionalPanel(
          "input.enable_intervention",
          sliderInput("inter.eff", "Intervention Efficacy",
                      min = 0, max = 1, value = 0.5, step = 0.01),
          helpText("Proportional reduction in transmission probability."),
          numericInput("inter.start", "Intervention Start (time step)",
                       value = 100, min = 1, step = 10)
        )
      ),

      # -- Vital Dynamics --
      accordion_panel(
        "Vital Dynamics",
        checkboxInput("enable_vital", "Include Births & Deaths",
                      value = FALSE),
        conditionalPanel(
          "input.enable_vital",
          numericInput("a.rate", "Birth Rate (per person per time step)",
                       value = 0.001, min = 0, step = 0.001),
          numericInput("ds.rate", "Death Rate",
                       value = 0.001, min = 0, step = 0.001),
          checkboxInput("diff_death_rates",
                        "Different death rates by compartment",
                        value = FALSE),
          conditionalPanel(
            "input.diff_death_rates",
            numericInput("di.rate", "Death Rate, Infected",
                         value = 0.001, min = 0, step = 0.001),
            conditionalPanel(
              "input.modtype == 'SIR'",
              numericInput("dr.rate", "Death Rate, Recovered",
                           value = 0.001, min = 0, step = 0.001)
            )
          )
        )
      )
    ) # end accordion
  ), # end sidebar


  # ===== Main Panel =====

  # -- Main Plot Card --
  card(
    full_screen = TRUE,
    card_header(
      class = "d-flex justify-content-between align-items-center",
      "Epidemic Trajectory",
      div(
        class = "d-flex align-items-center gap-2",
        selectInput("compsel", NULL,
                    choices = c("Compartment Prevalence",
                                "Compartment Size",
                                "Disease Incidence"),
                    width = "330px"),
        downloadButton("dlMainPlot", "PDF",
                       class = "btn-sm btn-outline-secondary")
      )
    ),
    card_body(
      plotlyOutput("MainPlotly", height = "500px")
    )
  ),

  # Bottom row: Summary / Data / Guide tabs
  navset_card_tab(
    id = "main_tabs",
    height = "500px",

    nav_panel(
      "Summary",
      div(class = "p-3", style = "overflow-y: auto; height: 100%;",
          uiOutput("outSummary"))
    ),

    nav_panel(
      "Data",
      div(style = "overflow-y: auto; height: 100%;",
        div(
          class = "mb-3",
          fluidRow(
            column(4,
              selectInput("datasel", "Data View",
                          choices = c("Means", "Standard Deviations",
                                      "Individual Simulations"))
            ),
            column(4,
              conditionalPanel(
                "input.datasel == 'Individual Simulations'",
                uiOutput("simnoControl")
              )
            )
          )
        ),
        DTOutput("outData"),
        div(
          class = "mt-2",
          downloadButton("dlData", "Download CSV",
                         class = "btn-sm btn-outline-secondary")
        )
      )
    ),

    nav_panel(
      "Guide",
      div(
        class = "p-3",
        style = "max-width: 900px; overflow-y: auto; height: 100%;",

        # --- Overview ---
        h4("User Guide"),
        p("This application solves and visualizes",
          tags$strong("stochastic individual contact models (ICMs)"),
          "of infectious disease transmission using the",
          tags$a("EpiModel", href = "https://www.epimodel.org/",
                 target = "_blank"),
          "package for R. ICMs are agent-based microsimulation analogs to
          deterministic compartmental models. Each person in the population
          is represented as a discrete agent, and disease transmission is a
          stochastic (random) process that depends on the contact patterns
          and transmission probability per contact event."),

        p("Because ICMs are stochastic, running the model multiple times
          with the same parameters produces different outcomes. The
          application runs multiple simulations and displays their mean
          trajectory along with the interquartile range (IQR) to show the
          distribution of possible outcomes."),

        # --- Disease Types ---
        hr(),
        h5("Disease Types"),
        p("Three compartmental structures are available, each representing
          a different natural history of infection:"),
        tags$ul(
          tags$li(tags$strong("SI (Susceptible-Infected):"),
                  "Individuals move from susceptible to infected and remain
                  infected and infectious permanently. There is no recovery.
                  This is appropriate for chronic infections like HIV (without
                  treatment) or herpes."),
          tags$li(tags$strong("SIR (Susceptible-Infected-Recovered):"),
                  "Infected individuals recover and gain lasting immunity.
                  Once recovered, they cannot be re-infected. This applies to
                  many acute viral infections like measles, influenza, and
                  SARS-CoV-2 (as a simplification)."),
          tags$li(tags$strong("SIS (Susceptible-Infected-Susceptible):"),
                  "Infected individuals recover but return to the susceptible
                  state with no lasting immunity. They can be re-infected.
                  This is typical for many bacterial STIs like gonorrhea
                  and chlamydia.")
        ),

        # --- Epidemic Parameters ---
        hr(),
        h5("Epidemic Parameters"),
        tags$ul(
          tags$li(tags$strong("Transmission Probability per Act:"),
                  "The probability that an infection is transmitted during a
                  single contact (act) between an infected and a susceptible
                  individual. Ranges from 0 to 1."),
          tags$li(tags$strong("Act Rate:"),
                  "The average number of contact events (acts) per person
                  per unit time. Higher act rates mean more opportunities for
                  transmission."),
          tags$li(tags$strong("Recovery Rate:"),
                  "(SIR and SIS only.) The per-capita rate at which infected
                  individuals recover per time step. In a closed population,
                  the average duration of infection is 1 / recovery rate. With
                  vital dynamics enabled, the average duration in the infected
                  state is 1 / (recovery rate + departure rate for infecteds).")
        ),
        p("Together, these parameters determine the",
          tags$strong("basic reproduction number"),
          HTML("(R<sub>0</sub>),"),
          "the average number of secondary infections caused by one infected
          individual in a fully susceptible population:"),
        p(class = "text-center",
          style = "font-size: 1.1rem;",
          HTML("R<sub>0</sub> = (transmission probability &times;
                act rate) / recovery rate")),
        p(HTML("When R<sub>0</sub> > 1, the epidemic is expected to grow.
          When R<sub>0</sub> < 1, the infection is expected to die out.
          Because ICMs are stochastic, individual simulations may deviate
          from this expectation, especially with small populations.")),

        # --- Stochastic Variation ---
        hr(),
        h5("Stochastic Variation and Simulations"),
        p("Unlike deterministic compartmental models (DCMs), ICMs are
          stochastic. Each simulation may produce a different epidemic
          trajectory even with identical parameters, because individual
          transmission events are governed by random draws. This means
          outcomes like peak timing, peak size, and final epidemic size
          will vary across simulations."),
        p("The", tags$strong("Number of Simulations"), "control sets how
          many independent runs are performed. More simulations give a
          better picture of the range of possible outcomes. The plot shows:"),
        tags$ul(
          tags$li(tags$strong("Mean line:"), "The average trajectory across
                  all simulations, shown as a solid line."),
          tags$li(tags$strong("IQR band:"), "The interquartile range (25th
                  to 75th percentile) shown as a shaded ribbon. This captures
                  the middle 50% of simulation outcomes at each time step.")
        ),
        p("With only one simulation, only the individual trajectory is shown."),

        # --- Scenario Presets ---
        hr(),
        h5("Scenario Presets"),
        p("Three built-in presets configure all parameters to reasonable
          values for common infectious diseases. In all presets, each time
          step represents one day:"),
        tags$ul(
          tags$li(tags$strong("Flu-like (SIR):"),
                  "Low per-act transmission probability (0.03) with a high
                  contact rate (10 acts/day), reflecting airborne spread.
                  Average duration of infection of about 7 days (recovery
                  rate = 0.15). Initial population of 501."),
          tags$li(tags$strong("STI-like (SIS):"),
                  "Moderate transmission probability (0.2) with a low contact
                  rate (0.5 acts/day), reflecting sexual transmission.
                  Average duration of infection of about 100 days (recovery
                  rate = 0.01). Initial population of 510."),
          tags$li(tags$strong("Measles-like (SIR):"),
                  "High transmission probability (0.5) and moderate contact
                  rate (3 acts/day), producing a high",
                  HTML("R<sub>0</sub>"),
                  "of 15. Average duration of infection of about 10 days
                  (recovery rate = 0.1). Initial population of 501.")
        ),
        p("Select", tags$em("Custom"), "to set parameters manually."),

        # --- Interventions ---
        hr(),
        h5("Interventions"),
        p("When enabled, an intervention reduces the transmission probability
          by a proportional amount starting at a specified time step. For
          example, an efficacy of 0.5 cuts the transmission probability in
          half. This can represent public health measures such as vaccination
          campaigns, mask mandates, contact tracing, or behavioral
          interventions."),
        p("The intervention effect is shown as a vertical dashed line on the
          plot. Compare epidemic trajectories with and without the intervention
          to assess its impact."),

        # --- Vital Dynamics ---
        hr(),
        h5("Vital Dynamics"),
        p("By default, the population is closed (no births or deaths). Enabling
          vital dynamics adds a constant per-capita birth rate (new susceptibles
          entering the population) and per-capita death rates. By default, a
          single death rate is applied uniformly across all compartments.
          Checking", tags$em("Different death rates by compartment"),
          "allows setting separate death rates for susceptible, infected,
          and (for SIR models) recovered individuals. This is useful when
          disease-induced mortality differs from background mortality. Vital
          dynamics are important for modeling endemic equilibria over longer
          time horizons, where demographic turnover replenishes the susceptible
          pool."),

        # --- Reading the Output ---
        hr(),
        h5("Reading the Output"),
        p("The application provides three output views:"),
        tags$ul(
          tags$li(tags$strong("Plot:"),
                  "Interactive time-series plots (powered by plotly) showing
                  compartment sizes, prevalence, or disease incidence.
                  Mean lines and IQR bands summarize the stochastic variation
                  across simulations. Hover over any point to see exact values.
                  Use the plotly toolbar to zoom, pan, or download the plot
                  as a PNG."),
          tags$li(tags$strong("Summary:"),
                  HTML("Key epidemic metrics including R<sub>0</sub>, peak
                  infected count and timing, cumulative infections, incidence
                  rate, and (when applicable) attack rate, plus stochastic
                  variation statistics. Additional sections appear when
                  interventions or vital dynamics are enabled.")),
          tags$li(tags$strong("Data:"),
                  "The full simulation output as a searchable, sortable table.
                  Choose to view means across simulations, standard deviations,
                  or individual simulation values. Download the raw data as a
                  CSV file for further analysis.")
        ),

        tags$h6(class = "fw-semibold mt-3", "Incidence Metrics"),
        p("The summary tab reports two measures of disease incidence:"),
        tags$ul(
          tags$li(tags$strong("Incidence Rate:"),
                  "Total new infections divided by total susceptible
                  person-time (reported per 1,000 person-timesteps). This
                  measure is always shown and is valid for all model types,
                  including SIS models with re-infection and models with vital
                  dynamics, because it accounts for the changing size of the
                  susceptible population over time."),
          tags$li(tags$strong("Attack Rate:"),
                  "Total new infections divided by the initial number of
                  susceptibles. This is a simple proportion representing the
                  fraction of the original susceptible population that became
                  infected. It is only displayed for SI and SIR models without
                  vital dynamics, because it requires a closed population with
                  no re-infection. In SIS models (where individuals can be
                  re-infected) or models with births and deaths (where the
                  susceptible pool changes), the attack rate is not
                  well-defined and is therefore hidden.")
        ),

        # --- ICM vs DCM ---
        hr(),
        h5("ICM vs. DCM"),
        p("Individual contact models (ICMs) and deterministic compartmental
          models (DCMs) both track the flow of individuals between disease
          states, but they differ in key ways:"),
        tags$ul(
          tags$li(tags$strong("Stochasticity:"), "ICMs model each individual
                  agent and each contact event stochastically, while DCMs solve
                  a system of ordinary differential equations (ODEs)
                  deterministically."),
          tags$li(tags$strong("Population representation:"), "ICMs track
                  discrete individuals; DCMs track aggregate compartment sizes
                  that can take fractional values."),
          tags$li(tags$strong("Small populations:"), "ICMs naturally handle
                  small populations where integer effects and random extinction
                  matter. DCMs assume large, well-mixed populations."),
          tags$li(tags$strong("Computational cost:"), "ICMs are slower because
                  each agent is simulated individually. Use the Run Model
                  button to control when the simulation executes.")
        ),

        # --- Further Resources ---
        hr(),
        h5("Further Resources"),
        p("This application uses a subset of EpiModel's capabilities. The
          full R package supports deterministic compartmental models (DCMs)
          and stochastic network models built on exponential-family random
          graph models (ERGMs). For tutorials and documentation, visit the",
          tags$a("EpiModel website.", href = "https://www.epimodel.org/",
                 target = "_blank")),
        tags$ul(
          tags$li("Source code and bug reports:",
                  tags$a("GitHub",
                         href = "https://github.com/EpiModel/EpiModel",
                         target = "_blank")),
          tags$li("R package on CRAN:",
                  tags$a("EpiModel",
                         href = "https://cran.r-project.org/package=EpiModel",
                         target = "_blank")),
          tags$li("Course materials:",
                  tags$a("SISMID EpiModel Workshop",
                         href = "https://epimodel.github.io/sismid/",
                         target = "_blank"))
        ),

        # --- Citation ---
        hr(),
        h5("Citation"),
        p("Jenness SM, Goodreau SM, Morris M (2018).",
          tags$em("EpiModel: An R Package for Mathematical Modeling of
                   Infectious Disease over Networks."),
          "Journal of Statistical Software, 84(8), 1-47.",
          tags$a("doi:10.18637/jss.v084.i08",
                 href = "https://doi.org/10.18637/jss.v084.i08",
                 target = "_blank")),

        # --- Authors ---
        hr(),
        h5("Authors"),
        p("Samuel M. Jenness, Department of Epidemiology, Emory University"),
        p("Steven M. Goodreau, Department of Anthropology, University of Washington"),
        p("Martina Morris, Departments of Statistics and Sociology, University of Washington")
      )
    )
  ) # end navset_card_tab
) # end page_sidebar

##
## Server File for epiicm Shiny Application
##
## Run local: epiweb(class = "icm")
##

library(shiny)
library(EpiModel)

shinyServer(function(input, output, session) {

  # =========================================================================
  # Presets
  # =========================================================================
  # Track which modtype a preset expects, so we can distinguish
  # preset-driven modtype changes from manual ones
  preset_expected_modtype <- reactiveVal(NULL)

  observeEvent(input$preset, {
    if (input$preset == "Flu-like (SIR)") {
      preset_expected_modtype("SIR")
      updateSelectInput(session, "modtype", selected = "SIR")
      updateSliderInput(session, "inf.prob", value = 0.03)
      updateSliderInput(session, "act.rate", value = 10)
      updateSliderInput(session, "rec.rate", value = round(1 / 7, 3))
      updateNumericInput(session, "s.num", value = 500)
      updateNumericInput(session, "i.num", value = 1)
      updateNumericInput(session, "r.num", value = 0)
      updateNumericInput(session, "nsteps", value = 200)
      updateNumericInput(session, "nsims", value = 10)
      updateCheckboxInput(session, "enable_vital", value = FALSE)
      updateCheckboxInput(session, "enable_intervention", value = FALSE)
    } else if (input$preset == "STI-like (SIS)") {
      preset_expected_modtype("SIS")
      updateSelectInput(session, "modtype", selected = "SIS")
      updateSliderInput(session, "inf.prob", value = 0.2)
      updateSliderInput(session, "act.rate", value = 0.5)
      updateSliderInput(session, "rec.rate", value = 0.01)
      updateNumericInput(session, "s.num", value = 500)
      updateNumericInput(session, "i.num", value = 10)
      updateNumericInput(session, "nsteps", value = 500)
      updateNumericInput(session, "nsims", value = 10)
      updateCheckboxInput(session, "enable_vital", value = FALSE)
      updateCheckboxInput(session, "enable_intervention", value = FALSE)
    } else if (input$preset == "Measles-like (SIR)") {
      preset_expected_modtype("SIR")
      updateSelectInput(session, "modtype", selected = "SIR")
      updateSliderInput(session, "inf.prob", value = 0.5)
      updateSliderInput(session, "act.rate", value = 3)
      updateSliderInput(session, "rec.rate", value = 0.1)
      updateNumericInput(session, "s.num", value = 500)
      updateNumericInput(session, "i.num", value = 1)
      updateNumericInput(session, "r.num", value = 0)
      updateNumericInput(session, "nsteps", value = 100)
      updateNumericInput(session, "nsims", value = 10)
      updateCheckboxInput(session, "enable_vital", value = FALSE)
      updateCheckboxInput(session, "enable_intervention", value = FALSE)
    }
  })

  # Reset preset to "Custom" when user manually changes disease type.
  # If the modtype change matches what a preset just requested, consume
  # the expectation and leave the preset alone.
  observeEvent(input$modtype, {
    expected <- preset_expected_modtype()
    if (!is.null(expected) && input$modtype == expected) {
      preset_expected_modtype(NULL)
    } else if (input$preset != "Custom") {
      preset_expected_modtype(NULL)
      updateSelectInput(session, "preset", selected = "Custom")
    }
  })

  # =========================================================================
  # Dynamic UI: update plot type choices based on model type and vital
  # =========================================================================
  observe({
    type <- input$modtype
    vital <- isTRUE(input$enable_vital)
    choices <- c("Compartment Prevalence", "Compartment Size",
                 "Disease Incidence")
    if (type == "SIR") {
      choices <- c(choices, "Recovery Flow")
    }
    if (type == "SIS") {
      choices <- c(choices, "Re-susceptibility Flow")
    }
    if (vital) {
      choices <- c(choices, "Arrival Flow", "Departure Flows")
    }
    updateSelectInput(session, "compsel", choices = choices)
  })

  # Update death rate label based on differential checkbox
  observeEvent(input$diff_death_rates, {
    lbl <- if (isTRUE(input$diff_death_rates)) {
      "Death Rate, Susceptible"
    } else {
      "Death Rate"
    }
    updateNumericInput(session, "ds.rate", label = lbl)
  })


  # =========================================================================
  # Model state: reactiveValues to support both auto-run and button-run
  # =========================================================================
  rv <- reactiveValues(mod = NULL)

  # Shared function to build and run the model from current inputs
  build_and_run <- function() {
    s_num     <- input$s.num
    i_num     <- input$i.num
    r_num     <- input$r.num
    nsteps    <- input$nsteps
    nsims     <- input$nsims
    inf_prob  <- input$inf.prob
    act_rate  <- input$act.rate
    rec_rate  <- input$rec.rate
    modtype   <- input$modtype

    # Build param
    p_args <- list(inf.prob = inf_prob, act.rate = act_rate)
    if (modtype != "SI") {
      p_args$rec.rate <- rec_rate
    }

    # Intervention
    if (isTRUE(input$enable_intervention)) {
      p_args$inter.eff <- input$inter.eff
      p_args$inter.start <- as.numeric(input$inter.start)
    }

    # Vital dynamics
    if (isTRUE(input$enable_vital)) {
      p_args$a.rate <- input$a.rate
      p_args$ds.rate <- input$ds.rate
      if (isTRUE(input$diff_death_rates)) {
        p_args$di.rate <- input$di.rate
        if (modtype == "SIR") {
          p_args$dr.rate <- input$dr.rate
        }
      } else {
        p_args$di.rate <- input$ds.rate
        if (modtype == "SIR") {
          p_args$dr.rate <- input$ds.rate
        }
      }
    }

    param <- do.call(param.icm, p_args)

    # Build init — as.numeric() is critical: Shiny numericInput returns
    # integer class, but init.icm internally filters with
    # sapply(init, class) == "numeric" which excludes integers
    i_args <- list(s.num = as.numeric(s_num), i.num = as.numeric(i_num))
    if (modtype == "SIR") {
      i_args$r.num <- as.numeric(r_num)
    }
    init <- do.call(init.icm, i_args)

    # Build control
    control <- control.icm(type = modtype, nsteps = nsteps,
                            nsims = nsims, verbose = FALSE)

    # Run model
    tryCatch({
      icm(param, init, control)
    }, error = function(e) {
      showNotification(paste("Model error:", e$message),
                       type = "error", duration = 10)
      NULL
    })
  }

  # Run on button click
  observeEvent(input$runMod, {
    withProgress(message = "Running simulations...", {
      rv$mod <- build_and_run()
    })
  })

  # Auto-run once on startup after inputs are initialized
  observe({
    req(input$modtype, input$s.num, input$i.num,
        input$inf.prob, input$act.rate)
    isolate({
      if (is.null(rv$mod)) {
        withProgress(message = "Running initial model...", {
          rv$mod <- build_and_run()
        })
      }
    })
  })


  # =========================================================================
  # Summary
  # =========================================================================
  output$outSummary <- renderUI({
    m <- rv$mod
    req(m)

    p <- m$param
    nsims <- m$control$nsims
    type <- m$control$type
    nsteps <- m$control$nsteps

    # Get mean data for summary statistics
    df_mean <- as.data.frame(m, out = "mean")

    # Helper for styled stat boxes (matching DCM app)
    stat_box <- function(label, value, color = "#2C3E50") {
      div(
        class = "d-inline-block text-center me-4 mb-2",
        style = "min-width: 100px;",
        div(style = paste0("font-size: 1.17rem; font-weight: 500; color: ",
                           color, ";"),
            value),
        div(class = "text-muted", style = "font-size: 0.81rem;", label)
      )
    }

    # --- R0 / Force of Infection ---
    has_vital <- !is.null(p$a.rate)
    if (type == "SI") {
      foi <- p$inf.prob * p$act.rate
      foi_label <- formatC(foi, format = "g", digits = 2)
      if (has_vital) {
        foi_desc <- paste0(
          "In SI models, all infections are permanent at the individual level.",
          " The force of infection (inf.prob \u00D7 act.rate) drives transmission.",
          " With vital dynamics, new susceptibles enter through arrivals,",
          " so prevalence approaches an endemic equilibrium rather than 100%.")
      } else {
        foi_desc <- paste0(
          "In SI models, all infections are permanent.",
          " The force of infection (inf.prob \u00D7 act.rate) determines",
          " how quickly the susceptible population is depleted.")
      }
      r0_section <- tagList(
        tags$h6(class = "text-uppercase fw-semibold mb-2", "Transmission"),
        stat_box("Force of Infection", foi_label, "#E74C3C"),
        p(class = "text-muted", style = "font-size: 0.81rem;", foi_desc)
      )
    } else {
      removal <- p$rec.rate
      if (!is.null(p$di.rate)) removal <- removal + p$di.rate
      r0 <- (p$inf.prob * p$act.rate) / removal
      if (r0 > 1) {
        r0_interp <- "greater than 1, so the infection is expected to spread."
      } else if (r0 == 1) {
        r0_interp <- "exactly 1, so the epidemic is at a tipping point."
      } else {
        r0_interp <- "less than 1, so the infection is expected to die out."
      }
      r0_qual <- if (has_vital || type == "SIS") {
        paste0(" The effective reproduction number changes over time",
               " as the susceptible pool changes due to infection",
               if (has_vital) ", arrivals, and departures." else " and recovery.")
      } else {
        " As susceptibles are depleted, the effective reproduction number declines."
      }
      r0_section <- tagList(
        tags$h6(class = "text-uppercase fw-semibold mb-2", "Transmission"),
        stat_box(HTML("R<sub>0</sub>"), round(r0, 2),
                 if (r0 > 1) "#E74C3C" else "#18BC9C"),
        p(class = "text-muted", style = "font-size: 0.81rem;",
          HTML(paste0("R<sub>0</sub> is ", r0_interp,
                      " In a fully susceptible population, the first infected",
                      " person is expected to generate on average <strong>",
                      round(r0, 2), "</strong> new infections before",
                      " recovering.", r0_qual)))
      )
    }

    # --- Peak & Timeline (from mean trajectory) ---
    peak_i <- max(df_mean$i.num)
    peak_t <- df_mean$time[which.max(df_mean$i.num)]
    peak_prev <- peak_i / df_mean$num[which.max(df_mean$i.num)]
    final_prev <- df_mean$i.num[nrow(df_mean)] / df_mean$num[nrow(df_mean)]

    # Cumulative infections and incidence metrics (from mean)
    attack_rate_valid <- type %in% c("SI", "SIR") && !has_vital
    if ("si.flow" %in% names(df_mean)) {
      cum_inf <- sum(df_mean$si.flow, na.rm = TRUE)
      cum_inc_rate <- cum_inf / sum(df_mean$s.num, na.rm = TRUE)
      if (attack_rate_valid) {
        attack_rate <- cum_inf / df_mean$s.num[1]
      }
    } else {
      cum_inf <- NA
      cum_inc_rate <- NA
    }

    timeline_section <- tagList(
      hr(),
      tags$h6(class = "text-uppercase fw-semibold mb-2",
              if (nsims > 1) "Epidemic Timeline (Mean)" else "Epidemic Timeline"),
      div(
        class = "d-flex flex-wrap",
        stat_box("Peak Infected", format(round(peak_i), big.mark = ","),
                 "#E74C3C"),
        stat_box("Peak Time", paste0("t = ", peak_t), "#3498DB"),
        stat_box("Peak Prevalence", paste0(round(peak_prev * 100, 1), "%"),
                 "#E74C3C")
      ),
      if (!is.na(cum_inf)) {
        div(
          class = "d-flex flex-wrap",
          stat_box("Cumulative Infections",
                   format(round(cum_inf), big.mark = ","), "#2C3E50"),
          stat_box("Incidence Rate",
                   paste0(round(cum_inc_rate * 1000, 2),
                          " per 1,000 person-timesteps"),
                   "#2C3E50"),
          if (attack_rate_valid) {
            stat_box("Attack Rate",
                     paste0(round(attack_rate * 100, 1), "%"), "#2C3E50")
          }
        )
      },
      if (final_prev < 0.001 && type != "SI") {
        p(class = "text-muted", style = "font-size: 0.81rem;",
          "The epidemic has largely resolved by the end of the simulation.")
      } else if (type == "SI" && !has_vital) {
        p(class = "text-muted", style = "font-size: 0.81rem;",
          "In SI models without vital dynamics, prevalence increases monotonically toward 100%.")
      } else if (type == "SI" && has_vital) {
        p(class = "text-muted", style = "font-size: 0.81rem;",
          "In SI models with vital dynamics, prevalence approaches an endemic equilibrium.")
      } else {
        p(class = "text-muted", style = "font-size: 0.81rem;",
          paste0("At the end of the simulation (t = ", nsteps,
                 "), mean prevalence is ", round(final_prev * 100, 1), "%."))
      }
    )

    # --- Stochastic Variation ---
    if (nsims > 1) {
      peaks <- numeric(nsims)
      peak_times <- numeric(nsims)
      final_prevs <- numeric(nsims)
      for (s in seq_len(nsims)) {
        df_s <- as.data.frame(m, out = "vals", sim = s)
        peaks[s] <- max(df_s$i.num, na.rm = TRUE)
        peak_times[s] <- df_s$time[which.max(df_s$i.num)]
        final_prevs[s] <- df_s$i.num[nrow(df_s)] / df_s$num[nrow(df_s)]
      }

      stoch_section <- tagList(
        hr(),
        tags$h6(class = "text-uppercase fw-semibold mb-2",
                "Stochastic Variation"),
        div(
          class = "d-flex flex-wrap",
          stat_box("Simulations", nsims, "#2C3E50"),
          stat_box("Peak Infected Range",
                   paste0(format(round(min(peaks)), big.mark = ","),
                          " \u2013 ",
                          format(round(max(peaks)), big.mark = ",")),
                   "#E74C3C"),
          stat_box("Peak Time Range",
                   paste0(min(peak_times), " \u2013 ", max(peak_times)),
                   "#3498DB")
        ),
        p(class = "text-muted", style = "font-size: 0.81rem;",
          paste0("Across ", nsims, " simulations, peak infected ranged from ",
                 format(round(min(peaks)), big.mark = ","), " to ",
                 format(round(max(peaks)), big.mark = ","),
                 " (SD = ", round(sd(peaks), 1), "). ",
                 "Final prevalence ranged from ",
                 round(min(final_prevs) * 100, 1), "% to ",
                 round(max(final_prevs) * 100, 1), "% ",
                 "(SD = ", round(sd(final_prevs) * 100, 1), " pp)."))
      )
    } else {
      stoch_section <- NULL
    }

    # --- Intervention impact ---
    if (!is.null(p$inter.eff) && !is.null(p$inter.start)) {
      inter_section <- tagList(
        hr(),
        tags$h6(class = "text-uppercase fw-semibold mb-2", "Intervention"),
        p(style = "font-size: 0.81rem;",
          paste0("An intervention reducing transmission by ",
                 round(p$inter.eff * 100), "% begins at t = ",
                 p$inter.start, ". ",
                 "Effective transmission probability drops from ",
                 round(p$inf.prob, 3), " to ",
                 round(p$inf.prob * (1 - p$inter.eff), 3),
                 " per act."))
      )
    } else {
      inter_section <- NULL
    }

    # --- Vital dynamics ---
    if (!is.null(p$a.rate) && p$vital) {
      # Build death rate description
      rates_same <- (p$ds.rate == p$di.rate) &&
        (is.null(p$dr.rate) || p$ds.rate == p$dr.rate)
      if (rates_same) {
        death_desc <- paste0("Deaths occur at rate ", p$ds.rate,
                             " across all compartments.")
      } else {
        death_parts <- paste0("S: ", p$ds.rate, ", I: ", p$di.rate)
        if (!is.null(p$dr.rate)) {
          death_parts <- paste0(death_parts, ", R: ", p$dr.rate)
        }
        death_desc <- paste0("Death rates per compartment: ", death_parts, ".")
      }
      vital_section <- tagList(
        hr(),
        tags$h6(class = "text-uppercase fw-semibold mb-2", "Vital Dynamics"),
        p(class = "text-muted", style = "font-size: 0.81rem;",
          paste0("Births enter at rate ", p$a.rate,
                 " per person per time step. ",
                 death_desc, " ",
                 "Final mean population size: ",
                 format(round(df_mean$num[nrow(df_mean)]), big.mark = ","),
                 " (started at ",
                 format(round(df_mean$num[1]), big.mark = ","), ")."))
      )
    } else {
      vital_section <- NULL
    }

    # --- Model configuration ---
    config_section <- tagList(
      hr(),
      tags$h6(class = "text-uppercase fw-semibold mb-2",
              "Model Configuration"),
      tags$table(
        class = "table table-sm table-borderless",
        style = "max-width: 400px; font-size: 0.81rem;",
        tags$tr(tags$td(class = "text-muted", "Model Class"),
                tags$td(tags$strong("ICM (Stochastic)"))),
        tags$tr(tags$td(class = "text-muted", "Disease Type"),
                tags$td(tags$strong(type))),
        tags$tr(tags$td(class = "text-muted", "Initial Population"),
                tags$td(format(round(df_mean$num[1]), big.mark = ","))),
        tags$tr(tags$td(class = "text-muted", "Time Steps"),
                tags$td(nsteps)),
        tags$tr(tags$td(class = "text-muted", "Simulations"),
                tags$td(nsims)),
        tags$tr(tags$td(class = "text-muted", "Transmission Prob."),
                tags$td(p$inf.prob)),
        tags$tr(tags$td(class = "text-muted", "Act Rate"),
                tags$td(p$act.rate)),
        if (type != "SI") {
          tags$tr(tags$td(class = "text-muted", "Recovery Rate"),
                  tags$td(p$rec.rate))
        }
      )
    )

    # Assemble
    tagList(
      r0_section,
      timeline_section,
      stoch_section,
      inter_section,
      vital_section,
      config_section
    )
  })


  # =========================================================================
  # Main Plot (plotly — directly in UI, no renderUI wrapper)
  # =========================================================================
  output$MainPlotly <- plotly::renderPlotly({
    m <- rv$mod
    req(m)

    nsims <- m$control$nsims

    # Determine what to plot based on compsel
    if (input$compsel == "Compartment Prevalence") {
      y_cols <- c("s.num", "i.num")
      if (m$control$type == "SIR") y_cols <- c(y_cols, "r.num")
      is_prev <- TRUE
      y_label <- "Prevalence"
    } else if (input$compsel == "Compartment Size") {
      y_cols <- c("s.num", "i.num")
      if (m$control$type == "SIR") y_cols <- c(y_cols, "r.num")
      is_prev <- FALSE
      y_label <- "Number"
    } else if (input$compsel == "Disease Incidence") {
      y_cols <- "si.flow"
      is_prev <- FALSE
      y_label <- "New Infections"
    } else if (input$compsel == "Recovery Flow") {
      y_cols <- "ir.flow"
      is_prev <- FALSE
      y_label <- "Recoveries"
    } else if (input$compsel == "Re-susceptibility Flow") {
      y_cols <- "is.flow"
      is_prev <- FALSE
      y_label <- "Re-susceptible"
    } else if (input$compsel == "Arrival Flow") {
      y_cols <- "a.flow"
      is_prev <- FALSE
      y_label <- "Arrivals"
    } else if (input$compsel == "Departure Flows") {
      y_cols <- c("ds.flow", "di.flow")
      if (m$control$type == "SIR") y_cols <- c(y_cols, "dr.flow")
      is_prev <- FALSE
      y_label <- "Departures"
    } else {
      y_cols <- "i.num"
      is_prev <- FALSE
      y_label <- "Number"
    }

    # Nice labels and colors
    label_map <- c("s.num" = "Susceptible", "i.num" = "Infected",
                    "r.num" = "Recovered", "si.flow" = "S \u2192 I",
                    "ir.flow" = "I \u2192 R", "is.flow" = "I \u2192 S",
                    "a.flow" = "Arrivals",
                    "ds.flow" = "Deaths (S)", "di.flow" = "Deaths (I)",
                    "dr.flow" = "Deaths (R)")
    col_map <- c("s.num" = "#3498DB", "i.num" = "#E74C3C",
                  "r.num" = "#18BC9C", "si.flow" = "#E74C3C",
                  "ir.flow" = "#18BC9C", "is.flow" = "#3498DB",
                  "a.flow" = "#18BC9C",
                  "ds.flow" = "#3498DB", "di.flow" = "#E74C3C",
                  "dr.flow" = "#18BC9C")

    p <- plotly::plot_ly()

    if (nsims == 1) {
      # --- Single simulation: simple lines ---
      df <- as.data.frame(m, out = "vals")
      if (is_prev) {
        for (col in y_cols) {
          df[[paste0(col, ".prev")]] <- df[[col]] / df$num
        }
        plot_cols <- paste0(y_cols, ".prev")
      } else {
        plot_cols <- y_cols
      }
      for (i in seq_along(plot_cols)) {
        col <- plot_cols[i]
        orig <- y_cols[i]
        clr <- col_map[[orig]]
        lbl <- label_map[[orig]]
        p <- plotly::add_trace(p, x = df$time, y = df[[col]],
                               type = "scatter", mode = "lines",
                               name = lbl,
                               line = list(color = clr, width = 2.5))
      }
    } else {
      # --- Multiple simulations: mean lines + IQR ribbons ---
      df_mean <- as.data.frame(m, out = "mean")
      df_q25 <- as.data.frame(m, out = "qnt", qval = 0.25)
      df_q75 <- as.data.frame(m, out = "qnt", qval = 0.75)

      if (is_prev) {
        for (col in y_cols) {
          df_mean[[paste0(col, ".prev")]] <- df_mean[[col]] / df_mean$num
          df_q25[[paste0(col, ".prev")]] <- df_q25[[col]] / df_q25$num
          df_q75[[paste0(col, ".prev")]] <- df_q75[[col]] / df_q75$num
        }
        plot_cols <- paste0(y_cols, ".prev")
      } else {
        plot_cols <- y_cols
      }

      for (i in seq_along(plot_cols)) {
        col <- plot_cols[i]
        orig <- y_cols[i]
        clr <- col_map[[orig]]
        lbl <- label_map[[orig]]

        # IQR ribbon: lower bound (invisible line)
        p <- plotly::add_trace(p,
          x = df_q25$time, y = df_q25[[col]],
          type = "scatter", mode = "lines",
          line = list(color = "transparent"),
          showlegend = FALSE, name = paste0(lbl, " Q25"),
          hoverinfo = "skip")

        # IQR ribbon: upper bound (filled to lower)
        p <- plotly::add_trace(p,
          x = df_q75$time, y = df_q75[[col]],
          type = "scatter", mode = "lines",
          fill = "tonexty",
          fillcolor = paste0(clr, "33"),
          line = list(color = "transparent"),
          showlegend = FALSE, name = paste0(lbl, " IQR"),
          hoverinfo = "skip")

        # Mean line
        p <- plotly::add_trace(p,
          x = df_mean$time, y = df_mean[[col]],
          type = "scatter", mode = "lines",
          name = lbl,
          line = list(color = clr, width = 2.5))
      }
    }

    # Intervention line
    if (isTRUE(input$enable_intervention) &&
        !is.null(m$param$inter.eff)) {
      p <- plotly::layout(
        p,
        shapes = list(
          list(type = "line",
               x0 = m$param$inter.start, x1 = m$param$inter.start,
               y0 = 0, y1 = 1, yref = "paper",
               line = list(color = "#95A5A6", width = 1.5, dash = "dash"))
        ),
        annotations = list(
          list(x = m$param$inter.start, y = 1, yref = "paper",
               text = "intervention", showarrow = FALSE,
               font = list(color = "#95A5A6", size = 11))
        )
      )
    }

    p <- plotly::layout(p,
                        xaxis = list(title = "Time"),
                        yaxis = list(title = y_label),
                        legend = list(x = 1.02, y = 1),
                        margin = list(t = 30))
    p <- plotly::config(p, displayModeBar = TRUE,
                        modeBarButtonsToRemove = c("lasso2d", "select2d"))
    p
  })


  # =========================================================================
  # Download main plot (base R plot.icm for PDF)
  # =========================================================================
  output$dlMainPlot <- downloadHandler(
    filename = "EpiModel_ICM_Plot.pdf",
    content = function(file) {
      m <- rv$mod
      req(m)
      pdf(file = file, height = 8, width = 12)
      par(mar = c(3.5, 3.5, 1.2, 1), mgp = c(2.1, 1, 0))
      if (input$compsel == "Compartment Prevalence") {
        plot(m, popfrac = TRUE, mean.line = TRUE, sim.lines = FALSE,
             qnts = 0.5, lwd = 3, legend = TRUE,
             leg.cex = 1.1, main = "")
      } else if (input$compsel == "Compartment Size") {
        plot(m, popfrac = FALSE, mean.line = TRUE, sim.lines = FALSE,
             qnts = 0.5, lwd = 3, legend = TRUE,
             leg.cex = 1.1, main = "")
      } else if (input$compsel == "Disease Incidence") {
        plot(m, y = "si.flow", popfrac = FALSE, mean.line = TRUE,
             sim.lines = FALSE, qnts = 0.5, lwd = 3,
             legend = TRUE, leg.cex = 1.1, main = "")
      } else {
        plot(m, popfrac = TRUE, mean.line = TRUE, sim.lines = FALSE,
             qnts = 0.5, lwd = 3, legend = TRUE,
             leg.cex = 1.1, main = "")
      }
      dev.off()
    }
  )


  # =========================================================================
  # Data Table
  # =========================================================================

  # Simulation number control for individual sim view
  output$simnoControl <- renderUI({
    m <- rv$mod
    req(m)
    maxsims <- m$control$nsims
    numericInput("datasim", "Simulation Number",
                 value = 1, min = 1, max = maxsims, step = 1)
  })

  output$outData <- DT::renderDT({
    m <- rv$mod
    req(m)
    if (input$datasel == "Means") {
      round(as.data.frame(m, out = "mean"), 2)
    } else if (input$datasel == "Standard Deviations") {
      round(as.data.frame(m, out = "sd"), 2)
    } else if (input$datasel == "Individual Simulations") {
      sim_num <- if (!is.null(input$datasim)) max(1, input$datasim) else 1
      as.data.frame(m, out = "vals", sim = sim_num)
    }
  }, options = list(lengthMenu = c(10, 25, 50, 100),
                    pageLength = 10,
                    scrollX = TRUE))

  output$dlData <- downloadHandler(
    filename = "EpiModel_ICM_Data.csv",
    content = function(file) {
      m <- rv$mod
      if (input$datasel == "Means") {
        write.csv(as.data.frame(m, out = "mean"), file, row.names = FALSE)
      } else if (input$datasel == "Standard Deviations") {
        write.csv(as.data.frame(m, out = "sd"), file, row.names = FALSE)
      } else if (input$datasel == "Individual Simulations") {
        sim_num <- if (!is.null(input$datasim)) max(1, input$datasim) else 1
        write.csv(as.data.frame(m, out = "vals", sim = sim_num),
                  file, row.names = FALSE)
      }
    }
  )

})

## ----echo=FALSE---------------------------------------------------------------
vers <- packageVersion("EpiModel")
year <- format(Sys.time(), "%Y")

## ----step1, results = "hide", message = FALSE---------------------------------
library(EpiModel)
set.seed(12345)

nw <- network_initialize(n = 500)

formation <- ~edges + concurrent + degrange(from = 4)
target.stats <- c(175, 110, 0)
coef.diss <- dissolution_coefs(
  dissolution = ~offset(edges),
  duration = 50   # mean partnership duration in time steps
)

est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

## ----step2, results = "hide"--------------------------------------------------
dx <- netdx(est, nsims = 5, nsteps = 500, verbose = FALSE)

## ----step2_plot1, fig.width = 8-----------------------------------------------
plot(dx)

## ----step2_plot2--------------------------------------------------------------
plot(dx, type = "duration")

## ----step3, results = "hide"--------------------------------------------------
param <- param.net(inf.prob = 0.4, act.rate = 2, rec.rate = 0.05)
init <- init.net(i.num = 10)
control <- control.net(type = "SIS", nsims = 5, nsteps = 500, verbose = FALSE)

sim <- netsim(est, param, init, control)

## ----step4_epi, fig.width = 8-------------------------------------------------
plot(sim, sim.lines = TRUE, sim.alpha = 0.3,
     mean.lwd = 3, qnts = 0.5, main = "SIS Epidemic on a Dynamic Network")

## ----step4_prev---------------------------------------------------------------
plot(sim, y = "i.num", popfrac = TRUE, sim.lines = TRUE, sim.alpha = 0.3,
     mean.lwd = 3, qnts = 0.5, main = "Infection Prevalence",
     ylab = "Prevalence", legend = FALSE)

## ----step4_summary------------------------------------------------------------
summary(sim, at = 500)

## ----step4_df-----------------------------------------------------------------
d <- as.data.frame(sim)
head(d)

## ----step4_transmat-----------------------------------------------------------
# Run a separate SI model seeded with a single infection for a cleaner tree
set.seed(42)
param2 <- param.net(inf.prob = 0.5, act.rate = 2)
init2 <- init.net(i.num = 5)
control2 <- control.net(type = "SI", nsims = 1, nsteps = 60, verbose = FALSE)
sim2 <- netsim(est, param2, init2, control2)

tm <- get_transmat(sim2)
head(tm)

## ----step4_tree, fig.width = 8, fig.height = 5--------------------------------
tmPhylo <- as.phylo.transmat(tm)
par(mar = c(2, 1, 2, 1))
plot(tmPhylo, show.node.label = TRUE, root.edge = TRUE, cex = 0.5,
     main = "Transmission Tree")

## ----extension, eval = FALSE--------------------------------------------------
# # Example: a custom progression module for an SEIR model
# progress_module <- function(dat, at) {
#   active <- get_attr(dat, "active")
#   status <- get_attr(dat, "status")
# 
#   # E -> I transition
#   ids.EtoI <- which(active == 1 & status == "e" & runif(length(status)) < 0.1)
#   status[ids.EtoI] <- "i"
# 
#   # I -> R transition
#   ids.ItoR <- which(active == 1 & status == "i" & runif(length(status)) < 0.02)
#   status[ids.ItoR] <- "r"
# 
#   dat <- set_attr(dat, "status", status)
#   dat <- set_epi(dat, "ei.flow", at, length(ids.EtoI))
#   dat <- set_epi(dat, "ir.flow", at, length(ids.ItoR))
#   return(dat)
# }

## ----eval=FALSE---------------------------------------------------------------
# help(package = "EpiModel")

## ----echo = FALSE, include = FALSE--------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, echo = FALSE, include = FALSE-------------------------------------
library(EpiModel)

## -----------------------------------------------------------------------------
get_core_attributes()

## ----accessing-attr, eval = FALSE---------------------------------------------
# active <- get_attr(dat, "active")

## ----modyfing-attr, eval = FALSE----------------------------------------------
# aging_module <- function(dat, at) {
# 
#   # Extract current attribute
#   age <- get_attr(dat, "age")
# 
#   # Aging process
#   new_age <- age + 1
# 
#   # Output updated attributes
#   dat <- set_attr(dat, "age", new_age)
# 
#   return(dat)
# }

## ----recording_attr_hist, eval = FALSE----------------------------------------
# viral_load_logger_module <- function(dat, at) {
# 
#   # Run every 10 time steps
#   if (at %% 10 == 0) {
# 
#     # Attributes
#     status <- get_attr(dat, "status")
#     viral_load <- get_attr(dat, "viral_load")
# 
#     infected <- which(status == "i")
# 
#     dat <- record_attr_history(
#       dat, at,
#       "viral_load",
#       infected,
#       viral_load[infected]
#     )
#   }
# 
#   return(dat)
# }

## ----access_attr_hist, eval = FALSE-------------------------------------------
# sim <- netsim(est, param, init, control)
# attr_history <- get_attr_history(sim)

## ----get_attr_ex, eval = FALSE------------------------------------------------
# get_attr_history(sim)
# 
# # $viral_load
# #    sim step attribute    uids values
# # 1    1   10 viral_load   1001   2000
# # 2    1   10 viral_load   1002   1878
# # 3    1   20 viral_load   1001   1500
# # 4    1   20 viral_load   1002    300
# # ...
# #
# # $status
# #    sim step attribute  uids  values
# # 1    1   22 status     1001       i
# # 2    1   64 status     1002       i
# # 3    1  110 status     1001       r
# # 4    1  220 status     1002       r
# # ...

## ----epi-in-modules, eval = FALSE---------------------------------------------
# aging_track_module <- function(dat, at) {
# 
#   # Attributes
#   age <- get_attr(dat, "age")
# 
#   # Aging process
#   new_age <- age + 1
# 
#   # Calculate summary statistics
#   mean_age <- mean(new_age)
#   prev_mean_age <- get_epi(dat, at - 1, "mean_age")
#   age_change <- mean_age - prev_mean_age
# 
#   # Update nodal attributes
#   dat <- set_attr(dat, "age", new_age)
# 
#   # Update epidemic trackers
#   dat <- set_epi(dat, "mean_age", at, mean_age)
#   dat <- set_epi(dat, "age_change", at, age_change)
# 
#   return(dat)
# }

## ----eval = FALSE-------------------------------------------------------------
# sim <- netsim(est, param, init, control)
# 
# # Raw per-simulation data
# as.data.frame(sim)
# 
# # Means across simulations
# as.data.frame(sim, out = "mean")
# 
# # Plot and summary
# plot(sim)
# summary(sim, at = 50)

## ----eval = FALSE-------------------------------------------------------------
# # Add incidence rate and prevalence
# sim <- mutate_epi(sim, prev = i.num / num)
# sim <- mutate_epi(sim, ir = (si.flow / s.num) * 100)
# 
# # These new variables appear in the data.frame output
# as.data.frame(sim)

## ----tracker-epiby, results = "hide", message = FALSE-------------------------
set.seed(0)

nw <- network_initialize(n = 100)
nw <- set_vertex_attribute(nw, "risk", rep(0:1, each = 50))

est <- netest(
  nw, formation = ~edges + nodefactor("risk"),
  target.stats = c(40, 20),
  coef.diss = dissolution_coefs(~offset(edges), 20),  # duration = 20
  verbose = FALSE
)

param <- param.net(inf.prob = 0.3, act.rate = 1)
init <- init.net(i.num = 5)
control <- control.net(
  type = "SI", nsims = 1, nsteps = 50,
  epi.by = "risk",
  verbose = FALSE
)

sim <- netsim(est, param, init, control)

## -----------------------------------------------------------------------------
d <- as.data.frame(sim)
names(d)

## ----tracker-example----------------------------------------------------------
epi_s_num <- function(dat) {
  needed_attributes <- c("status")
  output <- with(get_attr_list(dat, needed_attributes), {
    sum(status == "s", na.rm = TRUE)
  })
  return(output)
}

## ----tracker-commented--------------------------------------------------------
epi_prop_infected <- function(dat) {
  needed_attributes <- c("status", "active")
  output <- with(get_attr_list(dat, needed_attributes), {
    pop <- active == 1
    infected <- status == "i"
    sum(infected & pop, na.rm = TRUE) / sum(pop, na.rm = TRUE)
  })
  return(output)
}

## ----tracker-list-------------------------------------------------------------
some.trackers <- list(
  prop_infected = epi_prop_infected,
  s_num         = epi_s_num
)

control <- control.net(
  type = "SI",
  nsims = 1,
  nsteps = 50,
  verbose = FALSE,
  .tracker.list = some.trackers
)

param <- param.net(
  inf.prob = 0.3,
  act.rate = 0.1
)

nw <- network_initialize(n = 50)
nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
est <- netest(
  nw,
  formation = ~edges,
  target.stats = 25,
  coef.diss = dissolution_coefs(~offset(edges), 10, 0),  # duration = 10, closed population
  verbose = FALSE
)

init <- init.net(i.num = 10)
sim <- netsim(est, param, init, control)

d <- as.data.frame(sim)

knitr::kable(tail(d, n = 15))

## ----record-raw, eval = FALSE-------------------------------------------------
# introspect_module <- function(dat, at) {
#   age <- get_attr(dat, "age")
# 
#   if (mean(age, na.rm = TRUE) > 50) {
#     obj <- data.frame(
#         age = age,
#         status = get_attr(dat, "status")
#     )
#     dat <- record_raw_object(dat, at, "old pop", obj)
#   }
# 
#   return(dat)
# }

## ----record-raw-access, eval = FALSE------------------------------------------
# sim <- netsim(est, param, init, control)
# 
# # Access raw records for simulation 1
# # Returns a nested list keyed by label, then time step
# raw <- sim[["raw.records"]][[1]]
# 
# # Access the "old pop" data.frame recorded at step 75
# raw[["old pop"]][[75]]

## ----echo = FALSE, include = FALSE--------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, echo = FALSE, include = FALSE-------------------------------------
library(EpiModel)

## ----create-el_cuml, eval = FALSE---------------------------------------------
# dat <- update_cumulative_edgelist(dat, network, truncate = Inf)

## ----el_cuml-snippet, eval = FALSE--------------------------------------------
# for (n_network in seq_along(dat$run$nw)) {
#   dat <- update_cumulative_edgelist(dat, n_network, truncate = 100)
# }

## ----get_el_cuml, eval = FALSE------------------------------------------------
# el_cuml <- get_cumulative_edgelist(dat, network)

## ----get_el_cuml_dfs, eval = FALSE--------------------------------------------
# el_cumls <- get_cumulative_edgelists_df(dat, networks = NULL)

## ----get_partners, eval = FALSE-----------------------------------------------
# partner_list <- get_partners(
#     dat,
#     index_posit_ids,
#     networks = NULL,
#     truncate = Inf,
#     only.active.nodes = FALSE
# )

## ----echo = FALSE, include = FALSE--------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----echo=FALSE, message=FALSE------------------------------------------------
library(EpiModel)

## ----setup, results = "hide"--------------------------------------------------
set.seed(10)

nw <- network_initialize(n = 200)
est <- netest(nw,
  formation = ~edges, target.stats = 60,
  coef.diss = dissolution_coefs(~offset(edges), 10, 0),  # duration = 10, closed population
  verbose = FALSE
)

param <- param.net(inf.prob = 0.9, rec.rate = 0.01, act.rate = 2)
init <- init.net(i.num = 10)
control <- control.net(type = "SIS", nsims = 1, nsteps = 250, verbose = FALSE)

## ----scenarios_setup----------------------------------------------------------
library(dplyr)

scenarios.df <- tribble(
  ~.scenario.id, ~.at, ~inf.prob, ~rec.rate,
  "base", 0, 0.9, 0.01,
  "initial_change", 0, 0.2, 0.01,
  "multiple_changes", 0, 0.1, 0.04,
  "multiple_changes", 100, 0.9, 0.01,
  "multiple_changes", 200, 0.1, 0.1
)

knitr::kable(scenarios.df)

## ----scenarios_list-----------------------------------------------------------
scenarios.list <- create_scenario_list(scenarios.df)
str(scenarios.list, max.level = 2)

## ----simulations--------------------------------------------------------------
# List to hold simulation results
d_list <- vector(mode = "list", length = length(scenarios.list))
names(d_list) <- names(scenarios.list)

for (scenario in scenarios.list) {
  print(scenario$id)
  sc.param <- use_scenario(param, scenario)
  sim <- netsim(est, sc.param, init, control)
  d_sim <- as.data.frame(sim)
  d_sim[["scenario"]] <- scenario$id
  d_list[[scenario$id]] <- d_sim
}

## ----plotting, fig.width = 8--------------------------------------------------
plot(d_list$base$time, d_list$base$i.num,
     type = "l", col = 1, lwd = 2, ylim = c(0, 250),
     xlab = "Time Step", ylab = "Number Infected")
lines(d_list$initial_change$time, d_list$initial_change$i.num,
      type = "l", col = 2, lwd = 2)
lines(d_list$multiple_changes$time, d_list$multiple_changes$i.num,
      type = "l", col = 3, lwd = 2)
abline(v = c(100, 200), lty = 2)
legend("topleft", legend = names(d_list),
       col = 1:3, lwd = 2, cex = 0.9, bty = "n")

## ----multi_param--------------------------------------------------------------
scenarios.df <- tribble(
  ~.scenario.id, ~.at, ~hiv.test.rate_1, ~hiv.test.rate_2, ~hiv.test.rate_3,
  "base", 0, 0.001, 0.001, 0.001,
  "initial_change", 0, 0.002, 0.001, 0.002,
  "multiple_changes", 0, 0.002, 0.001, 0.002,
  "multiple_changes", 100, 0.004, 0.001, 0.004,
  "multiple_changes", 200, 0.008, 0.001, 0.008
)
knitr::kable(scenarios.df)

## ----eval = FALSE-------------------------------------------------------------
# # CSV
# write.csv(scenarios.df, "scenarios.csv", row.names = FALSE)
# scenarios.df <- read.csv("scenarios.csv")
# 
# # RDS
# saveRDS(scenarios.df, "scenarios.rds")
# scenarios.df <- readRDS("scenarios.rds")

## ----param_table--------------------------------------------------------------
df_params <- tribble(
  ~param, ~value, ~type,
  "hiv.test.rate_1",  "0.003",        "numeric",
  "hiv.test.rate_2",  "0.102",        "numeric",
  "hiv.test.rate_3",  "0.492",        "numeric",
  "prep.require.lnt", "TRUE",         "logical",
  "group_1",          "first",        "character",
  "group_2",          "second",       "character"
)
knitr::kable(df_params)

## ----param_table2-------------------------------------------------------------
param <- param.net(data.frame.params = df_params,
                   other.param = c(5, 10), act.rate = 1)
param

## ----model--------------------------------------------------------------------
nw <- network_initialize(n = 50)

est <- netest(
  nw, formation = ~edges,
  target.stats = 25,
  coef.diss = dissolution_coefs(~offset(edges), 10, 0),  # duration = 10, closed population
  verbose = FALSE
)

param <- param.net(
  inf.prob = 0.3,
  act.rate = 0.5,
  dummy.param = 4,
  dummy.strat.param = c(0, 1)
)

init <- init.net(i.num = 10)
control <- control.net(type = "SI", nsims = 1, nsteps = 5, verbose = FALSE)
mod <- netsim(est, param, init, control)
mod

## ----generators---------------------------------------------------------------
my.randoms <- list(
  act.rate = param_random(c(0.25, 0.5, 0.75)),
  dummy.param = function() rbeta(1, 1, 2),
  dummy.strat.param = function() {
    c(rnorm(1, 0.05, 0.01),
      rnorm(1, 0.15, 0.03))
  }
)

param <- param.net(
  inf.prob = 0.3,
  random.params = my.randoms
)

param

## ----generators_run-----------------------------------------------------------
control <- control.net(type = "SI", nsims = 3, nsteps = 5, verbose = FALSE)
mod <- netsim(est, param, init, control)

mod

## ----generators_inspect-------------------------------------------------------
all.params <- get_param_set(mod)
all.params

## -----------------------------------------------------------------------------
epi <- as.data.frame(mod)
left_join(epi, all.params)

## ----set_df-------------------------------------------------------------------
n <- 5

related.param <- data.frame(
  dummy.param = rbeta(n, 1, 2)
)

related.param$dummy.strat.param_1 <- related.param$dummy.param + rnorm(n)
related.param$dummy.strat.param_2 <- related.param$dummy.param * 2 + rnorm(n)

related.param

## ----set_param----------------------------------------------------------------
my.randoms <- list(
  act.rate = param_random(c(0.25, 0.5, 0.75)),
  param.random.set = related.param
)

param <- param.net(
  inf.prob = 0.3,
  random.params = my.randoms
)

param

## ----set_run------------------------------------------------------------------
control <- control.net(type = "SI", nsims = 3, nsteps = 5, verbose = FALSE)
mod <- netsim(est, param, init, control)

mod

## -----------------------------------------------------------------------------
related.param
get_param_set(mod)

## ----updater-example----------------------------------------------------------
list(
  at = 10,
  param = list(
    inf.prob = 0.3,
    act.rate = 0.5
  )
)

## ----updater-list-------------------------------------------------------------
list.of.updaters <- list(
  list(
    at = 100,
    param = list(
      inf.prob = 0.3,
      act.rate = 0.3
    )
  ),
  list(
    at = 125,
    param = list(
      inf.prob = 0.01
    )
  )
)

## ----updater-module-----------------------------------------------------------
param <- param.net(
 inf.prob = 0.1,
 act.rate = 0.1,
 .param.updater.list = list.of.updaters
)
init <- init.net(i.num = 10)
control <- control.net(
 type = "SI",
 nsims = 1,
 nsteps = 200,
 verbose = FALSE
)

## ----updater-module2----------------------------------------------------------
nw <- network_initialize(n = 100)
est <- netest(
  nw,
  formation = ~edges,
  target.stats = 50,
  coef.diss = dissolution_coefs(~offset(edges), 10, 0),  # duration = 10, closed population
  verbose = FALSE
)
mod <- netsim(est, param, init, control)

## -----------------------------------------------------------------------------
plot(mod, mean.smooth = FALSE)
abline(v = c(100, 125), lty = 2, col = "grey50")

## ----updater-verbose----------------------------------------------------------
list(
  at = 10,
  param = list(
    inf.prob = 0.3,
    act.rate = 0.5
  ),
  verbose = TRUE
)

## ----updater-function---------------------------------------------------------
list(
  at = 10,
  param = list(
    inf.prob = function(x) plogis(qlogis(x) + log(2)),
    act.rate = 0.5
  )
)

## ----control-updater-example--------------------------------------------------
list.of.updaters <- list(
  list(
    at = 100,
    control = list(
      resimulate.network = FALSE
    )
  ),
  list(
    at = 125,
    control = list(
      verbose = FALSE
    )
  )
)

## -----------------------------------------------------------------------------
control <- control.net(
 type = "SI",
 nsims = 1,
 nsteps = 200,
 verbose = TRUE,
 .control.updater.list = list.of.updaters
)

##
## Server File for epidcm Shiny Application
##
## Run local: epiweb(class = "dcm")
##

library(shiny)
library(EpiModel)

shinyServer(function(input, output, session) {

  # =========================================================================
  # Presets
  # =========================================================================
  # Track which modtype a preset expects, so we can distinguish

  # preset-driven modtype changes from manual ones
  preset_expected_modtype <- reactiveVal(NULL)

  observeEvent(input$preset, {
    if (input$preset == "Flu-like (SIR)") {
      preset_expected_modtype("SIR")
      updateSelectInput(session, "modtype", selected = "SIR")
      updateSliderInput(session, "inf.prob", value = 0.03)
      updateSliderInput(session, "act.rate", value = 10)
      updateSliderInput(session, "rec.rate", value = round(1 / 7, 3))
      updateNumericInput(session, "s.num", value = 10000)
      updateNumericInput(session, "i.num", value = 1)
      updateNumericInput(session, "r.num", value = 0)
      updateNumericInput(session, "nsteps", value = 200)
      updateCheckboxInput(session, "enable_vital", value = FALSE)
      updateCheckboxInput(session, "enable_intervention", value = FALSE)
      updateCheckboxInput(session, "enable_sensitivity", value = FALSE)
    } else if (input$preset == "STI-like (SIS)") {
      preset_expected_modtype("SIS")
      updateSelectInput(session, "modtype", selected = "SIS")
      updateSliderInput(session, "inf.prob", value = 0.2)
      updateSliderInput(session, "act.rate", value = 0.5)
      updateSliderInput(session, "rec.rate", value = 0.01)
      updateNumericInput(session, "s.num", value = 1000)
      updateNumericInput(session, "i.num", value = 10)
      updateNumericInput(session, "nsteps", value = 500)
      updateCheckboxInput(session, "enable_vital", value = FALSE)
      updateCheckboxInput(session, "enable_intervention", value = FALSE)
      updateCheckboxInput(session, "enable_sensitivity", value = FALSE)
    } else if (input$preset == "Measles-like (SIR)") {
      preset_expected_modtype("SIR")
      updateSelectInput(session, "modtype", selected = "SIR")
      updateSliderInput(session, "inf.prob", value = 0.5)
      updateSliderInput(session, "act.rate", value = 3)
      updateSliderInput(session, "rec.rate", value = 0.1)
      updateNumericInput(session, "s.num", value = 10000)
      updateNumericInput(session, "i.num", value = 1)
      updateNumericInput(session, "r.num", value = 0)
      updateNumericInput(session, "nsteps", value = 100)
      updateCheckboxInput(session, "enable_vital", value = FALSE)
      updateCheckboxInput(session, "enable_intervention", value = FALSE)
      updateCheckboxInput(session, "enable_sensitivity", value = FALSE)
    }
  })

  # Reset preset to "Custom" when user manually changes disease type.
  # If the modtype change matches what a preset just requested, consume

  # the expectation and leave the preset alone.
  observeEvent(input$modtype, {
    expected <- preset_expected_modtype()
    if (!is.null(expected) && input$modtype == expected) {
      preset_expected_modtype(NULL)
    } else if (input$preset != "Custom") {
      preset_expected_modtype(NULL)
      updateSelectInput(session, "preset", selected = "Custom")
    }
  })

  # =========================================================================
  # Dynamic UI updates
  # =========================================================================

  # Update flow choices based on model type and vital dynamics
  observe({
    type <- input$modtype
    vital <- isTRUE(input$enable_vital)
    choices <- c("Compartment Prevalence", "Compartment Size",
                 "Disease Incidence")
    if (type == "SIR") {
      choices <- c(choices, "Recovery Flow")
    }
    if (type == "SIS") {
      choices <- c(choices, "Re-susceptibility Flow")
    }
    if (vital) {
      choices <- c(choices, "Arrival Flow", "Departure Flows")
    }
    updateSelectInput(session, "compsel", choices = choices)
  })

  # Update sensitivity parameter choices based on model type
  observe({
    type <- input$modtype
    choices <- c("inf.prob", "act.rate")
    if (type != "SI") {
      choices <- c(choices, "rec.rate")
    }
    updateSelectInput(session, "sens_param", choices = choices)
  })

  # Update death rate label based on differential checkbox
  observeEvent(input$diff_death_rates, {
    lbl <- if (isTRUE(input$diff_death_rates)) {
      "Death Rate, Susceptible"
    } else {
      "Death Rate"
    }
    updateNumericInput(session, "ds.rate", label = lbl)
  })


  # =========================================================================
  # Model state: reactiveValues to support button-triggered runs
  # =========================================================================
  rv <- reactiveValues(mod = NULL)

  # Shared function to build and run the model from current inputs
  build_and_run <- function() {
    # Build param
    p <- list(inf.prob = input$inf.prob,
              act.rate = input$act.rate)

    # Recovery rate
    if (input$modtype != "SI") {
      p$rec.rate <- input$rec.rate
    }

    # Intervention
    if (isTRUE(input$enable_intervention)) {
      p$inter.eff <- input$inter.eff
      p$inter.start <- input$inter.start
    }

    # Vital dynamics
    if (isTRUE(input$enable_vital)) {
      p$a.rate <- input$a.rate
      p$ds.rate <- input$ds.rate
      if (isTRUE(input$diff_death_rates)) {
        p$di.rate <- input$di.rate
        if (input$modtype == "SIR") {
          p$dr.rate <- input$dr.rate
        }
      } else {
        p$di.rate <- input$ds.rate
        if (input$modtype == "SIR") {
          p$dr.rate <- input$ds.rate
        }
      }
    }

    # Sensitivity analysis: override selected param with vector
    if (isTRUE(input$enable_sensitivity)) {
      nruns <- as.integer(input$sens_nruns)
      sens_vals <- seq(input$sens_min, input$sens_max, length.out = nruns)
      p[[input$sens_param]] <- sens_vals
    }

    param <- do.call(param.dcm, p)

    # Build init
    i_args <- list(s.num = input$s.num, i.num = input$i.num)
    if (input$modtype == "SIR") {
      i_args$r.num <- input$r.num
    }
    init <- do.call(init.dcm, i_args)

    # Build control
    control <- control.dcm(type = input$modtype,
                           nsteps = input$nsteps,
                           dt = 1,
                           odemethod = "lsoda",
                           verbose = FALSE)

    # Run model
    tryCatch({
      dcm(param, init, control)
    }, error = function(e) {
      showNotification(paste("Model error:", e$message),
                       type = "error", duration = 8)
      NULL
    })
  }

  # Run on button click
  observeEvent(input$runMod, {
    rv$mod <- build_and_run()
  })

  # Auto-run once on startup after inputs are initialized
  observe({
    req(input$modtype, input$s.num, input$i.num,
        input$inf.prob, input$act.rate)
    isolate({
      if (is.null(rv$mod)) {
        rv$mod <- build_and_run()
      }
    })
  })


  # =========================================================================
  # Main Plot (plotly — directly in UI, no renderUI wrapper)
  # =========================================================================
  output$MainPlotly <- plotly::renderPlotly({
    m <- rv$mod
    req(m)

    df <- as.data.frame(m)
    nruns <- m$control$nruns
    type <- m$control$type

    # Determine what to plot
    if (input$compsel == "Compartment Prevalence") {
      y_cols <- c("s.num", "i.num")
      if (type == "SIR") y_cols <- c(y_cols, "r.num")
      y_cols <- intersect(y_cols, names(df))
      for (col in y_cols) {
        df[[paste0(col, ".prev")]] <- df[[col]] / df$num
      }
      plot_cols <- paste0(y_cols, ".prev")
      y_label <- "Prevalence"
    } else if (input$compsel == "Compartment Size") {
      plot_cols <- c("s.num", "i.num")
      if (type == "SIR") plot_cols <- c(plot_cols, "r.num")
      plot_cols <- intersect(plot_cols, names(df))
      y_label <- "Number"
    } else if (input$compsel == "Disease Incidence") {
      plot_cols <- "si.flow"
      y_label <- "New Infections"
    } else if (input$compsel == "Recovery Flow") {
      plot_cols <- "ir.flow"
      y_label <- "Recoveries"
    } else if (input$compsel == "Re-susceptibility Flow") {
      plot_cols <- "is.flow"
      y_label <- "Re-susceptible"
    } else if (input$compsel == "Arrival Flow") {
      plot_cols <- "a.flow"
      y_label <- "Arrivals"
    } else if (input$compsel == "Departure Flows") {
      plot_cols <- intersect(c("ds.flow", "di.flow", "dr.flow"), names(df))
      y_label <- "Departures"
    } else {
      plot_cols <- "i.num"
      y_label <- "Number"
    }

    # Guard: ensure requested columns exist in the model output
    plot_cols <- intersect(plot_cols, names(df))
    req(length(plot_cols) > 0)

    # Color and label maps
    col_map <- c("s.num" = "#3498DB", "s.num.prev" = "#3498DB",
                  "i.num" = "#E74C3C", "i.num.prev" = "#E74C3C",
                  "r.num" = "#18BC9C", "r.num.prev" = "#18BC9C",
                  "si.flow" = "#E74C3C", "ir.flow" = "#18BC9C",
                  "is.flow" = "#3498DB",
                  "a.flow" = "#18BC9C",
                  "ds.flow" = "#3498DB", "di.flow" = "#E74C3C",
                  "dr.flow" = "#18BC9C")
    label_map <- c("s.num" = "Susceptible", "s.num.prev" = "Susceptible",
                    "i.num" = "Infected", "i.num.prev" = "Infected",
                    "r.num" = "Recovered", "r.num.prev" = "Recovered",
                    "si.flow" = "S \u2192 I",
                    "ir.flow" = "I \u2192 R", "is.flow" = "I \u2192 S",
                    "a.flow" = "Arrivals",
                    "ds.flow" = "Deaths (S)", "di.flow" = "Deaths (I)",
                    "dr.flow" = "Deaths (R)")

    # Build plot
    if (nruns == 1) {
      p <- plotly::plot_ly()
      for (col in plot_cols) {
        clr <- ifelse(col %in% names(col_map), col_map[[col]], "#2C3E50")
        lbl <- ifelse(col %in% names(label_map), label_map[[col]], col)
        p <- plotly::add_trace(p, x = df$time, y = df[[col]],
                               type = "scatter", mode = "lines",
                               name = lbl,
                               line = list(color = clr, width = 2.5))
      }
    } else {
        p <- plotly::plot_ly()

        # Get sensitivity parameter values for legend labels
        sens_param_name <- input$sens_param
        sens_vals <- m$param[[sens_param_name]]
        run_labels <- paste0(sens_param_name, " = ", round(sens_vals, 4))

        # For multi-run, reduce to primary variable to match base R plot.dcm
        is_prev <- input$compsel == "Compartment Prevalence"
        if (is_prev) {
          plot_cols <- "i.num.prev"
          y_label <- "Prevalence (Infected)"
        } else if (input$compsel == "Compartment Size") {
          plot_cols <- "i.num"
          y_label <- "Number Infected"
        }

        palette <- grDevices::colorRampPalette(c("#3498DB", "#E74C3C"))(nruns)
        for (i in seq_len(nruns)) {
          run_df <- as.data.frame(m, run = i)
          if (is_prev) {
            run_df[["i.num.prev"]] <- run_df$i.num / run_df$num
          }
          for (col in plot_cols) {
            p <- plotly::add_trace(
              p, x = run_df$time, y = run_df[[col]],
              type = "scatter", mode = "lines",
              name = run_labels[i],
              line = list(color = palette[i], width = 2)
            )
          }
        }
      }

      # Intervention line
      if (!is.null(m$param$inter.eff) && !is.null(m$param$inter.start)) {
        p <- plotly::layout(
          p,
          shapes = list(
            list(type = "line",
                 x0 = m$param$inter.start, x1 = m$param$inter.start,
                 y0 = 0, y1 = 1, yref = "paper",
                 line = list(color = "#95A5A6", width = 1.5, dash = "dash"))
          ),
          annotations = list(
            list(x = m$param$inter.start, y = 1, yref = "paper",
                 text = "intervention", showarrow = FALSE,
                 font = list(color = "#95A5A6", size = 11))
          )
        )
      }

      p <- plotly::layout(p,
                          xaxis = list(title = "Time"),
                          yaxis = list(title = y_label),
                          legend = list(x = 1.02, y = 1),
                          margin = list(t = 30))
      p <- plotly::config(p, displayModeBar = TRUE,
                          modeBarButtonsToRemove = c("lasso2d", "select2d"))
      p
    })


  # =========================================================================
  # Download main plot
  # =========================================================================
  output$dlMainPlot <- downloadHandler(
    filename = "EpiModel_DCM_Plot.pdf",
    content = function(file) {
      m <- rv$mod
      req(m)
      pdf(file = file, height = 8, width = 12)
      par(mar = c(3.5, 3.5, 1.2, 1), mgp = c(2.1, 1, 0))
      if (input$compsel == "Compartment Prevalence") {
        plot(m, popfrac = TRUE, lwd = 3, legend = "full",
             leg.cex = 1.1, main = "")
      } else if (input$compsel == "Compartment Size") {
        plot(m, popfrac = FALSE, lwd = 3, legend = "full",
             leg.cex = 1.1, main = "")
      } else if (input$compsel == "Disease Incidence") {
        plot(m, y = "si.flow", popfrac = FALSE, lwd = 3,
             legend = "n", main = "")
      } else {
        plot(m, popfrac = TRUE, lwd = 3, legend = "full",
             leg.cex = 1.1, main = "")
      }
      dev.off()
    }
  )


  # =========================================================================
  # Summary
  # =========================================================================
  output$outSummary <- renderUI({
    m <- rv$mod
    req(m)

    p <- m$param
    nruns <- m$control$nruns
    type <- m$control$type
    df <- as.data.frame(m, run = 1)
    nsteps <- m$control$nsteps

    # Helper for styled stat boxes
    stat_box <- function(label, value, color = "#2C3E50") {
      div(
        class = "d-inline-block text-center me-4 mb-2",
        style = "min-width: 100px;",
        div(style = paste0("font-size: 1.17rem; font-weight: 500; color: ", color, ";"),
            value),
        div(class = "text-muted", style = "font-size: 0.81rem;", label)
      )
    }

    # --- R0 / Force of Infection ---
    has_vital <- !is.null(p$a.rate)
    if (type == "SI") {
      foi <- p$inf.prob[1] * p$act.rate[1]
      foi_label <- formatC(foi, format = "g", digits = 2)
      if (has_vital) {
        foi_desc <- paste0(
          "In SI models, all infections are permanent at the individual level.",
          " The force of infection (inf.prob \u00D7 act.rate) drives transmission.",
          " With vital dynamics, new susceptibles enter through arrivals,",
          " so prevalence approaches an endemic equilibrium rather than 100%.")
      } else {
        foi_desc <- paste0(
          "In SI models, all infections are permanent.",
          " The force of infection (inf.prob \u00D7 act.rate) determines",
          " how quickly the susceptible population is depleted.")
      }
      r0_section <- tagList(
        tags$h6(class = "text-uppercase fw-semibold mb-2", "Transmission"),
        stat_box("Force of Infection", foi_label, "#E74C3C"),
        p(class = "text-muted", style = "font-size: 0.81rem;", foi_desc)
      )
    } else {
      removal <- p$rec.rate[1]
      if (!is.null(p$di.rate)) removal <- removal + p$di.rate[1]
      r0 <- (p$inf.prob[1] * p$act.rate[1]) / removal
      if (r0 > 1) {
        r0_interp <- "greater than 1, so the infection will spread in the population."
      } else if (r0 == 1) {
        r0_interp <- "exactly 1, so the epidemic is at a tipping point."
      } else {
        r0_interp <- "less than 1, so the infection will die out."
      }
      r0_qual <- if (has_vital || type == "SIS") {
        paste0(" The effective reproduction number changes over time",
               " as the susceptible pool changes due to infection",
               if (has_vital) ", arrivals, and departures." else " and recovery.")
      } else {
        " As susceptibles are depleted, the effective reproduction number declines."
      }
      r0_section <- tagList(
        tags$h6(class = "text-uppercase fw-semibold mb-2", "Transmission"),
        stat_box(HTML("R<sub>0</sub>"), round(r0, 2),
                 if (r0 > 1) "#E74C3C" else "#18BC9C"),
        p(class = "text-muted", style = "font-size: 0.81rem;",
          HTML(paste0("R<sub>0</sub> is ", r0_interp,
                      " In a fully susceptible population, the first infected",
                      " person would generate on average <strong>",
                      round(r0, 2), "</strong> new infections before",
                      " recovering.", r0_qual)))
      )
    }

    # --- Peak & Timeline (run 1) ---
    peak_i <- max(df$i.num)
    peak_t <- df$time[which.max(df$i.num)]
    peak_prev <- peak_i / df$num[which.max(df$i.num)]
    final_prev <- df$i.num[nrow(df)] / df$num[nrow(df)]

    # Cumulative infections and incidence metrics
    attack_rate_valid <- type %in% c("SI", "SIR") && !has_vital
    if ("si.flow" %in% names(df)) {
      cum_inf <- sum(df$si.flow, na.rm = TRUE)
      cum_inc_rate <- cum_inf / sum(df$s.num, na.rm = TRUE)
      if (attack_rate_valid) {
        attack_rate <- cum_inf / df$s.num[1]
      }
    } else {
      cum_inf <- NA
      cum_inc_rate <- NA
    }

    timeline_section <- tagList(
      hr(),
      tags$h6(class = "text-uppercase fw-semibold mb-2", "Epidemic Timeline"),
      div(
        class = "d-flex flex-wrap",
        stat_box("Peak Infected", format(round(peak_i), big.mark = ","),
                 "#E74C3C"),
        stat_box("Peak Time", paste0("t = ", peak_t), "#3498DB"),
        stat_box("Peak Prevalence", paste0(round(peak_prev * 100, 1), "%"),
                 "#E74C3C")
      ),
      if (!is.na(cum_inf)) {
        div(
          class = "d-flex flex-wrap",
          stat_box("Cumulative Infections",
                   format(round(cum_inf), big.mark = ","), "#2C3E50"),
          stat_box("Incidence Rate",
                   paste0(round(cum_inc_rate * 1000, 2),
                          " per 1,000 person-timesteps"),
                   "#2C3E50"),
          if (attack_rate_valid) {
            stat_box("Attack Rate",
                     paste0(round(attack_rate * 100, 1), "%"), "#2C3E50")
          }
        )
      },
      if (final_prev < 0.001 && type != "SI") {
        p(class = "text-muted", style = "font-size: 0.81rem;",
          "The epidemic has largely resolved by the end of the simulation.")
      } else if (type == "SI" && !has_vital) {
        p(class = "text-muted", style = "font-size: 0.81rem;",
          "In SI models without vital dynamics, prevalence increases monotonically toward 100%.")
      } else if (type == "SI" && has_vital) {
        p(class = "text-muted", style = "font-size: 0.81rem;",
          "In SI models with vital dynamics, prevalence approaches an endemic equilibrium.")
      } else {
        p(class = "text-muted", style = "font-size: 0.81rem;",
          paste0("At the end of the simulation (t = ", nsteps,
                 "), prevalence is ", round(final_prev * 100, 1), "%."))
      }
    )

    # --- Intervention impact ---
    if (!is.null(p$inter.eff) && !is.null(p$inter.start)) {
      inter_section <- tagList(
        hr(),
        tags$h6(class = "text-uppercase fw-semibold mb-2", "Intervention"),
        p(style = "font-size: 0.81rem;",
          paste0("An intervention reducing transmission by ",
                 round(p$inter.eff * 100), "% begins at t = ",
                 p$inter.start, ". ",
                 "Effective transmission probability drops from ",
                 round(p$inf.prob[1], 3), " to ",
                 round(p$inf.prob[1] * (1 - p$inter.eff), 3),
                 " per act."))
      )
    } else {
      inter_section <- NULL
    }

    # --- Vital dynamics ---
    if (!is.null(p$a.rate) && p$vital) {
      # Build death rate description
      rates_same <- (p$ds.rate == p$di.rate) &&
        (is.null(p$dr.rate) || p$ds.rate == p$dr.rate)
      if (rates_same) {
        death_desc <- paste0("Deaths occur at rate ", p$ds.rate,
                             " across all compartments.")
      } else {
        death_parts <- paste0("S: ", p$ds.rate, ", I: ", p$di.rate)
        if (!is.null(p$dr.rate)) {
          death_parts <- paste0(death_parts, ", R: ", p$dr.rate)
        }
        death_desc <- paste0("Death rates per compartment: ", death_parts, ".")
      }
      vital_section <- tagList(
        hr(),
        tags$h6(class = "text-uppercase fw-semibold mb-2", "Vital Dynamics"),
        p(class = "text-muted", style = "font-size: 0.81rem;",
          paste0("Births enter at rate ", p$a.rate,
                 " per person per time step. ",
                 death_desc, " ",
                 "Final population size: ",
                 format(round(df$num[nrow(df)]), big.mark = ","),
                 " (started at ",
                 format(round(df$num[1]), big.mark = ","), ")."))
      )
    } else {
      vital_section <- NULL
    }

    # --- Sensitivity note ---
    if (nruns > 1) {
      sens_section <- tagList(
        hr(),
        tags$h6(class = "text-uppercase fw-semibold mb-2", "Sensitivity Analysis"),
        p(class = "text-muted", style = "font-size: 0.81rem;",
          paste0(nruns, " model runs varying ", input$sens_param,
                 " from ", min(p[[input$sens_param]]),
                 " to ", max(p[[input$sens_param]]), ".",
                 " Statistics above are shown for run 1."))
      )
    } else {
      sens_section <- NULL
    }

    # --- Model configuration ---
    config_section <- tagList(
      hr(),
      tags$h6(class = "text-uppercase fw-semibold mb-2", "Model Configuration"),
      tags$table(
        class = "table table-sm table-borderless",
        style = "max-width: 400px; font-size: 0.81rem;",
        tags$tr(tags$td(class = "text-muted", "Model Class"),
                tags$td(tags$strong("DCM (Deterministic)"))),
        tags$tr(tags$td(class = "text-muted", "Disease Type"),
                tags$td(tags$strong(type))),
        tags$tr(tags$td(class = "text-muted", "Initial Population"),
                tags$td(format(round(df$num[1]), big.mark = ","))),
        tags$tr(tags$td(class = "text-muted", "Time Steps"),
                tags$td(nsteps)),
        tags$tr(tags$td(class = "text-muted", "Transmission Prob."),
                tags$td(p$inf.prob[1])),
        tags$tr(tags$td(class = "text-muted", "Act Rate"),
                tags$td(p$act.rate[1])),
        if (type != "SI") {
          tags$tr(tags$td(class = "text-muted", "Recovery Rate"),
                  tags$td(p$rec.rate[1]))
        }
      )
    )

    # Assemble
    tagList(
      r0_section,
      timeline_section,
      inter_section,
      vital_section,
      sens_section,
      config_section
    )
  })


  # =========================================================================
  # Data Table
  # =========================================================================
  output$outData <- DT::renderDT({
    m <- rv$mod
    req(m)
    round(as.data.frame(m), 2)
  }, options = list(lengthMenu = c(10, 25, 50, 100),
                    pageLength = 10,
                    scrollX = TRUE))

  output$dlData <- downloadHandler(
    filename = "EpiModel_DCM_Data.csv",
    content = function(file) {
      write.csv(as.data.frame(rv$mod), file, row.names = FALSE)
    }
  )

})

#' @title Departures: netsim Module
#'
#' @description This function simulates departure for use in [netsim]
#'        simulations.
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @seealso [netsim()]
#'
#' @export
#' @keywords netMod internal
#'
departures.net <- function(dat, at) {

  # Conditions --------------------------------------------------------------
  vital <- get_param(dat, "vital")
  if (vital == FALSE) {
    return(dat)
  }

  type <- get_control(dat, "type", override.null.error = TRUE)
  type <- if (is.null(type)) "None" else type

  active <- get_attr(dat, "active")
  status <- get_attr(dat, "status")
  exitTime <- get_attr(dat, "exitTime")
  rates.sus <- get_param(dat, "ds.rate")
  rates.inf <- get_param(dat, "di.rate")
  if (type == "SIR") {
    rates.rec <- get_param(dat, "dr.rate")
  }

  # Susceptible departures --------------------------------------------------
  nDepartures.sus <- 0
  idsElig.sus <- which(active == 1 & status == "s")
  nElig.sus <- length(idsElig.sus)
  if (nElig.sus > 0) {
    vecDepartures.sus <- which(rbinom(nElig.sus, 1, rates.sus) == 1)
    if (length(vecDepartures.sus) > 0) {
      idsDpt.sus <- idsElig.sus[vecDepartures.sus]
      nDepartures.sus <- length(idsDpt.sus)
      active[idsDpt.sus] <- 0
      exitTime[idsDpt.sus] <- at
    }
  }

  # Infected departures -----------------------------------------------------
  nDepartures.inf <- 0
  idsElig.inf <- which(active == 1 & status == "i")
  nElig.inf <- length(idsElig.inf)
  if (nElig.inf > 0) {
    vecDepartures.inf <- which(rbinom(nElig.inf, 1, rates.inf) == 1)
    if (length(vecDepartures.inf) > 0) {
      idsDpt.inf <- idsElig.inf[vecDepartures.inf]
      nDepartures.inf <- length(idsDpt.inf)
      active[idsDpt.inf] <- 0
      exitTime[idsDpt.inf] <- at
    }
  }

  # Recovered departures ----------------------------------------------------
  if (type == "SIR") {
    nDepartures.rec <- 0
    idsElig.rec <- which(active == 1 & status == "r")
    nElig.rec <- length(idsElig.rec)
    if (nElig.rec > 0) {
      vecDepartures.rec <- which(rbinom(nElig.rec, 1, rates.rec) == 1)
      if (length(vecDepartures.rec) > 0) {
        idsDpt.rec <- idsElig.rec[vecDepartures.rec]
        nDepartures.rec <- length(idsDpt.rec)
        active[idsDpt.rec] <- 0
        exitTime[idsDpt.rec] <- at
      }
    }
  }

  # Output ------------------------------------------------------------------

  dat <- set_attr(dat, "active", active)
  dat <- set_attr(dat, "exitTime", exitTime)

  dat <- set_epi(dat, "ds.flow", at, nDepartures.sus)
  dat <- set_epi(dat, "di.flow", at, nDepartures.inf)
  if (type == "SIR") {
    dat <- set_epi(dat, "dr.flow", at, nDepartures.rec)
  }
  return(dat)
}


#' @title Arrivals: netsim Module
#'
#' @description This function simulates new arrivals into the network
#'   for use in [netsim()] simulations.
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @seealso [netsim()]
#'
#' @export
#' @keywords netMod internal
#'
arrivals.net <- function(dat, at) {

  # Conditions --------------------------------------------------------------
  vital <- get_param(dat, "vital")
  if (vital == FALSE) {
    return(dat)
  }

  # Variables ---------------------------------------------------------------
  a.rate <- get_param(dat, "a.rate")
  index <- at - 1
  nOld <- get_epi(dat, "num", index)
  nArrivals <- 0

  # Add Nodes ---------------------------------------------------------------
  if (nOld > 0) {
    nArrivals <- rbinom(1, nOld, a.rate)
    if (nArrivals > 0) {
      dat <- append_core_attr(dat, at, nArrivals)
      dat <- append_attr(dat, "status", "s", nArrivals)
      dat <- append_attr(dat, "infTime", NA, nArrivals)
    }
  }

  # Output ------------------------------------------------------------------
  dat <- set_epi(dat, "a.flow", at, nArrivals)

  return(dat)
}


#' @title Departures: netsim Module
#'
#' @description This function simulates departure for use in [netsim]
#'        simulations.
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @seealso [netsim()]
#'
#' @export
#' @keywords netMod internal
#'
departures.2g.net <- function(dat, at) {

  # Conditions --------------------------------------------------------------
  vital <- get_param(dat, "vital")
  if (vital == FALSE) {
    return(dat)
  }

  # Variables ---------------------------------------------------------------

  type <- get_control(dat, "type", override.null.error = TRUE)
  type <- if (is.null(type)) "None" else type


  active <- get_attr(dat, "active")
  status <- get_attr(dat, "status")
  exitTime <- get_attr(dat, "exitTime")
  group <- get_attr(dat, "group")
  rates.sus <- c(get_param(dat, "ds.rate"), get_param(dat, "ds.rate.g2"))
  rates.inf <- c(get_param(dat, "di.rate"), get_param(dat, "di.rate.g2"))
  if (type == "SIR") {
    rates.rec <- c(get_param(dat, "dr.rate"), get_param(dat, "dr.rate.g2"))
  }

  # Susceptible departures --------------------------------------------------
  nDepartures.sus <- nDeparturesG2.sus <- 0
  idsElig.sus <- which(active == 1 & status == "s")
  nElig.sus <- length(idsElig.sus)
  if (nElig.sus > 0) {
    gElig.sus <- group[idsElig.sus]
    ratesElig.sus <- rates.sus[gElig.sus]
    vecDepartures.sus <- which(rbinom(nElig.sus, 1, ratesElig.sus) == 1)
    if (length(vecDepartures.sus) > 0) {
      idsDpt.sus <- idsElig.sus[vecDepartures.sus]
      nDepartures.sus <- sum(group[idsDpt.sus] == 1)
      nDeparturesG2.sus <- sum(group[idsDpt.sus] == 2)
      active[idsDpt.sus] <- 0
      exitTime[idsDpt.sus] <- at
    }
  }

  # Infected departures -----------------------------------------------------
  nDepartures.inf <- nDeparturesG2.inf <- 0
  idsElig.inf <- which(active == 1 & status == "i")
  nElig.inf <- length(idsElig.inf)
  if (nElig.inf > 0) {
    gElig.inf <- group[idsElig.inf]
    ratesElig.inf <- rates.inf[gElig.inf]
    vecDepartures.inf <- which(rbinom(nElig.inf, 1, ratesElig.inf) == 1)
    if (length(vecDepartures.inf) > 0) {
      idsDpt.inf <- idsElig.inf[vecDepartures.inf]
      nDepartures.inf <- sum(group[idsDpt.inf] == 1)
      nDeparturesG2.inf <- sum(group[idsDpt.inf] == 2)
      active[idsDpt.inf] <- 0
      exitTime[idsDpt.inf] <- at
    }
  }

  # Recovered departures ----------------------------------------------------
  if (type == "SIR") {
    nDepartures.rec <- nDeparturesG2.rec <- 0
    idsElig.rec <- which(active == 1 & status == "r")
    nElig.rec <- length(idsElig.rec)
    if (nElig.rec > 0) {
      gElig.rec <- group[idsElig.rec]
      ratesElig.rec <- rates.rec[gElig.rec]
      vecDepartures.rec <- which(rbinom(nElig.rec, 1, ratesElig.rec) == 1)
      if (length(vecDepartures.rec) > 0) {
        idsDpt.rec <- idsElig.rec[vecDepartures.rec]
        nDepartures.rec <- sum(group[idsDpt.rec] == 1)
        nDeparturesG2.rec <- sum(group[idsDpt.rec] == 2)
        active[idsDpt.rec] <- 0
        exitTime[idsDpt.rec] <- at
      }
    }
  }

  # Output ------------------------------------------------------------------
  dat <- set_attr(dat, "active", active)
  dat <- set_attr(dat, "exitTime", exitTime)

  dat <- set_epi(dat, "ds.flow", at, nDepartures.sus)
  dat <- set_epi(dat, "di.flow", at, nDepartures.inf)
  if (type == "SIR") {
    dat <- set_epi(dat, "dr.flow", at, nDepartures.rec)
  }
  dat <- set_epi(dat, "ds.flow.g2", at, nDeparturesG2.sus)
  dat <- set_epi(dat, "di.flow.g2", at, nDeparturesG2.inf)
  if (type == "SIR") {
    dat <- set_epi(dat, "dr.flow.g2", at, nDeparturesG2.rec)
  }
  return(dat)
}


#' @title Arrivals: netsim Module
#'
#' @description This function simulates new arrivals into the network
#'   for use in [netsim()] simulations.
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @seealso [netsim()]
#'
#' @export
#' @keywords netMod internal
#'
arrivals.2g.net <- function(dat, at) {

  # Conditions --------------------------------------------------------------
  vital <- get_param(dat, "vital")
  if (vital == FALSE) {
    return(dat)
  }

  # Variables ---------------------------------------------------------------
  a.rate <- get_param(dat, "a.rate")
  a.rate.g2 <- get_param(dat, "a.rate.g2")
  index <- at - 1
  nOld <- get_epi(dat, "num", index)
  nOldG2 <- get_epi(dat, "num.g2", index)
  totArr <- nArrivals <- nArrivalsG2 <- 0

  # Add Nodes ---------------------------------------------------------------
  if (nOld > 0) {

    if (is.na(a.rate.g2)) {
      nArrivals <- rbinom(1, nOld, a.rate)
      nArrivalsG2 <- rbinom(1, nOld, a.rate)
      totArr <- nArrivals + nArrivalsG2
    } else {
      nArrivals <- rbinom(1, nOld, a.rate)
      nArrivalsG2 <- rbinom(1, nOldG2, a.rate.g2)
      totArr <- nArrivals + nArrivalsG2
    }

    if (totArr > 0) {
      dat <- append_core_attr(dat, at, totArr)
      dat <- append_attr(dat, "group", 1, nArrivals)
      dat <- append_attr(dat, "group", 2, nArrivalsG2)
      dat <- append_attr(dat, "status", "s", totArr)
      dat <- append_attr(dat, "infTime", NA, totArr)
    }
  }

  # Output ------------------------------------------------------------------
  dat <- set_epi(dat, "a.flow", at, nArrivals)
  dat <- set_epi(dat, "a.flow.g2", at, nArrivalsG2)

  return(dat)
}

#' @title Extract Network Objects from Network Simulations
#'
#' @description Extracts the network object from either a network epidemic model
#'              object generated with `netsim`, a network diagnostic
#'              simulation generated with `netdx`, or a `netsim_dat`
#'              object used internally in `netsim`. For `netdx` or
#'              `netsim` with `tergmLite == FALSE`, the extracted
#'              network object is a `networkDynamic`, which can be
#'              collapsed down to a static `network` object with the
#'              `collapse` and `at` arguments. For `netsim` with
#'              `tergmLite == TRUE`, the extracted network object is the
#'              final `networkLite`, the `collapse` argument should be
#'              `FALSE`, and the `at` argument should be missing. For
#'              `netsim_dat`, the `collapse` and `at` arguments
#'              are not supported, and the network object is either the current
#'              `networkLite` (if `tergmLite == TRUE`) or the current
#'              `networkDynamic` (if `tergmLite == FALSE`).
#'
#' @param x An `EpiModel` object of class [netsim()],
#'        [netdx()], or `netsim_dat`.
#' @param sim Simulation number of extracted network, for `netdx` and
#'        `netsim`.
#' @param network Network number, for `netsim` or `netsim_dat` objects
#'        with multiple overlapping networks (advanced use, and not applicable
#'        to `netdx` objects).
#' @param collapse If `TRUE`, collapse the `networkDynamic` object to
#'        a static `network` object at a specified time step. Applicable to
#'        `netdx` objects and `netsim` objects with
#'        `tergmLite == FALSE`.
#' @param at If `collapse` is `TRUE`, the time step at which the
#'        extracted network should be collapsed. Applicable to `netdx`
#'        objects and `netsim` objects with `tergmLite == FALSE`.
#' @param ... Additional arguments.
#'
#' @details
#' This function requires that the network object is saved during the network
#' simulation while running either [netsim()] or [netdx()].
#' For the former, that is specified by setting the `save.network`
#' parameter in [control.net()] to `TRUE`. For the latter, that
#' is specified with the `keep.tnetwork` parameter directly in
#' [netdx()].
#'
#' @return For `netdx` or `netsim` with `tergmLite == FALSE`, a
#'         `networkDynamic` object (if `collapse = FALSE`) or a
#'         static `network` object (if `collapse = TRUE`). For
#'         `netsim` with `tergmLite == TRUE` or `netsim_dat` with
#'         `tergmLite == TRUE`, a `networkLite` object. For
#'         `netsim_dat` with `tergmLite == FALSE`, a
#'         `networkDynamic` object.
#'
#' @keywords extract
#' @export
#'
#' @examples
#' \donttest{
#' # Set up network and TERGM formula
#' nw <- network_initialize(n = 100)
#' nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#'
#' # Estimate the model
#' est <- netest(nw, formation, target.stats, coef.diss)
#'
#' # Run diagnostics, saving the networkDynamic objects
#' dx <- netdx(est, nsteps = 10, nsims = 3, keep.tnetwork = TRUE,
#'             verbose = FALSE)
#'
#' # Extract the network for simulation 2 from dx object
#' get_network(dx, sim = 2)
#'
#' # Extract and collapse the network from simulation 1 at time step 5
#' get_network(dx, collapse = TRUE, at = 5)
#'
#' # Parameterize the epidemic model, and simulate it
#' param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
#' init <- init.net(i.num = 10, i.num.g2 = 10)
#' control <- control.net(type = "SI", nsteps = 10, nsims = 3, verbose = FALSE)
#' mod <- netsim(est, param, init, control)
#'
#' # Extract the network for simulation 2 from mod object
#' get_network(mod, sim = 2)
#'
#' ## Extract and collapse the network from simulation 1 at time step 5
#' get_network(mod, collapse = TRUE, at = 5)
#' }
#'
get_network <- function(x, ...) {
  UseMethod("get_network")
}

#' @rdname get_network
#'
#' @export
#'
get_network.netdx <- function(x, sim = 1, collapse = FALSE, at = NULL, ...) {

  ## Warnings and checks ##
  nsims <- x$nsims
  if (length(sim) > 1 || sim > nsims) {
    stop("Specify a single sim between 1 and ", nsims)
  }

  if (is.null(x$network)) {
    stop("Network object not saved in netdx object.
          Check keep.tnetwork parameter")
  }

  nsteps <- x$nsteps
  if (collapse == TRUE && (is.null(at) || at > nsteps || at < 0)) {
    stop("Specify collapse time step between 0 and ", nsteps)
  }

  ## Extraction ##
  out <- x$network[[sim]]

  ## Collapsing
  if (collapse == TRUE) {
    out <- network.collapse(out, at = at)
  }

  return(out)
}

#' @rdname get_network
#'
#' @export
#'
get_network.netsim <- function(x, sim = 1, network = 1, collapse = FALSE,
                               at = NULL, ...) {
  ## Warnings and checks ##
  nsims <- x$control$nsims
  if (length(sim) > 1 || sim > nsims) {
    stop("Specify a single sim between 1 and ", nsims)
  }

  if (x$control$tergmLite == TRUE && collapse == TRUE) {
    stop("Argument `collapse` should be FALSE when x$control$tergmLite == TRUE")
  }

  if (x$control$tergmLite == TRUE && !is.null(at)) {
    stop("Argument `at` should be NULL when x$control$tergmLite == TRUE")
  }

  if (is.null(x$network)) {
    stop("Network object not saved in netsim object. Check control.net settings.")
  }

  if (network < 1 || network > x$num.nw) {
    stop("Specify network between 1 and ", x$num.nw)
  }

  nsteps <- x$control$nsteps
  if (collapse == TRUE && (is.null(at) || at > nsteps || at < 0)) {
    stop("Specify collapse time step between 0 and ", nsteps)
  }

  ## Extraction ##
  out <- x$network[[sim]][[network]]

  ## Collapsing
  if (collapse == TRUE) {
    out <- network.collapse(out, at = at)
  }

  return(out)
}

#' @rdname get_network
#'
#' @export
#'
get_network.netsim_dat <- function(x, network = 1L, ...) {
  if (get_control(x, "tergmLite") == FALSE) {
    ## networkDynamic
    nw <- x$run$nw[[network]]
  } else {
    ## networkLite
    attr_list <- raw_get_attr_list(x)
    nw <- networkLite(x$run$el[[network]], attr_list, x$run$net_attr[[network]])
  }
  return(nw)
}


#' @title Set Network State During netsim Simulation
#'
#' @description This function updates the `netsim_dat` object given a
#'              network representing the current state of the simulation.
#'
#' @param x a `netsim_dat` object
#' @param network the index of the network to set on `x`
#' @param nw the value of the network to set on `x`
#'
#' @details If running `tergmLite` simulation, this function updates
#' `x$el[[network]]` and (if `tergmLite.track.duration` is `TRUE`
#' for the network index `network`) the network attributes `"time"`
#' and `"lasttoggle"` in `x$net_attr[[network]]`. If not running
#' `tergmLite` simulation, this function updates the `networkDynamic`
#' object stored in `x$nw[[network]]`. The input `nw` should be of
#' class `networkLite` when running `tergmLite` simulation, and of
#' class `networkDynamic` when not running `tergmLite` simulation.
#'
#' @return the `netsim_dat` object with the network state updated
#'
#' @export
#' @keywords netUtils internal
#'
set_network <- function(x, ...) {
  UseMethod("set_network")
}

#' @rdname set_network
#'
#' @export
#'
set_network.netsim_dat <- function(x, network = 1L, nw, ...) {
  if (get_control(x, "tergmLite") == TRUE) {
    x$run$el[[network]] <- as.edgelist(nw)
    if (get_network_control(x, network, "tergmLite.track.duration") == TRUE) {
      x$run$net_attr[[network]][["time"]] <- nw %n% "time"
      x$run$net_attr[[network]][["lasttoggle"]] <- nw %n% "lasttoggle"
    }
  } else {
    x$run$nw[[network]] <- nw
  }
  return(x)
}

#' @title Extract Transmissions Matrix from Network Epidemic Model
#'
#' @description Extracts the matrix of transmission data for each transmission
#'              event that occurred within a network epidemic model.
#'
#' @param x An `EpiModel` object of class [netsim()].
#' @param sim Simulation number of extracted network.
#' @param deduplicate If `TRUE`, randomly select one transmission event in
#'        the case that multiple events occur per newly infected agent within
#'        a time step.
#'
#' @return
#' A data frame with the following standard columns:
#'
#'  * **at:** the time step at which the transmission occurred.
#'  * **sus:** the ID number of the susceptible (newly infected) node.
#'  * **inf:** the ID number of the infecting node.
#'  * **infDur:** the duration of the infecting node's disease at the
#'        time of the transmission.
#'  * **transProb:** the probability of transmission per act.
#'  * **actRate:** the rate of acts per unit time.
#'  * **finalProb:** the final transmission probability for the
#'        transmission event.
#'
#'
#' @keywords extract
#' @export
#'
#' @examples
#' \donttest{
#' ## Simulate SI epidemic on two-group Bernoulli random graph
#' nw <- network_initialize(n = 100)
#' nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#' est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#' param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
#' init <- init.net(i.num = 10, i.num.g2 = 10)
#' control <- control.net(type = "SI", nsteps = 10, nsims = 3, verbose = FALSE)
#' mod <- netsim(est, param, init, control)
#'
#' ## Extract the transmission matrix from simulation 2
#' get_transmat(mod, sim = 2)
#' }
#'
get_transmat <- function(x, sim = 1, deduplicate = TRUE) {

  ## Warnings and checks
  if (!inherits(x, "netsim")) {
    stop("x must be of class netsim")
  }

  if (sim > x$control$nsims) {
    stop("Specify sim between 1 and ", x$control$nsims)
  }

  if (x$control$save.transmat == FALSE || is.null(x$stats$transmat)) {
    stop("transmat not saved in netsim object, check control.net settings")
  }

  ## Extraction
  out <- x$stats$transmat[[sim]]
  out <- dplyr::as_tibble(out)

  if (deduplicate) {
    out <- dplyr::sample_n(dplyr::group_by(out, .data$at, .data$sus), 1)
  }

  class(out) <- c("transmat", class(out))
  return(out)
}


#' @title Extract Network Statistics from netsim or netdx Object
#'
#' @description Extracts network statistics from a network epidemic model
#'              simulated with `netsim` or a network diagnostics object
#'              simulated with `netdx`. Statistics can be returned either
#'              as a single data frame or as a list of matrices (one matrix
#'              for each simulation).
#'
#' @param x An `EpiModel` object of class [netsim()] or
#'        [netdx()].
#' @param sim A vector of simulation numbers from the extracted object.
#'        (Default = NULL, all simulations are included)
#' @param network Network number, for `netsim` objects with multiple
#'        overlapping networks (advanced use, and not applicable to `netdx`
#'        objects).
#' @param mode Either `"data.frame"` or `"list"`, indicating the
#'        desired output.
#'
#' @return A data frame or list of matrices containing the network statistics.
#'
#' @keywords extract
#' @export
#'
#' @examples
#' \donttest{
#' # Two-group Bernoulli random graph TERGM
#' nw <- network_initialize(n = 100)
#' nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#' est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' dx <- netdx(est, nsim = 3, nsteps = 10, verbose = FALSE,
#'             nwstats.formula = ~edges + isolates)
#' get_nwstats(dx)
#' get_nwstats(dx, sim = 1)
#'
#' # SI epidemic model
#' param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
#' init <- init.net(i.num = 10, i.num.g2 = 10)
#' control <- control.net(type = "SI", nsteps = 10, nsims = 3,
#'                        nwstats.formula = ~edges + meandeg + degree(0:5),
#'                        verbose = FALSE)
#' mod <- netsim(est, param, init, control)
#'
#' # Extract the network statistics from all or sets of simulations
#' get_nwstats(mod)
#' get_nwstats(mod, sim = 2)
#' get_nwstats(mod, sim = c(1, 3))
#'
#' # On the fly summary stats
#' summary(get_nwstats(mod))
#' colMeans(get_nwstats(mod))
#' }
#'
get_nwstats <- function(x, sim = NULL, network = 1, mode = c("data.frame", "list")) {

  mode <- match.arg(mode)

  ## Warnings and checks ##
  if (!(class(x) %in% c("netsim", "netdx"))) {
    stop("x must be of class netsim or netdx")
  }

  if (inherits(x, "netsim")) {
    nsims <- x$control$nsims
    nsteps <- x$control$nsteps
  } else {
    if (x$dynamic == TRUE) {
      nsims <- x$nsims
      nsteps <- x$nsteps
    } else {
      nsims <- 1
      nsteps <- x$nsims
    }
  }

  if (is.null(sim)) {
    sim <- seq_len(nsims)
  }
  if (max(sim) > nsims) {
    stop("Specify sims less than or equal to ", nsims)
  }

  if (inherits(x, "netsim")) {
    if (x$control$save.nwstats == FALSE || is.null(x$stats$nwstats)) {
      stop("Network statistics not saved in netsim object, check control.net
           settings")
    }
    if (network > x$num.nw) {
      stop("Specify network between 1 and ", x$num.nw)
    }
  }

  ## Extraction
  if (inherits(x, "netsim")) {
    out <- lapply(x$stats$nwstats, function(n) n[[network]])
    out <- out[sim]
  } else if (inherits(x, "netdx")) {
    out <- x$stats[sim]
  }

  if (mode == "list") {
    return(lapply(out, as.matrix))
  }

  out <- as.data.frame(do.call("rbind", out))
  out$time <- rep(seq_len(min(nsteps, nrow(out))), length(sim))
  out$sim <- rep(sim, each = min(nsteps, nrow(out)))
  row.names(out) <- seq_len(nrow((out)))
  out <- out[, c((ncol(out) - 1):ncol(out), 1:(ncol(out) - 2))]

  if (inherits(x, "netdx") && x$dynamic == FALSE) {
    out <- out[, -2]
    names(out)[1] <- "sim"
  }

  return(out)
}


#' @title Extract Network Model Parameters
#'
#' @description Extracts a list of network model parameters saved in the
#'              initialization module.
#'
#' @param x Main `netsim_dat` data object used in `netsim` simulations.
#' @param network Network number, for simulations with multiple networks
#'        representing the population.
#'
#' @keywords extract internal
#' @export
#'
get_nwparam <- function(x, network = 1) {
  x$nwparam[[network]]
}


#' @title Extract Network Simulations
#'
#' @description Subsets the entire `netsim` object to a subset of
#'              simulations, essentially functioning like a reverse of
#'              `merge`.
#'
#' @param x An object of class `netsim`.
#' @param sims Either a numeric vector of simulation numbers to retain in the
#'        output object, or `"mean"`, which selects the one simulation with
#'        the value of the variable specified in `var` closest to the mean
#'        of `var` across all simulations at the final step.
#' @param var A character vector of variables to retain from `x` if
#'        `sims` is a numeric vector, or a single variable name for
#'        selecting the average simulation from the set if `sims = "mean"`.
#'
#' @return An updated object of class `netsim` containing only the
#'         simulations specified in `sims` and the variables specified in
#'         `var`.
#'
#' @keywords extract
#' @export
#'
#' @examples
#' \donttest{
#' # Network model estimation
#' nw <- network_initialize(n = 100)
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#' est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' # Epidemic model
#' param <- param.net(inf.prob = 0.3)
#' init <- init.net(i.num = 10)
#' control <- control.net(type = "SI", nsteps = 10, nsims = 3, verbose.int = 0)
#' mod1 <- netsim(est1, param, init, control)
#'
#' # Get sim 2
#' s.g2 <- get_sims(mod1, sims = 2)
#'
#' # Get sims 2 and 3 and keep only a subset of variables
#' s.g2.small <- get_sims(mod1, sims = 2:3, var = c("i.num", "si.flow"))
#'
#' # Extract the mean simulation for the variable i.num
#' sim.mean <- get_sims(mod1, sims = "mean", var = "i.num")
#' }
#'
get_sims <- function(x, sims = NULL, var = NULL) {

  if (!inherits(x, "netsim")) {
    stop("x must be of class netsim")
  }

  nsims <- x$control$nsims

  if (is.null(sims)) {
    stop("Specify sims as a vector of simulations or \"mean\" ")
  }
  if (length(sims) == 1 && sims ==
        "mean" && (is.null(var) || length(var) > 1)) {
    stop("If sims == 'mean' then var must be a single variable name")
  }

  if (length(sims) == 1 && sims == "mean") {
    d <- tail(x$epi[[var]], 1)
    md <- mean(as.numeric(d))
    sims <- which.min(abs(d - md))
  }

  if (max(sims) > nsims) {
    stop("Maximum sims value for this object is ", nsims)
  }

  out <- x
  out$control$nsims <- length(sims)
  newnames <- paste0("sim", seq_len(out$control$nsims))

  delsim <- setdiff(1:nsims, sims)
  keepsim <- setdiff(seq_len(nsims), delsim)
  if (length(delsim) > 0) {
    for (i in seq_along(out$epi)) {
      out$epi[[i]] <- out$epi[[i]][, -delsim, drop = FALSE]
    }

    if (!is.null(out$run)) {
      out$run[delsim] <- NULL
      names(out$run) <- newnames
    }
    if (!is.null(out$network)) {
      out$network[delsim] <- NULL
      names(out$network) <- newnames
    }
    if (!is.null(out$stats$nwstats)) {
      out$stats$nwstats[delsim] <- NULL
      names(out$stats$nwstats) <- newnames
    }
    if (!is.null(out$stats$transmat)) {
      out$stats$transmat[delsim] <- NULL
      names(out$stats$transmat) <- newnames
    }
    if (!is.null(out$diss.stats)) {
      out$diss.stats[delsim] <- NULL
      names(out$diss.stats) <- newnames
    }
    if (!is.null(out$control$save.other)) {
      oname <- out$control$save.other
      for (i in seq_along(oname)) {
        out[[oname[i]]][delsim] <- NULL
        names(out[[oname[i]]]) <- newnames
      }
    }
  }

  out$param$random.params.values <- subset_random_params_values(
    out$param$random.params.values, keepsim, nsims
  )

  if (!is.null(var)) {
    match.vars <- which(names(x$epi) %in% var)
    out$epi <- out$epi[match.vars]
  }

  return(out)
}

random_param_draws <- function(value, nsims) {
  if (is.list(value)) {
    return(value)
  }

  if (nsims > 1 && length(value) == nsims) {
    return(as.list(value))
  }

  return(list(value))
}

random_param_from_draws <- function(draws) {
  if (length(draws) == 0) {
    return(draws)
  }

  draw_lengths <- vapply(draws, length, integer(1))
  draw_is_list <- vapply(draws, is.list, logical(1))
  if (all(draw_lengths == 1) && !any(draw_is_list)) {
    return(unname(unlist(draws, recursive = FALSE, use.names = FALSE)))
  }

  return(draws)
}

normalize_random_params_values <- function(values, nsims) {
  if (is.null(values)) {
    return(NULL)
  }

  lapply(values, function(value) {
    random_param_from_draws(random_param_draws(value, nsims))
  })
}

subset_random_params_values <- function(values, sims, nsims) {
  if (is.null(values)) {
    return(NULL)
  }

  lapply(values, function(value) {
    draws <- random_param_draws(value, nsims)
    random_param_from_draws(draws[sims])
  })
}

param_without_random_values <- function(param) {
  random_value_names <- names(param[["random.params.values"]])
  param[c(random_value_names, "random.params.values")] <- NULL
  return(param)
}

random_param_na_like <- function(value) {
  if (length(value) == 0) {
    return(NA)
  }

  out <- value
  out[] <- NA
  return(out)
}

random_param_missing_draws <- function(nsims, template) {
  rep(list(random_param_na_like(template)), nsims)
}

merge_random_params_values <- function(x_values, y_values, x_nsims, y_nsims) {
  if (is.null(x_values) && is.null(y_values)) {
    return(NULL)
  }

  random_names <- union(names(x_values), names(y_values))
  out <- vector("list", length(random_names))
  names(out) <- random_names

  for (name in random_names) {
    x_has_name <- name %in% names(x_values)
    y_has_name <- name %in% names(y_values)

    if (x_has_name) {
      x_draws <- random_param_draws(x_values[[name]], x_nsims)
    }
    if (y_has_name) {
      y_draws <- random_param_draws(y_values[[name]], y_nsims)
    }

    if (!x_has_name) {
      x_draws <- random_param_missing_draws(x_nsims, y_draws[[1]])
    }
    if (!y_has_name) {
      y_draws <- random_param_missing_draws(y_nsims, x_draws[[1]])
    }

    out[[name]] <- random_param_from_draws(c(x_draws, y_draws))
  }

  return(out)
}


#' @title Get Arguments from EpiModel Parameterization Functions
#'
#' @description Returns a list of argument names and values for use for
#'              parameter processing functions.
#'
#' @param formal.args The output of `formals(sys.function())`.
#' @param dot.args The output of `list(...)`.
#'
#' @return A list of argument names and values.
#'
#' @export
#' @keywords internal
#'
get_args <- function(formal.args, dot.args) {
  p <- list()
  formal.args[["..."]] <- NULL
  for (arg in names(formal.args)) {
    p[arg] <- list(get(arg, pos = parent.frame()))
  }

  names.dot.args <- names(dot.args)
  if (length(dot.args) > 0) {
    for (i in seq_along(dot.args)) {
      p[[names.dot.args[i]]] <- dot.args[[i]]
    }
  }
  return(p)
}

#' @title Extract the Parameter Set from Network Simulations
#'
#' @param sims An `EpiModel` object of class `netsim`.
#'
#' @return A `data.frame` with one row per simulation and one column per
#'   parameter or parameter element where the parameters are of size > 1.
#'
#' @section Output Format:
#' The outputted `data.frame` has one row per simulation and the columns
#' correspond to the parameters used in this simulation.
#'
#' The column name will match the parameter name if it is a size 1 parameter or
#' if the parameter is of size > 1, there will be N columns (with N being the
#' size of the parameter) named `parameter.name_1`,
#' `parameter.name_2`, ..., `parameter.name_N`.
#'
#'
#' @examples
#' \donttest{
#' # Setup network
#' nw <- network_initialize(n = 50)
#'
#' est <- netest(
#'   nw, formation = ~edges,
#'   target.stats = c(25),
#'   coef.diss = dissolution_coefs(~offset(edges), 10, 0),
#'   verbose = FALSE
#' )
#'
#' init <- init.net(i.num = 10)
#'
#' n <- 5
#'
#' related.param <- data.frame(
#'   dummy.param = rbeta(n, 1, 2)
#' )
#'
#'  my.randoms <- list(
#'    act.rate = param_random(c(0.25, 0.5, 0.75)),
#'    dummy.param = function() rbeta(1, 1, 2),
#'    dummy.strat.param = function() c(
#'      rnorm(1, 0, 10),
#'      rnorm(1, 10, 1)
#'    )
#'  )
#'
#' param <- param.net(
#'   inf.prob = 0.3,
#'   dummy = c(0, 1, 2),
#'   random.params = my.randoms
#' )
#'
#' control <- control.net(type = "SI", nsims = 3, nsteps = 5, verbose = FALSE)
#' mod <- netsim(est, param, init, control)
#'
#' get_param_set(mod)
#' }
#' @export
get_param_set <- function(sims) {
  if (!inherits(sims, "netsim")) {
    stop("`sims` must be of class netsim")
  }

  p.random <- normalize_random_params_values(
    sims[["param"]][["random.params.values"]],
    sims[["control"]][["nsims"]]
  )
  fixed.names <- setdiff(
    names(sims[["param"]]),
    c(names(p.random), "random.params", "random.params.values")
  )

  p.fixed <- sims[["param"]][fixed.names]

  d.param <- data.frame(sim = seq_len(sims[["control"]][["nsims"]]))

  # Fixed parameters
  for (i in seq_along(p.fixed)) {
    val <- p.fixed[[i]]
    name <- names(p.fixed)[i]
    l <- length(val)

    if (l > 1) {
      name <- paste0(name, "_", seq_len(l))
    }

    names(val) <- name
    d.param <- cbind(d.param, t(val))
  }

  # Random parameters
  for (i in seq_along(p.random)) {
    val <- p.random[[i]]
    name <- names(p.random)[i]

    if (is.list(val)) {
      l <- length(val[[1]])
      val <- matrix(Reduce(c, val), ncol = l, byrow = TRUE)
      colnames(val) <- paste0(name, "_", seq_len(l))
      d.param <- cbind(d.param, val)
    } else {
      d.param[name] <- val
    }
  }

  return(d.param)
}

#' @title Extract the Attributes History from Network Simulations
#'
#' @param sims An `EpiModel` object of class `netsim`.
#'
#' @return A list of `data.frame`s, one for each "measure" recorded in the
#' simulation by the `record_attr_history` function.
#'
#' @examples
#' \dontrun{
#'
#' # With `sims` the result of a `netsim` call
#' get_attr_history(sims)
#'
#' }
#'
#' @export
#'
get_attr_history <- function(sims) {
  if (!inherits(sims, "netsim")) {
    stop("`sims` must be of class netsim")
  }

  simnames <- names(sims[["attr.history"]])

  dfs <- list()

  for (name in simnames) {
    records <- sims[["attr.history"]][[name]]
    records <- records$as_list()
    attributes <- vapply(records, function(x) x[["attribute"]], "")
    attributes.names <- unique(attributes)

    simnum <- as.numeric(sub("[^0-9]*", "", name))

    for (a in attributes.names) {
      parts <- Filter(function(x) x[["attribute"]] == a, records)
      parts <- lapply(parts, dplyr::as_tibble)
      d <- dplyr::bind_rows(parts)
      d[["sim"]] <- simnum
      d <- dplyr::select(d, "sim", "time", "attribute", "uids", "values")
      dfs[[a]] <- dplyr::bind_rows(dfs[[a]], d)
    }
  }

  return(dfs)
}

context("Get Functions")


# Model simulation --------------------------------------------------------

nw <- network_initialize(n = 100)
nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
formation <- ~edges
target.stats <- 50
dissolution <- ~offset(edges)
duration <- 20
coef.diss <- dissolution_coefs(dissolution, duration)
est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
init <- init.net(i.num = 10, i.num.g2 = 10)
control <- control.net(type = "SI", nsteps = 10, nsims = 3,
                       verbose = FALSE)
mod <- netsim(est, param, init, control)


# get network -------------------------------------------------------------

test_that("get_network extracts to nD object", {
  skip_on_cran()
  a <- get_network(mod, sim = 2)
  expect_is(a, "networkDynamic")
})

test_that("get_network extracts and collapses to network object", {
  skip_on_cran()
  a <- get_network(mod, collapse = TRUE, at = 3)
  expect_is(a, "network")
})

test_that("get_network yields warning when missing network", {
  skip_on_cran()
  mod$network <- NULL
  expect_error(get_network(mod, sim = 1))
})

test_that("get_network yields warning for incorrect sim", {
  skip_on_cran()
  expect_error(get_network(mod, sim = 5))
})

test_that("get_network error flags", {
  skip_on_cran()
  expect_error(get_network(list(a = 1), 1), "no applicable method for 'get_network'")
  expect_error(get_network(mod, 4), "Specify a single sim between 1 and 3")
  expect_error(get_network(mod, 1, collapse = TRUE), "Specify collapse time")
  expect_error(get_network(mod, 1, collapse = TRUE, at = 200),
               "Specify collapse time")
  expect_error(get_network(mod, 1, 2), "Specify network")
})


# get transmat ------------------------------------------------------------

test_that("get_transmat extracts data frame", {
  skip_on_cran()
  a <- get_transmat(mod, sim = 1)
  expect_is(a, "data.frame")
})

test_that("get_transmat error flags", {
  skip_on_cran()
  expect_error(get_transmat(mod, sim = 5), "Specify sim between 1 and 3")
  expect_error(get_transmat(list(a = 1)), "x must be of class netsim")
  mod$stats$transmat <- NULL
  expect_error(get_transmat(mod, 1), "transmat not saved")
})


# get nwstats -------------------------------------------------------------

test_that("get_nwstats extracts data frame", {
  skip_on_cran()
  a <- get_nwstats(mod, sim = 1)
  expect_is(a, "data.frame")
  expect_equal(get_nwstats(mod, sim = 1:3), get_nwstats(mod))
})

test_that("get_nwstats error flags", {
  skip_on_cran()
  expect_error(get_nwstats(list(a = 1)), "x must be of class netsim")
  expect_error(get_nwstats(mod, sim = 5))
  expect_error(get_nwstats(mod, sim = 1, network = 2),
               "Specify network between 1")
  mod$stats$nwstats <- NULL
  expect_error(get_nwstats(mod), "Network statistics not saved")
})


# get sims ----------------------------------------------------------------

test_that("get_sims extracts simulations", {
  skip_on_cran()
  expect_is(get_sims(mod, sims = 1), "netsim")
  expect_is(get_sims(mod, sims = 2:3), "netsim")
  expect_is(get_sims(mod, sims = "mean", var = "i.num"), "netsim")
  expect_is(get_sims(mod, sims = 1:3), "netsim")
  expect_is(get_sims(mod, sims = 1, var = c("i.num", "s.num")), "netsim")
  expect_is(get_sims(mod, sims = c(1, 3), var = c("i.num", "s.num")), "netsim")
  expect_equal(length(get_sims(mod, sims = 1:3)$diss.stats), 3)
  expect_equal(names(get_sims(mod, sims = 1:3)$diss.stats), c("sim1", "sim2", "sim3"))
  expect_equal(length(get_sims(mod, sims = c(1, 3))$diss.stats), 2)
  expect_equal(names(get_sims(mod, sims = c(1, 3))$diss.stats), c("sim1", "sim2"))
  expect_equal(length(get_sims(mod, sims = 2)$diss.stats), 1)
  expect_equal(names(get_sims(mod, sims = 2)$diss.stats), c("sim1"))
  expect_equal(length(merge(get_sims(mod, sims = 2:3),
                            get_sims(mod, sims = c(1,3)))$diss.stats), 4)
  plot(get_sims(mod, sims = 2), type = "duration")
  plot(get_sims(mod, sims = c(1, 3)), type = "dissolution")
})


test_that("get_sims error flags", {
  skip_on_cran()
  expect_error(get_sims(list(a = 1)), "x must be of class netsim")
  expect_error(get_sims(mod), "Specify sims as a vector")

# get parameter set ------------------------------------------------------------
  nw <- network_initialize(n = 50)

  est <- netest(
    nw, formation = ~edges,
    target.stats = c(25),
    coef.diss = dissolution_coefs(~offset(edges), 10, 0),
    verbose = FALSE
  )

  init <- init.net(i.num = 10)

  my.randoms <- list(
    act.rate = param_random(c(0.25, 0.5, 0.75)),
    dummy.param = function() rbeta(1, 1, 2),
    dummy.strat.param = function() c(
      rnorm(1, 0, 10),
      rnorm(1, 10, 1)
    )
  )

  param <- param.net(
    inf.prob = 0.3,
    dummy = c(0, 1, 2),
    random.params = my.randoms
  )

  control <- control.net(type = "SI", nsims = 3, nsteps = 5, verbose = FALSE)
  mod <- netsim(est, param, init, control)
  d.set <- get_param_set(mod)

  set.colnames <- c(
    "sim",
    "inf.prob",
    "dummy_1",
    "dummy_2",
    "dummy_3",
    "act.rate",
    "vital",
    "dummy.param",
    "dummy.strat.param_1",
    "dummy.strat.param_2",
    "groups"
  )

  expect_is(d.set, "data.frame")
  expect_true(setequal(names(d.set), set.colnames))
  expect_error(get_param_set(control), "`sims` must be of class netsim")
  expect_equal(dim(get_param_set(mod)), c(3, length(set.colnames)))

  mod.sub <- get_sims(mod, sims = 2)
  d.sub <- get_param_set(mod.sub)
  expect_equal(nrow(d.sub), 1)
  expect_equal(d.sub$act.rate, d.set$act.rate[2])
  expect_equal(d.sub$dummy.param, d.set$dummy.param[2])
  expect_equal(d.sub$dummy.strat.param_1, d.set$dummy.strat.param_1[2])
  expect_equal(d.sub$dummy.strat.param_2, d.set$dummy.strat.param_2[2])
})

test_that("get_param_set handles vector random parameters with one simulation", {
  mod <- structure(
    list(
      param = list(
        inf.prob = 0.3,
        dummy.strat.param = c(0.1, 0.2),
        random.params = list(dummy.strat.param = function() NULL),
        random.params.values = list(dummy.strat.param = c(0.1, 0.2))
      ),
      control = list(nsims = 1)
    ),
    class = "netsim"
  )

  d.set <- get_param_set(mod)
  expect_equal(nrow(d.set), 1)
  expect_equal(d.set$dummy.strat.param_1, 0.1)
  expect_equal(d.set$dummy.strat.param_2, 0.2)
})

test_that("get_sims subsets random parameter values", {
  mod <- structure(
    list(
      param = list(
        inf.prob = 0.3,
        act.rate = 0.1,
        dummy.strat.param = c(1, 2),
        random.params = list(
          act.rate = function() NULL,
          dummy.strat.param = function() NULL
        ),
        random.params.values = list(
          act.rate = c(0.1, 0.2, 0.3),
          dummy.strat.param = list(c(1, 2), c(3, 4), c(5, 6))
        )
      ),
      control = list(nsims = 3, save.other = character(0)),
      epi = list(
        i.num = data.frame(matrix(1:3, nrow = 1))
      ),
      stats = list(nwstats = NULL, transmat = NULL),
      run = NULL,
      network = NULL,
      diss.stats = NULL
    ),
    class = "netsim"
  )

  mod.sub <- get_sims(mod, sims = 2)
  d.sub <- get_param_set(mod.sub)

  expect_equal(mod.sub$control$nsims, 1)
  expect_equal(mod.sub$param$random.params.values$act.rate, 0.2)
  expect_equal(mod.sub$param$random.params.values$dummy.strat.param, list(c(3, 4)))
  expect_equal(d.sub$act.rate, 0.2)
  expect_equal(d.sub$dummy.strat.param_1, 3)
  expect_equal(d.sub$dummy.strat.param_2, 4)
})

dxs <- netdx(est, dynamic = FALSE, nsims = 5,
             nwstats.formula = ~edges + nodemix("group", levels2 = TRUE), verbose = FALSE)
dxd <- netdx(est, dynamic = TRUE, nsims = 5, nsteps = 3, verbose = FALSE)

control1 <- control.net(type = "SI", nsteps = 2, nsims = 3,
                        verbose = FALSE, tergmLite = FALSE,
                        resimulate.network = TRUE,
                        nwstats.formula = ~edges + triangle)
mod1 <- netsim(est, param, init, control1)

control2 <- control.net(type = "SI", nsteps = 3, nsims = 4,
                        verbose = FALSE, tergmLite = FALSE,
                        resimulate.network = FALSE)
mod2 <- netsim(est, param, init, control2)

control3 <- control.net(type = "SI", nsteps = 4, nsims = 2,
                        verbose = FALSE, tergmLite = TRUE,
                        resimulate.network = TRUE,
                        nwstats.formula = ~edges +
                                           nodematch("group", diff = TRUE))
mod3 <- netsim(est, param, init, control3)

test_that("get_nwstats with mode = list behaves as expected", {
  skip_on_cran()
  expect_equal(unique(lapply(get_nwstats(dxs, mode = "list"), class)),
               list(c("matrix", "array")))
  expect_equal(unique(lapply(get_nwstats(dxd, mode = "list"), class)),
               list(c("matrix", "array")))
  expect_equal(unique(lapply(get_nwstats(mod1, mode = "list"), class)),
               list(c("matrix", "array")))
  expect_equal(unique(lapply(get_nwstats(mod2, mode = "list"), class)),
               list(c("matrix", "array")))
  expect_equal(unique(lapply(get_nwstats(mod3, mode = "list"), class)),
               list(c("matrix", "array")))

  expect_equal(unique(lapply(get_nwstats(dxs, mode = "list"), dim)),
               list(c(5, 4)))
  expect_equal(unique(lapply(get_nwstats(dxd, mode = "list"), dim)),
               list(c(3, 1)))
  expect_equal(unique(lapply(get_nwstats(mod1, mode = "list"), dim)),
               list(c(2, 2)))
  expect_equal(unique(lapply(get_nwstats(mod2, mode = "list"), dim)),
               list(c(3, 1)))
  expect_equal(unique(lapply(get_nwstats(mod3, mode = "list"), dim)),
               list(c(4, 3)))

  expect_equal(length(get_nwstats(dxs, mode = "list")), 1)
  expect_equal(length(get_nwstats(dxd, mode = "list")), 5)
  expect_equal(length(get_nwstats(mod1, mode = "list")), 3)
  expect_equal(length(get_nwstats(mod2, mode = "list")), 4)
  expect_equal(length(get_nwstats(mod3, mode = "list")), 2)

  expect_equal(length(get_nwstats(dxs, sim = c(1), mode = "list")), 1)
  expect_equal(length(get_nwstats(dxd, sim = c(5,3,1), mode = "list")), 3)
  expect_equal(length(get_nwstats(mod1, sim = c(2,3), mode = "list")), 2)
  expect_equal(length(get_nwstats(mod2, sim = c(1,3,2), mode = "list")), 3)
  expect_equal(length(get_nwstats(mod3, sim = c(1,2), mode = "list")), 2)
})

test_that("get_network_attributes functions as intended", {
  nw <- network.initialize(10, directed = FALSE, bipartite = 4)
  nw %n% "newattr" <- "string"
  expect_equal(get_network_attributes(nw), list(bipartite = 4,
                                                directed = FALSE,
                                                hyper = FALSE,
                                                loops = FALSE,
                                                multiple = FALSE,
                                                n = 10,
                                                newattr = "string"))
})

context("Standard Network Models")

nw <- network_initialize(n = 50)
nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
est <- netest(nw, formation = ~edges + nodematch("race"),
              target.stats = c(25, 10),
              coef.diss = dissolution_coefs(~offset(edges), 10, 0),
              verbose = FALSE)

for (trim in c(FALSE, TRUE)) {
  if (trim == TRUE) {
    est2 <- trim_netest(est)
  } else {
    est2 <- est
  }

  # Edges + nodematch, one-mode, closed

  test_that("netsim for edges only, SI, one-mode, closed, 1 sim", {
    param <- param.net(inf.prob = 0.3, act.rate = 0.5)
    init <- init.net(i.num = 10)
    control <- control.net(type = "SI", nsims = 1, nsteps = 5, verbose = FALSE)
    mod <- netsim(est2, param, init, control)
    expect_is(mod, "netsim")
    plot(mod)
    plot(mod, type = "formation")
    plot(mod, type = "duration")
    plot(mod, type = "dissolution")
    plot(mod, type = "network")
    test_net(mod)
  })

  test_that("netsim for edges only, SI, one-mode, closed, 2 sim", {
    skip_on_cran()
    param <- param.net(inf.prob = 0.3, act.rate = 0.5)
    init <- init.net(i.num = 10)
    control <- control.net(type = "SI", nsims = 2, nsteps = 5, verbose = FALSE)
    mod <- netsim(est2, param, init, control)
    expect_is(mod, "netsim")
    plot(mod)
    plot(mod, type = "formation")
    plot(mod, type = "network")
    test_net(mod)
  })

  test_that("netsim for edges only, SIS, one-mode, closed, 1 sim", {
    param <- param.net(inf.prob = 0.3, act.rate = 0.5, rec.rate = 0.05)
    init <- init.net(i.num = 10)
    control <- control.net(type = "SIS", nsims = 1, nsteps = 5, verbose = FALSE)
    mod <- netsim(est2, param, init, control)
    expect_is(mod, "netsim")
    plot(mod)
    plot(mod, type = "formation")
    plot(mod, type = "network")
    test_net(mod)
  })

  test_that("netsim for edges only, SIS, one-mode, closed, 2 sim", {
    skip_on_cran()
    param <- param.net(inf.prob = 0.3, act.rate = 0.5, rec.rate = 0.05)
    init <- init.net(i.num = 10)
    control <- control.net(type = "SIS", nsims = 2, nsteps = 5, verbose = FALSE)
    mod <- netsim(est2, param, init, control)
    expect_is(mod, "netsim")
  })

  test_that("netsim for edges only, SIR, one-mode, closed, 1 sim", {
    param <- param.net(inf.prob = 0.3, act.rate = 0.5, rec.rate = 0.05)
    init <- init.net(i.num = 10, r.num = 0)
    control <- control.net(type = "SIR", nsims = 1, nsteps = 5, verbose = FALSE)
    mod <- netsim(est2, param, init, control)
    expect_is(mod, "netsim")
    plot(mod)
    plot(mod, type = "formation")
    plot(mod, type = "network")
    test_net(mod)
  })

  test_that("netsim for edges only, SIR, one-mode, closed, 2 sim", {
    skip_on_cran()
    param <- param.net(inf.prob = 0.3, act.rate = 0.5, rec.rate = 0.05)
    init <- init.net(i.num = 10, r.num = 0)
    control <- control.net(type = "SIR", nsims = 2, nsteps = 5, verbose = FALSE)
    mod <- netsim(est2, param, init, control)
    expect_is(mod, "netsim")
    plot(mod)
    plot(mod, type = "formation")
    plot(mod, type = "network")
    test_net(mod)
  })
}

test_that("netsim implicit save.network option", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
  est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  # Epidemic model
  param <- param.net(inf.prob = 0.3)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsteps = 5, nsims = 2, verbose = FALSE)
  mod1 <- netsim(est1, param, init, control)
  expect_s3_class(get_network(mod1), "networkDynamic")

  control <- control.net(type = "SI", nsteps = 5, nsims = 2, verbose = FALSE,
                         save.network = FALSE)
  mod2 <- netsim(est1, param, init, control)
  expect_error(get_network(mod2))
})

test_that("netsim duration 1", {
  skip_on_cran()
  estd1 <- netest(nw, formation = ~edges + nodematch("race"),
                  target.stats = c(25, 10),
                  coef.diss = dissolution_coefs(~offset(edges), 1, 0),
                  verbose = FALSE)
  param <- param.net(inf.prob = 0.3, act.rate = 0.5, rec.rate = 0.05)
  init <- init.net(r.num = 0, status.vector = rep("s", 50))
  control <- control.net(type = "SIR", nsims = 1, nsteps = 5,
                         resimulate.network = TRUE, verbose = FALSE,
                         nwupdate.FUN = NULL)
  set.seed(0)
  mod <- netsim(estd1, param, init, control)
  expect_is(mod, "netsim")
  plot(mod)
  plot(mod, type = "formation")
  plot(mod, type = "network")
  test_net(mod)

  # compare to manually produced networkDynamic
  set.seed(0)
  sim <- simulate(estd1$formation,
                  coef = estd1$coef.form.crude,
                  basis = estd1$newnetwork,
                  control = control.simulate.formula(MCMC.burnin = 2e5),
                  dynamic = FALSE)
  sim <- networkDynamic::as.networkDynamic(sim)
  sim <- networkDynamic::activate.vertices(sim, onset = 0, terminus = Inf)
  sim <- networkDynamic::activate.edges(sim, onset = 0, terminus = Inf)

  sim %n% "net.obs.period" <- list(observations = list(c(0,1)),
                                   mode = "discrete",
                                   time.increment =  1,
                                   time.unit = "step")

  for(i in 1:5) {
    sim <- suppressWarnings(simulate(estd1$formation,
                                     basis = sim,
                                     time.slices = 1,
                                     time.start = i,
                                     time.offset = 0,
                                     control = list(MCMC.prop.args = list(discordance_fraction = 0)),
                                     coef = c(estd1$coef.form),
                                     dynamic = TRUE))
  }
  expect_identical(as.data.frame(sim), as.data.frame(mod$network$sim1[[1]]))
})

test_that("non-nested EDA works in netsim", {
  skip_on_cran()
  nw <- network.initialize(10, directed = FALSE)
  nw %v% "race" <- rep(1:2, length.out = 10)
  nw %v% "age" <- rep(1:5, length.out = 10)
  dc <- dissolution_coefs(~offset(edges) + offset(nodematch("age")), c(3, 7))
  est <- netest(nw, formation = ~edges + nodematch("race"), target.stats = c(10, 5),
                coef.diss = dc, nested.edapprox = FALSE)
  dxs <- netdx(est, nsteps = 2, nsims = 2, dynamic = FALSE, verbose = FALSE)
  dxd <- netdx(est, nsteps = 2, nsims = 2, dynamic = TRUE, verbose = FALSE)
  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsims = 1, nsteps = 5, verbose = FALSE)
  sim <- netsim(est, param, init, control)

  dc <- dissolution_coefs(~offset(edges) + offset(nodematch("age")), c(1, 1))
  est <- netest(nw, formation = ~edges + nodematch("race"), target.stats = c(10, 5),
                coef.diss = dc, nested.edapprox = FALSE)
  dxs <- netdx(est, nsteps = 2, nsims = 2, dynamic = FALSE, verbose = FALSE)
  sim <- netsim(est, param, init, control)
  expect_is(sim, "netsim")
})

test_that("netsim diss.stats", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)
  est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  # Epidemic model
  param <- param.net(inf.prob = 0.3)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsteps = 5, nsims = 2, verbose = FALSE)
  mod <- netsim(est, param, init, control)

  capture_output(
    print(mod)
  )
  expect_output(print(mod), "Duration Statistics.*Sim Mean")
  expect_output(print(mod), "Dissolution Statistics.*Sim Mean")
  expect_error(expect_output(print(mod), "Not available when:"))
  expect_equal(length(mod[["diss.stats"]]), 2)

  mod2 <- merge(mod, mod)
  expect_output(print(mod2), "Duration Statistics.*Sim Mean")
  expect_output(print(mod2), "Dissolution Statistics.*Sim Mean")
  expect_error(expect_output(print(mod2), "Not available when:"))
  expect_equal(length(mod2[["diss.stats"]]), 4)

  mod3 <- merge(mod, mod, keep.diss.stats = FALSE)
  expect_output(print(mod3), "Duration and Dissolution Statistics")
  expect_output(print(mod3), "Not available when:")
  expect_true(is.null(mod3[["diss.stats"]]))
})

test_that("save.other sim naming", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  est <- netest(nw, formation = ~edges,
                target.stats = c(25),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)
  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsims = 2, nsteps = 5, verbose = FALSE,
                         save.other = c("nw"), resimulate.network = TRUE)
  mod <- netsim(est, param, init, control)
  expect_equal(names(mod[["nw"]]), paste0("sim", 1:2))
  mod2 <- merge(mod, mod)
  expect_equal(names(mod2[["nw"]]), paste0("sim", 1:4))
  mod3 <- get_sims(mod2, c(1,3,4))
  expect_equal(names(mod3[["nw"]]), paste0("sim", 1:3))
  mod4 <- merge(mod, mod, keep.other = FALSE)
  expect_equal(names(mod4[["nw"]]), NULL)
})

test_that("saveout.net preserves NULL list elements across sims", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  est <- netest(nw, formation = ~edges,
                target.stats = c(25),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)
  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsims = 3, nsteps = 5, verbose = FALSE,
                         save.network = FALSE)
  mod <- netsim(est, param, init, control)

  # attr.history and raw.records are NULL per-sim in base models;
  # verify they are preserved as NULL entries rather than dropped
  expect_equal(length(mod$attr.history), 3)
  expect_equal(length(mod$raw.records), 3)
  for (s in 1:3) {
    expect_true(is.null(mod$attr.history[[s]]))
    expect_true(is.null(mod$raw.records[[s]]))
  }
})

test_that("name_saveout_elts unit", {
  skip_on_cran()
  simnames <- paste0("sim", 1:4)
  elt_name <- "this_elt"

  elt <- rep(list(sample(10)), 4)
  named_elt <- expect_silent(name_saveout_elts(elt, elt_name, simnames))
  expect_equal(names(named_elt), simnames)

  # wrong size produces a warning
  elt <- rep(list(sample(10)), 2)
  named_elt <- expect_warning(name_saveout_elts(elt, elt_name, simnames))
  expect_null(names(named_elt))
  expect_equal(elt, named_elt)

  # empty element returns silently
  elt <- NULL
  named_elt <- expect_silent(name_saveout_elts(elt, elt_name, simnames))
  expect_null(names(named_elt))
  expect_equal(elt, named_elt)
})

test_that("edges correction behaves as expected", {
  skip_on_cran()
  nw <- network_initialize(n = 500)

  est <- netest(nw, formation = ~edges + degree(1),
                target.stats = c(250, 100),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)

  param_g1 <- param.net(inf.prob = 0.4, act.rate = 1,
                        a.rate = 0.03, ds.rate = 0.03, di.rate = 0.03)

  param_g2 <- param.net(inf.prob = 0.4, act.rate = 1, inf.prob.g2 = 0.4,
                        a.rate = 0.03, ds.rate = 0.03, di.rate = 0.03,
                        a.rate.g2 = 0.03, ds.rate.g2 = 0.03, di.rate.g2 = 0.03)

  nsims <- 2

  init_g1 <- init.net(i.num = 100)
  init_g2 <- init.net(i.num = 100, i.num.g2 = 100)
  for (tergmLite in list(FALSE, TRUE)) {
    for (resimulate.network in unique(c(tergmLite, TRUE))) {
      for (ngroups in list(1,2)) {
        if (ngroups == 1) {
          est$newnetwork <- delete.vertex.attribute(est$newnetwork, "group")
          param <- param_g1
          init <- init_g1
        } else {
          est$newnetwork %v% "group" <- rep(c(1,2), length.out = network.size(est$newnetwork))
          param <- param_g2
          init <- init_g2
        }

        nsteps <- 5
        control <- control.net(type = "SI", nsteps = nsteps, nsims = nsims, ncores = 1,
                               resimulate.network = resimulate.network,
                               tergmLite = tergmLite,
                               verbose = FALSE,
                               save.network = TRUE,
                               save.run = TRUE,
                               save.other = c())
        sim <- netsim(est, param, init, control)

        for (simno in seq_len(nsims)) {
          if (ngroups == 1) {
            expect_equal(est$coef.form[1] + log(network.size(nw)),
                         sim$coef.form[[simno]][[1]][1] + log(sim$run[[simno]]$num),
                         tolerance = 1e-6)
          } else {
            n1.old <- sum(est$newnetwork %v% "group" == 1)
            n2.old <- sum(est$newnetwork %v% "group" == 2)
            n1.new <- sim$run[[simno]]$num
            n2.new <- sim$run[[simno]]$num.g2

            expect_equal(est$coef.form[1] + log(2*n1.old*n2.old/(n1.old+n2.old)),
                         sim$coef.form[[simno]][[1]][1] + log(2*n1.new*n2.new/(n1.new+n2.new)),
                         tolerance = 1e-6)
          }
        }

        if (tergmLite == FALSE) {
          control$start <- nsteps + 1
          nsteps <- 9
          control$nsteps <- nsteps
          sim2 <- netsim(sim, param, init, control)

          for (simno in seq_len(nsims)) {
            if (ngroups == 1) {
              expect_equal(est$coef.form[1] + log(network.size(nw)),
                          sim$coef.form[[simno]][[1]][1] + log(sim$run[[simno]]$num),
                          tolerance = 1e-6)
            } else {
              n1.old <- sum(est$newnetwork %v% "group" == 1)
              n2.old <- sum(est$newnetwork %v% "group" == 2)
              n1.new <- sim$run[[simno]]$num
              n2.new <- sim$run[[simno]]$num.g2

              expect_equal(est$coef.form[1] + log(2*n1.old*n2.old/(n1.old+n2.old)),
                          sim$coef.form[[simno]][[1]][1] + log(2*n1.new*n2.new/(n1.new+n2.new)),
                          tolerance = 1e-6)
              }
          }
        }
      }
    }
  }
})

test_that("networkDynamics produced by netsim match those produced by simulate when they should", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  est <- netest(nw, formation = ~edges,
                target.stats = c(25),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)
  param <- param.net(inf.prob = 0, act.rate = 0, rec.rate = 0)
  init <- init.net(i.num = 0, r.num = 0)
  control <- control.net(type = "SIR", nsims = 1, nsteps = 5, verbose = FALSE,
                         save.network = TRUE, resimulate.network = TRUE,
                         save.diss.stats = FALSE,
                         save.run = TRUE,
                         save.other = c(),
                         save.transmat = FALSE)
  set.seed(0)
  mod <- netsim(est, param, init, control)
  control$start <- 6
  control$nsteps <- 11
  mod2 <- netsim(mod, param, init, control)

  # compare to manually produced networkDynamic
  set.seed(0)
  sim <- simulate(est$formation,
                  coef = est$coef.form.crude,
                  basis = est$newnetwork,
                  control = control.simulate.formula(MCMC.burnin = 2e5),
                  dynamic = FALSE)
  sim <- networkDynamic::as.networkDynamic(sim)
  sim <- networkDynamic::activate.vertices(sim, onset = 0, terminus = Inf)
  sim <- networkDynamic::activate.edges(sim, onset = 0, terminus = Inf)

  sim %n% "net.obs.period" <- list(observations = list(c(0,1)),
                                   mode = "discrete",
                                   time.increment =  1,
                                   time.unit = "step")

  for(i in 1:5) {
    sim <- suppressWarnings(simulate(~Form(est$formation) + Persist(est$coef.diss$dissolution),
                                     basis = sim,
                                     time.slices = 1,
                                     time.start = i,
                                     time.offset = 0,
                                     coef = c(est$coef.form, est$coef.diss$coef.crude),
                                     dynamic = TRUE))
  }
  expect_identical(as.data.frame(sim), as.data.frame(mod$network$sim1[[1]]))

  for(i in 6:11) {
    sim <- suppressWarnings(simulate(~Form(est$formation) + Persist(est$coef.diss$dissolution),
                                     basis = sim,
                                     time.slices = 1,
                                     time.start = i,
                                     time.offset = 0,
                                     coef = c(est$coef.form, est$coef.diss$coef.crude),
                                     dynamic = TRUE))
  }
  expect_identical(as.data.frame(sim), as.data.frame(mod2$network$sim1[[1]]))
})

test_that("networkLites produced by netsim match those produced by simulate when they should", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  est <- netest(nw, formation = ~edges,
                target.stats = c(25),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)
  param <- param.net(inf.prob = 0, act.rate = 0, rec.rate = 0)
  init <- init.net(i.num = 0, r.num = 0)
  control <- control.net(type = "SIR", nsims = 1, nsteps = 5, verbose = FALSE,
                         save.network = TRUE, resimulate.network = TRUE,
                         tergmLite = TRUE,
                         save.run = TRUE,
                         save.other = c(),
                         tergmLite.track.duration = TRUE, save.transmat = FALSE)
  est <- trim_netest(est)
  set.seed(0)
  mod <- netsim(est, param, init, control)
  control$start <- 6
  control$nsteps <- 11
  mod2 <- netsim(mod, param, init, control)

  # compare to manually produced networkDynamic
  set.seed(0)
  sim <- simulate(est$formation,
                  coef = est$coef.form.crude,
                  basis = est$newnetwork,
                  control = control.simulate.formula(MCMC.burnin = 2e5),
                  dynamic = FALSE)

  sim %n% "time" <- 0L
  sim %n% "lasttoggle" <- cbind(as.edgelist(sim), 0L)

  for(i in 1:5) {
    sim <- suppressWarnings(simulate(~Form(est$formation) + Persist(est$coef.diss$dissolution),
                                     basis = sim,
                                     time.slices = 1,
                                     time.start = i - 1,
                                     time.offset = 1,
                                     output = "final",
                                     coef = c(est$coef.form, est$coef.diss$coef.crude),
                                     dynamic = TRUE))
  }
  expect_equal(as.edgelist(sim), as.edgelist(mod$network$sim1[[1]]), check.attributes = FALSE)
  expect_equal(sim %n% "lasttoggle", mod$network$sim1[[1]] %n% "lasttoggle")

  for(i in 6:11) {
    sim <- suppressWarnings(simulate(~Form(est$formation) + Persist(est$coef.diss$dissolution),
                                     basis = sim,
                                     time.slices = 1,
                                     time.start = i - 1,
                                     time.offset = 1,
                                     output = "final",
                                     coef = c(est$coef.form, est$coef.diss$coef.crude),
                                     dynamic = TRUE))
  }
  expect_equal(as.edgelist(sim), as.edgelist(mod2$network$sim1[[1]]), check.attributes = FALSE)
  expect_equal(sim %n% "lasttoggle", mod2$network$sim1[[1]] %n% "lasttoggle")
})

context("Netsim Checkpoint")

test_that("netsim with checkpoint", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  est <- netest(
    nw,
    formation = ~edges,
    target.stats = 24,
    coef.diss = dissolution_coefs(~ offset(edges), 10, 0),
    verbose = FALSE
  )

  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)
  control <- control.net(
    type = "SI", nsims = 3, nsteps = 10, ncores = 2, verbose = FALSE,
    .checkpoint.dir = "cp", .checkpoint.steps = 3, .checkpoint.keep = TRUE
  )

  mod <- netsim(est, param, init, control)

  # check that the "checkpoint dir" contains 3 files (.checkpoint.keep = TRUE)
  expect_length(list.files("cp"), 3)

  control <- control.net(
    type = "SI", nsims = 3, nsteps = 10, ncores = 2, verbose = FALSE,
    .checkpoint.dir = "cp", .checkpoint.steps = 3, .checkpoint.keep = FALSE
  )

  mod <- netsim(est, param, init, control)
  # check that the "checkpoint dir" has been removed (.checkpoint.keep = FALSE)
  expect_false(dir.exists("cp"))

  control <- control.net(
    type = "SI", nsims = 1, nsteps = 5, ncores = 1, verbose = FALSE,
    .checkpoint.dir = "cp", .checkpoint.steps = -3
  )
  mod <- netsim(est, param, init, control)

  # .checkpoint.steps cannot be negative, thus the ".checkpointed" flag is
  # set to FALSE
  expect_false(mod$control[[".checkpointed"]])
})

context("Netsim End Horizon")

test_that("netsim with checkpoint", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  est <- netest(
    nw,
    formation = ~edges,
    target.stats = 24,
    coef.diss = dissolution_coefs(~ offset(edges), 10, 0),
    verbose = FALSE
  )

  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)

  end_horizon <- list(at = 5, modules = c("resim_nets.FUN", "recovery.FUN"))

  control <- control.net(
    type = "SI", nsims = 1, nsteps = 10, ncores = 1, verbose = FALSE,
    end.horizon = end_horizon
  )

  netsim(est, param, init, control)

  end_horizon_err_at <- list(at = 0, modules = "resim_nets.FUN")
  control$end.horizon <- end_horizon_err_at
  expect_error(netsim(est, param, init, control))

  end_horizon_err_at <- list(at = "a", modules = "resim_nets.FUN")
  control$end.horizon <- end_horizon_err_at
  expect_error(netsim(est, param, init, control))

  end_horizon_err_mod <- list(at = "a", modules = "not_there.FUN")
  control$end.horizon <- end_horizon_err_mod
  expect_error(netsim(est, param, init, control))

  # test for actual removal of 2 modules at once
  #   - summary module (nwstats will stop being produced)
  #   - infection module (prevalence will stay constant; see notes)
  end_horizon <- list(at = 5, modules = c("summary_nets.FUN", "infection.FUN"))
  control <- control.net(
    type = "SI", nsims = 1, nsteps = 20, ncores = 1, resimulate.network = TRUE,
    verbose = FALSE, end.horizon = end_horizon
  )
  sim <- netsim(est, param, init, control)
  expect_true(nrow(get_nwstats(sim)) == 4)

  # SI module without arrival or departure
  # prevalence stays constant if the infection module is disabled
  expect_length(unique(sim$epi$i.num[5:20, 1]), 1)
})

context("Network Models with Formation Offsets")

test_that("netsim works with standard offset models", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
  est <- netest(nw, formation = ~edges + offset(nodematch("race")),
                target.stats = 25, coef.form = -Inf,
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)
  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsims = 2, nsteps = 5, verbose = FALSE)
  mod <- netsim(est, param, init, control)
  expect_is(mod, "netsim")
  plot(mod)
  plot(mod, type = "formation")
  plot(mod, type = "network")
  test_net(mod)
})

test_that("netsim works with faux offset models", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
  est <- netest(nw, formation = ~edges + nodematch("race"),
                target.stats = c(25, 0),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)
  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsims = 2, nsteps = 5, verbose = FALSE)
  mod <- netsim(est, param, init, control)
  expect_is(mod, "netsim")
  plot(mod)
  plot(mod, type = "formation")
  plot(mod, type = "network")
  test_net(mod)
})

context("Time-Varying Network Parameters")

test_that("time varying parameters for one-mode", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration  = 20)

  est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  probs <- c(0.25, 0.02)
  durs <- c(10, 1)
  inf.probs <- rep(probs, durs)

  acts <- c(0.5, 2)
  act.rates <- rep(acts, durs)

  rec.rates <- rep(0:1, c(20, 1))

  # Parameters, initial conditions, and controls for model
  param <- param.net(inf.prob = inf.probs, act.rate = act.rates,
                     rec.rate = rec.rates)
  init <- init.net(i.num = 50)
  control <- control.net(type = "SIS", nsteps = 10, nsims = 1,
                         tergmLite = FALSE, verbose = FALSE)
  mod <- netsim(est1, param, init, control)
  expect_is(mod, "netsim")
})


test_that("time varying parameters for two-group models", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  nw <- set_vertex_attribute(nw, "group", rep(c(1, 2), each = 50))
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration  = 20)

  est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  probs <- c(0.25, 0.01)
  durs <- c(10, 1)
  inf.probs <- rep(probs, durs)
  inf.probs.g2 <- inf.probs * 2

  acts <- c(1, 2)
  act.rates <- rep(acts, durs)

  rec.rates <- rep(0:1, c(20, 1))
  rec.rates.g2 <- rep(0:1, c(10, 1))

  param <- param.net(inf.prob = inf.probs,
                     inf.prob.g2 = inf.probs.g2,
                     act.rate = act.rates,
                     rec.rate = rec.rates,
                     rec.rate.g2 = rec.rates.g2)
  init <- init.net(i.num = 10, i.num.g2 = 10,
                   r.num = 0, r.num.g2 = 0)
  control <- control.net(type = "SIR", nsteps = 10, nsims = 1,
                         tergmLite = FALSE, verbose = FALSE)

  mod <- netsim(est1, param, init, control)
  expect_is(mod, "netsim")
})

#' @title Summary Model Statistics
#'
#' @description Extracts and prints model statistics solved with `dcm`.
#'
#' @param object An `EpiModel` object of class `dcm`.
#' @param at Time step for model statistics.
#' @param run Model run number, for `dcm` class models with multiple runs
#'        (sensitivity analyses).
#' @param digits Number of significant digits to print.
#' @param ... Additional summary function arguments (not used).
#'
#' @details
#' This function provides summary statistics for the main epidemiological
#' outcomes (state and transition size and prevalence) from a `dcm` model.
#' Time-specific summary measures are provided, so it is necessary to input a
#' time of interest. For multiple-run models (sensitivity analyses), input a
#' model run number. See examples below.
#'
#' @seealso [dcm()]
#'
#' @method summary dcm
#' @keywords extract
#' @export
#'
#' @examples
#' ## Deterministic SIR model with varying act.rate
#' param <- param.dcm(inf.prob = 0.2, act.rate = 2:4, rec.rate = 1/3,
#'                    a.rate = 0.011, ds.rate = 0.01,
#'                    di.rate = 0.03, dr.rate = 0.01)
#' init <- init.dcm(s.num = 1000, i.num = 1, r.num = 0)
#' control <- control.dcm(type = "SIR", nsteps = 50)
#' mod <- dcm(param, init, control)
#' summary(mod, at = 25, run = 1)
#' summary(mod, at = 25, run = 3)
#' summary(mod, at = 26, run = 3)
#'
summary.dcm <- function(object, at, run = 1, digits = 3, ...) {

  nruns <- object$control$nruns
  type <- object$control$type
  groups <- object$param$groups
  vital <- object$param$vital
  nsteps <- object$control$nsteps

  if (!is.null(object$control$new.mod)) {
    stop("summary method not available for new model types in dcm")
  }

  df <- as.data.frame(object, run = run)

  if (at > nsteps || at < 1) {
    stop("Specify at between 1 and ", nsteps)
  }
  df <- df[df$time == at, ]

  ## Prevalence calculations
  df$s.prev <- df$s.num / df$num
  df$i.prev <- df$i.num / df$num
  if (type == "SIR") {
    df$r.prev <- df$r.num / df$num
  }
  if (groups == 2) {
    df$s.prev.g2 <- df$s.num.g2 / df$num.g2
    df$i.prev.g2 <- df$i.num.g2 / df$num.g2
    if (type == "SIR") {
      df$r.prev.g2 <- df$r.num.g2 / df$num.g2
    }
  }

  if (type == "SI") {
    stats <- with(df, c(s.num, s.prev,
                        i.num, i.prev,
                        num, 1,
                        si.flow, NA))
    mat <- matrix(stats, byrow = TRUE, nrow = length(stats) / 2)
    rownames(mat) <- c("Suscept.", "Infect.", "Total", "S -> I")
    if (vital == TRUE) {
      stats <- with(df, c(a.flow, NA,
                          ds.flow, NA,
                          di.flow, NA))
      mat <- rbind(mat, matrix(stats, byrow = TRUE, nrow = length(stats) / 2))
      rownames(mat)[rownames(mat) == ""] <- c("Arrival ->", "S Departure ->",
                                              "I Departure ->")
    }
    if (groups == 2) {
      stats <- with(df, c(s.num.g2, s.prev.g2,
                          i.num.g2, i.prev.g2,
                          num.g2, 1,
                          si.flow.g2, NA))
      mat.g2 <- matrix(stats, byrow = TRUE, nrow = length(stats) / 2)
      if (vital == TRUE) {
        stats <- with(df, c(a.flow.g2, NA,
                            ds.flow.g2, NA,
                            di.flow.g2, NA))
        mat.g2 <- rbind(mat.g2, matrix(stats, byrow = TRUE,
                                       nrow = length(stats) / 2))
      }
      mat <- cbind(mat, mat.g2)
    }
  }
  if (type == "SIR") {
    stats <- with(df, c(s.num, s.prev,
                        i.num, i.prev,
                        r.num, r.prev,
                        num, 1,
                        si.flow, NA,
                        ir.flow, NA))
    mat <- matrix(stats, byrow = TRUE, nrow = length(stats) / 2)
    rownames(mat) <- c("Suscept.", "Infect.", "Recov.", "Total",
                       "S -> I", "I -> R")
    if (vital == TRUE) {
      stats <- c(df$a.flow, NA,
                 df$ds.flow, NA,
                 df$di.flow, NA,
                 df$dr.flow, NA)
      mat <- rbind(mat, matrix(stats, byrow = TRUE, nrow = length(stats) / 2))
      rownames(mat)[rownames(mat) == ""] <- c("Arrival ->",
                                              "S Departure ->",
                                              "I Departure ->",
                                              "R Departure ->")
    }
    if (groups == 2) {
      stats <- with(df, c(s.num.g2, s.prev.g2,
                          i.num.g2, i.prev.g2,
                          r.num.g2, r.prev.g2,
                          num.g2, 1,
                          si.flow.g2, NA,
                          ir.flow.g2, NA))
      mat.g2 <- matrix(stats, byrow = TRUE, nrow = length(stats) / 2)
      if (vital == TRUE) {
        stats <- with(df, c(a.flow.g2, NA,
                            ds.flow.g2, NA,
                            di.flow.g2, NA,
                            dr.flow.g2, NA))
        mat.g2 <- rbind(mat.g2, matrix(stats, byrow = TRUE,
                                       nrow = length(stats) / 2))
      }
      mat <- cbind(mat, mat.g2)
    }
  }
  if (type == "SIS") {
    stats <- with(df, c(s.num, s.prev,
                        i.num, i.prev,
                        num, 1,
                        si.flow, NA,
                        is.flow, NA))
    mat <- matrix(stats, byrow = TRUE, nrow = length(stats) / 2)
    rownames(mat) <- c("Suscept.", "Infect.", "Total", "S -> I", "I -> S")
    if (vital == TRUE) {
      stats <- c(df$a.flow, NA,
                 df$ds.flow, NA,
                 df$di.flow, NA)
      mat <- rbind(mat, matrix(stats, byrow = TRUE, nrow = length(stats) / 2))
      rownames(mat)[rownames(mat) == ""] <- c("Arrival ->", "S Departure ->",
                                              "I Departure ->")
    }
    if (groups == 2) {
      stats <- with(df, c(s.num.g2, s.prev.g2,
                          i.num.g2, i.prev.g2,
                          num.g2, 1,
                          si.flow.g2, NA,
                          is.flow.g2, NA))
      mat.g2 <- matrix(stats, byrow = TRUE, nrow = length(stats) / 2)
      if (vital == TRUE) {
        stats <- with(df, c(a.flow.g2, NA,
                            ds.flow.g2, NA,
                            di.flow.g2, NA))
        mat.g2 <- rbind(mat.g2, matrix(stats, byrow = TRUE,
                                       nrow = length(stats) / 2))
      }
      mat <- cbind(mat, mat.g2)
    }
  }

  if (groups == 1) {
    colnames(mat) <- c("n", "pct")
  }
  if (groups == 2) {
    colnames(mat) <- c("n:g1", "pct:g1", "n:g2", "pct:g2")
  }

  mat <- round(mat, digits)

  # print it
  cat("EpiModel Summary")
  cat("\n=======================")
  cat("\nModel class:", class(object))

  cat("\n\nSimulation Summary")
  cat("\n-----------------------")
  cat("\nModel type:", type)
  cat("\nNo. runs:", nruns)
  cat("\nNo. time steps:", object$nsteps)
  cat("\nNo. groups:", groups)

  if (groups == 1) {
    statsep <- paste(rep("-", 30), collapse = "")
  }
  if (groups == 2) {
    statsep <- paste(rep("-", 60), collapse = "")
  }

  cat("\n\nModel Statistics\n")
  cat(statsep)
  cat("\nTime:", at)
  cat("\t Run:", run, "\n")
  cat(statsep, "\n")
  print(mat, print.gap = 2)
  cat(statsep, "\n")

}


#' @title Summary Model Statistics
#'
#' @description Extracts and prints model statistics simulated with `icm`.
#'
#' @param object An `EpiModel` object of class `icm`.
#' @param at Time step for model statistics.
#' @param digits Number of significant digits to print.
#' @param ... Additional summary function arguments.
#'
#' @details
#' This function provides summary statistics for the main epidemiological
#' outcomes (state and transition size and prevalence) from an `icm` model.
#' Time-specific summary measures are provided, so it is necessary to input a
#' time of interest.
#'
#' @seealso [icm()]
#'
#' @method summary icm
#' @keywords extract
#' @export
#'
#' @examples
#' \donttest{
#' ## Stochastic ICM SI model with 3 simulations
#' param <- param.icm(inf.prob = 0.2, act.rate = 1)
#' init <- init.icm(s.num = 500, i.num = 1)
#' control <- control.icm(type = "SI", nsteps = 50,
#'                        nsims = 5, verbose = FALSE)
#' mod <- icm(param, init, control)
#' summary(mod, at = 25)
#' summary(mod, at = 50)
#' }
#'
summary.icm <- function(object, at, digits = 3, ...) {

  nsims <- object$control$nsims
  type <- object$control$type
  groups <- object$param$groups
  vital <- object$param$vital
  nsteps <- object$control$nsteps

  if (at > nsteps || at < 1) {
    stop("Specify a time step between 1 and ", nsteps)
  }

  df.mn <- as.data.frame(object, out = "mean")
  df.mn <- df.mn[df.mn$time == at, ]

  df.sd <- as.data.frame(object, out = "sd")
  df.sd <- df.sd[df.sd$time == at, ]

  if (type == "SI") {

    ## Prevalence calcs
    s.prev <- df.mn$s.num / df.mn$num
    i.prev <- df.mn$i.num / df.mn$num
    if (groups == 2) {
      s.prev.g2 <- df.mn$s.num.g2 / df.mn$num.g2
      i.prev.g2 <- df.mn$i.num.g2 / df.mn$num.g2
    }

    ## Group 1 stats
    stats <- c(df.mn$s.num, df.sd$s.num, s.prev,
               df.mn$i.num, df.sd$i.num, i.prev,
               df.mn$num, df.sd$num, 1,
               df.mn$si.flow, df.sd$si.flow, NA)
    mat <- matrix(stats, byrow = TRUE, nrow = length(stats) / 3)
    rownames(mat) <- c("Suscept.", "Infect.", "Total", "S -> I")
    if (vital == TRUE) {
      stats <- c(df.mn$a.flow, df.sd$a.flow, NA,
                 df.mn$ds.flow, df.sd$ds.flow, NA,
                 df.mn$di.flow, df.sd$di.flow, NA)
      mat <- rbind(mat, matrix(stats, byrow = TRUE, nrow = length(stats) / 3))
      rownames(mat)[rownames(mat) == ""] <- c("Arrival ->", "S Departure ->",
                                              "I Departure ->")
    }

    ## Group 2 stats
    if (groups == 2) {
      stats <- c(df.mn$s.num.g2, df.sd$s.num.g2, s.prev.g2,
                 df.mn$i.num.g2, df.sd$i.num.g2, i.prev.g2,
                 df.mn$num.g2, df.sd$num.g2, 1,
                 df.mn$si.flow.g2, df.sd$si.flow.g2, NA)
      mat.g2 <- matrix(stats, byrow = TRUE, nrow = length(stats) / 3)
      if (vital == TRUE) {
        stats <- c(df.mn$a.flow.g2, df.sd$a.flow.g2, NA,
                   df.mn$ds.flow.g2, df.sd$ds.flow.g2, NA,
                   df.mn$di.flow.g2, df.sd$di.flow.g2, NA)
        mat.g2 <- rbind(mat.g2, matrix(stats, byrow = TRUE,
                                       nrow = length(stats) / 3))
      }
      mat <- cbind(mat, mat.g2)
    }

  } # end SI summary


  if (type == "SIR") {

    ## Prevalence calcs
    s.prev <- df.mn$s.num / df.mn$num
    i.prev <- df.mn$i.num / df.mn$num
    r.prev <- df.mn$r.num / df.mn$num
    if (groups == 2) {
      s.prev.g2 <- df.mn$s.num.g2 / df.mn$num.g2
      i.prev.g2 <- df.mn$i.num.g2 / df.mn$num.g2
      r.prev.g2 <- df.mn$r.num.g2 / df.mn$num.g2
    }

    ## Group 1 stats
    stats <- c(df.mn$s.num, df.sd$s.num, s.prev,
               df.mn$i.num, df.sd$i.num, i.prev,
               df.mn$r.num, df.sd$r.num, r.prev,
               df.mn$num, df.sd$num, 1,
               df.mn$si.flow, df.sd$si.flow, NA,
               df.mn$ir.flow, df.sd$ir.flow, NA)
    mat <- matrix(stats, byrow = TRUE, nrow = length(stats) / 3)
    rownames(mat) <- c("Suscept.", "Infect.", "Recov.", "Total",
                       "S -> I", "I -> R")
    if (vital == TRUE) {
      stats <- c(df.mn$a.flow, df.sd$a.flow, NA,
                 df.mn$ds.flow, df.sd$ds.flow, NA,
                 df.mn$di.flow, df.sd$di.flow, NA,
                 df.mn$dr.flow, df.sd$dr.flow, NA)
      mat <- rbind(mat, matrix(stats, byrow = TRUE, nrow = length(stats) / 3))
      rownames(mat)[rownames(mat) == ""] <- c("Arrival ->",
                                              "S Departure ->",
                                              "I Departure ->",
                                              "R Departure ->")
    }

    ## Group 2 stats
    if (groups == 2) {
      stats <- c(df.mn$s.num.g2, df.sd$s.num.g2, s.prev.g2,
                 df.mn$i.num.g2, df.sd$i.num.g2, i.prev.g2,
                 df.mn$r.num.g2, df.sd$r.num.g2, r.prev.g2,
                 df.mn$num.g2, df.sd$num.g2, 1,
                 df.mn$si.flow.g2, df.sd$si.flow.g2, NA,
                 df.mn$ir.flow.g2, df.sd$ir.flow.g2, NA)
      mat.g2 <- matrix(stats, byrow = TRUE, nrow = length(stats) / 3)
      if (vital == TRUE) {
        stats <- c(df.mn$a.flow.g2, df.sd$a.flow.g2, NA,
                   df.mn$ds.flow.g2, df.sd$ds.flow.g2, NA,
                   df.mn$di.flow.g2, df.sd$di.flow.g2, NA,
                   df.mn$dr.flow.g2, df.sd$dr.flow.g2, NA)
        mat.g2 <- rbind(mat.g2, matrix(stats, byrow = TRUE,
                                       nrow = length(stats) / 3))
      }
      mat <- cbind(mat, mat.g2)
    }
  } # end SIR summary


  if (type == "SIS") {

    ## Prevalence calcs
    s.prev <- df.mn$s.num / df.mn$num
    i.prev <- df.mn$i.num / df.mn$num
    if (groups == 2) {
      s.prev.g2 <- df.mn$s.num.g2 / df.mn$num.g2
      i.prev.g2 <- df.mn$i.num.g2 / df.mn$num.g2
    }

    ## Group 1 stats
    stats <- c(df.mn$s.num, df.sd$s.num, s.prev,
               df.mn$i.num, df.sd$i.num, i.prev,
               df.mn$num, df.sd$num, 1,
               df.mn$si.flow, df.sd$si.flow, NA,
               df.mn$is.flow, df.sd$is.flow, NA)
    mat <- matrix(stats, byrow = TRUE, nrow = length(stats) / 3)
    rownames(mat) <- c("Suscept.", "Infect.", "Total", "S -> I", "I -> S")
    if (vital == TRUE) {
      stats <- c(df.mn$a.flow, df.sd$a.flow, NA,
                 df.mn$ds.flow, df.sd$ds.flow, NA,
                 df.mn$di.flow, df.sd$di.flow, NA)
      mat <- rbind(mat, matrix(stats, byrow = TRUE, nrow = length(stats) / 3))
      rownames(mat)[rownames(mat) == ""] <- c("Arrival ->", "S Departure ->",
                                              "I Departure ->")
    }

    ## Group 2 stats
    if (groups == 2) {
      stats <- c(df.mn$s.num.g2, df.sd$s.num.g2, s.prev.g2,
                 df.mn$i.num.g2, df.sd$i.num.g2, i.prev.g2,
                 df.mn$num.g2, df.sd$num.g2, 1,
                 df.mn$si.flow.g2, df.sd$si.flow.g2, NA,
                 df.mn$is.flow.g2, df.sd$is.flow.g2, NA)
      mat.g2 <- matrix(stats, byrow = TRUE, nrow = length(stats) / 3)
      if (vital == TRUE) {
        stats <- c(df.mn$a.flow.g2, df.sd$a.flow.g2, NA,
                   df.mn$ds.flow.g2, df.sd$ds.flow.g2, NA,
                   df.mn$di.flow.g2, df.sd$di.flow.g2, NA)
        mat.g2 <- rbind(mat.g2, matrix(stats, byrow = TRUE,
                                       nrow = length(stats) / 3))
      }
      mat <- cbind(mat, mat.g2)
    }
  } # end SIS summary

  if (groups == 1) colnames(mat) <- c("mean", "sd", "pct")
  if (groups == 2) colnames(mat) <- c("mean:g1", "sd:g1", "pct:g1",
                                      "mean:g2", "sd:g2", "pct:g2")
  mat <- round(mat, digits)

  ## Print it
  cat("EpiModel Summary")
  cat("\n=======================")
  cat("\nModel class:", class(object))

  cat("\n\nSimulation Details")
  cat("\n-----------------------")
  cat("\nModel type:", type)
  cat("\nNo. simulations:", nsims)
  cat("\nNo. time steps:", nsteps)
  cat("\nNo. groups:", groups)

  if (groups == 1) {
    statsep <- paste(rep("-", 30), collapse = "")
  }
  if (groups == 2) {
    statsep <- paste(rep("-", 60), collapse = "")
  }
  cat("\n\nModel Statistics\n")
  cat(statsep)
  cat("\nTime:", at, "\n")
  cat(statsep, "\n")
  print(mat, print.gap = 2)
  cat(statsep, "\n")

}


#' @title Summary Model Statistics
#'
#' @description Extracts and prints model statistics simulated with
#' `netsim`.
#'
#' @param object An `EpiModel` object of class `netsim`.
#' @param at Time step for model statistics.
#' @param digits Number of significant digits to print.
#' @param ... Additional summary function arguments.
#'
#' @details
#' This function provides summary statistics for the main epidemiological
#' outcomes (state and transition size and prevalence) from a `netsim`
#' model. Time-specific summary measures are provided, so it is necessary to
#' input a time of interest.
#'
#' @seealso [netsim()]
#'
#' @method summary netsim
#' @keywords extract
#' @export
#'
#' @examples
#' \dontrun{
#' ## SI Model without Network Feedback
#' # Initialize network and set network model parameters
#' nw <- network_initialize(n = 100)
#' nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#'
#' # Estimate the ERGM models (see help for netest)
#' est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' # Parameters, initial conditions, and controls for model
#' param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
#' init <- init.net(i.num = 10, i.num.g2 = 10)
#' control <- control.net(type = "SI", nsteps = 100, nsims = 5, verbose.int = 0)
#'
#' # Run the model simulation
#' mod <- netsim(est1, param, init, control)
#'
#' summary(mod, at = 1)
#' summary(mod, at = 50)
#' summary(mod, at = 100)
#' }
#'
summary.netsim <- function(object, at, digits = 3, ...) {

  nsims <- object$control$nsims
  type <- object$control$type
  groups <- object$param$groups
  vital <- object$param$vital
  nsteps <- object$control$nsteps

  if (at > nsteps || at < 1) {
    stop("Specify at between 1 and ", nsteps)
  }

  df.mn <- as.data.frame(object, out = "mean")
  df.mn <- df.mn[df.mn$time == at, ]

  df.sd <- as.data.frame(object, out = "sd")
  df.sd <- df.sd[df.sd$time == at, ]

  if (type == "SI") {

    ## Prevalence calcs
    s.prev <- df.mn$s.num / df.mn$num
    i.prev <- df.mn$i.num / df.mn$num
    if (groups == 2) {
      s.prev.g2 <- df.mn$s.num.g2 / df.mn$num.g2
      i.prev.g2 <- df.mn$i.num.g2 / df.mn$num.g2
    }

    ## Group 1 stats
    stats <- c(df.mn$s.num, df.sd$s.num, s.prev,
               df.mn$i.num, df.sd$i.num, i.prev,
               df.mn$num, df.sd$num, 1,
               df.mn$si.flow, df.sd$si.flow, NA)
    mat <- matrix(stats, byrow = TRUE, nrow = length(stats) / 3)
    rownames(mat) <- c("Suscept.", "Infect.", "Total", "S -> I")
    if (vital == TRUE) {
      stats <- c(df.mn$a.flow, df.sd$a.flow, NA,
                 df.mn$ds.flow, df.sd$ds.flow, NA,
                 df.mn$di.flow, df.sd$di.flow, NA)
      mat <- rbind(mat, matrix(stats, byrow = TRUE, nrow = length(stats) / 3))
      rownames(mat)[rownames(mat) == ""] <- c("Arrival ->", "S Departure ->",
                                              "I Departure ->")
    }

    ## Group 2 stats
    if (groups == 2) {
      stats <- c(df.mn$s.num.g2, df.sd$s.num.g2, s.prev.g2,
                 df.mn$i.num.g2, df.sd$i.num.g2, i.prev.g2,
                 df.mn$num.g2, df.sd$num.g2, 1,
                 df.mn$si.flow.g2, df.sd$si.flow.g2, NA)
      mat.g2 <- matrix(stats, byrow = TRUE, nrow = 4)
      if (vital == TRUE) {
        stats <- c(df.mn$a.flow.g2, df.sd$a.flow.g2, NA,
                   df.mn$ds.flow.g2, df.sd$ds.flow.g2, NA,
                   df.mn$di.flow.g2, df.sd$di.flow.g2, NA)
        mat.g2 <- rbind(mat.g2, matrix(stats, byrow = TRUE,
                                       nrow = length(stats) / 3))
      }
      mat <- cbind(mat, mat.g2)
    }

  } ## end SI summary


  if (type == "SIR") {

    ## Prevalence calcs
    s.prev <- df.mn$s.num / df.mn$num
    i.prev <- df.mn$i.num / df.mn$num
    r.prev <- df.mn$r.num / df.mn$num
    if (groups == 2) {
      s.prev.g2 <- df.mn$s.num.g2 / df.mn$num.g2
      i.prev.g2 <- df.mn$i.num.g2 / df.mn$num.g2
      r.prev.g2 <- df.mn$r.num.g2 / df.mn$num.g2
    }

    ## Group 1 stats
    stats <- c(df.mn$s.num, df.sd$s.num, s.prev,
               df.mn$i.num, df.sd$i.num, i.prev,
               df.mn$r.num, df.sd$r.num, r.prev,
               df.mn$num, df.sd$num, 1,
               df.mn$si.flow, df.sd$si.flow, NA,
               df.mn$ir.flow, df.sd$ir.flow, NA)
    mat <- matrix(stats, byrow = TRUE, nrow = length(stats) / 3)
    rownames(mat) <- c("Suscept.", "Infect.", "Recov.", "Total",
                       "S -> I", "I -> R")
    if (vital == TRUE) {
      stats <- c(df.mn$a.flow, df.sd$a.flow, NA,
                 df.mn$ds.flow, df.sd$ds.flow, NA,
                 df.mn$di.flow, df.sd$di.flow, NA,
                 df.mn$dr.flow, df.sd$dr.flow, NA)
      mat <- rbind(mat, matrix(stats, byrow = TRUE, nrow = length(stats) / 3))
      rownames(mat)[rownames(mat) == ""] <- c("Arrival ->",
                                              "S Departure ->",
                                              "I Departure ->",
                                              "R Departure ->")
    }

    ## Group 2 stats
    if (groups == 2) {
      stats <- c(df.mn$s.num.g2, df.sd$s.num.g2, s.prev.g2,
                 df.mn$i.num.g2, df.sd$i.num.g2, i.prev.g2,
                 df.mn$r.num.g2, df.sd$r.num.g2, r.prev.g2,
                 df.mn$num.g2, df.sd$num.g2, 1,
                 df.mn$si.flow.g2, df.sd$si.flow.g2, NA,
                 df.mn$ir.flow.g2, df.sd$ir.flow.g2, NA)
      mat.g2 <- matrix(stats, byrow = TRUE, nrow = length(stats) / 3)
      if (vital == TRUE) {
        stats <- c(df.mn$a.flow.g2, df.sd$a.flow.g2, NA,
                   df.mn$ds.flow.g2, df.sd$ds.flow.g2, NA,
                   df.mn$di.flow.g2, df.sd$di.flow.g2, NA,
                   df.mn$dr.flow.g2, df.sd$dr.flow.g2, NA)
        mat.g2 <- rbind(mat.g2, matrix(stats, byrow = TRUE,
                                       nrow = length(stats) / 3))
      }
      mat <- cbind(mat, mat.g2)
    }
  }


  if (type == "SIS") {

    ## Prevalence calcs
    s.prev <- df.mn$s.num / df.mn$num
    i.prev <- df.mn$i.num / df.mn$num
    if (groups == 2) {
      s.prev.g2 <- df.mn$s.num.g2 / df.mn$num.g2
      i.prev.g2 <- df.mn$i.num.g2 / df.mn$num.g2
    }

    ## Group 1 stats
    stats <- c(df.mn$s.num, df.sd$s.num, s.prev,
               df.mn$i.num, df.sd$i.num, i.prev,
               df.mn$num, df.sd$num, 1,
               df.mn$si.flow, df.sd$si.flow, NA,
               df.mn$is.flow, df.sd$is.flow, NA)
    mat <- matrix(stats, byrow = TRUE, nrow = length(stats) / 3)
    rownames(mat) <- c("Suscept.", "Infect.", "Total",
                       "S -> I", "I -> S")
    if (vital == TRUE) {
      stats <- c(df.mn$a.flow, df.sd$a.flow, NA,
                 df.mn$ds.flow, df.sd$ds.flow, NA,
                 df.mn$di.flow, df.sd$di.flow, NA)
      mat <- rbind(mat, matrix(stats, byrow = TRUE, nrow = length(stats) / 3))
      rownames(mat)[rownames(mat) == ""] <- c("Arrival ->", "S Departure ->",
                                              "I Departure ->")
    }

    ## Group 2 stats
    if (groups == 2) {
      stats <- c(df.mn$s.num.g2, df.sd$s.num.g2, s.prev.g2,
                 df.mn$i.num.g2, df.sd$i.num.g2, i.prev.g2,
                 df.mn$num.g2, df.sd$num.g2, 1,
                 df.mn$si.flow.g2, df.sd$si.flow.g2, NA,
                 df.mn$is.flow.g2, df.sd$is.flow.g2, NA)
      mat.g2 <- matrix(stats, byrow = TRUE, nrow = length(stats) / 3)
      if (vital == TRUE) {
        stats <- c(df.mn$a.flow.g2, df.sd$a.flow.g2, NA,
                   df.mn$ds.flow.g2, df.sd$ds.flow.g2, NA,
                   df.mn$di.flow.g2, df.sd$di.flow.g2, NA)
        mat.g2 <- rbind(mat.g2, matrix(stats, byrow = TRUE,
                                       nrow = length(stats) / 3))
      }
      mat <- cbind(mat, mat.g2)
    }
  }

  if (groups == 1) {
    colnames(mat) <- c("mean", "sd", "pct")
  }
  if (groups == 2) {
    colnames(mat) <- c("mean:g1", "sd:g1", "pct:g1",
                       "mean:g2", "sd:g2", "pct:g2")
  }
  mat <- round(mat, digits)

  ## Print it
  cat("\nEpiModel Summary")
  cat("\n=======================")
  cat("\nModel class:", class(object))

  cat("\n\nSimulation Details")
  cat("\n-----------------------")
  cat("\nModel type:", type)
  cat("\nNo. simulations:", nsims)
  cat("\nNo. time steps:", nsteps)
  cat("\nNo. NW groups:", groups)

  if (groups == 1) {
    statsep <- paste(rep("-", 30), collapse = "")
  }
  if (groups == 2) {
    statsep <- paste(rep("-", 60), collapse = "")
  }

  cat("\n\nModel Statistics\n")
  cat(statsep)
  cat("\nTime:", at, "\n")
  cat(statsep, "\n")
  print(mat, print.gap = 2)
  cat(statsep, "\n")

}


#' @title Summary for Network Model Fit
#'
#' @description Prints the summary model fit statistics for an ERGM or STERGM
#' fit.
#'
#' @param object An `EpiModel` object of class `netest`.
#' @param ... Additional summary function arguments.
#'
#' @method summary netest
#' @keywords internal
#' @export
#'
#' @details
#' This function is simply a wrapper function for `summary.ergm`.
#' Additionally, if the edges dissolution approximation was used to fit the
#' temporal ERGM, then the dissolution coefficient information will be printed.
#'
#' If the `fit` object is attached to the `netest` object, then
#' `summary.netest` will call `summary` on `fit` using the
#' `...` passed to `summary.netest`.  Otherwise,
#' `summary.netest` will print the stored summary of the fit generated
#' in the original `netest` call, using the `...` passed to
#' `netest`.
#'
summary.netest <- function(object, ...) {

  if (!is.null(object$fit)) {
    print(summary(object$fit, ...))
  } else {
    print(object$summary)
  }

  if (object$edapprox == TRUE) {
    cat("\n")
    print(object$coef.diss)
  }

}

#' Make a list of EpiModel scenarios from a data.frame of scenarios
#'
#' An EpiModel scenario allows one or multiple set of parameters to be applied
#' to a model a predefined timesteps. They are usually used by a researcher who
#' wants to model counterfactuals using a pre calibrated model.
#'
#' @param scenarios.df a `data.frame`
#'
#' @return a list of EpiModel scenarios
#'
#' @section scenarios.df:
#' The `scenarios.df` is a `data.frame` of values to be used as
#' parameters.
#'
#' It must contain a ".at" column, specifying when the changes should occur.
#' It requires the "updater" module of EpiModel. *See, vignette*. If the ".at"
#' value of a row is less than two, the changes will be applied to the
#' parameter list iteself. The second mandatory column is ".scenario.id". It
#' is used to distinguish the different scenarios. If multiple rows share the
#' same ".scenario.id", the resulting scenario will contain one updater per row.
#' This permits modifying parameters at multiple points in time. (e.g. an
#' intervention limited in time).
#'
#' The other column names must correspond either to:
#' the name of one parameter if this parameter is of size 1 or the name of the
#' parameter with "_1", "_2", "_N" with the second part being the position of
#' the value for a parameter of size > 1. This means that the parameter names
#' cannot contain any underscore "_". (e.g "a.rate", "d.rate_1", "d.rate_2")
#'
#' @export
create_scenario_list <- function(scenarios.df) {
  check_scenarios_df(scenarios.df)
  scenarios.names <- unique(scenarios.df[[".scenario.id"]])

  scenarios <- lapply(
    scenarios.names,
    function(id) {
      make_scenario(
                    scenarios.df[scenarios.df[[".scenario.id"]] == id, ])
    }
  )

  names(scenarios) <- scenarios.names

  return(scenarios)
}

#' a scenario is a list with the following elements:
#'   - `id`: the identifier of the scenario
#'   - `.param.updater.list`: as in the updater module
#'
#' when running "use_scenario", if any updater has an `at` value < 2, it is
#' applied immediatly (so before netsim)
#'
#' @noRd
make_scenario <- function(scenario.rows) {
  scenario <- list(
    id = scenario.rows[[".scenario.id"]][1],
    .param.updater.list = vector(mode = "list", length = nrow(scenario.rows))
  )

  elements.at <- scenario.rows[[".at"]]
  # ensures that `scenario.rows` is a `tibble`
  scenario.rows <- dplyr::select(scenario.rows, - c(".at", ".scenario.id"))

  for (i in seq_along(elements.at)) {
    scenario[[".param.updater.list"]][[i]] <- list(
      at = elements.at[[i]],
      param = unflatten_params(scenario.rows[i, ])
    )
  }

  return(scenario)
}

#' Apply a scenario object to a param.net object
#'
#' @param scenario a scenario object usually created from a `data.frame` of
#' scenarios using the `create_scenario_list` function. See the vignette
#' "network-model-scenarios".
#'
#' @section scenario:
#' A scenario is a list containing an "id" field, the name of the scenario and
#' a ".param.updater.list" containing a list of updaters that modifies the
#' parameters of the model at given time steps. If a scenario contains a
#' parameter not defined in the `param` object, an error will be produced.
#' See the vignette "model-parameters" for the technical detail of their
#' implementation.
#'
#' @inheritParams update_params
#' @inherit update_params return
#'
#' @export
use_scenario <- function(param, scenario) {
  scenario.params <- unique(unlist(lapply(
    scenario[[".param.updater.list"]],
    function(element) names(element[["param"]])
  )))

  undef.params <- setdiff(scenario.params, names(param))
  if (length(undef.params) > 0) {
    stop("Some of the scenario parameters are not defined in `param`: \n'",
         paste0(undef.params, collapse = "', '"), "'")
  }

  elements.at <- vapply(
    scenario[[".param.updater.list"]],
    function(element) element[["at"]],
    numeric(1)
  )

  for (i in which(elements.at < 2)) {
    param <- update_params(param,
                           scenario[[".param.updater.list"]][[i]][["param"]])
  }

  param[[".param.updater.list"]] <- c(
    param[[".param.updater.list"]],
    scenario[[".param.updater.list"]][elements.at >= 2]
  )

  param[[".scenario.id"]] <- scenario[["id"]]

  return(param)
}

#' Helper function validating the format of a `scenarios.df`
#' @noRd
check_scenarios_df <- function(scenarios.df) {
  checks <-
    all(c(".scenario.id", ".at") %in% names(scenarios.df)) &&
    is.numeric(scenarios.df[[".at"]]) &&
    all(as.integer(scenarios.df[[".at"]]) == scenarios.df[[".at"]])

  if (!checks) {
    stop(
      "A `data.frame` of scenarios must have a '.scenario.id' column \n",
      "and a '.at' column containing integers."
    )
  }
}

#' helper function to make a ragged param list into a flat one
#' @noRd
flatten_params <- function(params) {
  params <- remove_special_params(params)
  params.length <- vapply(params, length, 0)
  params.n <- sum(params.length)
  params.flat <- vector(mode = "list", length = params.n)

  i <- 1
  n <- 1
  while (n <= params.n) {
    l <- params.length[i]
    cur.param <- as.list(params[[i]])
    cur.name <- names(params)[i]
    if (l > 1)
      cur.name <- paste0(cur.name, "_", seq_len(l))
    params.flat[n:(n + l - 1)] <- cur.param
    names(params.flat)[n:(n + l - 1)] <- cur.name

    n <- n + l
    i <- i + 1
  }
  return(params.flat)
}

#' list the "special parameters" from a param list. They include some EpiModel
#' internals as well as all parameters starting with "."
#' @noRd
list_special_params <- function(params) {
  builtin.special.params <- c(
    "random.params",
    "random.params.values"
  )

  builtin.special.params <- intersect(builtin.special.params, names(params))
  dot.special.params <- names(params)[grep("^\\.", names(params))]

  return(unique(c(builtin.special.params, dot.special.params)))
}

#' helper function to remove the "special parameters" from a param list.
#' @noRd
remove_special_params <- function(params) {
  special.params.names <- list_special_params(params)
  params[!names(params) %in% special.params.names]
}

#' helper function to make a flat param list into a ragged one
#' @noRd
unflatten_params <- function(params.flat) {
  check_params_names(names(params.flat))
  check_params_flat(params.flat)

  # get the param name and position for each element.
  # With pos == 1 if none provided
  elts <- strsplit(names(params.flat), "_")
  set.elements.names <- vapply(elts, function(x) x[1], "")
  set.elements.pos <- vapply(elts, function(x) {
    as.numeric(if (length(x) == 2) x[2] else 1)
  }, numeric(1))

  params.names <- unique(set.elements.names)
  params <- vector(mode = "list", length = length(params.names))
  names(params) <- params.names

  for (i in seq_along(set.elements.names)) {
    nme <- set.elements.names[[i]]
    params[[nme]][set.elements.pos[i]] <- params.flat[[i]]
  }

  return(params)
}

#' helper function to check if a list of flat parameters is actually flat
#' @noRd
check_params_flat <- function(params.flat) {
  params.length <- vapply(params.flat, length, 0)
  params.list <- vapply(params.flat, is.list, TRUE)
  if (any(params.length != 1) || any(params.list)) {
    stop("A flat parameter list should contain only length 1 non list elements")
  }
  invisible(TRUE)
}

#' helper function to check the correctness of the flat parameters names
#' @noRd
check_params_names <- function(params.names) {
  params.pattern <- "^[[:alpha:]][[:alnum:].]*(_[1-9]+)?$"
  correct.format <- grepl(params.pattern, params.names)

  if (!all(correct.format)) {
    stop("The following flat parameter names are malformed: \n`",
      paste0(params.names[!correct.format], collapse = "`, `"), "`\n\n",
      "you can check the names with ",
      '`grepl("', params.pattern, '", your.names)` \n',
      "Example: 'unique.param', 'param.set_1', 'param.set_2'"
    )
  }

  special.params.names <- list_special_params(params.names)
  if (length(special.params.names) != 0) {
    stop("The following special parameter names are not allowed: \n`",
      paste0(special.params.names, collapse = "`, `"), "`\n\n"
    )
  }

  invisible(return(TRUE))
}

#' @title Get the List of Modules
#'
#' @inheritParams recovery.net
#'
#' @return A named list of modules to be run by the model
#'
#' @keywords internal
get_modules <- function(dat) {
  dat[["_modules_list"]]
}


#' @title Set the List of Modules
#'
#' @inheritParams recovery.net
#' @param modules A named list of modules to be run by the model
#'
#' @inherit recovery.net return
#'
#' @keywords internal
set_modules <- function(dat, modules) {
  dat[["_modules_list"]] <- modules
  return(dat)
}

#' @title Populate the Module List After the Initialization Module is run
#'
#' @inheritParams recovery.net
#' @inherit recovery.net return
#'
#' @keywords internal
make_module_list <- function(dat) {
  ## Module order
  morder <- get_control(dat, "module.order", override.null.error = TRUE)
  if (is.null(morder)) {
    bi.mods <- get_control(dat, "bi.mods")
    user.mods <- get_control(dat, "user.mods")
    lim.bi.mods <- bi.mods[
      -which(bi.mods %in% c("initialize.FUN", "verbose.FUN"))
    ]
    morder <- c(user.mods, lim.bi.mods)
  }

  ## Make the `modules` list
  modules <- vector(mode = "list", length = length(morder))
  for (i in seq_along(morder)) {
    modules[[i]] <- get_control(dat, morder[i])
  }
  names(modules) <- morder
  dat <- set_modules(dat, modules)

  return(dat)
}

#' @title Remove a Set of Modules From the Module List
#'
#' @inheritParams recovery.net
#' @param names.to.remove a character vector containing the name of the modules
#'   to remove.
#' @inherit recovery.net return
#'
#' @keywords internal
remove_modules <- function(dat, names.to.remove) {
  modules <- get_modules(dat)
  at <- get_current_timestep(dat)

  # Trying to remove modules not present => warning
  missing_mods <- setdiff(names.to.remove, names(modules))
  if (length(missing_mods) > 0) {
    stop(
      "\n\nAt timestep = ", at, ":\n",
      "    an attempt was made to remove the following modules: \n'",
      paste0(missing_mods, collapse = "', '"), "'\n",
      "But they were not in the modules list."
    )

    names.to.remove <- intersect(names.to.remove, names(modules))
  }

  # In any case => message
  modules <- modules[!names(modules) %in% names.to.remove]
  message(
    "\n\nAt timestep = ", at, ":\n",
    "    the following modules were removed: \n'",
    paste0(names.to.remove, collapse = "', '"), "'\n"
  )

  dat <- set_modules(dat, modules)
  return(dat)
}

#' @title Initialize Networks Used in netsim
#'
#' @description This function initializes the networks used in
#'              [netsim()]. The initial edge set for a given network
#'              is obtained either from simulating the cross-sectional model
#'              (if `edapprox == TRUE`) or from the `newnetwork`
#'              element of the `netest` object (if
#'              `edapprox == FALSE`). Once the initial edge sets are
#'              determined, the first time step is simulated if
#'              `resimulate.network == TRUE`, and all time steps are
#'              simulated if `resimulate.network == FALSE`. Initializes the
#'              `num(.g2)` epi fields used in
#'              [edges_correct()] for computing edge coefficient
#'              adjustments.
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @export
#' @keywords netUtils internal
#'
sim_nets_t1 <- function(dat) {
  dat.updates <- NVL(get_control(dat, "dat.updates"), function(dat, ...) dat)

  ## simulate zeroth timestep cross-sectional network
  ## (default to newnetwork, set already in mod.initialize, if full tergm fit)
  dat <- dat.updates(dat = dat, at = 0L, network = 0L)
  for (network in seq_len(dat$num.nw)) {
    nwparam <- get_nwparam(dat, network = network)

    ## (re)construct input network
    nw <- get_network(dat, network)

    ## simulate t0 basis network nw if using edapprox
    if (nwparam$edapprox == TRUE) {
      nw <- simulate(nwparam$formula,
                     coef = nwparam$coef.form.crude,
                     basis = nw,
                     constraints = nwparam$constraints,
                     control = get_network_control(dat, network, "set.control.ergm"),
                     dynamic = FALSE)
    }

    if (get_control(dat, "tergmLite") == TRUE) {
      ## set up time and lasttoggle if tracking duration
      if (get_network_control(dat, network, "tergmLite.track.duration") == TRUE) {
        nw %n% "time" <- 0L
        nw %n% "lasttoggle" <- cbind(as.edgelist(nw), 0L)
      }
    } else {
      ## set up vertex and edge activity in networkDynamic case
      nw <- networkDynamic::as.networkDynamic(nw)
      nw <- networkDynamic::activate.vertices(nw, onset = 0L, terminus = Inf)
      nw <- networkDynamic::activate.edges(nw, onset = 0L, terminus = Inf)
      if (get_control(dat, "resimulate.network") == TRUE) {
        nw %n% "net.obs.period" <- list(observations = list(c(0, 1)),
                                        mode = "discrete",
                                        time.increment = 1,
                                        time.unit = "step")
      }
    }
    ## set network on dat object
    dat <- set_network(dat, network, nw)
    ## update dat object as needed
    dat <- dat.updates(dat = dat, at = 0L, network = network)
  }

  if (get_control(dat, "resimulate.network") == TRUE) {
    nsteps <- 1L
  } else {
    nsteps <- get_control(dat, "nsteps")
  }

  ## initialize num(.g2) run fields
  dat <- update_sim_num(dat)

  ## In tergmLite mode the post-simulate state replaces dat$run$el and the
  ## cross-section edgelist is no longer recoverable. Stash it now so
  ## seed_cumulative_edgelist_t1() can diff t=0 vs t=1 below. See #1016.
  if (get_control(dat, "tergmLite") && get_control(dat, "cumulative.edgelist")) {
    dat$run$el_t0_seed <- lapply(seq_len(dat$num.nw), function(network) {
      dat$run$el[[network]]
    })
  }

  ## simulate first timestep (if resimulate.network == TRUE)
  ## or all timesteps (if resimulate.network == FALSE)
  dat <- dat.updates(dat = dat, at = 1L, network = 0L)
  for (network in seq_len(dat$num.nw)) {
    dat <- simulate_dat(dat, at = 1L, network = network, nsteps = nsteps)
    dat <- dat.updates(dat = dat, at = 1L, network = network)
  }

  dat <- seed_cumulative_edgelist_t1(dat)

  return(dat)
}

#' @title Simulate a Network for a Specified Number of Time Steps
#'
#' @description This function simulates a dynamic network over one or multiple
#'              time steps for TERGMs or one or multiple cross-sectional network
#'              panels for ERGMs, for use in [netsim()] modeling.
#'              Network statistics are also extracted and saved if
#'              `save.nwstats == TRUE` and
#'              `resimulate.network == FALSE`.
#'
#' @inheritParams recovery.net
#' @param network index of the network to simulate
#' @param nsteps number of time steps to simulate
#'
#' @inherit recovery.net return
#'
#' @export
#' @keywords netUtils internal
#'
simulate_dat <- function(dat, at, network = 1L, nsteps = 1L) {
  ## determine formula and coefficients; set discordance_fraction in ergm case
  nwparam <- get_nwparam(dat, network = network)
  simulation_control <- get_network_control(dat, network, "set.control.tergm")
  if (nwparam$coef.diss$duration[1] > 1) {
    formula <- ~Form(nwparam$formation) +
      Persist(nwparam$coef.diss$dissolution)
    coef <- c(nwparam$coef.form, nwparam$coef.diss$coef.adj)
  } else {
    formula <- nwparam$formation
    coef <- nwparam$coef.form
    simulation_control$MCMC.prop.args <- list(discordance_fraction = 0)
  }

  ## determine output type
  if (get_control(dat, "tergmLite") == FALSE) {
    output <- "networkDynamic"
  } else {
    output <- "final"
  }

  ## determine monitor, if needed; note that we only obtain
  ## stats in simulate_dat if resimulate.network == FALSE
  if (get_control(dat, "save.nwstats") == TRUE &&
        get_control(dat, "resimulate.network") == FALSE) {
    monitor <- get_network_control(dat, network, "nwstats.formula")
  } else {
    monitor <- NULL # will be handled by summary_nets, if needed
  }

  if (get_control(dat, "tergmLite") == FALSE &&
        get_control(dat, "resimulate.network") == TRUE) {
    time_offset <- 0L
  } else {
    time_offset <- 1L
  }

  ## always TERGM simulation
  nw <- suppressWarnings(simulate(formula,
                                  coef = coef,
                                  basis = get_network(x = dat, network = network),
                                  constraints = nwparam$constraints,
                                  time.start = at - time_offset,
                                  time.offset = time_offset,
                                  time.slices = nsteps,
                                  output = output,
                                  control = simulation_control,
                                  monitor = monitor,
                                  dynamic = TRUE))

  ## update network (and el, if tergmLite) on the dat object
  dat <- set_network(x = dat, network = network, nw = nw)

  ## if monitor was used, record the results
  if (!is.null(monitor)) {
    new.nwstats <- attributes(nw)$stats
    keep.cols <- which(!duplicated(colnames(new.nwstats)))
    new.nwstats <- new.nwstats[, keep.cols, drop = FALSE]
    dat$stats$nwstats[[network]] <- list(as_tibble(new.nwstats))
  }

  return(dat)
}

#' @title Resimulate Dynamic Network at Time 2+
#'
#' @description This function resimulates the dynamic network in stochastic
#'              network models simulated in [netsim()] with dependence
#'              between the epidemic and demographic processes and the network
#'              structure.
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @export
#' @keywords netUtils internal
#'
resim_nets <- function(dat, at) {

  # Calculate active attribute
  active <- get_attr(dat, "active")
  idsActive <- which(active == 1)
  anyActive <- ifelse(length(idsActive) > 0, TRUE, FALSE)
  if (dat$param$groups == 2) {
    group <- get_attr(dat, "group")
    groupids.1 <- which(group == 1)
    groupids.2 <- which(group == 2)
    nActiveG1 <- length(intersect(groupids.1, idsActive))
    nActiveG2 <- length(intersect(groupids.2, idsActive))
    anyActive <- ifelse(nActiveG1 > 0 & nActiveG2 > 0, TRUE, FALSE)
  }

  # Network resimulation, with dat.updates interspersed
  if (anyActive == TRUE && get_control(dat, "resimulate.network") == TRUE) {
    ## Edges Correction
    dat <- edges_correct(dat, at)
    dat <- update_sim_num(dat)

    ## network resimulation
    dat.updates <- NVL(get_control(dat, "dat.updates"), function(dat, ...) dat)
    dat <- dat.updates(dat = dat, at = at, network = 0L)
    for (network in seq_len(dat$num.nw)) {
      dat <- simulate_dat(dat = dat, at = at, network = network)
      dat <- dat.updates(dat = dat, at = at, network = network)
    }
  }

  # Cummulative edgelist
  truncate.el.cuml <- get_control(dat, "truncate.el.cuml")
  for (network in seq_len(dat$num.nw)) {
    dat <- update_cumulative_edgelist(dat, network, truncate.el.cuml)
  }

  return(dat)
}


#' @title Adjustment for the Edges Coefficient with Changing Network Size
#'
#' @description Adjusts the edges coefficient in a dynamic network model
#'              simulated in [netsim()] to preserve the mean
#'              degree of nodes in the network. Requires `at >= 2`.
#'              Maintains the `num(.g2)` epi fields (initialized in
#'              [sim_nets_t1()]) for computing the coefficient
#'              adjustment.
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @keywords internal
#' @export
#'
edges_correct <- function(dat, at) {

  resimulate.network <- get_control(dat, "resimulate.network")
  groups <- get_param(dat, "groups")
  active <- get_attr(dat, "active")

  if (resimulate.network == TRUE) {
    if (groups == 1) {
      old.num <- dat$run$num
      new.num <- sum(active == 1)
      adjustment <- log(old.num) - log(new.num)
    }
    if (groups == 2) {
      old.num.g1 <- dat$run$num
      old.num.g2 <- dat$run$num.g2
      group <- get_attr(dat, "group")
      new.num.g1 <- sum(active == 1 & group == 1)
      new.num.g2 <- sum(active == 1 & group == 2)
      adjustment <-
        log(2 * old.num.g1 * old.num.g2 / (old.num.g1 + old.num.g2)) -
        log(2 * new.num.g1 * new.num.g2 / (new.num.g1 + new.num.g2))
    }

    for (network in seq_len(dat$num.nw)) {
      dat$nwparam[[network]]$coef.form[1] <-
        dat$nwparam[[network]]$coef.form[1] + adjustment
    }
  }

  return(dat)
}

# Store the current number of nodes in the network or per network group in
# `dat$run`.
# This is used to adjuste the `edges` coefficient of tergm
update_sim_num <- function(dat) {
  active <- get_attr(dat, "active")
  if (get_param(dat, "groups") == 1) {
    dat$run$num <- sum(active == 1)
  } else {
    group <- get_attr(dat, "group")
    dat$run$num <- sum(active == 1 & group == 1)
    dat$run$num.g2 <- sum(active == 1 & group == 2)
  }
  return(dat)
}

#' @title Stochastic Individual Contact Models
#'
#' @description Simulates stochastic individual contact epidemic models for
#'              infectious disease.
#'
#' @param param Model parameters, as an object of class [param.icm()].
#' @param init Initial conditions, as an object of class [init.icm()].
#' @param control Control settings, as an object of class
#'        [control.icm()].
#'
#' @details
#' Individual contact models are intended to be the stochastic microsimulation
#' analogs to deterministic compartmental models. ICMs simulate disease spread
#' on individual agents in discrete time as a function of processes with
#' stochastic variation. The stochasticity is inherent in all transition
#' processes: infection, recovery, and demographics.
#'
#' The `icm` function performs modeling of both the base model types
#' and original models. Base model types include one-group and two-group
#' models with disease types for Susceptible-Infected (SI),
#' Susceptible-Infected-Recovered (SIR), and Susceptible-Infected-Susceptible
#' (SIS). Original models may be built by writing new process modules that
#' either take the place of existing modules (for example, disease recovery),
#' or supplement the set of existing processes with a new one contained in an
#' original module.
#'
#' @return
#' A list of class `icm` with the following elements:
#'
#'  * **param:** the epidemic parameters passed into the model through
#'        `param`, with additional parameters added as necessary.
#'  * **control:** the control settings passed into the model through
#'        `control`, with additional controls added as necessary.
#'  * **epi:** a list of data frames, one for each epidemiological
#'        output from the model. Outputs for base models always include the
#'        size of each compartment, as well as flows in, out of, and between
#'        compartments.
#'
#'
#' @keywords model
#'
#' @seealso Extract the model results with [as.data.frame.icm()].
#' Summarize the time-specific model results with [summary.icm()].
#' Plot the model results with [plot.icm()]. Plot a compartment flow
#' diagram with [comp_plot()].
#'
#' @export
#'
#' @examples
#' \dontrun{
#' ## Example 1: SI Model
#' param <- param.icm(inf.prob = 0.2, act.rate = 0.25)
#' init <- init.icm(s.num = 500, i.num = 1)
#' control <- control.icm(type = "SI", nsteps = 500, nsims = 10)
#' mod1 <- icm(param, init, control)
#' mod1
#' plot(mod1)
#'
#' ## Example 2: SIR Model
#' param <- param.icm(inf.prob = 0.2, act.rate = 0.25, rec.rate = 1/50)
#' init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
#' control <- control.icm(type = "SIR", nsteps = 500, nsims = 10)
#' mod2 <- icm(param, init, control)
#' mod2
#' plot(mod2)
#'
#' ## Example 3: SIS Model
#' param <- param.icm(inf.prob = 0.2, act.rate = 0.25, rec.rate = 1/50)
#' init <- init.icm(s.num = 500, i.num = 1)
#' control <- control.icm(type = "SIS", nsteps = 500, nsims = 10)
#' mod3 <- icm(param, init, control)
#' mod3
#' plot(mod3)
#'
#' ## Example 4: SI Model with Vital Dynamics (Two-Group)
#' param <- param.icm(inf.prob = 0.4,  inf.prob.g2 = 0.1,
#'                    act.rate = 0.25, balance = "g1",
#'                    a.rate = 1/100, a.rate.g2 = NA,
#'                    ds.rate = 1/100, ds.rate.g2 = 1/100,
#'                    di.rate = 1/50, di.rate.g2 = 1/50)
#' init <- init.icm(s.num = 500, i.num = 1,
#'                  s.num.g2 = 500, i.num.g2 = 0)
#' control <- control.icm(type = "SI", nsteps = 500, nsims = 10)
#' mod4 <- icm(param, init, control)
#' mod4
#' plot(mod4)
#' }
#'
icm <- function(param, init, control) {
  check.control.class("icm", "EpiModel icm")

  crosscheck.icm(param, init, control)
  verbose.icm(control, type = "startup")

  # Simulation loop start
  for (s in 1:control$nsims) {

    ## Initialization module
    if (!is.null(control[["initialize.FUN"]])) {
      dat <- do.call(control[["initialize.FUN"]], list(param, init, control))
    }


    # Timestep loop
    for (at in 2:control$nsteps) {

      ## Infection
      if (!is.null(control[["infection.FUN"]])) {
        dat <- do.call(control[["infection.FUN"]], list(dat, at))
      }


      ## Recovery
      if (!is.null(control[["recovery.FUN"]])) {
        dat <- do.call(control[["recovery.FUN"]], list(dat, at))
      }


      ## Departure Module
      if (!is.null(control[["departures.FUN"]])) {
        dat <- do.call(control[["departures.FUN"]], list(dat, at))
      }


      ## Arrival Module
      if (!is.null(control[["arrivals.FUN"]])) {
        dat <- do.call(control[["arrivals.FUN"]], list(dat, at))
      }


      ## Outputs
      if (!is.null(control[["prevalence.FUN"]])) {
        dat <- do.call(control[["prevalence.FUN"]], list(dat, at))
      }


      ## Track progress
      verbose.icm(dat, type = "progress", s, at)
    }

    # Set output
    if (s == 1) {
      out <- saveout.icm(dat, s)
    } else {
      out <- saveout.icm(dat, s, out)
    }



  } # Simulation loop end


  class(out) <- "icm"
  return(out)
}

#' @title Initialization: netsim Module
#'
#' @description This function initializes the main `netsim_dat` class data
#'              object on which data are stored, simulates the initial state of
#'              the networks, and simulates disease status and other attributes.
#'
#' @param x If `control$start == 1`, either a fitted network model object
#'        of class `netest` or a list of such objects. If
#'        `control$start > 1`, an object of class `netsim`. When
#'        multiple networks are used, the node sets (including network size
#'        and nodal attributes) are assumed to be the same for all networks.
#' @param param An `EpiModel` object of class [param.net()].
#' @param init An `EpiModel` object of class [init.net()].
#' @param control An `EpiModel` object of class [control.net()].
#' @param s Simulation number, used for restarting dependent simulations.
#' @details When re-initializing a simulation, the `netsim` object passed
#'          to `initialize.net` must contain the elements `param`,
#'          `nwparam`, `epi`, `coef.form`, and `num.nw`.
#'
#' @return A `netsim_dat` class main data object.
#'
#' @export
#' @keywords internal
#'
initialize.net <- function(x, param, init, control, s) {

  if (control$start == 1) {

    # Main Data List --------------------------------------------------------
    dat <- create_dat_object(param, init, control)

    # network and stats initialization
    dat <- init_nets(dat, x)

    ## Store current proportions of attr
    if (!is.null(dat$run$nwterms)) {
      dat$run$t1.tab <- get_attr_prop(dat, dat$run$nwterms)
    }

    # simulate first time step
    dat <- sim_nets_t1(dat)
    dat <- summary_nets(dat, at = 1L)

    ## Infection Status and Time
    dat <- init_status.net(dat)

    # Summary Stats -----------------------------------------------------------
    dat <- do.call(control[["prevalence.FUN"]], list(dat, at = 1))

    # Restart/Reinit Simulations ----------------------------------------------
  } else if (control$start > 1) {
    ## check that required names are present
    required_names <- c("param", "nwparam", "epi", "run", "coef.form", "num.nw")
    missing_names <- setdiff(required_names, names(x))
    if (length(missing_names) > 0) {
      stop(
        "x is missing the following elements required for re-initialization: ",
        paste.and(missing_names)
      )
    }

    # recycle sims in the restart object
    # e.g. 5 sim out of a size 3 restart object we will give: 1, 2, 3, 1, 2
    s <- (s - 1) %% length(x$run) + 1

    dat <- create_dat_object(
      param = param,
      control = control,
      run = x$run[[s]]
    )

    missing_params <- setdiff(names(x$param), names(param))
    for (mp in missing_params) {
      dat <- set_param(dat, mp, x$param[[mp]])
    }

    dat$num.nw <- x$num.nw

    dat$nwparam <- x$nwparam
    for (network in seq_len(dat$num.nw)) {
      dat$nwparam[[network]]$coef.form <- x$coef.form[[s]][[network]]
    }
    dat$epi <- sapply(x$epi, function(var) var[s])
    names(dat$epi) <- names(x$epi)

    dat$stats <- lapply(x$stats, function(var) var[[s]])
    if (get_control(dat, "save.nwstats") == TRUE) {
      nsteps <- get_control(dat, "nsteps")
      start <- get_control(dat, "start")
      dat$stats$nwstats <- lapply(dat$stats$nwstats,
        function(oldstats) padded_vector(list(oldstats), nsteps - start + 2L)
      )
    }
    if (is.data.frame(dat$stats$transmat)) {
      nsteps <- get_control(dat, "nsteps")
      dat$stats$transmat <- padded_vector(list(dat$stats$transmat), nsteps)
    }
  }

  return(dat)
}


#' @title Disease Status Initialization Module for netsim
#'
#' @description This function sets the initial disease status on the
#'              network given the specified initial conditions.
#'
#' @inheritParams recovery.net
#'
#' @details
#' This internal function sets, either randomly or deterministically, the nodes
#' that are infected at \eqn{t_1}, the starting time of network simulations. If
#' the number to be initially infected is passed, this function sets the initial
#' number infected based on the number specified, either as a set of random
#' draws from a binomial distribution or as the exact number specified. In
#' either case, the specific nodes infected are a random sample from the
#' network. In contrast, a set of specific nodes may be infected by passing a
#' vector containing the status of each node to [netsim()].
#'
#' For the initially infected nodes, this module sets the time of infection as
#' \eqn{t_1}, the starting time of network simulations. For models with vital
#' dynamics, the infection time for those initially infected nodes is a random
#' draw from an exponential distribution with the rate parameter defined by the
#' `di.rate` argument. For models without vital dynamics, the infection
#' time is a random draw from a uniform distribution of integers with a minimum
#' of 1 and a maximum of the number of time steps in the model. In both cases,
#' to set the infection times to be in the past, these times are multiplied by
#' -1, and 2 is added to allow for possible infection times up until step 2,
#' when the disease simulation time loop starts.
#'
#' @inherit recovery.net return
#'
#' @seealso This is an initialization module for [netsim()].
#'
#' @export
#' @keywords netMod internal
#'
init_status.net <- function(dat) {

  type <- get_control(dat, "type", override.null.error = TRUE)
  type <- if (is.null(type)) "None" else type

  nsteps <- get_control(dat, "nsteps")
  tergmLite <- get_control(dat, "tergmLite")
  vital <- get_param(dat, "vital")
  groups <- get_param(dat, "groups")
  status.vector <- get_init(dat, "status.vector", override.null.error = TRUE)
  if (type %in% c("SIS", "SIR")) {
    rec.rate <- get_param(dat, "rec.rate")
  }
  if (vital == TRUE) {
    di.rate <- get_param(dat, "di.rate")
  }

  # Variables ---------------------------------------------------------------
  i.num <- get_init(dat, "i.num", override.null.error = TRUE)
  if (type  == "SIR" && is.null(status.vector)) {
    r.num <- get_init(dat, "r.num")
  }

  num <- sum(get_attr(dat, "active") == 1)

  if (groups == 2) {
    group <- get_attr(dat, "group")
    if (!all(group %in% c(1, 2))) {
      stop(
        "When using the `group` attribute, the only authorized values",
        " are 1 and 2.\n",
        "The values found were: ", paste0(unique(group), collapse = ", ")
      )
    }

    i.num.g2 <- get_init(dat, "i.num.g2")
    if (type  == "SIR" && is.null(status.vector)) {
      r.num.g2 <- get_init(dat, "r.num.g2", override.null.error = TRUE)
    }
  } else {
    group <- rep(1, num)
  }

  statOnNw <- "status" %in% dat$run$nwterms

  # Status ------------------------------------------------------------------

  ## Status passed on input network
  if (statOnNw == FALSE) {
    if (!is.null(status.vector)) {
      status <- status.vector
    } else {
      status <- rep("s", num)
      status[sample(which(group == 1), size = i.num)] <- "i"
      if (groups == 2) {
        status[sample(which(group == 2), size = i.num.g2)] <- "i"
      }
      if (type == "SIR") {
        status[sample(which(group == 1 & status == "s"), size = r.num)] <- "r"
        if (groups == 2) {
          status[sample(which(group == 2 & status == "s"),
                        size = r.num.g2)] <- "r"
        }
      }
    }
    dat <- set_attr(dat, "status", status)
  } else {
    status <- get_attr(dat, "status") # already copied in copy_nwattr_to_datattr
  }


  ## Set up TEA status
  if (tergmLite == FALSE) {
    if (statOnNw == FALSE) {
      for (net_index in seq_len(dat$num.nw)) {
        net <- get_network(dat, network = net_index)
        net <- set_vertex_attribute(net, "status", status)
        dat <- set_network(dat, net, network = net_index)
      }
    }

    for (net_index in seq_len(dat$num.nw)) {
      net <- get_network(dat, network = net_index)
      net <- activate.vertex.attribute(
        net,
        prefix = "testatus",
        value = status,
        onset = 1,
        terminus = Inf
      )
      dat <- set_network(dat, net, network = net_index)
    }
  }


  # Infection Time ----------------------------------------------------------
  ## Set up inf.time vector
  if (type == "None") {
    infTime <- rep(NA, num)
    idsInf <- idsInf <- which(status == "i")
    infTime[idsInf] <- 1
    dat <- set_attr(dat, "infTime", infTime)
  } else {
    idsInf <- which(status == "i")
    infTime <- rep(NA, length(status))
    infTime.vector <- get_init(dat, "infTime.vector",
                               override.null.error = TRUE)

    if (!is.null(infTime.vector)) {
      infTime <- infTime.vector
    } else {
      # If vital dynamics, infTime is a geometric draw over the duration of
      # infection
      if (vital == TRUE && di.rate > 0) {
        if (type == "SI") {
          infTime[idsInf] <- -rgeom(n = length(idsInf), prob = di.rate) + 2
        } else {
          infTime[idsInf] <- -rgeom(n = length(idsInf),
                                    prob = di.rate +
                                      (1 - di.rate) * mean(rec.rate)) + 2
        }
      } else {
        if (type == "SI" || mean(rec.rate) == 0) {
          # if no recovery, infTime a uniform draw over the number of sim time
          # steps
          infTime[idsInf] <- ssample(1:(-nsteps + 2),
                                     length(idsInf), replace = TRUE)
        } else {
          infTime[idsInf] <- -rgeom(n = length(idsInf),
                                    prob = mean(rec.rate)) + 2
        }
      }
    }

    dat <- set_attr(dat, "infTime", infTime)
  }

  return(dat)
}

#' @title Network Data and Stats Initialization
#'
#' @description This function initializes the network data and stats on the main
#'              `netsim_dat` class data object.
#'
#' @param dat A main data object of class `netsim_dat` obtained from
#'        [create_dat_object()], including the `control`
#'        argument.
#' @param x Either a fitted network model object of class `netest`, or a
#'        list of such objects.
#'
#' @return A `netsim_dat` class main data object with network data and
#'         stats initialized.
#'
#' @export
#' @keywords internal
#'
init_nets <- function(dat, x) {
  if (inherits(x, "netest")) {
    x <- list(x)
  }

  ## initialize network data on dat object
  dat$num.nw <- length(x)
  dat$nwparam <- lapply(x, function(y) y[!(names(y) %in% c("fit", "newnetwork"))])
  nws <- lapply(x, `[[`, "newnetwork")
  nw <- nws[[1]]
  if (get_control(dat, "tergmLite") == TRUE) {
    dat$run$el <- lapply(nws, as.edgelist)
    dat$run$net_attr <- lapply(nws, get_network_attributes)
  } else {
    dat$run$nw <- nws
  }

  # Nodal Attributes --------------------------------------------------------

  # Standard attributes
  num <- network.size(nw)
  dat <- append_core_attr(dat, 1, num)

  groups <- length(unique(get_vertex_attribute(nw, "group")))
  dat <- set_param(dat, "groups", groups)

  ## Pull attr on nw to dat$attr
  dat <- copy_nwattr_to_datattr(dat, nw)

  ## record names of relevant vertex attributes
  dat$run$nwterms <- get_network_term_attr(nw)

  ## initialize stats data structure
  if (get_control(dat, "save.nwstats") == TRUE) {
    if (get_control(dat, "resimulate.network") == TRUE) {
      dat$stats$nwstats <- rep(list(padded_vector(list(), get_control(dat, "nsteps"))),
                               length.out = length(dat$nwparam))
    } else {
      dat$stats$nwstats <- rep(list(list()), length.out = length(dat$nwparam))
    }
  }

  return(dat)
}

#' @title Helper to use a `data.frame` to initialize some attributes
#'
#' @description Uses `dat$init$init_attr` to overwrite some attributes of the
#' nodes at initialization
#'
#' @details
#' If an `init_attr` `data.frame` is present in `dat$init`, use it to overwrite
#' the attributes it contains.
#' `init_attr` must have a number of rows equal to the number of nodes in the
#' model as the attributes will be overwritten one to one, ensuring the correct
#' ordering.
#' `init_attr` columns MUST have a corresponding attribute already initialized.
#' See "R/default_attributes.R" for adding new attributes to the model.
#' `init_attr` is removed from `dat$init` at the end of the function to free up
#' its memory.
#'
#' @inheritParams recovery.net
#' @inherit recovery.net return
#'
#' @export
#'
overwrite_attrs <- function(dat) {
  init_attr <- get_init(dat, "init_attr", override.null.error = TRUE)
  if (is.null(init_attr)) {
    return(dat)
  }
  message("init_attr used for initialization of attributes")

  status <- get_attr(dat, "status")
  if (nrow(init_attr) != length(status)) {
    stop("init_attr should contains the same number of nodes as the model")
  }

  new_attrs <- setdiff(names(init_attr), names(dat$attr))
  if (length(new_attrs) > 0) {
    stop(
      "Some attributes in `init_attr` are not present in `dat`: ",
      paste0(new_attrs, collapse = " ,")
    )
  }

  core_attrs <- c("active", "entrTime", "exitTime", "unique_id")
  for (attr_name in setdiff(names(init_attr), core_attrs)) {
    dat <- set_attr(dat, attr_name, init_attr[[attr_name]])
  }

  dat$init$init_attr <- NULL
  dat
}

#' Get the `param.df` formated type of a value
#' @noRd
param.df_type <- function(x) {
  if (is.logical(x)) {
    "logical"
  } else if (is.numeric(x)) {
    "numeric"
  } else {
    "character"
  }
}

#' Coerce a value to a `param.df` type
#' @noRd
as_param.df_type <- function(x, type) {
  if (type == "logical") {
    as.logical(x)
  } else if (type == "numeric") {
    as.numeric(x)
  } else {
    as.character(x)
  }
}

#' List the recognized `param.df` types
#' @noRd
list_param.df_types <- function() {
  c("numeric", "logical", "character")
}

#' @title Parameters List for Stochastic Network Models from a Formatted
#'        Data Frame
#'
#' @description Sets the epidemic parameters for stochastic network models with
#'              [netsim()] using a specially formatted data frame of
#'              parameters.
#'
#' @param long.param.df A `data.frame` of parameters. See details for the
#'                      expected format.
#'
#' @return A list object of class `param.net`, which can be passed to
#'         [netsim()].
#'
#' @section long.param.df:
#' It is possible to set input parameters using a specifically formatted
#' `data.frame` object. The first 3 columns of this `data.frame` must
#' be:
#'
#'  * `param`: The name of the parameter. If this is a non-scalar
#'    parameter (a vector of length > 1), end the parameter name with the
#'    position on the vector (e.g., `"p_1"`, `"p_2"`, ...).
#'  * `value`: the value for the parameter (or the value of the
#'    parameter in the Nth position if non-scalar).
#'  * `type`: a character string containing either `"numeric"`,
#'    `"logical"`, or `"character"` to define the parameter object
#'    class.
#'
#'
#' In addition to these 3 columns, the `data.frame` can contain any number
#' of other columns, such as `details` or `source` columns to document
#' parameter meta-data. However, these extra columns will not be used by
#' EpiModel.
#'
#' @export
param.net_from_table <- function(long.param.df) {
  # Checks
  if (!all(c("param", "value", "type") %in% names(long.param.df))) {
    stop(
      "The `data.frame` must contain the following 3 columns:\n",
      "'param', 'value'", " and 'type"
    )
  }
  if (!all(long.param.df$type %in% list_param.df_types())) {
    stop("The `type` column must contain only: ",
         paste0(list_param.df_types(), collapse = ", "))
  }
  check_params_names(long.param.df$param)

  duplicated_params <- duplicated(long.param.df$param)
  duplicated_params <- long.param.df$param[duplicated_params]
  if (length(duplicated_params) > 0) {
    stop("The following parameters are duplicated: `",
         paste0(duplicated_params, collapse = "`, `"), "`")
  }

  # To flat params
  flat.params <- Map(as_param.df_type, long.param.df$value, long.param.df$type)
  names(flat.params) <- long.param.df$param

  # To param.list
  param <- unflatten_params(flat.params)
  class(param) <- c("param.net", "list")

  return(param)
}


#' Coerce a list of parameters to a `long.param.df`
#'
#' @param params A list of parameters to be formatted into a `long.param.df`
#'
#' @inheritSection param.net_from_table long.param.df
#'
#' @return  A `data.frame` of parameters.
#'
#' @export
param.net_to_table <- function(params) {
  flat_params <- flatten_params(params)
  check_params_names(names(flat_params))
  dplyr::tibble(
    param = names(flat_params),
    value = vapply(flat_params, as.character, ""),
    type = vapply(flat_params, param.df_type, "")
  )
}

#' @title Plot Data from a Stochastic Individual Contact Epidemic Model
#'
#' @description Plots epidemiological data from a stochastic individual contact
#'              model simulated with [icm()].
#'
#' @param x An `EpiModel` model object of class `icm`.
#' @param y Output compartments or flows from `icm` object to plot. -------
#' @param sims A vector of simulation numbers to plot.
#' @inheritParams plot.netsim
#' @inheritParams graphics::plot
#'
#' @details
#' This plotting function will extract the epidemiological output from a model
#' object of class `icm` and plot the time series data of disease
#' prevalence and other results. The summary statistics that the function
#' calculates and plots are individual simulation lines, means of the individual
#' simulation lines, and quantiles of those individual simulation lines. The
#' mean line, toggled on with `mean.line=TRUE`, is calculated as the row
#' mean across simulations at each time step.
#'
#' Compartment prevalences are the size of a compartment over some denominator.
#' To plot the raw numbers from any compartment, use `popfrac=FALSE`; this
#' is the default for any plots of flows. The `popfrac` parameter
#' calculates and plots the denominators of all specified compartments using
#' these rules: 1) for one-group models, the prevalence of any compartment is
#' the compartment size divided by the total population size; 2) for two-group
#' models, the prevalence of any compartment is the compartment size divided by
#' the group population size. For any prevalences that are not automatically
#' calculated, the [mutate_epi()] function may be used to add new
#' variables to the `icm` object to plot or analyze.
#'
#' The quantiles show the range of outcome values within a certain specified
#' quantile range. By default, the interquartile range is shown: that is the
#' middle 50\% of the data. This is specified by `qnts=0.5`. To show the
#' middle 95\% of the data, specify `qnts=0.95`. To toggle off the polygons
#' where they are plotted by default, specify `qnts=FALSE`.
#'
#' @method plot icm
#' @export
#'
#' @keywords plot
#' @seealso [icm()]
#'
#' @examples
#' \donttest{
#' ## Example 1: Plotting multiple compartment values from SIR model
#' param <- param.icm(inf.prob = 0.5, act.rate = 0.5, rec.rate = 0.02)
#' init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
#' control <- control.icm(type = "SIR", nsteps = 100,
#'                        nsims = 3, verbose = FALSE)
#' mod <- icm(param, init, control)
#' plot(mod, grid = TRUE)
#'
#' ## Example 2: Plot only infected with specific output from SI model
#' param <- param.icm(inf.prob = 0.25, act.rate = 0.25)
#' init <- init.icm(s.num = 500, i.num = 10)
#' control <- control.icm(type = "SI", nsteps = 100,
#'                        nsims = 3, verbose = FALSE)
#' mod2 <- icm(param, init, control)
#'
#' # Plot prevalence
#' plot(mod2, y = "i.num", mean.line = FALSE, sim.lines = TRUE)
#'
#' # Plot incidence
#' par(mfrow = c(1, 2))
#' plot(mod2, y = "si.flow", mean.smooth = TRUE, grid = TRUE)
#' plot(mod2, y = "si.flow", qnts.smooth = FALSE, qnts = 1)
#' }
#'
plot.icm <- function(x, y = NULL, popfrac = FALSE, sim.lines = FALSE,
                     sims = NULL, sim.col = NULL, sim.lwd = NULL,
                     sim.alpha = NULL, mean.line = TRUE, mean.smooth = TRUE,
                     mean.col = NULL, mean.lwd = 2, mean.lty = 1, qnts = 0.5,
                     qnts.col = NULL, qnts.alpha = 0.5, qnts.smooth = TRUE,
                     legend = NULL, leg.cex = 0.8, grid = FALSE, add = FALSE,
                     xlim = NULL, ylim = NULL, main = "", xlab = "Time",
                     ylab = NULL,
                     ...) {

  ## Model dimensions and class ##
  nsteps <- x$control$nsteps
  start <- if (is.null(x$control$start)) 1 else x$control$start
  offset <- start - 1
  nsims <- x$control$nsims
  if (is.null(sims)) {
    sims <- seq_len(nsims)
  }
  if (max(sims) > nsims) {
    stop("Set sim to between 1 and ", nsims)
  }
  if (is.null(x$param$groups) || !is.numeric(x$param$groups)) {
    x$param$groups <- 1
  }
  groups <- x$param$groups


  ## Compartments ##
  nocomp <- is.null(y)
  if (nocomp) {
    legend <- if (is.null(legend)) TRUE else legend
    if (groups == 1) {
      y <- grep(".num$", names(x$epi), value = TRUE)
    }
    if (groups == 2) {
      if (inherits(x, "icm")) {
        y <- c(grep(".num$", names(x$epi), value = TRUE),
               grep(".num.g2$", names(x$epi), value = TRUE))
      }
    }
  } else if (!all(y %in% names(x$epi))) {
    stop("Specified y is not available in object")
  }
  lcomp <- length(y)


  ## Color palettes ##

  # Main color palette
  bpal <- c(4, 2, 3, 5:100)

  # Mean line
  if (is.null(mean.col)) {
    mean.col <- bpal
  }
  mean.pal <- adjustcolor(mean.col, 0.9)

  # Quantile bands
  if (is.null(qnts.col)) {
    qnts.col <- bpal
  }
  qnts.pal <- adjustcolor(qnts.col, qnts.alpha)

  # Sim lines
  if (is.null(sim.lwd)) {
    sim.lwd <- rep(0.75, lcomp)
  } else {
    if (length(sim.lwd) < lcomp) {
      sim.lwd <- rep(sim.lwd, lcomp)
    }
  }

  if (is.null(sim.col)) {
    sim.col <- bpal
  } else {
    if (length(sim.col) < lcomp) {
      sim.col <- rep(sim.col, lcomp)
    }
  }

  if (is.null(sim.alpha) && nsims == 1) {
    sim.alpha <- 0.9
  } else if (is.null(sim.alpha) && nsims > 1) {
    sim.alpha <- max(c(0.05, 1 - log10(nsims) / 3))
  }
  sim.pal <- adjustcolor(sim.col, sim.alpha)

  ## Prevalence calculations ##
  x <- denom(x, y, popfrac)

  # Compartment max
  if (!popfrac) {
    min.prev <- min(sapply(y, function(comps) min(x$epi[[comps]], na.rm = TRUE)))
    max.prev <- max(sapply(y, function(comps) max(x$epi[[comps]], na.rm = TRUE)))
  } else {
    min.prev <- 0
    max.prev <- 1
  }


  ## Missing args ##
  if (is.null(xlim)) {
    xlim <- c(offset, offset + nsteps)
  }

  #Initialize ylim min max values
  qnt.min <- 1E10
  qnt.max <- -1E10
  mean.min <- 1E10
  mean.max <- -1E10

  ## Quantile and mean line setup (used in both ylim calc and drawing) ##
  if (qnts == FALSE || nsims == 1) {
    disp.qnts <- FALSE
  } else {
    disp.qnts <- TRUE
  }
  if (disp.qnts && (qnts > 1 || qnts < 0)) {
    stop("qnts must be between 0 and 1")
  }

  if (mean.line) {
    if (length(mean.lwd) < lcomp) {
      mean.lwd <- rep(mean.lwd, lcomp)
    }
    if (length(mean.lty) < lcomp) {
      mean.lty <- rep(mean.lty, lcomp)
    }
  }

  ## Quantiles - ylim min max ##
  if (disp.qnts) {
    qnt.min <- draw_qnts(x, y, qnts, qnts.pal, qnts.smooth, "epi", 0, "min", offset)
    qnt.max <- draw_qnts(x, y, qnts, qnts.pal, qnts.smooth, "epi", 0, "max", offset)
  }

  ## Mean lines - ylim max ##
  if (mean.line) {
    mean.min <- draw_means(x, y, mean.smooth, mean.lwd,
                           mean.pal, mean.lty, "epi", 0, "min", offset)
    mean.max <- draw_means(x, y, mean.smooth, mean.lwd,
                           mean.pal, mean.lty, "epi", 0, "max", offset)
  }

  # Dynamic scaling based on sim.lines and mean lines and quantile bands
  if (is.null(ylim) && (popfrac || sim.lines)) {
    ylim <- c(min.prev, max.prev)
  } else if (is.null(ylim) && !popfrac && !sim.lines &&
               (mean.line || disp.qnts)) {
    ylim <- c(min(qnt.min * 0.9, mean.min * 0.9),
              max(qnt.max * 1.1, mean.max * 1.1))
  }

  if (is.null(ylab)) {
    ylab <- if (popfrac) "Prevalence" else "Number"
  }

  ## Main plot window ##
  if (!add) {
    plot(1, 1, type = "n", bty = "n",
         xlim = xlim, ylim = ylim,
         xlab = xlab, ylab = ylab, main = main, ...)
  }


  ## Quantiles - Plotting ##
  if (disp.qnts) {
    draw_qnts(x, y, qnts, qnts.pal, qnts.smooth, "epi", 1, "max", offset)
  }

  ## Simulation lines ##
  if (sim.lines) {
    for (j in seq_len(lcomp)) {
      for (i in sims) {
        lines(seq_len(nrow(x$epi[[y[j]]])) + offset,
              x$epi[[y[j]]][, i], lwd = sim.lwd[j], col = sim.pal[j])
      }
    }
  }


  ## Mean lines - plotting ##
  if (mean.line) {
    draw_means(x, y, mean.smooth, mean.lwd, mean.pal, mean.lty,
               "epi", 1, "max", offset)
  }

  ## Grid
  if (grid) {
    grid()
  }

  ## Legends ##
  if (!is.null(legend) && legend) {
    if (groups == 2 && nocomp) {
      leg.lty <- mean.lty
    } else {
      leg.lty <- 1
    }
    legend("topright", legend = y, lty = leg.lty, lwd = 2,
           col = mean.pal, cex = leg.cex, bg = "white")
  }
}

#' @title Print Method for DCM Objects
#'
#' @description Prints a summary of a deterministic compartmental model object,
#'   including the model type, number of runs, time steps, model parameters, and
#'   output variable names.
#'
#' @param x An object of class `dcm`, from [dcm()].
#' @param ... Additional arguments (currently ignored).
#'
#' @details
#' Given a `dcm` object, `print.dcm` displays:
#' - **Simulation summary**: model class, model type (e.g., SI, SIR, SIS;
#'   omitted for new/custom models), number of runs, number of time steps, and
#'   number of groups.
#' - **Model parameters**: all parameters passed via [param.dcm()], excluding
#'   internal bookkeeping parameters (`groups`, `vital`).
#' - **Model output**: the names of all epidemic output variables stored in the
#'   model.
#'
#' @method print dcm
#' @export
print.dcm <- function(x, ...) {

  # New model
  new.mod <- ifelse(!is.null(x$control$new.mod), TRUE, FALSE)

  cat("EpiModel Simulation")
  cat("\n=======================")
  cat("\nModel class:", class(x))

  cat("\n\nSimulation Summary")
  cat("\n-----------------------")
  if (new.mod == FALSE) {
    cat("\nModel type:", x$control$type)
  }
  cat("\nNo. runs:", x$control$nruns)
  cat("\nNo. time steps:", x$control$nsteps)
  if (new.mod == FALSE) {
    cat("\nNo. groups:", x$param$groups)
  }

  cat("\n\nModel Parameters")
  cat("\n-----------------------\n")
  pToPrint <- which(!(names(x$param) %in% c("groups", "vital")))
  for (i in pToPrint) {
    cat(names(x$param)[i], "=", x$param[[i]], fill = 60)
  }

  if (!is.null(x$init)) {
    cat("\nInitial Conditions")
    cat("\n-----------------------\n")
    flow.names <- grep("\\.flow", names(x$init), value = TRUE)
    iToPrint <- which(!(names(x$init) %in% flow.names))
    for (i in iToPrint) {
      cat(names(x$init)[i], "=", x$init[[i]], fill = 60)
    }
  }

  cat("\nModel Output")
  cat("\n-----------------------")
  cat("\nVariables:", names(x$epi), fill = 60)

  invisible()
}


#' @title Print Method for ICM Objects
#'
#' @description Prints a summary of a stochastic individual contact model
#'   object, including the model type, number of simulations, time steps, model
#'   parameters, and output variable names.
#'
#' @param x An object of class `icm`, from [icm()].
#' @param ... Additional arguments (currently ignored).
#'
#' @details
#' Given an `icm` object, `print.icm` displays:
#' - **Simulation summary**: model class, model type (e.g., SI, SIR, SIS),
#'   number of simulations, number of time steps, and number of groups.
#' - **Model parameters**: all parameters passed via [param.icm()], excluding
#'   internal bookkeeping parameters (`groups`, `vital`).
#' - **Model output**: the names of all epidemic output variables stored in the
#'   model.
#'
#' @method print icm
#' @export
print.icm <- function(x, ...) {

  cat("EpiModel Simulation")
  cat("\n=======================")
  cat("\nModel class:", class(x))

  cat("\n\nSimulation Summary")
  cat("\n-----------------------")
  cat("\nModel type:", x$control$type)
  cat("\nNo. simulations:", x$control$nsims)
  cat("\nNo. time steps:", x$control$nsteps)
  cat("\nNo. groups:", x$param$groups)

  cat("\n\nModel Parameters")
  cat("\n-----------------------\n")
  pToPrint <- which(!(names(x$param) %in% c("groups", "vital")))
  for (i in pToPrint) {
    cat(names(x$param)[i], "=", x$param[[i]], fill = 60)
  }

  cat("\nModel Output")
  cat("\n-----------------------")
  cat("\nVariables:", names(x$epi), fill = 60)

  invisible()
}


#' @export
print.netest <- function(x, digits = 3, ...) {

  cat("EpiModel Network Estimation")
  cat("\n=======================")
  cat("\nModel class:", class(x))
  estmeth <- ifelse(x$edapprox == TRUE, "ERGM with Edges Approximation",
                    "Full STERGM Fit")
  cat(paste("\nEstimation Method:", estmeth))

  cat("\n\nModel Form")
  cat("\n-----------------------")
  cat("\nFormation: ")
  print(x$formation)
  cat("Target Statistics:", x$target.stats)
  cat("\nConstraints: ")
  cat(paste0(as.character(x$constraints)[1],
             as.character(x$constraints)[2]))
  cat("\n\nDissolution: ")
  cat(as.character(x$coef.diss$dissolution), sep = "")
  cat("\nTarget Statistics:", x$coef.diss$duration)

  invisible()
}

#' @rdname print.netdx
#' @title Utility Function for Printing netdx Object
#' @description Prints basic information and statistics from a `netdx`
#'              object.
#' @param x an object of class `netdx`
#' @param digits number of digits to print in statistics tables
#' @param ... additional arguments (currently ignored)
#' @details
#' Given a `netdx` object, `print.netdx` prints the diagnostic method
#' (static/dynamic), number of simulations, and (if dynamic) the number of time
#' steps per simulation used in generating the `netdx` object, as well as
#' printing the formation statistics table and (if present) the duration and
#' dissolution statistics tables.  The statistics tables are interpreted as
#' follows.
#'
#' Each row has the name of a particular network statistic.  In the formation
#' table, these correspond to actual network statistics in the obvious way.
#' In the duration and dissolution tables, these correspond to dissolution
#' model dyad types: in a homogeneous dissolution model, all dyads are of the
#' `edges` type; in a heterogeneous dissolution model, a dyad with a
#' nonzero `nodematch` or `nodemix` change statistic in the
#' dissolution model has type equal to that statistic, and has type equal to
#' `edges` otherwise.  The statistics of interest for the duration and
#' dissolution tables are, respectively, the mean age of extant edges and the
#' edge dissolution rate, broken down by dissolution model dyad type.  (The
#' current convention is to treat the mean age and dissolution rate for a
#' particular dissolution dyad type as 0 on time steps with no edges of that
#' type; this behavior may be changed in the future.)
#'
#' The columns are named `Target`, `Sim Mean`, `Pct Diff`,
#' `Sim SE`, `Z Score`, `SD(Sim Means)`, and
#' `SD(Statistic)`.  The `Sim Mean` column refers to the mean
#' statistic value, across all time steps in all simulations in the dynamic
#' case, and across all sampled networks in all simulations in the static case.
#' The `Sim SE` column refers to the standard error in the mean, estimated
#' using [`coda::effectiveSize`].  The `Target`
#' column indicates the target value (if present) for the statistic, and the
#' `Pct Diff` column gives `(Sim Mean - Target)/Target` when
#' `Target` is present.  The `Z Score` column gives
#' `(Sim Mean - Target)/(Sim SE)`.  The `SD(Sim Means)` column gives
#' the empirical standard deviation across simulations of the mean statistic
#' value within simulation, and `SD(Statistic)` gives the empirical
#' standard deviation of the statistic value across all the simulated data.
#' @export
print.netdx <- function(x, digits = 3, ...) {

  cat("EpiModel Network Diagnostics")
  cat("\n=======================")
  dxmethod <- ifelse(x$dynamic == TRUE, "Dynamic", "Static")
  cat("\nDiagnostic Method:", dxmethod)
  cat("\nSimulations:", x$nsims)
  if (x$dynamic == TRUE) {
    cat("\nTime Steps per Sim:", x$nsteps)
  }

  cat("\n\nFormation Diagnostics")
  cat("\n----------------------- \n")
  print_nwstats_table(x$stats.table.formation, digits)

  if (x$dynamic == TRUE && !is.null(x$stats.table.duration)) {
    cat("\nDuration Diagnostics")
    cat("\n----------------------- \n")
    print_nwstats_table(x$stats.table.duration, digits)
  }
  if (x$dynamic == TRUE && !is.null(x$stats.table.dissolution)) {
    cat("\nDissolution Diagnostics")
    cat("\n----------------------- \n")
    print_nwstats_table(x$stats.table.dissolution, digits)
  }
  if (x$anyNA == TRUE) {
    cat("\nNOTE: Duration and dissolution data contains undefined values due to",
        "having zero edges of some dissolution dyad type(s) on some time",
        "steps; these undefined values will be set to 0 when",
        "processing the data; this behavior, which introduces a bias",
        "towards 0, may be changed in the future.")
  }
  invisible()
}


#' @title Print Method for Network Model Simulations
#'
#' @description Prints a detailed summary of a stochastic network model
#'   simulation object, including simulation metadata, model parameters,
#'   output variables, and (optionally) network formation, duration, and
#'   dissolution statistics with target comparisons.
#'
#' @param x An object of class `netsim`, from [netsim()].
#' @param nwstats If `TRUE` (the default), print network statistics tables
#'   (formation, duration, and dissolution) when available.
#' @param digits Number of digits to print in the network statistics tables.
#' @param network Integer index of the network for which to display statistics,
#'   for multi-network models. Default is `1`.
#' @param ... Additional arguments (currently ignored).
#'
#' @details
#' Given a `netsim` object, `print.netsim` displays the following sections:
#'
#' **Simulation summary**: model class, model type (e.g., SI, SIR, SIS),
#' number of simulations, number of time steps, and number of network groups.
#'
#' **Model parameters**: printed via `print.param.net()`, showing fixed and
#' (if applicable) random parameters.
#'
#' **Model functions**: for extension models (`type = NULL`), lists the names
#' of all custom module functions.
#'
#' **Model output**: the names of all epidemic output variables, saved network
#' objects, transmission matrices, and any other saved elements.
#'
#' **Network statistics** (when `nwstats = TRUE` and network statistics were
#' saved during the simulation):
#'
#' The formation, duration, and dissolution statistics tables are computed and
#' displayed in the same way as for [print.netdx()].  Each table contains the
#' columns `Target`, `Sim Mean`, `Pct Diff`, `Sim SE`, `Z Score`,
#' `SD(Sim Means)`, and `SD(Statistic)`.  The `Sim Mean` column is the mean
#' statistic value across all time steps and simulations.  `Sim SE` is the
#' standard error estimated using [coda::effectiveSize].  `Pct Diff` gives
#' `(Sim Mean - Target) / Target` and `Z Score` gives
#' `(Sim Mean - Target) / Sim SE` when a target is available.  `SD(Sim Means)`
#' is the standard deviation of per-simulation means, and `SD(Statistic)` is
#' the overall standard deviation of the statistic.
#'
#' *Formation statistics*: each row corresponds to a network statistic from the
#' formation formula (e.g., `edges`, `nodematch`), compared against the target
#' statistics from network estimation.  These statistics assess whether the
#' network structure is maintained at target levels during the epidemic
#' simulation (which is particularly important in open-population models where
#' demographic turnover can shift network structure).
#'
#' *Duration statistics*: each row corresponds to a dissolution model dyad
#' type.  In a homogeneous dissolution model (`~offset(edges)`), all dyads are
#' of the `edges` type.  The statistic of interest is the mean age of extant
#' edges, compared against the target duration from [dissolution_coefs()].
#'
#' *Dissolution statistics*: same row structure as the duration table.  The
#' statistic of interest is the edge dissolution rate (proportion of edges
#' dissolving per time step), compared against `1 / target duration`.
#'
#' Duration and dissolution tables are only available when
#' `control$save.diss.stats = TRUE`, `control$save.network = TRUE`,
#' `control$tergmLite = FALSE`, and the dissolution formula is
#' `~offset(edges)`.  When these conditions are not met, a note listing the
#' requirements is printed instead.
#'
#' @seealso [netsim()], [print.netdx()], [summary.netsim()].
#'
#' @method print netsim
#' @export
print.netsim <- function(x, nwstats = TRUE, digits = 3, network = 1, ...) {

  nsims <- x$control$nsims
  if (nsims == 1) {
    simnames <- "sim1"
  }
  if (nsims == 2) {
    simnames <- "sim1 sim2"
  }
  if (nsims > 2) {
    simnames <- paste0("sim1 ... sim", nsims)
  }

  cat("EpiModel Simulation")
  cat("\n=======================")
  cat("\nModel class:", class(x))

  cat("\n\nSimulation Summary")
  cat("\n-----------------------")
  cat("\nModel type:", x$control$type)
  cat("\nNo. simulations:", nsims)
  cat("\nNo. time steps:", x$control$nsteps)
  cat("\nNo. NW groups:", x$param$groups)

  # Parameters
  cat("\n\n")
  print(x$param)

  if (is.null(x$control$type)) {
    cat("\nModel Functions")
    cat("\n-----------------------\n")
    for (i in seq_along(x$control$f.names)) {
      cat(x$control$f.names[i], "\n")
    }
  }

  cat("\nModel Output")
  cat("\n-----------------------")
  cat("\nVariables:", names(x$epi), fill = 60)
  if (!(is.null(x$network))) {
    cat("Networks:", simnames)
  }
  if (!(is.null(x$stats$transmat))) {
    if (!is.null(x$network)) {
      cat("\nTransmissions:", simnames)
    } else {
      cat("Transmissions:", simnames)
    }
  }
  if (!is.null(x$control$save.other)) {
    names_present <- intersect(x$control$save.other, names(x))
    if (length(names_present) > 0) {
      cat("\nOther Elements:", names_present)
    }
  }
  cat("")

  if (nwstats && !is.null(x$stats$nwstats)) {
    stats <- get_nwstats(x, network = network, mode = "list")
    nsims <- x$control$nsims

    target.stats <- x$nwparam[[network]]$target.stats
    ts.attr.names <- x$nwparam[[network]]$target.stats.names
    names(target.stats) <- ts.attr.names

    stats.table.formation <- make_stats_table(stats, target.stats)

    cat("\n\nFormation Statistics")
    cat("\n----------------------- \n")
    print_nwstats_table(stats.table.formation, digits = digits)
    cat("\n")

    if (x$control$save.diss.stats &&
          x$control$save.network &&
          ! x$control$tergmLite &&
          ! is.null(x$diss.stats) &&
          x$nwparam[[network]]$coef.diss$dissolution == ~ offset(edges)) {

      if (any(unlist(lapply(x$diss.stats, `[[`, "anyNA")))) {
        cat("\nNOTE: Duration & dissolution data contains undefined values due to zero edges of some dissolution
            dyad type(s) on some time step; these undefined values will be set to 0 when processing the data.")
      }

      dur_stats <- lapply(x$diss.stats, function(ds) ds[[network]][["meanageimputed"]])
      diss_stats <- lapply(x$diss.stats, function(ds) ds[[network]][["propdiss"]])

      dur_table <- make_stats_table(dur_stats, x$nwparam[[network]]$coef.diss$duration)
      diss_table <- make_stats_table(diss_stats, 1 / x$nwparam[[network]]$coef.diss$duration)

      cat("\nDuration Statistics")
      cat("\n----------------------- \n")
      print_nwstats_table(dur_table, digits)

      cat("\nDissolution Statistics")
      cat("\n----------------------- \n")
      print_nwstats_table(diss_table, digits)

    } else {
      cat("\nDuration and Dissolution Statistics")
      cat("\n----------------------- \n")
      cat("Not available when:")
      cat("\n- `control$tergmLite == TRUE`")
      cat("\n- `control$save.network == FALSE`")
      cat("\n- `control$save.diss.stats == FALSE`")
      cat("\n- dissolution formula is not `~ offset(edges)`")
      cat("\n- `keep.diss.stats == FALSE` (if merging)")
      cat("\n")
    }
  }

  cat("\n")
  invisible()
}


#' @export
print.disscoef <- function(x, ...) {

  cat("Dissolution Coefficients")
  cat("\n=======================")
  cat("\nDissolution Model: ")
  cat(as.character(x$dissolution), sep = "")
  cat("\nTarget Statistics:", x$duration)
  cat("\nCrude Coefficient:", x$coef.crude)
  cat("\nMortality/Exit Rate:", x$d.rate)
  cat("\nAdjusted Coefficient:", x$coef.adj)
  cat("\n")

  invisible()
}

#' @title Format One Parameter for Printing with the `print.param.xxx`
#'        Functions
#'
#' @param param_name The name of the parameter to print.
#' @param param_value The value of the parameter to print.
#'
#' @keywords internal
format_param <- function(param_name, param_value) {
  if (is.numeric(param_value) && length(param_value) > 10) {
    cat(param_name, "=", param_value[1:10], "...", fill = 80)
  } else if (is.data.frame(param_value)) {
    cat(param_name, "= <data.frame>\n")
  } else if (is.list(param_value)) {
    cat(param_name, "= <list>\n")
  } else if (inherits(param_value, "lm")) {
    cat(param_name, "= <lm/glm>\n")
  } else {
    cat(param_name, "=", param_value, fill = 80)
  }
}

#' @export
print.param.dcm <- function(x, ...) {

  pToPrint <- seq_along(x)

  cat("DCM Parameters")
  cat("\n===========================\n")
  for (i in pToPrint) {
    format_param(names(x)[i], x[[i]])
  }

  invisible()
}

#' @export
print.param.icm <- function(x, ...) {

  pToPrint <- which(!(names(x) %in% c("vital")))

  cat("ICM Parameters")
  cat("\n===========================\n")
  for (i in pToPrint) {
    format_param(names(x)[i], x[[i]])
  }

  invisible()
}

#' @export
print.param.net <- function(x, ...) {

  randoms <- c("random.params", "random.params.values")
  pToPrint <- which(!(names(x) %in% c("vital", randoms)))
  rng_values <- list()
  rng_defs <- NULL

  if (all(randoms %in% names(x))) {
    rng_values <- x$random.params.values
    pToPrint <- pToPrint[! names(x)[pToPrint] %in% names(rng_values)]
  } else if (randoms[1] %in% names(x)) {
    rng_defs <- names(x[[randoms[1]]])
    pToPrint <- pToPrint[! names(x)[pToPrint] %in% rng_defs]
  }

  cat("Fixed Parameters")
  cat("\n---------------------------\n")
  for (i in pToPrint) {
    format_param(names(x)[i], x[[i]])
  }

  if (!is.null(rng_defs)) {
    cat("\nRandom Parameters")
    cat("\n(Not drawn yet)")
    cat("\n---------------------------\n")
    for (prm in rng_defs) {
      if (prm == "param.random.set") {
        cat(prm, "= <data.frame> ( dimensions:",
            dim(x$random.param$param.random.set), ")\n")
      } else {
        cat(prm, "= <function>\n")
      }
    }
  }

  if (length(rng_values) > 0) {
    cat("\nRandom Parameters")
    cat("\n---------------------------\n")
    for (i in seq_along(rng_values)) {
      format_param(names(rng_values)[i], rng_values[[i]])
    }
  }

  invisible()
}

#' @export
print.init.dcm <- function(x, ...) {

  pToPrint <- seq_along(x)

  cat("DCM Initial Conditions")
  cat("\n===========================\n")
  for (i in pToPrint) {
    if (inherits(x[[i]], c("integer", "numeric")) && length(x[[i]]) > 10) {
      cat(names(x)[i], "=", x[[i]][1:5], "...", fill = 80)
    } else if (inherits(x[[i]], "data.frame")) {
      cat(names(x)[i], "= <data.frame>\n")
    } else if (inherits(x[[i]], "list")) {
      cat(names(x)[i], "= <list>\n")
    } else {
      cat(names(x)[i], "=", x[[i]], fill = 80)
    }
  }

  invisible()
}

#' @export
print.init.icm <- function(x, ...) {

  pToPrint <- seq_along(x)

  cat("ICM Initial Conditions")
  cat("\n===========================\n")
  for (i in pToPrint) {
    if (inherits(x[[i]], c("integer", "numeric")) && length(x[[i]]) > 10) {
      cat(names(x)[i], "=", x[[i]][1:5], "...", fill = 80)
    } else if (inherits(x[[i]], "data.frame")) {
      cat(names(x)[i], "= <data.frame>\n")
    } else if (inherits(x[[i]], "list")) {
      cat(names(x)[i], "= <list>\n")
    } else {
      cat(names(x)[i], "=", x[[i]], fill = 80)
    }
  }

  invisible()
}

#' @export
print.init.net <- function(x, ...) {

  pToPrint <- seq_along(x)

  cat("Network Model Initial Conditions")
  cat("\n=================================\n")
  for (i in pToPrint) {
    if (inherits(x[[i]], c("integer", "numeric")) && length(x[[i]]) > 10) {
      cat(names(x)[i], "=", x[[i]][1:5], "...", fill = 80)
    } else if (inherits(x[[i]], "data.frame")) {
      cat(names(x)[i], "= <data.frame>\n")
    } else if (inherits(x[[i]], "list")) {
      cat(names(x)[i], "= <list>\n")
    } else {
      cat(names(x)[i], "=", x[[i]], fill = 80)
    }
  }

  invisible()
}

#' @export
print.control.dcm <- function(x, ...) {

  pToPrint <- seq_along(names(x))
  pToPrint <- pToPrint[-which(names(x) == "new.mod")]
  if (!is.null(x$new.mod)) {
    names(x)[which(names(x) == "new.mod.name")] <- "new.mod"
  }

  cat("DCM Control Settings")
  cat("\n===========================\n")
  for (i in pToPrint) {
    cat(names(x)[i], "=", x[[i]], fill = 80)
  }

  invisible()
}

#' @export
print.control.icm <- function(x, ...) {

  pToPrint <- which(!grepl(".FUN", names(x)))

  cat("ICM Control Settings")
  cat("\n===========================\n")
  for (i in pToPrint) {
    cat(names(x)[i], "=", x[[i]], fill = 80)
  }

  invisible()
}

#' @export
print.control.net <- function(x, ...) {

  pToPrint <- which(
    !grepl(".FUN", names(x)) &
      names(x) != "f.args" &
      names(x) != "f.names" &
      names(x) != "set.control.tergm" &
      names(x) != "set.control.ergm" &
      names(x) != "dat.updates" &
      !(names(x) %in% c("bi.mods", "user.mods"))
  )



  cat("Network Model Control Settings")
  cat("\n===============================\n")
  for (i in pToPrint) {
    if (inherits(x[[i]], "formula")) {
      cat(names(x)[i], "= ")
      cat(paste0(as.character(x[[i]])[1],
                 as.character(x[[i]])[2]), "\n")
    } else if (inherits(x[[i]], "data.frame")) {
      cat(names(x)[i], "= <data.frame>\n")
    } else if (inherits(x[[i]], "list")) {
      cat(names(x)[i], "= <list>\n")
    } else {
      cat(names(x)[i], "=", x[[i]], fill = 80)
    }
  }

  if (!is.null(x$module.order)) {
    funToPrint <- x$module.order
  } else {
    funToPrint <- names(x)[grep(".FUN", names(x))]
    funToPrint <- funToPrint[!funToPrint %in% c("initialize.FUN",
                                                "verbose.FUN")]
  }

  cat("Dynamic Modules:", funToPrint)
  cat("\n")

  invisible()
}

#' @title Print Helper For Network Stats Tables
#'
#' @param nwtable A formation or dissolution statistics `data.frame`.
#' @param digits Argument to be passed to `round`.
#'
#' @keywords internal
print_nwstats_table <- function(nwtable, digits) {
  print(as.data.frame(round(as.matrix(nwtable), digits = digits)))
}

#' @title Dynamic Network Updates
#'
#' @description This function handles all calls to the network object contained
#'              on the main `netsim_dat` object handled in `netsim`.
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @export
#'
nwupdate.net <- function(dat, at) {
  ## Attributes
  status <- get_attr(dat, "status")
  active <- get_attr(dat, "active")
  entrTime <- get_attr(dat, "entrTime")
  exitTime <- get_attr(dat, "exitTime")

  ## Controls
  tergmLite <- get_control(dat, "tergmLite")
  resimulate.network <- get_control(dat, "resimulate.network")

  ## Vital Dynamics
  arrivals <- which(active == 1 & entrTime == at)
  departures <- which(active == 0 & exitTime == at)

  nArrivals <- length(arrivals)
  if (nArrivals > 0) {
    ## Arrivals
    nwterms <- dat$run$nwterms
    if (!is.null(nwterms)) {
      curr.tab <- get_attr_prop(dat, nwterms)
      dat <- auto_update_attr(dat, arrivals, curr.tab)
    }
    if (length(unique(vapply(get_attr_list(dat), length, 1))) != 1) {
      stop(
        "Attribute list of unequal length. Check arrivals.net module.\n",
        print(cbind(vapply(get_attr_list(dat), length, 1)))
      )
    }
    dat <- arrive_nodes(dat, nArrivals)
  }

  ## Departures
  dat <- depart_nodes(dat, departures)

  ## Copy static attributes to network object
  if (tergmLite == FALSE && resimulate.network == TRUE) {
    dat <- copy_datattr_to_nwattr(dat)
  }

  ## Update temporally extended disease status
  if (tergmLite == FALSE) {
    for (net_index in seq_len(dat$num.nw)) {
      net <- get_network(dat, network = net_index)
      net <- activate.vertex.attribute(
        net,
        prefix = "testatus",
        value = status,
        onset = at,
        terminus = Inf
      )
      dat <- set_network(dat, nw = net, network = net_index)
    }
  }

  ## Output
  return(dat)
}

#' @title Departure: icm Module
#'
#' @description This function simulates departure for use in [icm()]
#'              simulations.
#'
#' @inheritParams prevalence.icm
#'
#' @inherit recovery.icm return
#'
#' @seealso [icm()]
#'
#' @export
#' @keywords internal
#'
departures.icm <- function(dat, at) {

  # Conditions --------------------------------------------------------------
  if (dat$param$vital == FALSE) {
    return(dat)
  }

  # Susceptible departures --------------------------------------------------
  nDepartures <- 0
  idsElig <- which(dat$attr$active == 1 & dat$attr$status == "s")
  nElig <- length(idsElig)
  if (nElig > 0) {

    rates <- dat$param$ds.rate

    vecDepartures <- which(rbinom(nElig, 1, rates) == 1)
    if (length(vecDepartures) > 0) {
      idsDpt <- idsElig[vecDepartures]
      nDepartures <- length(idsDpt)
      dat$attr$active[idsDpt] <- 0
    }
  }

  if (at == 2) {
    dat$epi$ds.flow <- c(0, nDepartures)
  } else {
    dat$epi$ds.flow[at] <- nDepartures
  }


  # Infected Departures -----------------------------------------------------
  nDepartures <- 0
  idsElig <- which(dat$attr$active == 1 & dat$attr$status == "i")
  nElig <- length(idsElig)
  if (nElig > 0) {

    rates <- dat$param$di.rate

    vecDepartures <- which(rbinom(nElig, 1, rates) == 1)
    if (length(vecDepartures) > 0) {
      idsDpt <- idsElig[vecDepartures]
      nDepartures <- length(idsDpt)
      dat$attr$active[idsDpt] <- 0
    }
  }

  if (at == 2) {
    dat$epi$di.flow <- c(0, nDepartures)
  } else {
    dat$epi$di.flow[at] <- nDepartures
  }


  # Recovered Departures ----------------------------------------------------
  nDepartures <- 0
  idsElig <- which(dat$attr$active == 1 & dat$attr$status == "r")
  nElig <- length(idsElig)
  if (nElig > 0) {

    rates <- dat$param$dr.rate

    vecDepartures <- which(rbinom(nElig, 1, rates) == 1)
    if (length(vecDepartures) > 0) {
      idsDpt <- idsElig[vecDepartures]
      nDepartures <- length(idsDpt)
      dat$attr$active[idsDpt] <- 0
    }
  }

  if (at == 2) {
    dat$epi$dr.flow <- c(0, nDepartures)
  } else {
    dat$epi$dr.flow[at] <- nDepartures
  }

  return(dat)
}


#' @title Arrivals: icm Module
#'
#' @description This function simulates arrival for use in [icm()]
#'              simulations.
#'
#' @inheritParams prevalence.icm
#'
#' @inherit recovery.icm return
#'
#' @seealso [icm()]
#'
#' @export
#' @keywords internal
#'
arrivals.icm <- function(dat, at) {

  # Conditions --------------------------------------------------------------
  if (dat$param$vital == FALSE) {
    return(dat)
  }

  # Variables ---------------------------------------------------------------
  a.rate <- dat$param$a.rate
  nOld <- dat$epi$num[at - 1]


  # Process -----------------------------------------------------------------
  nArrivals <- sum(rbinom(nOld, 1, a.rate))


  ## Set attributes
  totArrivals <- nArrivals
  dat$attr$active <- c(dat$attr$active, rep(1, totArrivals))
  dat$attr$group <- c(dat$attr$group, c(rep(1, nArrivals)))
  dat$attr$status <- c(dat$attr$status, rep("s", totArrivals))
  dat$attr$infTime <- c(dat$attr$infTime, rep(NA, totArrivals))


  # Output ------------------------------------------------------------------
  if (at == 2) {
    dat$epi$a.flow <- c(0, nArrivals)
  } else {
    dat$epi$a.flow[at] <- nArrivals
  }

  return(dat)
}


#' @title Departure: Bipartite icm Module
#'
#' @description This function simulates departure for use in [icm()]
#'              simulations.
#'
#' @inheritParams prevalence.icm
#'
#' @inherit recovery.icm return
#'
#' @seealso [icm()]
#'
#' @export
#' @keywords internal
#'
departures.icm.bip <- function(dat, at) {

  # Conditions --------------------------------------------------------------
  if (dat$param$vital == FALSE) {
    return(dat)
  }


  # Variables ---------------------------------------------------------------
  group <- dat$attr$group


  # Susceptible departures --------------------------------------------------
  nDepartures <- nDeparturesG2 <- 0
  idsElig <- which(dat$attr$active == 1 & dat$attr$status == "s")
  nElig <- length(idsElig)
  if (nElig > 0) {

    gElig <- group[idsElig]
    rates <- c(dat$param$ds.rate, dat$param$ds.rate.g2)
    ratesElig <- rates[gElig]

    vecDepartures <- which(rbinom(nElig, 1, ratesElig) == 1)
    if (length(vecDepartures) > 0) {
      idsDpt <- idsElig[vecDepartures]
      nDepartures <- sum(group[idsDpt] == 1)
      nDeparturesG2 <- sum(group[idsDpt] == 2)
      dat$attr$active[idsDpt] <- 0
    }
  }

  if (at == 2) {
    dat$epi$ds.flow <- c(0, nDepartures)
    dat$epi$ds.flow.g2 <- c(0, nDeparturesG2)
  } else {
    dat$epi$ds.flow[at] <- nDepartures
    dat$epi$ds.flow.g2[at] <- nDeparturesG2
  }


  # Infected Departures -----------------------------------------------------
  nDepartures <- nDeparturesG2 <- 0
  idsElig <- which(dat$attr$active == 1 & dat$attr$status == "i")
  nElig <- length(idsElig)
  if (nElig > 0) {

    gElig <- group[idsElig]
    rates <- c(dat$param$di.rate, dat$param$di.rate.g2)
    ratesElig <- rates[gElig]

    vecDepartures <- which(rbinom(nElig, 1, ratesElig) == 1)
    if (length(vecDepartures) > 0) {
      idsDpt <- idsElig[vecDepartures]
      nDepartures <- sum(group[idsDpt] == 1)
      nDeparturesG2 <- sum(group[idsDpt] == 2)
      dat$attr$active[idsDpt] <- 0
    }
  }

  if (at == 2) {
    dat$epi$di.flow <- c(0, nDepartures)
    dat$epi$di.flow.g2 <- c(0, nDeparturesG2)
  } else {
    dat$epi$di.flow[at] <- nDepartures
    dat$epi$di.flow.g2[at] <- nDeparturesG2
  }


  # Recovered Departures ----------------------------------------------------
  nDepartures <- nDeparturesG2 <- 0
  idsElig <- which(dat$attr$active == 1 & dat$attr$status == "r")
  nElig <- length(idsElig)
  if (nElig > 0) {

    gElig <- group[idsElig]
    rates <- c(dat$param$dr.rate, dat$param$dr.rate.g2)
    ratesElig <- rates[gElig]

    vecDepartures <- which(rbinom(nElig, 1, ratesElig) == 1)
    if (length(vecDepartures) > 0) {
      idsDpt <- idsElig[vecDepartures]
      nDepartures <- sum(group[idsDpt] == 1)
      nDeparturesG2 <- sum(group[idsDpt] == 2)
      dat$attr$active[idsDpt] <- 0
    }
  }

  if (at == 2) {
    dat$epi$dr.flow <- c(0, nDepartures)
    dat$epi$dr.flow.g2 <- c(0, nDeparturesG2)
  } else {
    dat$epi$dr.flow[at] <- nDepartures
    dat$epi$dr.flow.g2[at] <- nDeparturesG2
  }

  return(dat)
}

#' @title Arrivals: Bipartite icm Module
#'
#' @description This function simulates arrival for use in [icm()]
#'              simulations.
#'
#' @inheritParams prevalence.icm
#'
#' @inherit recovery.icm return
#'
#' @seealso [icm()]
#'
#' @export
#' @keywords internal
#'
arrivals.icm.bip <- function(dat, at) {

  # Conditions --------------------------------------------------------------
  if (dat$param$vital == FALSE) {
    return(dat)
  }
  # Variables ---------------------------------------------------------------
  a.rate <- dat$param$a.rate
  a.rate.g2 <- dat$param$a.rate.g2
  nOld <- dat$epi$num[at - 1]

  # Process -----------------------------------------------------------------
  nArrivals <- nArrivalsG2 <- 0
  nOldG2 <- dat$epi$num.g2[at - 1]

  if (is.na(a.rate.g2)) {
    nArrivals <- sum(rbinom(nOld, 1, a.rate))
    nArrivalsG2 <- sum(rbinom(nOld, 1, a.rate))
  } else {
    nArrivals <- sum(rbinom(nOld, 1, a.rate))
    nArrivalsG2 <- sum(rbinom(nOldG2, 1, a.rate.g2))
  }

  ## Set attributes
  totArrivals <- nArrivals + nArrivalsG2
  dat$attr$active <- c(dat$attr$active, rep(1, totArrivals))
  dat$attr$group <- c(dat$attr$group, c(rep(1, nArrivals), rep(2, nArrivalsG2)))
  dat$attr$status <- c(dat$attr$status, rep("s", totArrivals))
  dat$attr$infTime <- c(dat$attr$infTime, rep(NA, totArrivals))

  # Output ------------------------------------------------------------------
  if (at == 2) {
    dat$epi$a.flow <- c(0, nArrivals)
    dat$epi$a.flow.g2 <- c(0, nArrivalsG2)
  } else {
    dat$epi$a.flow[at] <- nArrivals
    dat$epi$a.flow.g2[at] <- nArrivalsG2
  }

  return(dat)
}

#' @title Message to Find in Which Module a `condition` Occurred
#'
#' @description This function returns a formatted string describing when, where,
#'              and why an error, message, or warning occurred.
#'
#' @param cond The type of `condition` handled (message, warning, error).
#' @param module The name of the module where the `condition` occurred.
#' @param at The time step the `condition` occurred.
#' @param msg The `condition`'s message.
#'
#' @return A formatted string describing where and when the `condition`
#'         occurred as well as the `condition`'s message.
#'
#' @keywords internal
netsim_cond_msg <- function(cond, module, at, msg) {
  paste0("\n\tA ", cond, " occured in module '", module, "' at step ", at)
}

#'  Handle the Logging of Traceback and Dumping of Frames on Error
#'
#'  If `control$.traceback.on.error == TRUE`, this function prints the traceback
#'  of the current simulation to STDIN. This is useful when `ncores > 1` or in
#'  HPC settings.
#'  If `control$.dump.frames.on.error == TRUE`, this function saves a debugging
#'  dump for "postmortem debugging". The dumps are named
#'  "dump_%Y%m%d_%H%M%S_s.rda" and stored at the root of the working directory.
#'
#' @inheritParams recovery.net
#' @param s The number of the simulation that failed
#'
#' @return Nothing, after logging and dumping frames, the function gives the
#'   control back to the general error handler
#'
#' @keywords internal
netsim_error_logger <- function(dat, s) {
  if (get_control(dat, ".traceback.on.error")) {
    message("\n",
      "***************\n",
      "** TRACEBACK **\n",
      "***************"
    )
    traceback(0)
  }

  if (get_control(dat, ".dump.frame.on.error")) {
    dump_name <- format(Sys.time(), format = "dump_%Y%m%d_%H%M%S")
    dump_name <- paste0(dump_name, "_", s, ".rda")
    star_header <- paste0(rep("*", nchar(dump_name)))
    message("\n",
      star_header, "\n",
      "DUMP FILE:\n",
      dump_name, "\n",
      star_header, "\n"
    )
    utils::dump.frames()
    save.image(file = dump_name)
  }
}

##
## Testing EpiModel output
##
##  The main testing functions below check for consistency of output:
##  that the compartment sizes and flows are consistent according to
##  discrete-time difference equations
##


#' @title Write Out Test Progress to Console
#'
#' @description Writes the name of a test and `...` to console to show test
#'              progress.
#'
#' @param test Character string with the name of a test.
#'
#' @keywords internal
#' @export
#'
mcat <- function(test) {
  cat("\n", test, " ... ", sep = "")
  }


#' @title Test the Model Output from a Network Model
#'
#' @description Tests whether the model output from a network model is
#'              consistent with key balancing equations for compartment and
#'              flow sizes for each simulation for each time step.
#'
#' @param x An object of class `netsim`.
#'
#' @keywords internal
#' @export
#'
test_net <- function(x) {

  type <- x$control$type
  nsims <- x$control$nsims
  vital <- x$param$vital
  groups <- x$param$groups

  for (s in 1:nsims) {
    df <- as.data.frame(x, out = "vals", sim = s)
    for (i in 2:nrow(df)) {

      if (type == "SI") {
        if (vital == FALSE) {
          test <- with(df, s.num[i] == s.num[i - 1] - si.flow[i])
          if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
          test <- with(df, i.num[i] == i.num[i - 1] + si.flow[i])
          if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          if (groups == 2) {
            test <- with(df, s.num.g2[i] == s.num.g2[i - 1] - si.flow.g2[i])
            if (test == FALSE) stop("\nFailed *s.num.g2* at SIM", s, "TIME", i)
            test <- with(df, i.num.g2[i] == i.num.g2[i - 1] + si.flow.g2[i])
            if (test == FALSE) stop("\nFailed *i.num.g2* at SIM", s, "TIME", i)
          }
        }
        if (vital == TRUE) {
          test <- with(df, num[i] == num[i - 1] + a.flow[i] -
                         ds.flow[i] - di.flow[i])
          if (test == FALSE) stop("\nFailed *num* at SIM", s, "TIME", i)
          test <- with(df, s.num[i] == s.num[i - 1] - si.flow[i] +
                         a.flow[i] - ds.flow[i])
          if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
          test <- with(df, i.num[i] == i.num[i - 1] + si.flow[i] - di.flow[i])
          if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          if (groups == 2) {
            test <- df$num.g2[i] == df$num.g2[i - 1] + df$a.flow.g2[i] -
                                    df$ds.flow.g2[i] - df$di.flow.g2[i]
            if (test == FALSE) stop("\nFailed *num.g2* at SIM", s, "TIME", i)
            test <- df$s.num.g2[i] == df$s.num.g2[i - 1] - df$si.flow.g2[i] +
                                      df$a.flow.g2[i] - df$ds.flow.g2[i]
            if (test == FALSE) stop("\nFailed *s.num.g2* at SIM", s, "TIME", i)
            test <- with(df, i.num.g2[i] == i.num.g2[i - 1] +
                           si.flow.g2[i] - di.flow.g2[i])
            if (test == FALSE) stop("\nFailed *i.num.g2* at SIM", s, "TIME", i)
          }
        }
      }

      if (type == "SIR") {
        if (vital == FALSE) {
          test <- with(df, s.num[i] == s.num[i - 1] - si.flow[i])
          if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
          test <- with(df, i.num[i] == i.num[i - 1] + si.flow[i] - ir.flow[i])
          if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          test <- with(df, r.num[i] == r.num[i - 1] + ir.flow[i])
          if (test == FALSE) stop("\nFailed *r.num* at SIM", s, "TIME", i)
          if (groups == 2) {
            test <- with(df, s.num.g2[i] == s.num.g2[i - 1] - si.flow.g2[i])
            if (test == FALSE) stop("\nFailed *s.num.g2* at SIM", s, "TIME", i)
            test <- with(df, i.num.g2[i] == i.num.g2[i - 1] +
                           si.flow.g2[i] - ir.flow.g2[i])
            if (test == FALSE) stop("\nFailed *i.num.g2* at SIM", s, "TIME", i)
            test <- with(df, r.num.g2[i] == r.num.g2[i - 1] + ir.flow.g2[i])
            if (test == FALSE) stop("\nFailed *r.num.g2* at SIM", s, "TIME", i)
          }
        }
        if (vital == TRUE) {
          test <- with(df, num[i] == num[i - 1] + a.flow[i] - ds.flow[i] -
                         di.flow[i] - dr.flow[i])
          if (test == FALSE) stop("\nFailed *num* at SIM", s, "TIME", i)
          test <- with(df, s.num[i] == s.num[i - 1] - si.flow[i] +
                         a.flow[i] - ds.flow[i])
          if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
          test <- with(df, i.num[i] == i.num[i - 1] + si.flow[i] -
                         ir.flow[i] - di.flow[i])
          if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          test <- with(df, r.num[i] == r.num[i - 1] + ir.flow[i] - dr.flow[i])
          if (test == FALSE) stop("\nFailed *r.num* at SIM", s, "TIME", i)
          if (groups == 2) {
            test <- with(df, num.g2[i] == num.g2[i - 1] +
                           a.flow.g2[i] - ds.flow.g2[i] -
                           di.flow.g2[i] - dr.flow.g2[i])
            if (test == FALSE) stop("\nFailed *num* at SIM", s, "TIME", i)
            test <- with(df, s.num.g2[i] == s.num.g2[i - 1] - si.flow.g2[i] +
                           a.flow.g2[i] - ds.flow.g2[i])
            if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
            test <- with(df, i.num.g2[i] == i.num.g2[i - 1] + si.flow.g2[i] -
                                            ir.flow.g2[i] - di.flow.g2[i])
            if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
            test <- with(df, r.num.g2[i] == r.num.g2[i - 1] +
                           ir.flow.g2[i] - dr.flow.g2[i])
            if (test == FALSE) stop("\nFailed *r.num* at SIM", s, "TIME", i)
          }
        }
      }

      if (type == "SIS") {
        if (vital == FALSE) {
          test <- with(df, s.num[i] == s.num[i - 1] - si.flow[i] + is.flow[i])
          if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
          test <- with(df, i.num[i] == i.num[i - 1] + si.flow[i] - is.flow[i])
          if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          if (groups == 2) {
            test <- with(df, s.num.g2[i] == s.num.g2[i - 1] -
                           si.flow.g2[i] + is.flow.g2[i])
            if (test == FALSE) stop("\nFailed *s.num.g2* at SIM", s, "TIME", i)
            test <- with(df, i.num.g2[i] == i.num.g2[i - 1] +
                           si.flow.g2[i] - is.flow.g2[i])
            if (test == FALSE) stop("\nFailed *i.num.g2* at SIM", s, "TIME", i)
          }
        }
        if (vital == TRUE) {
          test <- with(df, num[i] == num[i - 1] + a.flow[i] -
                         ds.flow[i] - di.flow[i])
          if (test == FALSE) stop("\nFailed *num* at SIM", s, "TIME", i)
          test <- with(df, s.num[i] == s.num[i - 1] - si.flow[i] + is.flow[i] +
                         a.flow[i] - ds.flow[i])
          if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
          test <- with(df, i.num[i] == i.num[i - 1] + si.flow[i] -
                         is.flow[i] - di.flow[i])
          if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          if (groups == 2) {
            test <- with(df, num.g2[i] == num.g2[i - 1] + a.flow.g2[i] -
                           ds.flow.g2[i] - di.flow.g2[i])
            if (test == FALSE) stop("\nFailed *num* at SIM", s, "TIME", i)
            test <- with(df, s.num.g2[i] == s.num.g2[i - 1] - si.flow.g2[i] +
                           is.flow.g2[i] + a.flow.g2[i] - ds.flow.g2[i])
            if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
            test <- with(df, i.num.g2[i] == i.num.g2[i - 1] + si.flow.g2[i] -
                           is.flow.g2[i] - di.flow.g2[i])
            if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          }
        }
      }

    }
  }

}


#' @title Test the Model Output from a Stochastic Individual Contact Model
#'
#' @description Tests whether the model output from an individual contact model
#'              is consistent with key balancing equations for compartment and
#'              flow sizes for each simulation for each time step.
#'
#' @param x An object of class `icm`.
#'
#' @keywords internal
#' @export
#'
test_icm <- function(x) {

  type <- x$control$type
  nsims <- x$control$nsims
  vital <- x$param$vital
  groups <- x$param$groups

  for (s in 1:nsims) {
    df <- as.data.frame(x, out = "vals", sim = s)
    for (i in 2:nrow(df)) {

      if (type == "SI") {
        if (vital == FALSE) {
          test <- with(df, s.num[i] == s.num[i - 1] - si.flow[i])
          if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
          test <- with(df, i.num[i] == i.num[i - 1] + si.flow[i])
          if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          if (groups == 2) {
            test <- with(df, s.num.g2[i] == s.num.g2[i - 1] - si.flow.g2[i])
            if (test == FALSE) stop("\nFailed *s.num.g2* at SIM", s, "TIME", i)
            test <- with(df, i.num.g2[i] == i.num.g2[i - 1] + si.flow.g2[i])
            if (test == FALSE) stop("\nFailed *i.num.g2* at SIM", s, "TIME", i)
          }
        }
        if (vital == TRUE) {
          test <- with(df, num[i] == num[i - 1] + a.flow[i] -
                         ds.flow[i] - di.flow[i])
          if (test == FALSE) stop("\nFailed *num* at SIM", s, "TIME", i)
          test <- with(df, s.num[i] == s.num[i - 1] - si.flow[i] +
                         a.flow[i] - ds.flow[i])
          if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
          test <- with(df, i.num[i] == i.num[i - 1] + si.flow[i] - di.flow[i])
          if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          if (groups == 2) {
            test <- df$num.g2[i] == df$num.g2[i - 1] + df$a.flow.g2[i] -
                                    df$ds.flow.g2[i] - df$di.flow.g2[i]
            if (test == FALSE) stop("\nFailed *num.g2* at SIM", s, "TIME", i)
            test <- df$s.num.g2[i] == df$s.num.g2[i - 1] - df$si.flow.g2[i] +
                                      df$a.flow.g2[i] - df$ds.flow.g2[i]
            if (test == FALSE) stop("\nFailed *s.num.g2* at SIM", s, "TIME", i)
            test <- with(df, i.num.g2[i] == i.num.g2[i - 1] +
                           si.flow.g2[i] - di.flow.g2[i])
            if (test == FALSE) stop("\nFailed *i.num.g2* at SIM", s, "TIME", i)
          }
        }
      }

      if (type == "SIR") {
        if (vital == FALSE) {
          test <- with(df, s.num[i] == s.num[i - 1] - si.flow[i])
          if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
          test <- with(df, i.num[i] == i.num[i - 1] + si.flow[i] - ir.flow[i])
          if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          test <- with(df, r.num[i] == r.num[i - 1] + ir.flow[i])
          if (test == FALSE) stop("\nFailed *r.num* at SIM", s, "TIME", i)
          if (groups == 2) {
            test <- with(df, s.num.g2[i] == s.num.g2[i - 1] - si.flow.g2[i])
            if (test == FALSE) stop("\nFailed *s.num.g2* at SIM", s, "TIME", i)
            test <- with(df, i.num.g2[i] == i.num.g2[i - 1] +
                           si.flow.g2[i] - ir.flow.g2[i])
            if (test == FALSE) stop("\nFailed *i.num.g2* at SIM", s, "TIME", i)
            test <- with(df, r.num.g2[i] == r.num.g2[i - 1] + ir.flow.g2[i])
            if (test == FALSE) stop("\nFailed *r.num.g2* at SIM", s, "TIME", i)
          }
        }
        if (vital == TRUE) {
          test <- with(df, num[i] == num[i - 1] + a.flow[i] - ds.flow[i] -
                         di.flow[i] - dr.flow[i])
          if (test == FALSE) stop("\nFailed *num* at SIM", s, "TIME", i)
          test <- with(df, s.num[i] == s.num[i - 1] - si.flow[i] +
                         a.flow[i] - ds.flow[i])
          if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
          test <- with(df, i.num[i] == i.num[i - 1] + si.flow[i] -
                         ir.flow[i] - di.flow[i])
          if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          test <- with(df, r.num[i] == r.num[i - 1] + ir.flow[i] - dr.flow[i])
          if (test == FALSE) stop("\nFailed *r.num* at SIM", s, "TIME", i)
          if (groups == 2) {
            test <- with(df, num.g2[i] == num.g2[i - 1] + a.flow.g2[i] -
                           ds.flow.g2[i] - di.flow.g2[i] - dr.flow.g2[i])
            if (test == FALSE) stop("\nFailed *num.g2* at SIM", s,
                                    "TIME", i, sep = " ")
            test <- with(df, s.num.g2[i] == s.num.g2[i - 1] - si.flow.g2[i] +
                           a.flow.g2[i] - ds.flow.g2[i])
            if (test == FALSE) stop("\nFailed *s.num.g2* at SIM", s, "TIME", i)
            test <- with(df, i.num.g2[i] == i.num.g2[i - 1] + si.flow.g2[i] -
                           ir.flow.g2[i] - di.flow.g2[i])
            if (test == FALSE) stop("\nFailed *i.num.g2* at SIM", s, "TIME", i)
            test <- with(df, r.num.g2[i] == r.num.g2[i - 1] +
                           ir.flow.g2[i] - dr.flow.g2[i])
            if (test == FALSE) stop("\nFailed *r.num.g2* at SIM", s, "TIME", i)
          }
        }
      }

      if (type == "SIS") {
        if (vital == FALSE) {
          test <- with(df, s.num[i] == s.num[i - 1] - si.flow[i] + is.flow[i])
          if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
          test <- with(df, i.num[i] == i.num[i - 1] + si.flow[i] - is.flow[i])
          if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          if (groups == 2) {
            test <- with(df, s.num.g2[i] == s.num.g2[i - 1] -
                           si.flow.g2[i] + is.flow.g2[i])
            if (test == FALSE) stop("\nFailed *s.num.g2* at SIM", s, "TIME", i)
            test <- with(df, i.num.g2[i] == i.num.g2[i - 1] +
                           si.flow.g2[i] - is.flow.g2[i])
            if (test == FALSE) stop("\nFailed *i.num.g2* at SIM", s, "TIME", i)
          }
        }
        if (vital == TRUE) {
          test <- with(df, num[i] == num[i - 1] + a.flow[i] -
                         ds.flow[i] - di.flow[i])
          if (test == FALSE) stop("\nFailed *num* at SIM", s, "TIME", i)
          test <- with(df, s.num[i] == s.num[i - 1] - si.flow[i] + is.flow[i] +
                         a.flow[i] - ds.flow[i])
          if (test == FALSE) stop("\nFailed *s.num* at SIM", s, "TIME", i)
          test <- with(df, i.num[i] == i.num[i - 1] + si.flow[i] -
                         is.flow[i] - di.flow[i])
          if (test == FALSE) stop("\nFailed *i.num* at SIM", s, "TIME", i)
          if (groups == 2) {
            test <- with(df, num.g2[i] == num.g2[i - 1] + a.flow.g2[i] -
                           ds.flow.g2[i] - di.flow.g2[i])
            if (test == FALSE) stop("\nFailed *num.g2* at SIM", s, "TIME", i)
            test <- with(df, s.num.g2[i] == s.num.g2[i - 1] - si.flow.g2[i] +
                           is.flow.g2[i] + a.flow.g2[i] - ds.flow.g2[i])
            if (test == FALSE) stop("\nFailed *s.num.g2* at SIM", s, "TIME", i)
            test <- with(df, i.num.g2[i] == i.num.g2[i - 1] + si.flow.g2[i] -
                           is.flow.g2[i] - di.flow.g2[i])
            if (test == FALSE) stop("\nFailed *i.num.g2* at SIM", s, "TIME", i)
          }
        }
      }

    }
  }

}

#' @title Plot Compartment Diagram for Epidemic Models
#'
#' @description Plots a compartment flow diagram for deterministic compartmental
#'              models, stochastic individual contact models, and stochastic
#'              network models.
#'
#' @param x An `EpiModel` object of class `dcm`, `icm`, or
#'        `netsim`.
#' @param at Time step for model statistics.
#' @param digits Number of significant digits to print.
#' @param ... Additional arguments passed to plot (not currently used).
#'
#' @details
#' The `comp_plot` function provides a visual summary of an epidemic model
#' at a specific time step. The information contained in `comp_plot` is the
#' same as in the `summary` functions for a model, but presented
#' graphically as a compartment flow diagram.
#'
#' For `dcm` class plots, specify the model run number if the model
#' contains multiple runs, as in a sensitivity analysis. For `icm` and
#' `netsim` class plots, the `run` argument is not used; the plots
#' show the means and standard deviations across simulations at the specified
#' time step.
#'
#' These plots are currently limited to one-group models for each of the three
#' model classes. That functionality may be expanded in future software
#' releases.
#'
#' @export
#' @keywords plot
#'
#' @examples
#' ## Example 1: DCM SIR model with varying act.rate
#' param <- param.dcm(inf.prob = 0.2, act.rate = 5:7,
#'                    rec.rate = 1/3, a.rate = 1/90, ds.rate = 1/100,
#'                    di.rate = 1/35, dr.rate = 1/100)
#' init <- init.dcm(s.num = 1000, i.num = 1, r.num = 0)
#' control <- control.dcm(type = "SIR", nsteps = 25, verbose = FALSE)
#' mod1 <- dcm(param, init, control)
#' comp_plot(mod1, at = 25, run = 3)
#'
#' ## Example 2: ICM SIR model with 3 simulations
#' param <- param.icm(inf.prob = 0.2, act.rate = 3, rec.rate = 1/50,
#'                    a.rate = 1/100, ds.rate = 1/100,
#'                    di.rate = 1/90, dr.rate = 1/100)
#' init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
#' control <- control.icm(type = "SIR", nsteps = 25,
#'                        nsims = 3, verbose = FALSE)
#' mod2 <- icm(param, init, control)
#' comp_plot(mod2, at = 25, digits = 1)
#'
comp_plot <- function(x, at, digits, ...) {
  UseMethod("comp_plot")
}

#' @method comp_plot netsim
#' @rdname comp_plot
#' @export
comp_plot.netsim <- function(x, at = 1, digits = 3, ...) {

  comp_plot.icm(x = x, at = at, digits = digits, ...)

}

#' @method comp_plot icm
#' @rdname comp_plot
#' @export
comp_plot.icm <- function(x, at = 1, digits = 3, ...) {

  # Variables
  nsteps <- x$control$nsteps
  dis.type <- x$control$type
  vital <- x$param$vital

  # Standardize groups
  if (inherits(x, "icm")) {
    groups <- x$param$groups
  }
  if (inherits(x, "netsim")) {
    groups <- x$param$groups
  }
  if (groups != 1) {
    stop("Only 1-group models currently supported")
  }

  # Time
  if (at > nsteps || at < 1) {
    stop("Specify a timestep between 1 and ", nsteps)
  }

  ## Dataframe subsets for plots
  df.mn <- as.data.frame(x, out = "mean")
  df.mn <- round(df.mn[at == df.mn$time, ], digits)
  df.sd <- as.data.frame(x, out = "sd")
  df.sd <- round(df.sd[at == df.sd$time, ], digits)

  ## Change graphical parameters
  # Register restore handler BEFORE mutating par()/options() so user state is
  # restored on both normal and error exit (CRAN policy).
  ops <- list(mar = par()$mar, mfrow = par()$mfrow, mgp = par()$mgp)
  old_scipen <- getOption("scipen")
  on.exit({
    par(ops)
    options(scipen = old_scipen)
  })
  par(mar = c(0, 0, 2, 0))
  options(scipen = 10)

  ## Main Plot
  plot(0:100, 0:100, type = "n", axes = FALSE)
  title(main = paste(dis.type, "Model Diagram"))
  mtext(paste0("Simulation means(sd) | time=", at),
        side = 3, cex = 0.8, line = -1)

  ## 1. SI Model
  if (dis.type == "SI" && groups == 1) {
    mbox(22, 40, "Susceptible", paste0(df.mn$s.num, "(", df.sd$s.num, ")"))
    mbox(57, 40, "Infected", paste0(df.mn$i.num, "(", df.sd$i.num, ")"))
    harrow(22, 40, "si.flow", df.mn$si.flow, dir = "right")
    if (vital == TRUE) {
      varrow(22, 40, "ds.flow", df.mn$ds.flow, dir = "out")
      varrow(57, 40, "di.flow", df.mn$di.flow, dir = "out")
      varrow(22, 40, "a.flow", df.mn$a.flow, dir = "in")
    }
  }

  ## 2. SIR Model
  if (dis.type == "SIR" && groups == 1) {
    mbox(5, 40, "Susceptible", paste0(df.mn$s.num, "(", df.sd$s.num, ")"))
    mbox(40, 40, "Infected", paste0(df.mn$i.num, "(", df.sd$i.num, ")"))
    mbox(75, 40, "Recovered", paste0(df.mn$r.num, "(", df.sd$r.num, ")"))
    harrow(5, 40, "si.flow", df.mn$si.flow, dir = "right")
    harrow(40, 40, "ir.flow", df.mn$ir.flow, dir = "right")
    if (vital == TRUE) {
      varrow(5, 40, "ds.flow", df.mn$ds.flow, dir = "out")
      varrow(40, 40, "di.flow", df.mn$di.flow, dir = "out")
      varrow(75, 40, "dr.flow", df.mn$dr.flow, dir = "out")
      varrow(5, 40, "a.flow", df.mn$a.flow, dir = "in")
    }
  }

  ## 3. SIS Model
  if (dis.type == "SIS" && groups == 1) {
    mbox(22, 40, "Susceptible", paste0(df.mn$s.num, "(", df.sd$s.num, ")"))
    mbox(57, 40, "Infected", paste0(df.mn$i.num, "(", df.sd$i.num, ")"))
    harrow(22, 40, "si.flow", df.mn$si.flow, dir = "right")
    harrow(22, 40, "is.flow", df.mn$is.flow, dir = "left")
    if (vital == TRUE) {
      varrow(22, 40, "ds.flow", df.mn$ds.flow, dir = "out")
      varrow(57, 40, "di.flow", df.mn$di.flow, dir = "out")
      varrow(22, 40, "a.flow", df.mn$a.flow, dir = "in")
    }
  }
}

#' @param run Model run number, for `dcm` class models with multiple runs
#'        (sensitivity analyses).
#' @method comp_plot dcm
#' @rdname comp_plot
#' @export
comp_plot.dcm <- function(x, at = 1, digits = 3, run = 1, ...) {


  ## Variables
  nsteps <- x$control$nsteps
  dis.type <- x$control$type
  groups <- x$param$groups
  vital <- x$param$vital

  ## Errors
  if (groups != 1) {
    stop("Only 1-group dcm models currently supported")
  }

  ## Time
  if (at > nsteps || at < 1) {
    stop("Specify a time step between 1 and ", nsteps)
  }
  intime <- at
  at <- which(x$control$timesteps == intime)

  ## Dataframe subsets
  df <- as.data.frame(x, run = run)
  df <- round(df[at, ], digits)

  ## Change graphical parameters
  # Register restore handler BEFORE mutating par()/options() so user state is
  # restored on both normal and error exit (CRAN policy).
  ops <- list(mar = par()$mar, mfrow = par()$mfrow, mgp = par()$mgp)
  old_scipen <- getOption("scipen")
  on.exit({
    par(ops)
    options(scipen = old_scipen)
  })
  par(mar = c(0, 0, 2, 0))
  options(scipen = 10)

  ## Main Plot
  plot(0:100, 0:100, type = "n", axes = FALSE)
  title(main = paste(dis.type, "Model Diagram"))
  mtext(paste0("time=", intime, "  |  run=", run),
        side = 3, cex = 0.8, line = -1)

  ## 1. SI Model
  if (dis.type == "SI") {
    mbox(22, 40, "Susceptible", df$s.num)
    mbox(57, 40, "Infected", df$i.num)
    harrow(22, 40, "si.flow", df$si.flow, dir = "right")
    if (vital == TRUE) {
      varrow(22, 40, "ds.flow", df$ds.flow, dir = "out")
      varrow(57, 40, "di.flow", df$di.flow, dir = "out")
      varrow(22, 40, "a.flow", df$a.flow, dir = "in")
    }
  }

  ## 2. SIR Model
  if (dis.type == "SIR") {
    mbox(5, 40, "Susceptible", df$s.num)
    mbox(40, 40, "Infected", df$i.num)
    mbox(75, 40, "Recovered", df$r.num)
    harrow(5, 40, "si.flow", df$si.flow, dir = "right")
    harrow(40, 40, "ir.flow", df$ir.flow, dir = "right")
    if (vital == TRUE) {
      varrow(5, 40, "ds.flow", df$ds.flow, dir = "out")
      varrow(40, 40, "di.flow", df$di.flow, dir = "out")
      varrow(75, 40, "dr.flow", df$dr.flow, dir = "out")
      varrow(5, 40, "a.flow", df$a.flow, dir = "in")
    }
  }

  ## 3. SIS Model
  if (dis.type == "SIS") {
    mbox(22, 40, "Susceptible", df$s.num)
    mbox(57, 40, "Infected", df$i.num)
    harrow(22, 40, "si.flow", df$si.flow, dir = "right")
    harrow(22, 40, "is.flow", df$is.flow, dir = "left")
    if (vital == TRUE) {
      varrow(22, 40, "ds.flow", df$ds.flow, dir = "out")
      varrow(57, 40, "di.flow", df$di.flow, dir = "out")
      varrow(22, 40, "a.flow", df$a.flow, dir = "in")
    }
  }
}

## comp_plot helper utilities ##
#  Text box
mbox <- function(x, y, title, val) {
  polygon(c(x, x + 20, x + 20, x), c(y, y, y + 20, y + 20))
  text(x + 10, y + 10, paste(title, "\n n=", val, sep = ""), cex = 0.9)
}
#  Horizontal arrow
harrow <- function(xbox, ybox, title, val, dir) {
  if (dir == "right") {
    arrows(xbox + 20, ybox + 12, xbox + 35, lwd = 2, length = 0.15)
    text(xbox + 27.5, ybox + 17, paste(title, val, sep = "="), cex = 0.8)
  }
  if (dir == "left") {
    arrows(xbox + 20 + 15, ybox + 5, xbox + 20, lwd = 2, length = 0.15)
    text(xbox + 27.5, ybox + 2, paste(title, val, sep = "="), cex = 0.8)
  }
}
#  Vertical arrow
varrow <- function(xbox, ybox, title, val, dir) {
  if (dir == "out") {
    arrows(xbox + 10, ybox, xbox + 10, ybox - 25, lwd = 2, length = 0.15)
    text(xbox + 10, ybox - 12.5, paste(title, val, sep = "="),
         cex = 0.8, pos = 4)
  }
  if (dir == "in") {
    arrows(xbox + 10, ybox + 45, xbox + 10, ybox + 20, lwd = 2, length = 0.15)
    text(xbox + 10, ybox + 32.5, paste(title, val, sep = "="),
         cex = 0.8, pos = 4)
  }
}

#' @title Record Attribute History
#'
#' @description
#' This function records values specific to a time-step and a group of nodes.
#' In the records, the `posit_ids` are converted to `unique_ids` which
#' allows the recording of data for nodes that are no longer in the network by
#' the end of the run. The records are stored in `dat[["attr.history"]]`
#' where `dat` is the main `netsim_dat` class object, and can be
#' accessed from the `netsim` object with `get_attr_history`.
#'
#' @inheritParams recovery.net
#' @param at The time where the recording happens.
#' @param attribute The name of the value to record.
#' @param posit_ids A numeric vector of posit_ids to which the measure applies.
#'   (see `get_posit_ids`).
#' @param values The values to be recorded.
#'
#' @inherit recovery.net return
#'
#' @details
#' See the "Time-Varying Parameters" section of the "Working With Model
#' Parameters" vignette.
#'
#' @examples
#' \dontrun{
#' # This function must be used inside a custom module
#' dat <- record_attr_history(dat, at, "attr_1", get_posit_ids(dat), 5)
#' some_nodes <- get_posit_ids(dat)
#' some_nodes <- some_nodes[runif(length(some_nodes)) < 0.2]
#' dat <- record_attr_history(
#'   dat, at,
#'   "attr_2",
#'   some_nodes,
#'   rnorm(length(some_nodes))
#' )
#' }
#'
#' @export
record_attr_history <- function(dat, at, attribute, posit_ids, values) {
  if (is.null(dat[["attr.history"]])) {
    dat[["attr.history"]] <- collections::queue()
  }

  if (length(values) != 1 && length(values) != length(posit_ids)) {
    stop(
      "When trying to record a value for `", attribute, "` at time ", at,
      "The size of the `values` vector is not equal to the number of nodes ",
      "selected by the `posit_ids` vector nor of length 1. \n",
      "Expected: ", length(posit_ids), " or 1 \n",
      "Given: ", length(posit_ids)
    )
  }

  element <- list(at, attribute, get_unique_ids(dat, posit_ids), values)
  names(element) <- c("time", "attribute", "uids", "values")

  dat[["attr.history"]]$push(element)
  return(dat)
}

#' @title Record an Arbitrary Object During a Simulation
#'
#' @description
#' This function records any object during a simulation to allow its
#' inspection afterward. The records are stored in `dat[["raw.records"]]`
#' during the simulation, where `dat` is the main `netsim_dat` class
#' object, and in the `netsim` object under the `raw.records`
#' `collections::queue` object.
#'
#' @inheritParams recovery.net
#' @param at The time where the recording happens.
#' @param label The name to give to the recorded object.
#' @param object The object to be recorded.
#'
#' @inherit recovery.net return
#'
#' @details
#' See the "Time-Varying Parameters" section of the "Working With Model
#' Parameters" vignette.
#'
#' @examples
#' \dontrun{
#'
#' dat <- record_raw_object(dat, at, "a.df", data.frame(x = 2:200))
#' dat <- record_raw_object(dat, at, "a.message", "I recorded something")
#'
#' }
#'
#' @export
record_raw_object <- function(dat, at, label, object) {
  if (is.null(dat[["raw.records"]])) {
    dat[["raw.records"]] <- collections::queue()
  }

  element <- list(at, label, object)
  names(element) <- c("time", "label", "object")

  dat[["raw.records"]]$push(element)

  return(dat)
}

# Check if the `.checkpoint` controls are present and well formed
netsim_is_checkpointed <- function(control) {
  ckpt_file_check <-
    is.character(control[[".checkpoint.dir"]]) &&
    length(control[[".checkpoint.dir"]]) == 1

  ckpt_steps_check <-
    is.numeric(control[[".checkpoint.steps"]]) &&
    length(control[[".checkpoint.steps"]]) == 1 &&
    control[[".checkpoint.steps"]] > 0

  return(ckpt_file_check && ckpt_steps_check)
}

# Check the existence of a checkpoint file
netsim_is_resume_checkpoint <- function(control, s) {
  resume_checkpoint <- FALSE
  if (control[[".checkpointed"]])
    resume_checkpoint <- file.exists(netsim_get_checkpoint_filename(control, s))

  return(resume_checkpoint)
}

netsim_load_checkpoint <- function(control, s) {
  checkpoint_path <- netsim_get_checkpoint_filename(control, s)
  dat <- readRDS(checkpoint_path)
  message(
    "Simulation ", s, " - Restarting from step ", get_current_timestep(dat),
    " - file: \"", checkpoint_path, "\""
  )
  return(dat)
}

netsim_save_checkpoint <- function(dat, s) {
  if (!dir.exists(get_control(dat, ".checkpoint.dir")))
    dir.create(get_control(dat, ".checkpoint.dir"), recursive = TRUE)

  checkpoint_file_name <- netsim_get_checkpoint_filename(dat[["control"]], s)
  tmp_file_name <- paste0(checkpoint_file_name, "_tmp")

  saveRDS(dat, tmp_file_name,
          compress = get_control(dat, ".checkpoint.compress"))

  if (file.exists(checkpoint_file_name)) file.remove(checkpoint_file_name)
  file.rename(tmp_file_name, checkpoint_file_name)

  message(
    "Simulation ", s, " - Checkpoint at step ", get_current_timestep(dat),
    " - file: \"", checkpoint_file_name, "\""
  )

  invisible(dat)
}


netsim_get_checkpoint_filename <- function(control, s) {
  paste0(control[[".checkpoint.dir"]], "/", s, ".rds")
}

# Remove all checkpoint files and directories
#
netsim_clear_checkpoint <- function(control) {
  if (control[[".checkpointed"]])
    unlink(control[[".checkpoint.dir"]], recursive = TRUE)
  invisible(TRUE)
}

#' @title Epidemic Parameters for Deterministic Compartmental Models
#'
#' @description Sets the epidemic parameters for deterministic compartmental
#'              models simulated with `dcm`.
#'
#' @param inf.prob Probability of infection per transmissible act between
#'        a susceptible and an infected person. In two-group models, this is the
#'        probability of infection for the group 1 members.
#' @param inter.eff Efficacy of an intervention which affects the per-act
#'        probability of infection. Efficacy is defined as 1 - the relative
#'        hazard of infection given exposure to the intervention, compared to no
#'        exposure.
#' @param inter.start Time step at which the intervention starts, between 1 and
#'        the number of time steps specified in the model. This will default to
#'        1 if `inter.eff` is defined but this parameter is not.
#' @param act.rate Average number of transmissible acts per person per unit
#'        time. For two-group models, this is the number of acts per group 1
#'        person per unit time; a balance between the acts in groups 1 and 2 is
#'        necessary, and set using the `balance` parameter (see details).
#' @param rec.rate Average rate of recovery with immunity (in `SIR` models)
#'        or re-susceptibility (in `SIS` models). The recovery rate is the
#'        reciprocal of the disease duration. For two-group models, this is the
#'        recovery rate for group 1 persons only. This parameter is only used
#'        for `SIR` and `SIS` models.
#' @param a.rate Arrival or entry rate. For one-group models, the arrival rate
#'        is the rate of new arrivals per person per unit time. For two-group
#'        models, the arrival rate is parameterized as a rate per group 1
#'        person per unit time, with the `a.rate.g2` rate set as described
#'        below.
#' @param ds.rate Departure or exit rate for susceptible persons. For two-group
#'        models, it is the rate for the group 1 susceptible persons only.
#' @param di.rate Departure or exit rate for infected persons. For two-group
#'        models, it is the rate for the group 1 infected persons only.
#' @param dr.rate Departure or exit rate for recovered persons. For two-group
#'        models, it is the rate for the group 1 recovered persons only. This
#'        parameter is only used for `SIR` models.
#' @param inf.prob.g2 Probability of infection per transmissible act
#'        between a susceptible group 2 person and an infected group 1 person.
#'        It is the probability of infection to group 2 members.
#' @param act.rate.g2 Average number of transmissible acts per group 2 person
#'        per unit time; a balance between the acts in groups 1 and 2 is
#'        necessary, and set using the `balance` parameter (see details).
#' @param rec.rate.g2 Average rate of recovery with immunity (in `SIR`
#'        models) or re-susceptibility (in `SIS` models) for group 2
#'        persons. This parameter is only used for two-group `SIR` and
#'        `SIS` models.
#' @param a.rate.g2 Arrival or entry rate for group 2. This may either be
#'        specified numerically as the rate of new arrivals per group 2 persons
#'        per unit time, or as `NA` in which case the group 1 rate,
#'        `a.rate`, governs the group 2 rate. The latter is used when,
#'        for example, the first group is conceptualized as female, and the
#'        female population size determines the arrival rate. Such arrivals are
#'        evenly allocated between the two groups.
#' @param ds.rate.g2 Departure or exit rate for group 2 susceptible persons.
#' @param di.rate.g2 Departure or exit rate for group 2 infected persons.
#' @param dr.rate.g2 Departure or exit rate for group 2 recovered persons. This
#'        parameter is only used for `SIR` model types.
#' @param balance For two-group models, balance the `act.rate` to the rate
#'        set for group 1 (with `balance="g1"`) or group 2 (with
#'        `balance="g2"`). See details.
#' @param ... Additional arguments passed to model.
#'
#' @details
#' `param.dcm` sets the epidemic parameters for deterministic compartmental
#' models solved with the [dcm()] function. The models may use the
#' base types, for which these parameters are used, or original model
#' specifications for which these parameters may be used (but not necessarily).
#'
#' For base models, the model specification will be selected as a function
#' of the model parameters entered here and the control settings in
#' [control.dcm()]. One-group and two-group models are available,
#' where the former assumes a homogeneous mixing in the population and the
#' latter assumes some form of heterogeneous mixing between two distinct
#' partitions in the population (e.g., men and women). Specifying any group two
#' parameters (those with a `.g2`) implies the simulation of a two-group
#' model. All the parameters for a desired model type must be specified, even if
#' they are zero.
#'
#' @section Act Balancing:
#' In two-group models, a balance between the number of acts for group 1 members
#' and those for group 2 members must be maintained. With purely heterogeneous
#' mixing, the product of one group size and act rate must equal the product of
#' the other group size and act rate: \eqn{N_1 \alpha_1 = N_2 \alpha_2}, where
#' \eqn{N_i} is the group size and \eqn{\alpha_i} the group-specific act rate
#' at time \eqn{t}. The `balance` parameter here specifies which group's
#' act rate should control the others with respect to balancing.
#'
#' @section Sensitivity Analyses:
#' `dcm` has been designed to easily run DCM sensitivity analyses, where a
#' series of models varying one or more of the model parameters is run. This is
#' possible by setting any parameter as a vector of length greater than one.
#'
#' @section New Model Types:
#' An original model may use either the existing model parameters named here,
#' an original set of parameters, or a combination of both. The `...`
#' argument allows the user to pass an arbitrary set of new model parameters into
#' `param.dcm`. Whereas there are strict checks for base models that the
#' model parameters are valid, parameter validity is the user's responsibility
#' with these original models.
#'
#' @return An `EpiModel` object of class `param.dcm`.
#'
#' @seealso Use [init.dcm()] to specify the initial conditions and
#'          [control.dcm()] to specify the control settings. Run the
#'          parameterized model with [dcm()].
#'
#' @keywords parameterization
#'
#' @examples
#' # SI model parameters
#' param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
#'
#' # SIS model with sensitivity analysis over act.rate
#' param <- param.dcm(inf.prob = 0.2, act.rate = seq(0.1, 0.5, 0.1),
#'                    rec.rate = 1/50)
#'
#' @export
#'
param.dcm <- function(inf.prob, inter.eff, inter.start, act.rate, rec.rate,
                      a.rate, ds.rate, di.rate, dr.rate, inf.prob.g2,
                      act.rate.g2, rec.rate.g2, a.rate.g2, ds.rate.g2,
                      di.rate.g2, dr.rate.g2, balance, ...) {

  # Get arguments
  p <- list()
  formal.args <- formals(sys.function())
  formal.args[["..."]] <- NULL
  for (arg in names(formal.args)) {
    if (as.logical(mget(arg) != "")) {
      p[arg] <- list(get(arg))
    }
  }
  dot.args <- list(...)
  names.dot.args <- names(dot.args)
  if (length(dot.args) > 0) {
    for (i in seq_along(dot.args)) {
      p[[names.dot.args[i]]] <- dot.args[[i]]
    }
  }


  if (!is.null(p$inter.eff) && is.null(p$inter.start)) {
    p$inter.start <- 1
  }

  class(p) <- c("param.dcm", "list")
  return(p)
}


#' @title Initial Conditions for Deterministic Compartmental Models
#'
#' @description Sets the initial conditions for deterministic compartmental
#'              models simulated with `dcm`.
#'
#' @param s.num Number of initial susceptible persons. For two-group models,
#'        this is the number of initial group 1 susceptible persons.
#' @param i.num Number of initial infected persons. For two-group models, this
#'        is the number of initial group 1 infected persons.
#' @param r.num Number of initial recovered persons. For two-group models, this
#'        is the number of initial group 1 recovered persons. This parameter is
#'        only used for the `SIR` model type.
#' @param s.num.g2 Number of initial susceptible persons in group 2. This
#'        parameter is only used for two-group models.
#' @param i.num.g2 Number of initial infected persons in group 2. This parameter
#'        is only used for two-group models.
#' @param r.num.g2 Number of initial recovered persons in group 2. This
#'        parameter is only used for two-group `SIR` models.
#' @param ... Additional initial conditions passed to model.
#'
#' @details
#' The initial conditions for a model solved with [dcm()] should be
#' input into the `init.dcm` function. This function handles initial
#' conditions for both base model types and original models.
#'
#' Original models may use the parameter names listed as arguments here, a new
#' set of names, or a combination of both. With new models, initial conditions
#' must be input in the same order that the solved derivatives from the model
#' are output.
#'
#' @section Sensitivity Analyses:
#' Like [param.dcm()], initial conditions may be specified as vectors of length
#' greater than one to run sensitivity analyses over initial conditions. When
#' `sens.param = TRUE` in [control.dcm()] (the default), each element of the
#' vector produces a separate model run. If both parameters and initial
#' conditions have vector values, all vectors must have the same length.
#'
#' @return An `EpiModel` object of class `init.dcm`.
#'
#' @seealso Use [param.dcm()] to specify model parameters and
#'          [control.dcm()] to specify the control settings. Run the
#'          parameterized model with [dcm()].
#'
#' @keywords parameterization
#'
#' @examples
#' # SI model initial conditions
#' init <- init.dcm(s.num = 500, i.num = 1)
#'
#' # Sensitivity analysis over initial infected count
#' init <- init.dcm(s.num = 500, i.num = c(1, 5, 25))
#'
#' @export
#'
init.dcm <- function(s.num, i.num, r.num, s.num.g2, i.num.g2, r.num.g2,
                     ...) {

  # Get arguments
  p <- list()
  formal.args <- formals(sys.function())
  formal.args[["..."]] <- NULL
  for (arg in names(formal.args)) {
    if (as.logical(mget(arg) != "")) {
      p[arg] <- list(get(arg))
    }
  }
  dot.args <- list(...)
  names.dot.args <- names(dot.args)
  if (length(dot.args) > 0) {
    for (i in seq_along(dot.args)) {
      p[[names.dot.args[i]]] <- dot.args[[i]]
    }
  }

  # Reorder arguments
  mc <- names(as.list(sys.call()[-1]))
  out.p <- list()
  for (i in seq_along(mc)) {
    out.p[[mc[i]]] <- p[[mc[i]]]
  }

  ## Output
  class(out.p) <- c("init.dcm", "list")
  return(out.p)
}


#' @title Control Settings for Deterministic Compartmental Models
#'
#' @description Sets the controls for deterministic compartmental models
#'              simulated with [dcm()].
#'
#' @param type Disease type to be modeled, with the choice of `"SI"` for
#'        Susceptible-Infected diseases, `"SIR"` for
#'        Susceptible-Infected-Recovered diseases, and `"SIS"` for
#'        Susceptible-Infected-Susceptible diseases.
#' @param nsteps Number of time steps to solve the model over or vector of times
#'        to solve the model over. If the number of time steps, then this must
#'        be a positive integer of length 1.
#' @param dt Time unit for model solutions, with the default of 1. Model
#'        solutions for fractional time steps may be obtained by setting this to
#'        a number between 0 and 1.
#' @param odemethod Ordinary differential equation (ODE) integration method,
#'        with the default of `"lsoda"` (see [`deSolve::ode`] for other
#'        options).
#' @param dede If `TRUE`, use the delayed differential equation solver,
#'        which allows for time-lagged variables.
#' @param new.mod If not running a base model type, a function with a new
#'        model to be simulated (see details).
#' @param sens.param If `TRUE`, evaluate arguments in parameters or initial
#'        conditions with length greater than 1 as sensitivity analyses, with
#'        one model run per value. If `FALSE`, one model will be run with
#'        parameters and initial conditions of arbitrary length (the model may
#'        error unless the model function is designed to accommodate vectors).
#' @param print.mod If `TRUE`, print the model form to the console.
#' @param verbose If `TRUE`, print model progress to the console.
#' @param ... additional control settings passed to model.
#'
#' @details
#' `control.dcm` sets the required control settings for any deterministic
#' compartmental models solved with the [dcm()] function. Controls are
#' required for both base model types and original models. For all base models,
#' the `type` argument is a necessary parameter and it has no default.
#'
#' @section ODE Solver:
#' As of EpiModel 2.5.5, the default ODE solver was changed from `"rk4"`
#' (fixed-step Runge-Kutta 4) to `"lsoda"` (adaptive step-size). The `"lsoda"`
#' method automatically adjusts its internal step size, providing numerical
#' stability across a wide range of parameter values. The fixed-step `"rk4"`
#' solver could produce numerical instability (negative compartment sizes, NaN
#' values) for models with high transmission rates or stiff dynamics. Users who
#' require the previous behavior can set `odemethod = "rk4"` explicitly.
#'
#' @section New Model Functions:
#' The form of the model function for base models may be displayed with the
#' `print.mod` argument set to `TRUE`. In this case, the model will
#' not be run. These model forms may be used as templates to write original
#' model functions.
#'
#' These new models may be input and solved with [dcm()] using the
#' `new.mod` argument, which requires as input a model function.
#'
#' @return An `EpiModel` object of class `control.dcm`.
#'
#' @seealso Use [param.dcm()] to specify model parameters and
#'          [init.dcm()] to specify the initial conditions. Run the
#'          parameterized model with [dcm()].
#'
#' @keywords parameterization
#'
#' @export
#'
control.dcm <- function(type, nsteps, dt = 1, odemethod = "lsoda",
                        dede = FALSE, new.mod = NULL, sens.param = TRUE,
                        print.mod = FALSE, verbose = FALSE, ...) {



  # Get arguments
  p <- list()
  formal.args <- formals(sys.function())
  formal.args[["..."]] <- NULL
  for (arg in names(formal.args)) {
    if (as.logical(mget(arg) != "")) {
      p[arg] <- list(get(arg))
    }
  }
  dot.args <- list(...)
  names.dot.args <- names(dot.args)
  if (length(dot.args) > 0) {
    for (i in seq_along(dot.args)) {
      p[[names.dot.args[i]]] <- dot.args[[i]]
    }
  }
  if (!is.null(p$new.mod)) {
    p$new.mod.name <- as.list(match.call())$new.mod
  }

  ## Defaults and checks
  if (is.null(p$nsteps)) {
    stop("Specify nsteps")
  }


  # Check type for base models
  if (is.null(p$new.mod)) {
    if (is.null(p$type) || !(p$type %in% c("SI", "SIS", "SIR"))) {
      stop("Specify type as \"SI\", \"SIS\", or \"SIR\" ")
    }
  }

  ## Output
  p <- set.control.class("control.dcm", p)
  return(p)
}


#' @title Cross Checking of Inputs for Deterministic Compartmental Models
#'
#' @description This function checks that the three parameter lists from
#'              [param.dcm()], [init.dcm()], and
#'              [control.dcm()] are consistent.
#'
#' @param param An `EpiModel` object of class [param.dcm()].
#' @param init An `EpiModel` object of class [init.dcm()].
#' @param control An `EpiModel` object of class [control.dcm()].
#'
#' @return
#' This function returns no objects.
#'
#' @export
#' @keywords internal
#'
crosscheck.dcm <- function(param, init, control) {
  check.control.class("dcm", "EpiModel crosscheck.dcm")

  # Main class check --------------------------------------------------------
  if (!inherits(param, "param.dcm")) {
    stop("param must be an object of class param.dcm")
  }
  if (!inherits(init, "init.dcm")) {
    stop("init must be an object of class init.dcm")
  }
  if (!inherits(control, "control.dcm")) {
    stop("control must be an object of class control.dcm")
  }

  # Parameter checks for base models ----------------------------------
  if (is.null(control$new.mod)) {

    ## Defaults
    if (is.null(param$act.rate)) {
      param$act.rate <- 1
    }
    if (is.null(param$vital)) {
      if (!is.null(param$a.rate) ||
            !is.null(param$ds.rate) ||
            !is.null(param$di.rate) ||
            !is.null(param$dr.rate)) {
        param$vital <- TRUE
      } else {
        param$vital <- FALSE
      }
    }

    if (any(grepl(".g2", names(param))) == TRUE) {
      param$groups <- 2
    } else {
      param$groups <- 1
    }

    if (param$groups == 2 && (is.null(param$balance) ||
                                !(param$balance %in% c("g1", "g2")))) {
      stop("Specify balance=\"g1\" or balance=\"g2\" with 2-group models")
    }

    ## Error checks
    # Specify inf.prob
    if (is.null(param$inf.prob)) {
      stop("Specify inf.prob in param.dcm")
    }

    # Check that rec.rate is supplied for SIR/SIS models
    if (control$type %in% c("SIR", "SIS") && is.null(param$rec.rate)) {
      stop("Specify rec.rate in param.dcm")
    }
    if (control$type %in% c("SIR", "SIS") && param$groups == 2 &&
          is.null(param$rec.rate.g2)) {
      stop("Specify rec.rate.g2 in param.dcm")
    }

    # Check that r.num is supplied for SIR models
    if (control$type == "SIR" && is.null(init$r.num)) {
      stop("Specify r.num in init.dcm")
    }
    if (control$type == "SIR" && param$groups == 2 && is.null(init$r.num.g2)) {
      stop("Specify r.num.g2 in init.dcm")
    }

    # Check that groups implied by init and params are consistent
    if (any(grepl(".g2", names(init))) == TRUE) {
      init.groups <- 2
    } else {
      init.groups <- 1
    }
    if (param$groups == 2 && init.groups == 1) {
      stop("Group 2 parameters specified in param.dcm,
           \rbut missing group 2 initial states in init.dcm")
    }
    if (param$groups == 1 && init.groups == 2) {
      stop("Group 2 initial stats specified in init.dcm,
           but missing group 2 parameters in param.dcm")
    }

    # Over-specified initial conditions
    if (control$type != "SIR" && any(c("r.num", "r.num.g2") %in% names(init))) {
      stop("Specified initial number recovered for non-SIR model")
    }

    if (!is.null(param$trans.rate)) {
      stop("The trans.rate parameter has been removed. Use inf.prob instead.")
    }
    if (!is.null(param$trans.rate.g2)) {
      stop("The trans.rate.g2 parameter has been removed. Use inf.prob.g2 instead.")
    }
  }

  on.exit(assign("param", param, pos = parent.frame()))
}

context("Mutate Functions")

test_that("mutate_epi.netsim", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
  est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  # Epidemic model
  param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
  init <- init.net(i.num = 1, i.num.g2 = 0)
  control <- control.net(type = "SI", nsteps = 10, nsims = 2,
                         verbose = FALSE)
  mod1 <- netsim(est1, param, init, control)

  mod1 <- mutate_epi(mod1, i.prev = i.num / num,
                     i.prev.g2 = i.num.g2 / num.g2)
  expect_equal(names(mod1$epi), c("s.num", "i.num", "num",
                                  "s.num.g2", "i.num.g2", "num.g2",
                                  "si.flow", "si.flow.g2", "i.prev",
                                  "i.prev.g2"))

  # Add incidence rate per 100 person years (assume time step = 1 week)
  mod1 <- mutate_epi(mod1, ir100 = 5200 * (si.flow + si.flow.g2) /
                       (s.num + s.num.g2))
  df <- as.data.frame(mod1)
  expect_true("ir100" %in% names(df))
})

test_that("mutate_epi.dcm", {

  param <- param.dcm(inf.prob = 0.3, inf.prob.g2 = 0.15, balance = "g1")
  init <- init.dcm(s.num = 1000, s.num.g2 = 1000, i.num = 1, i.num.g2 = 0)
  control <- control.dcm(type = "SI", nsteps = 100)
  mod1 <- dcm(param, init, control)

  mod1 <- mutate_epi(mod1, i.prev = i.num / num,
                     i.prev.g2 = i.num.g2 / num.g2)
  expect_equal(names(mod1$epi), c("s.num", "s.num.g2", "i.num", "i.num.g2",
                                  "si.flow", "si.flow.g2", "num", "num.g2",
                                  "i.prev", "i.prev.g2"))

  # Add incidence rate per 100 person years (assume time step = 1 week)
  mod1 <- mutate_epi(mod1, ir100 = 5200 * (si.flow + si.flow.g2) /
                       (s.num + s.num.g2))
  df <- as.data.frame(mod1)
  expect_true("ir100" %in% names(df))
})

test_that("mutate_epi.dcm, new mod", {

  syph <- function(t, t0, parms) {
    with(as.list(c(t0, parms)), {

      # 1. track the total population size
      num <- s.num + i.num + r.num

      # 2. define lambda, mu, gamma, and sigma
      ce <- R0 / dur.inf
      lambda <- ce * i.num / num
      mu <- 1 / life.expt
      gamma <- 1 / dur.inf
      sigma <- 1 / dur.imm

      # 3. Write out the four differential equations
      dS <- -lambda * s.num + mu * num - mu * s.num + sigma * r.num
      dI <- lambda * s.num - gamma * i.num - mu * i.num
      dR <- gamma * i.num - mu * r.num - sigma * r.num

      # 4. Outputs
      list(c(dS,
             dI,
             dR))
    })
  }

  param <- param.dcm(R0 = 1.5, life.expt = 365 * 30,
                     dur.inf = 60, dur.imm = 365 * c(8, 10, 12))
  init <- init.dcm(s.num = 1e5, i.num = 1, r.num = 0)
  control <- control.dcm(nsteps = 365, new.mod = syph)
  sim <- dcm(param, init, control)

  sim <- mutate_epi(sim, num = s.num + i.num + r.num)
  sim <- mutate_epi(sim, prev = i.num / num)

  df <- as.data.frame(sim)
  expect_true("num" %in% names(df))
  expect_true("prev" %in% names(df))
  expect_true("num" %in% names(sim$epi))
  expect_true("prev" %in% names(sim$epi))
})

test_that("mutate DCM with constant", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SI", nsteps = 500)
  mod1 <- dcm(param, init, control)
  mod1 <- mutate_epi(mod1, prev = i.num / num,
                     cm = 3)
  expect_true(all(sapply(mod1$epi, class) == "data.frame"))
  expect_true(length(unique(sapply(mod1$epi, nrow))) == 1)

  # by itself
  mod1 <- dcm(param, init, control)
  mod1 <- mutate_epi(mod1, cm = 3)
  expect_true(all(sapply(mod1$epi, class) == "data.frame"))
  expect_true(length(unique(sapply(mod1$epi, nrow))) == 1)
})

context("DCM Extended Models (All SOC)")

# SI Models ---------------------------------------------------------------

test_that("SI, 1G, CL: 1 run", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SI", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$control$nruns, 1)
  expect_equal(x$epi$i.num[150, 1], 388.1552, tol = 0.0001)
  expect_equal(max(x$epi$i.num), 501)
  expect_equal(max(x$epi$s.num + x$epi$i.num), 501)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SI, 1G, CL: varying inf.prob", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.9, 0.1),
                     act.rate = 0.25)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SI", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$control$nruns, 9)
  expect_equal(x$epi$i.num[150, 1], 38.3715, tol = 0.0001)
  expect_equal(x$epi$i.num[150, 4], 500.9153, tol = 0.0001)
  expect_equal(x$epi$i.num[150, 9], 501)
  expect_equal(max(x$epi$i.num), 501)
  expect_equal(max(x$epi$s.num + x$epi$i.num), 501)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"), run = 2)
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SI, 1G, CL: varying act.rate", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.1,
                     act.rate = seq(0.25, 2, 0.25))
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SI", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$control$nruns, 8)
  expect_equal(x$epi$i.num[150, 1], 38.3715, tol = 0.0001)
  expect_equal(x$epi$i.num[150, 2], 388.1552, tol = 0.0001)
  expect_equal(x$epi$i.num[150, 5], 500.998, tol = 0.0001)
  expect_equal(max(x$epi$i.num), 501)
  expect_equal(max(x$epi$s.num + x$epi$i.num), 501)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"), run = 2)
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SI, 1G, CL: varying inf.prob and act.rate", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.8, 0.1),
                     act.rate = seq(0.25, 2, 0.25))
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SI", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$control$nruns, 8)
  expect_equal(x$epi$i.num[150, 1], 38.3715, tol = 0.0001)
  expect_equal(x$epi$i.num[150, 2], 500.9153, tol = 0.0001)
  expect_equal(x$epi$i.num[10, 5], 178.344, tol = 0.5)
  expect_equal(max(x$epi$i.num), 501)
  expect_equal(max(x$epi$s.num + x$epi$i.num), 501)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"), run = 2)
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SI, 2G, CL: 1 run", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.2,
                     act.rate = 0.25, inf.prob.g2 = 0.1, balance = "g1")
  init <- init.dcm(s.num = 500, i.num = 1,
                   s.num.g2 = 500, i.num.g2 = 0)
  control <- control.dcm(type = "SI", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[200, 1], 278.8645, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[200, 1], 212.65, tol = 0.0001)
  expect_equal(max(x$epi$i.num), 501, tol = 0.0001)
  expect_equal(max(x$epi$s.num + x$epi$i.num), 501)
  expect_equal(max(x$epi$i.num + x$epi$i.num.g2 +
                     x$epi$s.num + x$epi$s.num.g2), 1001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SI, 2G, CL: varying inf.prob", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.9, 0.1),
                     act.rate = 0.25, inf.prob.g2 = 0.1, balance = "g1")
  init <- init.dcm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 0)
  control <- control.dcm(type = "SI", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[200, 2], 278.8645, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[200, 2], 212.65, tol = 0.0001)
  expect_equal(x$epi$i.num[200, 4], 494.4093, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[200, 4], 403.4559, tol = 0.0001)
  expect_equal(max(x$epi$i.num), 501)
  expect_equal(max(x$epi$s.num + x$epi$i.num), 501)
  expect_equal(max(x$epi$i.num + x$epi$i.num.g2 +
                     x$epi$s.num + x$epi$s.num.g2), 1001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SI, 1G, OP: 1 run", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.2, act.rate = 0.25,
                     a.rate = 1 / 100, ds.rate = 1 / 100, di.rate = 1 / 90)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SI", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[150, 1], 177.8674, tol = 0.0001)
  expect_equal(x$epi$s.num[200, 1], 154.4402, tol = 0.0001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SI, 1G, OP: varying inf.prob", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.5, 0.05),
                     act.rate = 0.25, a.rate = 1 / 100,
                     ds.rate = 1 / 100, di.rate = 1 / 90)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SI", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[50, 1], 1.9681, tol = 0.0001)
  expect_equal(x$epi$i.num[50, 5], 21.7698, tol = 0.0001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"), run = 2)
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SI, 2G, OP: 1 run", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.2,
                     act.rate = 0.25, inf.prob.g2 = 0.1, balance = "g1",
                     a.rate = 1 / 100, a.rate.g2 = NA, ds.rate = 1 / 100,
                     ds.rate.g2 = 1 / 100, di.rate = 1 / 90,
                     di.rate.g2 = 1 / 90)
  init <- init.dcm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 0)
  control <- control.dcm(type = "SI", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[250, 1], 134.9654, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[250, 1], 98.9927, tol = 0.0001)
  expect_equal(max(x$epi$i.num), 327, tol = 0.001)
  expect_equal(max(x$epi$i.num.g2), 277, tol = 0.001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SI, 2G, OP: varying inf.prob", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.5, 0.05),
                     act.rate = 0.25, inf.prob.g2 = 0.1, balance = "g1",
                     a.rate = 1 / 100, a.rate.g2 = NA, ds.rate = 1 / 100,
                     ds.rate.g2 = 1 / 100, di.rate = 1 / 90,
                     di.rate.g2 = 1 / 90)
  init <- init.dcm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 0)
  control <- control.dcm(type = "SI", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[250, 1], 15.08, tol = 0.0001)
  expect_equal(x$epi$i.num[250, 5], 320.8842, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[250, 1], 15.0784, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[250, 5], 222.7064, tol = 0.0001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})


# SIR Models --------------------------------------------------------------

test_that("SIR, 1G, CL: 1 run", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.2, act.rate = 0.25,
                     rec.rate = 1 / 50)
  init <- init.dcm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.dcm(type = "SIR", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[250, 1], 107.788, tol = 0.0001)
  expect_equal(max(x$epi$s.num + x$epi$i.num + x$epi$r.num), 501)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIR, 1G, CL: varying inf.prob", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.5, 0.05),
                     act.rate = 0.25, rec.rate = 1 / 50)
  init <- init.dcm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.dcm(type = "SIR", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[100, 1], 1.6207, tol = 0.0001)
  expect_equal(x$epi$i.num[100, 5], 122.3146, tol = 0.0001)
  expect_equal(max(x$epi$s.num + x$epi$i.num + x$epi$r.num), 501)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"), run = 2)
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIR, 2G, CL: 1 run", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.2,
                     act.rate = 0.25, inf.prob.g2 = 0.1, balance = "g1",
                     rec.rate = 1 / 100, rec.rate.g2 = 1 / 100)
  init <- init.dcm(s.num = 500, i.num = 1, r.num = 0,
                   s.num.g2 = 500, i.num.g2 = 0, r.num.g2 = 0)
  control <- control.dcm(type = "SIR", nsteps = 1000, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[250, 1], 137.0072, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[250, 1], 102.8639, tol = 0.0001)
  expect_equal(max(x$epi$s.num + x$epi$i.num + x$epi$r.num), 501)
  expect_equal(max(x$epi$s.num + x$epi$i.num + x$epi$r.num +
                     x$epi$s.num.g2 + x$epi$i.num.g2 + x$epi$r.num.g2), 1001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIR, 2G, CL: varying inf.prob", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.5, 0.05),
                     act.rate = 0.25, inf.prob.g2 = 0.1, balance = "g1",
                     rec.rate = 1 / 100, rec.rate.g2 = 1 / 100)
  init <- init.dcm(s.num = 500, i.num = 1, r.num = 0,
                   s.num.g2 = 500, i.num.g2 = 0, r.num.g2 = 0)
  control <- control.dcm(type = "SIR", nsteps = 1000, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[250, 1], 18.9723, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[250, 5], 179.5086, tol = 0.0001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"), run = 2)
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIR, 1G, OP: 1 run", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.2,
                     act.rate = 2, rec.rate = 1 / 50, a.rate = 1 / 100,
                     ds.rate = 1 / 100, di.rate = 1 / 90, dr.rate = 1 / 100)
  init <- init.dcm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.dcm(type = "SIR", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[50, 1], 272.9788, tol = 0.0001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIR, 1G, OP: varying inf.prob", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.5, 0.05),
                     act.rate = 1, rec.rate = 1 / 50, a.rate = 1 / 100,
                     ds.rate = 1 / 100, di.rate = 1 / 90, dr.rate = 1 / 100)
  init <- init.dcm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.dcm(type = "SIR", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[50, 1], 26.55524, tol = 0.0001)
  expect_equal(x$epi$i.num[50, 5], 297.3653, tol = 0.0001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"), run = 2)
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIR, 2G, OP: 1 run", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.2, inf.prob.g2 = 0.1,
                     act.rate = 1, balance = "g1",
                     rec.rate = 1 / 50, rec.rate.g2 = 1 / 50,
                     a.rate = 1 / 100, a.rate.g2 = NA,
                     ds.rate = 1 / 100, ds.rate.g2 = 1 / 100,
                     di.rate = 1 / 90, di.rate.g2 = 1 / 90,
                     dr.rate = 1 / 100, dr.rate.g2 = 1 / 100)
  init <- init.dcm(s.num = 500, i.num = 1, r.num = 0,
                   s.num.g2 = 500, i.num.g2 = 1, r.num.g2 = 0)
  control <- control.dcm(type = "SIR", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[50, 1], 154.4498, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[50, 1], 114.8753, tol = 0.0001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIR, 2G, OP: varying inf.prob", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.5, 0.05), inf.prob.g2 = 0.1,
                     act.rate = 1, balance = "g1",
                     rec.rate = 1 / 50, rec.rate.g2 = 1 / 50,
                     a.rate = 1 / 100, a.rate.g2 = NA,
                     ds.rate = 1 / 100, ds.rate.g2 = 1 / 100,
                     di.rate = 1 / 90, di.rate.g2 = 1 / 90,
                     dr.rate = 1 / 100, dr.rate.g2 = 1 / 100)
  init <- init.dcm(s.num = 500, i.num = 1, r.num = 0,
                   s.num.g2 = 500, i.num.g2 = 1, r.num.g2 = 0)
  control <- control.dcm(type = "SIR", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[50, 1], 26.55524, tol = 0.0001)
  expect_equal(x$epi$i.num[50, 5], 298.6245, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[50, 5], 213.1567, tol = 0.0001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"), run = 2)
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})


# SIS Models --------------------------------------------------------------

test_that("SIS, 1G, CL: 1 run", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.2,
                     act.rate = 0.25, rec.rate = 1 / 50)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SIS", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[250, 1], 256.7584, tol = 0.0001)
  expect_equal(max(x$epi$s.num + x$epi$i.num), 501)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIS, 1G, CL: varying inf.prob", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.9, 0.1),
                     act.rate = 0.25, rec.rate = 1 / 50)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SIS", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[50, 1], 1.2741, tol = 0.0001)
  expect_equal(x$epi$i.num[50, 5], 122.0875, tol = 0.0001)
  expect_equal(x$epi$i.num[50, 9], 447.6173, tol = 0.0001)
  expect_equal(max(x$epi$s.num + x$epi$i.num), 501)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"), run = 2)
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIS, 2G, CL: 1 run", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.2,
                     act.rate = 0.25, inf.prob.g2 = 0.1, balance = "g1",
                     rec.rate = 1 / 100, rec.rate.g2 = 1 / 100)
  init <- init.dcm(s.num = 500, i.num = 1,
                   s.num.g2 = 500, i.num.g2 = 0)
  control <- control.dcm(type = "SIS", nsteps = 1000, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[250, 1], 164.0793, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[250, 1], 121.0725, tol = 0.0001)
  expect_equal(max(x$epi$s.num + x$epi$i.num +
                     x$epi$s.num.g2 + x$epi$i.num.g2), 1001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIS, 2G, CL: varying inf.prob", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.9, 0.1),
                     act.rate = 0.25, inf.prob.g2 = 0.1, balance = "g1",
                     rec.rate = 1 / 100, rec.rate.g2 = 1 / 100)
  init <- init.dcm(s.num = 500, i.num = 1,
                   s.num.g2 = 500, i.num.g2 = 0)
  control <- control.dcm(type = "SIS", nsteps = 1000, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[100, 1], 2.2167, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[100, 1], 2.1833, tol = 0.0001)
  expect_equal(x$epi$i.num[100, 9], 199.7184, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[100, 9], 74.7278, tol = 0.0001)
  expect_equal(max(x$epi$s.num + x$epi$i.num +
                     x$epi$s.num.g2 + x$epi$i.num.g2), 1001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"), run = 2)
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIS, 1G, OP: 1 run", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.2,
                     act.rate = 0.5, rec.rate = 1 / 50,
                     a.rate = 1 / 100, ds.rate = 1 / 100, di.rate = 1 / 90)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SIS", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[100, 1], 249.2884, tol = 0.0001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIS, 1G, OP: varying inf.prob", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.9, 0.1),
                     act.rate = 0.5, rec.rate = 1 / 50,
                     a.rate = 1 / 100, ds.rate = 1 / 100, di.rate = 1 / 90)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SIS", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[100, 1], 6.3054, tol = 0.0001)
  expect_equal(x$epi$i.num[100, 9], 428.3781, tol = 0.0001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"), run = 2)
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIS, 2G, OP: 1 run", {
  skip_on_cran()
  param <- param.dcm(inf.prob = 0.2,
                     inf.prob.g2 = 0.1, act.rate = 0.5,
                     balance = "g1", rec.rate = 1 / 50, rec.rate.g2 = 1 / 50,
                     a.rate = 1 / 100, a.rate.g2 = NA, ds.rate = 1 / 100,
                     ds.rate.g2 = 1 / 100, di.rate = 1 / 90,
                     di.rate.g2 = 1 / 90)
  init <- init.dcm(s.num = 500, i.num = 1,
                   s.num.g2 = 500, i.num.g2 = 1)
  control <- control.dcm(type = "SIS", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[150, 1], 182.0338, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[150, 1], 136.3023, tol = 0.0001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

test_that("SIS, 2G, OP: varying inf.prob", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.9, 0.1), inf.prob.g2 = 0.1,
                     act.rate = 0.5, balance = "g1",
                     rec.rate = 1 / 50, rec.rate.g2 = 1 / 50,
                     a.rate = 1 / 100, a.rate.g2 = NA,
                     ds.rate = 1 / 100, ds.rate.g2 = 1 / 100,
                     di.rate = 1 / 90, di.rate.g2 = 1 / 90)
  init <- init.dcm(s.num = 500, i.num = 1,
                   s.num.g2 = 500, i.num.g2 = 1)
  control <- control.dcm(type = "SIS", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_equal(x$epi$i.num[100, 1], 6.3054, tol = 0.0001)
  expect_equal(x$epi$i.num[100, 9], 424.9165, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[100, 1], 6.3054, tol = 0.0001)
  expect_equal(x$epi$i.num.g2[100, 9], 279.5831, tol = 0.0001)
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"), run = 2)
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})


# Other cases -------------------------------------------------------------

test_that("SIS, 2G, OP: balance = g2", {
  skip_on_cran()
  param <- param.dcm(inf.prob = seq(0.1, 0.9, 0.1), inf.prob.g2 = 0.1,
                     act.rate.g2 = 0.5, balance = "g2",
                     rec.rate = 1 / 50, rec.rate.g2 = 1 / 50,
                     a.rate = 1 / 100, a.rate.g2 = NA,
                     ds.rate = 1 / 100, ds.rate.g2 = 1 / 100,
                     di.rate = 1 / 90, di.rate.g2 = 1 / 90)
  init <- init.dcm(s.num = 5000, i.num = 1,
                   s.num.g2 = 500, i.num.g2 = 1)
  control <- control.dcm(type = "SIS", nsteps = 500, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = c("i.num", "s.num"))
  expect_output(summary(x, at = 2), regexp = "EpiModel Summary")
})

context("DCM Standard Models")

# SI Models ---------------------------------------------------------------

test_that("SI, 1G, CL: 1 run", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SI", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
})

test_that("SI, 1G, CL: varying inf.prob", {
  param <- param.dcm(inf.prob = seq(0.1, 0.9, 0.1),
                     act.rate = 0.25)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SI", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
})


test_that("SI, 1G, CL: varying inf.prob and act.rate", {
  param <- param.dcm(inf.prob = seq(0.1, 0.8, 0.1),
                     act.rate = seq(0.25, 2, 0.25))
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SI", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
})


test_that("SI, 1G, OP: varying inf.prob", {
  param <- param.dcm(inf.prob = seq(0.1, 0.5, 0.05),
                     act.rate = 0.25, a.rate = 1 / 100,
                     ds.rate = 1 / 100, di.rate = 1 / 90)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SI", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
})


test_that("SI, 2G, OP: varying inf.prob", {
  param <- param.dcm(inf.prob = seq(0.1, 0.5, 0.05),
                     act.rate = 0.25, inf.prob.g2 = 0.1, balance = "g1",
                     a.rate = 1 / 100, a.rate.g2 = NA, ds.rate = 1 / 100,
                     ds.rate.g2 = 1 / 100, di.rate = 1 / 90,
                     di.rate.g2 = 1 / 90)
  init <- init.dcm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 0)
  control <- control.dcm(type = "SI", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
})


# SIR Models --------------------------------------------------------------

test_that("SIR, 1G, CL: varying inf.prob", {
  param <- param.dcm(inf.prob = seq(0.1, 0.5, 0.05),
                     act.rate = 0.25, rec.rate = 1 / 50)
  init <- init.dcm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.dcm(type = "SIR", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
})

test_that("SIR, 1G, OP: 1 run", {
  param <- param.dcm(inf.prob = 0.2,
                     act.rate = 2, rec.rate = 1 / 50, a.rate = 1 / 100,
                     ds.rate = 1 / 100, di.rate = 1 / 90, dr.rate = 1 / 100)
  init <- init.dcm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.dcm(type = "SIR", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
})

test_that("SIR, 2G, OP: varying inf.prob", {
  param <- param.dcm(inf.prob = seq(0.1, 0.5, 0.05), inf.prob.g2 = 0.1,
                     act.rate = 1, balance = "g1",
                     rec.rate = 1 / 50, rec.rate.g2 = 1 / 50,
                     a.rate = 1 / 100, a.rate.g2 = NA,
                     ds.rate = 1 / 100, ds.rate.g2 = 1 / 100,
                     di.rate = 1 / 90, di.rate.g2 = 1 / 90,
                     dr.rate = 1 / 100, dr.rate.g2 = 1 / 100)
  init <- init.dcm(s.num = 500, i.num = 1, r.num = 0,
                   s.num.g2 = 500, i.num.g2 = 1, r.num.g2 = 0)
  control <- control.dcm(type = "SIR", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
})



# SIS Models --------------------------------------------------------------

test_that("SIS, 1G, CL: 1 run", {
  param <- param.dcm(inf.prob = 0.2,
                     act.rate = 0.25, rec.rate = 1 / 50)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SIS", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
})

test_that("SIS, 2G, CL: varying inf.prob", {
  param <- param.dcm(inf.prob = seq(0.1, 0.9, 0.1),
                     act.rate = 0.25, inf.prob.g2 = 0.1, balance = "g1",
                     rec.rate = 1 / 100, rec.rate.g2 = 1 / 100)
  init <- init.dcm(s.num = 500, i.num = 1,
                   s.num.g2 = 500, i.num.g2 = 0)
  control <- control.dcm(type = "SIS", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
})

test_that("SIS, 1G, OP: 1 run", {
  param <- param.dcm(inf.prob = 0.2,
                     act.rate = 0.5, rec.rate = 1 / 50,
                     a.rate = 1 / 100, ds.rate = 1 / 100, di.rate = 1 / 90)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SIS", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
})



# Check flows -------------------------------------------------------------

test_that("si.flow correct for closed SI model, lsoda method", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 3.4)
  init <- init.dcm(s.num = 28650, i.num = 100)
  control <- control.dcm(type = "SI", nsteps = 2)
  mod <- dcm(param, init, control)
  df <- as.data.frame(mod)
  expect_equal(df$i.num[2], df$i.num[1] + df$si.flow[1])
  expect_equal(df$s.num[2], df$s.num[1] - df$si.flow[1])
  expect_equal(df$si.flow[1], 96.72141, tol = 0.0001)
})


test_that("si.flow correct for open SI model, lsoda method", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 3.4,
                     a.rate = 0.02, ds.rate = 0.01, di.rate = 0.01)
  init <- init.dcm(s.num = 28650, i.num = 100)
  control <- control.dcm(type = "SI", nsteps = 2)
  mod <- dcm(param, init, control)
  df <- as.data.frame(mod)
  expect_equal(df$num[2],
               df$num[1] + df$a.flow[1] - df$ds.flow[1] - df$di.flow[1])
  expect_equal(df$i.num[2], df$i.num[1] + df$si.flow[1] - df$di.flow[1])
  expect_equal(df$si.flow[1], 96.19311, tol = 0.0001)
})

test_that("si.flow correct for closed SI model, Euler method", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 3.4)
  init <- init.dcm(s.num = 28650, i.num = 100)
  control <- control.dcm(type = "SI", nsteps = 2, odemethod = "euler")
  mod <- dcm(param, init, control)
  df <- as.data.frame(mod)
  expect_equal(df$i.num[2], df$i.num[1] + df$si.flow[1])
  expect_equal(df$s.num[2], df$s.num[1] - df$si.flow[1])
  expect_equal(df$si.flow[1], 67.76348, tol = 0.0001)
})


test_that("si.flow correct for closed SI model, Euler method", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 3.4,
                     a.rate = 0.02, ds.rate = 0.01, di.rate = 0.01)
  init <- init.dcm(s.num = 28650, i.num = 100)
  control <- control.dcm(type = "SI", nsteps = 2, odemethod = "euler")
  mod <- dcm(param, init, control)
  df <- as.data.frame(mod)
  expect_equal(df$num[2],
               df$num[1] + df$a.flow[1] - df$ds.flow[1] - df$di.flow[1])
  expect_equal(df$i.num[2], df$i.num[1] + df$si.flow[1] - df$di.flow[1])
  expect_equal(df$si.flow[1], 67.76348, tol = 0.0001)
})


# Check dt fractional -----------------------------------------------------

test_that("fractional dt returns, RK4 method", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 3.4)
  init <- init.dcm(s.num = 28650, i.num = 100)
  control <- control.dcm(type = "SI", nsteps = 10, dt = 0.5)
  mod <- dcm(param, init, control)
  df <- as.data.frame(mod)
  expect_equal(df$time, seq(1, 10, 0.5))
})

test_that("fractional dt returns, Euler method", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 3.4)
  init <- init.dcm(s.num = 28650, i.num = 100)
  control <- control.dcm(type = "SI", nsteps = 10, dt = 0.5,
                         odemethod = "euler")
  mod <- dcm(param, init, control)
  df <- as.data.frame(mod)
  expect_equal(df$time, seq(1, 10, 0.5))
})

test_that("test extension model, NA output", {
  eMod2 <- function(t, t0, parms) {
    with(as.list(c(t0, parms)), {

      # Total population size
      num <- s.num + i.num + r.num

      # Define lambdas
      lambda.direct <- beta.direct * i.num
      lambda.envirn <- beta.envirn * e.num

      # Differential equations
      dS <- -lambda.direct*s.num - lambda.envirn*s.num + mu*num - mu*s.num
      dI <- lambda.direct*s.num + lambda.envirn*s.num - mu*i.num - gamma*i.num
      dR <- gamma*i.num - mu*r.num
      dE <- xi*i.num - delta*e.num

      # Outputs
      list(c(dS, dI, dR, dE,
             si.direct.flow = lambda.direct*s.num,
             si.envirn.flow = lambda.envirn*s.num))
    })
  }

  # Parameterize the model with the following parameters this time
  param <- param.dcm(beta.direct = 0.0001,
                     beta.envirn = 0.00001,
                     mu = 0.01,
                     gamma = 0.2,
                     xi = 2,
                     delta = 0.5)

  # Use similar initial conditions and control settings as the first model, but
  # make sure to update the new.mod function name!
  init <- init.dcm(s.num = 10000, i.num = 1, r.num = 0, e.num = 0,
                   si.direct.flow = 0, si.envirn.flow = 0)
  control <- control.dcm(nsteps = 10, dt = 0.1, new.mod = eMod2)

  # Run the simulation and examine the output as a data frame
  sim <- dcm(param, init, control)
  expect_s3_class(sim, "dcm")

  df <- as.data.frame(sim)
  expect_true(all(!is.na(df)))
})


# DCM interventions -------------------------------------------------------

test_that("DCM interventions, SI model", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 3.4,
                     inter.eff = 1)
  expect_equal(param$inter.start, 1)

  param <- param.dcm(inf.prob = 0.2, act.rate = 3.4,
                     inter.eff = 1, inter.start = 5)
  expect_equal(param$inter.start, 5)

  init <- init.dcm(s.num = 28650, i.num = 100)
  control <- control.dcm(type = "SI", nsteps = 10)
  mod <- dcm(param, init, control)

  df <- as.data.frame(mod)
  expect_equal(sum(df$si.flow[5:10], na.rm = TRUE), 0, tol = 0.01)
  expect_equal(max(df$i.num[5:10]) - min(df$i.num[5:10]), 0, tol = 0.01)
  expect_equal(max(df$s.num[5:10]) - min(df$s.num[5:10]), 0, tol = 0.01)
})

test_that("DCM interventions, SIS model", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 3.4, rec.rate = 0.1,
                     inter.eff = 1, inter.start = 5)
  expect_equal(param$inter.start, 5)
  init <- init.dcm(s.num = 28650, i.num = 100)
  control <- control.dcm(type = "SIS", nsteps = 10)
  mod <- dcm(param, init, control)
  df <- as.data.frame(mod)
  expect_equal(sum(df$si.flow[5:10], na.rm = TRUE), 0, tol = 0.01)
  expect_equal(df$s.num[10], 28155.07472, tol = 0.0001)
  expect_equal(df$i.num[10], 594.92528, tol = 0.0001)
})

test_that("DCM interventions, SIR model", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 3.4, rec.rate = 0.1,
                     inter.eff = 1, inter.start = 5)
  expect_equal(param$inter.start, 5)
  init <- init.dcm(s.num = 28650, i.num = 100, r.num = 0)
  control <- control.dcm(type = "SIR", nsteps = 10)
  mod <- dcm(param, init, control)
  df <- as.data.frame(mod)
  expect_equal(sum(df$si.flow[5:10], na.rm = TRUE), 0, tol = 0.01)
  expect_equal(max(df$s.num[5:10]) - min(df$s.num[5:10]), 0, tol = 0.01)
  expect_equal(df$i.num[10], 592.14131, tol = 0.5)
})



# param, init, control ------------------------------------------------------

test_that("control checks", {
  control <- control.dcm(type = "SI", nsteps = 10, foo = "boo")
  expect_true(control$foo == "boo")
  expect_error(control.dcm(type = "SI"), "Specify nsteps")
  expect_error(control.dcm(type = "SEIR", nsteps = 10), "Specify type as")
})



# dcm extension check -----------------------------------------------------


context("DCM Delayed Differential Equation Models")

test_that("Delayed differntial equation models function", {

  lagsi <- function(t, t0, parms) {
    with(as.list(c(t0, parms)), {

      num <- s.num + i.num

      if (t < 2) {
        i.num.foi <- 0
      } else {
        i.num.foi <- lagvalue(t - 1, 2)
      }

      lambda <- inf.prob * act.rate * i.num.foi / num

      dS <- -lambda * s.num
      dI <- lambda * s.num

      list(c(dS, dI),
           num = num,
           si.flow = lambda * s.num,
           i.num.foi = i.num.foi)
    })
  }

  param <- param.dcm(inf.prob = 0.5, act.rate = 0.25)
  init <- init.dcm(s.num = 100, i.num = 10)
  control <- control.dcm(nsteps = 25, new.mod = lagsi, dede = TRUE)
  mod <- dcm(param, init, control)

  expect_is(mod, "dcm")
  expect_true(mod$control$dede)

})


context("New DCM Models")


test_that("New DCMs Example 1", {

  ## SI function
  intSI <- function(t, t0, parms) {
    with(as.list(c(t0, parms)), {

      ## Dynamic Calculations
      # Population size
      num <- s.num + i.num

      # Intervention start time: if time > start,
      #   then multiply lambda by relative hazard
      if (t < start.time) {
        lambda <- inf.prob * act.rate * i.num / num
      } else {
        lambda <- (inf.prob * act.rate * i.num / num) * rel.haz
      }

      ## Flows
      si.flow <- lambda * s.num

      ## Differential Equations
      dS <- -lambda * s.num
      dI <- lambda * s.num

      ## Output
      list(c(dS, dI, si.flow),
           num = num)
    })
  }


  init <- init.dcm(s.num = 999, i.num = 1, si.flow = 0)
  control <- control.dcm(nsteps = 250, dt = 1, new.mod = intSI, verbose = FALSE)
  param1 <- param.dcm(inf.prob = 0.5, act.rate = 0.1,
                      start.time = seq(100, 200, 100), rel.haz = 1)
  param05 <- param.dcm(inf.prob = 0.5, act.rate = 0.1,
                       start.time = seq(100, 200, 100), rel.haz = 0.5)
  mod1 <- dcm(param1, init, control)
  mod2 <- dcm(param05, init, control)

  expect_is(mod1, "dcm")
  expect_is(mod2, "dcm")
})


test_that("New DCMs Example 2", {

  ## Q Mod Function
  Qmod <- function(t, t0, parms) {
    with(as.list(c(t0, parms)), {

      ## Dynamic Calculations ##

      # Popsize and prevalence
      h.num <- sh.num + ih.num
      l.num <- sl.num + il.num
      num <- h.num + l.num
      prev <- (ih.num + il.num) / num

      # Contact rates for high specified as a function of
      #   mean and low rates
      c.high <- (c.mean * num - c.low * l.num) / h.num

      # Mixing matrix calculations based on variable Q statistic
      g.hh <- ((c.high * h.num) + (Q * c.low * l.num)) /
        ((c.high * h.num) + (c.low * l.num))
      g.lh <- 1 - g.hh
      g.hl <- (1 - g.hh) * ((c.high * h.num) / (c.low * l.num))
      g.ll <- 1 - g.hl

      # Probability that contact is infected based on mixing probabilities
      p.high <- (g.hh * ih.num / h.num) + (g.lh * il.num / l.num)
      p.low <- (g.ll * il.num / l.num) + (g.hl * ih.num / h.num)

      # Force of infection for high and low groups
      lambda.high <- rho * c.high * p.high
      lambda.low <- rho * c.low * p.low


      ## Differential Equations ##
      dS.high <- -lambda.high * sh.num + nu * ih.num
      dI.high <- lambda.high * sh.num - nu * ih.num

      dS.low <- -lambda.low * sl.num + nu * il.num
      dI.low <- lambda.low * sl.num - nu * il.num


      ## Output ##
      list(c(dS.high, dI.high,
             dS.low, dI.low),
           num = num,
           prev = prev)
    })
  }


  param <- param.dcm(c.mean = 2,
                     c.low = 1.4,
                     rho = 0.75,
                     nu = 6,
                     Q = c(-0.45, 0.5, 1))
  init <- init.dcm(sh.num = 2e7 * 0.02,
                   ih.num = 1,
                   sl.num = 2e7 * 0.98,
                   il.num = 1)
  control <- control.dcm(nsteps = 6, dt = 0.02, new.mod = Qmod, verbose = FALSE)

  mod <- dcm(param, init, control)
  expect_is(mod, "dcm")

})


test_that("DCM inital conditions ordering correct", {

  SEIR <- function(t, t0, parms) {
    with(as.list(c(t0, parms)), {
      num <- s.num + e.num + i.num + r.num
      lambda <- inf.prob * act.rate * i.num / num
      se.flow <-  lambda * s.num
      ei.flow <-  sx.rate * e.num
      ir.flow <-  rec.rate * i.num
      dS <- -lambda * s.num
      dE <- lambda * s.num - sx.rate * e.num
      dI <- sx.rate * e.num - rec.rate * i.num
      dR <- rec.rate * i.num
      list(c(dS, dE, dI, dR, se.flow, ei.flow, ir.flow),
           num = num)
    })
  }

  init <- init.dcm(s.num = 980, e.num = 10, i.num = 10, r.num = 0,
                   se.flow = 0, ei.flow = 0, ir.flow = 0)
  expect_identical(names(init), c("s.num", "e.num", "i.num", "r.num",
                                  "se.flow", "ei.flow", "ir.flow"))

  param <- param.dcm(inf.prob = 0.2, act.rate = 0.5, sx.rate = 0.1,
                     rec.rate = 0.05)
  control <- control.dcm(nsteps = 10, dt = 1, new.mod = SEIR)

  mod <- dcm(param, init, control)
  expect_is(mod, "dcm")
  expect_identical(names(as.data.frame(mod))[3], "e.num")
})


test_that("Non-sensitivity parameter vector", {

  ## SI function
  intSI <- function(t, t0, parms) {
    with(as.list(c(t0, parms)), {

      ## Dynamic Calculations
      # Population size
      num <- s.num + i.num

      if (t < start.time) {
        lambda <- inf.prob[1] * act.rate * i.num / num
      } else {
        lambda <- inf.prob[2] * act.rate * i.num / num
      }

      ## Flows
      si.flow <- lambda * s.num

      ## Differential Equations
      dS <- -si.flow
      dI <- si.flow

      ## Output
      list(c(dS, dI, si.flow),
           num = num)
    })
  }

  param <- param.dcm(inf.prob = c(0.5, 0.05), act.rate = 0.1, start.time = 100)
  init <- init.dcm(s.num = 999, i.num = 1, si.flow = 0)
  control <- control.dcm(nsteps = 250, new.mod = intSI, sens.param = FALSE)
  mod <- dcm(param, init, control)
  expect_is(mod, "dcm")
  expect_true(any(!is.na(as.data.frame(mod))))
})


# Init sensitivity analysis ------------------------------------------------

test_that("SI, 1G, CL: varying i.num in init", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.dcm(s.num = 500, i.num = c(1, 5, 10))
  control <- control.dcm(type = "SI", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
  expect_equal(x$control$nruns, 3)
  expect_equal(x$epi$i.num[1, 1], 1)
  expect_equal(x$epi$i.num[1, 2], 5)
  expect_equal(x$epi$i.num[1, 3], 10)
})

test_that("SIR, 1G, CL: varying init and param together", {
  param <- param.dcm(inf.prob = c(0.1, 0.2, 0.3),
                     act.rate = 0.25, rec.rate = 1 / 50)
  init <- init.dcm(s.num = c(500, 400, 300), i.num = c(1, 5, 10), r.num = 0)
  control <- control.dcm(type = "SIR", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
  expect_equal(x$control$nruns, 3)
  expect_equal(x$epi$s.num[1, 1], 500)
  expect_equal(x$epi$s.num[1, 3], 300)
})

test_that("DCM init sensitivity: mismatched init vector lengths error", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.dcm(s.num = c(500, 400), i.num = c(1, 5, 10))
  control <- control.dcm(type = "SI", nsteps = 10, verbose = FALSE)
  expect_error(dcm(param, init, control), "same length")
})

test_that("DCM init sensitivity: mismatched param and init vector lengths error", {
  param <- param.dcm(inf.prob = c(0.1, 0.2), act.rate = 0.25)
  init <- init.dcm(s.num = 500, i.num = c(1, 5, 10))
  control <- control.dcm(type = "SI", nsteps = 10, verbose = FALSE)
  expect_error(dcm(param, init, control), "same length")
})

test_that("DCM init sensitivity: as.data.frame works", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.dcm(s.num = 500, i.num = c(1, 5, 10))
  control <- control.dcm(type = "SI", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  df <- as.data.frame(x)
  expect_true("run" %in% names(df))
  expect_equal(length(unique(df$run)), 3)
})

test_that("DCM init sensitivity: custom model with varying init", {
  my_mod <- function(t, t0, parms) {
    with(as.list(c(t0, parms)), {
      num <- s.num + i.num
      lambda <- inf.prob * act.rate * i.num / num
      dS <- -lambda * s.num
      dI <- lambda * s.num
      list(c(dS, dI))
    })
  }
  param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.dcm(s.num = 500, i.num = c(1, 5, 10))
  control <- control.dcm(nsteps = 10, new.mod = my_mod, verbose = FALSE)
  x <- dcm(param, init, control)
  expect_is(x, "dcm")
  expect_equal(x$control$nruns, 3)
})

test_that("DCM init sensitivity: print method shows init", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.dcm(s.num = 500, i.num = c(1, 5, 10))
  control <- control.dcm(type = "SI", nsteps = 10, verbose = FALSE)
  x <- dcm(param, init, control)
  out <- capture.output(print(x))
  expect_true(any(grepl("Initial Conditions", out)))
  expect_true(any(grepl("i.num", out)))
})

context("Print Functions")

test_that("print.dcm", {
  param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.dcm(s.num = 500, i.num = 1)
  control <- control.dcm(type = "SI", nsteps = 500)
  mod <- dcm(param, init, control)
  expect_output(print(mod), "EpiModel Simulation")
  expect_output(print(mod), "Model class: dcm")
  expect_output(print(mod), "Model type: SI")
  expect_output(print(mod), "inf.prob = 0.2")
  expect_output(print(mod), "act.rate = 0.25")
  expect_output(print(mod), "Variables: s.num i.num si.flow num")
})

test_that("print.icm", {
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SI", nsteps = 10, nsims = 2, verbose = FALSE)
  mod <- icm(param, init, control)
  expect_output(print(mod), "EpiModel Simulation")
  expect_output(print(mod), "Model class: icm")
  expect_output(print(mod), "Model type: SI")
  expect_output(print(mod), "inf.prob = 0.2")
  expect_output(print(mod), "act.rate = 0.25")
  expect_output(print(mod), "Variables: s.num i.num num si.flow")
})

test_that("print.netest", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)
  est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
  expect_output(print(est), "EpiModel Network Estimation")
  expect_output(print(est), "Model class: netest")
  expect_output(print(est), "Estimation Method: ERGM with Edges Approximation")
  expect_output(print(est), "Formation: ~edges")
  expect_output(print(est), "Target Statistics: 50")
  expect_output(print(est), "Target Statistics: 10")
})

test_that("print.netsim", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  nw <- set_vertex_attribute(nw, "group", rep(c(1, 2), each = 50))
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
  est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
  param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
  init <- init.net(i.num = 10, i.num.g2 = 10)
  control <- control.net(type = "SI", nsteps = 10, nsims = 1, verbose = FALSE)
  mod <- netsim(est, param, init, control)
  expect_output(print(mod), "EpiModel Simulation")
  expect_output(print(mod), "Model class: netsim")
  expect_output(print(mod), "Model type: SI")
  expect_output(print(mod), "No. NW groups: 2")
  expect_output(print(mod), "Variables: s.num i.num num s.num.g2")
})

test_that("print.disscoefs", {
  o <- dissolution_coefs(dissolution = ~offset(edges), duration = 25)
  expect_output(print(o), "Dissolution Coefficients")
  expect_output(print(o), "Crude Coefficient: 3.178054")
  expect_output(print(o), "Mortality/Exit Rate: 0")

  o <- dissolution_coefs(dissolution = ~offset(edges), duration = 25,
                         d.rate = 0.001)
  expect_is(o, "disscoef")
  expect_output(print(o), "Mortality/Exit Rate: 0.001")

  o <- dissolution_coefs(dissolution =
                           ~offset(edges) + offset(nodematch("race")),
                         duration = c(20, 10))
  expect_output(print(o), "2.944439 -0.7472144")
  expect_output(print(o), "Mortality/Exit Rate: 0")
  expect_output(print(o), "Target Statistics: 20 10")

  o <- dissolution_coefs(dissolution =
                           ~offset(edges) + offset(nodematch("race")),
                         duration = c(20, 10), d.rate = 0.001)
  expect_output(print(o), "Crude Coefficient: 2.944439 -0.7472144")
  expect_output(print(o), "Adjusted Coefficient: 2.98524 -0.7678231")
})

test_that("print.param", {
  p <- param.dcm(inf.prob = 0.1, rec.rate = 0.1)
  expect_output(print(p), "DCM Parameters")

  p <- param.icm(inf.prob = 0.1, rec.rate = 0.1)
  expect_output(print(p), "ICM Parameters")

  p <- param.net(inf.prob = 0.1, rec.rate = 0.1)
  expect_output(print(p), "Fixed Parameters")
  expect_output(print(p), "act.rate = 1")
})

test_that("print.init", {
  i <- init.dcm(s.num = 10, i.num = 10)
  expect_output(print(i), "DCM Initial Conditions")

  i <- init.icm(s.num = 10, i.num = 10)
  expect_output(print(i), "ICM Initial Conditions")

  i <- init.net(s.num = 10, i.num = 10)
  expect_output(print(i), "Network Model Initial Conditions")
})

test_that("print.control", {
  co <- control.dcm(type = "SI", nsteps = 10)
  expect_output(print(co), "DCM Control Settings")
  expect_output(print(co), "odemethod = lsoda")

  co <- control.icm(type = "SI", nsteps = 10)
  expect_output(print(co), "ICM Control Settings")

  co <- control.net(type = "SI", nsteps = 10)
  expect_output(print(co), "Network Model Control Settings")
  #FLAG 4/23
  #expect_output(print(co), "Base Modules: initialize.FUN")
})

#' @title Merge Data across Stochastic Individual Contact Model Simulations
#'
#' @description Merges epidemiological data from two independent simulations of
#'              stochastic individual contact models from [icm()].
#'
#' @param x An `EpiModel` object of class [icm()].
#' @param y Another `EpiModel` object of class [icm()], with the
#'        identical model parameterization as `x`.
#' @param ...  Additional merge arguments (not used).
#'
#' @details
#' This merge function combines the results of two independent simulations of
#' [icm()] class models, simulated under separate function calls. The
#' model parameterization between the two calls must be exactly the same, except
#' for the number of simulations in each call. This allows for manual
#' parallelization of model simulations.
#'
#' This merge function does not work the same as the default merge, which allows
#' for a combined object where the structure differs between the input elements.
#' Instead, the function checks that objects are identical in model
#' parameterization in every respect (except number of simulations) and binds
#' the results.
#'
#' @return An `EpiModel` object of class [icm()] containing the
#'         data from both `x` and `y`.
#'
#' @method merge icm
#' @keywords extract
#' @export
#'
#' @examples
#' param <- param.icm(inf.prob = 0.2, act.rate = 0.8)
#' init <- init.icm(s.num = 1000, i.num = 100)
#' control <- control.icm(type = "SI", nsteps = 10,
#'                        nsims = 3, verbose = FALSE)
#' x <- icm(param, init, control)
#'
#' control <- control.icm(type = "SI", nsteps = 10,
#'                        nsims = 1, verbose = FALSE)
#' y <- icm(param, init, control)
#'
#' z <- merge(x, y)
#'
#' # Examine separate and merged data
#' as.data.frame(x)
#' as.data.frame(y)
#' as.data.frame(z)
#'
merge.icm <- function(x, y, ...) {

  ## Check structure
  if (length(x) != length(y) || !identical(names(x), names(y))) {
    stop("x and y have different structure")
  }
  if (x$control$nsims > 1 && y$control$nsims > 1 &&
        !identical(sapply(x, class), sapply(y, class))) {
    stop("x and y have different structure")
  }

  ## Check params
  check1 <- identical(x$param, y$param)
  check2 <- identical(x$control[-which(names(x$control) == "nsims")],
                      y$control[-which(names(y$control) == "nsims")])

  if (check1 == FALSE) {
    stop("x and y have different parameters")
  }
  if (check2 == FALSE) {
    stop("x and y have different controls")
  }


  z <- x
  new.range <- (x$control$nsims + 1):(x$control$nsims + y$control$nsims)

  # Merge data
  for (i in seq_along(x$epi)) {
    if (x$control$nsims == 1) {
      x$epi[[i]] <- data.frame(x$epi[[i]])
    }
    if (y$control$nsims == 1) {
      y$epi[[i]] <- data.frame(y$epi[[i]])
    }
    z$epi[[i]] <- cbind(x$epi[[i]], y$epi[[i]])
    names(z$epi[[i]])[new.range] <- paste0("sim", new.range)
  }

  z$control$nsims <- max(new.range)

  return(z)
}


#' @title Merge Model Simulations across netsim Objects
#'
#' @description Merges epidemiological data from two independent simulations of
#'              stochastic network models from `netsim`.
#'
#' @param x An `EpiModel` object of class [netsim()].
#' @param y Another `EpiModel` object of class [netsim()],
#'        with the identical model parameterization as `x`.
#' @param keep.transmat If `TRUE`, keep the transmission matrices from the
#'        original `x` and `y` elements. Note: transmission matrices
#'        only saved when (`save.transmat == TRUE`).
#' @param keep.network If `TRUE`, keep the `networkDynamic` objects
#'        from the original `x` and `y` elements. Note: network
#'        only saved when (`tergmLite == FALSE`).
#' @param keep.nwstats If `TRUE`, keep the network statistics (as set by
#'        the `nwstats.formula` parameter in `control.net`) from
#'        the original `x` and `y` elements.
#' @param keep.other If `TRUE`, keep the other simulation elements (as set
#'        by the `save.other` parameter in `control.net`) from the
#'        original `x` and `y` elements.
#' @param param.error If `TRUE`, if `x` and `y` have different
#'        params (in [param.net()]) or controls (passed in
#'        [control.net()]) an error will prevent the merge. Use
#'        `FALSE` to override that check.
#' @param keep.diss.stats If `TRUE`, keep `diss.stats` from the
#'        original `x` and `y` objects.
#' @param ...  Additional merge arguments (not currently used).
#'
#' @details
#' This merge function combines the results of two independent simulations of
#' [netsim()] class models, simulated under separate function calls.
#' The model parameterization between the two calls must be exactly the same,
#' except for the number of simulations in each call. This allows for manual
#' parallelization of model simulations.
#'
#' This merge function does not work the same as the default merge, which allows
#' for a combined object where the structure differs between the input elements.
#' Instead, the function checks that objects are identical in model
#' parameterization in every respect (except number of simulations) and binds
#' the results.
#'
#' @return An `EpiModel` object of class [netsim()] containing
#'         the data from both `x` and `y`.
#'
#' @method merge netsim
#' @keywords extract
#' @export
#'
#' @examples
#' \donttest{
#' # Network model
#' nw <- network_initialize(n = 100)
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)
#' est <- netest(nw, formation = ~edges, target.stats = 25,
#'               coef.diss = coef.diss, verbose = FALSE)
#'
#' # Epidemic models
#' param <- param.net(inf.prob = 1)
#' init <- init.net(i.num = 1)
#' control <- control.net(type = "SI", nsteps = 20, nsims = 2,
#'                        save.nwstats = TRUE,
#'                        nwstats.formula = ~edges + degree(0),
#'                        verbose = FALSE)
#' x <- netsim(est, param, init, control)
#' y <- netsim(est, param, init, control)
#'
#' # Merging
#' z <- merge(x, y)
#'
#' # Examine separate and merged data
#' as.data.frame(x)
#' as.data.frame(y)
#' as.data.frame(z)
#' }
#'
merge.netsim <- function(x, y, keep.transmat = TRUE, keep.network = TRUE,
                         keep.nwstats = TRUE, keep.other = TRUE,
                         param.error = TRUE, keep.diss.stats = TRUE, ...) {

  ## Check structure
  if (length(x) != length(y) || !identical(names(x), names(y))) {
    stop("x and y have different structure")
  }
  x$control$nsims <- as.integer(x$control$nsims)
  y$control$nsims <- as.integer(y$control$nsims)
  if (x$control$nsims > 1 && y$control$nsims > 1 &&
        !all(sapply(x, function(i) class(i)[1]) ==
               sapply(y, function(i) class(i)[1]))) {
    stop("x and y have different structure")
  }

  ## Check params
  check1 <- identical(param_without_random_values(x$param),
                      param_without_random_values(y$param))
  check2 <- identical(x$control[-which(names(x$control) %in%
                                         c("nsims", "monitors",
                                           "nwstats.formula"))],
                      y$control[-which(names(y$control) %in%
                                         c("nsims", "monitors",
                                           "nwstats.formula"))])

  ## handle formulas separately due to environments
  check2 <- check2 &&
    isTRUE(all.equal(x$control$monitors, y$control$monitors)) &&
    isTRUE(all.equal(x$control$nwstats.formula, y$control$nwstats.formula))

  if (check1 == FALSE && param.error == TRUE) {
    stop("x and y have different parameters")
  }
  if (check2 == FALSE && param.error == TRUE) {
    stop("x and y have different controls")
  }

  z <- x
  z$control$nsims <- x$control$nsims + y$control$nsims
  newnames <- paste0("sim", seq_len(z$control$nsims))
  z$param$random.params.values <- merge_random_params_values(
    x$param$random.params.values, y$param$random.params.values,
    x$control$nsims, y$control$nsims
  )

  # Merge epi data
  for (i in seq_along(x$epi)) {
    z$epi[[i]] <- cbind(x$epi[[i]], y$epi[[i]])
    names(z$epi[[i]]) <- newnames
  }

  ## Transmission matrix
  if (keep.transmat == TRUE && !is.null(x$stats$transmat) &&
        !is.null(y$stats$transmat)) {
    z$stats$transmat <- c(x$stats$transmat, y$stats$transmat)
    names(z$stats$transmat) <- newnames
  } else {
    z$stats$transmat <- NULL
  }

  ## Network objects
  if (keep.network == TRUE && !is.null(x$network) && !is.null(y$network)) {
    z$network <- c(x$network, y$network)
    names(z$network) <- newnames

  } else {
    z$network <- NULL
  }

  ## Network statistics
  if (keep.nwstats == TRUE && !is.null(x$stats$nwstats) && !is.null(y$stats$nwstats)) {
    z$stats$nwstats <- c(x$stats$nwstats, y$stats$nwstats)
    names(z$stats$nwstats) <- newnames
  } else {
    z$stats$nwstats <- NULL
  }

  ## Other
  if (!is.null(x$control$save.other) && !is.null(y$control$save.other)) {
    other.x <- x$control$save.other
    other.y <- y$control$save.other
    if (keep.other == TRUE) {
      if (!identical(other.x, other.y)) {
        stop("Elements in save.other differ between x and y")
      }
      new.range <- (x$control$nsims + 1):(x$control$nsims + y$control$nsims)
      for (j in seq_along(other.x)) {
        for (i in new.range) {
          z[[other.x[j]]][[i]] <- y[[other.x[j]]][[i - x$control$nsims]]
        }
        if (!is.null(z[[other.x[j]]])) {
          names(z[[other.x[j]]]) <- newnames
        }
      }
    } else {
      for (j in seq_along(other.x)) {
        z[[other.x[j]]] <- NULL
      }
    }
  }

  if (keep.diss.stats == TRUE && !is.null(x$diss.stats) &&
        !is.null(y$diss.stats)) {
    z$diss.stats <- c(x$diss.stats, y$diss.stats)
    names(z$diss.stats) <- newnames
  } else {
    z$diss.stats <- NULL
  }

  return(z)
}

#' @title Function to Trigger the End Horizon
#'
#' @inheritParams recovery.net
#' @inherit recovery.net return
#'
#' @details
#' This function triggers the end horizon if a control `end.horizon` exists and
#' its `at` value is equal to the current timestep. The end horizon consists on
#' the removal of a set of modules from the module list.
#'
#' @keywords internal
trigger_end_horizon <- function(dat) {
  end_horizon <- get_control(dat, "end.horizon", override.null.error = TRUE)
  if (is.null(end_horizon) || end_horizon$at != get_current_timestep(dat)) {
    return(dat)
  }

  dat <- remove_modules(dat, end_horizon$modules)
  dat <- set_control(dat, "end.horizon", NULL)

  return(dat)
}

#' Check the format of the end.horizon control
#'
#' @inheritParams recovery.net
#'
#' @return TRUE invisibly on succes, errors otherwise
#'
#' @keywords internal
check_end_horizon_control <- function(dat) {
  end_horizon <- get_control(dat, "end.horizon", override.null.error = TRUE)
  if (is.null(end_horizon))
    invisible(return(TRUE))

  start <- get_control(dat, "start")
  if (!is.numeric(end_horizon$at) || end_horizon$at <= start) {
    stop(
      "The control `end.horizon` `at` field must be greater than the starting",
      " time step: ", start, "\n"
    )
  }

  missing_mods <- setdiff(end_horizon$modules, names(get_modules(dat)))
  if (length(missing_mods) > 0) {
    stop(
      "The control `end.horizon` `modules` field contains modules names not",
      " present in the module list:\n'",
      paste0(missing_mods, collapse = "', '"), "'\n"
    )
  }

  return(invisible(TRUE))
}

#' @title EpiModel Web
#'
#' @description Launches interactive Shiny applications for exploring
#'              deterministic compartmental models (DCMs) and stochastic
#'              individual contact models (ICMs) of infectious disease
#'              transmission.
#'
#' @param class Model class, with options of `"dcm"` and `"icm"`.
#' @param ... Additional arguments passed to [shiny::runApp], such as
#'        `port`, `host`, or `launch.browser`.
#'
#' @details
#' `epiweb` launches a web-based graphical interface for configuring, running,
#' and analyzing epidemic models built on EpiModel. Each application provides
#' interactive controls for model parameters, real-time (or on-demand)
#' visualization with plotly, narrative model summaries, and downloadable
#' data tables. Both applications include a **Guide** tab with a comprehensive
#' user manual covering the underlying theory, parameter definitions, and
#' instructions for interpreting output.
#'
#' @section DCM Application (`class = "dcm"`):
#' The DCM app solves systems of ordinary differential equations for one-group
#' SI, SIR, and SIS models. Models run instantly and the interface is fully
#' reactive (no run button). Features include:
#' \itemize{
#'   \item **Scenario presets:** Flu-like (SIR), STI-like (SIS), and
#'     Measles-like (SIR) configurations that set all parameters to
#'     epidemiologically plausible values.
#'   \item **Interventions:** An optional mid-epidemic intervention that
#'     reduces the transmission probability by a user-specified efficacy,
#'     shown as a vertical dashed line on the plot.
#'   \item **Sensitivity analysis:** Vary any epidemic parameter across a
#'     range of values to compare trajectories side by side with a
#'     blue-to-red color ramp.
#'   \item **Vital dynamics:** Optional births and deaths to model endemic
#'     equilibria over longer time horizons.
#'   \item **Summary tab:** Displays the basic reproduction number
#'     (R\ifelse{html}{\out{<sub>0</sub>}}{0}), peak timing, cumulative
#'     infections, attack rate, and contextual interpretation.
#'   \item **Data tab:** Searchable table of model output with CSV download.
#' }
#'
#' @section ICM Application (`class = "icm"`):
#' The ICM app runs agent-based stochastic microsimulations for one-group
#' SI, SIR, and SIS models. Because ICMs are computationally more expensive,
#' a **Run Model** button controls when simulations execute. Features include:
#' \itemize{
#'   \item **Stochastic visualization:** Multiple simulations are summarized
#'     with mean trajectory lines and interquartile range (IQR) ribbons
#'     showing the middle 50\% of outcomes. Single simulations display
#'     individual trajectories.
#'   \item **Scenario presets:** The same Flu-like, STI-like, and Measles-like
#'     configurations as the DCM app, adapted for smaller populations suitable
#'     for agent-based simulation.
#'   \item **Interventions:** Same intervention system as DCM, natively
#'     supported by [param.icm].
#'   \item **Vital dynamics:** Optional births and deaths, as in DCM.
#'   \item **Summary tab:** Displays R\ifelse{html}{\out{<sub>0</sub>}}{0},
#'     epidemic timeline from the mean trajectory, and a stochastic variation
#'     section showing the range and standard deviation of peak infections and
#'     final prevalence across simulations.
#'   \item **Data tab:** View simulation output as means, standard deviations,
#'     or individual simulation values, with CSV download.
#' }
#'
#' @section Required Packages:
#' Both applications require the `shiny`, `bslib`, `DT`, and `plotly` packages.
#' Install any missing packages with
#' `install.packages(c("shiny", "bslib", "DT", "plotly"))`.
#'
#' @seealso
#' [dcm] and [param.dcm] for the DCM modeling API;
#' [icm] and [param.icm] for the ICM modeling API.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' ## Deterministic compartmental models
#' epiweb(class = "dcm")
#'
#' ## Stochastic individual contact models
#' epiweb(class = "icm")
#' }
#'
epiweb <- function(class, ...) {
  if (class == "dcm") {
    if (!requireNamespace("shiny", quietly = TRUE)) {
      stop("Package \"shiny\" is required. Install with: ",
           "install.packages(\"shiny\")")
    }
    if (!requireNamespace("bslib", quietly = TRUE)) {
      stop("Package \"bslib\" is required. Install with: ",
           "install.packages(\"bslib\")")
    }
    if (!requireNamespace("DT", quietly = TRUE)) {
      stop("Package \"DT\" is required. Install with: ",
           "install.packages(\"DT\")")
    }
    if (!requireNamespace("plotly", quietly = TRUE)) {
      stop("Package \"plotly\" is required. Install with: ",
           "install.packages(\"plotly\")")
    }
    shiny::runApp(system.file("shiny", "epidcm", package = "EpiModel"), ...)
  } else if (class == "icm") {
    if (!requireNamespace("shiny", quietly = TRUE)) {
      stop("Package \"shiny\" is required. Install with: ",
           "install.packages(\"shiny\")")
    }
    if (!requireNamespace("bslib", quietly = TRUE)) {
      stop("Package \"bslib\" is required. Install with: ",
           "install.packages(\"bslib\")")
    }
    if (!requireNamespace("DT", quietly = TRUE)) {
      stop("Package \"DT\" is required. Install with: ",
           "install.packages(\"DT\")")
    }
    if (!requireNamespace("plotly", quietly = TRUE)) {
      stop("Package \"plotly\" is required. Install with: ",
           "install.packages(\"plotly\")")
    }
    shiny::runApp(system.file("shiny", "epiicm", package = "EpiModel"), ...)
  } else {
    stop("Specify class as either \"dcm\", \"icm\" ")
  }
}

#' @title Extract Summary Statistics of Networks Used in netsim
#'
#' @description This function calls `summary` on each network being
#'              simulated in `netsim`, provided `save.nwstats` and
#'              `resimulate.network` are both `TRUE`. It records the
#'              statistics represented by `nwstats.formula` in
#'              `dat$stats$nwstats`, where `dat` is the main
#'              `netsim_dat` class object.
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @export
#' @keywords netUtils internal
#'
summary_nets <- function(dat, at) {
  if (get_control(dat, "save.nwstats") == TRUE &&
        get_control(dat, "resimulate.network") == TRUE) {
    for (network in seq_len(dat$num.nw)) {
      nwstats <- summary(get_network_control(dat, network, "nwstats.formula"),
                         basis = get_network(dat, network = network),
                         at = at, # needed for networkDynamic case
                         dynamic = TRUE,
                         term.options = get_network_control(dat, network, "set.control.tergm")$term.options)
      if (is(nwstats, "matrix")) {
        nwstats <- nwstats[, !duplicated(colnames(nwstats)), drop = TRUE]
      } else {
        nwstats <- nwstats[!duplicated(names(nwstats))]
      }
      start_time <- get_control(dat, "start")
      if (start_time > 1L) {
        loc <- at - start_time + 2L
      } else {
        loc <- at
      }
      dat$stats$nwstats[[network]][[loc]] <- nwstats
    }
  }
  return(dat)
}

#' @title RColorBrewer Color Ramp for EpiModel Plots
#'
#' @description Returns a vector of colors consistent with a high-brightness set
#'              of colors from an `RColorBrewer` palette.
#'
#' @param plt `RColorBrewer` palette from [`RColorBrewer::brewer.pal`].
#' @param n Number of colors to return.
#' @param delete.lights If TRUE, delete the lightest colors from the color
#'        palette; this helps with plotting in many high-contrast palettes.
#'
#' @details
#' `RColorBrewer` provides easy access to helpful color palettes, but the
#' built-in palettes are limited to the set of colors in the existing palette.
#' This function expands the palette size to any number of colors by filling
#' in the gaps. Also, colors within the "div" and "seq" set of palettes whose
#' colors are very light (close to white) are deleted by default for better
#' visualization of plots.
#'
#' @return
#' A vector of length equal to `n` with a range of color values consistent
#' with an RColorBrewer color palette.
#'
#' @seealso [RColorBrewer::RColorBrewer]
#' @keywords colorUtils internal
#' @export
#'
#' @examples
#' # Shows a 100-color ramp for 4 RColorBrewer palettes
#' par(mfrow = c(2, 2), mar=c(1, 1, 2, 1))
#' pals <- c("Spectral", "Greys", "Blues", "Set1")
#' for (i in seq_along(pals)) {
#'  plot(1:100, 1:100, type = "n", axes = FALSE, main = pals[i])
#'  abline(v = 1:100, lwd = 6, col = brewer_ramp(100, pals[i]))
#' }
#'
brewer_ramp <- function(n, plt, delete.lights = TRUE) {

  if (n < 1) {
    stop("n must be a positive integer")
  }

  bpi <- brewer.pal.info
  if (!(plt %in% row.names(bpi))) {
    stop("plt must match an RColorBrewer palette name. See
         RColorBrewer::brewer.pal.info",
         .call = FALSE)
  }

  pltmax <- bpi[row.names(bpi) == plt, ]$maxcolors
  pltcat <- bpi[row.names(bpi) == plt, ]$category

  if (pltcat == "div") {
    if (delete.lights == TRUE) {
      colors <- brewer.pal(pltmax, plt)[-c(4:7)]
    } else {
      colors <- brewer.pal(pltmax, plt)
    }
  }
  if (pltcat == "qual") {
    colors <- brewer.pal(pltmax, plt)
  }
  if (pltcat == "seq") {
    if (delete.lights == TRUE) {
      colors <- rev(brewer.pal(pltmax, plt)[-c(1:3)])
    } else {
      colors <- rev(brewer.pal(pltmax, plt))
    }
  }
  if (plt == "Set1") {
    colors <- brewer.pal(9, "Set1")[-6]
  }

  pal <- colorRampPalette(colors)

  return(pal(n))
}

#' @title Delete Elements from Attribute List
#'
#' @description Deletes elements from the main attribute list.
#'
#' @param dat Either the `netsim_dat` class object passed through
#'            `netsim` simulations, or the main data object passed through
#'            `icm` simulations.
#' @param ids ID numbers to delete from the list.
#'
#' @inherit recovery.net return
#'
#' @export
#' @keywords internal
delete_attr <- function(dat, ids) {
  remove_node_attr(dat, ids)
}


#' @title Stable Sampling Function
#'
#' @description Provides a sampling function useful for dynamic simulations, in
#'              which the length of the input vector may be multiple lengths and
#'              the size of the sample may be 0.
#'
#' @param x Either a vector of one or more elements from which to choose, or a
#'        positive integer.
#' @param size Non-negative integer giving the number of items to choose.
#' @param replace Should sampling be with replacement?
#' @param prob Vector of probability weights for obtaining the elements of the
#'        vector being sampled.
#'
#' @return A vector containing the sampled value(s).
#'
#' @export
#' @keywords internal
ssample <- function(x, size, replace = FALSE, prob = NULL) {

  if (length(x) > 1) {
    return(sample(x, size, replace, prob))
  }

  if (length(x) == 1 && size > 0) {
    return(x)
  }

  if (length(x) == 1 && size == 0) {
    return(NULL)
  }

}


#' @title Add New Epidemiology Variables
#'
#' @description Inspired by `dplyr::mutate`, `mutate_epi` adds new
#'              variables to the epidemiological and related variables within
#'              simulated model objects of any class in `EpiModel`.
#'
#' @param x An `EpiModel` object of class `dcm`, `icm`, or
#'        `netsim`.
#' @param ... Name-value pairs of expressions (see examples below).
#'
#' @return The updated `EpiModel` object of class `dcm`, `icm`,
#'         or `netsim`.
#'
#' @export
#'
#' @examples
#' # DCM example
#' param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
#' init <- init.dcm(s.num = 500, i.num = 1)
#' control <- control.dcm(type = "SI", nsteps = 500)
#' mod1 <- dcm(param, init, control)
#' mod1 <- mutate_epi(mod1, prev = i.num/num)
#' plot(mod1, y = "prev")
#'
#' \donttest{
#' # Network model example
#' nw <- network_initialize(n = 100)
#' nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#' est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
#' init <- init.net(i.num = 1, i.num.g2 = 0)
#' control <- control.net(type = "SI", nsteps = 10, nsims = 3,
#'                        verbose = FALSE)
#' mod1 <- netsim(est1, param, init, control)
#' mod1
#'
#' # Add the prevalences to the dataset
#' mod1 <- mutate_epi(mod1, i.prev = i.num / num,
#'                          i.prev.g2 = i.num.g2 / num.g2)
#' plot(mod1, y = c("i.prev", "i.prev.g2"), qnts = 0.5, legend = TRUE)
#'
#' # Add incidence rate per 100 person years (assume time step = 1 week)
#' mod1 <- mutate_epi(mod1, ir100 = 5200*(si.flow + si.flow.g2) /
#'                                       (s.num + s.num.g2))
#' as.data.frame(mod1)
#' as.data.frame(mod1, out = "mean")
#' }
#'
mutate_epi <- function(x, ...) {

  dt <- lazy_dots(...)
  ndat <- lazy_eval(dt, x$epi)

  not.df <- which(sapply(ndat, class) != "data.frame")
  if (length(not.df) > 0) {
    existing_names <- names(x$epi[[1]])
    ncols <- length(existing_names)
    for (jj in not.df) {
      val <- rep(ndat[jj][[1]], length.out = x$control$nsteps)
      ndat[jj][[1]] <- data.frame(replicate(ncols, val))
      names(ndat[[jj]]) <- existing_names
    }
  }

  x$epi <- c(x$epi, ndat)
  return(x)

}

#' @title Apportion Using the Largest Remainder Method
#'
#' @description Apportions a vector of values given a specified frequency
#'              distribution of those values such that the length of the output
#'              is robust to rounding and other instabilities.
#'
#' @param vector.length Length for the output vector.
#' @param values Values for the output vector.
#' @param proportions Proportion distribution with one number for each value.
#'        This must sum to 1.
#' @param shuffled If `TRUE`, randomly shuffle the order of the vector.
#'
#' @return A vector of length `vector.length` containing the apportioned
#'         values from `values`.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' ## Example 1: Without rounding
#' apportioned_vec_1 <- apportion_lr(4, c(1, 2, 3, 4, 5),
#'                                      c(0.25, 0, 0.25, 0.25, 0.25))
#'
#' ## Example 2: With rounding
#' apportioned_vec_2 <- apportion_lr(5, c(1, 2, 3, 4, 5),
#'                                      c(0.21, 0, 0.29, 0.25, 0.25))
#' }
#'
apportion_lr <- function(vector.length, values,
                         proportions, shuffled = FALSE) {

  if (vector.length != round(vector.length)) {
    stop("argument vector.length must be a positive integer")
  }
  if (vector.length <= 0) {
    stop("argument vector.length must be a positive integer")
  }
  if (is.vector(values) == FALSE) {
    stop("argument values must be a vector")
  }
  if (!(length(proportions) == length(values) && round(sum(proportions),
                                                       10) == 1) &&
        (!(length(proportions) == length(values) - 1 && round(sum(proportions),
                                                              10) <= 1 &&
             round(sum(proportions), 10) >= 0))) {
    stop("error in proportions length or proportions sum")
  }

  if (length(proportions) == length(values) - 1) {
    proportions <- c(proportions, 1 - round(sum(proportions), 10))
  }
  result <- rep(NA, vector.length)
  exp.nums <- proportions * vector.length
  counts <- floor(exp.nums)
  remainders <- exp.nums - counts
  leftovers <- vector.length - sum(counts)
  if (leftovers > 0) {
    additions <- order(remainders, decreasing = TRUE)[1:leftovers]
    counts[additions]   <- counts[additions] + 1
  }
  result <- rep(values, counts)
  if (shuffled == TRUE) {
    result <- sample(result, length(result))
  }

  return(result)
}

#' @title Plot Data from a Deterministic Compartmental Epidemic Model
#'
#' @description Plots epidemiological data from a deterministic compartment
#'              epidemic model solved with [dcm()].
#'
#' @param x An `EpiModel` object of class `dcm`.
#' @param y Output compartments or flows from `dcm` object to plot.
#' @param popfrac If `TRUE`, plot prevalence of values rather than numbers
#'        (see details).
#' @param run Run number to plot, for models with multiple runs
#'        (default is run 1).
#' @param col Color for lines, either specified as a single color in a standard
#'        R color format, or alternatively as a color palette from
#'        [RColorBrewer::RColorBrewer] (see details).
#' @param lwd Line width for output lines.
#' @param lty Line type for output lines.
#' @param alpha Transparency level for lines, where 0 = transparent and
#'        1 = opaque (see `adjustcolor` function).
#' @param legend Type of legend to plot. Values are `"n"` for no legend,
#'        `"full"` for full legend, and `"lim"` for limited legend
#'        (see details).
#' @param leg.name Character string to use for legend, with the default
#'        determined automatically based on the `y` input.
#' @param leg.cex Legend scale size.
#' @param grid If `TRUE`, a grid is added to the background of plot
#'        (see [grid()] for details), with default of nx by ny.
#' @param add If `TRUE`, new plot window is not called and lines are added
#'        to existing plot window.
#' @param ... Additional arguments to pass to main plot window (see
#'        [plot.default()]).
#'
#' @inheritParams plot.netsim
#' @inheritParams graphics::plot
#'
#' @details
#' This function plots epidemiological outcomes from a deterministic
#' compartmental model solved with [dcm()]. Depending on the number of
#' model runs (sensitivity analyses) and number of groups, the default plot is
#' the fractional proportion of each compartment in the model over time. The
#' specific compartments or flows to plot may be set using the `y`
#' parameter, and in multiple run models the specific run may also be specified.
#'
#' @section The `popfrac` Argument:
#' Compartment prevalence is the size of a compartment over some denominator.
#' To plot the raw numbers from any compartment, use `popfrac=FALSE`; this
#' is the default. The `popfrac` parameter calculates
#' and plots the denominators of all specified compartments using these rules:
#' 1) for one-group models, the prevalence of any compartment is the compartment
#' size divided by the total population size; 2) for two-group models, the
#' prevalence of any compartment is the compartment size divided by the group
#' size.
#'
#' @section Color Palettes:
#' Since [dcm()] supports multiple run sensitivity models, plotting
#' the results of such models uses a complex color scheme for distinguishing
#' runs. This is accomplished using the [`RColorBrewer::RColorBrewer`] color
#' palettes, which include a range of linked colors using named palettes. For
#' `plot.dcm`, one may either specify a brewer color palette listed in
#' [`RColorBrewer::brewer.pal.info`], or, alternatively, a vector of standard R
#' colors (named, hexidecimal, or positive integers; see [col2rgb()]).
#'
#' @section Plot Legends:
#' There are three automatic legend types available, and the legend is
#' added by default for plots. To turn off the legend, use `legend="n"`. To
#' plot a legend with values for every line in a sensitivity analysis, use
#' `legend="full"`. With models with many runs, this may be visually
#' overwhelming. In those cases, use `legend="lim"` to plot a legend
#' limited to the highest and lowest values of the varying parameter in the
#' model. In cases where the default legend names are not helpful, one may
#' override those names with the `leg.name` argument.
#'
#' @method plot dcm
#' @export
#'
#' @keywords plot
#' @seealso [dcm()], [`RColorBrewer::brewer.pal.info`]
#'
#' @examples
#' # Deterministic SIR model with varying act rate
#' param <- param.dcm(inf.prob = 0.2, act.rate = 1:10,
#'                    rec.rate = 1/3, a.rate = 0.011, ds.rate = 0.01,
#'                    di.rate = 0.03, dr.rate = 0.01)
#' init <- init.dcm(s.num = 1000, i.num = 1, r.num = 0)
#' control <- control.dcm(type = "SIR", nsteps = 100, dt = 0.25)
#' mod <- dcm(param, init, control)
#'
#' # Plot disease prevalence by default
#' plot(mod)
#'
#' # Plot prevalence of susceptibles
#' plot(mod, y = "s.num", popfrac = TRUE, col = "Greys")
#'
#' # Plot number of susceptibles
#' plot(mod, y = "s.num", popfrac = FALSE, col = "Greys", grid = TRUE)
#'
#' # Plot multiple runs of multiple compartments together
#' plot(mod, y = c("s.num", "i.num"),
#'      run = 5, xlim = c(0, 50), grid = TRUE)
#' plot(mod, y = c("s.num", "i.num"),
#'      run = 10, lty = 2, legend = "n", add = TRUE)
#'
plot.dcm <- function(x, y = NULL, popfrac = FALSE, run = NULL, col = NULL,
                     lwd = NULL, lty = NULL, alpha = 0.9, legend = NULL,
                     leg.name = NULL, leg.cex = 0.8, grid = FALSE, add = FALSE,
                     main = "", xlim = NULL, ylim = NULL, xlab = "Time", ylab = NULL,
                     ...) {

  ## Set missing flags
  noy <- is.null(y)
  norun <- is.null(run)
  nocol <- is.null(col)
  nolwd <- is.null(lwd)
  nolty <- is.null(lty)
  noleg <- is.null(legend)

  ## Model dimensions
  nsteps <- x$control$nsteps
  nruns <- x$control$nruns
  if (norun == FALSE && any(run > nruns)) {
    stop("Specify run between 1 and", nruns)
  }

  if (!is.null(x$control$new.mod) && noy == TRUE) {
    stop("Specify y when simulating a new model type in dcm")
  }

  groups <- x$param$groups
  dis.type <- x$control$type

  ## Defaults for missing y
  if (noy == TRUE && nruns == 1) {
    y <- grep(".num", names(x$epi), value = TRUE)
  }
  if (noy == TRUE && nruns > 1) {
    y <- grep("i.num", names(x$epi), value = TRUE)
  }
  if (all(y %in% names(x$epi)) == FALSE) {
    stop("Specified y is unavailable")
  }
  lcomp <- length(y)


  ## Prevalence calculations
  x <- denom(x, y, popfrac)


  ## Compartment ymax calculations
  if (popfrac == FALSE) {
    allmax <- sapply(1:lcomp, function(i) max(x$epi[[y[i]]], na.rm = TRUE))
    ymax <- ceiling(max(allmax))
  } else {
    ymax <- 1
  }


  ## Defaults for ylim, xlim
  if (is.null(ylim)) {
    ylim <- c(0, ymax)
  }

  if (is.null(xlim)) {
    xlim <- c(0, nsteps)
  }

  ## Defaults for lwd
  if (nolwd == FALSE && lcomp > 1 && length(lwd) < lcomp) {
    lwd <- rep(lwd, lcomp)
  }
  if (nolwd == FALSE && lcomp == 1 && length(lwd) < nruns) {
    lwd <- rep(lwd, nruns)
  }
  if (nolwd == TRUE) {
    lwd <- rep(2.5, lcomp * nruns)
  }


  ## Defaults for lty
  if (nolty == FALSE && lcomp > 1 && length(lty) < lcomp) {
    lty <- rep(lty, lcomp)
  }
  if (nolty == FALSE && lcomp == 1 && length(lty) < nruns) {
    lty <- rep(lty, nruns)
  }
  if (nolty == TRUE) {
    lty <- rep(1, lcomp * nruns)
    if (groups == 2 && noy == TRUE) {
      lty <- rep(1:2, each = lcomp / 2)
    }
  }

  ## Defaults for ylab
  if (is.null(ylab)) {
    if (popfrac == FALSE) {
      ylab <- "Number"
    } else {
      ylab <- "Prevalence"
    }
  }

  ## Main plot window
  if (add == FALSE) {
    plot(1, 1, type = "n", bty = "n",
         xlim = xlim, ylim = ylim,
         xlab = xlab, ylab = ylab, main = main, ...)
  }


  ## Default line colors
  pal <- NULL
  # Missing col
  if (nocol == TRUE) {
    if (lcomp == 1) {
      if (nruns == 1) {
        col <- "black"
      }
      if (nruns > 1) {
        col <- "Set1"
      }
      if (nruns > 5) {
        col <- "Spectral"
      }
      if (norun == FALSE && length(run) == 1) {
        col <- "black"
      }
    }
    if (lcomp > 1) {
      col <- "Set1"
    }
  }


  # Test if using a RColorBrewer palette
  if (length(col) == 1 && col %in% row.names(brewer.pal.info)) {
    use.brewer <- TRUE
  } else {
    use.brewer <- FALSE
  }

  # Set color palette
  if (is.null(pal)) {
    if (lcomp == 1) {
      if (use.brewer == TRUE) {
        if (nruns < 6) {
          pal <- adjustcolor(brewer.pal(5, col)[1:nruns], alpha)
        } else {
          pal <- adjustcolor(brewer_ramp(nruns, col), alpha)
        }
      }
      if (use.brewer == FALSE) {
        pal <- adjustcolor(rep(col, nruns), alpha)
      }
    }
    if (lcomp > 1) {
      if (use.brewer == TRUE) {
        if (lcomp > 4) {
          pal <- adjustcolor(brewer_ramp(lcomp, col), alpha)
        } else {
          pal <- adjustcolor(brewer.pal(max(c(lcomp, 4)), col), alpha)
          fixpal <- pal
          fixpal[1] <- pal[2]
          fixpal[2] <- pal[1]
          pal <- fixpal
        }
        if (groups == 2 && noy == TRUE) {
          pal <- adjustcolor(brewer.pal(3, col), alpha)
          fixpal <- pal
          fixpal[1] <- pal[2]
          fixpal[2] <- pal[1]
          pal <- fixpal
          if (dis.type != "SIR") {
            pal <- pal[1:2]
          }
          pal <- rep(pal, times = lcomp / 2)
        }
      }
      if (use.brewer == FALSE) {
        pal <- adjustcolor(rep(col, lcomp), alpha)
        if (groups == 2 && noy == TRUE) {
          pal <- adjustcolor(rep(col, times = 2), alpha)
        }
      }
    }
  }


  ## Plot lines
  if (lcomp == 1) {
    if (nruns == 1) {
      lines(x$control$timesteps, x$epi[[y]][, 1],
            lwd = lwd[1], lty = lty[1], col = pal[1])
    }
    if (nruns > 1) {
      if (norun == TRUE) {
        for (i in 1:nruns) {
          lines(x$control$timesteps, x$epi[[y]][, i],
                lwd = lwd[i], lty = lty[i], col = pal[i])
        }
      } else {
        if (length(run) == 1) {
          lines(x$control$timesteps, x$epi[[y]][, run],
                lwd = lwd[1], lty = lty[1], col = pal[1])
        }
        if (length(run) > 1) {
          for (i in seq_along(run)) {
            lines(x$control$timesteps, x$epi[[y]][, run[i]],
                  lwd = lwd[i], lty = lty[i], col = pal[i])
          }
        }
      }
    }
  }
  if (lcomp > 1) {
    if (nruns == 1) {
      for (i in 1:lcomp) {
        lines(x$control$timesteps, x$epi[[y[i]]][, 1],
              lwd = lwd, lty = lty[i], col = pal[i])
      }
    }
    if (nruns > 1) {
      if (norun == TRUE) {
        for (i in 1:lcomp) {
          run <- 1
          lines(x$control$timesteps, x$epi[[y[i]]][, run],
                lwd = lwd[i], lty = lty[i], col = pal[i])
        }
      }
      if (norun == FALSE) {
        if (length(run) > 1) {
          stop("Plotting multiple runs of multiple y is not supported")
        }
        for (i in 1:lcomp) {
          lines(x$control$timesteps, x$epi[[y[i]]][, run],
                lwd = lwd[i], lty = lty[i], col = pal[i])
        }
      }
    }
  }

  ## Grid
  if (grid == TRUE) {
    grid()
  }

  ## Legend

  # Default legend type
  if (noleg == TRUE) {
    legend <- "n"
    if (lcomp == 1 && nruns < 3) {
      legend <- "full"
    }
    if (lcomp == 1 && nruns >= 3) {
      legend <- "lim"
    }
    if (lcomp > 1) {
      legend <- "full"
    }
    if (noy == FALSE) {
      legend <- "n"
    }
  } else {
    if (legend == "lim" && nruns < 3) {
      legend <- "full"
    }
    if (legend == "lim" && lcomp == 2) {
      legend <- "full"
    }
  }

  # Default legend names
  if (is.null(leg.name)) {
    if (nruns == 1) {
      leg.names <- y
    }
    if (nruns > 1) {
      if (norun == TRUE && lcomp == 1) {
        leg.names <- names(x$epi[[y[1]]])
      }
      if (norun == FALSE && lcomp == 1) {
        if (length(run) == 1) {
          leg.names <- y
        }
        if (length(run) > 1) {
          leg.names <- names(x$epi[[y[1]]][run])
        }
      }
      if (lcomp > 1) {
        leg.names <- y
      }
    }
  } else {
    if (nruns == 1) {
      leg.names <- leg.name
    }
    if (nruns > 1) {
      if (lcomp == 1) {
        leg.names <- paste(leg.name, 1:nruns)
      }
      if (lcomp > 1) {
        leg.names <- y
        warning("Legend names ignored for multiple y plots of multiple run models")
      }
    }
  }

  # Legend
  if (norun == TRUE) {
    if (legend == "full") {
      legend("topright", legend = leg.names,
             bg = "white", lty = lty, lwd = lwd,
             col = pal, cex = leg.cex)
    }
    if (legend == "lim") {
      legend("topright",
             legend = c(leg.names[1], "...", leg.names[nruns]),
             bg = "white",
             lty = c(lty[1], 1, lty[nruns]), lwd = lwd + 1,
             col = c(pal[1], "white", pal[nruns]), cex = leg.cex)
    }
  }
  if (norun == FALSE && legend != "n") {
    if (lcomp == 1) {
      legend("topright", legend = leg.names,
             bg = "white", lty = lty[seq_along(run)],
             lwd = lwd[seq_along(run)],
             col = pal[seq_along(run)], cex = leg.cex)
    }
    if (lcomp > 1) {
      legend("topright", legend = leg.names,
             bg = "white", lty = lty, lwd = lwd,
             col = pal, cex = leg.cex)
    }
  }

}

# Network-related Utility Functions -----------------------------------------

cap_ncores <- function(ncores, detected = parallel::detectCores()) {
  if (!is.numeric(ncores) || length(ncores) != 1 || is.na(ncores) ||
        !is.finite(ncores) || ncores < 1) {
    stop("`ncores` must be a positive finite number.", call. = FALSE)
  }

  if (!is.numeric(detected) || length(detected) != 1 || is.na(detected) ||
        !is.finite(detected) || detected < 1) {
    return(ncores)
  }

  return(min(detected, ncores))
}

#' @title Check Degree Distribution for Balance in Target Statistics
#'
#' @description Checks for consistency in the implied network statistics
#'              of a two-group network in which the group size and
#'              group-specific degree distributions are specified.
#'
#' @param num.g1 Number of nodes in group 1.
#' @param num.g2 Number of nodes in group 2.
#' @param deg.dist.g1 Vector with fractional degree distribution for group 1.
#' @param deg.dist.g2 Vector with fractional degree distribution for group 2.
#'
#' @details
#' This function outputs the number of nodes of degree 0 to g, where g is the
#' length of a fractional degree distribution vector, given that vector and the
#' size of the group. This utility is used to check for balance in implied
#' degree given that fractional distribution within two-group network
#' simulations, in which the degree-constrained counts must be equal across
#' groups.
#'
#' @export
#' @keywords netUtils
#'
#' @examples
#' # An unbalanced distribution
#' check_degdist_bal(num.g1 = 500, num.g2 = 500,
#'                   deg.dist.g2 = c(0.40, 0.55, 0.03, 0.02),
#'                   deg.dist.g1 = c(0.48, 0.41, 0.08, 0.03))
#'
#' # A balanced distribution
#' check_degdist_bal(num.g1 = 500, num.g2 = 500,
#'                   deg.dist.g1 = c(0.40, 0.55, 0.04, 0.01),
#'                   deg.dist.g2 = c(0.48, 0.41, 0.08, 0.03))
#'
check_degdist_bal <- function(num.g1, num.g2,
                              deg.dist.g1, deg.dist.g2) {
  deg.counts.g1 <- deg.dist.g1 * num.g1
  deg.counts.g2 <- deg.dist.g2 * num.g2
  tot.deg.g1 <- sum(deg.counts.g1 * (seq_along(deg.dist.g1) - 1))
  tot.deg.g2 <- sum(deg.counts.g2 * (seq_along(deg.dist.g2) - 1))
  mat <- matrix(c(deg.dist.g1, deg.counts.g1,
                  deg.dist.g2, deg.counts.g2), ncol = 4)
  mat <- rbind(mat, c(sum(deg.dist.g1), tot.deg.g1, sum(deg.dist.g2),
                      tot.deg.g2))
  colnames(mat) <- c("g1.dist", "g1.cnt", "g2.dist", "g2.cnt")
  rownames(mat) <- c(paste0("Deg", 0:(length(deg.dist.g1) - 1)), "Edges")
  cat("Degree Distribution Check\n")
  cat("=============================================\n")
  print(mat, print.gap = 3)
  cat("=============================================\n")
  reldiff <- (tot.deg.g1 - tot.deg.g2) / tot.deg.g2
  absdiff <- abs(tot.deg.g1 - tot.deg.g2)
  if (sum(deg.dist.g1) <= 0.999 || sum(deg.dist.g1) >= 1.001 ||
        sum(deg.dist.g2) <= 0.999 || sum(deg.dist.g2) >= 1.001 || absdiff > 1) {
    if (sum(deg.dist.g1) <= 0.999 || sum(deg.dist.g1) >= 1.001) {
      cat("** deg.dist.g1 TOTAL != 1 \n")
    }
    if (sum(deg.dist.g2) <= 0.999 || sum(deg.dist.g2) >= 1.001) {
      cat("** deg.dist.g2 TOTAL != 1 \n")
    }
    if (absdiff > 1) {
      if (tot.deg.g1 > tot.deg.g2) {
        msg <- "Group 1 Edges > Group 2 Edges:"
      } else {
        msg <- "Group 1 Edges < Group 2 Edges:"
      }
      cat("**", msg, round(reldiff, 3), "Rel Diff \n")
    }
  } else {
    cat("** Edges balanced ** \n")
  }
  invisible(c(tot.deg.g1, deg.counts.g1, deg.counts.g2))
}


#' @title Create a TEA Variable for Infection Status for `ndtv` Animations
#'
#' @description Creates a new color-named temporally-extended attribute (TEA)
#'              variable in a `networkDynamic` object containing a disease
#'              status TEA in numeric format.
#'
#' @param nd An object of class `networkDynamic`.
#' @param old.var Old TEA variable name.
#' @param old.sus Status value for susceptible in old TEA variable.
#' @param old.inf Status value for infected in old TEA variable.
#' @param old.rec Status value for recovered in old TEA variable.
#' @param new.var New TEA variable name to be stored in `networkDynamic`
#'        object.
#' @param new.sus Status value for susceptible in new TEA variable.
#' @param new.inf Status value for infected in new TEA variable.
#' @param new.rec Status value for recovered in new TEA variable.
#' @param verbose If `TRUE`, print progress to console.
#'
#' @details
#' The `ndtv` package (<https://cran.r-project.org/package=ndtv>)
#' produces animated visuals for dynamic networks with evolving edge structures
#' and nodal attributes. Nodal attribute dynamics in `ndtv` movies require
#' a temporally extended attribute (TEA) containing a standard R color for each
#' node at each time step. By default, the `EpiModel` package uses TEAs to
#' store disease status history in network model simulations run in
#' [netsim()]. But that status TEA is in numeric format (0, 1, 2).
#' The `color_tea` function transforms those numeric values of that disease
#' status TEA into a TEA with color values in order to visualize status changes
#' in `ndtv`.
#'
#' The convention in [plot.netsim()] is to color the susceptible
#' nodes as blue, infected nodes as red, and recovered nodes as green. Alternate
#' colors may be specified using the `new.sus`, `new.inf`, and
#' `new.rec` parameters, respectively.
#'
#' Using the `color_tea` function with a `netsim` object requires that
#' TEAs for disease status be used and that the `networkDynamic` object be
#' saved in the output: `tergmListe` must be  set to `FALSE` in
#' [control.net()].
#'
#' @return The updated object of class `networkDynamic`.
#'
#' @seealso [netsim()] and the `ndtv` package documentation.
#' @keywords colorUtils
#' @export
#'
color_tea <- function(nd, old.var = "testatus", old.sus = "s", old.inf = "i",
                      old.rec = "r", new.var = "ndtvcol", new.sus = NULL,
                      new.inf = NULL, new.rec = NULL, verbose = TRUE) {
  if (is.null(new.inf)) {
    new.inf <- adjustcolor(2, 0.75)
  }
  if (is.null(new.sus)) {
    new.sus <- adjustcolor(4, 0.75)
  }
  if (is.null(new.rec)) {
    new.rec <- adjustcolor(3, 0.75)
  }
  times <- seq_len(max(get.change.times(nd)))
  for (at in times) {
    stat <- get.vertex.attribute.active(nd, old.var, at = at)
    infected <- which(stat == old.inf)
    uninfected <- which(stat == old.sus)
    recovered <- which(stat == old.rec)
    nd <- activate.vertex.attribute(nd, prefix = new.var, value = new.inf,
                                    onset = at, terminus = Inf, v = infected)
    nd <- activate.vertex.attribute(nd, prefix = new.var, value = new.sus,
                                    onset = at, terminus = Inf, v = uninfected)
    nd <- activate.vertex.attribute(nd, prefix = new.var, value = new.rec,
                                    onset = at, terminus = Inf, v = recovered)
    if (verbose == TRUE) {
      cat("\n", at, "/", max(times), "\t", sep = "")
    }
  }
  return(nd)
}


#' @title Copy Vertex Attributes From Network to `netsim_dat` List
#'
#' @description Copies the vertex attributes stored on the network object to the
#'              main `attr` list in the `netsim_dat` data object.
#'
#' @inheritParams recovery.net
#' @param nw Network from which to copy vertex attributes.
#'
#' @inherit recovery.net return
#'
#' @seealso [get_formula_term_attr()], [get_attr_prop()],
#'          [auto_update_attr()], and
#'          [copy_datattr_to_nwattr()].
#' @keywords netUtils internal
#' @export
#'
copy_nwattr_to_datattr <- function(dat, nw) {
  otha <- list.vertex.attributes(nw)
  otha <- setdiff(otha, c("na", "vertex.names", "active",
                          "testatus.active", "tergm_pid"))
  if (length(otha) > 0) {
    epi_by <- get_control(dat, "epi.by", override.null.error = TRUE)
    for (i in seq_along(otha)) {
      va <- get_vertex_attribute(nw, otha[i])
      dat <- set_attr(dat, otha[i], va)
      if (!is.null(epi_by) && epi_by == otha[i]) {
        dat$run$epi.by.vals <- unique(va)
      }
    }
  }
  return(dat)
}


#' @title Copy Vertex Attributes from the `netsim_dat` List to the Network
#'        Objects
#'
#' @description Copies the vertex attributes stored on the main `attr` list
#'              of the `netsim_dat` object to each of the network objects
#'              stored on the `netsim_dat` object.
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @seealso [get_formula_term_attr()], [get_attr_prop()],
#'          [auto_update_attr()], and
#'          [copy_nwattr_to_datattr()].
#' @keywords netUtils internal
#' @export
#'
copy_datattr_to_nwattr <- function(dat) {
  nwterms <- dat$run$nwterms
  special.attr <- "status"
  if (dat$param$groups == 2) {
    special.attr <- c(special.attr, "group")
  }
  nwterms <- union(nwterms, special.attr)
  attr.to.copy <- union(nwterms, special.attr)
  attr <- get_attr_list(dat, attr.to.copy)
  if (length(attr.to.copy) > 0) {
    if (length(attr.to.copy) == 1) {
      for (net_index in seq_len(dat$num.nw)) {
        net <- get_network(dat, network = net_index)
        net <- set_vertex_attribute(net, names(attr), attr[[1]])
        dat <- set_network(dat, nw = net, network = net_index)
      }
    } else {
      for (net_index in seq_len(dat$num.nw)) {
        net <- get_network(dat, network = net_index)
        net <- set_vertex_attribute(net, names(attr), attr)
        dat <- set_network(dat, nw = net, network = net_index)
      }
    }
  }
  return(dat)
}

#' @title Dissolution Coefficients for Stochastic Network Models
#'
#' @description Calculates dissolution coefficients, given a dissolution model
#'              and average edge duration, to pass as offsets to an ERGM/TERGM
#'              model fit in `netest`.
#'
#' @param dissolution Right-hand sided STERGM dissolution formula
#'        (see [netest()]). See below for list of supported
#'        dissolution models.
#' @param duration A vector of mean edge durations in arbitrary time units.
#' @param d.rate Departure or exit rate from the population, as a single
#'        homogeneous rate that applies to the entire population.
#'
#' @details
#' This function performs two calculations for dissolution coefficients
#' used in a network model estimated with [netest()]:
#'
#'  1. **Transformation:** the mean durations of edges in a network are
#'     mathematically transformed to logit coefficients.
#'  2. **Adjustment:** in a dynamic network simulation in an open
#'     population (in which there are departures), it is further necessary to
#'     adjust these coefficients; this upward adjustment accounts for
#'     departure as a competing risk to edge dissolution.
#'
#'
#' The current dissolution models supported by this function and in network
#' model estimation in [netest()] are as follows:
#'
#'  * `~offset(edges)`: a homogeneous dissolution model in which the
#'         edge duration is the same for all partnerships. This requires
#'         specifying one duration value.
#'  * `~offset(edges) + offset(nodematch("<attr>"))`: a heterogeneous
#'         model in which the edge duration varies by whether the nodes in the
#'         dyad have similar values of a specified attribute. The duration
#'         vector should now contain two values: the first is the mean edge
#'         duration of non-matched dyads, and the second is the duration of the
#'         matched dyads.
#'  * `~offset(edges) + offset(nodemix("<attr>"))`: a heterogeneous
#'         model that extends the nodematch model to include non-binary
#'         attributes for homophily. The duration vector should first contain
#'         the base value, then the values for every other possible combination
#'         in the term.
#'
#'
#' @return
#' A list of class `disscoef` with the following elements:
#'
#'  * **dissolution:** right-hand sided STERGM dissolution formula
#'         passed in the function call.
#'  * **duration:** mean edge durations passed into the function.
#'  * **coef.crude:** mean durations transformed into logit
#'        coefficients.
#'  * **coef.adj:** crude coefficients adjusted for the risk of
#'        departure on edge persistence, if the `d.rate` argument is
#'        supplied.
#'  * **coef.form.corr:** corrections to be subtracted from formation
#'        coefficients.
#'  * **d.rate:** the departure rate.
#'  * **diss.model.type:** the form of the dissolution model; options
#'        include `edgesonly`, `nodematch`, and `nodemix`.
#'
#'
#' @export
#' @keywords netUtils
#'
#' @examples
#' ## Homogeneous dissolution model with no departures
#' dissolution_coefs(dissolution = ~offset(edges), duration = 25)
#'
#' ## Homogeneous dissolution model with departures
#' dissolution_coefs(dissolution = ~offset(edges), duration = 25,
#'                   d.rate = 0.001)
#'
#' ## Heterogeneous dissolution model in which same-race edges have
#' ## shorter duration compared to mixed-race edges, with no departures
#' dissolution_coefs(dissolution = ~offset(edges) + offset(nodematch("race")),
#'                   duration = c(20, 10))
#'
#' ## Heterogeneous dissolution model in which same-race edges have
#' ## shorter duration compared to mixed-race edges, with departures
#' dissolution_coefs(dissolution = ~offset(edges) + offset(nodematch("race")),
#'                   duration = c(20, 10), d.rate = 0.001)
#'
#' \dontrun{
#' ## Extended example for differential homophily by age group
#' # Set up the network with nodes categorized into 5 age groups
#' nw <- network_initialize(n = 1000)
#' age.grp <- sample(1:5, 1000, TRUE)
#' nw <- set_vertex_attribute(nw, "age.grp", age.grp)
#'
#' # durations = non-matched, age.grp1 & age.grp1, age.grp2 & age.grp2, ...
#' # TERGM will include differential homophily by age group with nodematch term
#' # Target stats for the formation model are overall edges, and then the number
#' # matched within age.grp 1, age.grp 2, ..., age.grp 5
#' form <- ~edges + nodematch("age.grp", diff = TRUE)
#' target.stats <- c(450, 100, 125, 40, 80, 100)
#'
#' # Target stats for the dissolution model are duration of non-matched edges,
#' # then duration of edges matched within age.grp 1, age.grp 2, ..., age.grp 5
#' durs <- c(60, 30, 80, 100, 125, 160)
#' diss <- dissolution_coefs(~offset(edges) +
#'                             offset(nodematch("age.grp", diff = TRUE)),
#'                           duration = durs)
#'
#' # Fit the TERGM
#' fit <- netest(nw, form, target.stats, diss)
#'
#' # Full diagnostics to evaluate model fit
#' dx <- netdx(fit, nsims = 10, ncores = 4, nsteps = 300)
#' print(dx)
#'
#' # Simulate one long time series to examine timed edgelist
#' dx <- netdx(fit, nsims = 1, nsteps = 5000, keep.tedgelist = TRUE)
#'
#' # Extract timed-edgelist
#' te <- as.data.frame(dx)
#' head(te)
#'
#' # Limit to non-censored edges
#' te <- te[which(te$onset.censored == FALSE & te$terminus.censored == FALSE),
#'          c("head", "tail", "duration")]
#' head(te)
#'
#' # Look up the age group of head and tail nodes
#' te$ag.head <- age.grp[te$head]
#' te$ag.tail <- age.grp[te$tail]
#' head(te)
#'
#' # Recover average edge durations for age-group pairing
#' mean(te$duration[te$ag.head != te$ag.tail])
#' mean(te$duration[te$ag.head == 1 & te$ag.tail == 1])
#' mean(te$duration[te$ag.head == 2 & te$ag.tail == 2])
#' mean(te$duration[te$ag.head == 3 & te$ag.tail == 3])
#' mean(te$duration[te$ag.head == 4 & te$ag.tail == 4])
#' mean(te$duration[te$ag.head == 5 & te$ag.tail == 5])
#' durs
#' }
#'
dissolution_coefs <- function(dissolution, duration, d.rate = 0) {
  # Error check for duration < 1
  if (any(duration < 1)) {
    stop("All values in duration must be >= 1")
  }
  # Check form of dissolution formula
  diss.model.type <- NA
  form.length <- length(strsplit(as.character(dissolution)[2], "[+]")[[1]])
  t1.edges <- grepl("offset[(]edges",
                    strsplit(as.character(dissolution)[2], "[+]")[[1]][1])
  if (form.length == 1 && t1.edges == TRUE) {
    diss.model.type <- "edgesonly"
  } else {
    if (form.length == 2 && t1.edges == TRUE) {
      t2 <- strsplit(as.character(dissolution)[2], "[+]")[[1]][2]
      t2.term <- NULL
      if (grepl("offset[(]nodematch", t2)) {
        t2.term <- diss.model.type <- "nodematch"
      } else if (grepl("offset[(]nodemix", t2)) {
        t2.term <- diss.model.type <- "nodemix"
      } else {
        stop("The form of the dissolution argument is invalid. Type
              help(\'dissolution_coefs\') to see the set of options
              allowed.")
      }
    }
  }

  if (length(d.rate) > 1) {
    stop("Length of d.rate must be 1")
  }
  # Log transformation of duration to coefficent
  if (t1.edges == FALSE) {
    stop("Dissolution models must start with offset(edges)")
  }
  if (form.length == 1) {
    if (length(duration) > 1) {
      stop("Dissolution model length is 1, but number of duration was ",
           length(duration))
    }
    pg <- (duration[1] - 1) / duration[1]
    ps2 <- (1 - d.rate) ^ 2
    if (ps2 <= pg) {
      d.rate_ <- round(1 - sqrt(pg), 5)
      str <- paste("The competing risk of departure is too high for the given",
                   " duration of ", duration[1],
                   "; specify a d.rate lower than ", d.rate_, ".", sep = "")
      stop(str)
    }

    coef.crude <- log(pg / (1 - pg))
    coef.adj <- log(pg / (ps2 - pg))
    coef.form.corr <- log(1 + pg / (1 - pg))
  }
  if (form.length == 2) {
    if (t2.term %in% c("nodematch", "nodemix")) {
      coef.crude <- coef.adj <- coef.form.corr <- NA
      for (i in seq_along(duration)) {
        pg <- (duration[i] - 1) / duration[i]
        ps2 <- (1 - d.rate) ^ 2

        if (ps2 <= pg) {
          d.rate_ <- round(1 - sqrt(pg), 5)
          stop("The competing risk of departure is too high for the given",
               " edge duration of ", duration[i], " in place ", i, ". ",
               "Specify a d.rate lower than ", d.rate_, ".", sep = "")
        }
        if (i == 1) {
          coef.crude[i] <- log(pg / (1 - pg))
          coef.adj[i] <- log(pg / (ps2 - pg))
          coef.form.corr[i] <- log(1 + pg / (1 - pg))
        } else {
          coef.crude[i] <- log(pg / (1 - pg)) - coef.crude[1]
          coef.adj[i] <- log(pg / (ps2 - pg)) - coef.adj[1]
          coef.form.corr[i] <- log(1 + pg / (1 - pg)) - coef.form.corr[1]
        }
      }
    } else {
      stop("Supported heterogeneous dissolution model terms are nodematch ",
           "or nodemix")
    }
  }
  out <- list()
  out$dissolution <- dissolution
  out$duration <- duration
  out$coef.crude <- coef.crude
  out$coef.adj <- coef.adj
  out$coef.form.corr <- coef.form.corr
  out$d.rate <- d.rate
  out$diss.model.type <- diss.model.type
  class(out) <- "disscoef"
  return(out)
}


#' @title Table of Edge Censoring
#'
#' @description Outputs a table of the number and percent of edges that are
#'              left-censored, right-censored, both-censored, or uncensored for
#'              a `networkDynamic` object.
#'
#' @param el A timed edgelist with start and end times extracted from a
#'        `networkDynamic` object using the
#'        `as.data.frame.networkDynamic` function.
#'
#' @return A 4 x 2 table containing the number and percent of edges in `el`
#'         that are left-censored, right-censored, both-censored, or uncensored.
#'
#' @export
#' @keywords netUtils
#'
#' @details
#' Given a STERGM simulation over a specified number of time steps, the edges
#' within that simulation may be left-censored (started before the first step),
#' right-censored (continued after the last step), right and left-censored, or
#' uncensored. The amount of censoring will increase when the average edge
#' duration approaches the length of the simulation.
#'
#' @examples
#' \donttest{
#' # Initialize and parameterize network model
#' nw <- network_initialize(n = 100)
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#'
#' # Model estimation
#' est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' # Simulate the network and extract a timed edgelist
#' dx <- netdx(est, nsims = 1, nsteps = 100, keep.tedgelist = TRUE,
#'       verbose = FALSE)
#' el <- as.data.frame(dx)
#'
#' # Calculate censoring
#' edgelist_censor(el)
#' }
#'
edgelist_censor <- function(el) {
  # left censored
  leftcens <- el$onset.censored
  leftcens.num <- sum(leftcens)
  leftcens.pct <- leftcens.num / nrow(el)
  # right censored
  rightcens <- el$terminus.censored
  rightcens.num <- sum(rightcens)
  rightcens.pct <- rightcens.num / nrow(el)
  # partnership lasts for entire window (left and right censored)
  lrcens <- el$onset.censored & el$terminus.censored
  lrcens.num <- sum(lrcens)
  lrcens.pct <- lrcens.num / nrow(el)
  # fully observed
  nocens <- el$onset.censored == FALSE & el$terminus.censored == FALSE
  nocens.num <- sum(nocens)
  nocens.pct <- nocens.num / nrow(el)
  ## Table
  nums <- rbind(leftcens.num, rightcens.num, lrcens.num, nocens.num)
  pcts <- rbind(leftcens.pct, rightcens.pct, lrcens.pct, nocens.pct)
  out <- cbind(nums, pcts)
  rownames(out) <- c("Left Cens.", "Right Cens.", "Both Cens.", "No Cens.")
  colnames(out) <- c("num", "pct")
  return(out)
}


#' @title Proportional Table of Vertex Attributes
#'
#' @description Calculates the proportional distribution of each vertex
#'              attribute contained in a network.
#'
#' @inheritParams recovery.net
#' @param nwterms Vector of attributes on the network object, usually as
#'        output of [get_formula_term_attr()].
#'
#' @return
#' A table containing the proportional distribution of each attribute in
#' `nwterms`.
#'
#' @seealso [get_formula_term_attr()],
#'          [copy_nwattr_to_datattr()],
#'          [auto_update_attr()].
#' @keywords netUtils internal
#' @export
#'
get_attr_prop <- function(dat, nwterms) {

  if (is.null(nwterms)) {
    return(NULL)
  }
  attr_list <- get_attr_list(dat)
  nwVal <- names(attr_list)
  nwVal <- setdiff(nwVal, c("na", "vertex.names", "active", "entrTime",
                            "exitTime", "infTime", "group", "status"))
  out <- list()
  if (length(nwVal) > 0) {
    for (i in seq_along(nwVal)) {
      tab <- prop.table(table(attr_list[[nwVal[i]]]))
      out[[i]] <- tab
    }
    names(out) <- nwVal
  }

  return(out)
}


#' @title Output ERGM Formula Attributes into a Character Vector
#'
#' @description Given a formation formula for a network model, outputs a
#'              character vector of vertex attributes to be used in
#'              [netsim()] simulations.
#'
#' @param form An ERGM model formula.
#' @param nw A network object.
#'
#' @return A character vector of vertex attributes.
#'
#' @export
#'
get_formula_term_attr <- function(form, nw) {

  nw_attr <- list.vertex.attributes(nw)
  nw_attr <- setdiff(nw_attr, c("active", "vertex.names", "na"))

  if (length(nw_attr) == 0) {
    return(NULL)
  }
  matches <- sapply(nw_attr, function(x) grepl(x, form))
  matches <- colSums(matches)

  out <- names(matches)[which(matches == 1)]
  if (length(out) == 0) {
    return(NULL)
  } else {
    return(out)
  }

}

#' @title Output Network Attributes into a Character Vector
#'
#' @description Given a simulated network, outputs a character vector of vertex
#'              attributes to be used in [netsim()] simulations.
#'
#' @param nw A network object.
#'
#' @return A character vector of vertex attributes.
#'
#' @export
#'
get_network_term_attr <- function(nw) {

  nw_attr <- list.vertex.attributes(nw)
  nw_attr <- setdiff(nw_attr, c("active", "vertex.names", "na",
                                "testatus.active", "tergm_pid"))

  if (length(nw_attr) == 0) {
    return(NULL)
  }

  out <- nw_attr
  if (length(out) == 0) {
    return(NULL)
  } else {
    return(out)
  }

}

#' @title Group Numbers for Two-Group Network
#'
#' @description Outputs group numbers given ID numbers for a two-group network.
#'
#' @param nw Object of class `network` or `networkDynamic`.
#' @param ids Vector of ID numbers for which the group number
#'        should be returned. If `NULL` (default), return all IDs.
#'
#' @return A vector containing the group number for each of the specified nodes.
#'
#' @export
#' @keywords netUtils internal
#'
#' @examples
#' nw <- network_initialize(n = 10)
#' nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 5))
#' idgroup(nw)
#' idgroup(nw, ids = c(3, 6))
#'
idgroup <- function(nw, ids = NULL) {
  n <- network.size(nw)
  if (is.null(ids)) {
    ids <- seq_len(n)
  }
  if (any(ids > n)) {
    stop("Specify ids between 1 and ", n)
  }

  flag <- "group" %in% list.vertex.attributes(nw)
  if (!flag) {
    out <- rep(1, n)
  } else {
    groups <- get_vertex_attribute(nw, "group")
    out <- groups[ids]
  }

  return(out)
}

#' @title Update Vertex Attributes for Incoming Vertices
#'
#' @description Updates the vertex attributes on a network for new nodes
#'              incoming into that network, based on a set of rules for each
#'              attribute that the user specifies in [control.net()].
#'
#' @inheritParams recovery.net
#' @param newNodes Vector of nodal IDs for incoming nodes at the current time
#'        step.
#' @param curr.tab Current proportional distribution of all vertex attributes.
#'
#' @inherit recovery.net return
#'
#' @seealso [copy_nwattr_to_datattr()], [get_attr_prop()],
#'          [auto_update_attr()].
#' @keywords netUtils internal
#' @export
#'
auto_update_attr <- function(dat, newNodes, curr.tab) {

  rules <- get_control(dat, "attr.rules")
  active <- get_attr(dat, "active")
  t1.tab <- dat$run$t1.tab

  for (i in seq_along(curr.tab)) {
    vname <- names(curr.tab)[i]
    needs.updating <- ifelse(length(get_attr(dat, vname)) < length(active),
                             TRUE, FALSE)
    if (length(vname) > 0 && needs.updating == TRUE) {
      rule <- rules[[vname]]

      if (is.null(rule)) {
        rule <- "current"
      }
      if (rule == "current") {
        vclass <- class(get_attr(dat, vname))
        if (vclass == "character") {
          nattr <- sample(names(curr.tab[[vname]]),
                          size = length(newNodes),
                          replace = TRUE,
                          prob = curr.tab[[vname]])
        } else {
          nattr <- sample(as.numeric(names(curr.tab[[i]])),
                          size = length(newNodes),
                          replace = TRUE,
                          prob = curr.tab[[i]])
        }
      } else if (rule == "t1") {
        vclass <- class(get_attr(dat, vname))
        if (vclass == "character") {
          nattr <- sample(names(t1.tab[[vname]]),
                          size = length(newNodes),
                          replace = TRUE,
                          prob = t1.tab[[vname]])
        } else {
          nattr <- sample(as.numeric(names(t1.tab[[i]])),
                          size = length(newNodes),
                          replace = TRUE,
                          prob = t1.tab[[i]])
        }
      } else {
        nattr <- rep(rules[[vname]], length(newNodes))
      }
      dat <- append_attr(dat, vname, nattr, length(nattr))
    }
  }

  return(dat)
}


#' @title Get Individual Degree from Network or Edgelist
#'
#' @description A fast method for querying the current degree of all individuals
#'              within a network.
#'
#' @param x Either an object of class `network` or `edgelist`
#'        generated from a network. If `x` is an edgelist, then it must
#'        contain an attribute for the total network size, `n`.
#'
#' @details
#' Individual-level data on the current degree of nodes within a network is
#' often useful for summary statistics. Given a `network` class object,
#' `net`, one way to look up the current degree is to get a summary of the
#' ERGM term, `sociality`, as in:
#' `summary(net ~ sociality(nodes = NULL))`. But that is computationally
#' inefficient for a number of reasons. This function provides a fast method for
#' generating the vector of degrees using a query of the edgelist. It is even
#' faster if the parameter `x` is already transformed into an edgelist.
#'
#' @return A vector of length equal to the total network size, containing the
#'         current degree of each node in the network.
#'
#' @export
#'
#' @examples
#' \donttest{
#' nw <- network_initialize(n = 500)
#'
#' set.seed(1)
#' fit <- ergm(nw ~ edges, target.stats = 250)
#' sim <- simulate(fit)
#'
#' # Slow ERGM-based method
#' ergm.method <- unname(summary(sim ~ sociality(nodes = NULL)))
#' ergm.method
#'
#' # Fast tabulate method with network object
#' deg.net <- get_degree(sim)
#' deg.net
#'
#' # Even faster if network already transformed into an edgelist
#' el <- as.edgelist(sim)
#' deg.el <- get_degree(el)
#' deg.el
#'
#' identical(as.integer(ergm.method), deg.net, deg.el)
#' }
#'
get_degree <- function(x) {
  if (inherits(x, "network")) {
    x <- as.edgelist(x)
  }
  if (is.null(attr(x, "n"))) {
    stop("x missing an n attribute")
  }
  n <- attr(x, "n")
  out <- tabulate(x, nbins = n)
  return(out)
}

#' @title Truncate Simulation Time Series
#'
#' @description Left-truncates simulation epidemiological summary statistics
#'              and network statistics at a specified time step.
#'
#' @param x Object of class `dcm`, `netsim`, or `icm`.
#' @param at Time step at which to left-truncate the time series.
#' @param reset.time If `TRUE`, the time step sequence in the truncated model
#'        will be reset to start at 1. If `FALSE`, the original time step
#'        values will be preserved. Default is `TRUE`.
#'
#' @details
#' This function would be used when running a follow-up simulation from time
#' steps `b` to `c` after a burn-in period from time `a` to
#' `b`, where the final time window of interest for data analysis is
#' `b` to `c` only.
#'
#' @return The updated object of class `dcm`, `netsim`, or `icm`.
#'
#' @export
#'
#' @examples
#' # DCM examples
#' param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
#' init <- init.dcm(s.num = 500, i.num = 1)
#' control <- control.dcm(type = "SI", nsteps = 200)
#' mod1 <- dcm(param, init, control)
#' plot(mod1)
#'
#' # Reset time
#' mod2a <- truncate_sim(mod1, at = 150)
#' plot(mod2a)
#' head(as.data.frame(mod2a))
#'
#' # Do not reset time
#' mod2b <- truncate_sim(mod1, at = 150, reset.time = FALSE)
#' plot(mod2b)
#' head(as.data.frame(mod2b))
#'
#' \donttest{
#' # ICM example
#' param <- param.icm(inf.prob = 0.2, act.rate = 0.25)
#' init <- init.icm(s.num = 500, i.num = 1)
#' control <- control.icm(type = "SI", nsteps = 200, nsims = 1)
#' mod1 <- icm(param, init, control)
#'
#' # Reset time
#' mod2a <- truncate_sim(mod1, at = 150)
#' plot(mod2a)
#' head(as.data.frame(mod2a))
#'
#' # Do not reset time
#' mod2b <- truncate_sim(mod1, at = 150, reset.time = FALSE)
#' plot(mod2b)
#' head(as.data.frame(mod2b))
#' }
#'
truncate_sim <- function(x, at, reset.time) {
  UseMethod("truncate_sim")
}

#' @method truncate_sim dcm
#' @rdname truncate_sim
#' @export
truncate_sim.dcm <- function(x, at, reset.time = TRUE) {
  row_start <- which(x$control$timesteps == at)
  if (length(row_start) == 0) {
    stop("Specified value of at is not in the control$timesteps vector")
  }
  rows <- row_start:nrow(x$epi[[1]])
  # epi
  x$epi <- lapply(x$epi, function(r) r[rows, , drop = FALSE])
  # control settings
  if (reset.time) {
    x$control$timesteps <- x$control$timesteps[rows] - (at - 1)
  } else {
    x$control$timesteps <- x$control$timesteps[rows]
  }
  x$control$nsteps <- max(x$control$timesteps)
  return(x)
}

#' @method truncate_sim icm
#' @rdname truncate_sim
#' @export
truncate_sim.icm <- function(x, at, reset.time = TRUE) {
  rows <- at:(x$control$nsteps)
  # epi
  x$epi <- lapply(x$epi, function(r) r[rows, , drop = FALSE])
  # control settings
  x$control$nsteps <- max(seq_along(rows))
  if (reset.time) {
    x$control$start <- 1
  } else {
    x$control$start <- at
  }
  return(x)
}

#' @method truncate_sim netsim
#' @rdname truncate_sim
#' @export
truncate_sim.netsim <- function(x, at, reset.time = TRUE) {
  truncate_sim.icm(x, at = at, reset.time = reset.time)
}

#' Make a Lightweight Restart Point From a `netsim` Object with tergmLite
#'
#' Extract the elements required for re-initializing a `netsim` simulation from
#' a completed simulation. This function also resets the Unique IDs and Time
#' values to reduce the size of the simulation. This function only works for
#' simulations where `control$tergmLite = TRUE`
#'
#' @param sim_obj a `netsim` object from an ended `netsim` call.
#' @param sim_num the number of the simulation to extract from the `netsim`
#'        object (default = 1).
#' @param keep_steps The number of simulation steps to keep from the previous
#'        run. By default only keep one but more is possible if some
#'        back-history is wanted.
#' @param time_attrs a `character` vector containing the names of the attributes
#'        that are expressed in time-steps. These will be offsetted so the last
#'        step in the original simulation become the step 1 (default) in the new
#'        ones. If no such attributes exist, pass `c()`.
#'
#' @details
#' The restart point created always contains a single simulation and drops the
#' `attr.history`, the `raw.records` and `stats` from the initial simulation.
#'
#' The epi trackers, cumulative edgelists, transmission matrix and `nwstats` are
#' truncated to only contain the last `keep_steps` entries.
#'
#' Warning: the `time_attrs` argument is mandatory. Almost all simulation worth
#' restarting have such attributes (e.g. time.of.hiv.infection). If no such
#' argument exists, passing `c()` will allow the function to run while ensuring
#' that this was done on purpose.
#'
#' When restarting from the output of this function, it is suggested to express
#' the time steps in a relative maner in `control.net`:
#' ```
#' control.net(
#'   start = restart_point$control$nsteps + 1),
#'   nsteps = restart_point$control$nsteps + 1 + 104)
#' )
#' ```
#'
#' @examples
#' \dontrun{
#' # With  pre-existing `sim`, `param` and `init` object (see `netsim`)
#'
#' # List all attributes that store a time step
#' time_attrs <- c(
#'   "inf.time",
#'   "stage.time",
#'   "aids.time",
#'   "prep.start.last"
#' )
#' # Make a restart point a re-run for 10 more timesteps
#' x <- make_restart_point(sim, time_attrs, sim_num = 1, keep_steps = 1)
#' control <- control_msm(
#'   start = x$control$nsteps + 1,
#'   nsteps = x$control$nsteps + 1 + 10
#' )
#' sim <- netsim(x, param, init, control)
#' }
#'
#' @return a trimed `netsim` object with only one simulation that is ready to be
#'         used as a restart point.
#'
#' @export
make_restart_point <- function(sim_obj, time_attrs,
                               sim_num = 1, keep_steps = 1) {
  if (!inherits(sim_obj, c("netsim"))) {
    stop("`sim_obj` must be  an object of class `netsim`")
  }
  required_names <- c(
    "control", "param", "nwparam", "epi", "run", "coef.form", "num.nw"
  )
  missing_names <- setdiff(required_names, names(sim_obj))
  if (length(missing_names) > 0) {
    stop(
      "`sim_obj` is missing the following elements required for",
      " re-initialization: ", paste.and(missing_names)
    )
  }
  if (sim_num < 1 || sim_num > sim_obj$control$nsims) {
    stop("`sim_num` must be be >= 1 and <= `sim_obj$control$nsims`")
  }
  if (!sim_obj$control$tergmLite) {
    stop("Only `netsim` object with `tergmLite == TRUE` are supported")
  }

  # Select  the simulation of interest, that renames the selected sim: `sim1`
  x <- get_sims(sim_obj, sims = sim_num)
  n_steps <- x$control$nsteps
  run_ls <- x$run$sim1

  # Keep only the last `keep_steps` rows of each epi
  if (keep_steps < 1 || keep_steps > n_steps) {
    stop("`keep_steps` must be >= 1 and <= `sim_obj$control$nsteps`")
  }
  keep_rows <- (n_steps - keep_steps + 1):n_steps
  x$epi <- lapply(x$epi, function(r) r[keep_rows, , drop = FALSE])

  # If `nwstats` are saved, keep only the last rows
  if (x$control$save.nwstats) {
    x$stats$nwstats$sim1 <- lapply(
      x$stats$nwstats$sim1,
      function(d) d[keep_rows, , drop = FALSE]
    )
  }

  # Fix UIDs
  uid_offset <- min(run_ls$attr$unique_id) - 1
  run_ls$attr$unique_id <- run_ls$attr$unique_id - uid_offset
  run_ls$last_unique_id <- run_ls$last_unique_id - uid_offset

  # Time correction
  time_offset <- n_steps - keep_steps
  x$control$start <- 1
  x$control$nsteps <- keep_steps

  # Time attributes - offset so last step is now `keep_steps`
  time_attrs <- union(c("entrTime", "exitTime"), time_attrs)
  missing_attrs <- setdiff(time_attrs, names(run_ls$attr))
  if (length(missing_attrs) > 0) {
    stop(
      "Some time attributes are not present in the attributes list:",
      paste.and(missing_names)
    )
  }
  run_ls$attr[time_attrs] <- lapply(
    run_ls$attr[time_attrs],
    function(v) v - time_offset
  )

  # Cumulative Edgelist - fix time and UIDs
  run_ls$el_cuml_cur <- lapply(
    run_ls$el_cuml_cur,
    function(el) {
      el$head <- el$head - uid_offset
      el$tail <- el$tail - uid_offset
      el$start <- el$start - time_offset
      el
    }
  )
  # For Historical one - truncate to 1 (only edges in the kept history)
  run_ls$el_cuml_hist <- lapply(
    run_ls$el_cuml_hist,
    function(el) {
      el$head <- el$head - uid_offset
      el$tail <- el$tail - uid_offset
      el$start <- el$start - time_offset
      el$stop <- el$stop - time_offset
      el[el$stop >= 1, , drop = FALSE]
    }
  )

  # the edgelist stores the name of the vertices. We don't use it with
  # `tergmLite` and it takes a lot of space
  run_ls$el <- lapply(run_ls$el, function(x) {
    attr(x, "vnames") <- NULL
    x
  })

  # If transmat was saved, trim it and offset the `at` column
  if (x$control$save.transmat) {
    tsmt <- x$stats$transmat$sim1
    tsmt$at <- tsmt$at - time_offset
    x$stats$transmat$sim1 <- tsmt[tsmt$at > 0, , drop = FALSE]
  }

  x$run$sim1 <- run_ls
  x$attr.history <- list()
  x$raw.records <- list()
  return(x)
}

#' @title Function to Reduce the Size of a `netest` Object
#'
#' @description Trims formula environments from the `netest` object.
#'              Optionally converts the `newnetwork` element of the
#'              `netest` object to a `networkLite` class, and removes
#'              the `fit` element (if present) from the `netest`
#'              object.
#'
#' @param object A `netest` class object.
#' @param as.networkLite If `TRUE`, converts `object$newnetwork`
#'        to a `networkLite`.
#' @param keep.fit If `FALSE`, removes the `object$fit` (if present)
#'        on the `netest` object.
#' @param keep Character vector of object names to keep in formula environments.
#'        By default, all objects are removed.
#'
#' @details
#' With larger, more complex network structures with epidemic models, it is
#' generally useful to reduce the memory footprint of the fitted TERGM model
#' object (estimated with [netest()]). This utility function removes
#' all but the bare essentials needed for simulating a network model with
#' [netsim()].
#'
#' The function always trims the environments of `object$constraints` and
#' `object$coef.diss$dissolution`.
#'
#' When both `edapprox = TRUE` and `nested.edapprox = TRUE` in the
#' `netest` call, also trims the environments of `object$formula`
#' and `object$formation`.
#'
#' When both `edapprox = TRUE` and `nested.edapprox = FALSE` in the
#' `netest` call, also trims the environments of `object$formula`,
#' `environment(object$formation)$formation`, and
#' `environment(object$formation)$dissolution`.
#'
#' When `edapprox = FALSE` in the `netest` call, also trims the
#' environments of `object$formation`,
#' `environment(object$formula)$formation` and
#' `environment(object$formula)$dissolution`.
#'
#' By default all objects are removed from these trimmed environments. Specific
#' objects may be retained by passing their names as the `keep` argument.
#' For the output of `trim_netest` to be usable in [netsim()]
#' simulation, any objects referenced in the formulas should be included in the
#' `keep` argument.
#'
#' If `as.networkLite = TRUE`, converts `object$newnetwork` to a
#' `networkLite` object. If `keep.fit = FALSE`, removes `fit` (if
#' present) from `object`.
#'
#' @return
#' A `netest` object with formula environments trimmed, optionally with the
#' `newnetwork` element converted to a `networkLite` and the
#' `fit` element removed.
#'
#' @export
#'
#' @examples
#' \donttest{
#' nw <- network_initialize(n = 100)
#' formation <- ~edges + concurrent
#' target.stats <- c(50, 25)
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)
#' est <- netest(nw, formation, target.stats, coef.diss,
#'               set.control.ergm = control.ergm(MCMC.burnin = 1e5,
#'                                               MCMC.interval = 1000))
#' print(object.size(est), units = "KB")
#'
#' est.small <- trim_netest(est)
#' print(object.size(est.small), units = "KB")
#' }
#'
trim_netest <- function(object, as.networkLite = TRUE, keep.fit = FALSE,
                        keep = character(0)) {
  if (object$edapprox == TRUE) {
    object$formula <- trim_env(object$formula, keep = keep)
    if (object$nested.edapprox == TRUE) {
      object$formation <- trim_env(object$formation, keep = keep)
    } else {
      # trim environments for formation and dissolution inside formation
      environment(object$formation)$formation <-
        trim_env(environment(object$formation)$formation, keep = keep)
      environment(object$formation)$dissolution <-
        trim_env(environment(object$formation)$dissolution, keep = keep)
    }
  } else {
    object$formation <- trim_env(object$formation, keep = keep)
    # trim environments for formation and dissolution inside formula
    environment(object$formula)$formation <-
      trim_env(environment(object$formula)$formation, keep = keep)
    environment(object$formula)$dissolution <-
      trim_env(environment(object$formula)$dissolution, keep = keep)
  }

  object$coef.diss$dissolution <- trim_env(object$coef.diss$dissolution, keep = keep)
  object$constraints <- trim_env(object$constraints, keep = keep)

  if (keep.fit == FALSE) {
    object$fit <- NULL
    object$summary <- NULL
  }

  if (as.networkLite == TRUE) {
    object$newnetwork <- as.networkLite(object$newnetwork)
  }

  return(object)
}

context("Utility Functions")

test_that("brewer_ramp", {

  expect_true(length(brewer_ramp(100, plt = "Spectral")) == 100)
  expect_true(length(brewer_ramp(100, plt = "Spectral",
                                 delete.lights = FALSE)) == 100)
  expect_false(length(brewer_ramp(100, plt = "Spectral")) == 50)
  expect_false(length(brewer_ramp(100, plt = "Spectral",
                                  delete.lights = FALSE)) == 50)

  expect_true(length(brewer_ramp(100, plt = "Accent")) == 100)
  expect_true(length(brewer_ramp(100, plt = "Accent",
                                 delete.lights = FALSE)) == 100)

  expect_true(length(brewer_ramp(100, plt = "Oranges")) == 100)
  expect_true(length(brewer_ramp(100, plt = "Oranges",
                                 delete.lights = FALSE)) == 100)

  expect_true(length(brewer_ramp(100, plt = "Set1")) == 100)

  expect_error(brewer_ramp(100, plt = "Jimmy"))
  expect_error(brewer_ramp(100, plt = "Jimmy", delete.lights = FALSE))
  expect_error(brewer_ramp(-1, plt = "Spectral"))

})


test_that("color_tea", {
  nw <- network_initialize(n = 100)
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
  est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
  param <- param.net(inf.prob = 0.3, rec.rate = 0.001)
  init <- init.net(i.num = 25, r.num = 25)
  control <- control.net(type = "SIR", nsteps = 10, nsims = 1, verbose = FALSE)
  mod <- netsim(est, param, init, control)
  nd <- get_network(mod)
  nd <- color_tea(nd, verbose = FALSE)
  expect_is(nd, "networkDynamic")
  expect_true(length(unique(
    get.vertex.attribute.active(nd, "ndtvcol", at = 1))) == 3)
})

test_that("delete_attr", {
  dat <- create_dat_object()
  dat <- append_core_attr(dat, 1, 5)
  dat <- append_attr(dat, "a", 1:5, 5)
  dat <- append_attr(dat, "b", 6:10, 5)

  dat <- delete_attr(dat, 5)
  expect_is(dat, "list")

  dat <- delete_attr(dat, 2:3)
  expect_true(length(unique(vapply(get_attr_list(dat), length, 1))) == 1)

  l2 <- dat
  l3 <- delete_attr(dat, NULL)

  expect_equal(l2, l3)
})

test_that("ssample", {

  expect_true(length(ssample(1:5, 1)) == 1)
  expect_equal(ssample(5, 1), 5)
  expect_equal(ssample(5, 2), 5)
  expect_null(ssample(5, 0))

})

test_that("check_degdist_bal", {
  expect_output(check_degdist_bal(num.g1 = 500, num.g2 = 500,
                                  deg.dist.g2 = c(0.40, 0.55, 0.03, 0.02),
                                  deg.dist.g1 = c(0.48, 0.41, 0.08, 0.03)),
                "-0.015 Rel Diff")
  expect_output(check_degdist_bal(num.g1 = 500, num.g2 = 500,
                                  deg.dist.g1 = c(0.40, 0.55, 0.04, 0.01),
                                  deg.dist.g2 = c(0.48, 0.41, 0.08, 0.03)),
                "Edges balanced")
  expect_output(check_degdist_bal(num.g1 = 500, num.g2 = 500,
                                  deg.dist.g1 = c(0.45, 0.55, 0.04, 0.01),
                                  deg.dist.g2 = c(0.48, 0.41, 0.08, 0.03)),
                "deg.dist.g1 TOTAL != 1")
  expect_output(check_degdist_bal(num.g1 = 500, num.g2 = 500,
                                  deg.dist.g1 = c(0.40, 0.55, 0.04, 0.01),
                                  deg.dist.g2 = c(0.55, 0.41, 0.08, 0.03)),
                "deg.dist.g2 TOTAL != 1")
})

test_that("cap_ncores handles unknown detected core counts", {
  expect_equal(EpiModel:::cap_ncores(2, detected = NA_integer_), 2)
  expect_equal(EpiModel:::cap_ncores(2, detected = 0), 2)
  expect_equal(EpiModel:::cap_ncores(2, detected = Inf), 2)
  expect_equal(EpiModel:::cap_ncores(2, detected = 4), 2)
  expect_equal(EpiModel:::cap_ncores(4, detected = 2), 2)
  expect_error(EpiModel:::cap_ncores(NA_real_, detected = 2), "`ncores` must be")
})

test_that("edgelist_censor", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
  est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
  sim <- netdx(est, nsims = 1, nsteps = 100,
               keep.tedgelist = TRUE, verbose = FALSE)
  el <- as.data.frame(sim)
  expect_is(edgelist_censor(el), "matrix")
})

test_that("get_degree", {
  nw <- network_initialize(n = 500)

  set.seed(1)
  fit <- ergm(nw ~ edges, target.stats = 250, eval.loglik = FALSE)
  sim <- simulate(fit)

  # Slow ERGM-based method
  ergm.method <- unname(summary(sim ~ sociality(nodes = TRUE)))
  ergm.method

  # Fast tabulate method with network object
  deg.net <- get_degree(sim)
  deg.net

  # Even faster if network already transformed into an edgelist
  el <- as.edgelist(sim)
  deg.el <- get_degree(el)
  deg.el

  expect_equal(ergm.method, deg.net)
  expect_equal(ergm.method, deg.el)
})

test_that("dissolution_coefs returns error for incompatible departure rate", {
  #Dividing by zero:
  d.rate_ <- round(1 - sqrt(59 / 60), 5)
  err.msg <- paste("The competing risk of departure is too high for the given",
                   " duration of ", 60, "; specify a d.rate lower than ",
                   d.rate_, ".", sep = "")
  dissolution <-  ~offset(edges)
  expect_that(dissolution_coefs(dissolution, duration = 60, d.rate = 1 / 60),
              throws_error(err.msg))
  expect_that(dissolution_coefs(dissolution, duration = 60, d.rate = 0.01),
              throws_error(err.msg))
  dissolution <-  ~offset(edges) + offset(nodematch("group", diff = TRUE))
  duration <- c(60, 30, 80, 100, 125, 160)
  dissolution <-  ~offset(edges) + offset(nodematch("age.grp", diff = TRUE))
  err.msg <- paste("The competing risk of departure is too high for the given",
                   "edge duration of 60 in place 1.",
                   "Specify a d.rate lower than 0.00837.")
  expect_that(dissolution_coefs(dissolution,
                                duration = duration, d.rate = 1 / 60),
              throws_error(err.msg))
})


test_that("get_formula_term_attr checks", {
  nw <- network_initialize(n = 100)

  expect_null(get_formula_term_attr(~edges, nw))

  nw <- network_initialize(n = 100)
  riskg <- sample(rep(1:2, each = 50))
  race <- sample(rep(0:1, each = 50))
  nw <- set_vertex_attribute(nw, "riskg", riskg)
  nw <- set_vertex_attribute(nw, "race", race)
  expect_null(get_formula_term_attr(~edges, nw))

  expect_equal(get_formula_term_attr(~edges + nodefactor("race"), nw), "race")
  expect_equal(get_formula_term_attr(~edges + nodefactor("race") +
                                       nodematch("riskg"), nw),
               c("race", "riskg"))
  expect_equal(get_formula_term_attr(~edges + nodefactor(c("race", "riskg")) +
                                       nodematch("riskg"), nw),
               c("race", "riskg"))

})

context("Shiny App")

test_that("epiweb works", {
  expect_error(epiweb(class = "foo"))
})

context("update.R Functionality")

test_that("add_vertices, delete_vertices, delete_edges behave as expected", {
  net_size <- 10L
  nw <- network_initialize(net_size)
  nw[1, 2] <- 1
  nw[3, 6] <- 1
  nw[2, 5] <- 1
  nw[8, 10] <- 1
  el <- as.edgelist(nw)
  el <- matrix(c(el), ncol = 2)
  attr(el, "n") <- net_size

  ## add some vertices
  el_add <- add_vertices(el, 4)
  expect_equal(el, el_add, check.attributes = FALSE)
  expect_equal(attr(el_add, "n"), net_size + 4)

  ## delete no vertices, no edges
  vertices <- NULL
  el_del <- matrix(c(1, 2, 3, 8, 2, 5, 6, 10), ncol = 2)
  el_drop <- matrix(c(1, 2, 3, 8, 2, 5, 6, 10), ncol = 2)
  el_del_edges <- delete_edges(el, vertices)
  el_del_verts <- delete_vertices(el, vertices)
  expect_equal(el_del_edges, el_del, check.attributes = FALSE)
  expect_equal(el_del_verts, el_drop, check.attributes = FALSE)
  expect_equal(attr(el_del_edges, "n"), net_size)
  expect_equal(attr(el_del_verts, "n"), net_size - length(vertices))

  ## delete some vertices, no edges
  vertices <- c(4, 7)
  el_del <- matrix(c(1, 2, 3, 8, 2, 5, 6, 10), ncol = 2)
  el_drop <- matrix(c(1, 2, 3, 6, 2, 4, 5, 8), ncol = 2)
  el_del_edges <- delete_edges(el, vertices)
  el_del_verts <- delete_vertices(el, vertices)
  expect_equal(el_del_edges, el_del, check.attributes = FALSE)
  expect_equal(el_del_verts, el_drop, check.attributes = FALSE)
  expect_equal(attr(el_del_edges, "n"), net_size)
  expect_equal(attr(el_del_verts, "n"), net_size - length(vertices))

  ## delete some vertices, some edges
  vertices <- c(2, 8, 9)
  el_del <- matrix(c(3, 6), ncol = 2)
  el_drop <- matrix(c(2, 5), ncol = 2)
  el_del_edges <- delete_edges(el, vertices)
  el_del_verts <- delete_vertices(el, vertices)
  expect_equal(el_del_edges, el_del, check.attributes = FALSE)
  expect_equal(el_del_verts, el_drop, check.attributes = FALSE)
  expect_equal(attr(el_del_edges, "n"), net_size)
  expect_equal(attr(el_del_verts, "n"), net_size - length(vertices))

  ## delete some vertices, all edges
  vertices <- c(1, 2, 3, 4, 5, 6, 8, 10)
  el_del <- matrix(integer(0), ncol = 2)
  el_drop <- matrix(integer(0), ncol = 2)
  el_del_edges <- delete_edges(el, vertices)
  el_del_verts <- delete_vertices(el, vertices)
  expect_equal(el_del_edges, el_del, check.attributes = FALSE)
  expect_equal(el_del_verts, el_drop, check.attributes = FALSE)
  expect_equal(attr(el_del_edges, "n"), net_size)
  expect_equal(attr(el_del_verts, "n"), net_size - length(vertices))

  ## delete all vertices, all edges
  vertices <- c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
  el_del <- matrix(integer(0), ncol = 2)
  el_drop <- matrix(integer(0), ncol = 2)
  el_del_edges <- delete_edges(el, vertices)
  el_del_verts <- delete_vertices(el, vertices)
  expect_equal(el_del_edges, el_del, check.attributes = FALSE)
  expect_equal(el_del_verts, el_drop, check.attributes = FALSE)
  expect_equal(attr(el_del_edges, "n"), net_size)
  expect_equal(attr(el_del_verts, "n"), net_size - length(vertices))
})

context("Network Diagnostics (All SOC)")

for (trim in c(FALSE, TRUE)) {

  test_that("Edges only models", {
    skip_on_cran()
    num <- 50
    nw <- network_initialize(n = num)
    formation <- ~edges
    target.stats <- 15
    coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
    est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
    if (trim == TRUE) {
      est1 <- trim_netest(est1)
    }
    ## Single simulation
    dx1 <- netdx(est1, nsims = 1, nsteps = 10, verbose = FALSE)
    expect_is(dx1, "netdx")
    capture_output(
      print(dx1)
    )
    plot(dx1)
    plot(dx1, method = "b")
    plot(dx1, type = "duration", mean.smooth = FALSE)
    plot(dx1, method = "b", type = "duration")
    plot(dx1, type = "dissolution")
    plot(dx1, method = "b", type = "dissolution")

    ## Multiple simulations
    dx2 <- netdx(est1, nsims = 2, nsteps = 10, verbose = FALSE)
    expect_is(dx2, "netdx")
    capture_output(
      print(dx2)
    )
    plot(dx2)
    plot(dx2, method = "b")
    plot(dx2, type = "duration")
    plot(dx2, method = "b", type = "duration")
    plot(dx2, type = "dissolution")
    plot(dx2, method = "b", type = "dissolution")

    ## Expanded monitoring formula
    dx3 <- netdx(est1, nsims = 2, nsteps = 10, verbose = FALSE,
                 nwstats.formula = ~edges + concurrent)
    expect_is(dx3, "netdx")
    capture_output(
      print(dx3)
    )
    plot(dx3)
    plot(dx3, plots.joined = FALSE)
    plot(dx3, method = "b")
    plot(dx3, type = "duration")
    plot(dx3, method = "b", type = "duration")
    plot(dx3, type = "dissolution")
    plot(dx3, method = "b", type = "dissolution")
    plot(dx3, type = "formation", sim.lines = FALSE, plots.joined = FALSE,
         mean.line = FALSE, qnts = FALSE)

    ## Reduced monitoring formula
    dx4 <- netdx(est1, nsims = 2, nsteps = 10, verbose = FALSE,
                 nwstats.formula = ~meandeg)
    expect_is(dx4, "netdx")
    capture_output(
      print(dx4)
    )
    plot(dx4)
    plot(dx4, method = "b")
    plot(dx4, type = "duration")
    plot(dx4, method = "b", type = "duration")
    plot(dx4, type = "dissolution")
    plot(dx4, method = "b", type = "dissolution")
  })

  test_that("Formation plot color vector length", {
    skip_on_cran()
    n <-  100
    mean.degree <- ((0 * 0.10) + (1 * 0.41) + (2 * 0.25) + (3 * 0.22))
    expected.concurrent <- n * 0.49
    expected.edges <- (mean.degree) * (n / 2)
    nw <- network_initialize(n = n)
    formation <- ~edges + concurrent + degrange(from = 4)
    target.stats <- c(expected.edges, expected.concurrent, 0)
    coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 40,
                                   d.rate = 0.01)
    est <- netest(nw, formation, target.stats, coef.diss)
    if (trim == TRUE) {
     est <- trim_netest(est)
    }
    dx <- netdx(est, nsims = 2, nsteps = 500,
                nwstats.formula = ~edges + meandeg + degree(0:4) + concurrent, verbose = FALSE)

    expect_error(plot(dx, sim.col = c("green", "orange")))
    expect_error(plot(dx, mean.col = c("green", "orange")))
    expect_error(plot(dx, targ.col = c("green", "orange")))
    expect_error(plot(dx, qnts.col = c("green", "orange")))
  })

  test_that("Netdx duration and dissolution plots error when
            skip.dissolution = TRUE", {
    skip_on_cran()
    nw <- network_initialize(n = 100)
    formation <- ~edges
    target.stats <- 50
    coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 2)
    est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
    if (trim == TRUE) {
     est <- trim_netest(est)
    }
    dx2 <- netdx(est, nsims = 1, nsteps = 500, skip.dissolution = TRUE, verbose = FALSE)

    expect_error(plot(dx2, type = "duration"))
    expect_error(plot(dx2, type = "dissolution"))
  })

  test_that("Offset terms", {
    skip_on_cran()
    n <- 50
    nw <- network_initialize(n = n)
    nw <- set_vertex_attribute(nw, "loc", rep(0:1, each = n / 2))
    dissolution <- ~offset(edges)
    duration <- 40
    coef.diss <- dissolution_coefs(dissolution, duration)
    formation <- ~edges + offset(nodemix("loc", levels2 = -c(1, 3)))
    target.stats <- 15
    est2 <- netest(nw, formation, target.stats, coef.diss, coef.form = -Inf,
                   verbose = FALSE)
    if (trim == TRUE) {
     est2 <- trim_netest(est2)
    }
    dx <- netdx(est2, nsims = 2, nsteps = 50, verbose = FALSE)
    expect_is(dx, "netdx")
    capture_output(
      print(dx)
    )
    plot(dx)
    plot(dx, plots.joined = FALSE)
    plot(dx, method = "b")
    plot(dx, type = "duration")
    plot(dx, method = "b", type = "duration")
    plot(dx, type = "dissolution")
    plot(dx, method = "b", type = "dissolution")
    rm(dx)

    ## Offset term with expanded formula
    dx <- netdx(est2, nsims = 2, nsteps = 50, verbose = FALSE,
                nwstats.formula = ~edges + meandeg + concurrent + nodematch("loc"))
    expect_is(dx, "netdx")
    capture_output(
      print(dx)
    )
    plot(dx)
    plot(dx, plots.joined = FALSE)
    plot(dx, method = "b")
    plot(dx, type = "duration")
    plot(dx, method = "b", type = "duration")
    plot(dx, type = "dissolution")
    plot(dx, method = "b", type = "dissolution")
  })

  test_that("Faux offset term", {
    skip_on_cran()
    n <- 50
    nw <- network_initialize(n = n)
    nw <- set_vertex_attribute(nw, "loc", rep(0:1, each = n / 2))
    coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 40)
    formation <- ~edges + nodemix("loc", levels2 = -c(1, 3))
    target.stats <- c(15, 0)
    est3 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
    if (trim == TRUE) {
     est3 <- trim_netest(est3)
    }

    dx <- netdx(est3, nsims = 2, nsteps = 50, verbose = FALSE)
    expect_is(dx, "netdx")
    capture_output(
      print(dx)
    )
    plot(dx)
    plot(dx, plots.joined = FALSE)
    plot(dx, method = "b")
    plot(dx, type = "duration")
    plot(dx, method = "b", type = "duration")
    plot(dx, type = "dissolution")
    plot(dx, method = "b", type = "dissolution")
  })


  test_that("More complicated faux offset term", {
    skip_on_cran()
    nw <- network_initialize(n = 1000)
    nw <- set_vertex_attribute(nw, "sexor",
                               sample(c(rep(1, 20), rep(2, 460),
                                        rep(3, 20), rep(4, 500))))
    nw <- set_vertex_attribute(nw, "region", sample(rep(1:5, 200)))
    fit <- netest(nw,
                  formation =
                    ~edges + nodemix("sexor", levels2 = c(-1, -3, -5)) + nodematch("region"),
                  target.stats = c(463, 0, 0, 0, 10, 210, 10, 110, 90),
                  coef.diss = dissolution_coefs(~offset(edges), 60),
                  set.control.ergm = control.ergm(MCMLE.termination = "Hummel",
                                                  MCMLE.samplesize = 1024,
                                                  MCMLE.interval = 1024,
                                                  MCMLE.effectiveSize = NULL))
    if (trim == TRUE) {
     fit <- trim_netest(fit)
    }
    dx <- netdx(fit, nsteps = 10, verbose = FALSE)
    capture_output(
      print(dx)
    )
    expect_is(dx, "netdx")
  })


  test_that("Static diagnostic simulations", {
    skip_on_cran()
    nw <- network_initialize(n = 100)
    formation <- ~edges + concurrent
    target.stats <- c(50, 20)
    coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)
    est4 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
    if (trim == TRUE) {
     est4 <- trim_netest(est4)
    }

    dx <- netdx(est4, dynamic = FALSE, nsims = 250,
                nwstats.formula = ~edges + meandeg + concurrent, verbose = FALSE)
    expect_is(dx, "netdx")

    plot(dx)
    plot(dx, stats = "meandeg")
    plot(dx, plots.joined = FALSE)

    plot(dx, method = "b", col = "bisque")

    expect_error(plot(dx, method = "b", type = "duration"))
    expect_error(plot(dx, method = "b", type = "dissolution"))

    # test for default formation model
    dx <- netdx(est4, dynamic = FALSE, nsims = 250, verbose = FALSE)
    expect_is(dx, "netdx")
  })


  test_that("Parallel methods", {
    skip_on_cran()
    skip_on_os(os = "windows")
    num <- 50
    nw <- network_initialize(n = num)
    formation <- ~edges
    target.stats <- 15
    coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
    est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
    if (trim == TRUE) {
     est1 <- trim_netest(est1)
    }

    dx1 <- netdx(est1, nsims = 2, nsteps = 25, ncores = 2, verbose = FALSE)
    expect_is(dx1, "netdx")
    expect_true(inherits(future::plan(), "sequential"))
  })

  test_that("error checking", {
    skip_on_cran()
    nw <- network_initialize(n = 25)
    est <- netest(nw, formation = ~edges, target.stats = 25,
                  coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                  edapprox = TRUE, verbose = FALSE)
    if (trim == TRUE) {
     est <- trim_netest(est)
    }
    expect_error(netdx(x = 1, nsteps = 100))
    expect_error(netdx(est), "Specify number of time steps with nsteps")
  })

  test_that("Cross sectional ergm dynamic error check", {
    skip_on_cran()
    nw <- network_initialize(n = 100)
    formation <- ~edges
    target.stats <- 50
    coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 1)
    est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
    if (trim == TRUE) {
     est <- trim_netest(est)
    }
    expect_error(netdx(est, nsims = 5, nsteps = 500),
                 "Running dynamic diagnostics on a cross-sectional")
  })

  test_that("print.netdx output", {
    skip_on_cran()
    nw <- network_initialize(n = 100)
    formation <- ~edges + concurrent
    target.stats <- c(50, 25)
    coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 50)
    est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
    if (trim == TRUE) {
     est <- trim_netest(est)
    }
    dx <- netdx(est, nsims = 1, nsteps = 10, verbose = FALSE)
    expect_output(print(dx), "edges")
    expect_output(print(dx), "concurrent")
    expect_output(print(dx), "Duration Diagnostics")
    expect_output(print(dx), "Dissolution Diagnostics")
    expect_output(print(dx), "Target")
    expect_output(print(dx), "Sim Mean")
    expect_output(print(dx), "Pct Diff")
    expect_output(print(dx), "Sim SE")

    dx <- netdx(est, nsims = 1, nsteps = 100, verbose = FALSE)
    expect_output(print(dx), "NA")
  })

  test_that("print.netdx and plot.netdx with heterogeneous diss", {
    skip_on_cran()
    set.seed(12345)
    nw <- network_initialize(n = 100)
    nw <- set_vertex_attribute(nw, "neighborhood", rep(1:10, 10))
    nw <- set_vertex_attribute(nw, "position", rep(1:4, 25))
    formation <- ~edges+nodematch("neighborhood", diff = TRUE)
    target.stats <- c(100, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13)
    coef.diss <- dissolution_coefs(dissolution =
                   ~offset(edges)+offset(nodematch("neighborhood", diff = TRUE)),
                     duration = c(20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30))
    est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
    if (trim == TRUE) {
      est <- trim_netest(est)
    }
    dx11 <- netdx(est, nsims = 5, nsteps = 100, verbose = FALSE)
    expect_length(dx11$stats.table.duration$Target, 11)
    expect_length(dx11$stats.table.dissolution$`Sim Mean`, 11)
    suppressMessages({
      expect_output(print(dx11), "match.neighborhood.7")
      plot(dx11)
      capture_output(
        plot(dx11, type = "duration")
      )
      capture_output(
        plot(dx11, type = "dissolution")
      )
    })

    formation <- ~edges + nodemix("position")
    target.stats <- c(100, 9, 7, 8, 8, 12, 8, 15, 5, 10)
    coef.diss <- dissolution_coefs(dissolution = ~offset(edges) + offset(nodemix("position")),
                                   duration = c(80, 75, 80, 70, 85, 65, 70, 75, 60, 80))
    est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
    if (trim == TRUE) {
      est <- trim_netest(est)
    }
    dx12 <- netdx(est, nsims = 7, nsteps = 30, verbose = FALSE)
    expect_output(print(dx12), "60")
    expect_output(print(dx12, digits = 4), "0.0133")
    expect_output(print(dx12), "mix.position.1.4")
    plot(dx12, qnts = FALSE)
    plot(dx12, type = "duration", qnts=0.8)
    plot(dx12, type = "dissolution", sim.col = 1:10, verbose = FALSE)

    dx13 <- netdx(est, nsims = 1, nsteps = 1, verbose = FALSE)
    expect_true(dim(dx13$pages)[1]==1 & dim(dx13$pages)[2]==10 & dim(dx13$pages)[3]==1)
    expect_true(dim(dx13$prop.diss)[1]==1 & dim(dx13$prop.diss)[2]==10 & dim(dx13$prop.diss)[3]==1)
  })
}

test_that("z scores are not large for a reasonably long simulation", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  nw <- set_vertex_attribute(nw, "race", rep(0:1, length.out = 100))
  est <- netest(nw, formation = ~edges + nodematch("race", diff = TRUE),
                target.stats = c(50, 10, 10),
                coef.diss = dissolution_coefs(~offset(edges) +
                                                offset(nodematch("race", diff = TRUE)),
                                              c(10, 20, 15)),
                verbose = FALSE
  )

  dx <- netdx(est, nsteps = 1000, nsims = 1, verbose = FALSE)

  expect_true(all(abs(dx$stats.table.formation[["Z Score"]]) < 20))
  expect_true(all(abs(dx$stats.table.duration[["Z Score"]]) < 20))
  expect_true(all(abs(dx$stats.table.dissolution[["Z Score"]]) < 20))

  dxs <- netdx(est, nsims = 1000, dynamic = FALSE, verbose = FALSE)

  expect_true(all(abs(dxs$stats.table.formation[["Z Score"]]) < 20))
})

test_that("make_stats_table behaves as expected", {
  skip_on_cran()
  stat_names <- letters[1:5]
  stats_1 <- 1:5
  stats_2 <- c(6, 9, 5, 2, 7)
  m1 <- matrix(stats_1, ncol = 5, nrow = 10, byrow = TRUE)
  m2 <- matrix(stats_2, ncol = 5, nrow = 10, byrow = TRUE)
  colnames(m2) <- colnames(m1) <- stat_names
  stats <- list(m1, m2)
  targets <- c(a = 3, f = 6, e = 7, c = 9, x = 10, d = 2000)
  targs <- c(3, NA, 9, 2000, 7)
  mst <- make_stats_table(stats, targets)
  expect_equal(rownames(mst), colnames(m1))
  expect_equal(mst[["Sim Mean"]], (stats_1 + stats_2)/2)
  expect_equal(mst[["Target"]], targs)
  expect_equal(mst[["Pct Diff"]], 100*((stats_1 + stats_2)/2 - targs)/targs)
})

context("EpiModel Terms")

nw <- network_initialize(n = 50)
age <- runif(50)
sex <- rep(c(0,1), length.out = 50)
nw %v% "age" <- age
nw %v% "sex" <- sex

test_that("netest works for EpiModel terms", {
  skip_on_cran()
  est <- netest(nw, formation = ~edges + absdiffby("age", "sex", 1) + nodematch("sex"), target.stats = c(25, 25, 0),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)
  expect_is(est, "netest")

  est2 <- netest(nw, formation = ~edges + absdiffnodemix("age", "sex"), target.stats = c(40, 5, 10, 5),
                 coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                 verbose = FALSE)
  expect_is(est2, "netest")
})

test_that("EpiModel terms produce correct summary statistics", {
  skip_on_cran()
  nw1 <- san(nw ~ edges + offset(nodematch("sex")), target.stats = c(30), offset.coef = c(-Inf))
    
  el1 <- as.edgelist(nw1)
  
  expect_equal(summary(nw1 ~ absdiffby("age", "sex", 2.3)),
               sum(abs(age[el1[,1]] - 2.3*sex[el1[,1]] - age[el1[,2]] + 2.3*sex[el1[,2]])),
               check.attributes = FALSE)
  
  nw2 <- san(nw ~ edges, target.stats = c(40))

  el2 <- as.edgelist(nw2)

  expect_equal(summary(nw2 ~ absdiffnodemix("age", "sex")),
               c(sum(abs(age[el2[,1]] - age[el2[,2]])*(sex[el2[,1]] == 0 & sex[el2[,2]] == 0)),
                 sum(abs(age[el2[,1]] - age[el2[,2]])*(((sex[el2[,1]] == 0) + (sex[el2[,2]] == 0)) == 1)),
                 sum(abs(age[el2[,1]] - age[el2[,2]])*(sex[el2[,1]] == 1 & sex[el2[,2]] == 1))),
               check.attributes = FALSE)
})

test_that("EpiModel terms produce correct change statistics", {
  skip_on_cran()
  nw1 <- simulate(nw ~ edges + offset(nodematch("sex")),
                  coef = c(-3, -Inf),
                  monitor = ~absdiffby("age", "sex", 2.3))
  
  stats1 <- attr(nw1, "stats")
  
  el1 <- as.edgelist(nw1)
  
  expect_equal(stats1[-c(1,2)],
               sum(abs(age[el1[,1]] - 2.3*sex[el1[,1]] - age[el1[,2]] + 2.3*sex[el1[,2]])),
               check.attributes = FALSE)
  
  nw2 <- simulate(nw ~ edges,
                  coef = c(-3),
                  monitor = ~absdiffnodemix("age", "sex"))
  
  stats2 <- attr(nw2, "stats")
  
  el2 <- as.edgelist(nw2)

  expect_equal(stats2[-c(1)],
               c(sum(abs(age[el2[,1]] - age[el2[,2]])*(sex[el2[,1]] == 0 & sex[el2[,2]] == 0)),
                 sum(abs(age[el2[,1]] - age[el2[,2]])*(((sex[el2[,1]] == 0) + (sex[el2[,2]] == 0)) == 1)),
                 sum(abs(age[el2[,1]] - age[el2[,2]])*(sex[el2[,1]] == 1 & sex[el2[,2]] == 1))),
               check.attributes = FALSE)
})

context("fuzzynodematch Term")

test_that("fuzzynodematch works as intended", {
  skip_on_cran()
  n <- 1000L
  bip <- 400L
  nv <- 10L
  nvm <- 2000L
  prob <- 0.1

  for (directed in list(FALSE, TRUE)) {
    for (bipartite in list(FALSE, bip)) {
      for (split in list("|", ".")) {
        for (binary in list(FALSE, TRUE)) {
          if (directed == TRUE && bipartite == bip) {
            next
          }

          vids <- as.character(sample(seq_len(nvm), nv, FALSE))
          vids <- paste0(rep(c("a","b",""), length.out = length(vids)), vids)

          vcs <- matrix(as.logical(rbinom(n*nv, 1L, prob)), nrow = n)

          duplicate <- sample(c(FALSE, TRUE), n, TRUE)

          attr <- character(n)
          for (i in seq_along(attr)) {
            charvec <- vids[vcs[i,]]
            if (duplicate[i] == TRUE) {
              charvec <- c(charvec, sample(charvec, length(charvec), TRUE))
            }
            charvec <- sample(charvec)
            attr[i] <- paste(charvec, collapse = split)
          }

          nw <- network.initialize(n, directed = directed, bipartite = bipartite)
          nw %v% "attr" <- attr

          el <- as.edgelist(san(nw ~ edges, target = c(n)))

          toggles <- rbind(el, el)
          toggles <- toggles[sample(seq_len(NROW(toggles))), , drop = FALSE]
          toggles <- cbind(seq_len(NROW(toggles)), toggles)

          changes <- cbind(toggles, 1L)
          for (i in seq_len(NROW(changes))) {
            if(min(which(changes[,2L] == changes[i,2L] & changes[,3L] == changes[i,3L])) < i) {
              changes[i,4L] <- 0L
            }
          }

          gf_stats <- tergm.godfather(nw ~ fuzzynodematch(~attr, split = split, binary = binary), toggles = toggles, stats.start = TRUE)

          manual_stats <- integer(NROW(toggles))
          for (i in seq_along(manual_stats)) {
            manual_stats[i] <- sum(vcs[toggles[i,2L],]*vcs[toggles[i,3L],])
            if (binary == TRUE) {
              manual_stats[i] <- as.integer(manual_stats[i] > 0)
            }
            if (changes[i,4L] == 0L) {
              manual_stats[i] <- -manual_stats[i]
            }
          }
          manual_stats <- cumsum(c(0L, manual_stats))

          expect_identical(manual_stats, as.integer(gf_stats))
        }
      }
    }
  }
})

#' @title Functions to Access and Edit the Main netsim_dat Object in Network Models
#'
#' @description These `get_`, `set_`, `append_`, and `add_`
#'              functions allow a safe and efficient way to retrieve and mutate
#'              the main `netsim_dat` class object of network models
#'              (typical variable name `dat`). They are intended for use
#'              *inside module functions* that run during a [netsim()]
#'              simulation, not for editing the user-facing `param.net`,
#'              `init.net`, or `control.net` inputs before a run or the
#'              output object returned by [netsim()]. See the **Intended
#'              Usage** section below for details and alternatives.
#'
#' @inheritParams recovery.net
#' @param item A character vector containing the name of the element to access
#'        (for `get_` functions), create (for `add_` functions), or
#'        edit (for `set_` and `append_` functions). Can be of length greater
#'        than 1 for `get_*_list` functions.
#' @param posit_ids For `set_attr` and `get_attr`, a numeric vector of
#'        posit_ids to subset the desired `item`.
#' @param value New value to be attributed in the `set_` and `append_`
#'        functions.
#' @param override.null.error If TRUE, `get_` will return NULL if the
#'         `item` does not exist instead of throwing an error.
#'         (default = FALSE).
#' @param override.length.check If TRUE, `set_attr` allows the modification
#'        of the `item` size. (default = FALSE).
#' @param n.new For `append_core_attr`, the number of new nodes to initiate
#'        with core attributes; for `append_attr`, the number of new
#'        elements to append at the end of `item`.
#' @param at For `get_epi`, the timestep at which to access the specified
#'        `item`; for `set_epi`, the timestep at which to add the new
#'        value for the epi output `item`; for `append_core_attr`, the
#'        current time step.
#' @return A vector or a list of vectors for `get_` functions; the main
#'         list object for `set_`, `append_`, and `add_`
#'         functions.
#'
#' @section Intended Usage:
#' These accessors operate on the live `netsim_dat` object that EpiModel
#' constructs internally when a simulation starts and passes through each
#' module at every time step. The expected calling context is **inside a
#' user-supplied module function** (e.g., a custom `infection.FUN`,
#' `recovery.FUN`, or arrivals/departures module) registered with
#' [control.net()].
#'
#' They are *not* a general-purpose tool for editing the user-facing input
#' or output objects of [netsim()]:
#'
#'  * Calling them on a [param.net()], [init.net()], or [control.net()]
#'    object before a run will appear to succeed (those objects are also
#'    list-like) but produces an object that is not a valid simulation
#'    input.
#'  * Calling them on the object returned by [netsim()] modifies only the
#'    saved input slot, not anything that would change the result of a
#'    re-run. Note also that [netsim()] strips the `*.FUN` entries from
#'    its returned `control` slot, so feeding that slot back into a new
#'    simulation will fail.
#'
#' For modifications outside a running simulation, use the following
#' instead:
#'
#'  * **Editing parameters before a run:** [update_params()] for a
#'    `param.net` object, or direct list assignment
#'    (e.g., `p$inf.prob <- 0.5`).
#'  * **Editing init or control before a run:** direct list assignment
#'    (e.g., `ctrl$nsteps <- 1000`), or rebuild with a fresh call to the
#'    [init.net()] / [control.net()] constructor.
#'  * **Scheduled mid-simulation changes:** the `.param.updater.list`
#'    argument to [param.net()] and the `.control.updater.list` argument
#'    to [control.net()]. See the "model-parameters" vignette for the
#'    underlying updater module and the scenario API built on top of it.
#'
#' @section Core Attribute:
#' The `append_core_attr` function initializes the attributes necessary for
#' EpiModel to work (the four core attributes are: "active", "unique_id",
#' "entrTime", and "exitTime"). These attributes are used in the initialization
#' phase of the simulation, to create the nodes (see
#' [initialize.net()]); and also used when adding nodes during the
#' simulation (see [arrivals.net()]).
#'
#' @section Mutability:
#' The `set_`, `append_`, and `add_` functions DO NOT modify the
#' `netsim_dat` object in place. The result must be assigned back to
#' `dat` in order to be registered: `dat <- set_*(dat, item, value)`.
#'
#' @section `set_` and `append_` vs `add_`:
#' The `set_` and `append_` functions edit a pre-existing element or
#' create a new one if it does not exist already by calling the `add_`
#' functions internally.
#'
#' @examples
#' dat <- create_dat_object(control = list(nsteps = 150))
#' dat <- append_core_attr(dat, 1, 100)
#'
#' dat <- add_attr(dat, "age")
#' dat <- set_attr(dat, "age", runif(100))
#' dat <- set_attr(dat, "status", rbinom(100, 1, 0.9))
#' dat <- append_attr(dat, "status", 1, 10)
#' dat <- append_attr(dat, "age", NA, 10)
#' get_attr_list(dat)
#' get_attr_list(dat, c("age", "active"))
#' get_attr(dat, "status")
#' get_attr(dat, "status", c(1, 4))
#'
#' dat <- add_epi(dat, "i.num")
#' dat <- set_epi(dat, "i.num", 150, 10)
#' dat <- set_epi(dat, "s.num", 150, 90)
#' get_epi_list(dat)
#' get_epi_list(dat, c("i.num", "s.num"))
#' get_epi(dat, "i.num")
#' get_epi(dat, "i.num", c(1, 4))
#'
#' dat <- add_param(dat, "x")
#' dat <- set_param(dat, "x", 0.4)
#' dat <- set_param(dat, "y", 0.8)
#' get_param_list(dat)
#' get_param_list(dat, c("x", "y"))
#' get_param(dat, "x")
#'
#' dat <- add_init(dat, "x")
#' dat <- set_init(dat, "x", 0.4)
#' dat <- set_init(dat, "y", 0.8)
#' get_init_list(dat)
#' get_init_list(dat, c("x", "y"))
#' get_init(dat, "x")
#'
#' dat <- add_control(dat, "x")
#' dat <- set_control(dat, "x", 0.4)
#' dat <- set_control(dat, "y", 0.8)
#' get_control_list(dat)
#' get_control_list(dat, c("x", "y"))
#' get_control(dat, "x")
#'
#' @seealso [update_params()] for editing a `param.net` object outside a
#'   simulation; [param.net()], [init.net()], [control.net()] for input
#'   constructors; [netsim()] for running a simulation.
#'
#' @name net-accessor
NULL

raw_get_attr_list <- function(dat) return(dat$run$attr)
raw_set_attr_list <- function(dat, attr_list) {
  dat$run$attr <- attr_list
  return(dat)
}
raw_set_attr <- function(dat, item, val) {
  dat$run$attr[[item]] <- val
  return(dat)
}

#' @rdname net-accessor
#' @export
get_attr_list <- function(dat, item = NULL) {
  attr_list <- raw_get_attr_list(dat)
  if (is.null(item)) {
    return(attr_list)
  }

  missing_item <- setdiff(item, names(attr_list))
  if (length(missing_item) > 0L) {
    stop("There are no attributes called `",
         paste(missing_item, collapse = ", "),
         "` in the attributes list of the main list object (dat)")
  }

  out <- attr_list[item]

  return(out)
}

#' @rdname net-accessor
#' @export
get_attr <- function(dat, item, posit_ids = NULL, override.null.error = FALSE) {
  attr_list <- raw_get_attr_list(dat)

  if (!item %in% names(attr_list)) {
    if (!override.null.error) {
      stop("There is no attribute called `", item,
           "` in the attributes list of the main list object (dat)")
    }
    return(NULL)
  }

  if (is.null(posit_ids)) {
    return(attr_list[[item]])
  }

  assert_valid_posit_ids(dat, posit_ids)
  attr <- attr_list[[item]]
  return(attr[posit_ids])
}

#' @rdname net-accessor
#' @export
add_attr <- function(dat, item) {
  attr_list <- get_attr_list(dat)
  if (item %in% names(attr_list)) {
    stop("Cannot create the attribute '", item, "': exists already")
  }
  dat <- raw_set_attr(dat, item, rep(NA, length(attr_list$active)))
  return(dat)
}

#' @rdname net-accessor
#' @export
set_attr <- function(dat, item, value, posit_ids = NULL,
                     override.length.check = FALSE) {
  attr_list <- raw_get_attr_list(dat)
  if (!item %in% names(attr_list)) {
    dat <- add_attr(dat, item)
  }

  if (is.null(posit_ids)) {
    if (!override.length.check &&
          length(value) != length(attr_list$active)) {
      stop(
        "When trying to edit the ", `item`, " nodal attribute: ",
        "The size of the `value` vector is not equal to the number of nodes in",
        " the network. \n",
        "Expected: ", length(attr_list$active), "\n",
        "Given: ", length(value)
      )
    }
    dat <- raw_set_attr(dat, item, value)
  } else {
    assert_valid_posit_ids(dat, posit_ids)
    if (length(value) != 1L && length(value) != length(posit_ids)) {
      stop(
        "When trying to edit the `", item, "` nodal attribute: ",
        "The size of the `value` vector is not equal to the number of nodes ",
        "selected by the `posit_ids` vector nor of length 1. \n",
        "Expected: ", length(attr_list$active[posit_ids]), " or 1 \n",
        "Given: ", length(value)
      )
    }
    attr <- attr_list[[item]]
    attr[posit_ids] <- value
    dat <- raw_set_attr(dat, item, attr)
  }

  return(dat)
}

#' @rdname net-accessor
#' @export
append_attr <- function(dat, item, value, n.new) {
  if (!is.numeric(n.new) || n.new < 0L) {
    stop("`n.new` must be numeric and greater than or equal to zero.")
  }

  if (length(value) == 1L) {
    new_vals <- rep(value, n.new)
  } else if (length(value) == n.new) {
    new_vals <- value
  } else {
    stop("`value` must be of length one or `n.new`.")
  }

  old_vals <- get_attr(dat, item, override.null.error = TRUE)
  dat <- set_attr(dat, item, c(old_vals, new_vals),
                  override.length.check = TRUE)

  return(dat)
}

#' @rdname net-accessor
#' @export
remove_node_attr <- function(dat, posit_ids) {
  if (is.null(posit_ids)) {
    return(dat)
  }
  assert_valid_posit_ids(dat, posit_ids)
  attr_list <- raw_get_attr_list(dat)
  attr_list <- lapply(attr_list, function(x) x[-posit_ids])
  dat <- raw_set_attr_list(dat, attr_list)
  return(dat)
}

#' @rdname net-accessor
#' @export
get_epi_list <- function(dat, item = NULL) {
  if (is.null(item)) {
    out <- dat$epi
  } else {
    missing_item <- setdiff(item, names(dat$epi))
    if (length(missing_item) > 0L) {
      stop("There is no epi output called `",
           paste(missing_item, collapse = ", "),
           "` in the epi output list of the main list object (dat)")
    }
    out <- dat$epi[item]
  }
  return(out)
}

#' @rdname net-accessor
#' @export
get_epi <- function(dat, item, at = NULL, override.null.error = FALSE) {
  if (!item %in% names(dat$epi)) {
    if (!override.null.error) {
      stop("There is no epi output called `", item,
           "` in the epi output list of the main list object (dat)")
    }
    return(NULL)
  }

  if (is.null(at)) {
    return(dat$epi[[item]])
  }

  assert_valid_time_steps(dat, at)
  return(dat$epi[[item]][at])
}

#' @rdname net-accessor
#' @export
add_epi <- function(dat, item) {
  if (item %in% names(dat$epi)) {
    stop("Cannot create the epi output, ", item, ": exists already")
  }
  dat$epi[[item]] <- padded_vector(NA_integer_, get_control(dat, "nsteps"))
  return(dat)
}

#' @rdname net-accessor
#' @export
set_epi <- function(dat, item, at,  value) {
  if (length(at) != 1L || !is.numeric(at)) {
    stop("`at` must be numeric and of length one")
  }

  if (!item %in% names(dat$epi)) {
    dat <- add_epi(dat, item)
  }

  # ensure that the vector is of size `nsteps`, right padded with NA
  nsteps <- get_control(dat, "nsteps")
  if (at > length(dat$epi[[item]])) {
    dat$epi[[item]] <- padded_vector(dat$epi[[item]], nsteps)
  }

  dat$epi[[item]][at] <- value
  return(dat)
}

#' @rdname net-accessor
#' @export
get_param_list <- function(dat, item = NULL) {
  if (is.null(item)) {
    return(dat$param)
  }

  missing_item <- setdiff(item, names(dat$param))
  if (length(missing_item) > 0L) {
    stop("There are no parameters called `",
         paste(missing_item, collapse = ", "),
         "` in the parameter list of the main list object (dat)")
  }
  return(dat$param[item])
}

#' @rdname net-accessor
#' @export
get_param <- function(dat, item, override.null.error = FALSE) {
  if (!item %in% names(dat$param)) {
    if (!override.null.error) {
      stop("There is no parameter called `", item,
           "` in the parameter list of the main list object (dat)")
    }
    return(NULL)
  }

  return(dat$param[[item]])
}

#' @rdname net-accessor
#' @export
add_param <- function(dat, item) {
  if (item %in% names(dat$param)) {
    stop("Cannot create the parameter, ", item, ": exists already")
  }
  dat$param[[item]] <- NA
  return(dat)
}

#' @rdname net-accessor
#' @export
set_param <- function(dat, item, value) {
  if (!item %in% names(dat$param)) {
    dat <- add_param(dat, item)
  }
  dat$param[[item]] <- value
  return(dat)
}

#' @rdname net-accessor
#' @export
get_control_list <- function(dat, item = NULL) {
  if (is.null(item)) {
    return(dat$control)
  }

  missing_item <- setdiff(item, names(dat$control))
  if (length(missing_item) > 0L) {
    stop("There is no control value called `",
         paste(missing_item, collapse = ", "),
         "` in the control list of the main list object (dat)")
  }

  return(dat$control[item])
}

#' @rdname net-accessor
#' @export
get_control <- function(dat, item, override.null.error = FALSE) {
  if (!item %in% names(dat$control)) {
    if (!override.null.error) {
      stop("There is no control value called `", item,
           "` in the control list of the main list object (dat)")
    }
    return(NULL)
  }
  return(dat$control[[item]])
}

#' @rdname net-accessor
#' @param network index of network for which to get control
#' @export
get_network_control <- function(dat, network, item, override.null.error = FALSE) {
  if (!item %in% names(dat$control)) {
    if (!override.null.error) {
      stop("There is no control value called `", item,
           "` in the control list of the main list object (dat)")
    }
    return(NULL)
  }
  out <- dat$control[[item]]

  if (!is(out, "multilayer")) {
    stop("Control value `", item, "` accessed through `get_network_control` ",
         "is not of class `multilayer`.")
  }
  return(out[[network]])
}

#' @rdname net-accessor
#' @export
add_control <- function(dat, item) {
  if (item %in% names(dat$control)) {
    stop("Cannot create the control value, ", item,
         ": exists already")
  }
  dat$control[[item]] <- NA
  return(dat)
}

#' @rdname net-accessor
#' @export
set_control <- function(dat, item, value) {
  if (!item %in% names(dat$control)) {
    dat <- add_control(dat, item)
  }
  dat$control[[item]] <- value
  return(dat)
}

#' @rdname net-accessor
#' @export
get_init_list <- function(dat, item = NULL) {
  if (is.null(item)) {
    return(dat$init)
  }

  missing_item <- setdiff(item, names(dat$init))
  if (length(missing_item) > 0L) {
    stop("There is no init value called `",
         paste(missing_item, collapse = ", "),
         "` in the init list of the main list object (dat)")
  }
  return(dat$init[item])
}

#' @rdname net-accessor
#' @export
get_init <- function(dat, item, override.null.error = FALSE) {
  if (!item %in% names(dat$init)) {
    if (!override.null.error) {
      stop("There is no init value called `", item,
           "` in the init list of the main list object (dat)")
    }
    return(NULL)
  }
  return(dat$init[[item]])
}

#' @rdname net-accessor
#' @export
add_init <- function(dat, item) {
  if (item %in% names(dat$init)) {
    stop("Cannot create the init value, ", item,
         ": exists already")
  }
  dat$init[[item]] <- NA
  return(dat)
}

#' @rdname net-accessor
#' @export
set_init <- function(dat, item, value) {
  if (!item %in% names(dat$init)) {
    dat <- add_init(dat, item)
  }
  dat$init[[item]] <- value
  return(dat)
}

# Core Attributes --------------------------------------------------------------

#' Return the List of Core EpiModel Attributes with Correct Types
#'
#' This function returns an exhaustive named list of the attributes managed by
#' EpiModel itself. It can be used to check the validity of an attributes list
#' and of its types.
#'
#' @rdname net-accessor
#' @export
get_core_attributes <- function() {
  list(
    active = 1L,
    entrTime = 1L,
    exitTime =  NA_integer_,
    unique_id = 1L
  )
}

#' @rdname net-accessor
#' @export
append_core_attr <- function(dat, at, n.new) {
  dat <- append_attr(dat, "active", 1L, n.new)
  dat <- append_attr(dat, "entrTime", as.integer(at), n.new)
  dat <- append_attr(dat, "exitTime", NA_integer_, n.new)
  dat <- update_unique_ids(dat, n.new)
  return(dat)
}

#' @title Create the Unique Identifiers for New Nodes
#'
#' @description This function is called by [append_core_attr()] and
#' appends new `unique_ids` to the created nodes. It also keeps track of
#' the already used `unique_ids` with the `dat$run$last_unique_id`
#' variable.
#'
#' @inheritParams recovery.net
#' @param n.new The number of new nodes to give `unique_ids` to.
#'
#' @inherit recovery.net return
#'
#' @keywords internal
update_unique_ids <- function(dat, n.new) {
  last_unique_id <- dat$run$last_unique_id
  next_unique_ids <- seq_len(n.new) + last_unique_id
  dat$run$last_unique_id <- last_unique_id + as.integer(n.new)
  dat <- append_attr(dat, "unique_id", next_unique_ids, n.new)
  return(dat)
}

#' @title Check that All Attributes in the Main `netsim_dat` Object are of Equal
#'        Length
#'
#' @inheritParams recovery.net
#'
#' @return invisible(TRUE) if everything is correct; an error if not.
#'
#' @keywords internal not_used
check_attr_lengths <- function(dat) {
  attr_list <- raw_get_attr_list(dat)
  attr_lengths <- vapply(attr_list, length, integer(1))
  expected_length <- attr_lengths$active
  wrong_lengths <- which(attr_lengths != expected_length)

  if (length(wrong_lengths > 0L)) {
    msg <- c(
      "Some attributes are not of the correct length \n",
      "Expected length: ", expected_length, "\n",
      "Wrong length attributes: \n"
    )

    for (i in seq_along(wrong_lengths)) {
      msg <- c(msg, "`", names(wrong_lengths)[i], "`: ", wrong_lengths[i], "\n")
    }

    stop(msg)
  }

  return(invisible(TRUE))
}

# Unique / Positional identifier converters ------------------------------------

#' @title Convert Unique Identifiers to/from Positional Identifiers
#'
#' @description EpiModel refers to its nodes either by positional identifiers
#'              (`posit_ids`), which describe the position of a node in the
#'              `attr` vector, or by unique identifiers
#'              (`unique_ids`), which allow references to nodes even after
#'              they are deactivated.
#'
#' @section All elements:
#'   When `unique_ids` or `posit_ids` is NULL (default)
#'   the full list of positional IDs or unique IDs is returned.
#'
#' @section Deactivated nodes:
#'   When providing `unique_ids` of deactivated nodes to
#'   `get_posit_ids`, `NA`s are returned instead and a warning is
#'   produced.
#'
#' @inheritParams recovery.net
#' @param unique_ids A vector of node unique identifiers (default = NULL).
#' @param posit_ids A vector of node positional identifiers (default = NULL).
#' @return A vector of unique or positional identifiers.
#'
#' @name unique_id-tools
NULL

#' @rdname unique_id-tools
#' @export
get_unique_ids <- function(dat, posit_ids = NULL) {
  if (is.null(posit_ids)) {
    return(get_attr(dat, "unique_id"))
  }
  unique_ids <- get_attr(dat, "unique_id", posit_ids = posit_ids)
  return(unique_ids)
}

#' @rdname unique_id-tools
#' @export
get_posit_ids <- function(dat, unique_ids = NULL) {
  if (is.null(unique_ids)) {
    return(seq_along(get_attr(dat, "active")))
  }
  posit_ids <- base::match(unique_ids, get_attr(dat, "unique_id"))
  if (any(is.na(posit_ids))) {
    warning(
      "While converting `unique_ids` to `posit_ids`, some `unique_ids`",
      " correspond to deactivated nodes and NAs were produced"
    )
  }
  return(posit_ids)
}

#' @title Are These Nodes Active (Positional IDs)
#'
#' @inheritParams recovery.net
#' @param posit_ids A vector of node positional identifiers.
#'
#' @return A logical vector with TRUE if the node is still active and FALSE
#' otherwise.
#'
#' @export
is_active_posit_ids <- function(dat, posit_ids) {
  active <- get_attr(dat, "active")
  return(active[posit_ids] %in% 1L)
}

#' @title Are These Nodes Active (Unique IDs)
#'
#' @inheritParams recovery.net
#' @param unique_ids A vector of node unique identifiers.
#'
#' @return A logical vector with TRUE if the node is still active and FALSE
#' otherwise.
#'
#' @export
is_active_unique_ids <- function(dat, unique_ids) {
  suppressWarnings({
    posit_ids <- get_posit_ids(dat, unique_ids)
  })
  return(is_active_posit_ids(dat, posit_ids))
}

#' @title Grow a Vector to a Given Size, Padding it With Empty Elements
#'
#' @description Grow a vector to a given size, padding it with `NULL` if `orig` is a `list`
#' and with `NA` otherwise
#'
#' @param orig A vector to grow.
#' @param size The final size of the vector.
#'
#' @return
#' A vector of size `size` padded with `NULL`s or `NA`s at the end.
#'
#' @export
padded_vector <- function(orig, size) {
  if (is.list(orig)) {
    out <- c(orig, vector(mode = "list", size - length(orig)))
  } else {
    out <- c(orig, rep(NA, size - length(orig)))
  }
  return(out)
}

#' Check if `posit_ids` are corrects (numeric and <= NNodes)
#' @noRd
assert_valid_posit_ids <- function(dat, posit_ids) {
  if (!is.numeric(posit_ids))
    stop("`posit_ids` must be `numeric`")
  if (any(posit_ids > length(get_attr(dat, "active"))))
    stop("Some `posit_ids` are larger than the number of nodes ",
         " in the network")
  invisible(posit_ids)
}

#' Check if `time_steps` are corrects (numeric and <= nsteps)
#' @noRd
assert_valid_time_steps <- function(dat, time_steps) {
  if (!is.numeric(time_steps))
    stop("time steps must be `numeric`")
  if ((any(time_steps > get_control(dat, "nsteps"))))
    stop("Some time steps are larger than the number of
          steps planned for for the simulation (control$nsteps)")
  invisible(time_steps)
}

context("Accessors: Getters and Setters")

test_that("`dat` getters and setter", {

  n_nodes <- 100
  dat <- create_dat_object(control = list(nsteps = 150))
  dat <- append_core_attr(dat, at = 1,  n.new = 100)

  ## Attr tests
  dat <- add_attr(dat, "age")
  expect_equal(get_attr(dat, "age"), rep(NA, n_nodes))
  expect_error(dat <- add_attr(dat, "age"))

  expect_error(set_attr(dat, "age", 4))

  new_ages <- runif(n_nodes)
  dat <- set_attr(dat, "age", new_ages)
  expect_equal(get_attr(dat, "age"), new_ages)

  dat <- set_attr(dat, "age2", new_ages)
  expect_silent(dat <- set_attr(dat, "age2", rep(new_ages, 2),
                                override.length.check = TRUE))
  expect_length(get_attr(dat, "age2"), 2 * length(new_ages))


  expect_equal(get_attr(dat, "age"), new_ages)
  expect_equal(get_attr(dat, "age", c(1, 5)), new_ages[c(1, 5)])

  expect_error(get_attr(dat, "age_absent"))
  expect_null(get_attr(dat, "age_absent", override.null.error = TRUE))
  expect_error(get_attr(dat, "age", c(1, 1000)))
  expect_error(get_attr(dat, "age", c(TRUE, FALSE)))

  expect_silent(dat <- set_attr(dat, "status", rbinom(n_nodes, 1, 0.4)))

  expect_silent(dat <- set_attr(dat, "age", 2, posit_ids = 1:4))
  expect_equal(get_attr(dat, "age", 1:4), rep(2, 4))

  expect_error(dat <- set_attr(dat, "age", c(1, 2), posit_ids = 1:4))
  expect_error(dat <- set_attr(dat, "age", 1, posit_ids = c(1, 1000)))

  expect_error(dat <- set_attr(dat, "age", 1, posit_ids = TRUE))
  expect_error(dat <- set_attr(dat, "age", 1, posit_ids = "a"))


  expect_error(get_attr_list(dat, "sex"))

  expect_error(dat <- append_attr(dat, "status", rbinom(n_nodes, 10)))

  expect_error(dat <- append_attr(dat, "status", 1, -1))

  dat <- append_attr(dat, "active", 1, 10)
  expect_length(get_attr(dat, "active"), 110)

  dat <- append_attr(dat, "status", sample(0:1, 10, TRUE), 10)
  expect_length(get_attr(dat, "status"), 110)

  ## Epi tests
  dat <- add_epi(dat, "i")
  expect_equal(dat$epi$i, rep(NA_integer_, get_control(dat, "nsteps")))

  expect_error(set_epi(dat, "i", c(1, 4), 4))

  dat <- set_epi(dat, "i", 150, 10)
  expect_equal(get_epi(dat, "i")[150], 10)

  dat <- set_epi(dat, "s", 110, 10)
  expect_equal(get_epi(dat, "s")[110], 10)

  expect_equal(get_epi(dat, "i", c(1, 100)), dat$epi$i[c(1, 100)])

  expect_error(get_epi(dat, "age_absent"))
  expect_null(get_epi(dat, "age_absent", override.null.error = TRUE))
  expect_error(get_epi(dat, "i", c(1, 300)))
  expect_error(get_epi(dat, "i", c(TRUE, FALSE)))

  dat$control$nsteps <- 200
  dat <- set_epi(dat, "i", 160, 8)
  expect_length(dat$epi$i, 200)

  expect_equal(get_epi_list(dat, c("i", "s")), get_epi_list(dat)[c("i", "s")])

  expect_error(get_epi_list(dat, "r"))

  # param, init, control tests
  dat <- add_param(dat, "x")

  dat <- add_init(dat, "x")
  dat <- add_control(dat, "x")

  expect_equal(dat$param$x, NA)
  expect_equal(dat$init$x, NA)
  expect_equal(dat$control$x, NA)

  expect_silent(dat <- set_param(dat, "y", 4))
  expect_silent(dat <- set_init(dat, "y", 4))
  expect_silent(dat <- set_control(dat, "y", 4))

  dat <- set_param(dat, "y", 5)
  dat <- set_init(dat, "y", 5)
  dat <- set_control(dat, "y", 5)

  expect_equal(get_param(dat, "y"), 5)
  expect_equal(get_init(dat, "y"), 5)
  expect_equal(get_control(dat, "y"), 5)

  expect_error(get_param(dat, "z"))
  expect_error(get_init(dat, "z"))
  expect_error(get_control(dat, "z"))

  expect_null(get_param(dat, "z", override.null.error = TRUE))
  expect_null(get_init(dat, "z", override.null.error = TRUE))
  expect_null(get_control(dat, "z", override.null.error = TRUE))

  expect_equal(get_param_list(dat), dat$param)
  expect_equal(get_init_list(dat), dat$init)
  expect_equal(get_control_list(dat), dat$control)

  expect_equal(get_param_list(dat, "x"), get_param_list(dat)["x"])
  expect_equal(get_init_list(dat, "x"), get_init_list(dat)["x"])
  expect_equal(get_control_list(dat, "x"), get_control_list(dat)["x"])

  expect_error(get_param_list(dat, "z"))
  expect_error(get_init_list(dat, "z"))
  expect_error(get_control_list(dat, "z"))

})

test_that("Net core attributes", {

  dat <- create_dat_object(control = list(nsteps = 150))

  # Append the first nodes (empty list before)
  dat <- append_core_attr(dat, at = 1,  n.new = 100)
  expect_equal(get_attr(dat, "active"), rep(1, 100))
  expect_equal(get_attr(dat, "unique_id"), 1:100)

  # Remove some nodes to check if unique_ids are unique
  dat <- delete_attr(dat, 21:30)
  dat <- append_core_attr(dat, at = 2, n.new = 100)
  expect_equal(get_attr(dat, "active"), rep(1, 190))
  expect_equal(get_attr(dat, "unique_id"), c(1:20, 31:200))
  expect_type(get_attr(dat, "unique_id"), "integer")

  # Test unique_ids posit_ids converters
  expect_equal(
    get_attr(dat, "unique_id"),
    get_unique_ids(dat, seq_along(get_attr(dat, "active")))
  )
  expect_equal(
    seq_along(get_attr(dat, "active")),
    get_posit_ids(dat, get_attr(dat, "unique_id"))
  )
  expect_warning(get_posit_ids(dat, 25:35))
})

context("Accessors: Attribute Copying")

################################################################################

test_that("Copying attributes from network to attribute list", {
  skip_on_cran()

  num1 <- num2 <- 500
  nw <- network_initialize(num1 + num2)
  nw <- set_vertex_attribute(nw, "group", rep(1:2, each = num1))
  nw <- set_vertex_attribute(nw, "race", sample(c("B", "W"), num1 + num2,
                                                replace = TRUE))
  nw <- set_vertex_attribute(nw, "region", sample(1:4, num1 + num2,
                                                  replace = TRUE))
  formation <- ~edges + nodematch("group")
  target.stats <- c(400, 0)
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 25)
  est <- netest(nw, formation, target.stats, coef.diss)

  param <- param.net(inf.prob = 0.1, inf.prob.g2 = 0.2,
                     act.rate = 5)
  init <- init.net(i.num = 50, i.num.g2 = 50)
  control <- control.net(type = "SI", nsteps = 10, nsims = 2, tergmLite = FALSE,
                         raw.output = TRUE, verbose = FALSE)

  sim <- netsim(est, param, init, control)

  for (simno in c(1, 2)) {
    dat <- sim[[simno]]
    # Character attribute
    dat.attr <- prop.table(table(get_attr(dat, "race")))
    nw.attr <- prop.table(table(get_vertex_attribute(dat$run$nw[[1]], "race")))

    expect_equal(dat.attr, nw.attr)

    # Numeric attribute
    dat.attr <- prop.table(table(get_attr(dat, "region")))
    nw.attr <- prop.table(table(get_vertex_attribute(dat$run$nw[[1]], "region")))

    expect_equal(dat.attr, nw.attr)
  }
})

context("New Network Models")

test_that("New network models vignette example", {
  skip_on_cran()

  ## New Aging Module
  aging <- function(dat, at) {

    age <- get_attr(dat, "age", override.null.error = TRUE)
    if (is.null(age)) {
      active <- get_attr(dat, "active")
      n <- sum(active == 1)
      age <- sample(18:49, n, replace = TRUE)
    } else {
      age <- get_attr(dat, "age") + 1 / 12
    }
    dat <- set_attr(dat, "age", age)

    return(dat)
  }


  ## Replacement Departure Module
  dfunc <- function(dat, at) {
    active <- get_attr(dat, "active")
    exitTime <- get_attr(dat, "exitTime")
    idsElig <- which(active == 1)
    nElig <- length(idsElig)

    nDepartures <- 0

    if (nElig > 0) {
      ages <- get_attr(dat, "age")[idsElig]
      life.expt <- get_param(dat, "life.expt")
      departure.rates <- pmin(1, 1 / (life.expt * 12 - ages * 12))
      vecDepartures <- which(rbinom(nElig, 1, departure.rates) == 1)
      idsDepartures <- idsElig[vecDepartures]
      nDepartures <- length(idsDepartures)
      if (nDepartures > 0) {
        active[idsDepartures] <- 0
        exitTime[idsDepartures] <- at
        dat <- set_attr(dat, "active", active)
        dat <- set_attr(dat, "exitTime", exitTime)
      }
    }

    # Output
    dat <- set_epi(dat, "d.flow", at, nDepartures)
    return(dat)
  }


  ## Replacement Arrival Module
  afunc <- function(dat, at) {

    # Variables
    growth.rate <- get_param(dat, "growth.rate")
    exptPopSize <- get_epi(dat, "num", 1) * (1 + growth.rate * at)
    active <- get_attr(dat, "active")
    numNeeded <- exptPopSize - sum(active == 1)

    if (numNeeded > 0) {
      nArrivals <- rpois(1, numNeeded)
    } else {
      nArrivals <- 0
    }

    dat <- append_core_attr(dat, at, nArrivals)
    dat <- append_attr(dat, "status", "s", nArrivals)
    dat <- append_attr(dat, "infTime", NA, nArrivals)
    dat <- append_attr(dat, "age", 0, nArrivals)

    # Output
    dat <- set_epi(dat, "a.flow", at, nArrivals)

    return(dat)
  }


  ## Network Model
  nw <- network.initialize(50, directed = FALSE)
  est <- netest(nw, formation = ~edges, target.stats = 15,
                coef.diss = dissolution_coefs(~offset(edges), 60, 0.000274),
                verbose = FALSE)


  ## EpiModel
  param <- param.net(inf.prob = 0.35, growth.rate = 0.00083, life.expt = 70)
  init <- init.net(i.num = 10)
  control <- control.net(type = NULL, nsims = 1, nsteps = 5,
                         departures.FUN = dfunc,
                         arrivals.FUN = afunc, aging.FUN = aging,
                         infection.FUN = infection.net,
                         tergmLite = FALSE, resimulate.network = TRUE, verbose = FALSE)
  mod1 <- netsim(est, param, init, control)
  capture_output(
    mod1
  )

  expect_is(mod1, "netsim")
  expect_output(print(mod1), "resim_nets.FUN")
  expect_output(print(mod1), "infection.FUN")
  expect_output(print(mod1), "departures.FUN")
  expect_output(print(mod1), "arrivals.FUN")
  expect_output(print(mod1), "aging.FUN")

  ## Test module reordering
  control <- control.net(type = NULL, nsims = 1, nsteps = 10,
                         departures.FUN = dfunc,
                         arrivals.FUN = afunc, aging.FUN = aging,
                         infection.FUN = infection.net,
                         module.order = c("resim_nets.FUN", "infection.FUN",
                                          "aging.FUN", "arrivals.FUN",
                                          "departures.FUN", "prevalence.FUN"),
                         tergmLite = FALSE, resimulate.network = TRUE, verbose = FALSE)
  mod2 <- netsim(est, param, init, control)
  expect_is(mod2, "netsim")

  ### tergmLite replication
  param <- param.net(inf.prob = 0.35, growth.rate = 0.00083, life.expt = 70)
  init <- init.net(i.num = 10)
  control <- control.net(type = NULL, nsims = 1, nsteps = 10,
                         infection.FUN = infection.net,
                         departures.FUN = dfunc,
                         arrivals.FUN = afunc, aging.FUN = aging,
                         tergmLite = TRUE, verbose = FALSE,
                         resimulate.network = TRUE)
  mod3 <- netsim(est, param, init, control)
  expect_is(mod3, "netsim")

  ## Test module reordering
  control <- control.net(type = NULL, nsims = 1, nsteps = 10,
                         departures.FUN = dfunc,
                         arrivals.FUN = afunc, aging.FUN = aging,
                         infection.FUN = infection.net,
                         module.order = c("resim_nets.FUN", "infection.FUN",
                                          "aging.FUN", "arrivals.FUN",
                                          "departures.FUN", "prevalence.FUN"),
                         tergmLite = TRUE, resimulate.network = TRUE, verbose = FALSE)
  mod4 <- netsim(est, param, init, control)
  expect_is(mod4, "netsim")

  ## "updated" infection module
  infect <- infection.net
  control <- control.net(type = NULL, nsims = 1, nsteps = 10,
                         departures.FUN = dfunc,
                         arrivals.FUN = afunc, aging.FUN = aging,
                         infection.FUN = infect,
                         module.order = c("resim_nets.FUN", "infection.FUN",
                                          "aging.FUN", "arrivals.FUN",
                                          "departures.FUN", "prevalence.FUN"),
                         tergmLite = TRUE, resimulate.network = TRUE, verbose = FALSE)
  mod5 <- netsim(est, param, init, control)
  expect_is(mod5, "netsim")

  expect_output(print(mod5), "resim_nets.FUN")
  expect_output(print(mod5), "infection.FUN")
  expect_output(print(mod5), "departures.FUN")
  expect_output(print(mod5), "arrivals.FUN")
  expect_output(print(mod5), "aging.FUN")

})

test_that("module.order is independent of resimulate.network", {
  skip_on_cran()

  nw <- network_initialize(n = 50)
  est <- netest(nw, formation = ~edges, target.stats = 15,
                coef.diss = dissolution_coefs(~offset(edges), 60),
                verbose = FALSE)
  param <- param.net(inf.prob = 0.3)
  init <- init.net(i.num = 5)

  custom_order <- c("resim_nets.FUN", "infection.FUN",
                     "nwupdate.FUN", "prevalence.FUN")

  # module.order preserved with resimulate.network = TRUE
  ctrl1 <- control.net(type = "SI", nsims = 1, nsteps = 5,
                        module.order = custom_order,
                        resimulate.network = TRUE, verbose = FALSE)
  expect_equal(ctrl1$module.order, custom_order)
  mod1 <- netsim(est, param, init, ctrl1)
  expect_s3_class(mod1, "netsim")

  # module.order preserved with resimulate.network = FALSE
  ctrl2 <- control.net(type = "SI", nsims = 1, nsteps = 5,
                        module.order = custom_order,
                        resimulate.network = FALSE, verbose = FALSE)
  expect_equal(ctrl2$module.order, custom_order)
  mod2 <- netsim(est, param, init, ctrl2)
  expect_s3_class(mod2, "netsim")
})

test_that("tergmLite = TRUE warns when overriding resimulate.network", {
  expect_warning(
    control.net(type = "SI", nsteps = 10,
                tergmLite = TRUE, resimulate.network = FALSE),
    "resimulate.network"
  )

  # confirm the override took effect
  ctrl <- suppressWarnings(
    control.net(type = "SI", nsteps = 10,
                tergmLite = TRUE, resimulate.network = FALSE)
  )
  expect_true(ctrl$resimulate.network)

  # no warning when resimulate.network = TRUE (no conflict)
  expect_no_warning(
    control.net(type = "SI", nsteps = 10,
                tergmLite = TRUE, resimulate.network = TRUE)
  )
})

context("Network Model with Param Updater")

test_that("netsim with param updater", {
  skip_on_cran()
  # Create the list.param.updaters
  list.param.updaters <- list(
    # this is one updater
    list(
      at = 10,
      verbose = TRUE,
      param = list(
        inf.prob = 0.3,
        act.rate = 0.3
      )
    ),
    # this is another updater
    list(
      at = 20,
      verbose = TRUE,
      param = list(
        # inf.prob = function(x) plogis(qlogis(x) - log(10)),
        # act.rate = function(x) plogis(qlogis(x) - log(10))
        inf.prob = 0.01
      )
    )
  )

  # Create the list.control.updaters
  list.control.updaters <- list(
    # this is one updater
    list(
      at = 30,
      verbose = TRUE,
      control = list(
        resimulate.network = FALSE
      )
    )
  )

  # Do not forget to add it to `param`
  param <- param.net(
    inf.prob = 0.1,
    act.rate = 0.1,
    .param.updater.list = list.param.updaters
  )

  # Enable the module in `control`
  control <- control.net(
    type = NULL, # must be NULL as we use a custom module
    nsims = 1,
    nsteps = 50,
    verbose = FALSE,
    infection.FUN = infection.net,
    .control.updater.list = list.control.updaters,
    resimulate.network = TRUE
  )

  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
  est <- netest(
    nw,
    formation = ~edges,
    target.stats = 25,
    coef.diss = dissolution_coefs(~offset(edges), 10, 0),
    verbose = FALSE
  )

  init <- init.net(i.num = 10)

  expect_message(mod <- netsim(est, param, init, control))

  # `resimulate.network` is turned of at step 30. We check that the number of
  # observations in the "networkDynamic" object is < than 31 and not 50 (the
  # number of timestep in the simulation)
  n_obs <- length(
    get.network.attribute(mod$network[[1]][[1]], 'net.obs.period')$observations
  )
  expect_lt(n_obs, 31)
})

context("Network Model with Scenarios")

test_that("SIS with scenarios", {
  skip_on_cran()
  set.seed(10)

  nw <- network_initialize(n = 200)
  est <- netest(nw,
    formation = ~edges, target.stats = 60,
    coef.diss = dissolution_coefs(~offset(edges), 10, 0),
    verbose = FALSE
  )

  param <- param.net(inf.prob = 0.9, rec.rate = 0.01, act.rate = 2)
  control <- control.net(type = "SIS", nsims = 1, nsteps = 50, verbose = FALSE)
  init <- init.net(i.num = 10)

  scenarios.df <- dplyr::tribble(
    ~.scenario.id, ~.at, ~inf.prob, ~rec.rate,
    "base", 0, 0.9, 0.01,
    "multiple_changes", 0, 0.1, 0.04,
    "multiple_changes", 20, 0.9, 0.01,
    "multiple_changes", 40, 0.1, 0.1
  )

  scenarios.list <- create_scenario_list(scenarios.df)
  expect_length(scenarios.list, 2)

  sc.param <- use_scenario(param, scenarios.list[[1]])
  expect_silent(netsim(est, sc.param, init, control))
  sc.param <- use_scenario(param, scenarios.list[[2]])
  expect_message(netsim(est, sc.param, init, control))

  # .at not a integer
  scenarios.df <- dplyr::tribble(
    ~.scenario.id, ~.at, ~inf.prob, ~rec.rate,
    "multiple_changes", "text", 0.1, 0.1
  )
  expect_error(create_scenario_list(scenarios.df))

  # inf_prob with an underscore
  scenarios.df <- dplyr::tribble(
    ~.scenario.id, ~.at, ~inf_prob, ~rec.rate,
    "multiple_changes", 0, 0.1, 0.1
  )
  expect_error(scenarios.list <- create_scenario_list(scenarios.df))

  # rec.rate2 not in param
  scenarios.df <- dplyr::tribble(
    ~.scenario.id, ~.at, ~inf.prob, ~rec.rate2,
    "multiple_changes", 0, 0.1, 0.1
  )
  scenarios.list <- create_scenario_list(scenarios.df)
  expect_error(sc.param <- use_scenario(param, scenarios.list[[1]]))
})

context("Records: attr_history and Raw Objects")

test_that("Time varying elements", {
  skip_on_cran()
  test_logger <- function(dat, at) {
    nodes <- get_posit_ids(dat)

    some_nodes <- sample(nodes, 5)
    dat <- record_attr_history(
      dat, at,
      "attr_norm",
      some_nodes,
      rnorm(length(some_nodes))
    )

    some_nodes <- sample(nodes, 5)
    dat <- record_attr_history(
      dat, at,
      "attr_unif",
      some_nodes,
      runif(length(some_nodes))
    )

    some_nodes <- sample(nodes, 5)
    dat <- record_attr_history(
      dat, at,
      "attr_fix",
      some_nodes,
      at
    )

    # test when 0 nodes selected
    some_nodes <- integer(0)
    dat <- record_attr_history(
      dat, at,
      "attr_none",
      some_nodes,
      at
    )

    return(dat)
  }

  param <- param.net(
    inf.prob = 0.1,
    act.rate = 0.1
  )

  # Enable the module in `control`
  control <- control.net(
    type = NULL, # must be NULL as we use a custom module
    nsims = 1,
    nsteps = 20,
    verbose = FALSE,
    infection.FUN = infection.net,
    logger.FUN = test_logger
  )

  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
  est <- netest(
    nw,
    formation = ~edges,
    target.stats = 25,
    coef.diss = dissolution_coefs(~ offset(edges), 10, 0),
    verbose = FALSE
  )

  init <- init.net(i.num = 10)

  mod <- netsim(est, param, init, control)
  attr_history <- get_attr_history(mod)
  expect_is(attr_history, "list")
  expect_is(attr_history[[1]], "data.frame")
  expect_equal(
    names(attr_history),
    c("attr_norm", "attr_unif", "attr_fix", "attr_none"))
})

context("Custom Trackers")

test_that("netsim, SI, custom trackers", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
  est <- netest(
    nw,
    formation = ~edges,
    target.stats = 25,
    coef.diss = dissolution_coefs(~ offset(edges), 10, 0),
    verbose = FALSE
  )

  init <- init.net(i.num = 10)

  epi_s_num <- function(dat, at) {
    needed_attributes <- c("status")
    output <- with(get_attr_list(dat, needed_attributes), {
      sum(status == "s", na.rm = TRUE)
    })
    return(output)
  }

  epi_prop_infected <- function(dat, at) {
    needed_attributes <- c("status", "active")
    output <- with(get_attr_list(dat, needed_attributes), {
      pop <- active == 1
      cond <- status == "i"
      sum(cond & pop, na.rm = TRUE) / sum(pop, na.rm = TRUE)
    })
    return(output)
  }

  some.trackers <- list(
    prop_infected = epi_prop_infected,
    s_num = epi_s_num
  )

  control <- control.net(
    type = "SI",
    nsims = 1,
    nsteps = 50,
    verbose = FALSE,
    infection.FUN = infection.net,
    .tracker.list = some.trackers
  )

  param <- param.net(
    inf.prob = 0.3,
    act.rate = 0.1
  )

  mod <- netsim(est, param, init, control)

  d <- as.data.frame(mod)

  expect_true(all(c("prop_infected", "s_num") %in% names(d)))
  expect_is(d[["prop_infected"]], "numeric")
  # the custom epi trackers are not run during intialization so the first value
  # is always NA
  expect_true(all(d$s_num[2:50] == d$s.num[2:50]))
})

context("Load Parameters from data.frame")

test_that("Load parameters from data.frame", {
  skip_on_cran()
  params.df <- dplyr::tribble(
    ~param, ~value, ~type, ~detail,
    "p1", "10", "numeric", "foo",
    "p2", "TRUE", "logical", "bar",
    "p3_1", "1", "numeric", "baz",
    "p3_2", "3", "numeric", "foobar",
    "p4", "tsa", "character", "foobaz"
  )

  expect_silent(param <- param.net_from_table(params.df))
  expect_s3_class(param, "param.net")
  expect_type(param, "list")

  expect_silent(param <- param.net(data.frame.parameters = params.df))
  expect_s3_class(param, "param.net")
  expect_type(param, "list")

  # convert back to a `long.param.df`
  param.df_back <- param.net_from_table(params.df) |> param.net_to_table()
  expect_true(all(params.df[c("param", "value", "type")] == param.df_back))

  # wrong column name
  params.df <- dplyr::tribble(~name, ~value, ~type, "p1", "10", "numeric")
  expect_error(param <- param.net_from_table(params.df))
  params.df <- dplyr::tribble(~param, ~val, ~type, "p1", "10", "numeric")
  expect_error(param <- param.net_from_table(params.df))
  params.df <- dplyr::tribble(~param, ~value, ~class, "p1", "10", "numeric")
  expect_error(param <- param.net_from_table(params.df))

  # wrong "type" value
  params.df <- dplyr::tribble(
    ~param, ~value, ~type, ~detail,
    "p1", "10", "numeric", "foo",
    "p2", "TRUE", "logical", "bar",
    "p3_1", "1", "factor", "baz",
    "p3_2", "3", "numeric", "foobar",
    "p4", "tsa", "character", "foobaz"
  )
  expect_error(param <- param.net_from_table(params.df))

  # wrong "param" format
  params.df <- dplyr::tribble(
    ~param, ~value, ~type, ~detail,
    ".p1", "10", "numeric", "foo",
    "p2", "TRUE", "logical", "bar",
    "p_3_1", "1", "numeric", "baz",
    "p3_2", "3", "numeric", "foobar",
    "p4", "tsa", "character", "foobaz"
  )
  expect_error(param <- param.net_from_table(params.df))
})

context("Random Parameter Generators")

test_that("Random parameters generators", {
  skip_on_cran()

  my_randoms <- list(
    act.rate = param_random(c(0.25, 0.5, 0.75)),
    tx.halt.part.prob = function() rbeta(1, 1, 2),
    hiv.test.rate = function() c(
      rnorm(1, 0.015, 0.01),
      rnorm(1, 0.010, 0.01),
      rnorm(1, 0.020, 0.01)
    )
  )

  expect_warning(param <- param.net(
      inf.prob = 0.3,
      act.rate = 0.3,
      random.params = my_randoms)
  )
  expect_message(generate_random_params(param, verbose = TRUE))
  expect_silent(generate_random_params(param, verbose = FALSE))

  param <- param.net(inf.prob = 0.3, act.rate = 0.1)
  expect_equal(generate_random_params(param), param)

  param <- param.net(inf.prob = 0.3, random.params = list())
  expect_equal(generate_random_params(param), param)

  param <- param.net(inf.prob = 0.3, random.params = 4)
  expect_error(generate_random_params(param))

  param <- param.net(inf.prob = 0.3, random.params = list(1))
  expect_error(generate_random_params(param))


  generate_correlated_params <- function() {
    param.unique <- runif(1)
    param.set.1 <- param.unique + runif(2)
    param.set.2 <- param.unique * rnorm(3)

    return(list(param.unique, param.set.1, param.set.2))
  }

  # Data.frame set of random parameters :
  correlated_params <- t(replicate(10, unlist(generate_correlated_params())))
  correlated_params <- as.data.frame(correlated_params)
  colnames(correlated_params) <- c(
    "param.unique",
    "param.set.1_1", "param.set.1_2",
    "param.set.2_1", "param.set.2_2", "param.set.2_3"
  )

  randoms <- c(my_randoms, list(param.random.set = correlated_params))
  param <- param.net(inf.prob = 0.3, random.params = randoms)
  expect_silent(generate_random_params(param))

  # duplicated `act.rate` random definition
  colnames(correlated_params) <- c(
    "act.rate",
    "param.set.1_1", "param.set.1_2",
    "param.set.2_1", "param.set.2_2", "param.set.2_3"
  )
  randoms <- c(my_randoms, list(param.random.set = correlated_params))
  expect_warning(
    param <- param.net(inf.prob = 0.3, act.rate = 0.1, random.params = randoms)
  )
  expect_warning(generate_random_params(param))

  # malformed name "param_set.1_1"
  colnames(correlated_params) <- c(
    "act.rate",
    "param_set.1_1", "param.set.1_2",
    "param.set.2_1", "param.set.2_2", "param.set.2_3"
  )
  randoms <- c(my_randoms, list(param.random.set = correlated_params))
  param <- param.net(inf.prob = 0.3, random.params = randoms)
  expect_error(generate_random_params(param))

  # param.random.set not a data.frame
  randoms <- c(my_randoms, list(param.random.set = list()))
  param <- param.net(inf.prob = 0.3, random.params = randoms)
  expect_error(generate_random_params(param))
})

#' @title Plot Data from a Stochastic Network Epidemic Model
#'
#' @description Plots epidemiological and network data from a stochastic network
#'              model simulated with [netsim()].
#'
#' @param x An `EpiModel` model object of class `netsim`.
#' @param type Type of plot: `"epi"` for epidemic model results,
#'        `"network"` for a static network plot (`plot.network`),
#'        or `"formation"`, `"duration"`, or `"dissolution"` for
#'        network formation, duration, or dissolution statistics.
#' @param y Output compartments or flows from `netsim` object to plot.
#' @param popfrac If `TRUE`, plot prevalence of values rather than numbers
#'        (see details).
#' @param sim.lines If `TRUE`, plot individual simulation lines. Default is
#'        to plot lines for one-group models but not for two-group models.
#' @param sims If `type="epi"` or `"formation"`, a vector of
#'        simulation numbers to plot. If `type="network"`, a single
#'        simulation number for which to plot the network, or else `"min"`
#'        to plot the simulation number with the lowest disease prevalence,
#'        `"max"` for the simulation with the highest disease prevalence,
#'        or `"mean"` for the simulation with the prevalence closest to the
#'        mean across simulations at the specified time step.
#' @param sim.col Vector of any standard R color format for simulation lines.
#' @param sim.lwd Line width for simulation lines.
#' @param sim.alpha Transparency level for simulation lines, where
#'        0 = transparent and 1 = opaque (see `adjustcolor` function).
#' @param mean.line If `TRUE`, plot mean of simulations across time.
#' @param mean.smooth If `TRUE`, use a loess smoother on the mean line.
#' @param mean.col Vector of any standard R color format for mean lines.
#' @param mean.lwd Line width for mean lines.
#' @param mean.lty Line type for mean lines.
#' @param qnts If numeric, plot polygon of simulation quantiles based on the
#'        range implied by the argument (see details). If `FALSE`, suppress
#'        polygon from plot.
#' @param qnts.col Vector of any standard R color format for polygons.
#' @param qnts.alpha Transparency level for quantile polygons, where 0 =
#'        transparent and 1 = opaque (see `adjustcolor` function).
#' @param qnts.smooth If `TRUE`, use a loess smoother on quantile polygons.
#' @param legend If `TRUE`, plot default legend.
#' @param leg.cex Legend scale size.
#' @param grid If `TRUE`, a grid is added to the background of plot
#'        (see [grid()] for details), with default of nx by ny.
#' @param add If `TRUE`, new plot window is not called and lines are added
#'        to existing plot window.
#' @param network Network number, for simulations with multiple networks
#'        representing the population.
#' @param at If `type = "network"`, time step for network graph.
#' @param col.status If `TRUE` and `type="network"`, automatic disease
#'        status colors (blue = susceptible, red = infected, green = recovered).
#' @param shp.g2 If `type = "network"` and `x` is for a two-group model,
#'        shapes for the Group 2 vertices, with acceptable inputs of "triangle"
#'        and "square". Group 1 vertices will remain circles.
#' @param vertex.cex Relative size of plotted vertices if `type="network"`,
#'        with implicit default of 1.
#' @param stats If `type="formation","duration","dissolution"`, statistics
#'        to plot. For `type = "formation"`, `stats` are among those
#'        specified in `nwstats.formula` of [control.net()]; for
#'        `type = "duration", "dissolution"`, `stats` are among those
#'        of the dissolution model (without `offset()`). The default is
#'        to plot all statistics.
#' @param targ.line If `TRUE`, plot target or expected value line for
#'        the statistic of interest.
#' @param targ.col Vector of standard R colors for target statistic lines, with
#'        default colors based on `RColorBrewer` color palettes.
#' @param targ.lwd Line width for the line showing the target statistic values.
#' @param targ.lty Line type for the line showing the target statistic values.
#' @param plots.joined If `TRUE` and
#'        `type="formation","duration","dissolution"`, combine all
#'        statistics in one plot, versus one plot per statistic if
#'        `FALSE`.
#' @param method Plot method for `type="formation", "duration", "dissolution"`,
#'        with options of `"l"` for line plots and `"b"` for box plots.
#' @param duration.imputed If `type = "duration"`, a logical indicating
#'        whether or not to impute starting times for relationships extant at
#'        the start of the simulation. Defaults to `TRUE` when
#'        `type = "duration"`.
#' @param ... Additional arguments to pass.
#' @inheritParams graphics::plot
#'
#' @details
#' This plot function can produce three types of plots with a stochastic network
#' model simulated through [netsim()]:
#'
#'  1. **`type="epi"`**: epidemic model results (e.g., disease
#'        prevalence and incidence) may be plotted.
#'  2. **`type="network"`**: a static network plot will be
#'        generated. A static network plot of a dynamic network is a
#'        cross-sectional extraction of that dynamic network at a specific
#'        time point. This plotting function wraps the
#'        [`network::plot.network`] function in the `network` package.
#'        Consult the help page for `plot.network` for all of the plotting
#'        parameters. In addition, four plotting parameters specific to
#'        `netsim` plots are available: `sim`, `at`,
#'        `col.status`, and `shp.g2`.
#'  3. **`type="formation"`**: summary network statistics related
#'        to the network model formation are plotted. These plots are similar
#'        to the formation plots for `netdx` objects. When running a
#'        `netsim` simulation, one must specify there that
#'        `save.nwstats=TRUE`; the plot here will then show the network
#'        statistics requested explicitly in `nwstats.formula`, or will use
#'        the formation formula set in `netest` otherwise.
#'  4. **`type="duration","dissolution"`**: as in
#'        [plot.netdx()]; supported in `plot.netsim` only when
#'        the dissolution model is `~offset(edges)`, `tergmLite` is
#'        `FALSE`, and `save.network` is `TRUE`.
#'
#' @details
#' When `type="epi"`, this plotting function will extract the
#' epidemiological output from a model object of class `netsim` and plot
#' the time series data of disease prevalence and other results. The summary
#' statistics that the function calculates and plots are individual simulation
#' lines, means of the individual simulation lines, and quantiles of those
#' individual simulation lines. The mean line, toggled on with
#' `mean.line=TRUE`, is calculated as the row mean across simulations at
#' each time step.
#'
#' Compartment prevalences are the size of a compartment over some denominator.
#' To plot the raw numbers from any compartment, use `popfrac=FALSE`; this
#' is the default for any plots of flows. The `popfrac` parameter
#' calculates and plots the denominators of all specified compartments using
#' these rules: 1) for one-group models, the prevalence of any compartment is
#' the compartment size divided by the total population size; 2) for two-group
#' models, the prevalence of any compartment is the compartment size divided by
#' the group population size. For any prevalences that are not automatically
#' calculated, the [mutate_epi()] function may be used to add new
#' variables to the `netsim` object to plot or analyze.
#'
#' The quantiles show the range of outcome values within a certain specified
#' quantile range. By default, the interquartile range is shown: that is the
#' middle 50% of the data. This is specified by `qnts=0.5`. To show the
#' middle 95% of the data, specify `qnts=0.95`. To toggle off the polygons
#' where they are plotted by default, specify `qnts=FALSE`.
#'
#' When `type="network"`, this function will plot cross sections of the
#' simulated networks at specified time steps. Because it is only possible to
#' plot one time step from one simulation at a time, it is necessary to enter
#' these in the `at` and `sims` parameters. To aid in visualizing
#' representative and extreme simulations at specific time steps, the
#' `sims` parameter may be set to `"mean"` to plot the simulation in
#' which the disease prevalence is closest to the average across all
#' simulations, `"min"` to plot the simulation in which the prevalence is
#' lowest, and `"max"` to plot the simulation in which the prevalence is
#' highest.
#'
#' @method plot netsim
#' @export
#'
#' @keywords plot
#' @seealso [`network::plot.network`], [mutate_epi()]
#'
#' @examples
#' \donttest{
#' ## SI Model without Network Feedback
#' # Initialize network and set network model parameters
#' nw <- network_initialize(n = 100)
#' nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#'
#' # Estimate the network model
#' est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' # Simulate the epidemic model
#' param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
#' init <- init.net(i.num = 10, i.num.g2 = 10)
#' control <- control.net(type = "SI", nsteps = 20, nsims = 3,
#'                        verbose = FALSE, save.nwstats = TRUE,
#'                        nwstats.formula = ~edges + meandeg + concurrent)
#' mod <- netsim(est, param, init, control)
#'
#' # Plot epidemic trajectory
#' plot(mod)
#' plot(mod, type = "epi", grid = TRUE)
#' plot(mod, type = "epi", popfrac = TRUE)
#' plot(mod, type = "epi", y = "si.flow", qnts = 1, ylim = c(0, 4))
#'
#' # Plot static networks
#' par(mar = c(0, 0, 0, 0))
#' plot(mod, type = "network", vertex.cex = 1.5)
#'
#' # Automatic coloring of infected nodes as red
#' par(mfrow = c(1, 2), mar = c(0, 0, 2, 0))
#' plot(mod, type = "network", main = "Min Prev | Time 50",
#'      col.status = TRUE, at = 20, sims = "min", vertex.cex = 1.25)
#' plot(mod, type = "network", main = "Max Prev | Time 50",
#'      col.status = TRUE, at = 20, sims = "max", vertex.cex = 1.25)
#'
#' # Automatic shape by group number (circle = group 1)
#' par(mar = c(0, 0, 0, 0))
#' plot(mod, type = "network", at = 20, col.status = TRUE,
#'      shp.g2 = "square")
#' plot(mod, type = "network", at = 20, col.status = TRUE,
#'      shp.g2 = "triangle", vertex.cex = 2)
#'
#' # Plot formation statistics
#' par(mfrow = c(1,1), mar = c(3,3,1,1), mgp = c(2,1,0))
#' plot(mod, type = "formation", grid = TRUE)
#' plot(mod, type = "formation", plots.joined = FALSE)
#' plot(mod, type = "formation", sims = 2:3)
#' plot(mod, type = "formation", plots.joined = FALSE,
#'      stats = c("edges", "concurrent"))
#' plot(mod, type = "formation", stats = "meandeg",
#'      mean.lwd = 1, qnts.col = "seagreen", mean.col = "black")
#' }
#'
plot.netsim <- function(x, type = "epi", y = NULL, popfrac = FALSE,
                        sim.lines = FALSE, sims = NULL, sim.col = NULL,
                        sim.lwd = NULL, sim.alpha = NULL, mean.line = TRUE,
                        mean.smooth = TRUE, mean.col = NULL, mean.lwd = 2,
                        mean.lty = 1, qnts = 0.5, qnts.col = NULL,
                        qnts.alpha = 0.5, qnts.smooth = TRUE, legend = NULL,
                        leg.cex = 0.8, grid = FALSE, add = FALSE, network = 1,
                        at = 1, col.status = FALSE, shp.g2 = NULL,
                        vertex.cex = NULL, stats = NULL, targ.line = TRUE,
                        targ.col = NULL, targ.lwd = 2, targ.lty = 2,
                        plots.joined = NULL, duration.imputed = TRUE,
                        method = "l", main = NULL, xlim = NULL, xlab = NULL,
                        ylim = NULL, ylab = NULL, ...) {

  type <- match.arg(
    type,
    c("epi", "network", "formation", "duration", "dissolution")
  )

  if (type == "network") {
    plot_netsim_network(x, at, sims, network, shp.g2, col.status, vertex.cex,
                        ...)
  } else if (type == "epi") {
    plot_netsim_epi(
      x, y, sims, legend, mean.col, qnts.col, sim.lwd, sim.col, sim.alpha,
      popfrac, qnts, qnts.alpha, qnts.smooth, mean.line, mean.smooth, add,
      mean.lwd, mean.lty, xlim, ylim, main, xlab, ylab, sim.lines, grid,
      leg.cex, ...
    )
  } else {
    plot_netsim_stats(
      x, type, sims, stats, network, duration.imputed, method, sim.lines,
      sim.col, sim.lwd, mean.line, mean.smooth, mean.col, mean.lwd, mean.lty,
      qnts, qnts.col, qnts.alpha, qnts.smooth, targ.line, targ.col, targ.lwd,
      targ.lty, plots.joined, legend, grid, xlim, xlab, ylim, ylab, ...
    )
  }
}

plot_netsim_network <- function(x, at, sims, network, shp.g2, col.status, vertex.cex, ...) {
  # Network plot ------------------------------------------------------------
  if (x$control$tergmLite) {
    stop("networkDyanmic object is not saved in tergmLite netsim simulation.
      Check control setting tergmLite")
  }

  nsteps <- x$control$nsteps
  if (at > nsteps) {
    stop("Specify a time step between 1 and ", nsteps)
  }

  sims <- if (is.null(sims)) 1 else sims
  if (length(sims) > 1 ||
        (!is.numeric(sims) && !(sims %in% c("mean", "max", "min")))) {
    stop("sims argument must be single simulation number",
         "or \"mean\", \"max\", or \"min\" ")
  }

  sims.arg <- sims
  if (sims == "mean") {
    sims <- which.min(
      abs(as.numeric(x$epi$i.num[at, ]) - mean(as.numeric(x$epi$i.num[at, ])))
    )
    sims.val <- as.numeric(x$epi$i.num[at, sims])
  } else if (sims == "max") {
    sims <- as.numeric(which.max(x$epi$i.num[at, ]))
    sims.val <- x$epi$i.num[at, sims]
  } else if (sims == "min") {
    sims <- as.numeric(which.min(x$epi$i.num[at, ]))
    sims.val <- x$epi$i.num[at, sims]
  }

  obj <- get_network(x, sims, network, collapse = TRUE, at = at)
  tergmLite <- x$control$tergmLite
  miss_vertex.cex <- is.null(vertex.cex)

  if (!is.null(shp.g2)) {
    if (all(!shp.g2 %in% c("square", "triangle"))) {
      stop("shp.g2 accepts inputs of either \"square\" or \"triangle\" ")
    }

    grp.flag <- length(unique(get_vertex_attribute(obj, "group")))
    if (grp.flag == 2) {
      mids <- idgroup(obj)
      if (shp.g2 == "square") {
        vertex.sides <- ifelse(mids == 1, 50, 4)
        vertex.rot <- 45
      }
      if (shp.g2 == "triangle") {
        vertex.sides <- ifelse(mids == 1, 50, 3)
        vertex.rot <- 90
      }
    } else {
      warning("shp.g2 applies to two-group networks only, so ignoring.")
      vertex.sides <- 50
      vertex.rot <- 0
    }
  } else {
    vertex.sides <- 50
    vertex.rot <- 0
  }
  if (miss_vertex.cex) {
    vertex.cex <- 1
  }
  if (col.status) {
    if (tergmLite) {
      stop("Plotting status colors requires tergmLite=FALSE in netsim
        control settings.")
    }
    pal <- adjustcolor(c(4, 2, 3), 0.75)
    testatus <- get.vertex.attribute.active(obj, "testatus", at = at)
    cols <- rep(pal[1], length(testatus))
    cols[testatus == "i"] <- pal[2]
    cols[testatus == "r"] <- pal[3]
    plot.network(obj, vertex.col = cols, vertex.border = "grey60",
                 edge.col = "grey40", vertex.sides = vertex.sides,
                 vertex.rot = vertex.rot, vertex.cex = vertex.cex,
                 displaylabels = FALSE, ...)
    if (sims.arg %in% c("mean", "max", "min")) {
      mtext(side = 1, text = paste("Sim =", sims, " | Prev =", sims.val))
    }
  } else {
    plot.network(obj, vertex.sides = vertex.sides, vertex.rot = vertex.rot,
                 vertex.cex = vertex.cex, displaylabels = FALSE, ...)
  }
}

plot_netsim_epi <- function(x, y = NULL, sims = NULL, legend = NULL,
                            mean.col = NULL, qnts.col = NULL, sim.lwd = NULL,
                            sim.col = NULL, sim.alpha = NULL, popfrac = FALSE,
                            qnts = 0.5, qnts.alpha = 0.5, qnts.smooth = TRUE,
                            mean.line = TRUE, mean.smooth = TRUE, add = FALSE,
                            mean.lwd = 2, mean.lty = 1, xlim = NULL,
                            ylim = NULL, main = NULL, xlab = NULL, ylab = NULL,
                            sim.lines = FALSE, grid = FALSE, leg.cex = 0.8,
                            ...) {
  ## Model dimensions and class ##
  nsteps <- x$control$nsteps
  start <- if (is.null(x$control$start)) 1 else x$control$start
  offset <- start - 1
  nsims <- x$control$nsims
  sims <- if (is.null(sims)) seq_len(nsims) else sims
  if (max(sims) > nsims) stop("Set sim to between 1 and ", nsims)
  if (is.null(x$param$groups) || !is.numeric(x$param$groups)) {
    x$param$groups <- 1
  }
  groups <- x$param$groups

  ## Compartments ##
  nocomp <- is.null(y)
  if (nocomp) {
    legend <- if (is.null(legend)) TRUE else legend
    if (groups == 1) {
      y <- grep(".num$", names(x$epi), value = TRUE)
    } else if (groups == 2) {
      y <- c(
        grep(".num$", names(x$epi), value = TRUE),
        grep(".num.g2$", names(x$epi), value = TRUE)
      )
    }
  } else if (!all(y %in% names(x$epi))) {
    stop("Specified y is not available in object")
  }

  lcomp <- length(y)

  ## Color palettes ##

  # Main color palette
  bpal <- c(4, 2, 3, 5:100)

  # Mean line
  mean.col <- if (is.null(mean.col)) bpal else mean.col
  mean.pal <- adjustcolor(mean.col, 0.9)

  # Quantile bands
  qnts.col <- if (is.null(qnts.col)) bpal else qnts.col
  qnts.pal <- adjustcolor(qnts.col, qnts.alpha)

  # Sim lines
  sim.lwd <- if (is.null(sim.lwd)) 0.75 else sim.lwd
  if (length(sim.lwd) < lcomp) {
    sim.lwd <- rep(sim.lwd, lcomp)
  }

  sim.col <- if (is.null(sim.col)) bpal else sim.col
  if (length(sim.col) < lcomp) {
    sim.col <- rep(sim.col, lcomp)
  }

  if (is.null(sim.alpha) && nsims == 1) {
    sim.alpha <- 0.9
  } else if (is.null(sim.alpha) && nsims > 1) {
    sim.alpha <- max(c(0.05, 1 - log10(nsims) / 3))
  }
  sim.pal <- adjustcolor(sim.col, sim.alpha)

  ## Prevalence calculations ##
  x <- denom(x, y, popfrac)

  # Compartment max
  if (!popfrac) {
    min.prev <- min(sapply(y, \(comps) min(x$epi[[comps]], na.rm = TRUE)))
    max.prev <- max(sapply(y, \(comps) max(x$epi[[comps]], na.rm = TRUE)))
  } else {
    min.prev <- 0
    max.prev <- 1
  }

  # Initialize ylim max values
  qnt.min <- 1E10
  qnt.max <- -1E10
  mean.min <- 1E10
  mean.max <- -1E10

  ## Quantile and mean line setup (used in both ylim calc and drawing) ##
  if (qnts == FALSE || nsims == 1) {
    disp.qnts <- FALSE
  } else {
    disp.qnts <- TRUE
  }
  if (disp.qnts && (qnts > 1 || qnts < 0)) {
    stop("qnts must be between 0 and 1")
  }

  if (mean.line) {
    if (length(mean.lwd) < lcomp) {
      mean.lwd <- rep(mean.lwd, lcomp)
    }
    if (length(mean.lty) < lcomp) {
      mean.lty <- rep(mean.lty, lcomp)
    }
  }

  ## Quantiles - ylim max ##
  if (disp.qnts) {
    qnt.max <- draw_qnts(x, y, qnts, qnts.pal, qnts.smooth, "epi", 0, "max", offset)
    qnt.min <- draw_qnts(x, y, qnts, qnts.pal, qnts.smooth, "epi", 0, "min", offset)
  }

  ## Mean lines - ylim max ##
  if (mean.line) {
    mean.max <- draw_means(
      x, y,
      mean.smooth, mean.lwd, mean.pal, mean.lty,
      "epi", 0, "max", offset
    )
    mean.min <- draw_means(
      x, y,
      mean.smooth, mean.lwd, mean.pal, mean.lty,
      "epi", 0, "min", offset
    )
  }

  ## Missing args ##
  xlim <- if (is.null(xlim)) c(offset, offset + nsteps) else xlim

  if (is.null(ylim) && (popfrac || sim.lines)) {
    ylim <- c(min.prev, max.prev)
  } else if (is.null(ylim) && !popfrac && !sim.lines &&
               (mean.line || disp.qnts)) {
    ylim <- c(
      min(qnt.min * 0.9, mean.min * 0.9),
      max(qnt.max * 1.1, mean.max * 1.1)
    )
  }

  main <- if (is.null(main)) "" else main

  xlab <- if (is.null(xlab)) "Time" else xlab

  if (is.null(ylab)) {
    ylab <- if (popfrac) "Prevalence" else "Number"
  }

  ## Main plot window ##
  if (!add) {
    do.call(plot, list(
      x = 1, y = 1, type = "n", bty = "n",
      xlim = xlim, xlab = xlab, ylim = ylim, ylab = ylab, main = main, ...
    ))
  }

  ## Quantiles ##
  if (disp.qnts) {
    draw_qnts(x, y, qnts, qnts.pal, qnts.smooth, "epi", 1, "max", offset)
  }


  ## Simulation lines ##
  if (sim.lines) {
    for (j in seq_len(lcomp)) {
      for (i in sims) {
        lines(seq_len(nrow(x$epi[[y[j]]])) + offset,
              x$epi[[y[j]]][, i], lwd = sim.lwd[j], col = sim.pal[j])
      }
    }
  }


  ## Mean lines ##
  if (mean.line) {
    draw_means(x, y, mean.smooth, mean.lwd, mean.pal, mean.lty,
               "epi", 1, "max", offset)
  }

  ## Grid
  if (grid) grid()

  ## Legends ##
  if (!is.null(legend) && legend) {
    if (groups == 2 && nocomp) {
      leg.lty <- mean.lty
    } else {
      leg.lty <- 1
    }
    legend(
      "topright", legend = y, lty = leg.lty, lwd = 2, col = mean.pal,
      cex = leg.cex, bg = "white"
    )
  }
}

plot_netsim_stats <- function(x, type, sims, stats, network, duration.imputed,
                              method, sim.lines, sim.col, sim.lwd,
                              mean.line, mean.smooth, mean.col, mean.lwd,
                              mean.lty, qnts, qnts.col, qnts.alpha, qnts.smooth,
                              targ.line, targ.col, targ.lwd, targ.lty,
                              plots.joined, legend, grid, xlim, xlab,
                              ylim, ylab, ...) {
  sims <- if (is.null(sims)) seq_len(x$control$nsims) else sims
  if (max(sims) > x$control$nsims)
    stop("Maximum sim number is ", x$control$nsims)
  nsims <- length(sims)

  # Formation plot ----------------------------------------------------------
  if (type == "formation") {
    data <- get_nwstats(x, sims, network, mode = "list")
    ## target stats
    nwparam <- get_nwparam(x, network)
    ts <- nwparam$target.stats
    tsn <- nwparam$target.stats.names
    names(ts) <- tsn
  } else {
    ## duration/dissolution plot
    if (x$control$save.diss.stats && x$control$save.network &&
          !x$control$tergmLite && !is.null(x$diss.stats) &&
          x$nwparam[[network]]$coef.diss$diss.model.type == "edgesonly") {

      if (any(unlist(lapply(x$diss.stats, `[[`, "anyNA")))) {
        message(
          "\nNOTE: Duration & dissolution data contains undefined values due ",
          "to zero edges of some dissolution dyad type(s) on some time step;",
          " these undefined values will be set to 0 when processing the data."
        )
      }

      if (type == "duration") {
        elt <- if (duration.imputed) "meanageimputed" else "meanage"
        data <- lapply(sims, \(sim) x$diss.stats[[sim]][[network]][[elt]])
        ts <- x$nwparam[[network]]$coef.diss$duration
      } else { # if type is "dissolution"
        elt <- "propdiss"
        data <- lapply(sims, \(sim) x$diss.stats[[sim]][[network]][[elt]])
        ts <- 1 / x$nwparam[[network]]$coef.diss$duration
      }
    } else {
      stop(
        "Cannot produce duration/dissolution plot from `netsim` object ",
        "unless `save.diss.stats` is `TRUE`, `save.network` is `TRUE`, ",
        "`tergmLite` is `FALSE`, `keep.diss.stats` is `TRUE` (if ",
        "merging), and dissolution model is edges-only"
      )
    }
  }

  stats_table <- make_stats_table(data, ts)
  data <- array(unlist(data), dim = c(dim(data[[1]]), nsims))

  sel <- validate_stats_selection(stats_table, data, stats, "netsim object")
  data <- sel$data
  nmstats <- sel$nmstats
  targets <- sel$targets

  plot_stats_table(
    data = data,
    nmstats = nmstats,
    method = method,
    sim.lines = sim.lines,
    sim.col = sim.col,
    sim.lwd = sim.lwd,
    mean.line = mean.line,
    mean.smooth = mean.smooth,
    mean.col = mean.col,
    mean.lwd = mean.lwd,
    mean.lty = mean.lty,
    qnts = qnts,
    qnts.col = qnts.col,
    qnts.alpha = qnts.alpha,
    qnts.smooth = qnts.smooth,
    targ.line = targ.line,
    targ.col = targ.col,
    targ.lwd = targ.lwd,
    targ.lty = targ.lty,
    plots.joined = plots.joined,
    draw_legend = legend,
    grid = grid,
    targets = targets,
    dynamic = TRUE, # always dynamic in netsim
    xlim = xlim, xlab = xlab,
    ylim = ylim, ylab = ylab,
    ...
  )
}

#' @title ggplot2 Geom for Quantile Bands
#'
#' @description Plots quantile bands given a data.frame with stochastic model
#'              results from [icm()] or [netsim()].
#'
#' @param mapping Standard aesthetic mapping `aes()` input for ggplot2.
#' @param lower Lower quantile for the time series.
#' @param upper Upper quantile for the time series.
#' @param alpha Transparency of the ribbon fill.
#' @param ... Additional arguments passed to `stat_summary`.
#'
#' @details
#' This is a wrapper around `ggplot::stat_summary` with a ribbon geom as
#' aesthetic output.
#'
#' @export
#' @keywords plot
#'
#' @examples
#' \donttest{
#' param <- param.icm(inf.prob = 0.2, act.rate = 0.25)
#' init <- init.icm(s.num = 500, i.num = 1)
#' control <- control.icm(type = "SI", nsteps = 250, nsims = 5)
#' mod1 <- icm(param, init, control)
#' df <- as.data.frame(mod1)
#' df.mean <- as.data.frame(mod1, out = "mean")
#'
#' library(ggplot2)
#' ggplot() +
#'    geom_line(data = df, mapping = aes(time, i.num, group = sim),
#'    alpha = 0.25, lwd = 0.25, color = "firebrick") +
#'    geom_bands(data = df, mapping = aes(time, i.num),
#'               lower = 0.1, upper = 0.9, fill = "firebrick") +
#'    geom_line(data = df.mean, mapping = aes(time, i.num)) +
#'    theme_minimal()
#' }
#'
geom_bands <- function(mapping, lower = 0.25, upper = 0.75, alpha = 0.25, ...) {
  stat_summary(mapping,
               geom = "ribbon",
               fun.min = function(x) quantile(x, lower),
               fun.max = function(x) quantile(x, upper),
               alpha = alpha, ...)
}

#' @title Epidemic Parameters for Stochastic Individual Contact Models
#'
#' @description Sets the epidemic parameters for stochastic individual contact
#'              models simulated with `icm`.
#'
#' @inheritParams param.dcm
#'
#' @details
#' `param.icm` sets the epidemic parameters for the stochastic individual
#' contact models simulated with the [icm()] function. Models
#' may use the base types, for which these parameters are used, or new process
#' modules which may use these parameters (but not necessarily).
#'
#' For base models, the model specification will be chosen as a result of
#' the model parameters entered here and the control settings in
#' [control.icm()]. One-group and two-group models are available,
#' where the former assumes a homogeneous mixing in the population and the
#' latter assumes some form of heterogeneous mixing between two distinct
#' partitions in the population (e.g., men and women). Specifying any group two
#' parameters (those with a `.g2`) implies the simulation of a two-group
#' model. All the parameters for a desired model type must be specified, even if
#' they are zero.
#'
#' @section Act Balancing:
#' In two-group models, a balance between the number of acts for group 1 members
#' and those for group 2 members must be maintained. With purely heterogeneous
#' mixing, the product of one group size and act rate must equal the product of
#' the other group size and act rate: \eqn{N_1 \alpha_1 = N_2 \alpha_2}, where
#' \eqn{N_i} is the group size and \eqn{\alpha_i} the group-specific act rate
#' at time \eqn{t}. The `balance` parameter here specifies which group's
#' act rate should control the others with respect to balancing.
#'
#' @return An `EpiModel` object of class `param.icm`.
#'
#' @seealso Use [init.icm()] to specify the initial conditions and
#'          [control.icm()] to specify the control settings. Run the
#'          parameterized model with [icm()].
#'
#' @keywords parameterization
#'
#' @export
#'
param.icm <- function(inf.prob, inter.eff, inter.start, act.rate, rec.rate,
                      a.rate, ds.rate, di.rate, dr.rate, inf.prob.g2,
                      act.rate.g2, rec.rate.g2, a.rate.g2, ds.rate.g2,
                      di.rate.g2, dr.rate.g2, balance, ...) {

  # Get arguments
  p <- list()
  formal.args <- formals(sys.function())
  formal.args[["..."]] <- NULL
  for (arg in names(formal.args)) {
    if (as.logical(mget(arg) != "")) {
      p[arg] <- list(get(arg))
    }
  }
  dot.args <- list(...)
  names.dot.args <- names(dot.args)
  if (length(dot.args) > 0) {
    for (i in seq_along(dot.args)) {
      p[[names.dot.args[i]]] <- dot.args[[i]]
    }
  }


  ## Defaults and checks
  if (is.null(p$act.rate)) {
    p$act.rate <- 1
  }
  p$vital <- ifelse(!is.null(p$a.rate) | !is.null(p$ds.rate) |
                      !is.null(p$di.rate) | !is.null(p$dr.rate), TRUE, FALSE)

  p$groups <- ifelse(any(grepl(".g2", names(p))) == TRUE, 2, 1)

  if (p$groups == 2 && (is.null(p$balance) ||
                          !(p$balance %in% c("g1", "g2")))) {
    stop("Specify balance=\"g1\" or balance=\"g2\" with 2-group models")
  }

  if (!is.null(p$inter.eff) && is.null(p$inter.start)) {
    p$inter.start <- 1
  }

  ## Output
  class(p) <- c("param.icm", "list")
  return(p)
}


#' @title Initial Conditions for Stochastic Individual Contact Models
#'
#' @description Sets the initial conditions for stochastic individual contact
#'              models simulated with `icm`.
#'
#' @param s.num Number of initial susceptible persons. For two-group models,
#'        this is the number of initial group 1 susceptible persons.
#' @param i.num Number of initial infected persons. For two-group models, this
#'        is the number of initial group 1 infected persons.
#' @param r.num Number of initial recovered persons. For two-group models, this
#'        is the number of initial group 1 recovered persons. This parameter is
#'        only used for the `SIR` model type.
#' @param s.num.g2 Number of initial susceptible persons in group 2. This
#'        parameter is only used for two-group models.
#' @param i.num.g2 Number of initial infected persons in group 2. This parameter
#'        is only used for two-group models.
#' @param r.num.g2 Number of initial recovered persons in group 2. This
#'        parameter is only used for two-group `SIR` models.
#' @param ... Additional initial conditions passed to model.
#'
#' @details
#' The initial conditions for a model solved with [icm()] should be
#' input into the `init.icm` function. This function handles initial
#' conditions for both base models and original models using new modules.
#'
#' @return An `EpiModel` object of class `init.icm`.
#'
#' @seealso Use [param.icm()] to specify model parameters and
#'          [control.icm()] to specify the control settings. Run the
#'          parameterized model with [icm()].
#'
#' @keywords parameterization
#'
#' @export
#'
init.icm <- function(s.num, i.num, r.num,
                     s.num.g2, i.num.g2, r.num.g2, ...) {

  # Get arguments
  p <- list()
  formal.args <- formals(sys.function())
  formal.args[["..."]] <- NULL
  for (arg in names(formal.args)) {
    if (as.logical(mget(arg) != "")) {
      p[arg] <- list(get(arg))
    }
  }
  dot.args <- list(...)
  names.dot.args <- names(dot.args)
  if (length(dot.args) > 0) {
    for (i in seq_along(dot.args)) {
      p[[names.dot.args[i]]] <- dot.args[[i]]
    }
  }


  ## Output
  class(p) <- c("init.icm", "list")
  return(p)
}


#' @title Control Settings for Stochastic Individual Contact Models
#'
#' @description Sets the controls for stochastic individual contact models
#'              simulated with [icm()].
#'
#' @param type Disease type to be modeled, with the choice of `"SI"` for
#'        Susceptible-Infected diseases, `"SIR"` for
#'        Susceptible-Infected-Recovered diseases, and `"SIS"` for
#'        Susceptible-Infected-Susceptible diseases.
#' @param nsteps Number of time steps to solve the model over. This must be a
#'        positive integer.
#' @param nsims Number of simulations to run.
#' @param verbose If `TRUE`, print model progress to the console.
#' @param verbose.int Time step interval for printing progress to console, where
#'        0 (the default) prints completion status of entire simulation and
#'        positive integer `x` prints progress after every `x` time
#'        steps.
#'
#' @details
#' `control.icm` sets the required control settings for any stochastic
#' individual contact model solved with the [icm()] function. ICM simulations
#' use the built-in SI, SIR, and SIS disease types only. The `type` argument
#' is required and has no default. For custom or extension epidemic models, use
#' the network model class via [control.net()] instead.
#'
#' @return An `EpiModel` object of class `control.icm`.
#'
#' @seealso Use [param.icm()] to specify model parameters and
#'          [init.icm()] to specify the initial conditions. Run the
#'          parameterized model with [icm()].
#'
#' @keywords parameterization
#'
#' @export
#'
control.icm <- function(type, nsteps, nsims = 1,
                        verbose = FALSE, verbose.int = 0) {

  # Get arguments
  p <- list()
  formal.args <- formals(sys.function())
  for (arg in names(formal.args)) {
    if (as.logical(mget(arg) != "")) {
      p[arg] <- list(get(arg))
    }
  }

  ## Defaults and checks
  if (is.null(p$type) || !(p$type %in% c("SI", "SIS", "SIR"))) {
    stop("Specify type as \"SI\", \"SIS\", or \"SIR\" ")
  }
  if (is.null(p$nsteps)) {
    stop("Specify nsteps")
  }


  ## Output
  p <- set.control.class("control.icm", p)
  return(p)
}


#' @title Cross Checking of Inputs for Stochastic Individual Contact Models
#'
#' @description This function checks that the three parameter lists from
#'              [param.icm()], [init.icm()], and
#'              [control.icm()] are consistent.
#'
#' @param param An `EpiModel` object of class [param.icm()].
#' @param init An `EpiModel` object of class [init.icm()].
#' @param control An `EpiModel` object of class [control.icm()].
#'
#' @return
#' This function returns no objects.
#'
#' @export
#' @keywords internal
#'
crosscheck.icm <- function(param, init, control) {
  check.control.class("icm", "EpiModel crosscheck.icm")

  ## Main class check
  if (!inherits(param, "param.icm")) {
    stop("param must be an object of class param.icm")
  }
  if (!inherits(init, "init.icm")) {
    stop("init must be an object of class init.icm")
  }
  if (!inherits(control, "control.icm")) {
    stop("control must be an object of class control.icm")
  }

  ## Check that rec.rate is supplied for SIR models
  if (control$type %in% c("SIR", "SIS")) {
    if (is.null(param$rec.rate)) {
      stop("Specify rec.rate in param.icm")
    }
    if (param$groups == 2 && is.null(param$rec.rate.g2)) {
      stop("Specify rec.rate.g2 in param.icm")
    }
  }

  ## Check that parameters and init are supplied for SIR models
  if (control$type == "SIR") {
    if (is.null(init$r.num)) {
      stop("Specify r.num in init.icm")
    }
    if (param$groups == 2 && is.null(init$r.num.g2)) {
      stop("Specify r.num.g2 in init.icm")
    }
  }

  ## Check that groups implied by init and params are consistent
  if (any(grepl(".g2", names(init))) == TRUE) {
    init.groups <- 2
  } else {
    init.groups <- 1
  }
  if (param$groups == 2 && init.groups == 1) {
    stop("Group 2 parameters specified in param.icm, but missing group 2, ",
         "initial states in init.icm")
  }
  if (param$groups == 1 && init.groups == 2) {
    stop("Group 2 initial stats specified in init.icm, but missing group 2 ",
         "parameters in param.icm")
  }

  if (!is.null(param$trans.rate)) {
    stop("The trans.rate parameter has been removed. Use inf.prob instead.")
  }
  if (!is.null(param$trans.rate.g2)) {
    stop("The trans.rate.g2 parameter has been removed. Use inf.prob.g2 instead.")
  }

  ## Assign built-in modules based on group parameter
  ## initialize.icm handles both 1-group and 2-group (no .bip variant)
  control[["initialize.FUN"]] <- initialize.icm
  bi.mods <- c("infection.FUN", "recovery.FUN",
               "departures.FUN", "arrivals.FUN", "prevalence.FUN")
  if (param$groups == 1) {
    for (mod in bi.mods) {
      control[[mod]] <- get(gsub(".FUN", ".icm", mod))
    }
  } else {
    for (mod in bi.mods) {
      control[[mod]] <- get(gsub(".FUN", ".icm.bip", mod))
    }
  }

  ## In-place assignment to update param and control
  on.exit(assign("param", param, pos = parent.frame()))
  on.exit(assign("control", control, pos = parent.frame()), add = TRUE)
}

context("ICM Extended Models (All SOC)")

# SI Models ---------------------------------------------------------------

test_that("SI, 1G, CL: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SI", nsteps = 100, nsims = 1, verbose = FALSE,
                         verbose.int = 1)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_true(all(x$epi$num == 501))
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 100)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SI, 1G, CL: 2 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SI", nsteps = 100, nsims = 2, verbose = FALSE,
                         verbose.int = 5)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_true(all(x$epi$num == 501))
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, qnts.smooth = FALSE, sim.lines = TRUE, mean.smooth = FALSE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 100)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SI, 1G, CL: inf.prob = 0", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0, act.rate = 0.25)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SI", nsteps = 25, nsims = 10, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_true(all(x$epi$num == 501))
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 13))
  expect_output(summary(x, at = 25), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 25)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SI, 2G, CL: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.5, act.rate = 0.25, inf.prob.g2 = 0.1,
                     balance = "g1")
  init <- init.icm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 0)
  control <- control.icm(type = "SI", nsteps = 100, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_true(all(x$epi$num == 501))
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  expect_error(comp_plot(x, at = 100))
  test_icm(x)
})

test_that("SI, 2G, CL: 2 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25, inf.prob.g2 = 0.1,
                     balance = "g1")
  init <- init.icm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 0)
  control <- control.icm(type = "SI", nsteps = 50, nsims = 2, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_true(all(x$epi$num == 501))
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 3))
  expect_output(summary(x, at = 50), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  expect_error(comp_plot(x, at = 50))
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SI, 1G, OP: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25,
                     a.rate = 1 / 100, ds.rate = 1 / 100, di.rate = 1 / 90)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SI", nsteps = 100, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 100)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SI, 1G, OP: 2 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25,
                     a.rate = 1 / 100, ds.rate = 1 / 100, di.rate = 1 / 90)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SI", nsteps = 50, nsims = 2, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 3))
  expect_output(summary(x, at = 50), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 50)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SI, 2G, OP: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25, inf.prob.g2 = 0.1,
                     balance = "g1", a.rate = 1 / 100, a.rate.g2 = NA,
                     ds.rate = 1 / 100, ds.rate.g2 = 1 / 100,
                     di.rate = 1 / 90, di.rate.g2 = 1 / 90)
  init <- init.icm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 0)
  control <- control.icm(type = "SI", nsteps = 100, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  expect_error(comp_plot(x, at = 100))
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SI, 2G, OP: 2 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25, inf.prob.g2 = 0.1,
                     balance = "g1", a.rate = 1 / 100, a.rate.g2 = NA,
                     ds.rate = 1 / 100, ds.rate.g2 = 1 / 100,
                     di.rate = 1 / 90, di.rate.g2 = 1 / 90)
  init <- init.icm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 0)
  control <- control.icm(type = "SI", nsteps = 50, nsims = 2, verbose = FALSE,
                         verbose.int = 5)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 3))
  expect_output(summary(x, at = 50), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  expect_error(comp_plot(x, at = 50))
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})


# SIR Models --------------------------------------------------------------

test_that("SIR, 1G, CL: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25, rec.rate = 1 / 50)
  init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.icm(type = "SIR", nsteps = 100, nsims = 1, verbose = FALSE,
                         verbose.int = 1)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "r.num")
  plot(x, y = "ir.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 100)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SIR, 1G, CL: 2 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25, rec.rate = 1 / 50)
  init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.icm(type = "SIR", nsteps = 50, nsims = 2, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 3))
  expect_output(summary(x, at = 50), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "r.num")
  plot(x, y = "ir.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 50)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SIR, 1G, CL: inf.prob = 0", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0, act.rate = 0.25, rec.rate = 0)
  init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.icm(type = "SIR", nsteps = 10, nsims = 10, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_equal(max(x$epi$i.num), 1)
  expect_equal(min(x$epi$i.num), 1)
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 13))
  expect_output(summary(x, at = 10), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 10)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SIR, 2G, CL: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25, inf.prob.g2 = 0.1,
                     balance = "g1", rec.rate = 1 / 100, rec.rate.g2 = 1 / 100)
  init <- init.icm(s.num = 500, i.num = 1, r.num = 0,
                   s.num.g2 = 500, i.num.g2 = 0, r.num.g2 = 0)
  control <- control.icm(type = "SIR", nsims = 1, nsteps = 100, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  expect_error(comp_plot(x, at = 100))
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SIR, 2G, CL: 2 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25, inf.prob.g2 = 0.1,
                     balance = "g1", rec.rate = 1 / 100, rec.rate.g2 = 1 / 100)
  init <- init.icm(s.num = 500, i.num = 1, r.num = 0,
                   s.num.g2 = 500, i.num.g2 = 0, r.num.g2 = 0)
  control <- control.icm(type = "SIR", nsims = 2, nsteps = 50, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 3))
  expect_output(summary(x, at = 50), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  expect_error(comp_plot(x, at = 50))
  test_icm(x)
})

test_that("SIR, 1G, OP: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 3, rec.rate = 1 / 50,
                     a.rate = 1 / 100, ds.rate = 1 / 100, di.rate = 1 / 90,
                     dr.rate = 1 / 100)
  init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.icm(type = "SIR", nsteps = 100, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 100)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SIR, 1G, OP: 2 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 3, rec.rate = 1 / 50,
                     a.rate = 1 / 100, ds.rate = 1 / 100, di.rate = 1 / 90,
                     dr.rate = 1 / 100)
  init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.icm(type = "SIR", nsteps = 50, nsims = 2, verbose = FALSE)

  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 3))
  expect_output(summary(x, at = 50), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 50)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SIR, 2G, OP: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, inf.prob.g2 = 0.1, act.rate = 3,
                     balance = "g1", rec.rate = 1 / 50, rec.rate.g2 = 1 / 50,
                     a.rate = 1 / 100, a.rate.g2 = NA, ds.rate = 1 / 100,
                     ds.rate.g2 = 1 / 100, di.rate = 1 / 90,
                     di.rate.g2 = 1 / 90, dr.rate = 1 / 100,
                     dr.rate.g2 = 1 / 100)
  init <- init.icm(s.num = 500, i.num = 1, r.num = 0,
                   s.num.g2 = 500, i.num.g2 = 0, r.num.g2 = 0)
  control <- control.icm(type = "SIR", nsteps = 100, nsims = 1, verbose = FALSE,
                         verbose.int = 5)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  expect_error(comp_plot(x, at = 100))
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SIR, 2G, OP: 2 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, inf.prob.g2 = 0.1, act.rate = 3,
                     balance = "g1", rec.rate = 1 / 50, rec.rate.g2 = 1 / 50,
                     a.rate = 1 / 100, a.rate.g2 = NA, ds.rate = 1 / 100,
                     ds.rate.g2 = 1 / 100, di.rate = 1 / 90,
                     di.rate.g2 = 1 / 90,  dr.rate = 1 / 100,
                     dr.rate.g2 = 1 / 100)
  init <- init.icm(s.num = 500, i.num = 1, r.num = 0,
                   s.num.g2 = 500, i.num.g2 = 0, r.num.g2 = 0)
  control <- control.icm(type = "SIR", nsteps = 50, nsims = 2, verbose = FALSE,
                         verbose.int = 1)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 3))
  expect_output(summary(x, at = 50), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  expect_error(comp_plot(x, at = 100))
  test_icm(x)
})


# SIS Models --------------------------------------------------------------

test_that("SIS, 1G, CL: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25, rec.rate = 1 / 50)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SIS", nsteps = 100, nsims = 1, verbose = FALSE,
                         verbose.int = 1)
  x <- icm(param, init, control)
  expect_equal(max(x$epi$s.num + x$epi$i.num), 501)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 100)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SIS, 1G, CL: 2 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25, rec.rate = 1 / 50)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SIS", nsteps = 50, nsims = 2, verbose = FALSE)
  x <- icm(param, init, control)
  expect_equal(max(x$epi$s.num + x$epi$i.num), 501)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 3))
  expect_output(summary(x, at = 50), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 50)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SIS, 1G, CL: inf.prob = 0", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0, act.rate = 0.25, rec.rate = 0)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SIS", nsteps = 10, nsims = 10, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 13))
  expect_output(summary(x, at = 10), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 10)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SIS, 2G, CL: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25, inf.prob.g2 = 0.1,
                     balance = "g1", rec.rate = 1 / 100, rec.rate.g2 = 1 / 100)
  init <- init.icm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 0)
  control <- control.icm(type = "SIS", nsteps = 100, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  expect_error(comp_plot(x, at = 100))
  test_icm(x)
})

test_that("SIS, 2G, CL: 2 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25, inf.prob.g2 = 0.1,
                     balance = "g1", rec.rate = 1 / 100, rec.rate.g2 = 1 / 100)
  init <- init.icm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 0)
  control <- control.icm(type = "SIS", nsteps = 50, nsims = 2, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 3))
  expect_output(summary(x, at = 50), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  expect_error(comp_plot(x, at = 50))
  test_icm(x)
})

test_that("SIS, 1G, OP: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.5, rec.rate = 1 / 50,
                     a.rate = 1 / 100, ds.rate = 1 / 100, di.rate = 1 / 90)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SIS", nsteps = 100, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 100)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SIS, 1G, OP: 2 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.5, rec.rate = 1 / 50,
                     a.rate = 1 / 100, ds.rate = 1 / 100, di.rate = 1 / 90)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SIS", nsteps = 50, nsims = 2, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 3))
  expect_output(summary(x, at = 50), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  comp_plot(x, at = 50)
  expect_error(comp_plot(x, at = 200))
  test_icm(x)
})

test_that("SIS, 2G, OP: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, inf.prob.g2 = 0.1, act.rate = 0.5,
                     balance = "g1", rec.rate = 1 / 50, rec.rate.g2 = 1 / 50,
                     a.rate = 1 / 100, a.rate.g2 = NA, ds.rate = 1 / 100,
                     ds.rate.g2 = 1 / 100, di.rate = 1 / 90,
                     di.rate.g2 = 1 / 90)
  init <- init.icm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 1)
  control <- control.icm(type = "SIS", nsteps = 100, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_output(summary(x, at = 100), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  expect_error(comp_plot(x, at = 100))
  test_icm(x)
})

test_that("SIS, 2G, OP: 2 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, inf.prob.g2 = 0.1, act.rate = 0.5,
                     balance = "g1", rec.rate = 1 / 50, rec.rate.g2 = 1 / 50,
                     a.rate = 1 / 100, a.rate.g2 = NA, ds.rate = 1 / 100,
                     ds.rate.g2 = 1 / 100, di.rate = 1 / 90,
                     di.rate.g2 = 1 / 90)
  init <- init.icm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 1)
  control <- control.icm(type = "SIS", nsteps = 50, nsims = 2, verbose = FALSE,
                         verbose.int = 5)
  x <- icm(param, init, control)
  expect_is(x, "icm")
  expect_is(as.data.frame(x), "data.frame")
  expect_is(as.data.frame(x, out = "vals"), "data.frame")
  expect_is(as.data.frame(x, out = "mean"), "data.frame")
  expect_is(as.data.frame(x, out = "sd"), "data.frame")
  expect_error(as.data.frame(x, out = "vals", sim = 3))
  expect_output(summary(x, at = 50), "EpiModel Summary")
  expect_error(summary(x, at = 200))
  plot(x)
  plot(x, y = "i.num")
  plot(x, y = "si.flow", mean.smooth = TRUE)
  expect_error(plot(x, y = "sr.flow"))
  expect_error(comp_plot(x, at = 50))
  test_icm(x)
})

context("ICM Standard Models")

# SI Models ---------------------------------------------------------------

test_that("SI, 1G, CL: 1 sim", {
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SI", nsteps = 5, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
})

test_that("SI, 1G, CL: 2 sims", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, act.rate = 0.25)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SI", nsteps = 5, nsims = 2, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
})

test_that("SI, 1G, OP: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2,
                     act.rate = 0.25, a.rate = 1 / 100,
                     ds.rate = 1 / 100, di.rate = 1 / 90)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SI", nsteps = 5, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
})

test_that("SI, 2G, OP: 2 sims", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2,
                     act.rate = 0.25, inf.prob.g2 = 0.1, balance = "g1",
                     a.rate = 1 / 100, a.rate.g2 = NA, ds.rate = 1 / 100,
                     ds.rate.g2 = 1 / 100, di.rate = 1 / 90,
                     di.rate.g2 = 1 / 90)
  init <- init.icm(s.num = 500, i.num = 1, s.num.g2 = 500, i.num.g2 = 0)
  control <- control.icm(type = "SI", nsteps = 5, nsims = 2, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
})


# SIR Models --------------------------------------------------------------

test_that("SIR, 1G, CL: 1 sim", {
  param <- param.icm(inf.prob = 0.2,
                     act.rate = 0.25, rec.rate = 1 / 50)
  init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.icm(type = "SIR", nsteps = 5, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
})

test_that("SIR, 1G, OP: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2,
                     act.rate = 2, rec.rate = 1 / 50, a.rate = 1 / 100,
                     ds.rate = 1 / 100, di.rate = 1 / 90, dr.rate = 1 / 100)
  init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
  control <- control.icm(type = "SIR", nsteps = 5, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
})

test_that("SIR, 2G, OP: 2 sims", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2, inf.prob.g2 = 0.1,
                     act.rate = 1, balance = "g1",
                     rec.rate = 1 / 50, rec.rate.g2 = 1 / 50,
                     a.rate = 1 / 100, a.rate.g2 = NA,
                     ds.rate = 1 / 100, ds.rate.g2 = 1 / 100,
                     di.rate = 1 / 90, di.rate.g2 = 1 / 90,
                     dr.rate = 1 / 100, dr.rate.g2 = 1 / 100)
  init <- init.icm(s.num = 500, i.num = 1, r.num = 0,
                   s.num.g2 = 500, i.num.g2 = 1, r.num.g2 = 0)
  control <- control.icm(type = "SIR", nsteps = 5, nsims = 2, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
})



# SIS Models --------------------------------------------------------------

test_that("SIS, 1G, CL: 1 sim", {
  param <- param.icm(inf.prob = 0.2,
                     act.rate = 0.25, rec.rate = 1 / 50)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SIS", nsteps = 5, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
})

test_that("SIS, 2G, CL: 2 sims", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2,
                     act.rate = 0.25, inf.prob.g2 = 0.1, balance = "g1",
                     rec.rate = 1 / 100, rec.rate.g2 = 1 / 100)
  init <- init.icm(s.num = 500, i.num = 1,
                   s.num.g2 = 500, i.num.g2 = 0)
  control <- control.icm(type = "SIS", nsteps = 5, nsims = 2, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
})

test_that("SIS, 1G, OP: 1 sim", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2,
                     act.rate = 0.5, rec.rate = 1 / 50,
                     a.rate = 1 / 100, ds.rate = 1 / 100, di.rate = 1 / 90)
  init <- init.icm(s.num = 500, i.num = 1)
  control <- control.icm(type = "SIS", nsteps = 10, nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
})



# Other -------------------------------------------------------------------

test_that("Act rate balance works when specified to g2", {
  skip_on_cran()
  param <- param.icm(inf.prob = 0.2,
                     inf.prob.g2 = 0.1,
                     act.rate.g2 = 0.5,
                     balance = "g2",
                     rec.rate = 1 / 50,
                     rec.rate.g2 = 1 / 50,
                     a.rate = 1 / 100,
                     a.rate.g2 = NA,
                     ds.rate = 1 / 100,
                     ds.rate.g2 = 1 / 100,
                     di.rate = 1 / 90,
                     di.rate.g2 = 1 / 90)
  init <- init.icm(s.num = 50,
                   i.num = 10,
                   s.num.g2 = 100,
                   i.num.g2 = 1)
  control <- control.icm(type = "SIS",
                         nsteps = 10,
                         nsims = 1,
                         verbose = FALSE)
  x <- icm(param, init, control)
  expect_is(x, "icm")
})

context("ICM Extension Models Not Supported")

test_that("ICM rejects custom module arguments", {
  expect_error(
    control.icm(type = "SI", nsteps = 10,
                aging.FUN = function(dat, at) dat),
    "unused argument"
  )
})

context("Stochastic Model Merging")


# merge.icm ---------------------------------------------------------------

test_that("merge for ICM", {
  param <- param.icm(inf.prob = 0.2, act.rate = 0.8)
  init <- init.icm(s.num = 1000, i.num = 100)
  control <- control.icm(type = "SI", nsteps = 10,
                         nsims = 3, verbose = FALSE)
  x <- icm(param, init, control)
  control <- control.icm(type = "SI", nsteps = 10,
                         nsims = 1, verbose = FALSE)
  y <- icm(param, init, control)
  z <- merge(x, y)
  expect_is(z, "icm")
  expect_true(z$control$nsims == 4)
  expect_true(dim(z$epi$i.num)[2] == 4)

  param <- param.icm(inf.prob = 0.2, act.rate = 0.8)
  init <- init.icm(s.num = 1000, i.num = 100)
  control <- control.icm(type = "SI", nsteps = 10,
                         nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  control <- control.icm(type = "SI", nsteps = 10,
                         nsims = 1, verbose = FALSE)
  y <- icm(param, init, control)
  z <- merge(x, y)
  expect_is(z, "icm")
})

test_that("merge 1 sim each", {
  param <- param.icm(inf.prob = 0.2, act.rate = 0.8)
  init <- init.icm(s.num = 1000, i.num = 100)
  control <- control.icm(type = "SI", nsteps = 10,
                         nsims = 1, verbose = FALSE)
  x <- icm(param, init, control)
  control <- control.icm(type = "SI", nsteps = 10,
                         nsims = 1, verbose = FALSE)
  y <- icm(param, init, control)
  z <- merge(x, y)
  expect_is(z, "icm")
})

test_that("merge errors", {
  param <- param.icm(inf.prob = 0.4, act.rate = 0.8)
  init <- init.icm(s.num = 1000, i.num = 100)
  control <- control.icm(type = "SI", nsteps = 10,
                         nsims = 3, verbose = FALSE)
  x <- icm(param, init, control)
  param <- param.icm(inf.prob = 0.2, act.rate = 0.8)
  control <- control.icm(type = "SI", nsteps = 10,
                         nsims = 1, verbose = FALSE)
  y <- icm(param, init, control)
  expect_error(merge(x, y), "x and y have different parameters")
})

# merge.netsim ------------------------------------------------------------

test_that("merge for netsim", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)
  est <- netest(nw, formation = ~edges, target.stats = 25,
                coef.diss = coef.diss, verbose = FALSE)
  param <- param.net(inf.prob = 1)
  init <- init.net(i.num = 1)
  control <- control.net(type = "SI", nsteps = 20, nsims = 2,
                         save.nwstats = TRUE,
                         nwstats.formula = ~edges + degree(0),
                         verbose = FALSE)
  x <- netsim(est, param, init, control)
  y <- netsim(est, param, init, control)
  z <- merge(x, y)
  expect_is(z, "netsim")
  expect_true(z$control$nsims == 4)
  expect_true(dim(z$epi$i.num)[2] == 4)
})

test_that("merge for netsim", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)
  est <- netest(nw, formation = ~edges, target.stats = 25,
                coef.diss = coef.diss, verbose = FALSE)
  param <- param.net(inf.prob = 1)
  init <- init.net(i.num = 1)
  control <- control.net(type = "SI", nsteps = 20, nsims = 2,
                         save.nwstats = TRUE,
                         nwstats.formula = ~edges + degree(0),
                         verbose = FALSE, save.other = "run")
  x <- netsim(est, param, init, control)
  y <- netsim(est, param, init, control)
  z <- merge(x, y, keep.other = TRUE)
  expect_is(z, "netsim")
  expect_true(length(z$run) == 4)
  expect_true(length(z$run[[1]]$attr) == 6)
  z <- merge(x, y, keep.other = FALSE)
  expect_true(any(names(z) == "run") == FALSE)
})

test_that("merge works for open sims saving nw stats", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  est <- netest(nw, formation = ~edges, target.stats = 20,
                coef.diss = dissolution_coefs(~offset(edges), 10, 0.01),
                verbose = FALSE)
  param <- param.net(inf.prob = 0.9, a.rate = 0.01, ds.rate = 0.01,
                     di.rate = 0.01)
  init <- init.net(i.num = 1)
  control <- control.net(type = "SI", nsteps = 5, save.nwstats = TRUE,
                         nwstats.formula =
                           ~edges + meandeg + degree(0) + concurrent,
                         resimulate.network = TRUE, verbose = FALSE)

  x <- netsim(est, param, init, control)
  y <- netsim(est, param, init, control)
  z <- merge(x, y)

  nws <- get_nwstats(z)
  expect_true(nrow(nws) == 10)
  expect_true(length(unique(nws$sim)) == 2)

})

test_that("merge.netsim works as expected for transmat", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)
  est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  # Epidemic model
  param <- param.net(inf.prob = 0.3)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsteps = 5, nsims = 2, verbose = FALSE)
  mod <- netsim(est, param, init, control)

  expect_equal(length(mod$stats$transmat), 2)

  mod2 <- merge(mod, mod)
  expect_equal(length(mod2$stats$transmat), 4)

  mod3 <- merge(mod, mod, keep.transmat = FALSE)
  expect_true(is.null(mod3$stats$transmat))

  mod4 <- merge(mod2, mod3)
  expect_true(is.null(mod4$stats$transmat))
})

test_that("merge.netsim preserves random parameter values", {
  random.params <- list(
    act.rate = function() NULL,
    dummy.strat.param = function() NULL
  )

  make_mod <- function(act.rate, dummy.strat.param) {
    nsims <- length(act.rate)
    simnames <- paste0("sim", seq_len(nsims))
    structure(
      list(
        param = list(
          inf.prob = 0.3,
          act.rate = act.rate[1],
          dummy.strat.param = dummy.strat.param[[1]],
          random.params = random.params,
          random.params.values = list(
            act.rate = act.rate,
            dummy.strat.param = dummy.strat.param
          )
        ),
        control = list(
          nsims = nsims,
          save.other = character(0),
          monitors = NULL,
          nwstats.formula = NULL
        ),
        epi = list(
          i.num = data.frame(
            matrix(seq_len(nsims), nrow = 1, dimnames = list(NULL, simnames))
          )
        ),
        stats = list(nwstats = NULL, transmat = NULL),
        run = NULL,
        network = NULL,
        diss.stats = NULL
      ),
      class = "netsim"
    )
  }

  x <- make_mod(c(0.1, 0.2), list(c(1, 2), c(3, 4)))
  y <- make_mod(c(0.3, 0.4), list(c(5, 6), c(7, 8)))

  z <- merge(x, y)
  d.set <- get_param_set(z)

  expect_equal(z$control$nsims, 4)
  expect_equal(z$param$random.params.values$act.rate, c(0.1, 0.2, 0.3, 0.4))
  expect_equal(z$param$random.params.values$dummy.strat.param,
               list(c(1, 2), c(3, 4), c(5, 6), c(7, 8)))
  expect_equal(d.set$act.rate, c(0.1, 0.2, 0.3, 0.4))
  expect_equal(d.set$dummy.strat.param_1, c(1, 3, 5, 7))
  expect_equal(d.set$dummy.strat.param_2, c(2, 4, 6, 8))
})

test_that("merge and print work as expected for save.other", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)
  est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  # Epidemic model
  param <- param.net(inf.prob = 0.3)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsteps = 5, nsims = 2, verbose = FALSE,
                         tergmLite = TRUE, resimulate.network = TRUE,
                         save.other = c("run", "el"))
  mod <- netsim(est, param, init, control)

  capture_output(
    print(mod)
  )
  expect_output(print(mod), "Other Elements: run el")
  expect_equal(length(mod[["run"]]), 2)
  expect_equal(length(mod[["el"]]), 2)

  mod2 <- merge(mod, mod)
  expect_output(print(mod2), "Other Elements: run el")
  expect_equal(length(mod2[["run"]]), 4)
  expect_equal(length(mod2[["el"]]), 4)

  mod3 <- merge(mod, mod, keep.other = FALSE)
  expect_error(expect_output(print(mod3), "Other Elements"))
  expect_true(is.null(mod3[["run"]]))
  expect_true(is.null(mod3[["el"]]))
})

validate_stats_selection <- function(stats_table, data, stats,
                                     obj_type = "object") {
  sts <- which(!is.na(stats_table[, "Sim Mean"]))
  nmstats <- rownames(stats_table)[sts]

  stats <- if (is.null(stats)) nmstats else stats
  if (!all(stats %in% nmstats)) {
    stop("One or more requested stats not contained in ", obj_type)
  }
  outsts <- which(nmstats %in% stats)
  nmstats <- nmstats[outsts]

  data <- data[, outsts, , drop = FALSE]
  targets <- stats_table$Target[sts][outsts]

  list(data = data, nmstats = nmstats, targets = targets)
}

plot_stats_table <- function(data, nmstats, method,
                             sim.lines,
                             sim.col = NULL, sim.lwd = NULL, mean.line,
                             mean.smooth, mean.col = NULL,
                             mean.lwd, mean.lty, qnts, qnts.col = NULL,
                             qnts.alpha,
                             qnts.smooth, targ.line, targ.col = NULL, targ.lwd,
                             targ.lty, plots.joined = NULL, draw_legend = NULL,
                             grid, targets,
                             dynamic,
                             xlim = NULL, ylim = NULL,
                             xlab = NULL, ylab = NULL,
                             ...) {
  nstats <- length(nmstats)

  if (is.null(plots.joined)) {
    plots.joined <- nstats <= 3
  }

  if (nstats == 1) {
    plots.joined <- TRUE
    sim.col <- "dodgerblue3"
  }

  xlim <- NVL(xlim, c(1, dim(data)[1]))

  xlab <- if (isFALSE(plots.joined)) "" else NVL(xlab, if (isTRUE(dynamic)) "time" else "simulation number")
  ylab <- if (isFALSE(plots.joined)) "" else NVL(ylab, if (nstats == 1) nmstats else "Statistic")

  if (is.null(sim.lwd)) {
    if (dim(data)[3] > 1) {
      sim.lwd <- max(c(1 - (dim(data)[3] * 0.05), 0.5))
    } else {
      sim.lwd <- 1
    }
  }

  ## Color Vector Validation
  # 1. Sim.col, mean.col, qnts.col, targ.col must be missing or a vector of
  #    length 1 or nstats
  # 2. If sim.col, mean.col, qnts.col, or targ.col is not missing
  #    but is a vector of length 1 and nstats is greater than 1,
  #    then replicate the color vector nstats times to achieve a vector of
  #    size nstats.
  check_len_rep <- function(object, default, name) {
    if (!is.null(object)) {
      if (length(object) %in% c(1, nstats)) {
        rep(object, length.out = nstats)
      } else {
        stop(paste0(name, " must be either missing or a vector of length 1 or nstats (", nstats, ")"))
      }
    } else {
      rep(default, length.out = nstats)
    }
  }

  sim.col <- check_len_rep(
    sim.col,
    if (isFALSE(plots.joined)) "dodgerblue3" else seq_len(nstats + 1L)[-1L],
    "sim.col"
  )

  mean.col <- check_len_rep(
    mean.col,
    if (isFALSE(plots.joined)) "black" else sim.col,
    "mean.col"
  )

  qnts.col <- check_len_rep(qnts.col, sim.col, "qnts.col")
  qnts.col <- adjustcolor(qnts.col, qnts.alpha)

  targ.col <- check_len_rep(
    targ.col,
    if (isFALSE(plots.joined) || nstats == 1) "black" else sim.col,
    "targ.col"
  )

  draw_legend <- isTRUE(plots.joined) &&
    ((!is.null(draw_legend) && isTRUE(draw_legend)) ||
       (is.null(draw_legend) && nstats == 1))

  draw_qnts <- isTRUE(dynamic) && is.numeric(qnts)

  mains <- if (isTRUE(plots.joined)) character(nstats) else nmstats

  if (method == "l") {

    qnts_list <- list()
    means <- list()
    ylims <- list()

    for (j in seq_len(nstats)) {
      dataj <- matrix(data[, j, ], nrow = dim(data)[1])

      if (isTRUE(draw_qnts)) {
        qnts_list[[j]] <- get_qnts(dataj, qnts, qnts.smooth)
      }

      if (isTRUE(mean.line)) {
        means[[j]] <- get_means(dataj, mean.smooth)
      }
    }

    for (j in seq_len(nstats)) {
      if (!is.null(ylim)) {
        ylims[[j]] <- ylim
      } else {
        limdat <- c(
          if (isTRUE(plots.joined) && isTRUE(sim.lines)) data,
          if (isFALSE(plots.joined) && isTRUE(sim.lines)) data[, j, ],
          if (isTRUE(plots.joined) && isTRUE(mean.line)) sapply(means, `[[`, "y"),
          if (isFALSE(plots.joined) && isTRUE(mean.line)) means[[j]]$y,
          if (isTRUE(plots.joined) && isTRUE(draw_qnts)) sapply(qnts_list, `[[`, "y"),
          if (isFALSE(plots.joined) && isTRUE(draw_qnts)) qnts_list[[j]]$y,
          if (isTRUE(plots.joined) && isTRUE(targ.line)) targets,
          if (isFALSE(plots.joined) && isTRUE(targ.line)) targets[j]
        )

        ylimsj <- suppressWarnings(c(min(limdat, na.rm = TRUE), max(limdat, na.rm = TRUE)))

        if (any(is.infinite(ylimsj))) {
          ## should both be infinite in this case, indicating no non-missing data to plot;
          ## set both limits to 0 simply to avoid errors when calling plot below
          ylimsj <- c(0, 0)
        } else {
          ## give +/- 10% buffer in a way that works for signed statistics
          ylimsj[1] <- ylimsj[1] - 0.1 * abs(ylimsj[1])
          ylimsj[2] <- ylimsj[2] + 0.1 * abs(ylimsj[2])
        }

        ylims[[j]] <- ylimsj
      }
    }

    if (isFALSE(plots.joined)) {
      if (nstats == 1) dimens <- c(1, 1)
      if (nstats == 2) dimens <- c(1, 2)
      if (nstats == 3) dimens <- c(1, 3)
      if (nstats == 4) dimens <- c(2, 2)
      if (nstats == 5) dimens <- c(2, 3)
      if (nstats == 6) dimens <- c(2, 3)
      if (nstats %in% 7:9) dimens <- c(3, 3)
      if (nstats %in% 10:12) dimens <- c(4, 3)
      if (nstats %in% 13:16) dimens <- c(4, 4)
      if (nstats > 16) dimens <- rep(ceiling(sqrt(nstats)), 2)

      # Pull graphical parameters
      # Register restore handler BEFORE mutating par() so user state is
      # restored on both normal and error exit (CRAN policy).
      ops <- list(mar = par()$mar, mfrow = par()$mfrow, mgp = par()$mgp)
      on.exit(par(ops))
      par(mar = c(2.5, 2.5, 2, 1), mgp = c(2, 1, 0), mfrow = dimens)
    }

    ## do actual plotting
    for (j in seq_len(nstats)) {
      if (j == 1 || isFALSE(plots.joined)) {
        plot(NULL,
             xlim = xlim,
             ylim = ylims[[j]],
             type = "n",
             xlab = xlab,
             ylab = ylab,
             main = mains[j])
      }
      dataj <- matrix(data[, j, ], nrow = dim(data)[1])

      if (isTRUE(draw_qnts)) {
        polygon(qnts_list[[j]]$x, qnts_list[[j]]$y, col = qnts.col[j], border = NA)
      }

      if (isTRUE(sim.lines)) {
        apply(dataj,
              2,
              function(y) lines(which(!is.na(y)), y[!is.na(y)], lwd = sim.lwd, col = sim.col[j]))
      }

      if (isTRUE(mean.line)) {
        lines(means[[j]]$x,
              means[[j]]$y,
              lwd = mean.lwd,
              col = mean.col[j],
              lty = mean.lty)
      }

      if (isTRUE(targ.line)) {
        abline(h = targets[j],
               lty = targ.lty,
               lwd = targ.lwd,
               col = targ.col[j])
      }

      if (isTRUE(grid) && isFALSE(plots.joined)) {
        grid()
      }
    }

    if (isTRUE(grid) && isTRUE(plots.joined)) {
      grid()
    }

    if (isTRUE(draw_legend)) {
      legend("topleft", legend = nmstats, lwd = 2,
             col = sim.col[1:nstats], cex = 0.75, bg = "white")
    }
  }

  if (method == "b") {

    data <- matrix(aperm(data, c(1, 3, 2)), nrow = dim(data)[1] * dim(data)[3])
    colnames(data) <- nmstats

    boxplot(data, ...)

    for (j in seq_len(nstats)) {
      points(x = j, y = targets[j],
             pch = 16, cex = 1.5, col = "blue")

      ## Grid
      if (isTRUE(grid)) {
        grid()
      }
    }
  }
}

#' @title Plot Dynamic Network Model Diagnostics
#'
#' @description Plots dynamic network model diagnostics calculated in
#'              [netdx()].
#'
#' @param x An `EpiModel` object of class `netdx`.
#' @param type Plot type, with options of `"formation"` for network
#'        model formation statistics, `"duration"` for dissolution model
#'        statistics for average edge duration, or `"dissolution"` for
#'        dissolution model statistics for proportion of ties dissolved per time
#'        step.
#' @param method Plot method, with options of `"l"` for line plots and
#'        `"b"` for box plots.
#' @param sims A vector of simulation numbers to plot.
#' @param stats Statistics to plot. For `type = "formation"`, `stats`
#'        are among those specified in the call to [netdx()];
#'        for `type = "duration", "dissolution"`, `stats` are among
#'        those of the dissolution model (without `offset()`). The default
#'        is to plot all statistics.
#' @param plots.joined If `TRUE`, combine all statistics in one plot,
#'        versus one plot per statistic if `FALSE`.
#' @inheritParams plot.netsim
#' @inheritParams graphics::plot
#'
#' @details
#' The plot function for `netdx` objects will generate plots of two types
#' of model diagnostic statistics that run as part of the diagnostic tools
#' within that function. The `formation` plot shows the summary statistics
#' requested in `nwstats.formula`, where the default includes those
#' statistics in the network model formation formula specified in the original
#' call to [netest()].
#'
#' The `duration` plot shows the average age of existing edges at each time
#' step, up until the maximum time step requested. The age is used as an
#' estimator of the average duration of edges in the equilibrium state. When
#' `duration.imputed = FALSE`, edges that exist at the beginning of the
#' simulation are assumed to start with an age of 1, yielding a burn-in period
#' before the observed mean approaches its target.  When
#' `duration.imputed = TRUE`, expected ages prior to the start of the
#' simulation are calculated from the dissolution model, typically eliminating
#' the need for a burn-in period.
#'
#' The `dissolution` plot shows the proportion of the extant ties that are
#' dissolved at each time step, up until the maximum time step requested.
#' Typically, the proportion of ties that are dissolved is the reciprocal of the
#' mean relational duration. This plot thus contains similar information to that
#' in the duration plot, but should reach its expected value more quickly, since
#' it is not subject to censoring.
#'
#' The `plots.joined` argument will control whether the statistics
#' are joined in one plot or plotted separately, assuming there are multiple
#' statistics in the model. The default is based on the number of network
#' statistics requested. The layout of the separate plots within the larger plot
#' window is also based on the number of statistics.
#'
#' @method plot netdx
#' @export
#'
#' @keywords plot
#' @seealso [netdx()]
#'
#' @examples
#' \dontrun{
#' # Network initialization and model parameterization
#' nw <- network_initialize(n = 500)
#' nw <- set_vertex_attribute(nw, "sex", rbinom(500, 1, 0.5))
#' formation <- ~edges + nodematch("sex")
#' target.stats <- c(500, 300)
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges) +
#'                   offset(nodematch("sex")), duration = c(50, 40))
#'
#' # Estimate the model
#' est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' # Static diagnostics
#' dx1 <- netdx(est, nsims = 1e4, dynamic = FALSE,
#'              nwstats.formula = ~edges + meandeg + concurrent +
#'                                 nodefactor("sex", levels = NULL) +
#'                                 nodematch("sex"))
#' dx1
#'
#' # Plot diagnostics
#' plot(dx1)
#' plot(dx1, stats = c("edges", "concurrent"), mean.col = "black",
#'      sim.lines = TRUE, plots.joined = FALSE)
#' plot(dx1, stats = "edges", method = "b",
#'      col = "seagreen3", grid = TRUE)
#'
#' # Dynamic diagnostics
#' dx2 <- netdx(est, nsims = 10, nsteps = 500,
#'              nwstats.formula = ~edges + meandeg + concurrent +
#'                                 nodefactor("sex", levels = NULL) +
#'                                 nodematch("sex"))
#' dx2
#'
#' # Formation statistics plots, joined and separate
#' plot(dx2, grid = TRUE)
#' plot(dx2, type = "formation", plots.joined = TRUE)
#' plot(dx2, type = "formation", sims = 1, plots.joined = TRUE,
#'      qnts = FALSE, sim.lines = TRUE, mean.line = FALSE)
#' plot(dx2, type = "formation", plots.joined = FALSE,
#'      stats = c("edges", "concurrent"), grid = TRUE)
#'
#' plot(dx2, method = "b", col = "bisque", grid = TRUE)
#' plot(dx2, method = "b", stats = "meandeg", col = "dodgerblue")
#'
#' # Duration statistics plot
#' par(mfrow = c(1, 2))
#' # With duration imputed
#' plot(dx2, type = "duration", sim.line = TRUE, sim.lwd = 0.3,
#'      targ.lty = 1, targ.lwd = 0.5)
#' # Without duration imputed
#' plot(dx2, type = "duration", sim.line = TRUE, sim.lwd = 0.3,
#'      targ.lty = 1, targ.lwd = 0.5, duration.imputed = FALSE)
#'
#' # Dissolution statistics plot
#' plot(dx2, type = "dissolution", qnts = 0.25, grid = TRUE)
#' plot(dx2, type = "dissolution", method = "b", col = "pink1")
#' }
#'
plot.netdx <- function(x, type = "formation", method = "l", sims = NULL,
                       stats = NULL, duration.imputed = TRUE, sim.lines = FALSE,
                       sim.col = NULL, sim.lwd = NULL, mean.line = TRUE,
                       mean.smooth = TRUE, mean.col = NULL, mean.lwd = 2,
                       mean.lty = 1, qnts = 0.5, qnts.col = NULL,
                       qnts.alpha = 0.5, qnts.smooth = TRUE, targ.line = TRUE,
                       targ.col = NULL, targ.lwd = 2, targ.lty = 2,
                       plots.joined = NULL, legend = NULL, grid = FALSE,
                       xlim = NULL, xlab = NULL, ylim = NULL, ylab = NULL,
                       ...) {

  type <- match.arg(type, c("formation", "duration", "dissolution"))
  sims <- if (is.null(sims)) seq_len(x$nsims) else sims
  if (max(sims) > x$nsims) stop("Maximum sim number is", x$nsims)

  # Formation Plot -------------------------------------------------------------
  if (type == "formation") {
    stats_table <- x$stats.table.formation
    data <- do.call("cbind", args = x$stats)
    dim3 <- if (x$dynamic) x$nsims else 1L
    data <- array(data, dim = c(dim(data)[1], dim(data)[2] / dim3, dim3))
  } else { # duration/dissolution case -----------------------------------------
    if (!x$dynamic || is.null(x$stats.table.dissolution)) {
      stop(
        "Plots of type duration and dissolution only available if netdx ",
        "run with `dynamic = TRUE` and `skip.dissolution = FALSE`"
      )
    }

    if (x$anyNA) {
      message(
        "\nNOTE: Duration & dissolution data contains undefined values due ",
        "to zero edges of some dissolution dyad type(s) on some time step;",
        " these undefined values will be set to 0 when processing the data."
      )
    }

    if (type == "duration") {
      data <- if (duration.imputed) x$pages_imptd else x$pages
      stats_table <- x$stats.table.duration
    } else { # if type is "dissolution"
      data <- x$prop.diss
      stats_table <- x$stats.table.dissolution
    }
  }

  sel <- validate_stats_selection(stats_table, data, stats, "netdx object")
  data <- sel$data
  nmstats <- sel$nmstats
  targets <- sel$targets
  # sims only used to subset data in dynamic case
  if (x$dynamic) data <- data[, , sims, drop = FALSE]

  plot_stats_table(
    data = data,
    nmstats = nmstats,
    method = method,
    sim.lines = sim.lines,
    sim.col = sim.col,
    sim.lwd = sim.lwd,
    mean.line = mean.line,
    mean.smooth = mean.smooth,
    mean.col = mean.col,
    mean.lwd = mean.lwd,
    mean.lty = mean.lty,
    qnts = qnts,
    qnts.col = qnts.col,
    qnts.alpha = qnts.alpha,
    qnts.smooth = qnts.smooth,
    targ.line = targ.line,
    targ.col = targ.col,
    targ.lwd = targ.lwd,
    targ.lty = targ.lty,
    plots.joined = plots.joined,
    draw_legend = legend,
    grid = grid,
    targets = targets,
    dynamic = x$dynamic,
    xlim = xlim, xlab = xlab,
    ylim = ylim, ylab = ylab,
    ...
  )
}

#' @title Fast Version of network::add.vertices for Edgelist-formatted Network
#'
#' @description This function performs a simple operation of updating the
#'              edgelist attribute `n` that tracks the total network
#'              size implicit in an edgelist representation of the network.
#'
#' @param el A two-column matrix of current edges (edgelist) with an attribute
#'           variable `n` containing the total current network size.
#' @param nv A integer equal to the number of nodes to add to the network
#'           size at the given time step.
#'
#' @details
#' This function is used in `EpiModel` modules to add vertices (nodes) to
#' the edgelist object to account for entries into the population (e.g., births
#' and in-migration).
#'
#' @return
#' Returns the matrix of current edges, `el`, with the population size
#' attribute updated based on the number of new vertices specified in `nv`.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' library("EpiModel")
#' nw <- network_initialize(100)
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#' x <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' param <- param.net(inf.prob = 0.3)
#' init <- init.net(i.num = 10)
#' control <- control.net(type = "SI", nsteps = 100, nsims = 5,
#'                        tergmLite = TRUE)
#'
#' # networkLite representation after initialization
#' dat <- crosscheck.net(x, param, init, control)
#' dat <- initialize.net(x, param, init, control)
#'
#' # Check current network size
#' attributes(dat$el[[1]])$n
#'
#' # Add 10 vertices
#' dat$el[[1]] <- add_vertices(dat$el[[1]], 10)
#'
#' # Check new network size
#' attributes(dat$el[[1]])$n
#' }
#'
add_vertices <- function(el, nv) {
  attributes(el)$n <- attributes(el)$n + nv
  return(el)
}


#' @title Fast Version of network::delete.vertices for Edgelist-formatted
#'        Network
#'
#' @description Given a current two-column matrix of edges and a vector of IDs
#'              to delete from the matrix, this function first removes any rows
#'              of the edgelist in which the IDs are present and then permutes
#'              downward the index of IDs on the edgelist that were numerically
#'              larger than the IDs deleted.
#'
#' @param el A two-column matrix of current edges (edgelist) with an attribute
#'           variable `n` containing the total current network size.
#' @param vid A vector of IDs to delete from the edgelist.
#'
#' @details
#' This function is used in `EpiModel` modules to remove vertices (nodes)
#' from the edgelist object to account for exits from the population (e.g.,
#' deaths and out-migration).
#'
#' @return
#' Returns an updated edgelist object, `el`, with the edges of deleted
#' vertices removed from the edgelist and the ID numbers of the remaining edges
#' permuted downward.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' library("EpiModel")
#' set.seed(12345)
#' nw <- network_initialize(100)
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#' x <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' param <- param.net(inf.prob = 0.3)
#' init <- init.net(i.num = 10)
#' control <- control.net(type = "SI", nsteps = 100, nsims = 5,
#'                        tergmLite = TRUE)
#'
#' # Set seed for reproducibility
#' set.seed(123456)
#'
#' # networkLite representation structure after initialization
#' dat <- crosscheck.net(x, param, init, control)
#' dat <- initialize.net(x, param, init, control)
#'
#' # Current edges
#' head(dat$el[[1]], 20)
#'
#' # Remove nodes 1 and 2
#' nodes.to.delete <- 1:2
#' dat$el[[1]] <- delete_vertices(dat$el[[1]], nodes.to.delete)
#'
#' # Newly permuted edges
#' head(dat$el[[1]], 20)
#' }
#'
delete_vertices <- function(el, vid) {

  vid <- sort(vid)

  new.el <- delete_edges(el, vid)
  if (length(vid) > 0 && NROW(new.el) > 0) {
    o1 <- order(new.el[, 1])
    new.el[, 1] <- shiftVec(new.el[o1, 1], vid)[order(o1)]
    o2 <- order(new.el[, 2])
    new.el[, 2] <- shiftVec(new.el[o2, 2], vid)[order(o2)]
  }
  if (!is.null(attr(el, "n"))) attr(new.el, "n") <- attr(el, "n") - length(vid)

  return(new.el)
}


#' @title Remove Edges That Include Specified Vertices
#'
#' @description Given a current two-column matrix of edges and a vector of
#'              vertex IDs, this function removes any rows of the edgelist in
#'              which the IDs are present.
#'
#' @param el A two-column matrix of current edges (edgelist).
#' @param vid A vector of vertex IDs whose edges are to be deleted from the
#'            edgelist.
#'
#' @return
#' Returns an updated edgelist object, with any edges including the specified
#' vertices removed.
#'
#' @export
#'
delete_edges <- function(el, vid) {
  new.el <- el
  if (length(vid) > 0) {
    el.rows.to.del <- which(el[, 1] %in% vid | el[, 2] %in% vid)
    if (length(el.rows.to.del) > 0) {
      new.el <- el[-el.rows.to.del, , drop = FALSE]
      if (!is.null(attr(el, "n"))) attr(new.el, "n") <- attr(el, "n")
    }
  }
  new.el
}


#' Depart Nodes from the netsim_dat Object
#'
#' @param dat the `netsim_dat` object
#' @param departures the vertex ids of nodes to depart
#'
#' @details If `tergmLite` is `FALSE`, the vertex ids
#' `departures` are deactivated (from the current timestep onward) in each
#' `networkDynamic` stored in `dat$nw`. If `tergmLite` is
#' `TRUE`, the vertex ids `departures` are deleted from `dat$el`,
#' `dat$attr`, and `dat$net_attr`.
#'
#' @return the updated `netsim_dat` object with the nodes in
#' `departures` departed
#'
#' @export
#'
depart_nodes <- function(dat, departures) {
  if (length(departures) > 0) {
    if (get_control(dat, "tergmLite") == FALSE) {
      for (net_index in seq_len(dat$num.nw)) {
        net <- get_network(dat, network = net_index)
        net <- deactivate.vertices(
          net,
          onset = get_current_timestep(dat),
          terminus = Inf,
          v = departures,
          deactivate.edges = TRUE
        )
        dat <- set_network(dat, nw = net, network = net_index)
      }
    } else {
      dat <- delete_attr(dat, departures)
      for (network in seq_len(dat$num.nw)) {
        dat$run$el[[network]] <- delete_vertices(
          dat$run$el[[network]],
          departures
        )
        dat$run$net_attr[[network]][["n"]] <-
          dat$run$net_attr[[network]][["n"]] - length(departures)

        if (get_network_control(dat, network, "tergmLite.track.duration")) {
          dat$run$net_attr[[network]][["lasttoggle"]] <- delete_vertices(
            dat$run$net_attr[[network]][["lasttoggle"]],
            departures
          )
        }
      }
    }
  }
  return(dat)
}


#' Arrive New Nodes to the netsim_dat Object
#'
#' @param dat the `netsim_dat` object
#' @param nArrivals number of new nodes to arrive
#'
#' @details `nArrivals` new nodes are added to the network data stored on
#' the `netsim_dat` object. If `tergmLite` is `FALSE`, these
#' nodes are activated from the current timestep onward. Attributes for the new
#' nodes must be set separately.
#'
#' Note that this function only supports arriving new nodes; returning to an
#' active state nodes that were previously active in the network is not
#' supported.
#'
#' @return the updated `netsim_dat` object with `nArrivals` new nodes
#' added
#'
#'
#' @export
#'
arrive_nodes <- function(dat, nArrivals) {
  if (nArrivals > 0) {
    if (!get_control(dat, "tergmLite")) {
      for (net_index in seq_len(dat$num.nw)) {
        net <- get_network(dat, network = net_index)
        net <- add.vertices.active(
          net,
          nv = nArrivals,
          onset = get_current_timestep(dat),
          terminus = Inf
        )
        dat <- set_network(dat, nw = net, network = net_index)
      }
    } else {
      for (net_index in seq_len(dat$num.nw)) {
        el <- add_vertices(dat$run$el[[net_index]], nv = nArrivals)
        dat$run$el[[net_index]] <- el
        dat$run$net_attr[[net_index]][["n"]] <-
          dat$run$net_attr[[net_index]][["n"]] + nArrivals
      }
    }
  }
  return(dat)
}

#' @title Get Epidemic Output from icm Model
#'
#' @description This function provides all active model state sizes from
#'              the network at the specified time step, output to a list of
#'              vectors.
#'
#' @param dat Main `icm_dat` class data object passed through `icm`
#'        simulations.
#' @param at Current time step.
#'
#' @inherit recovery.icm return
#'
#' @export
#' @keywords internal
#'
prevalence.icm <- function(dat, at) {

  if (at == 1) {
    dat$epi <- list()
    dat$epi$s.num <- sum(dat$attr$active == 1 &
                           dat$attr$status == "s" &
                           dat$attr$group == 1)
    dat$epi$i.num <- sum(dat$attr$active == 1 &
                           dat$attr$status == "i" &
                           dat$attr$group == 1)
    dat$epi$num <- dat$epi$s.num + dat$epi$i.num
    if (dat$control$type == "SIR") {
      dat$epi$r.num <- sum(dat$attr$active == 1 &
                             dat$attr$status == "r" &
                             dat$attr$group == 1)
      dat$epi$num <- dat$epi$s.num +
        dat$epi$i.num +
        dat$epi$r.num
    }
  } else {
    dat$epi$s.num[at] <- sum(dat$attr$active == 1 &
                               dat$attr$status == "s" &
                               dat$attr$group == 1)
    dat$epi$i.num[at] <- sum(dat$attr$active == 1 &
                               dat$attr$status == "i" &
                               dat$attr$group == 1)
    dat$epi$num[at] <- dat$epi$s.num[at] + dat$epi$i.num[at]
    if (dat$control$type == "SIR") {
      dat$epi$r.num[at] <- sum(dat$attr$active == 1 &
                                 dat$attr$status == "r" &
                                 dat$attr$group == 1)
      dat$epi$num[at] <- dat$epi$s.num[at] +
        dat$epi$i.num[at] +
        dat$epi$r.num[at]
    }
  }

  return(dat)
}


#' @title Get Epidemic Output from icm Model
#'
#' @description This function provides all active model state sizes from
#'              the network at the specified time step, output to a list of
#'              vectors.
#'
#' @inheritParams prevalence.icm
#'
#' @inherit recovery.icm return
#'
#' @export
#' @keywords internal
#'
prevalence.icm.bip <- function(dat, at) {

  if (at == 1) {
    dat$epi <- list()
    dat$epi$s.num <- sum(dat$attr$active == 1 &
                           dat$attr$status == "s" &
                           dat$attr$group == 1)
    dat$epi$i.num <- sum(dat$attr$active == 1 &
                           dat$attr$status == "i" &
                           dat$attr$group == 1)
    dat$epi$num <- dat$epi$s.num + dat$epi$i.num
    dat$epi$s.num.g2 <- sum(dat$attr$active == 1 &
                              dat$attr$status == "s" &
                              dat$attr$group == 2)
    dat$epi$i.num.g2 <- sum(dat$attr$active == 1 &
                              dat$attr$status == "i" &
                              dat$attr$group == 2)
    dat$epi$num.g2 <- dat$epi$s.num.g2 + dat$epi$i.num.g2
    if (dat$control$type == "SIR") {
      dat$epi$r.num <- sum(dat$attr$active == 1 &
                             dat$attr$status == "r" &
                             dat$attr$group == 1)
      dat$epi$num <- dat$epi$s.num +
        dat$epi$i.num +
        dat$epi$r.num
      dat$epi$r.num.g2 <- sum(dat$attr$active == 1 &
                                dat$attr$status == "r" &
                                dat$attr$group == 2)
      dat$epi$num.g2 <- dat$epi$s.num.g2 +
        dat$epi$i.num.g2 +
        dat$epi$r.num.g2
    }
  } else {
    dat$epi$s.num[at] <- sum(dat$attr$active == 1 &
                               dat$attr$status == "s" &
                               dat$attr$group == 1)
    dat$epi$i.num[at] <- sum(dat$attr$active == 1 &
                               dat$attr$status == "i" &
                               dat$attr$group == 1)
    dat$epi$num[at] <- dat$epi$s.num[at] + dat$epi$i.num[at]
    dat$epi$s.num.g2[at] <- sum(dat$attr$active == 1 &
                                  dat$attr$status == "s" &
                                  dat$attr$group == 2)
    dat$epi$i.num.g2[at] <- sum(dat$attr$active == 1 &
                                  dat$attr$status == "i" &
                                  dat$attr$group == 2)
    dat$epi$num.g2[at] <- dat$epi$s.num.g2[at] + dat$epi$i.num.g2[at]
    if (dat$control$type == "SIR") {
      dat$epi$r.num[at] <- sum(dat$attr$active == 1 &
                                 dat$attr$status == "r" &
                                 dat$attr$group == 1)
      dat$epi$num[at] <- dat$epi$s.num[at] +
        dat$epi$i.num[at] +
        dat$epi$r.num[at]
      dat$epi$r.num.g2[at] <- sum(dat$attr$active == 1 &
                                    dat$attr$status == "r" &
                                    dat$attr$group == 2)
      dat$epi$num.g2[at] <- dat$epi$s.num.g2[at] +
        dat$epi$i.num.g2[at] +
        dat$epi$r.num.g2[at]
    }
  }

  return(dat)
}

#' @title Definition for absdiffnodemix ERGM Term
#'
#' @description This function defines and initializes the absdiffnodemix ERGM
#'              term that allows for targeting homophily based on a non-binary
#'              attribute (e.g., age) by combinations of a binary attribute
#'              (e.g., race).
#'
#' @param nw An object of class `network`.
#' @param arglist A list of arguments as specified in the `ergm.userterms`
#'        package framework.
#' @param ... Additional data passed into the function as specified in the
#'        `ergm.userterms` package framework.
#'
#' @details
#' This ERGM user term was written to allow for age-based homophily in
#' partnership formation that is heterogeneous by race. The `absdiff`
#' component targets the distribution of age mixing on that continuous
#' variable, and the `nodemix` component differentiates this for
#' black-black, black-white, and white-white couples.
#'
#' @aliases absdiffnodemix
#'
InitErgmTerm.absdiffnodemix <- function(nw, arglist, ...) {

  a <- check.ErgmTerm(nw,
                      arglist,
                      directed = FALSE,
                      bipartite = FALSE,
                      varnames = c("attr", "by"),
                      vartypes = c(ERGM_VATTR_SPEC, ERGM_VATTR_SPEC),
                      defaultvalues = list(NULL, NULL),
                      required = c(TRUE, TRUE))

  nodecov <- ergm_get_vattr(a$attr, nw, accept = "numeric")
  nodecovby <- ergm_get_vattr(a$by, nw)
  nodecovbyname <- attr(nodecovby, "name")
  u <- sort(unique(nodecovby))
  if (any(is.na(nodecovby))) {
    u <- c(u, NA)
  }

  nodecovby <- match(nodecovby, u, nomatch = length(u) + 1)
  ui <- seq(along = u)

  uui <- matrix(seq_along(ui) ^ 2, length(ui), length(ui))
  urm <- t(sapply(ui, rep, length(ui)))
  ucm <- sapply(ui, rep, length(ui))
  uun <- outer(u, u, paste, sep = ".")
  uui <- uui[upper.tri(uui, diag = TRUE)]
  urm <- urm[upper.tri(urm, diag = TRUE)]
  ucm <- ucm[upper.tri(ucm, diag = TRUE)]
  uun <- uun[upper.tri(uun, diag = TRUE)]

  inputs <-  c(length(nodecov), length(urm), nodecov, nodecovby, urm, ucm)

  list(name = "absdiffnodemix",
       coef.names = paste("absdiffnodemix", attr(nodecov, "name"),
                          nodecovbyname, uun, sep = "."),
       pkgname = "EpiModel",
       inputs = inputs,
       dependence = FALSE)
}


#' @title Definition for absdiffby ERGM Term
#'
#' @description This function defines and initializes the absdiffby ERGM term
#'              that allows for representing homophily with respect to a
#'              non-binary attribute (e.g., age) differentially by a binary
#'              attribute (e.g., sex).
#'
#' @param nw An object of class `network`.
#' @param arglist A list of arguments as specified in the `ergm.userterms`
#'        package framework.
#' @param ... Additional data passed into the function as specified in the
#'        `ergm.userterms` package framework.
#'
#' @details
#' This ERGM user term was written to allow for age-based homophily in
#' partnership formation that is asymmetric by sex. The `absdiff` component
#' targets age-based homophily while the `by` component allows that to be
#' structured by a binary attribute such as "male", in order to enforce an
#' offset in the average difference. This allows, for example, a average age
#' difference in partnerships, but with males (on average) older than females.
#'
#' @aliases absdiffby
#'
InitErgmTerm.absdiffby <- function(nw, arglist, ...) {
  a <- check.ErgmTerm(nw,
                      arglist,
                      directed = FALSE,
                      bipartite = FALSE,
                      varnames = c("attr", "by", "assym"),
                      vartypes = c(ERGM_VATTR_SPEC, ERGM_VATTR_SPEC, "numeric"),
                      required = c(TRUE, TRUE, TRUE),
                      defaultvalues = list(NULL, NULL, NULL))

  nodecov <- ergm_get_vattr(a$attr, nw, accept = "numeric")
  nodeby <- ergm_get_vattr(a$by, nw)
  coef.names <- paste("absdiffby", attr(nodecov, "name"),
                      attr(nodeby, "name"), sep = ".")

  list(name = "absdiffby",
    coef.names = coef.names,
    pkgname = "EpiModel",
    inputs = c(a$assym, nodecov, nodeby),
    dependence = FALSE,
    emptynwstats = 0
  )
}


#' @title Definition for fuzzynodematch ERGM Term
#'
#' @description This function defines and initializes the fuzzynodematch ERGM
#'              term that allows for generalized homophily.
#'
#' @param nw An object of class `network`.
#' @param arglist A list of arguments as specified in the `ergm.userterms`
#'        package framework.
#' @param ... Additional data passed into the function as specified in the
#'        `ergm.userterms` package framework.
#'
#' @details
#' This ERGM user term was written to allow for generalized homophily.The
#' `attr` term argument should specify a character vertex attribute
#' encoding the "venues" associated to each node.  The `split` argument
#' should specify a string that separates different "venues" in the attribute
#' value for each node, as handled by `strsplit` with `fixed = TRUE`.
#' For example, if `split` is `"|"` (the default), and the attribute
#' value for a given node is `"a12|b476"`, then the associated venues for
#' this node are `"a12"` and `"b476"`.  The empty string `""` is
#' interpreted as "no venues".
#'
#' If the `binary` term argument is `FALSE` (the default), the change
#' statistic for an on-toggle is the number of unique venues associated to both
#' nodes (informally speaking, this could be described as the number of venues
#' on which the two nodes "match"); if `binary` is `TRUE`, the change
#' statistic for an on-toggle is `1` if any venue is associated to both
#' nodes, and `0` otherwise.
#'
#' @aliases fuzzynodematch
#'
InitErgmTerm.fuzzynodematch <- function(nw, arglist, ...) {
  a <- check.ErgmTerm(nw, arglist,
                      varnames = c("attr", "split", "binary"),
                      vartypes = c(ERGM_VATTR_SPEC, "character", "logical"),
                      defaultvalues = list(NULL, "|", FALSE),
                      required = c(TRUE, FALSE, FALSE))

  nodecov <- ergm_get_vattr(a$attr, nw, accept = "character")
  venues <- strsplit(nodecov, split = a$split, fixed = TRUE)

  ## drop "" from venues and enforce uniqueness of venues for each node
  venues <- lapply(venues, function(x) unique(x[nchar(x) > 0L]))

  ## record number of venues and offset in position for each node
  lengths <- unlist(lapply(venues, length))
  positions <- cumsum(lengths) - lengths

  ## convert venues to vector
  venues <- unlist(venues)

  ## convert venues from strings to integers
  levels <- sort(unique(venues))
  venues <- match(venues, levels)

  ## sort venues for each node
  venues <- unlist(lapply(seq_len(network.size(nw)), function(i) sort(venues[positions[i] + seq_len(lengths[i])])))

  binary <- a$binary

  list(name = "fuzzynodematch",
       coef.names = paste("fuzzynodematch", attr(nodecov, "name"),
                          binary, sep = "."),
       binary = as.integer(binary),
       venues = as.integer(venues),
       lengths = as.integer(lengths),
       positions = as.integer(positions),
       dependence = FALSE,
       minval = 0)
}

#' @title Dynamic Network Model Diagnostics
#'
#' @description Runs diagnostic simulations on an ERGM/STERGM estimated with
#'              [`netest`] to assess whether the fitted model reproduces the
#'              intended network features. Both static (cross-sectional) and
#'              dynamic (temporal) diagnostics are supported. This is the
#'              recommended second step in the network modeling pipeline, after
#'              estimation with [`netest`] and before epidemic simulation with
#'              [`netsim`].
#'
#' @param x An `EpiModel` object of class `netest`.
#' @param nsims Number of simulations to run. For dynamic diagnostics, 5--10
#'        simulations are usually sufficient to assess model fit. For static
#'        diagnostics, use 10,000+ draws to obtain stable estimates.
#' @param dynamic If `TRUE`, runs dynamic diagnostics that simulate the
#'        temporal network forward in time, checking both formation targets
#'        and partnership duration/dissolution. If `FALSE`, draws from the
#'        static ERGM fit to check cross-sectional network structure only
#'        (faster, but does not verify dissolution dynamics). Static
#'        diagnostics are only available when the model was fit with the
#'        edges dissolution approximation (`edapprox = TRUE` in [`netest`]).
#' @param nsteps Number of time steps per simulation (dynamic simulations
#'        only). Should be at least several multiples of the longest target
#'        partnership duration to allow the duration and dissolution statistics
#'        to stabilize. For example, if the target duration is 50, running for
#'        500 time steps is a reasonable starting point.
#' @param nwstats.formula A right-hand sided ERGM formula with the network
#'        statistics of interest. The default is the formation formula of the
#'        network model contained in `x`. You may track additional network
#'        statistics beyond the formation terms by specifying them here, such
#'        as `~ edges + meandeg + concurrent + degree(0:4)`. This is useful
#'        for verifying that the model produces reasonable values for network
#'        features that were not directly targeted in the formation model.
#' @param set.control.ergm Control arguments passed to `ergm`'s
#'        `simulate_formula.network` (see details).
#' @param set.control.tergm Control arguments passed to `tergm`'s
#'        `simulate_formula.network` (see details).
#' @param sequential For static diagnostics (`dynamic=FALSE`): if
#'        `FALSE`, each of the `nsims` simulated Markov chains begins
#'        at the initial network; if `TRUE`, the end of one simulation is
#'        used as the start of the next.
#' @param keep.tedgelist If `TRUE`, keep the timed edgelist generated from
#'        the dynamic simulations. Returned in the form of a list of matrices,
#'        with one entry per simulation. Accessible at `$edgelist`.
#' @param keep.tnetwork If `TRUE`, keep the full networkDynamic objects
#'        from the dynamic simulations. Returned in the form of a list of nD
#'        objects, with one entry per simulation. Accessible at `$network`.
#' @param verbose If `TRUE`, print progress to the console.
#' @param ncores Number of processor cores to run multiple simulations
#'        on, using the `future` framework.
#' @param skip.dissolution If `TRUE`, skip over the calculations of
#'        duration and dissolution stats in `netdx`.
#' @param future.use.plan If `FALSE`, `netdx` will use `multisession` is used with `workers = ncores for its
#'        parallelization. If `TRUE`, `netdx` will use the user defined plan from `globalEnv`. Finally, it can
#'        take the output of a `future::tweak()` call to setup a user defined temporary plan within `netdx`.
#'        Which can be useful for distributed computation (HPC).
#'
#' @details
#' The `netdx` function handles dynamic network diagnostics for network
#' models fit with the [`netest`] function. Given the fitted model,
#' `netdx` simulates a specified number of dynamic networks for a specified
#' number of time steps per simulation. The network statistics in
#' `nwstats.formula` are saved for each time step. Summary statistics for
#' the formation model terms, as well as dissolution model and relational
#' duration statistics, are then calculated and can be accessed when printing or
#' plotting the `netdx` object. See [`print.netdx`] and [`plot.netdx`]
#' for details on printing and plotting.
#'
#' @section Control Arguments:
#' Models fit with the full STERGM method in `netest` (setting the
#' `edapprox` argument to `FALSE`) require only a call to
#' `tergm`'s `simulate_formula.network`. Control parameters for those
#' simulations may be set using `set.control.tergm` in `netdx`.
#' The parameters should be input through the `control.simulate.formula.tergm`
#' function, with the available parameters listed in the
#' [`tergm::control.simulate.formula.tergm`] help page in the `tergm`
#' package.
#'
#' Models fit with the ERGM method with the edges dissolution approximation
#' (setting `edapprox` to `TRUE`) require a call first to
#' `ergm`'s `simulate_formula.network` for simulating an initial
#' network, and second to `tergm`'s `simulate_formula.network` for
#' simulating that static network forward through time. Control parameters may
#' be set for both processes in `netdx`. For the first, the parameters
#' should be input through the `control.simulate.formula()` function, with
#' the available parameters listed in the
#' [`ergm::control.simulate.formula`] help
#' page in the `ergm` package. For the second, parameters should be input
#' through the `control.simulate.formula.tergm()` function, with the
#' available parameters listed in the [`tergm::control.simulate.formula.tergm`]
#' help page in the `tergm` package. An example is shown below.
#'
#' @section Static vs. Dynamic Diagnostics:
#' Static diagnostics (`dynamic = FALSE`) draw many independent networks from
#' the fitted ERGM and compare the resulting statistics to the target values.
#' This is fast and checks whether the cross-sectional structure is correct,
#' but it does not verify partnership durations or dissolution rates. Dynamic
#' diagnostics (`dynamic = TRUE`) simulate the full temporal network forward
#' in time, checking both formation targets and dissolution/duration dynamics.
#' Dynamic diagnostics are slower but more comprehensive, and are required to
#' verify models that will be used with vital dynamics (arrivals/departures).
#'
#' @section Interpreting Diagnostics:
#' After running `netdx`, use `print()` and [`plot.netdx`] to inspect the
#' results. Key indicators of a good model fit include:
#'
#'  * **Formation statistics:** The "Sim Mean" should be close to the
#'    "Target" value. A small "Pct Diff" (< 5\%) and a "Z Score" near 0
#'    indicate good fit.
#'  * **Duration statistics** (dynamic only): The simulated mean edge
#'    durations should match the values passed to [`dissolution_coefs`].
#'  * **Dissolution statistics** (dynamic only): The simulated dissolution
#'    rates should be approximately `1 / duration`.
#'
#' Common problems: If formation statistics are off, the ERGM may need
#' increased burn-in (via `set.control.ergm`), or the target statistics
#' may be incompatible (e.g., specifying more edges than the network can
#' support). If durations are off but formation is correct, verify that
#' `d.rate` was correctly specified in [`dissolution_coefs`] for models
#' with vital dynamics.
#'
#' @return
#' A list of class `netdx`. Use `print()` to view summary tables of
#' formation statistics, duration, and dissolution diagnostics. Use
#' [`plot.netdx`] to visualize these diagnostics over time. Use
#' [as.data.frame.netdx()] to extract timed edgelists (if
#' `keep.tedgelist = TRUE`).
#'
#' @seealso
#' Estimate the network model with [`netest`] before running diagnostics.
#' Plot diagnostics with [`plot.netdx`] and print summary tables with
#' [`print.netdx`]. After diagnostics confirm a good fit, simulate the
#' epidemic with [`netsim`].
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Static diagnostics on a simple model
#' nw <- network_initialize(n = 100)
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 25)
#' est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#' dx <- netdx(est, nsims = 1e4, dynamic = FALSE, verbose = FALSE)
#' dx
#' plot(dx)
#' }
#'
#' \dontrun{
#' # Static diagnostics with additional network statistics
#' dx1 <- netdx(est,
#'   nsims = 1e4, dynamic = FALSE,
#'   nwstats.formula = ~ edges + meandeg + concurrent
#' )
#' dx1
#' plot(dx1, method = "b", stats = c("edges", "concurrent"))
#'
#' # Dynamic diagnostics on the STERGM approximation
#' dx2 <- netdx(est,
#'   nsims = 5, nsteps = 500,
#'   nwstats.formula = ~ edges + meandeg + concurrent,
#'   set.control.ergm = control.simulate.formula(MCMC.burnin = 1e6)
#' )
#' dx2
#' plot(dx2, stats = c("edges", "meandeg"), plots.joined = FALSE)
#' plot(dx2, type = "duration")
#' plot(dx2, type = "dissolution", qnts.col = "orange2")
#' plot(dx2, type = "dissolution", method = "b", col = "bisque")
#'
#' # Dynamic diagnostics on a more complex model
#' nw <- network_initialize(n = 1000)
#' nw <- set_vertex_attribute(nw, "neighborhood", rep(1:10, 100))
#' formation <- ~edges + nodematch("neighborhood", diff = TRUE)
#' target.stats <- c(800, 45, 81, 24, 16, 32, 19, 42, 21, 24, 31)
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges) +
#'                     offset(nodematch("neighborhood", diff = TRUE)),
#'                     duration = c(52, 58, 61, 55, 81, 62, 52, 64, 52, 68, 58))
#' est2 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#' dx3 <- netdx(est2, nsims = 5, nsteps = 100)
#' print(dx3)
#' plot(dx3)
#' plot(dx3, type = "duration", plots.joined = TRUE, qnts = 0.2, legend = TRUE)
#' plot(dx3, type = "dissolution", mean.smooth = FALSE, mean.col = "red")
#' }
#'
netdx <- function(x, nsims = 1, dynamic = TRUE, nsteps = NULL,
                  nwstats.formula = "formation",
                  set.control.ergm = control.simulate.formula(),
                  set.control.tergm = control.simulate.formula.tergm(MCMC.maxchanges = .Machine$integer.max),
                  sequential = TRUE, keep.tedgelist = FALSE,
                  keep.tnetwork = FALSE, verbose = TRUE, ncores = 1,
                  skip.dissolution = FALSE, future.use.plan = FALSE) {

  if (!inherits(x, "netest")) {
    stop("x must be an object of class netest")
  }

  ncores <- if (nsims == 1) 1 else cap_ncores(ncores)

  formation <- x$formation
  coef.form <- x$coef.form
  dissolution <- x$coef.diss$dissolution
  coef.diss <- x$coef.diss
  constraints <- x$constraints
  target.stats <- x$target.stats
  edapprox <- x$edapprox
  nw <- x$newnetwork

  if (dynamic && is.null(nsteps)) {
    stop("Specify number of time steps with nsteps")
  }

  if (x$coef.diss$duration[1] == 1 && dynamic == TRUE) {
    stop("Running dynamic diagnostics on a cross-sectional ERGM (duration = 1) is not possible.
         \nSet netdx parameter 'dynamic' to 'FALSE'"
    )
  }

  if (dynamic == FALSE && nwstats.formula == "formation") {
    nwstats.formula <- x$formation
  }

  if (dynamic == FALSE && edapprox == FALSE) {
    stop("cannot perform static simulation without edapprox")
  }

  if (verbose == TRUE) {
    cat("\nNetwork Diagnostics")
    cat("\n-----------------------\n")
  }

  if (verbose == TRUE) {
    if (nsims == 1) {
      cat("- Simulating 1 network")
    } else {
      cat("- Simulating", nsims, "networks")
    }
  }

  dosim <- function() {
    if (edapprox == TRUE) {
      init <- simulate(x$formula,
                       coef = x$coef.form.crude,
                       basis = x$newnetwork,
                       constraints = x$constraints,
                       control = set.control.ergm,
                       dynamic = FALSE,
                       nsim = if (dynamic == TRUE) 1L else nsims,
                       output = if (dynamic == TRUE) "network" else "stats",
                       sequential = if (dynamic == TRUE) FALSE else sequential,
                       monitor = if (dynamic == TRUE) NULL else nwstats.formula)
    } else {
      init <- x$newnetwork
    }

    if (dynamic == FALSE) {
      stats <- init
      attr(stats, "ess") <- ess(stats)
      return(list(stats = stats))
    }

    if (keep.tedgelist == TRUE || keep.tnetwork == TRUE) {
      output <- "networkDynamic"
    } else {
      output <- "changes"
    }

    init %n% "time" <- 0L
    init %n% "lasttoggle" <- cbind(as.edgelist(init), 0L)

    diag.sim <- simulate(init ~ Form(x$formation) + Persist(x$coef.diss$dissolution),
                         coef = c(x$coef.form, x$coef.diss$coef.crude),
                         constraints = x$constraints,
                         time.slices = nsteps,
                         monitor = nwstats.formula,
                         time.start = 0L,
                         nsim = 1L,
                         output = output,
                         control = set.control.tergm,
                         dynamic = TRUE)

    stats <- attr(diag.sim, "stats")
    attr(stats, "ess") <- ess(stats)
    out <- list(stats = stats)

    if (output == "networkDynamic") {
      sim.df <- as.data.frame(diag.sim)
      toggles <- tedgelist_to_toggles(sim.df)
    } else {
      changes <- diag.sim
      if (network.edgecount(init) > 0L) {
        changes <- rbind(cbind(0L, as.edgelist(init), 1L),
                         changes)
      }
      toggles <- changes[, -4L, drop = FALSE]
    }

    if (keep.tnetwork == TRUE) {
      out$tnetwork <- diag.sim
    }

    if (keep.tedgelist == TRUE) {
      out$tedgelist <- sim.df
    }

    if (skip.dissolution == FALSE) {
      out <- c(out, toggles_to_diss_stats(toggles, x$coef.diss, nsteps, init))
    }

    out
  }

  if (dynamic == FALSE || nsims == 1) {
    diag.sim <- list(dosim())
  } else if (ncores == 1 && isFALSE(future.use.plan)) {
    diag.sim <- list()
    if (verbose == TRUE) {
      cat("\n |")
    }
    for (i in seq_len(nsims)) {
      diag.sim[[i]] <- dosim()
      if (verbose == TRUE) {
        cat("*")
      }
    }
    if (verbose == TRUE) {
      cat("|")
    }
  } else {
    if (inherits(future.use.plan, c("tweaked", "future"))) {
      with(future::plan(future.use.plan), local = TRUE)
    } else if (ncores > 1 && isFALSE(future.use.plan)) {
      ncores_eff <- min(nsims, ncores)
      with(future::plan("multisession", workers = ncores_eff), local = TRUE)
    } # else if (isTRUE(future.use.plan)) - use plan defined by user
    diag.sim <- future.apply::future_replicate(
      nsims, dosim(), simplify = FALSE, future.seed = TRUE
    )
  }

  if (verbose == TRUE) {
    cat("\n- Calculating formation statistics")
  }

  ## List for stats for each simulation
  stats <- lapply(diag.sim, function(x) x$stats[, !duplicated(colnames(x$stats)), drop = FALSE])
  ts.attr.names <- x$target.stats.names

  ts.out <- data.frame(
    names = ts.attr.names,
    targets = target.stats
  )
  names(target.stats) <- ts.attr.names

  ## Calculate mean/sd from stats
  stats.table.formation <- make_stats_table(stats, target.stats)

  # Calculate dissolution / duration stats
  if (skip.dissolution == FALSE && dynamic == TRUE) {
    durs <- coef.diss$duration

    dims <- c(nsteps, length(durs), length(diag.sim))
    dissolution.stats <- list("stats.table.duration" = make_stats_table(lapply(diag.sim, `[[`, "meanageimputed"), durs),
                              "stats.table.dissolution" = make_stats_table(lapply(diag.sim, `[[`, "propdiss"),
                                                                           1 / durs),
                              "pages" = array(unlist(lapply(diag.sim, `[[`, "meanage")), dim = dims),
                              "pages_imptd" = array(unlist(lapply(diag.sim, `[[`, "meanageimputed")), dim = dims),
                              "prop.diss" = array(unlist(lapply(diag.sim, `[[`, "propdiss")), dim = dims),
                              "anyNA" = any(unlist(lapply(diag.sim, `[[`, "anyNA"))))
  }

  ## Save output
  out <- list()
  out$nw <- nw
  out$formation <- formation
  out$coef.form <- coef.form
  out$dissolution <- dissolution
  out$coef.diss <- coef.diss
  out$constraints <- constraints
  out$edapprox <- edapprox
  out$target.stats <- ts.out
  out$nsims <- nsims
  out$dynamic <- dynamic

  out$stats <- stats
  out$stats.table.formation <- stats.table.formation
  if (dynamic == TRUE) {
    out$nsteps <- nsteps
    if (skip.dissolution == FALSE) {
      out <- c(out, dissolution.stats)
    }
    if (keep.tedgelist == TRUE) {
      out$tedgelist <- lapply(diag.sim, `[[`, "tedgelist")
    }
    if (keep.tnetwork == TRUE) {
      out$network <- lapply(diag.sim, `[[`, "tnetwork")
    }
  }
  out$anyNA <- NVL(out$anyNA, FALSE)

  class(out) <- "netdx"
  return(out)
}

## internal wrapper around coda::effectiveSize, returning a vector of NAs
##   if coda::effectiveSize throws an error; argument x should be a matrix
##   with column names
ess <- function(x) {
  tryCatch(coda::effectiveSize(x),
           error = function(e) {
             structure(rep(NA, length.out = NCOL(x)), names = colnames(x))
           })
}

#' @title Create a Summary Table of Simulation Statistics
#'
#' @param stats A list of simulated statistics matrices, of length equal to the
#'   number of simulations performed.  Each matrix should have one row for each
#'   simulated network if `dynamic == FALSE`, one row for each time step
#'   if `dynamic == TRUE`, and one column for each statistic.  The columns
#'   should be named for the statistics they correspond to, with all matrices
#'   having the same statistics, in the same order.  Each matrix may have an
#'   `attr`-style attribute named `"ess"` attached, giving the
#'   effective sample sizes for the columns of the matrix; if this attribute is
#'   `NULL`, then the effective sample sizes will be computed within the
#'   call to `make_stats_table`.
#' @param targets A vector of target values for the statistics in `stats`.
#'   May be named (in which case targets will be matched to statistics based on
#'   column names in matrices in `stats`) or unnamed (in which case
#'   targets will be matched to statistics based on position, and the number of
#'   targets must equal the number of columns).
#'
#' @return A `data.frame` summarizing the simulated statistics.
#' @keywords internal
make_stats_table <- function(stats, targets) {
  ess_list <- lapply(stats, function(x) NVL(attr(x, "ess"), ess(x)))
  ess_sum <- colSums(do.call(rbind, ess_list), na.rm = TRUE)

  stats.onesim.sd <- apply(do.call(rbind, lapply(stats, colMeans, na.rm = TRUE)), 2, sd, na.rm = TRUE)

  stats <- do.call(rbind, stats)
  stats.means <- colMeans(stats, na.rm = TRUE)
  stats.sd <- apply(stats, 2L, sd, na.rm = TRUE)
  stats.se <- stats.sd / sqrt(ess_sum)

  if (!is.null(names(targets))) {
    stats.targets <- rep(NA, length.out = length(stats.means))
    matches <- match(names(targets), names(stats.means))
    stats.targets[na.omit(matches)] <- targets[!is.na(matches)]
  } else {
    stats.targets <- targets
  }

  stats.table <- data.frame("Target" = stats.targets,
                            "Sim Mean" = stats.means,
                            "Pct Diff" = 100 * (stats.means - stats.targets) / stats.targets,
                            "Sim SE" = stats.se,
                            "Z Score" = (stats.means - stats.targets) / stats.se,
                            "SD(Sim Means)" = stats.onesim.sd,
                            "SD(Statistic)" = stats.sd)
  colnames(stats.table) <- c("Target", "Sim Mean", "Pct Diff", "Sim SE", "Z Score", "SD(Sim Means)", "SD(Statistic)")
  rownames(stats.table) <- names(stats.means)

  return(stats.table)
}

#' @title Convert Timed Edgelist to Matrix of Toggles
#'
#' @param tedgelist A timed edgelist, as produced by
#'   [`networkDynamic::as.data.frame.networkDynamic`].
#'
#' @return The matrix of toggles corresponding to `tedgelist`.
#' @keywords internal
tedgelist_to_toggles <- function(tedgelist) {
  tedgelist <- as.matrix(tedgelist)
  toggles <- rbind(tedgelist[, c(1L, 3L, 4L), drop = FALSE],
                   tedgelist[!tedgelist[, 6L], c(2L, 3L, 4L), drop = FALSE])
  colnames(toggles) <- c("time", "tail", "head")
  toggles
}

#' @title Convert Matrix of Toggles to Dissolution and Duration Statistics
#'
#' @param toggles A matrix of toggles, as produced by
#'   [tedgelist_to_toggles()].
#' @param coef.diss Dissolution coefficients used in the simulation.
#' @param nsteps Number of time steps in the simulation.
#' @param nw Network used in the simulation.
#' @param time.start Starting time for the simulation.
#'
#' @return Named list containing dissolution and duration statistics matrices
#'   and other related information.
#' @keywords internal
toggles_to_diss_stats <- function(toggles, coef.diss,
                                  nsteps, nw, time.start = 0L) {
  nw <- as.network(nw) # drop nwd
  delete.network.attribute(nw, "time")
  delete.network.attribute(nw, "lasttoggle")
  nw[, ] <- FALSE

  # nolint start
  diss_formula <- coef.diss$dissolution
  # nolint end
  durs <- coef.diss$duration

  changestats <- as.matrix(tergm.godfather(nw ~ Passthrough(diss_formula)
                                           + EdgeAges(diss_formula)
                                           + Persist(diss_formula),
                                           toggles = toggles,
                                           start = time.start - 1L,
                                           end = time.start + nsteps,
                                           stats.start = FALSE))

  edgecounts <- changestats[, seq_along(durs), drop = FALSE]

  # drop offset() from names
  colnames(edgecounts) <- substr(colnames(edgecounts), 8L,
                                 nchar(colnames(edgecounts)) - 1L)

  edgeages <- changestats[, seq_along(durs) + length(durs), drop = FALSE]
  colnames(edgeages) <- colnames(edgecounts)

  edgepers <- changestats[, seq_along(durs) + 2L * length(durs), drop = FALSE]

  if (length(durs) > 1L) {
    edgecounts[, 1L] <- edgecounts[, 1L] - rowSums(edgecounts[, -1L, drop = FALSE])
    edgeages[, 1L] <- edgeages[, 1L] - rowSums(edgeages[, -1L, drop = FALSE])
    edgepers[, 1L] <- edgepers[, 1L] - rowSums(edgepers[, -1L, drop = FALSE])
  }

  edgediss <- edgecounts[-NROW(edgecounts), , drop = FALSE] -
    edgepers[-1L, , drop = FALSE]
  edgeages <- edgeages[-1L, , drop = FALSE]

  edgeagesimputed <- edgeages
  toggles <- toggles[order(toggles[, 2L], toggles[, 3L],
                           toggles[, 1L]), , drop = FALSE]
  w <- which(toggles[, 1] == time.start)
  if (length(w) > 0L) {
    # imputation
    changestats <- as.matrix(tergm.godfather(nw ~ Passthrough(diss_formula),
                                             toggles = cbind(seq_along(w),
                                                             toggles[w, -1L, drop = FALSE]),
                                             stats.start = TRUE))

    for (i in seq_along(w)) {
      dyad_type <- max(which(changestats[i, ] != changestats[i + 1L, ]))
      index <- w[i]
      if (index < NROW(toggles) &&
            toggles[index, 2L] == toggles[index + 1L, 2L] &&
            toggles[index, 3L] == toggles[index + 1L, 3L]) {
        terminus_time <- toggles[index + 1L, 1L]
      } else {
        terminus_time <- time.start + nsteps + 1L
      }
      if (terminus_time > time.start + 1L) {
        edgeagesimputed[seq_len(terminus_time - time.start - 1L), dyad_type] <-
          edgeagesimputed[seq_len(terminus_time - time.start - 1L), dyad_type] +
          rgeom(1L, 1 / durs[dyad_type])
      }
    }
  }

  ## 0/0 is possible, resulting in NaN, which we set to 0 for the time being...
  meanage <- edgeages / edgecounts[-1L, , drop = FALSE]
  meanageimputed <- edgeagesimputed / edgecounts[-1L, , drop = FALSE]
  propdiss <- edgediss / edgecounts[-NROW(edgecounts), , drop = FALSE]

  if (any(is.na(meanage)) || any(is.na(meanageimputed)) || any(is.na(propdiss))) {
    meanage[is.na(meanage)] <- 0
    meanageimputed[is.na(meanageimputed)] <- 0
    propdiss[is.na(propdiss)] <- 0
    anyNA <- TRUE
  } else {
    anyNA <- FALSE
  }

  attr(meanage, "ess") <- ess(meanage)
  attr(meanageimputed, "ess") <- ess(meanageimputed)
  attr(propdiss, "ess") <- ess(propdiss)

  return(list(meanage = meanage,
              meanageimputed = meanageimputed,
              propdiss = propdiss,
              anyNA = anyNA))
}

#' @title Modules for Stochastic Network Models
#'
#' @description
#' Stochastic network models of infectious disease in EpiModel require
#' statistical modeling of networks, simulation of those networks forward
#' through time, and simulation of epidemic dynamics on top of those evolving
#' networks. The [netsim()] function handles both the network and
#' epidemic simulation tasks. Within this function are a series of modules that
#' initialize the simulation and then simulate new infections, recoveries, and
#' demographics on the network. Modules also handle the resimulation of the
#' network and some bookkeeping calculations for disease prevalence.
#'
#' Writing original network models that expand upon our "base" model set will
#' require modifying the existing modules or adding new modules to the workflow
#' in [netsim()]. The existing modules may be used as a template for
#' replacement or new modules.
#'
#' This help page provides an orientation to these module functions, in the
#' order in which they are used within [netsim()], to help guide users
#' in writing their own functions. These module functions are not shown
#' on the help index since they are not called directly by the end-user. To
#' understand these functions in more detail, review the separate help pages
#' listed below.
#'
#' @section Initialization Module:
#' This function sets up nodal attributes, like disease status, on the network
#' at the starting time step of disease simulation, \eqn{t_1}. For
#' multiple-simulation function calls, these are reset at the beginning of each
#' individual simulation.
#'
#'  * [initialize.net()]: sets up the main `netsim_dat` data
#'        structure used in the simulation, initializes which nodes are infected
#'        (via the initial conditions passed in [init.net()]), and
#'        simulates a first time step of the networks given the network model
#'        fit from [netest()].
#'
#'
#' @section Disease Status Modification Modules:
#' The main disease simulation occurs at each time step given the current state
#' of the network at that step. Infection of nodes is simulated as a function of
#' attributes of the nodes and the edges. Recovery of nodes is likewise
#' simulated as a function of nodal attributes of those infected nodes. These
#' functions also calculate summary flow measures such as disease incidence.
#'
#'  * [infection.net()]: simulates disease transmission given an
#'        edgelist of discordant partnerships by calculating the relevant
#'        transmission and act rates for each edge, and then updating the nodal
#'        attributes and summary statistics.
#'  * [recovery.net()]: simulates recovery from infection either
#'        to a lifelong immune state (for SIR models) or back to the susceptible
#'        state (for SIS models), as a function of the recovery rate parameters
#'        specified in [param.net()].
#'
#'
#' @section Demographic Modules:
#' Demographics such as arrival and departure processes are simulated at each
#' time step to update entries into and exits from the network. These are used
#' in epidemic models with network feedback, in which the network is resimulated
#' at each time step to account for the nodal changes affecting the edges.
#'
#'  * [departures.net()]: randomly simulates departure for nodes
#'        given their disease status (susceptible, infected, recovered), and
#'        their group-specific departure rates specified in
#'        [param.net()]. Departures involve deactivating nodes.
#'  * [arrivals.net()]: randomly simulates new arrivals into the
#'        network given the current population size and the arrival rate
#'        specified in the `a.rate` parameters. This involves adding new
#'        nodes into the network.
#'
#'
#' @section Network Resimulation Module:
#' In dependent network models, the network object is resimulated at each time
#' step to account for changes in the size of the network (changed through
#' entries and exits), and the disease status of the nodes.
#'
#'  * [resim_nets()]: resimulates the network object one time step
#'        forward given the set of formation and dissolution coefficients
#'        estimated in [netest()].
#'
#'
#' @section Bookkeeping Module:
#' Network simulations require bookkeeping at each time step to calculate the
#' summary epidemiological statistics used in the model output analysis.
#'
#'  * [prevalence.net()]: calculates the number in each disease
#'        state (susceptible, infected, recovered) at each time step for those
#'        active nodes in the network. If the `epi.by` control is used, it
#'        calculates these statistics by a set of specified nodal attributes.
#'  * [verbose.net()]: summarizes the current state of the
#'        simulation and prints this to the console.
#'
#'
#' @section One- & Two-Group Modules:
#' If epidemic `type` is supplied within [control.net()],
#' EpiModel defaults each of the base epidemic and demographic modules described
#' above (arrivals.FUN, departures.FUN, infection.FUN, recovery.FUN) to the
#' correct .net function based on variables passed to [param.net()]
#' (e.g. num.g2, denoting population size of group two, would select the
#' two-group variants of the aforementioned modules). Two-group modules are
#' denoted by a .2g affix (e.g., recovery.2g.net)
#'
#'
#' @name modules.net
#' @aliases modules.net
#'
NULL

#' @title Get an Edgelist From the Specified Network
#'
#' @description This function outputs an edgelist from the specified network,
#'              selecting the method depending on the stored network type.
#' @inheritParams recovery.net
#' @param network Numerical index of the network from which the edgelist should
#'                be extracted. (May be > 1 for models with multiple overlapping
#'                networks.)
#'
#' @return
#' An edgelist in matrix form with two columns. Each column contains the
#' posit_ids (see `get_posit_ids`) of the nodes in each edge.
#'
#' @export
get_edgelist <- function(dat, network) {

  if (get_control(dat, "tergmLite")) {
    if (!network %in% seq_len(dat$num.nw)) {
      stop("There is no network '", network, "' to get an edgelist from")
    }
    el <- dat$run$el[[network]]
  } else {
    if (!network %in% seq_len(dat$num.nw)) {
      stop("There is no network '", network, "' to get an edgelist from")
    }
    at <- get_current_timestep(dat)
    net <- get_network(dat, network = network)
    el <- networkDynamic::get.dyads.active(net, at = at)
  }

  return(el)
}

## Discordant Edgelist ---------------------------------------------------------

#' @title Get Discordant Edgelist Based on Specified Status Variable
#'
#' @description This function returns a `data.frame` with a discordant
#'              edgelist, defined as the set of edges for which the status attribute
#'              of interest is discordant between the two partners.
#'
#' @inheritParams get_cumulative_edgelists_df
#' @param status.attr The name of the status attribute of interest.
#' @param head.status The value(s) of `status.attr` for which to look for the head of the edge.
#'        Can be a single value or a vector.
#' @param tail.status  The value(s) of `status.attr` for which to look for the tail of the edge.
#'        Can be a single value or a vector.
#'
#' @details
#' This is a generalized version of the `discord_edgelist` function.
#' It creates an edgelist of current partnerships in which the status attribute
#' of interest (as specified by the parameter `status.attr`) of one partner matches
#' the value (or one of the values) of the `head.status` parameter while the
#' corresponding status attribute of the other partner matches the value (or
#' one of the values) of the `tail.status` parameter.
#'
#' @return
#' A `data.frame` with the following columns:
#'  * `head`: Positional ID of the head node.
#'  * `tail`: Positional ID of the tail node.
#'  * `head_status`: Status of the head node.
#'  * `tail_status`: Status of the tail node.
#'  * `network`: The numerical index of the network on which the partnership is located.
#'
#' @seealso [discord_edgelist()]
#'
#' @export
#' @keywords netMod internal
#'
get_discordant_edgelist <- function(dat, status.attr, head.status,
                                    tail.status, networks = NULL) {
  if (get_current_timestep(dat) == get_control(dat, "start") + 1 &&
        length(intersect(head.status, tail.status)) > 0) {
    warning("The head.status and tail.status arguments should be discordant.")
  }

  status <- get_attr(dat, status.attr)

  d_el <- get_edgelists_df(dat, networks)

  d_el_ordered <- dplyr::filter(
    d_el,
    status[d_el$head] %in% head.status &
      status[d_el$tail] %in% tail.status
  )

  d_el_rev <- dplyr::filter(
    d_el,
    status[d_el$tail] %in% head.status &
      status[d_el$head] %in% tail.status
  ) |>
    dplyr::select(head = "tail", tail = "head", "network")

  d_el <- dplyr::bind_rows(d_el_ordered, d_el_rev) |>
    dplyr::mutate(
      head_status = status[.data$head],
      tail_status = status[.data$tail]
    ) |>
    dplyr::select(
      "head", "tail", "head_status", "tail_status", "network"
    )

  return(d_el)

}

#' @title Get the Edgelist(s) from the Specified Network(s)
#'
#' @inheritParams get_cumulative_edgelists_df
#'
#' @return
#' A `data.frame` with the following columns:
#'  * `head`: Positional ID of the head node.
#'  * `tail`: Positional ID of the tail node.
#'  * `network`: The numerical index of the network on which the edge is located.
#'
#' @export
get_edgelists_df <- function(dat, networks = NULL) {

  networks <- if (is.null(networks)) seq_len(dat$num.nw) else networks
  el_list <- lapply(networks, get_edgelist, dat = dat)
  el_tibble <- lapply(el_list, as_tibble_edgelist)
  d_el <- dplyr::bind_rows(el_tibble)

  el_sizes <- vapply(el_list, nrow, numeric(1))
  d_el[["network"]] <- rep(networks, el_sizes)

  return(d_el)
}

#' @title Convert an Edgelist into a Tibble
#'
#' @param el An edgelist in matrix or data frame form.
#'
#' @return
#' The edgelist in tibble form with two columns named `head` and `tail`.
#'
#' @export
as_tibble_edgelist <- function(el) {

  if (nrow(el) > 0) {
    t_el <- tibble::tibble(head = el[, 1], tail = el[, 2])
  } else {
    t_el <- tibble::tibble(head = integer(0), tail = integer(0))
  }

  return(t_el)
}

## Cumulative Edgelists --------------------------------------------------------

#' @title Get a Cumulative Edgelist From a Specified Network
#'
#' @inheritParams recovery.net
#' @param network Numerical index of the network from which the cumulative
#'                edgelist should be extracted. (May be > 1 for models with
#'                multiple overlapping networks.)
#'
#' @return
#' A cumulative edgelist in `data.frame` form with 4 columns:
#'
#'   * `head`: the unique ID (see `get_unique_ids`) of the
#'         head node on the edge.
#'   * `tail`: the unique ID (see `get_unique_ids`) of the
#'         tail node on the edge.
#'   * `start`: the time step in which the edge started.
#'   * `stop`: the time step in which the edge stopped; if ongoing,
#'         then `NA` is returned.
#'
#'
#' @export
get_cumulative_edgelist <- function(dat, network) {
  if (!network %in% seq_len(dat$num.nw)) {
    stop("There is no network '", network,
         "' from which to get the cumulative edgelist.")
  }

  if (!get_control(dat, "cumulative.edgelist")) {
    stop("Failed to get the cumulative edgelist. It is likely not stored because the
         `cumulative.edgelist` control setting is set to `FALSE`.")
  }

  el_cuml <- dplyr::bind_rows(
    get_raw_elcuml(dat, network, active = FALSE),
    get_raw_elcuml(dat, network, active = TRUE)
  )

  return(el_cuml)
}

#' @title Update a Cumulative Edgelist of the Specified Network
#'
#' @inheritParams recovery.net
#' @param network Numerical index of the network for which the cumulative
#'                edgelist will be updated. (May be > 1 for models with
#'                multiple overlapping networks.)
#' @param truncate After how many time steps a partnership that is no longer
#'                 active should be removed from the output.
#'
#' @section Truncation:
#' To avoid storing a cumulative edgelist too long, the `truncate`
#' parameter defines a number of steps after which an edge that is no longer
#' active is truncated out of the cumulative edgelist.
#' When `truncate = Inf`, no edges are ever removed. When
#' `truncate = 0`, only the active edges are kept. You may want this
#' behavior to keep track of the active edges' start step.
#'
#' @inherit recovery.net return
#'
#' @export
update_cumulative_edgelist <- function(dat, network, truncate = 0) {
  if (!get_control(dat, "cumulative.edgelist")) {
    return(dat)
  }

  el <- get_edgelist(dat, network)
  el_cuml_cur <- get_raw_elcuml(dat, network, active = TRUE)
  el_cuml_hist <- get_raw_elcuml(dat, network, active = FALSE)

  at <- get_current_timestep(dat)

  # truncate el_cuml_hist
  if (truncate != Inf && truncate != 0) {
    rel.age <- at - el_cuml_hist$stop
    el_cuml_hist <- el_cuml_hist[rel.age <= truncate, ]
  }

  el <- tibble::tibble(
    head = get_unique_ids(dat, el[, 1]),
    tail = get_unique_ids(dat, el[, 2]),
    current = TRUE
  )

  el_cuml_cur <- dplyr::full_join(el_cuml_cur, el, by = c("head", "tail"))

  # new edges
  new_edges_ids <- which(is.na(el_cuml_cur$start))
  if (length(new_edges_ids) > 0) {
    el_cuml_cur[new_edges_ids, ]$start <- at
  }

  # terminated edges
  terminated_edges_ids <- which(is.na(el_cuml_cur$current))
  el_cuml_cur$current <- NULL

  if (length(terminated_edges_ids) > 0) {
    el_cuml_term <- el_cuml_cur[terminated_edges_ids, ]

    # with truncate == 0, don't save any historic edges
    if (truncate != 0) {
      el_cuml_term$stop <- at - 1
      el_cuml_hist <- dplyr::bind_rows(el_cuml_hist, el_cuml_term)
    }

    el_cuml_cur <- el_cuml_cur[-terminated_edges_ids, ]
  }

  dat <- set_raw_elcuml(dat, network, el_cuml_cur, active = TRUE)
  dat <- set_raw_elcuml(dat, network, el_cuml_hist, active = FALSE)

  return(dat)
}

# Seed the cumulative edgelist once at initialization, at the end of
# sim_nets_t1(). Without it, the first call to update_cumulative_edgelist()
# happens at at=2 from resim_nets(), so persistent edges receive start=2 and
# edges that dissolved during the initial ERGM -> TERGM transition are lost
# entirely. See issue #1016.
#
# The seed records two pieces of state, on the same clock as the underlying
# networkDynamic object (where time 0 is the cross-sectional ergm sample and
# time 1 is the state after the first TERGM step, which is what EpiModel
# observes as at = 1):
#
#   * el_cuml_cur (active edges):
#       - persistent initial edges (in t=0 and t=1) get start=0, stop=NA
#       - edges formed during the first TERGM step (in t=1, not t=0) get
#         start=1, stop=NA
#   * el_cuml_hist (terminated edges, only when truncate.el.cuml != 0):
#       - edges active in the cross-section but dissolved by t=1 get
#         start=0, stop=0 (mirrors networkDynamic spell [0, 1))
#
# In tergmLite mode the cross-section edgelist is no longer recoverable from
# the post-simulation state (networkLite discards history), so sim_nets_t1
# stashes it on dat$run$el_t0_seed before simulate_dat() runs and we consume
# it here. In non-tergmLite mode the full networkDynamic history is intact
# and we query it directly via get.dyads.active().
seed_cumulative_edgelist_t1 <- function(dat) {
  if (!get_control(dat, "cumulative.edgelist")) {
    dat$run$el_t0_seed <- NULL
    return(dat)
  }

  truncate <- get_control(dat, "truncate.el.cuml")
  tergmLite <- get_control(dat, "tergmLite")

  for (network in seq_len(dat$num.nw)) {
    if (tergmLite) {
      el_t0 <- dat$run$el_t0_seed[[network]]
      el_t1 <- dat$run$el[[network]]
    } else {
      nw <- get_network(dat, network = network)
      el_t0 <- networkDynamic::get.dyads.active(nw, at = 0)
      el_t1 <- networkDynamic::get.dyads.active(nw, at = 1)
    }

    keys_t0 <- if (NROW(el_t0) > 0) {
      paste(el_t0[, 1], el_t0[, 2], sep = "_")
    } else {
      character(0)
    }
    keys_t1 <- if (NROW(el_t1) > 0) {
      paste(el_t1[, 1], el_t1[, 2], sep = "_")
    } else {
      character(0)
    }

    persistent_idx <- which(keys_t1 %in% keys_t0)
    new_idx        <- which(!keys_t1 %in% keys_t0)
    dropped_idx    <- which(!keys_t0 %in% keys_t1)

    seed_cur_pieces <- list()
    if (length(persistent_idx) > 0) {
      el_p <- el_t1[persistent_idx, , drop = FALSE]
      seed_cur_pieces[[length(seed_cur_pieces) + 1]] <- tibble::tibble(
        head  = get_unique_ids(dat, el_p[, 1]),
        tail  = get_unique_ids(dat, el_p[, 2]),
        start = 0,
        stop  = NA_real_
      )
    }
    if (length(new_idx) > 0) {
      el_n <- el_t1[new_idx, , drop = FALSE]
      seed_cur_pieces[[length(seed_cur_pieces) + 1]] <- tibble::tibble(
        head  = get_unique_ids(dat, el_n[, 1]),
        tail  = get_unique_ids(dat, el_n[, 2]),
        start = 1,
        stop  = NA_real_
      )
    }
    if (length(seed_cur_pieces) > 0) {
      el_cuml_cur <- get_raw_elcuml(dat, network, active = TRUE)
      dat <- set_raw_elcuml(
        dat, network,
        dplyr::bind_rows(c(list(el_cuml_cur), seed_cur_pieces)),
        active = TRUE
      )
    }

    if (truncate != 0 && length(dropped_idx) > 0) {
      el_d <- el_t0[dropped_idx, , drop = FALSE]
      seed_hist <- tibble::tibble(
        head  = get_unique_ids(dat, el_d[, 1]),
        tail  = get_unique_ids(dat, el_d[, 2]),
        start = 0,
        stop  = 0
      )
      el_cuml_hist <- get_raw_elcuml(dat, network, active = FALSE)
      dat <- set_raw_elcuml(
        dat, network,
        dplyr::bind_rows(el_cuml_hist, seed_hist),
        active = FALSE
      )
    }
  }

  dat$run$el_t0_seed <- NULL
  return(dat)
}

#' @title Get the Cumulative Edgelists of a Model
#'
#' @inheritParams recovery.net
#' @param networks Numerical indexes of the networks to extract the partnerships
#'                 from. (May be > 1 for models with multiple overlapping
#'                 networks.) If `NULL`, extract from all networks.
#'
#' @return
#' A `data.frame` with 5 columns:
#'
#'   * `index`: the unique ID (see `get_unique_ids`) of the
#'         indexes.
#'   * `partner`: the unique ID (see `get_unique_ids`) of the
#'         partners/contacts.
#'   * `start`: the time step in which the edge started.
#'   * `stop`: the time step in which the edge stopped; if ongoing,
#'         then `NA` is returned.
#'   * `network`: the numerical index for the network on which the
#'         partnership/contact is located.
#'
#'
#' @export
get_cumulative_edgelists_df <- function(dat, networks = NULL) {
  networks <- if (is.null(networks)) seq_len(dat$num.nw) else networks

  el_cuml_list <- lapply(networks, get_cumulative_edgelist, dat = dat)
  el_cuml_df <- dplyr::bind_rows(el_cuml_list)

  el_sizes <- vapply(el_cuml_list, nrow, numeric(1))
  el_cuml_df[["network"]] <- rep(networks, el_sizes)

  return(el_cuml_df)
}

#' @title Return the Historical Contacts (Partners) of a Set of Index Nodes
#'
#' @description
#' From a full cumulative edgelist that contains the history of contacts (both persistent and
#' one-time), this function returns a data frame containing details of the index (head) and partner
#' (tail) nodes, along with start and stop time steps for the partnership and the network location.
#'
#' @param index_posit_ids The positional IDs of the indexes of interest.
#' @param networks Numerical indexes of the networks to extract the partnerships from. (May be > 1
#'        for models with multi-layer networks.) If `NULL`, extract from all networks.
#' @param only.active.nodes If `TRUE`, then inactive (e.g., deceased) partners will be removed from
#'        the output.
#' @inheritParams update_cumulative_edgelist
#'
#' @return
#' A `data.frame` with 5 columns:
#'   * `index`: the unique IDs of the indexes.
#'   * `partner`: the unique IDs of the partners/contacts.
#'   * `start`: the time step at which the edge started.
#'   * `stop`: the time step in which the edge stopped; if ongoing, then `NA` is returned.
#'   * `network`: the numerical index for the network on which the partnership/contact is located.
#'
#' @details
#' Note that `get_partners` takes as input the positional IDs of the indexes of interest but returns
#' the unique IDs. That is by design, because while `get_partners` would be expected to be called
#' for active nodes, some partners (contacts) of nodes may be inactive in the network history.
#' Therefore, both index and partner IDs are returned as unique IDs for consistency. To convert
#' between a positional to a unique ID, you may use [`get_unique_ids`]; to convert between a
#' unique ID to a positional ID, you may use [`get_posit_ids`].
#'
#' @export
#'
get_partners <- function(dat, index_posit_ids, networks = NULL,
                         truncate = Inf, only.active.nodes = FALSE) {

  el_cuml_df <- get_cumulative_edgelists_df(dat, networks)
  index_unique_ids <- get_unique_ids(dat, index_posit_ids)

  partner_head_df <- el_cuml_df[el_cuml_df[["head"]] %in% index_unique_ids, ]
  partner_tail_df <- el_cuml_df[
    el_cuml_df[["tail"]] %in% index_unique_ids,
    c(2, 1, 3:5) # switch the head and tail columns
  ]

  colnames(partner_head_df) <- c("index", "partner", "start", "stop", "network")
  colnames(partner_tail_df) <- colnames(partner_head_df)

  partner_df <- dplyr::bind_rows(partner_head_df, partner_tail_df)

  if (only.active.nodes) {
    active_partners <- is_active_unique_ids(dat, partner_df[["partner"]])
    partner_df <- partner_df[active_partners, ]
  }

  if (truncate != Inf) {
    at <- get_current_timestep(dat)
    rel.age <- at - partner_df[["stop"]]
    rel.age <- ifelse(is.na(rel.age), 0, rel.age)
    partner_df <- partner_df[rel.age <= truncate, ]
  }

  return(partner_df)
}

#' @title Return the Cumulative Degree of a Set of Index Nodes
#'
#' @inheritParams get_partners
#'
#' @return
#' A `data.frame` with 2 columns:
#'
#'   * `index_pid`: the positional ID (see `get_posit_ids`) of the
#'         indexes.
#'   * `degree`: the cumulative degree of the index.
#'
#'
#' @section Cumulative Degree:
#' The cumulative degree of a node is the number of edges connected to this
#' node at during the time window. The time window is by default all the steps
#' stored in the `cumulative_edgelist` or set by the `truncate` parameter.
#'
#' @export
get_cumulative_degree <- function(dat, index_posit_ids, networks = NULL,
                                  truncate = Inf, only.active.nodes = FALSE) {
  get_partners(
    dat, index_posit_ids, networks,
    truncate, only.active.nodes
  ) |>
    dplyr::summarize(degree = dplyr::n(), .by = "index") |>
    dplyr::mutate(index = get_posit_ids(dat, .data$index)) |>
    dplyr::select(index_pid = "index", "degree")
}

# Helper functions to get and set the cumulative edgelists
#
# Cumulative edgelists are split in to:
#   - el_cuml_cur: for the edges not yet disolved
#   - el_cuml_hist: for the edges where the start and stop time are known
#   (dissolved edges)
get_raw_elcuml <- function(dat, network, active) {
  loc <- if (active) "el_cuml_cur" else "el_cuml_hist"

  if (length(dat$run[[loc]]) >= network) {
    el_cuml <- dat$run[[loc]][[network]]
  } else {
    el_cuml <- NULL
  }

  if (is.null(el_cuml)) {
    el_cuml <- empty_el_cuml()
  }

  return(el_cuml)
}

set_raw_elcuml <- function(dat, network, el_cuml, active) {
  loc <- if (active) "el_cuml_cur" else "el_cuml_hist"
  dat$run[[loc]][[network]] <- el_cuml
  return(dat)
}

# template for the cumulative edgelists
empty_el_cuml <- function() {
  tibble::tibble(
    head  = numeric(0),
    tail  = numeric(0),
    start = numeric(0),
    stop  = numeric(0)
  )
}

context("Cumulative Edgelist (All SOC)")

nw <- network_initialize(n = 100)
est <- netest(
  nw,
  formation = ~edges,
  target.stats = 25,
  coef.diss = dissolution_coefs(~ offset(edges), 10, 0),
  verbose = FALSE
)

param <- param.net(
  inf.prob = 0.3,
  act.rate = 0.1
)

init <- init.net(i.num = 10)

test_that("netsim, SI, Cumulative Edgelist", {
  skip_on_cran()
  control <- control.net(
    type = "SI",
    nsims = 1,
    nsteps = 100,
    resimulate.network = TRUE,
    tergmLite = TRUE,
    verbose = FALSE,
    truncate.el.cuml = NULL,
    cumulative.edgelist = TRUE,
    raw.output = TRUE
  )

  mod <- netsim(est, param, init, control)
  d <- get_cumulative_edgelists_df(mod[[1]])

  expect_is(d, "data.frame")
  expect_equal(colnames(d), c("head", "tail", "start", "stop", "network"))

  control <- control.net(
    type = "SI",
    nsims = 1,
    nsteps = 100,
    resimulate.network = TRUE,
    tergmLite = TRUE,
    verbose = FALSE,
    truncate.el.cuml = 40,
    cumulative.edgelist = TRUE,
    save.cumulative.edgelist = TRUE,
    raw.output = TRUE
  )

  mod <- netsim(est, param, init, control)
  d <- get_cumulative_edgelists_df(mod[[1]])
  expect_gte(min(d$stop, na.rm = TRUE), 40)

  d <- get_partners(dat = mod[[1]], index_posit_ids = 1:10, networks = 1)
  expect_equal(colnames(d), c("index", "partner", "start", "stop", "network"))

  # expect errors when trying to access networks that don't exist
  expect_error(
    get_partners(dat = mod[[1]], index_posit_ids = 1:10, networks = 2)
  )

  expect_error(
    update_cumulative_edgelist(dat = mod[[1]], network = 2)
  )

  # Cumulative edgelist extraction and reachable set
  sim <- process_out.net(mod)
  el_cuml <- sim$cumulative.edgelist[[1]]
  el_cuml <- dedup_cumulative_edgelist(el_cuml)

  nnodes <- max(el_cuml$head, el_cuml$tail)

  from_step <- min(el_cuml$start)
  to_step <- 100
  nodes <- sample(nnodes, 10)
  # Only test that the functions run without error. See the functions examples
  # to check correctness
  el_tp <- get_forward_reachable(el_cuml, from_step, to_step, nodes)
  el_tp <- get_backward_reachable(el_cuml, from_step, to_step, nodes, "yes")
})

test_that("netsim, SI, Cumulative Edgelist - missing args", {
  skip_on_cran()
  control <- control.net(
    type = "SI",
    nsims = 1,
    nsteps = 10,
    resimulate.network = TRUE,
    tergmLite = TRUE,
    verbose = FALSE,
    raw.output = TRUE
  )

  mod <- netsim(est, param, init, control)

  expect_error(get_cumulative_edgelists_df(mod[[1]]))

  expect_equal(mod[[1]]$control$cumulative.edgelist, FALSE)
  expect_equal(mod[[1]]$control$truncate.el.cuml, 0)

})

test_that("netsim non-tergmLite cumulative edgelist matches networkDynamic (#1016)", {
  skip_on_cran()
  set.seed(1)
  nw <- network_initialize(n = 500)
  est <- netest(
    nw,
    formation = ~edges + concurrent,
    target.stats = c(0.75 * 500 / 2, 0.08 * 500),
    coef.diss = dissolution_coefs(~offset(edges), duration = 80),
    verbose = FALSE
  )

  control <- control.net(
    type = "SIS", nsims = 1, nsteps = 50,
    cumulative.edgelist = TRUE,
    truncate.el.cuml = 99999,
    save.cumulative.edgelist = TRUE,
    save.network = TRUE,
    verbose = FALSE
  )

  set.seed(1)
  mod <- netsim(est, param.net(inf.prob = 0.5, act.rate = 2, rec.rate = 0.05),
                init.net(i.num = 25), control)

  cel_nd <- as.data.frame(get_network(mod, sim = 1))
  cel_epi <- mod$cumulative.edgelist$sim1

  # Row-count parity (the headline #1016 regression check).
  expect_equal(nrow(cel_epi), nrow(cel_nd))

  # Active-set parity at t=0 and t=1. networkDynamic spells are half-open
  # [onset, terminus); the cumulative edgelist is closed [start, stop] with
  # NA stop meaning ongoing. So an edge active at time t in nD has
  # onset <= t < terminus, and in the cumulative edgelist has
  # start <= t and (is.na(stop) | stop >= t).
  n_active_t0_nd  <- sum(cel_nd$onset <= 0 & cel_nd$terminus > 0)
  n_active_t0_epi <- sum(cel_epi$start <= 0 &
                           (is.na(cel_epi$stop) | cel_epi$stop >= 0))
  expect_equal(n_active_t0_epi, n_active_t0_nd)

  n_active_t1_nd  <- sum(cel_nd$onset <= 1 & cel_nd$terminus > 1)
  n_active_t1_epi <- sum(cel_epi$start <= 1 &
                           (is.na(cel_epi$stop) | cel_epi$stop >= 1))
  expect_equal(n_active_t1_epi, n_active_t1_nd)

  # Initial duration-1 edges (onset=0, terminus=1 in nD) -> start=0, stop=0.
  n_dur1_initial_nd  <- sum(cel_nd$onset == 0 & cel_nd$terminus == 1)
  n_dur1_initial_epi <- sum(cel_epi$start == 0 & cel_epi$stop == 0,
                            na.rm = TRUE)
  expect_equal(n_dur1_initial_epi, n_dur1_initial_nd)

  # Cross-section persistent edges live at start=0; nothing should be
  # earlier than that.
  expect_equal(min(cel_epi$start), 0)
})

test_that("truncate.el.cuml = 0 skips hist seed (#1016)", {
  skip_on_cran()
  set.seed(2)
  nw <- network_initialize(n = 100)
  est <- netest(
    nw,
    formation = ~edges,
    target.stats = 30,
    coef.diss = dissolution_coefs(~offset(edges), duration = 2),
    verbose = FALSE
  )

  control <- control.net(
    type = "SI", nsims = 1, nsteps = 3,
    cumulative.edgelist = TRUE,
    truncate.el.cuml = 0,
    save.cumulative.edgelist = TRUE,
    verbose = FALSE
  )

  mod <- netsim(est, param.net(inf.prob = 0.1),
                init.net(i.num = 5), control)
  el_cuml <- mod$cumulative.edgelist$sim1

  # truncate=0 keeps only active edges: all rows must have stop==NA, and
  # in particular the start=0/stop=0 hist seed must not be applied.
  expect_true(all(is.na(el_cuml$stop)))
})

test_that("tergmLite cumulative edgelist captures initial-step edges (#1016)", {
  skip_on_cran()
  set.seed(3)
  # Short duration so a substantial fraction of cross-section edges dissolves
  # during the initial TERGM step. Without the fix in #1017 those rows would
  # be missing entirely; with option (b) they should be present as
  # start=0/stop=0 rows.
  nw <- network_initialize(n = 200)
  est <- netest(
    nw,
    formation = ~edges,
    target.stats = 80,
    coef.diss = dissolution_coefs(~offset(edges), duration = 2),
    verbose = FALSE
  )

  control <- control.net(
    type = "SI", nsims = 1, nsteps = 5,
    resimulate.network = TRUE, tergmLite = TRUE,
    cumulative.edgelist = TRUE,
    truncate.el.cuml = Inf,
    save.cumulative.edgelist = TRUE,
    verbose = FALSE
  )

  set.seed(3)
  mod <- netsim(est, param.net(inf.prob = 0.1),
                init.net(i.num = 5), control)
  el_cuml <- mod$cumulative.edgelist$sim1

  # Cross-section persistent edges live at start=0 in tergmLite too, and
  # tergmLite is no longer limited on the initial-step dissolutions because
  # sim_nets_t1 stashes the t=0 edgelist before networkLite discards it.
  expect_equal(min(el_cuml$start), 0)
  expect_gt(sum(el_cuml$start == 0 & !is.na(el_cuml$stop) &
                  el_cuml$stop == 0), 0)
})

test_that("cumulative edgelist start values align with networkDynamic onsets (#1016)", {
  # Stronger pointwise check: every row's (start, stop) maps to a unique nD
  # spell (onset = start, terminus = stop + 1 with NA stop -> Inf).
  skip_on_cran()
  set.seed(4)
  nw <- network_initialize(n = 200)
  est <- netest(
    nw,
    formation = ~edges,
    target.stats = 60,
    coef.diss = dissolution_coefs(~offset(edges), duration = 5),
    verbose = FALSE
  )

  control <- control.net(
    type = "SI", nsims = 1, nsteps = 10,
    cumulative.edgelist = TRUE,
    truncate.el.cuml = Inf,
    save.cumulative.edgelist = TRUE,
    save.network = TRUE,
    verbose = FALSE
  )

  set.seed(4)
  mod <- netsim(est, param.net(inf.prob = 0.1),
                init.net(i.num = 5), control)

  cel_nd  <- as.data.frame(get_network(mod, sim = 1))
  cel_epi <- mod$cumulative.edgelist$sim1

  # Build comparable spell signatures. nD's terminus is exclusive; ours is
  # inclusive, so terminus = stop + 1 with NA stop -> Inf. Right-censored
  # terminus values (edges ongoing at end of observation) likewise map to
  # Inf so they compare equal to NA stops in the cumulative edgelist.
  nd_term  <- ifelse(cel_nd$terminus.censored, Inf, cel_nd$terminus)
  epi_term <- ifelse(is.na(cel_epi$stop), Inf, cel_epi$stop + 1)
  nd_key <- sort(paste(pmin(cel_nd$tail, cel_nd$head),
                       pmax(cel_nd$tail, cel_nd$head),
                       cel_nd$onset, nd_term, sep = "_"))
  epi_key <- sort(paste(pmin(cel_epi$head, cel_epi$tail),
                        pmax(cel_epi$head, cel_epi$tail),
                        cel_epi$start, epi_term, sep = "_"))
  expect_identical(epi_key, nd_key)
})

test_that("netsim, SI, Cumulative Edgelist with arrivals and departures", {
  skip_on_cran()
  nw <- network_initialize(n = 100)

  formation <- ~edges
  target.stats <- 70
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), 38, d.rate = 0.01)
  est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  param <- param.net(inf.prob = 0.1, act.rate = 1,
                     a.rate = 0.01, ds.rate = 0.01, di.rate = 0.01)

  init <- init.net(i.num = 10)
  control <- control.net(
    type = "SI", nsteps = 100, nsims = 1,
    tergmLite = TRUE, # required as we want the removal of inactive nodes
    resimulate.network = TRUE, verbose = FALSE,
    truncate.el.cuml = Inf,
    cumulative.edgelist = TRUE,
    raw.output = TRUE
  )

  sim1 <- netsim(est1, param, init, control)

  dat <- sim1[[1]]
  uid <- get_unique_ids(dat)
  pid <- get_posit_ids(dat)
  is_active_posit_ids(dat, pid)

  el_cuml <- get_cumulative_edgelists_df(dat)
  spids <- sample(pid, 20)
  partners <- get_partners(dat, spids)
  degree_cuml <- get_cumulative_degree(dat, spids)

  # checks the unicity of UIDs
  expect_true(all(table(uid) == 1))
  # checks that all attributed UID are not in the currently attributed UIDs
  expect_false(setequal(seq_len(dat$run$last_unique_id), get_unique_ids(dat)))
  # checks that all attributed UID are not in the currently attributed PIDs
  expect_false(setequal(seq_len(dat$run$last_unique_id), get_posit_ids(dat)))

  # checks the translation between UID and PID in get_partners
  expect_true(all(get_posit_ids(dat, unique(partners$index)) %in% spids))

})

#' @title Extract Model Data for Deterministic Compartmental Models
#'
#' @description This function extracts a model run from an object of class
#'              `dcm` into a data frame using the generic
#'              `as.data.frame` function.
#'
#' @param x An `EpiModel` object of class [dcm()].
#' @param run Run number for model; used for multiple-run sensitivity models. If
#'        `NULL` (default), will output data from all runs in a stacked data frame.
#' @param row.names See [as.data.frame.default()].
#' @param optional See [as.data.frame.default()].
#' @param ...  See [as.data.frame.default()].
#'
#' @details
#' Model output from [dcm()] simulations are available as a data
#' frame with this helper function. The output data frame will include
#' columns for time, the size of each compartment, the overall population
#' size (the sum of compartment sizes), and the size of each flow.
#'
#' For models with multiple runs (i.e., varying parameters - see example below),
#' the default with the `run` parameter not specified will output all runs
#' in a single stacked data frame.
#'
#' @return A data frame containing the data from `x`.
#'
#' @method as.data.frame dcm
#' @keywords extract
#' @export
#'
#' @examples
#' ## Example 1: One-group SIS model with varying act.rate
#' param <- param.dcm(inf.prob = 0.2, act.rate = seq(0.05, 0.5, 0.05),
#'                    rec.rate = 1/50)
#' init <- init.dcm(s.num = 500, i.num = 1)
#' control <- control.dcm(type = "SIS", nsteps = 10)
#' mod1 <- dcm(param, init, control)
#' as.data.frame(mod1)
#' as.data.frame(mod1, run = 1)
#' as.data.frame(mod1, run = 10)
#'
#' ## Example 2: Two-group SIR model with vital dynamics
#' param <- param.dcm(inf.prob = 0.2, inf.prob.g2 = 0.1,
#'                    act.rate = 3, balance = "g1",
#'                    rec.rate = 1/50, rec.rate.g2 = 1/50,
#'                    a.rate = 1/100, a.rate.g2 = NA,
#'                    ds.rate = 1/100, ds.rate.g2 = 1/100,
#'                    di.rate = 1/90, di.rate.g2 = 1/90,
#'                    dr.rate = 1/100, dr.rate.g2 = 1/100)
#' init <- init.dcm(s.num = 500, i.num = 1, r.num = 0,
#'                  s.num.g2 = 500, i.num.g2 = 1, r.num.g2 = 0)
#' control <- control.dcm(type = "SIR", nsteps = 10)
#' mod2 <- dcm(param, init, control)
#' as.data.frame(mod2)
#'
as.data.frame.dcm <- function(x, row.names = NULL, optional = FALSE, run = NULL,
                              ...) {

  df <- data.frame(time = x$control$timesteps)
  nruns <- x$control$nruns
  if (is.null(run)) {
    run <- seq_len(nruns)
  }

  # Output for models with 1 run
  if (nruns == 1) {
    if (length(run) > 1 || max(run) > 1) {
      stop("Maximum run is 1")
    }
    for (i in seq_along(x$epi)) {
      df[, i + 1] <- x$epi[[i]]
    }
  }

  # Output for models with multiple runs
  if (nruns > 1) {
    if (max(run) > nruns) {
      stop("Maximum run is ", nruns)
    }
    for (j in run) {
      if (j == min(run)) {
        for (i in seq_along(x$epi)) {
          df[, i + 1] <- x$epi[[i]][, j]
        }
        df$run <- j
      } else {
        tdf <- data.frame(time = seq(1, x$control$nsteps, x$control$dt))
        for (i in seq_along(x$epi)) {
          tdf[, i + 1] <- x$epi[[i]][, j]
        }
        tdf$run <- j
        df <- rbind(df, tdf)
      }
    }
    df <- df[, c(ncol(df), 1:(ncol(df) - 1))]
  }

  if (nruns > 1) {
    names(df)[3:ncol(df)] <- names(x$epi)
  } else {
    names(df)[2:ncol(df)] <- names(x$epi)
  }

  return(df)
}


#' @title Extract Model Data for Stochastic Models
#'
#' @description This function extracts model simulations for objects of classes
#'              `icm` and `netsim` into a data frame using
#'              the generic `as.data.frame` function.
#'
#' @param x An `EpiModel` object of class `icm` or `netsim`.
#' @param out Data output to data frame: `"mean"` for row means across
#'        simulations, `"sd"` for row standard deviations across
#'        simulations, `"qnt"` for row quantiles at the level specified in
#'        `qval`, or `"vals"` for values from individual simulations.
#' @param sim If `out="vals"`, the simulation number to output. If not
#'        specified, then data from all simulations will be output.
#' @param qval Quantile value required when `out="qnt"`.
#' @param row.names See [as.data.frame.default()].
#' @param optional See [as.data.frame.default()].
#' @param repair What to do with epi trackers that are too short. "drop" will
#'        remove them from the output, "pad" will add `NA` rows to get to the
#'        right size(Default = "drop").
#' @param ...  See [as.data.frame.default()].
#'
#' @details
#' These methods work for both `icm` and `netsim` class models. The
#' available output includes time-specific means, standard deviations,
#' quantiles, and simulation values (compartment and flow sizes) from these
#' stochastic model classes. Means, standard deviations, and quantiles are
#' calculated by taking the row summary (i.e., each row of data corresponds
#' to a time step) across all simulations in the model output.
#'
#' @return A data frame containing the data from `x`.
#'
#' @method as.data.frame icm
#' @keywords extract
#' @export
#'
#' @examples
#' ## Stochastic ICM SIS model
#' param <- param.icm(inf.prob = 0.8, act.rate = 2, rec.rate = 0.1)
#' init <- init.icm(s.num = 500, i.num = 1)
#' control <- control.icm(type = "SIS", nsteps = 10,
#'                        nsims = 3, verbose = FALSE)
#' mod <- icm(param, init, control)
#'
#' # Default output all simulation runs, default to all in stacked data.frame
#' as.data.frame(mod)
#' as.data.frame(mod, sim = 2)
#'
#' # Time-specific means across simulations
#' as.data.frame(mod, out = "mean")
#'
#' # Time-specific standard deviations across simulations
#' as.data.frame(mod, out = "sd")
#'
#' # Time-specific quantile values across simulations
#' as.data.frame(mod, out = "qnt", qval = 0.25)
#' as.data.frame(mod, out = "qnt", qval = 0.75)
#'
#' \dontrun{
#' ## Stochastic SI Network Model
#' nw <- network_initialize(n = 100)
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#' est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' param <- param.net(inf.prob = 0.5)
#' init <- init.net(i.num = 10)
#' control <- control.net(type = "SI", nsteps = 10, nsims = 3, verbose = FALSE)
#' mod <- netsim(est, param, init, control)
#'
#' # Same data extraction methods as with ICMs
#' as.data.frame(mod)
#' as.data.frame(mod, sim = 2)
#' as.data.frame(mod, out = "mean")
#' as.data.frame(mod, out = "sd")
#' as.data.frame(mod, out = "qnt", qval = 0.25)
#' as.data.frame(mod, out = "qnt", qval = 0.75)
#' }
#'
as.data.frame.icm <- function(x, row.names = NULL, optional = FALSE,
                              out = "vals", sim = NULL, qval = NULL,
                              repair = "drop", ...) {
  nsims <- x$control$nsims
  sim <- if (is.null(sim)) seq_len(nsims) else sim
  if (max(sim) > nsims) {
    stop("Maximum sim is ", nsims)
  }

  # Find and repair EPIs with the wrong length
  n_steps <- x$control$nsteps
  start <- if (is.null(x$control$start)) 1L else x$control$start
  epi_lengths <- vapply(x$epi, nrow, 0)
  broken_epi <- names(x$epi)[epi_lengths != n_steps]
  if (length(broken_epi) > 0) {
    if (repair == "drop") {
      warning("Dropping: ", paste(broken_epi, collapse = ", "), " - wrong length")
      x$epi <- x$epi[epi_lengths == n_steps]
    } else if (repair == "pad") {
      warning("Padding with NA: ", paste(broken_epi, collapse = ", "),
              " - wrong length")
      x$epi[broken_epi] <- lapply(x$epi[broken_epi], \(d) {
        n_miss <- n_steps - nrow(d)
        padding_rows <- as.data.frame(matrix(NA, nrow = n_miss, ncol = ncol(d)))
        colnames(padding_rows) <- colnames(d)
        rbind(d, padding_rows)
      })
    } else {
      stop(
        "Some EPIs had the wrong size: ", paste(broken_epi, collapse = ", "),
        "No repair method set, stopping"
      )
    }
  }

  # How to combine the different simulations
  combine_fun <- if (out == "vals") {
    function(epi) Reduce(c, epi[, sim, drop = FALSE])
  } else if (out == "mean") {
    function(epi) rowMeans(epi[, sim, drop = FALSE], na.rm = TRUE)
  } else if (out == "sd") {
    function(epi) apply(epi[, sim, drop = FALSE], 1, sd, na.rm = TRUE)
  } else if (out == "qnt") {
    if (is.null(qval) || length(qval) > 1 || (qval > 1 || qval < 0)) {
      stop("Must specify qval as single value between 0 and 1")
    }
    function(epi) {
      apply(epi[, sim, drop = FALSE], 1, quantile,
            probs = qval, na.rm = TRUE, names = FALSE)
    }
  }
  epi_ls <- lapply(x$epi, combine_fun)

  time_seq <- seq(start, start + n_steps - 1)
  if (out == "vals") { # add columns `time` and `sim`
    common_elts <- list(
      sim = rep(sim, each = n_steps),
      time = rep(time_seq, length(sim))
    )
    df <- as.epi.data.frame(as.data.frame(c(common_elts, epi_ls)))
  } else { # add column `time`
    df <- as.data.frame(c(list(time = time_seq), epi_ls))
  }

  return(df)
}


#' @method as.data.frame netsim
#' @export
#' @rdname as.data.frame.icm
as.data.frame.netsim <- function(x, row.names = NULL, optional = FALSE,
                                 out = "vals", sim = NULL, repair = "drop",
                                 ...) {
  as.data.frame.icm(x, row.names = row.names, optional = optional,
                    sim = sim, out = out, repair = repair, ...)
}

#' @title Extract Timed Edgelists for netdx Objects
#'
#' @description This function extracts timed edgelists for objects of class
#'              `netdx` into a data frame using the generic
#'              `as.data.frame` function.
#'
#' @param x An `EpiModel` object of class `netdx`.
#' @param sim The simulation number to output. If `NULL`, then data from
#'        all simulations will be output.
#' @param row.names See [as.data.frame.default()].
#' @param optional See [as.data.frame.default()].
#' @param ...  See [as.data.frame.default()].
#'
#' @return A data frame containing the data from `x`.
#'
#' @method as.data.frame netdx
#' @keywords extract
#' @export
#'
#' @examples
#' \donttest{
#' # Initialize and parameterize the network model
#' nw <- network_initialize(n = 100)
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#'
#' # Model estimation
#' est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' # Simulate the network with netdx
#' dx <- netdx(est, nsims = 3, nsteps = 10, keep.tedgelist = TRUE,
#'             verbose = FALSE)
#'
#' # Extract data from the first simulation
#' as.data.frame(dx, sim = 1)
#'
#' # Extract data from all simulations
#' as.data.frame(dx)
#' }
#'
as.data.frame.netdx <- function(x, row.names = NULL, optional = FALSE,
                                sim = NULL, ...) {

  if (is.null(x$tedgelist)) {
    stop("Edgelist not saved in netdx object, check keep.tedgelist parameter")
  }

  if (is.null(sim)) {
    sim <- seq_len(x$nsims)
  }
  if (max(sim) > x$nsims) {
    stop(paste("Maximum sim is", x$nsims))
  }

  if (length(sim) == 1) {
    df <- x$tedgelist[[sim]]
  } else {
    for (j in sim) {
      if (j == min(sim)) {
        df <- x$tedgelist[[j]]
        df$sim <- j
      } else {
        tdf <- x$tedgelist[[j]]
        tdf$sim <- j
        df <- rbind(df, tdf)
      }
    }
    df <- df[, c(ncol(df), 1:(ncol(df) - 1))]
  }

  return(df)
}

#' Validate and Convert to epi.data.frame
#'
#' This methods ensures that the `data.frame` is correctly formatted as an
#' `epi.data.frame`
#'
#' @param df A `data.frame` to convert into an `epi.data.frame`
#'
#' @export
as.epi.data.frame <- function(df) {
  if (!inherits(df, "data.frame"))
    stop("Input must be a `data.frame`")
  if (!all(c("time", "sim") %in% names(df)))
    stop("Input must contain a `time` and a `sim` column")
  n_steps <- length(unique(df$time))
  if (!all(by(df$time, df$sim, length) == n_steps))
    stop("Input must have the same number of time step per simulation")
  class(df) <- c("epi.data.frame", class(df))
  return(df)
}

#' @title Get Epidemic Output from netsim Model
#'
#' @description Provides all active model state sizes from the network at the
#'              specified time step, output to a list of vectors.
#'
#' @inheritParams recovery.net
#'
#' @details
#' This network utility is used during the [netsim()] simulation
#' process to efficiently query the current size of each state or compartment
#' in the model at any given timestep. For a two-group network, the current
#' state size for each group and overall is provided.
#'
#' @inherit recovery.net return
#'
#' @export
#' @keywords netUtils internal
#'
prevalence.net <- function(dat, at) {
  active <- get_attr(dat, "active")

  type <- get_control(dat, "type", override.null.error = TRUE)
  type <- if (is.null(type)) "None" else type

  groups <- get_param(dat, "groups")

  # Subset attr to active == 1
  attr_list <- get_attr_list(dat)
  l <- lapply(seq_along(attr_list), function(x) attr_list[[x]][active == 1])
  names(l) <- names(attr_list)
  l$active <- l$infTime <- NULL

  status <- l$status

  ## Subsetting for epi.by control
  ebn <- get_control(dat, "epi.by", override.null.error = TRUE)
  if (!is.null(ebn)) {
    ebv <- dat$run$epi.by.vals
    ebun <- paste0(".", ebn, ebv)
    assign(ebn, l[[ebn]])
  }

  if (groups == 1) {
    dat <- set_epi(dat, "s.num", at, sum(status == "s"))
    if (!is.null(ebn)) {
      for (i in seq_along(ebun)) {
        ebn.temp <- paste0("s.num", ebun[i])
        dat <- set_epi(dat, ebn.temp, at, sum(status == "s" &
                                                get(ebn) == ebv[i]))
      }
    }

    dat <- set_epi(dat, "i.num", at, sum(status == "i"))
    if (!is.null(ebn)) {
      for (i in seq_along(ebun)) {
        ebn.temp <- paste0("i.num", ebun[i])
        dat <- set_epi(dat, ebn.temp, at, sum(status == "i" &
                                                get(ebn) == ebv[i]))
      }
    }

    if (type == "SIR") {
      dat <- set_epi(dat, "r.num", at, sum(status == "r"))
      if (!is.null(ebn)) {
        for (i in seq_along(ebun)) {
          ebn.temp <- paste0("r.num", ebun[i])
          dat <- set_epi(dat, ebn.temp, at, sum(status == "r" &
                                                  get(ebn) == ebv[i]))
        }
      }
    }
    dat <- set_epi(dat, "num", at, length(status))
    if (!is.null(ebn)) {
      for (i in seq_along(ebun)) {
        ebn.temp <- paste0("num", ebun[i])
        dat <- set_epi(dat, ebn.temp, at, sum(get(ebn) == ebv[i]))
      }
    }
  }

  if (groups == 2) {
    group <- get_attr(dat, "group")
    group <- group[active == 1]

    dat <- set_epi(dat, "s.num", at, sum(status == "s" & group == 1))
    if (!is.null(ebn)) {
      for (i in seq_along(ebun)) {
        ebn.temp <- paste0("s.num", ebun[i])
        dat <- set_epi(dat, ebn.temp, at, sum(status == "s" &
                                                group == 1 &
                                                get(ebn) == ebv[i]))
      }
    }
    dat <- set_epi(dat, "i.num", at, sum(status == "i" & group == 1))
    if (!is.null(ebn)) {
      for (i in seq_along(ebun)) {
        ebn.temp <- paste0("i.num", ebun[i])
        dat <- set_epi(dat, ebn.temp, at, sum(status == "i" &
                                                group == 1 &
                                                get(ebn) == ebv[i]))
      }
    }
    if (type == "SIR") {
      dat <- set_epi(dat, "r.num", at, sum(status == "r" & group == 1))
      if (!is.null(ebn)) {
        for (i in seq_along(ebun)) {
          ebn.temp <- paste0("r.num", ebun[i])
          dat <- set_epi(dat, ebn.temp, at, sum(status == "r" &
                                                  group == 1 &
                                                  get(ebn) == ebv[i]))
        }
      }
    }
    dat <- set_epi(dat, "num", at, sum(group == 1))
    if (!is.null(ebn)) {
      for (i in seq_along(ebun)) {
        ebn.temp <- paste0("num", ebun[i])
        dat <- set_epi(dat, ebn.temp, at, sum(group == 1 &
                                                get(ebn) == ebv[i]))
      }
    }
    dat <- set_epi(dat, "s.num.g2", at, sum(status == "s" & group == 2))
    if (!is.null(ebn)) {
      for (i in seq_along(ebun)) {
        ebn.temp <- paste0("s.num.g2", ebun[i])
        dat <- set_epi(dat, ebn.temp, at, sum(status == "s" &
                                                group == 2 &
                                                get(ebn) == ebv[i]))
      }
    }
    dat <- set_epi(dat, "i.num.g2", at, sum(status == "i" & group == 2))
    if (!is.null(ebn)) {
      for (i in seq_along(ebun)) {
        ebn.temp <- paste0("i.num.g2", ebun[i])
        dat <- set_epi(dat, ebn.temp, at, sum(status == "i" &
                                                group == 2 &
                                                get(ebn) == ebv[i]))
      }
    }
    if (type == "SIR") {
      dat <- set_epi(dat, "r.num.g2", at, sum(status == "r" & group == 2))
      if (!is.null(ebn)) {
        for (i in seq_along(ebun)) {
          ebn.temp <- paste0("r.num.g2", ebun[i])
          dat <- set_epi(dat, ebn.temp, at, sum(status == "r" &
                                                  group == 2 &
                                                  get(ebn) == ebv[i]))
        }
      }
    }
    dat <- set_epi(dat, "num.g2", at, sum(group == 2))
    if (!is.null(ebn)) {
      for (i in seq_along(ebun)) {
        ebn.temp <- paste0("num.g2", ebun[i])
        dat <- set_epi(dat, ebn.temp, at, sum(group == 2 &
                                                get(ebn) == ebv[i]))
      }
    }
  }
  return(dat)
}

#' @title Initialize Network Object
#'
#' @description Initialize an undirected network object for use in EpiModel
#'              workflows.
#'
#' @param n Network size.
#'
#' @details
#' This function is used in `EpiModel` workflows to initialize an empty
#' network object.  The network attributes `directed`, `bipartite`,
#' `hyper`, `loops`, and `multiple` are set to `FALSE`.
#'
#' @return
#' Returns an object of class `network`.
#'
#' @export
#'
#' @examples
#' nw <- network_initialize(100)
#' nw
#'
network_initialize <- function(n) {
  nw <- network.initialize(n, directed = FALSE, hyper = FALSE, loops = FALSE,
                           multiple = FALSE, bipartite = FALSE)
  return(nw)
}

#' @title Set Vertex Attribute on Network Object
#'
#' @description Sets a vertex attribute on an object of class `network`.
#'              This function simplifies the related function in the
#'              `network` package.
#'
#' @param x An object of class network.
#' @param attrname The name of the attribute to set.
#' @param value A vector of values of the attribute to be set.
#' @param v IDs for the vertices whose attributes are to be altered.
#'
#' @details
#' This function is used in `EpiModel` workflows to set vertex attributes
#' on an initialized empty network object (see [network_initialize()]).
#'
#' @return
#' Returns an object of class `network`.
#'
#' @export
#'
#' @examples
#' nw <- network_initialize(100)
#' nw <- set_vertex_attribute(nw, "age", runif(100, 15, 65))
#' nw
#'
set_vertex_attribute <- function(x, attrname, value, v = NULL) {
  if (is.null(v)) {
    v <- seq_len(network.size(x))
  }
  g <- set.vertex.attribute(x, attrname, value, v)
  return(g)
}

#' @title Get Vertex Attribute on Network Object
#'
#' @description Gets a vertex attribute from an object of class `network`.
#'              This function simplifies the related function in the
#'              `network` package.
#'
#' @param x An object of class network.
#' @param attrname The name of the attribute to get.
#'
#' @details
#' This function is used in `EpiModel` workflows to query vertex
#' attributes on an initialized empty network object (see
#' [network_initialize()]).
#'
#' @return
#' Returns a vector of vertex attribute values for the attribute specified
#' by `attrname`.
#'
#' @export
#'
#' @examples
#' nw <- network_initialize(100)
#' nw <- set_vertex_attribute(nw, "age", runif(100, 15, 65))
#' get_vertex_attribute(nw, "age")
#'
get_vertex_attribute <- function(x, attrname) {
  attr <- get.vertex.attribute(x, attrname, na.omit = FALSE,
                               null.na = TRUE, unlist = TRUE)
  return(attr)
}

#' @title Get Network Attributes from a Network Object
#'
#' @description Gets all network attributes except `"mnext"` from its
#'              network argument.
#'
#' @param x An object of class `network` or `networkLite`.
#'
#' @details
#' This function is used in `EpiModel` workflows to copy relevant network
#' attributes from the network object to the `netsim_dat` object when
#' initializing `netsim` runs.
#'
#' @return
#' Returns the named list of network attributes.
#'
#' @export
#'
#' @examples
#' nw <- network_initialize(100)
#' get_network_attributes(nw)
#'
get_network_attributes <- function(x) {
  out <- list()
  for (name in setdiff(list.network.attributes(x), c("mnext"))) {
    new <- list(get.network.attribute(x, name))
    names(new) <- name
    out <- c(out, new)
  }
  out
}

context("Network Attributes with Arrivals")

test_that("Updating attributes in open populations", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, attrname = "group", rep(1:2, each = 25))

  formation <- ~edges + nodefactor("group")
  target.stats <- c(25, 36)
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), 38,
                                 d.rate = 0.002)
  est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  probs <- c(0.2055, 0.0088, 0.0614, 0)
  durs <- c(3, 100, 9, 10)
  inf.probs <- rep(probs, durs)
  inf.probsf <- inf.probs * 2
  param <- param.net(inf.prob = inf.probs, act.rate = 1,
                     inf.prob.g2 = inf.probs,
                     a.rate = 0.05, a.rate.g2 = NA,
                     ds.rate = 0.05, ds.rate.g2 = 0.05,
                     di.rate = 0.05, di.rate.g2 = 0.05)

  init <- init.net(i.num = 10, i.num.g2 = 10)
  control <- control.net(type = "SI", nsteps = 20, nsims = 1,
                         resimulate.network = TRUE, verbose = FALSE)

  sim1 <- netsim(est1, param, init, control)
  expect_is(sim1, "netsim")
})

test_that("SIR model with epi.by parameter", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, attrname = "race", rep(0:1, each = 25))
  formation <- ~edges + nodefactor("race")
  target.stats <- c(25, 25)
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), 50)
  est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
  param <- param.net(inf.prob = 0.1, act.rate = 1, rec.rate = 0.005)
  init <- init.net(i.num = 10, r.num = 0)
  control <- control.net(type = "SIR", nsteps = 10, nsims = 1,
                         epi.by = "race", verbose = FALSE, verbose.int = 0)
  sim <- netsim(est, param, init, control)
  expect_is(sim, "netsim")
  expect_true(all(c("s.num.race0", "s.num.race1", "i.num.race0", "i.num.race1",
                    "r.num.race0", "r.num.race1") %in% names(sim$epi)))
})


test_that("Serosorting model in open population", {
  skip_on_cran()
  n <- 100
  nw <- network_initialize(n = n)

  prev <- 0.2
  infIds <- sample(1:n, n * prev)
  nw <- set_vertex_attribute(nw, "status", "s")
  nw <- set_vertex_attribute(nw, "status", "i", infIds)
  nw <- set_vertex_attribute(nw, "race", rbinom(n, 1, 0.5))

  formation <- ~edges + nodefactor("status", levels = -1) +
                nodematch("status") + nodematch("race")
  target.stats <- c(36, 55, 25, 18)
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), 5, d.rate = 0.01)
  est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  param <- param.net(inf.prob = 0.8, a.rate = 0.05,
                     ds.rate = 0.01, di.rate = 0.01)
  init <- init.net()
  control <- control.net(type = "SI", nsteps = 20, nsims = 1,
                         nwstats.formula = ~edges +
                                            meandeg +
                                            nodefactor("status",
                                                       levels = NULL) +
                                            nodematch("status"),
                         tergmLite = FALSE,
                         resimulate.network = TRUE,
                         save.run = TRUE,
                         verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_is(sim, "netsim")

  nD <- get_network(sim)
  tea1 <- get.vertex.attribute.active(nD, "testatus", at = 1)
  expect_true(sum(!is.na(tea1)) == n)

  tea20 <- get.vertex.attribute.active(nD, "testatus", at = 20)
  expect_true(sum(is.na(tea20)) == 0)

  fstat.nw <- get_vertex_attribute(nD, "status")
  fstat.attr <- sim$run[[1]]$attr$status

  expect_identical(tea20, fstat.nw)
  expect_identical(fstat.nw, fstat.attr)

})

test_that("Serosorting model in closed population", {
  skip_on_cran()
  n <- 100
  nw <- network_initialize(n = n)

  prev <- 0.2
  infIds <- sample(1:n, n * prev)
  nw <- set_vertex_attribute(nw, "status", "s")
  nw <- set_vertex_attribute(nw, "status", "i", infIds)
  nw <- set_vertex_attribute(nw, "race", rbinom(n, 1, 0.5))

  formation <- ~edges + nodefactor("status", levels = -1) +
    nodematch("status") + nodematch("race")
  target.stats <- c(36, 55, 25, 18)
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), 5)
  est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  param <- param.net(inf.prob = 0.8)
  init <- init.net()
  control <- control.net(type = "SI", nsteps = 20, nsims = 1,
                         nwstats.formula = ~edges +
                           meandeg +
                           nodefactor("status", levels = NULL) +
                           nodematch("status"),
                         tergmLite = FALSE,
                         resimulate.network = TRUE,
                         save.run = TRUE,
                         verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_is(sim, "netsim")

  nD <- get_network(sim)
  nD
  tea1 <- get.vertex.attribute.active(nD, "testatus", at = 1)

  expect_true(sum(!is.na(tea1)) == n)

  tea20 <- get.vertex.attribute.active(nD, "testatus", at = 20)
  expect_true(sum(is.na(tea20)) == 0)

  fstat.nw <- get_vertex_attribute(nD, "status")
  fstat.attr <- sim$run[[1]]$attr$status

  expect_identical(tea20, fstat.nw)
  expect_identical(fstat.nw, fstat.attr)

})


test_that("Serosorting model in open population, with tergmLite", {
  skip_on_cran()
  n <- 100
  nw <- network_initialize(n = n)

  prev <- 0.2
  infIds <- sample(1:n, n * prev)
  nw <- set_vertex_attribute(nw, "status", "s")
  nw <- set_vertex_attribute(nw, "status", "i", infIds)
  nw <- set_vertex_attribute(nw, "race", rbinom(n, 1, 0.5))

  formation <- ~edges + nodefactor("status", levels = -1) +
    nodematch("status") + nodematch("race")
  target.stats <- c(36, 55, 25, 18)
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), 5, d.rate = 0.01)
  est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  param <- param.net(inf.prob = 0.8, a.rate = 0.05,
                     ds.rate = 0.01, di.rate = 0.01)
  init <- init.net()
  control <- control.net(type = "SI", nsteps = 20, nsims = 1,
                         nwstats.formula = ~edges +
                           meandeg +
                           nodefactor("status", levels = NULL) +
                           nodematch("status"),
                         tergmLite = TRUE,
                         resimulate.network = TRUE,
                         save.run = TRUE,
                         verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_is(sim, "netsim")

})

test_that("Save attributes to output", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 25))
  formation <- ~edges + nodematch("group")
  target.stats <- c(25, 0)
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), 38,
                                 d.rate = 0.01)
  est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  param <- param.net(inf.prob = 0.2, act.rate = 1,
                     inf.prob.g2 = 0.2,
                     a.rate = 0.01, a.rate.g2 = NA,
                     ds.rate = 0.01, ds.rate.g2 = 0.01,
                     di.rate = 0.01, di.rate.g2 = 0.01)

  init <- init.net(i.num = 10, i.num.g2 = 10)
  control <- control.net(type = "SI", nsteps = 10, nsims = 2,
                         save.run = TRUE, resimulate.network = TRUE,
                         verbose = FALSE)

  sim1 <- netsim(est1, param, init, control)
  expect_is(sim1, "netsim")
  expect_is(sim1$run[[1]]$attr, "list")
  expect_true(all(c("entrTime", "exitTime") %in% names(sim1$run[[1]]$attr)))
})


test_that("Check TE Status Variable Against Epi Stats", {

  skip_on_cran()

  nw <- network_initialize(n = 100)
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), 38)
  est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  # SIR
  param <- param.net(inf.prob = 0.4, act.rate = 2, rec.rate = 0.01)
  init <- init.net(i.num = 10, r.num = 0)
  control <- control.net(type = "SIR", nsims = 1, nsteps = 100, verbose = FALSE)
  sim <- netsim(est, param, init, control)
  times <- sample(1:100, 10)
  for (at in times) {
    df <- as.data.frame(sim)[at, ]
    nwd <- get_network(sim, collapse = TRUE, at = at)
    attr <- get_vertex_attribute(nwd, "testatus")
    expect_true(sum(attr == "s") == df$s.num)
    expect_true(sum(attr == "i") == df$i.num)
    expect_true(sum(attr == "r") == df$r.num)
  }

  # SIS
  param <- param.net(inf.prob = 0.4, act.rate = 2, rec.rate = 0.01)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SIS", nsims = 1, nsteps = 100, verbose = FALSE)
  sim <- netsim(est, param, init, control)
  times <- sample(1:100, 10)
  for (at in times) {
    df <- as.data.frame(sim)[at, ]
    nwd <- get_network(sim, collapse = TRUE, at = at)
    attr <- get_vertex_attribute(nwd, "testatus")
    expect_true(sum(attr == "s") == df$s.num)
    expect_true(sum(attr == "i") == df$i.num)
  }

})

context("Network Model Restart")

test_that("network models can be restarted", {
  skip_on_cran()

  nw <- network_initialize(n = 100)
  est.vit <- netest(nw, formation = ~edges, target.stats = 25,
                    coef.diss = dissolution_coefs(~offset(edges), 10, 0.02),
                    verbose = FALSE)

  param <- param.net(inf.prob = 0.5, act.rate = 2, a.rate = 0.02,
                     ds.rate = 0.02, di.rate = 0.02)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsteps = 5, nsims = 1,
                         resimulate.network = TRUE, verbose = FALSE,
                         save.run = TRUE,
                         save.other = c())
  x <- netsim(est.vit, param, init, control)

  control <- control.net(type = "SI", nsteps = 10, start = 6,
                         nsims = 1, verbose = FALSE)
  x2 <- netsim(x, param, init, control)

  expect_is(x, "netsim")
  expect_is(x2, "netsim")
  expect_true(x$control$nsteps == 5)
  expect_true(x2$control$nsteps == 10)

  plot(x)
  plot(x2)

})


test_that("restart error flags", {
  skip_on_cran()

  nw <- network_initialize(n = 100)
  est.vit <- netest(nw, formation = ~edges, target.stats = 25,
                    coef.diss = dissolution_coefs(~offset(edges), 10, 0.02),
                    verbose = FALSE)

  param <- param.net(inf.prob = 0.5, act.rate = 2, a.rate = 0.02,
                     ds.rate = 0.02, di.rate = 0.02)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsteps = 5,
                         nsims = 1, resimulate.network = TRUE,
                         verbose = FALSE,
                         save.run = TRUE)
  x <- netsim(est.vit, param, init, control)

  control <- control.net(type = "SI", nsteps = 5, start = 10,
                         nsims = 1, verbose = FALSE)
  expect_error(netsim(x, param, init, control),
               "control setting nsteps must be >")

  control <- control.net(type = "SI", nsteps = 10, start = 7,
                         nsims = 1, verbose = FALSE)
  expect_error(netsim(x, param, init, control),
               "control setting start must be 1")

  control <- control.net(type = "SI", nsteps = 10, start = 6,
                         nsims = 1, verbose = FALSE)
  x$run <- NULL
  expect_error(netsim(x, param, init, control), "x must contain `run` to restart simulation, see `save.run` control setting")

})

test_that("reinitialization works with open population, nwterms, and epi.by", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  nw %v% "race" <- rep(0:1, length.out = 50)
  est <- netest(nw, formation = ~edges + nodematch("race"),
                target.stats = c(25, 15),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0.05),
                verbose = FALSE)

  param <- param.net(inf.prob = 0.5, act.rate = 2, a.rate = 0.05,
                     ds.rate = 0.05, di.rate = 0.05)

  init <- init.net(i.num = 10)

  for (tergmLite in c(FALSE, TRUE)) {
    control <- control.net(type = "SI", nsteps = 5,
                           nsims = 2, resimulate.network = TRUE,
                           verbose = FALSE, tergmLite = tergmLite,
                           epi.by = "race",
                           save.run = TRUE,
                           save.other = c())

    x <- netsim(est, param, init, control)
    expect_is(x, "netsim")
    control$start <- 6
    control$nsteps <- 11
    y <- netsim(x, param, init, control)
    expect_is(y, "netsim")
  }
})

test_that("reinitialization a truncated netsim object", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  nw %v% "race" <- rep(0:1, length.out = 50)
  est <- netest(
    nw,
    formation = ~edges + nodematch("race"),
    target.stats = c(25, 15),
    coef.diss = dissolution_coefs(~offset(edges), 10, 0.05),
    verbose = FALSE
  )

  param <- param.net(
    inf.prob = 0.5,
    act.rate = 2,
    a.rate = 0.05,
    ds.rate = 0.05,
    di.rate = 0.05
  )

  init <- init.net(i.num = 10)

  control <- control.net(
    type = "SI", nsteps = 10,
    nsims = 2, resimulate.network = TRUE,
    verbose = FALSE, tergmLite = TRUE,
    epi.by = "race",
    save.run = TRUE,
    save.other = c()
  )

  sim <- netsim(est, param, init, control)
  expect_is(sim, "netsim")

  # From a `netsim`, chose the simulation to use and trim the all but one
  # timestep to get a lighter restart object.
  restart_point <- make_restart_point(
    sim_obj = sim,
    time_attrs = c("infTime"),
    sim_num = 1
  )

  # In this case, the `restart_point` contains a single timestap
  control$start <- restart_point$control$nsteps + 1
  control$nsteps <- restart_point$control$nsteps + 1 + 11
  y <- netsim(restart_point, param, init, control)
  expect_is(y, "netsim")

  control <- control.net(
    type = "SI", nsteps = 5,
    nsims = 2, resimulate.network = TRUE,
    verbose = FALSE, tergmLite = FALSE,
    epi.by = "race",
    save.run = TRUE,
    save.other = c()
  )
  sim <- netsim(est, param, init, control)
  expect_error(make_restart_point(
    sim_obj = sim,
    sim_num = 1,
    time_attrs = c("infTime")
  ))
})

#' @title Update List `x` Using the Elements of List `new.x`.
#'
#' @param x A list.
#' @param new.x A list.
#'
#' @return The full `x` list with the modifications added by `new.x`.
#'
#' @details
#' This function updates list `x` by name. If `x` and `new.x`
#' elements are not named, the function will not work properly. If a function is
#' provided to replace an element that was originally not a function, this
#' function will be applied to the original value.
#'
#' @keywords internal
update_list <- function(x, new.x) {
  for (nm in names(new.x)) {
    if (is.list(new.x[[nm]])) {
      x[[nm]] <- update_list(x[[nm]], new.x[[nm]])
    } else if (is.function(new.x[[nm]]) && !is.function(x[[nm]])) {
      x[[nm]] <- new.x[[nm]](x[[nm]])
    } else {
      x[[nm]] <- new.x[[nm]]
    }
  }

  return(x)
}

#' @title Function to Modify the Controls or Parameters During the Simulation
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @details
#' If a list `.param.updater.list` is present in the parameters, this
#' function will update the `param` list with new values at given time
#' steps. Similarly, if a list `.control.updater.list` is present in the
#' controls, this function will update the `param` list with new values at
#' given time steps. An updater is a list containing an `at` element
#' governing when the changes will happen, an optional `verbose` Boolean
#' controlling whether to output a message when a change is made (default =
#' TRUE), and a `param` or `control` named list containing the new
#' values(s) with which to update, with the names being the same as the
#' parameter / control names. If the new value is a function but the old one is
#' not, the function will be applied to the current element (see example).
#'
#' @examples
#' \dontrun{
#'
#' # Create the .param.updater.list
#' .param.updater.list <- list(
#'   # this is one updater
#'   list(
#'     at = 10,
#'     param = list(
#'       hiv.test.rate = rep(0.0128, 3),
#'       trans.scale = c(1.61, 0.836, 0.622)
#'     )
#'   ),
#'   # this is another updater
#'   list(
#'     at = 12,
#'     verbose = TRUE,
#'     param = list(
#'       hiv.test.rate = function(x) x * 3,
#'       trans.scale = function(x) x^2 / 3
#'     )
#'   )
#' )
#'
#'  # Add it to params
#'  param <- param.net(
#'    inf.prob = 0.3,
#'    act.rate = 0.5,
#'    hiv.test.rate = rep(0.256, 3),
#'    trans.scale = c(1, 2, 3),
#'    .param.updater.list = param.updater.list
#'  )
#'
#' # Create the .control.updater.list
#' # these updaters will toggle on and off the verbosity of the model
#' control.updater.list <- list(
#'   list(
#'     at = 50,
#'     verbose = TRUE,
#'     control = list(
#'       verbose = TRUE
#'     )
#'   ),
#'   # this is another updater
#'   list(
#'     at = 75,
#'     verbose = TRUE,
#'     control = list(
#'       verbose = FALSE
#'     )
#'   )
#' )
#'
#' Add the control updaters to the `control` object
#'  control <- control.net(
#'    nsims = 1,
#'    nsteps = 20,
#'    verbose = FALSE,
#'    .control.updater.list = control.updater.list
#'  )
#'
#' nw <- network_initialize(n = 50)
#' nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
#' est <- netest(
#'   nw,
#'   formation = ~edges,
#'   target.stats = 25,
#'   coef.diss = dissolution_coefs(~offset(edges), 10, 0),
#'   verbose = FALSE
#' )
#'
#' init <- init.net(i.num = 10)
#' mod <- netsim(est, param, init, control)
#'
#' }
#'
#' @keywords internal
#' @noRd
input_updater <- function(dat) {
  for (type in c("param", "control")) {
    dat <- common_updater(dat, type)
  }

  return(dat)
}

#' @title Update Either the "param" or "control" List
#'
#' @inheritParams recovery.net
#' @param type Either `"param"` or `"control"`.
#'
#' @inherit recovery.net return
#'
#' @keywords internal
common_updater <- function(dat, type) {
  # Set the variables and functions for either `param` or `control`
  if (type == "param") {
    type.label <- "parameters"
    type.set <- set_param
    type.get_list <- get_param_list
    updaters.label <- ".param.updater.list"
    updaters <- get_param(dat, updaters.label, override.null.error = TRUE)
  } else if (type == "control") {
    type.label <- "controls"
    type.set <- set_control
    type.get_list <- get_control_list
    updaters.label <- ".control.updater.list"
    updaters <- get_control(dat, updaters.label, override.null.error = TRUE)
  } else {
    stop("`type` must be either 'param' or 'control'")
  }

  # Common update mechanism
  if (is.null(updaters) || length(updaters) == 0) {
    return(dat)
  }

  used.updaters <- numeric(0)
  at <- get_current_timestep(dat)

  for (i in seq_along(updaters)) {
    if (updaters[[i]][["at"]] == at) {
      verbose <- updaters[[i]][["verbose"]]
      verbose <- if (is.null(verbose)) TRUE else verbose

      new.list <- updaters[[i]][[type]]

      if (verbose) {
        message(
          "\n\nAt timestep = ", at, " the following ", type.label,
          " were modified:",
          "\n'", paste0(names(new.list), collapse = "', '"), "'"
        )
      }

      old.list <- type.get_list(dat, names(new.list))
      updated.list <- update_list(old.list, new.list)

      for (nm in names(updated.list)) {
        dat <- type.set(dat, nm, updated.list[[nm]])
      }

      used.updaters <- c(used.updaters, i)
    }
  }

  # Remove the used updaters from the list
  if (length(used.updaters) > 0) {
    dat <- type.set(
      dat, updaters.label,
      updaters[-used.updaters]
    )
  }

  return(dat)
}

#' @title Function to run the user-provided epi trackers
#'
#' @description
#' see the "Working with Custom Attributes and Summary Statistics in EpiModel"
#' vignette.
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @section The `tracker.list` list:
#' `.tracker.list` is a list of NAMED functions stored in the
#' `control` list of the main `netsim_dat` class object.
#'
#' @section Tracker Functions:
#' This function will apply the tracker functions present in the control list
#' `.tracker.list`. Each tracker must be a function with EXACTLY one
#' argument: the `netsim_dat` main list object. They must return a VALUE of
#' length one (numeric, logical or character).
#'
#' @examples
#' \dontrun{
#'
#' # Create some trackers
#' epi_prop_infected <- function(dat) {
#'   # we need two attributes for our calculation: `status` and `active`
#'   needed_attributes <- c("status", "active")
#'   # we use `with` to simplify code
#'   output <- with(EpiModel::get_attr_list(dat, needed_attributes), {
#'     pop <- active == 1             # we only look at active nodes
#'     cond <- status == "i"   # which are infected
#'     # how many are `infected` among the `active`
#'     sum(cond & pop, na.rm = TRUE) / sum(pop, na.rm = TRUE)
#'   })
#'   return(output)
#' }
#'
#' epi_s_num <- function(dat) {
#'   needed_attributes <- c("status")
#'   output <- with(get_attr_list(dat, needed_attributes), {
#'     sum(status == "s", na.rm = TRUE)
#'   })
#'   return(output)
#' }
#'
#' # Store the trackers in a named list. The names will be used as column names
#' # for in the `epi` list
#' some.trackers <- list(
#'   prop_infected = epi_prop_infected,
#'   s_num         = epi_s_num
#' )
#'
#' # Make a simple SI model with custom trackers
#' control <- EpiModel::control.net(
#'   type = "SI",
#'   nsims = 1,
#'   nsteps = 50,
#'   verbose = FALSE,
#'   .tracker.list = some.trackers
#' )
#'
#' param <- EpiModel::param.net(
#'   inf.prob = 0.3,
#'   act.rate = 0.1
#' )
#'
#' nw <- network_initialize(n = 50)
#' nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
#' est <- EpiModel::netest(
#'   nw,
#'   formation = ~edges,
#'   target.stats = 25,
#'   coef.diss = dissolution_coefs(~offset(edges), 10, 0),
#'   verbose = FALSE
#' )
#'
#' init <- EpiModel::init.net(i.num = 10)
#' sim <- EpiModel::netsim(est, param, init, control)
#'
#' d <- as.data.frame(sim)
#' d
#' }
#'
#' @seealso [netsim()]
#'
#' @keywords internal
epi_trackers <- function(dat) {
  tracker.list <- get_control(dat, ".tracker.list", override.null.error = TRUE)

  if (is.null(tracker.list)) {
    return(dat)
  }

  at <- get_current_timestep(dat)

  tryCatch(
    expr = {
      for (nm in names(tracker.list)) {
        dat <- set_epi(dat, nm, at, tracker.list[[nm]](dat))
      }
    },
    message = function(e) message("\nIn tracker '", nm, "':\n", e),
    warning = function(e) warning("\nIn tracker '", nm, "':\n", e),
    error   = function(e) stop("\nIn tracker '", nm, "':\n", e)
  )

  return(dat)
}

#' @title Initialization: icm Module
#'
#' @description This function initializes the main `icm_dat` class data
#'              object, and simulates disease status and other attributes.
#'
#' @param param An `EpiModel` object of class [param.icm()].
#' @param init An `EpiModel` object of class [init.icm()].
#' @param control An `EpiModel` object of class [control.icm()].
#'
#' @inherit recovery.icm return
#'
#' @export
#' @keywords internal
#'
initialize.icm <- function(param, init, control) {

  ## Main List for Data ##
  dat <- list()
  dat$param <- param
  dat$init <- init
  dat$control <- control
  class(dat) <- c("icm_dat", class(dat))


  # Set attributes
  dat$attr <- list()
  numeric.init <- init[which(sapply(init, class) == "numeric")]
  n <- do.call("sum", numeric.init)
  dat$attr$active <- rep(1, n)
  if (dat$param$groups == 1) {
    dat$attr$group <- rep(1, n)
  } else {
    g2inits <- grep(".g2", names(numeric.init))
    g1inits <- setdiff(seq_along(numeric.init), g2inits)
    nG1 <- sum(sapply(g1inits, function(x) init[[x]]))
    nG2 <- sum(sapply(g2inits, function(x) init[[x]]))
    dat$attr$group <- c(rep(1, nG1), rep(2, max(0, nG2)))
  }

  # Initialize status and infection time
  dat <- init_status.icm(dat)


  # Summary out list
  dat <- do.call(control[["prevalence.FUN"]], list(dat, at = 1))

  return(dat)
}


#' @title Disease Status Initialization Module for icm
#'
#' @description This function sets the initial disease status on the
#'              network given the specified initial conditions.
#'
#' @inheritParams prevalence.icm
#'
#' @inherit recovery.icm return
#'
#' @seealso This is an initialization module for [icm()].
#'
#' @export
#' @keywords internal
#'
init_status.icm <- function(dat) {

  # Variables ---------------------------------------------------------------
  type <- dat$control$type
  group <- dat$attr$group
  nGroups <- dat$param$groups

  nG1 <- sum(group == 1)
  nG2 <- sum(group == 2)

  i.num <- dat$init$i.num
  r.num <- dat$init$r.num
  i.num.g2 <- dat$init$i.num.g2
  r.num.g2 <- dat$init$r.num.g2


  # Status ------------------------------------------------------------------
  status <- rep("s", nG1 + nG2)
  status[sample(which(group == 1), size = i.num)] <- "i"
  if (nGroups == 2) {
    status[sample(which(group == 2), size = i.num.g2)] <- "i"
  }
  if (type == "SIR") {
    status[sample(which(group == 1 & status == "s"), size = r.num)] <- "r"
    if (nGroups == 2) {
      status[sample(which(group == 2 & status == "s"), size = r.num.g2)] <- "r"
    }
  }
  dat$attr$status <- status


  # Infection Time ----------------------------------------------------------
  idsInf <- which(status == "i")
  infTime <- rep(NA, length(status))

  # If vital=TRUE, infTime is a uniform draw over the duration of infection
  if (dat$param$vital == TRUE && dat$param$di.rate > 0) {
    infTime[idsInf] <- -rgeom(n = length(idsInf), prob = dat$param$di.rate) + 2
  } else {
    if (dat$control$type == "SI" || dat$param$rec.rate == 0) {
      # infTime a uniform draw over the number of sim time steps
      infTime[idsInf] <- ssample(1:(-dat$control$nsteps + 2),
                                 length(idsInf), replace = TRUE)
    } else {
      if (nGroups == 1) {
        infTime[idsInf] <- ssample(1:(-round(1 / dat$param$rec.rate) + 2),
                                   length(idsInf), replace = TRUE)
      }
      if (nGroups == 2) {
        infG1 <- which(status == "i" & group == 1)
        infTime[infG1] <- ssample(1:(-round(1 / dat$param$rec.rate) + 2),
                                  length(infG1), replace = TRUE)
        infG2 <- which(status == "i" & group == 2)
        infTime[infG2] <- ssample(1:(-round(1 / dat$param$rec.rate.g2) + 2),
                                  length(infG2), replace = TRUE)
      }
    }
  }
  dat$attr$infTime <- infTime

  return(dat)
}

## Helper utilities ------------------------------------------------------------
draw_qnts <- function(x, y, qnts, qnts.pal, qnts.smooth,
                      loc = "epi", plot.qnts = 1, qnts.min_max = "max",
                      offset = 0) {

  qnt.min <- 1E10
  qnt.max <- -1E10

  lcomp <- length(y)
  for (j in seq_len(lcomp)) {
    quants <- c((1 - qnts) / 2, 1 - ((1 - qnts) / 2))
    qnt.prev <- apply(
      x[[loc]][[y[j]]], 1,
      function(x) quantile(x, c(quants[1], quants[2]), na.rm = TRUE)
    )
    complete_rows <- complete.cases(t(qnt.prev))
    x_cords <- seq_len(ncol(qnt.prev))[complete_rows] + offset
    xx <- c(x_cords, rev(x_cords))
    qnt.prev <- qnt.prev[, complete_rows]
    if (qnts.smooth == FALSE) {
      yy <- c(qnt.prev[1, ], rev(qnt.prev[2, ]))
    } else {
      yy <- c(
        suppressWarnings(supsmu(x = seq_len(ncol(qnt.prev)),
                                y = qnt.prev[1, ]))$y,
        rev(suppressWarnings(supsmu(x = seq_len(ncol(qnt.prev)),
                                    y = qnt.prev[2, ]))$y)
      )
    }
    if (plot.qnts == 1) {
      polygon(xx, yy, col = qnts.pal[j], border = NA)
    } else {
      qnt.max[j] <-  max(yy)
      qnt.min[j] <-  min(yy)
    }
  }
  if (plot.qnts == 0 && qnts.min_max == "max") {
    return(max(qnt.max))
  } else if (plot.qnts == 0 && qnts.min_max == "min") {
    return(min(qnt.min))
  }
}


draw_means <- function(x, y, mean.smooth, mean.lwd,
                       mean.pal, mean.lty, loc = "epi",
                       plot.means = 1, mean.min_max = "max",
                       offset = 0) {

  mean.min <- 1E10
  mean.max <- -1E10

  lcomp <- length(y)

  for (j in seq_len(lcomp)) {
    mean.prev <- rowMeans(x[[loc]][[y[j]]], na.rm = TRUE)
    xs <- seq_along(mean.prev) + offset
    if (mean.smooth == TRUE) {
      smoother <- suppressWarnings(supsmu(x = xs, y = mean.prev))
      mean.prev <- smoother$y
      xs <- smoother$x
    }
    if (plot.means == 1) {
      lines(x = xs, y = mean.prev, lwd = mean.lwd[j],
            col = mean.pal[j], lty = mean.lty[j])
    } else {
      mean.max[j] <-  max(mean.prev, na.rm = TRUE)
      mean.min[j] <-  min(mean.prev, na.rm = TRUE)
    }
  }
  if (plot.means == 0 && mean.min_max == "max") {
    return(max(mean.max))
  } else if (plot.means == 0 && mean.min_max == "min") {
    return(min(mean.min))
  }
}

get_qnts <- function(data, qnts, qnts.smooth) {
  if (qnts < 0 || qnts > 1) {
    stop("qnts must be between 0 and 1")
  }
  quants <- c((1 - qnts) / 2, 1 - ((1 - qnts) / 2))
  qnt.prev <- apply(data, 1, function(x) {
    quantile(x, c(quants[1], quants[2]), na.rm = TRUE)
  })
  if (isFALSE(qnts.smooth)) {
    xx <- c(seq_len(ncol(qnt.prev)), rev(seq_len(ncol(qnt.prev))))
    yy <- c(qnt.prev[1, ], rev(qnt.prev[2, ]))
    xx <- xx[!is.na(yy)]
    yy <- yy[!is.na(yy)]
  } else {
    ss1 <- suppressWarnings(supsmu(x = seq_len(ncol(qnt.prev)),
                                   y = qnt.prev[1, ]))
    ss2 <- suppressWarnings(supsmu(x = seq_len(ncol(qnt.prev)),
                                   y = qnt.prev[2, ]))

    xx <- c(ss1$x, rev(ss2$x))
    yy <- c(ss1$y, rev(ss2$y))
  }
  list(x = xx, y = yy)
}

get_means <- function(data, mean.smooth) {
  mean.prev <- rowMeans(data, na.rm = TRUE)
  if (isFALSE(mean.smooth)) {
    xx <- seq_along(mean.prev)
    yy <- mean.prev
    xx <- xx[!is.na(yy)]
    yy <- yy[!is.na(yy)]
  } else {
    ss <- suppressWarnings(
      supsmu(x = seq_along(mean.prev), y = mean.prev)
    )
    xx <- ss$x
    yy <- ss$y
  }
  list(x = xx, y = yy)
}

# Helper Functions --------------------------------------------------------


# Calculate denominators
denom <- function(x, y, popfrac) {

  cont.val <- if (inherits(x, "dcm")) "nruns" else "nsims"
  if (popfrac) {
    for (i in seq_along(y)) {
      dname <- paste(strsplit(y[i], "[.]")[[1]][-1], collapse = ".")
      x$epi[[y[i]]] <- x$epi[[y[i]]] / x$epi[[dname]]
    }
  }
  if (!popfrac && x$control[[cont.val]] == 1) {
    for (j in seq_along(y)) {
      x$epi[[y[j]]] <- data.frame(x$epi[[y[j]]])
    }
  }

  return(x)
}

# Helper function to convert a `data.frame` obtained with
# `as.data.frame(netsim)` back into an `epi` list format.
df2epi <- function(x) {
  n_steps <- max(x$time) - min(x$time) + 1L
  sims <- unique(x$sim)
  n_sims <- length(sims)
  colnames <- paste0("sim", sims)
  epi_names <- setdiff(names(x), c("time", "sim"))
  sapply(
    epi_names,
    \(nme) {
      t <- as.data.frame(matrix(x[[nme]], nrow = n_steps, ncol = n_sims))
      names(t) <- colnames
      t
    },
    USE.NAMES = TRUE,
    simplify = FALSE
  )
}

#' @title Plot Epidemic Model Results From a Netsim Data.Frame
#'
#' @description This function is a wrapper around `plot.netsim` accepting a
#'   `data.frame` obtain with `as.data.frame(netsim_object)`.
#'
#' @param x A `data.frame` obtain with `as.data.frame(netsim_object)`.
#' @inheritParams plot.netsim
#'
#' @method plot epi.data.frame
#' @export
#'
#' @keywords plot
plot.epi.data.frame <- function(x, y = NULL, sims = NULL, legend = NULL,
                                mean.col = NULL, qnts.col = NULL, sim.lwd = NULL,
                                sim.col = NULL, sim.alpha = NULL, popfrac = FALSE,
                                qnts = 0.5, qnts.alpha = 0.5, qnts.smooth = TRUE,
                                mean.line = TRUE, mean.smooth = TRUE, add = FALSE,
                                mean.lwd = 2, mean.lty = 1, xlim = NULL,
                                ylim = NULL, main = NULL, xlab = NULL, ylab = NULL,
                                sim.lines = FALSE, grid = FALSE, leg.cex = 0.8,
                                ...) {
  ntemp <- list(
    epi = df2epi(x),
    control = list(
      nsteps = max(x$time) - min(x$time) + 1L,
      start = min(x$time),
      nsims = max(x$sim)
    )
  )
  plot_netsim_epi(
    ntemp, y, sims, legend, mean.col, qnts.col, sim.lwd, sim.col, sim.alpha,
    popfrac, qnts, qnts.alpha, qnts.smooth, mean.line, mean.smooth, add,
    mean.lwd, mean.lty, xlim, ylim, main, xlab, ylab, sim.lines, grid,
    leg.cex, ...
  )
}

#' @title Create a Minimal netsim_dat Main List Object for a Network Model
#'
#' @description This helper function populates a `netsim_dat` main list
#'              object with the minimal required elements. All parameters are
#'              optional. When none are given the resulting object is only a
#'              shell list of class `netsim_dat` with the different named
#'              elements defined as empty lists.
#'
#' @param param An `EpiModel` object of class [param.net()].
#' @param init An `EpiModel` object of class [init.net()].
#' @param control An `EpiModel` object of class [control.net()].
#' @param run A `list` that will contains the objects created by
#' [netsim()] that are required for between step communication. This
#' list must be preserved for restarting models.
#'
#' @return A `netsim_dat` main list object.
#' @export
create_dat_object <- function(param = list(), init = list(), control = list(),
                              run = list()) {
  dat <- list(
    "run"       = validate_run(run),
    "param"     = param,
    "control"   = control,
    "epi"       = list(),
    "temp"      = list(),
    "stats"     = list(),
    "init"      = init
  )

  class(dat) <- c("netsim_dat", class(dat))

  return(dat)
}

#' Ensures that the `run` sublist contains all the mandatory elements
#'
#' @param run A `run` sublist to validate
#' @return A valid `run` sublist
#' @noRd
validate_run <- function(run) {
  defaults <- list(
    attr = list(),
    current_timestep = 1L,
    last_unique_id = 0L
  )

  for (elt_name in names(defaults)) {
    if (is.null(run[[elt_name]])) {
      run[[elt_name]] <- defaults[[elt_name]]
    }
  }

  return(run)
}

#' @title Return the Current Timestep
#'
#' @inheritParams recovery.net
#'
#' @return The current timestep.
#' @export
get_current_timestep <- function(dat) {
  return(as.integer(dat$run$current_timestep))
}

#' @title Set the Current Timestep
#'
#' @description Changes the current timestep in the `netsim_dat` object.
#'              Use with caution. This function exists to work around unforeseen
#'              corner cases. In most situation, `increment_timestep` is
#'              preferred.
#'
#' @inheritParams recovery.net
#' @param timestep The new value for the timestep.
#'
#' @inherit recovery.net return
#'
#' @section Mutability:
#' This DOES NOT modify the `netsim_dat` object in place. The result must
#' be assigned back to `dat` in order to be registered:
#' `dat <- increment_timestep(dat)`.
#'
#' @export
set_current_timestep <- function(dat, timestep) {
  dat$run$current_timestep <- as.integer(timestep)
  return(dat)
}

#' @title Increment the Current Timestep
#'
#' @description This function adds 1 to the timestep counter stored in the
#'              `netsim_dat` main list object.
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @section Mutability:
#' This DOES NOT modify the `netsim_dat` object in place. The result must
#' be assigned back to `dat` in order to be registered:
#' `dat <- increment_timestep(dat)`.
#'
#' @export
increment_timestep <- function(dat) {
  dat$run$current_timestep <- dat$run$current_timestep + 1L
  return(dat)
}

#' Mathematical Modeling of Infectious Disease Dynamics
#'
#' \tabular{ll}{
#'    Package: \tab EpiModel\cr
#'    Type: \tab Package\cr
#'    Version: \tab 2.6.1\cr
#'    Date: \tab 2026-05-12\cr
#'    License: \tab GPL-3\cr
#'    LazyLoad: \tab yes\cr
#' }
#'
#' @details
#' The EpiModel software package provides tools for building, solving, and
#' visualizing mathematical models of infectious disease dynamics. These tools
#' allow users to simulate epidemic models in multiple frameworks for both
#' pedagogical purposes ("base models") and novel research purposes
#' ("extension models").
#'
#' @section Model Classes and Infectious Disease Types:
#' EpiModel provides functionality for three classes of epidemic models:
#'
#'  * **Deterministic Compartmental Models:** these continuous-time
#'        models are solved using ordinary differential equations. EpiModel
#'        allows for easy specification of sensitivity analyses to compare
#'        multiple scenarios of the same model across different parameter
#'        values.
#'  * **Stochastic Individual Contact Models:** a novel class of
#'        individual-based, microsimulation models that were developed to add
#'        random variation in all components of the transmission system, from
#'        infection to recovery to vital dynamics (arrivals and departures).
#'  * **Stochastic Network Models:** with the underlying statistical
#'        framework of temporal exponential random graph models (ERGMs) recently
#'        developed in the **Statnet** suite of software in R, network
#'        models over epidemics simulate edge (e.g., partnership) formation and
#'        dissolution stochastically according to a specified statistical model,
#'        with disease spread across that network.
#'
#'
#' EpiModel supports three infectious disease types to be run across all of the
#' three classes.
#'
#'   * **Susceptible-Infectious (SI):** a two-state disease in which
#'         there is life-long infection without recovery. HIV/AIDS is one
#'         example, although for this case it is common to model infection
#'         stages as separate compartments.
#'   * **Susceptible-Infectious-Recovered (SIR):** a three-stage
#'         disease in which one has life-long recovery with immunity after
#'         infection. Measles is one example, but modern models for the disease
#'         also require consideration of vaccination patterns in the population.
#'   * **Susceptible-Infectious-Susceptible (SIS):** a two-stage
#'         disease in which one may transition back and forth from the
#'         susceptible to infected states throughout life. Examples include
#'         bacterial sexually transmitted diseases like gonorrhea.
#'
#' These basic disease types may be extended in any arbitrarily complex way to
#' simulate specific diseases for research questions.
#'
#' @section Model Parameterization and Simulation:
#' EpiModel uses three model setup functions for each model class to input the
#' necessary parameters, initial conditions, and control settings:
#'
#'  * [param.dcm()], [param.icm()], and
#'        [param.net()] are used to input epidemic parameters for each
#'        of the three model classes. Parameters include the rate of contacts or
#'        acts between actors, the probability of transmission per contact, and
#'        recovery and demographic rates for models that include those
#'        transitions.
#'  * [init.dcm()], [init.icm()], and
#'        [init.net()] are used to input the initial conditions for
#'        each class. The main conditions are limited to the numbers or, if
#'        applicable, the specific agents in the population who are infected or
#'        recovered at the simulation outset.
#'  * [control.dcm()], [control.icm()], and
#'        [control.net()] are used to specify the remaining control
#'        settings for each simulation. The core controls for base model
#'        types include the disease type, number of time steps, and number of
#'        simulations. Controls are also used to input new model functions (for
#'        DCMs) and new model modules (for ICMs and network models) to allow the
#'        user to simulate fully original epidemic models in EpiModel. See the
#'        documentation for the specific control functions help pages.
#'
#'
#' With the models parameterized, the functions for simulating epidemic models
#' are:
#'
#' * [dcm()] for deterministic compartmental models.
#' * [icm()] for individual contact models.
#' * Network models are simulated in a three-step process:
#'    1. [netest()] estimates the statistical model for the network
#'       structure itself (i.e., how partnerships form and dissolve over time
#'       given the parameterization of those processes). This function is a
#'       wrapper around the `ergm` and `tergm` functions in the
#'       `ergm` and `tergm` packages. The current statistical
#'       framework for model simulation is called "egocentric inference":
#'       target statistics summarizing these formation and dissolution
#'       processes collected from an egocentric sample of the population.
#'    2. [netdx()] runs diagnostics on the dynamic model fit by
#'       simulating the base network over time to ensure the model fits the
#'       targets for formation and dissolution.
#'    3. [netsim()] simulates the stochastic network epidemic
#'       models, with a given network model fit in [netest()]. Here
#'       the function requires this model fit object along with the
#'       parameters, initial conditions, and control settings as defined
#'       above.
#'
#'
#'
#' @references
#' The EpiModel website is at <https://www.epimodel.org/>, and the source
#' code is at <https://github.com/EpiModel/EpiModel>. Bug reports and
#' feature requests are welcome.
#'
#' Our primary methods paper on EpiModel is published in the **Journal of
#' Statistical Software**. If you use EpiModel for any research or teaching
#' purposes, please cite this reference:
#'
#' Jenness SM, Goodreau SM, and Morris M. EpiModel: An R Package for
#' Mathematical Modeling of Infectious Disease over Networks. Journal of
#' Statistical Software. 2018; 84(8): 1-47. \doi{10.18637/jss.v084.i08}.
#'
#' We have also developed two extension packages for modeling specific disease
#' dynamics. For HIV and bacterial sexually transmitted infections, we have
#' developed `EpiModelHIV`, which is available on Github at
#' <https://github.com/EpiModel/EpiModelHIV>. For COVID-19, we have
#' developed `EpiModelCOVID`, which is available at
#' <https://github.com/EpiModel/EpiModelCOVID>.
#'
#' @name EpiModel-package
#' @aliases EpiModel
#' @import ergm network networkLite networkDynamic tergm ggplot2
#' @importFrom deSolve dede ode
#' @importFrom RColorBrewer brewer.pal brewer.pal.info
#' @importFrom graphics abline arrows boxplot legend lines mtext par plot points polygon text title
#' @importFrom grDevices col2rgb colorRampPalette rgb adjustcolor
#' @importFrom stats complete.cases quantile rbinom rgeom sd setNames simulate supsmu terms.formula update dgeom coef na.omit
#' @importFrom utils head tail packageVersion
#' @importFrom ape as.phylo collapse.singles
#' @importFrom lazyeval lazy_dots lazy_eval
#' @importFrom statnet.common trim_env set.control.class check.control.class NVL paste.and .Deprecate_once
#' @importFrom methods is
#' @importFrom tibble as_tibble tibble
#' @importFrom coda effectiveSize
#' @importFrom rlang .data
#'
#' @useDynLib EpiModel, .registration = TRUE
#'
#' @keywords package
#'
"_PACKAGE"

#' @title Dynamic Network Model Estimation
#'
#' @description Estimates statistical network models using the exponential
#'              random graph modeling (ERGM) framework with extensions for
#'              dynamic/temporal models (STERGM). This is typically the first
#'              step in the network modeling pipeline, followed by [`netdx`]
#'              for model diagnostics and [`netsim`] for epidemic simulation.
#'
#' @param nw An object of class `network` or `egor`, with the latter
#'        indicating an `ergm.ego` fit.
#' @param formation Right-hand sided STERGM formation formula in the form
#'        `~edges + ...`, where `...` are additional network statistics
#'        (ERGM terms). This formula specifies which structural features of
#'        the network should be reproduced by the model. Common terms include
#'        `edges` (overall connectivity), `nodematch` (homophily by attribute),
#'        `concurrent` (overlapping partnerships), and `degree` (degree
#'        distribution constraints). See [`ergm::ergm-terms`] for the full
#'        list of available terms.
#' @param target.stats Vector of target statistics for the formation model, with
#'        one number for each network statistic in the model. These are the
#'        observed (or desired) values for each term in the formation formula.
#'        For example, if `formation = ~edges + concurrent`, then
#'        `target.stats = c(175, 40)` means the model should produce
#'        approximately 175 edges and 40 nodes with 2 or more partners. For
#'        an `edges`-only model, a useful starting value is
#'        `mean_degree * network_size / 2`. Ignored if fitting via `ergm.ego`.
#' @param coef.diss An object of class `disscoef` output from the
#'        [`dissolution_coefs`] function. This encodes the average partnership
#'        duration(s) and the corresponding dissolution model coefficients.
#'        For models with vital dynamics (arrivals and departures), the
#'        `d.rate` argument in [`dissolution_coefs`] should be set to adjust
#'        for the competing risk of node departure.
#' @param constraints Right-hand sided formula specifying constraints for the
#'        modeled network, in the form `~...`, where `...` are
#'        constraint terms. By default, no constraints are set.
#' @param coef.form Vector of coefficients for the offset terms in the formation
#'        formula.
#' @param edapprox If `TRUE`, use the indirect edges dissolution
#'        approximation method for the dynamic model fit, otherwise use the
#'        more time-intensive full STERGM estimation (see details). The
#'        approximation is recommended for most use cases, especially when
#'        average partnership durations are moderate to long (> 25 time steps).
#'        Direct STERGM estimation (`edapprox = FALSE`) is slower but may be
#'        preferred for very short durations or when inferential quantities
#'        (standard errors, p-values) on formation coefficients are needed.
#'        For `nw` of class `egor`, only `edapprox = TRUE` is supported.
#' @param set.control.ergm Control arguments passed to `ergm` (see details).
#' @param set.control.ergm.ego Control arguments passed to `ergm.ego` (see
#'        details).
#' @param set.control.tergm Control arguments passed to `tergm` (see details).
#' @param verbose If `TRUE`, print model fitting progress to console.
#' @param nested.edapprox Logical. If `edapprox = TRUE` the dissolution
#'        model is an initial segment of the formation model (see details).
#' @param ... Additional arguments passed to other functions.
#'
#' @details
#' `netest` is a wrapper function for the `ergm`, `ergm.ego`, and `tergm`
#' functions that estimate static and dynamic network models. Network model
#' estimation is the first step in simulating a stochastic network epidemic model
#' in `EpiModel`. The output from `netest` is a necessary input for running the
#' epidemic simulations in [`netsim`]. With a fitted network model, one should
#' always first proceed to model diagnostics, available through the [`netdx`]
#' function, to check model fit. A detailed description of fitting these
#' models, along with examples, may be found in the
#' [Network Modeling for Epidemics](https://epimodel.github.io/sismid/)
#' tutorials.
#'
#' @section Edges Dissolution Approximation:
#' The edges dissolution approximation method is described in Carnegie et al.
#' This approximation requires that the dissolution coefficients are known, that
#' the formation model is being fit to cross-sectional data conditional on those
#' dissolution coefficients, and that the terms in the dissolution model are a
#' subset of those in the formation model. Under certain additional conditions,
#' the formation coefficients of a STERGM model are approximately equal to the
#' coefficients of that same model fit to the observed cross-sectional data as
#' an ERGM, minus the corresponding coefficients in the dissolution model. The
#' approximation thus estimates this ERGM (which is typically much faster than
#' estimating a STERGM) and subtracts the dissolution coefficients.
#'
#' The conditions under which this approximation best hold are when there are
#' few relational changes from one time step to another; i.e. when either
#' average relational durations are long, or density is low, or both.
#' Conveniently, these are the same conditions under which STERGM estimation is
#' slowest. Note that the same approximation is also used to obtain starting
#' values for the STERGM estimate when the latter is being conducted. The
#' estimation does not allow for calculation of standard errors, p-values, or
#' likelihood for the formation model; thus, this approach is of most use when
#' the main goal of estimation is to drive dynamic network simulations rather
#' than to conduct inference on the formation model. The user is strongly
#' encouraged to examine the behavior of the resulting simulations to confirm
#' that the approximation is adequate for their purposes. For an example, see
#' the vignette for the package `tergm`.
#'
#' It has recently been found that subtracting a modified version of the
#' dissolution coefficients from the formation coefficients provides a more
#' principled approximation, and this is now the form of the approximation
#' applied by `netest`. The modified values subtracted from the formation
#' coefficients are equivalent to the (crude) dissolution coefficients with
#' their target durations increased by 1. The `nested.edapprox` argument
#' toggles whether to implement this modified version by appending the
#' dissolution terms to the formation model and appending the relevant values to
#' the vector of formation model coefficients (value = `FALSE`), whereas
#' the standard version subtracts the relevant values from the initial formation
#' model coefficients (value = `TRUE`).
#'
#' @section Control Arguments:
#' The `ergm`, `ergm.ego`, and `tergm` functions allow control settings for the
#' model fitting process. When fitting a STERGM directly (setting
#' `edapprox` to `FALSE`), control parameters may be passed to the
#' `tergm` function with the `set.control.tergm` argument in `netest`.
#' The controls should be input through the `control.tergm()` function,
#' with the available parameters listed in the [`tergm::control.tergm`] help
#' page in the `tergm` package.
#'
#' When fitting a STERGM indirectly (setting `edapprox` to `TRUE`), control
#' settings may be passed to the `ergm` function using `set.control.ergm`,
#' or to the `ergm.ego` function using `set.control.ergm.ego`. The controls
#' should be input through the `control.ergm()` and `control.ergm.ego()`
#' functions, respectively, with the available parameters listed in the
#' [`ergm::control.ergm`] help page in the `ergm` package and the
#' [`ergm.ego::control.ergm.ego`] help page in the `ergm.ego`
#' package. An example is below.
#'
#' @section Typical Workflow:
#' The network modeling pipeline in EpiModel typically follows these steps:
#'
#'  1. Initialize a network: [`network_initialize`]
#'  2. Specify formation and dissolution: a formation formula (e.g.,
#'     `~edges + concurrent`) with target statistics, and dissolution
#'     coefficients via [`dissolution_coefs`]
#'  3. Estimate the network model: `netest()`
#'  4. Diagnose model fit: [`netdx`]
#'  5. Simulate the epidemic: [`netsim`] with [`param.net`], [`init.net`],
#'     and [`control.net`]
#'
#' @return A fitted network model object of class `netest`. This object is
#' passed to [`netdx`] for diagnostics and to [`netsim`] for epidemic
#' simulation. Use `print()` to view the model form, including the
#' formation formula, target statistics, and dissolution model. Use
#' `summary()` to view the estimated model coefficients and goodness-of-fit
#' statistics from the underlying ERGM or STERGM fit.
#'
#' @references
#' Krivitsky PN, Handcock MS. "A Separable Model for Dynamic Networks." JRSS(B).
#' 2014; 76.1: 29-46.
#'
#' Carnegie NB, Krivitsky PN, Hunter DR, Goodreau SM. An Approximation Method
#' for Improving Dynamic Network Model Fitting. Journal of Computational and
#' Graphical Statistics. 2014; 24(2): 502-519.
#'
#' Jenness SM, Goodreau SM and Morris M. EpiModel: An R Package for Mathematical
#' Modeling of Infectious Disease over Networks. Journal of Statistical
#' Software. 2018; 84(8): 1-47.
#'
#' @keywords model
#' @seealso Use [`dissolution_coefs`] to compute dissolution coefficients
#'          before estimation. Use [`netdx`] to diagnose the fitted network
#'          model, and [`netsim`] to simulate epidemic spread over a simulated
#'          dynamic network consistent with the model fit. Parameterize the
#'          epidemic simulation with [`param.net`], [`init.net`], and
#'          [`control.net`].
#'
#' @export
#'
#' @examples
#' # Initialize a network of 100 nodes
#' nw <- network_initialize(n = 100)
#'
#' # Set formation formula
#' formation <- ~edges + concurrent
#'
#' # Set target statistics for formation
#' target.stats <- c(50, 25)
#'
#' # Obtain the offset coefficients
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)
#'
#' # Estimate the STERGM using the edges dissolution approximation
#' est <- netest(nw, formation, target.stats, coef.diss,
#'               set.control.ergm = control.ergm(MCMC.burnin = 1e5,
#'                                               MCMC.interval = 1000))
#' # View the model form (formation, targets, dissolution)
#' est
#'
#' # View the estimated coefficients
#' summary(est)
#'
#' \dontrun{
#' # Model with homophily on a nodal attribute
#' nw2 <- network_initialize(n = 500)
#' nw2 <- set_vertex_attribute(nw2, "risk", rep(0:1, each = 250))
#' formation2 <- ~edges + nodematch("risk")
#' target.stats2 <- c(175, 110)
#' coef.diss2 <- dissolution_coefs(dissolution = ~offset(edges), duration = 50)
#' est2 <- netest(nw2, formation2, target.stats2, coef.diss2)
#' est2
#'
#' # Direct STERGM estimation (slower, for short durations or inference)
#' est3 <- netest(nw, formation, target.stats, coef.diss, edapprox = FALSE)
#' }
#'
netest <- function(nw, formation, target.stats, coef.diss, constraints = NULL,
                   coef.form = NULL, edapprox = TRUE,
                   set.control.ergm = control.ergm(),
                   set.control.tergm = control.tergm(MCMC.maxchanges = .Machine$integer.max),
                   set.control.ergm.ego = NULL,
                   verbose = FALSE, nested.edapprox = TRUE, ...) {

  if (is.null(constraints)) {
    constraints	<- trim_env(~.)
  }

  if (!inherits(coef.diss, "disscoef")) {
    stop("dissolution must be input through the dissolution_coefs function")
  }
  dissolution <- coef.diss$dissolution
  if (coef.diss$duration[1] == 1) {
    is.tergm <- FALSE
  } else {
    is.tergm <- TRUE
  }

  if (is.tergm == TRUE && nested.edapprox == TRUE) {
    diss_check(formation, dissolution)
  }

  if (edapprox == FALSE) {

    fit <- tergm(~Form(formation) + Persist(dissolution),
                 basis = nw,
                 targets = "formation",
                 target.stats = target.stats,
                 offset.coef = c(coef.form, coef.diss$coef.crude),
                 constraints = constraints,
                 estimate = "EGMME",
                 eval.loglik = FALSE,
                 control = set.control.tergm,
                 verbose = verbose)

    coef.form <- fit # there is no longer a separate formation fit
    which_form <- which(grepl("^Form~", names(coef(fit))) |
                          grepl("^offset\\(Form~", names(coef(fit))))
    form_names <- names(coef(fit)[which_form])[!fit$offset[which_form]]

    out <- list()
    out$formation <- formation
    out$target.stats <- target.stats
    out$target.stats.names <-
      substr(form_names, 6, nchar(form_names))
    out$coef.form <- coef(coef.form)[which_form]
    out$dissolution <- dissolution
    out$coef.diss <- coef.diss
    out$constraints <- constraints
    out$edapprox <- edapprox
    # convert ergm_state to network
    out$newnetwork <- as.network(fit$newnetwork)
    delete.network.attribute(out$newnetwork, "time")
    delete.network.attribute(out$newnetwork, "lasttoggle")
    out$formula <- trim_env(fit$formula, keep = c("formation", "dissolution"))

  } else {

    if (inherits(nw, "egor")) {
      # ergm.ego case
      if (system.file(package = "ergm.ego") == "")
        stop("The `ergm.ego` package is required to estimate from `egor` objects.\n ")
      if (is.null(set.control.ergm.ego))
        set.control.ergm.ego <- ergm.ego::control.ergm.ego()

      fit <- ergm.ego::ergm.ego(
        formation,
        basis = nw,
        popsize = 0,
        constraints = constraints,
        offset.coef = coef.form,
        control = set.control.ergm.ego,
        verbose = verbose
      )
      target.stats <- fit$m

    } else {
      # ergm case
      fit <- ergm(formation,
                  basis = nw,
                  target.stats = target.stats,
                  constraints = constraints,
                  offset.coef = coef.form,
                  eval.loglik = FALSE,
                  control = set.control.ergm,
                  verbose = verbose)
    }

    coef.form <- coef(fit)
    coef.form.crude <- coef.form
    if (is.tergm == TRUE) {
      if (nested.edapprox == TRUE) {
        l.cfc <- length(coef.diss$coef.form.corr)
        coef.form[1:l.cfc] <- coef.form[1:l.cfc] - coef.diss$coef.form.corr
      } else {
        ## implement the edapprox by appending the dissolution model to the
        ## formation model and appending the relevant values to the vector of
        ## formation model coefficients
        formation <- trim_env(~Passthrough(formation) + Passthrough(dissolution),
                              keep = c("formation", "dissolution"))
        coef.form <- c(coef.form, -coef.diss$coef.form.corr)
      }
    }

    out <- list()
    out$formation <- formation
    out$target.stats <- target.stats
    ## subselect coef names for targeted statistics, including extremal targets
    out$target.stats.names <- names(coef(fit))[!fit$offset | (fit$drop != 0)]
    out$coef.form <- coef.form
    out$coef.form.crude <- coef.form.crude
    out$coef.diss <- coef.diss
    out$constraints <- constraints
    out$edapprox <- edapprox
    out$nested.edapprox <- nested.edapprox
    out$newnetwork <- NVL(fit$newnetwork, fit$network)
    out$formula <- fit$formula
  }

  out$summary <- summary(fit, ...)
  out$fit <- fit

  class(out) <- "netest"
  return(out)
}


diss_check <- function(formation, dissolution) {

  # Split formulas into separate terms
  form.terms <- strsplit(as.character(formation)[2], "[+]")[[1]]
  diss.terms <- strsplit(as.character(dissolution)[2], "[+]")[[1]]

  # Remove whitespace
  form.terms <- gsub("\\s", "", form.terms)
  diss.terms <- gsub("\\s", "", diss.terms)

  offpos.f <- grep("offset(", form.terms, fixed = TRUE)
  form.terms[offpos.f] <- substr(form.terms[offpos.f], nchar("offset(") + 1,
                                 nchar(form.terms[offpos.f]) - 1)
  offpos.d <- grep("offset(", diss.terms, fixed = TRUE)
  diss.terms[offpos.d] <- substr(diss.terms[offpos.d], nchar("offset(") + 1,
                                 nchar(diss.terms[offpos.d]) - 1)

  argpos.f <- regexpr("\\(", form.terms)
  argpos.d <- regexpr("\\(", diss.terms)

  # Matrix with terms in row 1, args in row 2
  form.terms <- vapply(regmatches(form.terms, argpos.f, invert = TRUE),
                       function(x) {
                         if (length(x) < 2) {
                           x <- c(x, "")
                         } else {
                           x[2] <- substr(x[2], 1, nchar(x[2]) - 1)
                         }
                         x
                       },
                       c(term = "", args = ""))
  diss.terms <- vapply(regmatches(diss.terms, argpos.d, invert = TRUE),
                       function(x) {
                         if (length(x) < 2) {
                           x <- c(x, "")
                         } else {
                           x[2] <- substr(x[2], 1, nchar(x[2]) - 1)
                         }
                         x
                       },
                       c(term = "", args = ""))

  matchpos <- match(diss.terms[1, ], form.terms[1, ])

  if (any(is.na(matchpos))) {
    stop("Dissolution model is not a subset of formation model.")
  }
  if (!all(diss.terms[1, ] %in% c("edges", "nodemix",
                                  "nodematch"))) {
    stop("The only allowed dissolution terms are edges, nodemix,
         and nodematch")
  }
  if (any(matchpos != seq_len(ncol(diss.terms)))) {
    stop("Order of terms in the dissolution model does not correspond to the ",
         "formation model.")
  }
  if (any(diss.terms[2, ] != form.terms[2, seq_len(ncol(diss.terms))])) {
    stop("Term options for one or more terms in dissolution model do not ",
         "match the options in the formation model.")
  }

}


#' @title Adjust Dissolution Component of Network Model Fit
#'
#' @description Adjusts the dissolution component of a dynamic ERGM fit using
#'              the [netest()] function with the edges dissolution
#'              approximation method.
#'
#' @param old.netest An object of class `netest`, from the
#'        [netest()] function.
#' @param new.coef.diss An object of class `disscoef`, from the
#'        [dissolution_coefs()] function.
#' @param nested.edapprox Logical. If `edapprox = TRUE` the dissolution
#'        model is an initial segment of the formation model (see details in
#'        [netest()]).
#'
#' @details
#' Fitting an ERGM is a computationally intensive process when the model
#' includes dyad dependent terms. With the edges dissolution approximation
#' method of Carnegie et al, the coefficients for a temporal ERGM are
#' approximated by fitting a static ERGM and adjusting the formation
#' coefficients to account for edge dissolution. This function provides a very
#' efficient method to adjust the coefficients of that model when one wants to
#' use a different dissolution model; a typical use case may be to fit several
#' different models with different average edge durations as targets. The
#' example below exhibits that case.
#'
#' @return An updated network model object of class `netest`.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' nw <- network_initialize(n = 1000)
#'
#' # Two dissolutions: an average duration of 300 versus 200
#' diss.300 <- dissolution_coefs(~offset(edges), 300, 0.001)
#' diss.200 <- dissolution_coefs(~offset(edges), 200, 0.001)
#'
#' # Fit the two reference models
#' est300 <- netest(nw = nw,
#'                 formation = ~edges,
#'                 target.stats = c(500),
#'                 coef.diss = diss.300)
#'
#' est200 <- netest(nw = nw,
#'                 formation = ~edges,
#'                 target.stats = c(500),
#'                 coef.diss = diss.200)
#'
#' # Alternatively, update the 300 model with the 200 coefficients
#' est200.compare <- update_dissolution(est300, diss.200)
#'
#' identical(est200$coef.form, est200.compare$coef.form)
#'}
#'
update_dissolution <- function(old.netest, new.coef.diss,
                               nested.edapprox = TRUE) {

  if (!inherits(old.netest, "netest")) {
    stop("old.netest must be an object of class netest")
  }
  if (!inherits(new.coef.diss, "disscoef")) {
    stop("new.coef.diss must be an object of class disscoef")
  }
  if (old.netest$edapprox != TRUE) {
    stop("Edges dissolution approximation must be used for this adjustment")
  }

  out <- old.netest

  ## if an old correction was applied...
  if (old.netest$coef.diss$coef.crude[1] != -Inf) {
    ## remove the old correction
    if (out$nested.edapprox == TRUE) {
      ## adjust the formation model coefficients to remove the old edapprox
      l.cd.o <- length(out$coef.diss$coef.form.corr)
      out$coef.form[1:l.cd.o] <- out$coef.form[1:l.cd.o] +
        out$coef.diss$coef.form.corr
    } else {
      ## remove the part of the formation model and coefficient vector
      ## corresponding to the old edapprox
      out$formation <- eval(quote(formation),
                            envir = environment(out$formation))
      out$coef.form <-
        out$coef.form[seq_len(length(out$coef.form) -
                                length(out$coef.diss$coef.form.corr))]
    }
  }

  ## if a new correction should be applied...
  if (new.coef.diss$coef.crude[1] != -Inf) {
    ## apply the new correction
    if (nested.edapprox == TRUE) {
      ## check that the new dissolution model is an initial segment of the
      ## formation model
      diss_check(out$formation, new.coef.diss$dissolution)

      ## implement new edapprox by adjusting the formation model coefficients
      l.cd.n <- length(new.coef.diss$coef.form.corr)
      out$coef.form[1:l.cd.n] <- out$coef.form[1:l.cd.n] -
        new.coef.diss$coef.form.corr
    } else {
      ## implement new edapprox by appending the new dissolution model to the
      ## formation model and appending the relevant values to the vector of
      ## formation model coefficients
      # nolint start
      formation <- out$formation
      dissolution <- new.coef.diss$dissolution
      # nolint end
      out$formation <- trim_env(~Passthrough(formation) + Passthrough(dissolution),
                                keep = c("formation", "dissolution"))
      out$coef.form <- c(out$coef.form, -new.coef.diss$coef.form.corr)
    }
  }

  out$coef.diss <- new.coef.diss
  out$nested.edapprox <- nested.edapprox

  return(out)
}

#' @title Deterministic Compartmental Models
#'
#' @description Solves deterministic compartmental epidemic models for
#'              infectious disease.
#'
#' @param param Model parameters, as an object of class [param.dcm()].
#' @param init Initial conditions, as an object of class [init.dcm()].
#' @param control Control settings, as an object of class
#'        [control.dcm()].
#'
#' @details
#' The `dcm` function uses the ordinary differential equation solver in
#' the `deSolve` package to model disease as a deterministic compartmental
#' system. The parameterization for these models follows the standard approach
#' in `EpiModel`, with epidemic parameters, initial conditions, and control
#' settings.
#'
#' The `dcm` function performs modeling of both base model types and
#' original models with new structures. Base model types include one-group
#' and two-group models with disease types for Susceptible-Infected (SI),
#' Susceptible-Infected-Recovered (SIR), and Susceptible-Infected-Susceptible
#' (SIS). Both base and original models require the `param`,
#' `init`, and `control` inputs.
#'
#' @return
#' A list of class `dcm` with the following elements:
#'
#'  * **param:** the epidemic parameters passed into the model through
#'        `param`, with additional parameters added as necessary.
#'  * **control:** the control settings passed into the model through
#'        `control`, with additional controls added as necessary.
#'  * **epi:** a list of data frames, one for each epidemiological
#'        output from the model. Outputs for base models always include the
#'        size of each compartment, as well as flows in, out of, and between
#'        compartments.
#'
#'
#' @references
#' Soetaert K, Petzoldt T, Setzer W. Solving Differential Equations in
#' R: Package deSolve. Journal of Statistical Software. 2010; 33(9): 1-25.
#' \doi{10.18637/jss.v033.i09}.
#'
#' @keywords model
#'
#' @seealso Extract the model results with [as.data.frame.dcm()].
#' Summarize the time-specific model results with [summary.dcm()].
#' Plot the model results with [plot.dcm()]. Plot a compartment flow
#' diagram with [comp_plot()].
#'
#' @export
#'
#' @examples
#' ## Example 1: SI Model (One-Group)
#' # Set parameters
#' param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
#' init <- init.dcm(s.num = 500, i.num = 1)
#' control <- control.dcm(type = "SI", nsteps = 500)
#' mod1 <- dcm(param, init, control)
#' mod1
#' plot(mod1)
#'
#' ## Example 2: SIR Model with Vital Dynamics (One-Group)
#' param <- param.dcm(inf.prob = 0.2, act.rate = 5,
#'                    rec.rate = 1/3, a.rate = 1/90, ds.rate = 1/100,
#'                    di.rate = 1/35, dr.rate = 1/100)
#' init <- init.dcm(s.num = 500, i.num = 1, r.num = 0)
#' control <- control.dcm(type = "SIR", nsteps = 500)
#' mod2 <- dcm(param, init, control)
#' mod2
#' plot(mod2)
#'
#' ## Example 3: SIS Model with act.rate Sensitivity Parameter
#' param <- param.dcm(inf.prob = 0.2, act.rate = seq(0.1, 0.5, 0.1),
#'                    rec.rate = 1/50)
#' init <- init.dcm(s.num = 500, i.num = 1)
#' control <- control.dcm(type = "SIS", nsteps = 500)
#' mod3 <- dcm(param, init, control)
#' mod3
#' plot(mod3)
#'
#' ## Example 4: SI Model with Vital Dynamics (Two-Group)
#' param <- param.dcm(inf.prob = 0.4,  inf.prob.g2 = 0.1,
#'                    act.rate = 0.25, balance = "g1",
#'                    a.rate = 1/100, a.rate.g2 = NA,
#'                    ds.rate = 1/100, ds.rate.g2 = 1/100,
#'                    di.rate = 1/50, di.rate.g2 = 1/50)
#' init <- init.dcm(s.num = 500, i.num = 1,
#'                  s.num.g2 = 500, i.num.g2 = 0)
#' control <- control.dcm(type = "SI", nsteps = 500)
#' mod4 <- dcm(param, init, control)
#' mod4
#' plot(mod4)
#'
#' ## Example 5: SI Model with Varying Initial Conditions
#' param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
#' init <- init.dcm(s.num = 500, i.num = c(1, 5, 25))
#' control <- control.dcm(type = "SI", nsteps = 500)
#' mod5 <- dcm(param, init, control)
#' mod5
#' plot(mod5)
#'
dcm <- function(param, init, control) {
  check.control.class("dcm", "EpiModel dcm")

  crosscheck.dcm(param, init, control)

  # Model selection ---------------------------------------------------------
  if (is.null(control$new.mod)) {

    if (control$type == "SI") {
      if (param$groups == 1) {
        if (param$vital == FALSE) {
          model <- mod_SI_1g_cl
          init <- c(init, si.flow = 0)
        } else {
          model <- mod_SI_1g_op
          init <- c(init, si.flow = 0, a.flow = 0, ds.flow = 0, di.flow = 0)
        }
      } else {
        if (param$vital == FALSE) {
          model <- mod_SI_2g_cl
          init <- c(init, si.flow = 0, si.flow.g2 = 0)
        } else {
          model <- mod_SI_2g_op
          init <- c(init,
                    si.flow = 0, a.flow = 0, ds.flow = 0, di.flow = 0,
                    si.flow.g2 = 0, a.flow.g2 = 0, ds.flow.g2 = 0,
                    di.flow.g2 = 0)
        }
      }
    }
    if (control$type == "SIR") {
      if (param$groups == 1) {
        if (param$vital == FALSE) {
          model <- mod_SIR_1g_cl
          init <- c(init, si.flow = 0, ir.flow = 0)
        } else {
          model <- mod_SIR_1g_op
          init <- c(init, si.flow = 0, ir.flow = 0, a.flow = 0,
                    ds.flow = 0, di.flow = 0, dr.flow = 0)
        }
      } else {
        if (param$vital == FALSE) {
          model <- mod_SIR_2g_cl
          init <- c(init, si.flow = 0, ir.flow = 0,
                    si.flow.g2 = 0, ir.flow.g2 = 0)
        } else {
          model <- mod_SIR_2g_op
          init <- c(init, si.flow = 0, ir.flow = 0, a.flow = 0,
                    ds.flow = 0, di.flow = 0, dr.flow = 0,
                    si.flow.g2 = 0, ir.flow.g2 = 0, a.flow.g2 = 0,
                    ds.flow.g2 = 0, di.flow.g2 = 0, dr.flow.g2 = 0)
        }
      }
    }
    if (control$type == "SIS") {
      if (param$groups == 1) {
        if (param$vital == FALSE) {
          model <- mod_SIS_1g_cl
          init <- c(init, si.flow = 0, is.flow = 0)
        } else {
          model <- mod_SIS_1g_op
          init <- c(init, si.flow = 0, is.flow = 0,
                    a.flow = 0, ds.flow = 0, di.flow = 0)
        }
      } else {
        if (param$vital == FALSE) {
          model <- mod_SIS_2g_cl
          init <- c(init, si.flow = 0, is.flow = 0,
                    si.flow.g2 = 0, is.flow.g2 = 0)
        } else {
          model <- mod_SIS_2g_op
          init <- c(init, si.flow = 0, is.flow = 0,
                    a.flow = 0, ds.flow = 0, di.flow = 0,
                    si.flow.g2 = 0, is.flow.g2 = 0,
                    a.flow.g2 = 0, ds.flow.g2 = 0, di.flow.g2 = 0)
        }
      }
    }

  } else {
    # Sub in new model
    model <- control$new.mod
  }


  # Print model option ------------------------------------------------------
  if (control$print.mod == TRUE) {
    return(print(model))
  }


  # Sensitivity parameters and initial conditions ----------------------------
  if (control$sens.param == FALSE) {
    control$nruns <- 1
  } else {
    control$nruns <- max(sapply(param, length), sapply(init, length))
  }

  # Validate that all varying vectors have the same length
  if (control$nruns > 1) {
    all.lengths <- c(sapply(param, length), sapply(init, length))
    long.lengths <- all.lengths[all.lengths > 1]
    if (length(unique(long.lengths)) > 1) {
      stop("All varying parameters and initial conditions must have the ",
           "same length")
    }

    longv <- which(sapply(param, length) == control$nruns)
    longvn <- names(longv)
    lim.p <- param[!(names(param) %in% longvn)]

    longi <- which(sapply(init, length) == control$nruns)
    longin <- names(longi)
    lim.i <- init[!(names(init) %in% longin)]
  }


  # Model runs --------------------------------------------------------------
  verbose.dcm(control, type = "startup")
  for (s in 1:control$nruns) {

    ## Sensitivity parameter input
    if (control$nruns > 1) {
      all.p <- as.list(lim.p)
      for (j in seq_along(longvn)) {
        all.p[[longvn[j]]] <- param[[longvn[j]]][s]
      }
    } else {
      all.p <- param
    }

    ## Per-run initial conditions
    if (control$nruns > 1 && length(longin) > 0) {
      all.i <- lim.i
      for (j in seq_along(longin)) {
        all.i[[longin[j]]] <- init[[longin[j]]][s]
      }
    } else {
      all.i <- init
    }
    t0 <- setNames(as.numeric(all.i), names(all.i))

    ## Timesteps
    if (length(control$nsteps) == 1) {
      times <- seq(1, control$nsteps, control$dt)
    } else {
      times <- control$nsteps
    }


    ## Differential equation solvers
    if (control$dede == FALSE) {
      df <- data.frame(ode(y = t0,
                           times = times,
                           func = model,
                           parms = all.p,
                           method = control$odemethod))
    } else {
      df <- data.frame(dede(y = t0,
                            times = times,
                            func = model,
                            parms = all.p))
    }

    ## Recalculate Flows
    isFlow <- grep(".flow", x = names(df))
    if (length(isFlow) > 0) {
      for (j in isFlow) {
        df[, j] <- c(diff(df[, j]), NA)
      }
    }


    # Output ------------------------------------------------------------------

    ## Save to out object
    if (s == 1) {
      out <- saveout.dcm(df, s, param, control)
    } else {
      out <- saveout.dcm(df, s, param, control, out)
    }

    ## Progress
    verbose.dcm(control, type = "progress", s)

  }

  out$init <- init

  class(out) <- "dcm"
  invisible(out)
}

#' @title Stochastic Network Models
#'
#' @description Simulates stochastic network epidemic models for infectious
#'              disease. This is typically the final step in the network
#'              modeling pipeline, after network estimation with [`netest`]
#'              and model diagnostics with [`netdx`].
#'
#' @param x If `control$start == 1`, either a fitted network model object
#'        of class `netest` or a list of such objects. If
#'        `control$start > 1`, an object of class `netsim`. When
#'        multiple networks are used (multi-layer models), pass a list of
#'        `netest` objects, one per network layer; the node sets (including
#'        network size and nodal attributes) are assumed to be the same for
#'        all networks.
#' @param param Model parameters, as an object of class [`param.net`].
#'        Includes transmission probability (`inf.prob`), act rate
#'        (`act.rate`), recovery rate (`rec.rate`), and demographic rates
#'        for models with vital dynamics. Custom parameters for extended
#'        models may also be passed through `param.net`.
#' @param init Initial conditions, as an object of class [`init.net`].
#'        Specifies the initial number of infected nodes (`i.num`) and,
#'        for SIR models, recovered nodes (`r.num`). For two-group models,
#'        the corresponding `.g2` parameters are also required.
#' @param control Control settings, as an object of class [`control.net`].
#'        Key settings include `type` (disease model: `"SI"`, `"SIR"`, or
#'        `"SIS"`), `nsteps` (number of time steps), `nsims` (number of
#'        simulations), `tergmLite` (lightweight mode for performance), and
#'        `resimulate.network` (required for models with vital dynamics).
#'        For extended models, custom module functions are also passed here.
#'
#' @details
#' Stochastic network models explicitly represent phenomena within and across
#' edges (pairs of nodes that remain connected) over time. This enables edges to
#' have duration, allowing for repeated transmission-related acts within the
#' same dyad, specification of edge formation and dissolution rates, control
#' over the temporal sequencing of multiple edges, and specification of
#' network-level features. A detailed description of these models, along with
#' examples, is found in the [Network Modeling for Epidemics](https://epimodel.github.io/sismid/)
#' course materials.
#'
#' The `netsim` function performs modeling of both the base model types
#' and original models. Base model types include one-group and two-group models
#' with disease types for Susceptible-Infected (SI), Susceptible-Infected-Recovered (SIR),
#' and Susceptible-Infected-Susceptible (SIS).
#'
#' Original models may be parameterized by writing new process modules that
#' either take the place of existing modules (for example, disease recovery), or
#' supplement the set of existing processes with a new one contained in a new
#' module. This functionality is documented in the
#' [Extending EpiModel](https://epimodel.github.io/sismid/9_extending/mod9-Intro.html)
#' section of the [Network Modeling for Epidemics](https://epimodel.github.io/sismid/)
#' course materials. The list of modules within `netsim` available for
#' modification is listed in [modules.net()].
#'
#' @section Module Pipeline:
#' At each time step, `netsim` executes a series of modules in sequence.
#' For base models, the default pipeline is:
#'
#'  1. **Network resimulation** ([`resim_nets`]): updates the network
#'     structure (if `resimulate.network = TRUE`).
#'  2. **Infection** ([`infection.net`]): simulates disease transmission
#'     across discordant edges (where one partner is susceptible and the
#'     other is infected).
#'  3. **Recovery** ([`recovery.net`]): simulates recovery from infection
#'     (SIR and SIS models only).
#'  4. **Departures** ([`departures.net`]): simulates node exits from the
#'     network (if vital dynamics are enabled).
#'  5. **Arrivals** ([`arrivals.net`]): simulates new node entries into the
#'     network (if vital dynamics are enabled).
#'  6. **Prevalence** ([`prevalence.net`]): calculates summary statistics.
#'
#' See [modules.net()] for full details on each module.
#'
#' @section Performance and tergmLite:
#' Setting `tergmLite = TRUE` in [`control.net`] uses a lightweight network
#' representation (`networkLite`) that is substantially faster than the full
#' `networkDynamic` representation, often by a factor of 20--50x. This is
#' recommended for large networks or when running many simulations. The
#' trade-off is that full `networkDynamic` objects are not available for
#' post-hoc analysis; use the cumulative edgelist
#' (`cumulative.edgelist = TRUE`) instead to track partnership histories.
#'
#' The `resimulate.network` control must be set to `TRUE` when demographic
#' processes (arrivals and departures) change the network composition over
#' time. Without it, the network structure evolves independently of the
#' epidemic and demographic dynamics.
#'
#' @section Multi-Network Models:
#' For models with multiple overlapping network layers (e.g., sexual and
#' needle-sharing networks), pass a list of [`netest`] objects to the `x`
#' argument, one per network layer. Each layer has its own
#' formation/dissolution dynamics but shares the same node set. See the
#' `multilayer` documentation and `test-multinets.R` for examples.
#'
#' @section Restarting and Checkpointing:
#' Simulations can be checkpointed and restarted if interrupted. Set
#' `.checkpoint.steps` and `.checkpoint.dir` in [`control.net`] to enable
#' automatic checkpointing. To restart a simulation from a prior `netsim`
#' output, pass the `netsim` object as `x` and set `control$start` to one
#' greater than the final time step of the prior simulation. See the
#' Checkpointing Simulations section of [`control.net`] for full details.
#'
#' @return
#' A list of class `netsim` with the following elements:
#'
#'  * **param:** the epidemic parameters passed into the model through
#'        `param`, with additional parameters added as necessary.
#'  * **control:** the control settings passed into the model through
#'        `control`, with additional controls added as necessary.
#'  * **epi:** a list of data frames, one for each epidemiological
#'        output from the model. Outputs for base models always include the
#'        size of each compartment, as well as flows in, out of, and between
#'        compartments.
#'  * **stats:** a list containing two sublists, `nwstats` for any
#'        network statistics saved in the simulation, and `transmat` for
#'        the transmission matrix saved in the simulation. See
#'        [control.net()] for further
#'        details.
#'  * **network:** a list of lists of `networkDynamic` or
#'        `networkLite` objects, with one list of objects for each model
#'        simulation.
#'
#' If `control$raw.output == TRUE`: A list of the raw (pre-processed)
#' `netsim_dat` objects, for use in simulation continuation.
#'
#' The `epi` data can be extracted as a data frame with
#' [as.data.frame.netsim()], with options for per-simulation values
#' (`out = "vals"`), means (`out = "mean"`), standard deviations
#' (`out = "sd"`), or quantiles (`out = "qnt"`).
#'
#' @references
#' Jenness SM, Goodreau SM and Morris M. EpiModel: An R Package for Mathematical
#' Modeling of Infectious Disease over Networks. Journal of Statistical
#' Software. 2018; 84(8): 1-47.
#'
#' @seealso Estimate the network model with [`netest`] and diagnose model
#'          fit with [`netdx`] before running simulations. Extract model
#'          results with [as.data.frame.netsim()]. Summarize the
#'          time-specific model results with [summary.netsim()]. Plot the
#'          model results with [plot.netsim()]. Extract the network with
#'          [`get_network`], the transmission matrix with [`get_transmat`],
#'          and derive new epi statistics with [`mutate_epi`].
#'
#' @keywords model
#' @export
#'
#' @examples
#' \dontrun{
#' ## Example 1: SI Model without Network Feedback
#' # Network model estimation
#' nw <- network_initialize(n = 100)
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#' est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' # Epidemic model
#' param <- param.net(inf.prob = 0.3)
#' init <- init.net(i.num = 10)
#' control <- control.net(type = "SI", nsteps = 100, nsims = 5, verbose.int = 0)
#' mod1 <- netsim(est1, param, init, control)
#'
#' # Print, plot, and summarize the results
#' mod1
#' plot(mod1)
#' summary(mod1, at = 50)
#'
#' ## Example 2: SIR Model with Network Feedback (Two-Group)
#' nw <- network_initialize(n = 100)
#' nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
#' formation <- ~edges
#' target.stats <- 50
#'
#' # Recalculate dissolution coefficient with departure rate
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20,
#'                                d.rate = 0.0021)
#'
#' # Reestimate the model with new coefficient
#' est2 <- netest(nw, formation, target.stats, coef.diss)
#'
#' # Set parameters to include demographic rates
#' param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15,
#'                    rec.rate = 0.02, rec.rate.g2 = 0.02,
#'                    a.rate = 0.002, a.rate.g2 = NA,
#'                    ds.rate = 0.001, ds.rate.g2 = 0.001,
#'                    di.rate = 0.001, di.rate.g2 = 0.001,
#'                    dr.rate = 0.001, dr.rate.g2 = 0.001)
#' init <- init.net(i.num = 10, i.num.g2 = 10,
#'                  r.num = 0, r.num.g2 = 0)
#' control <- control.net(type = "SIR", nsteps = 100, nsims = 5,
#'                        resimulate.network = TRUE, tergmLite = TRUE)
#'
#' # Simulate the model with new network fit
#' mod2 <- netsim(est2, param, init, control)
#'
#' # Print, plot, and summarize the results
#' mod2
#' plot(mod2)
#' summary(mod2, at = 40)
#'
#' ## Example 3: Post-Simulation Analysis
#' # Extract epi data as a data frame
#' as.data.frame(mod1)
#' as.data.frame(mod1, out = "mean")
#'
#' # Extract the transmission matrix from simulation 1
#' get_transmat(mod1, sim = 1)
#'
#' # Derive new epi statistics and plot them
#' mod1 <- mutate_epi(mod1, prev = i.num / num)
#' plot(mod1, y = "prev")
#' }
#'
netsim <- function(x, param, init, control) {
  crosscheck.net(x, param, init, control)
  control <- netsim_validate_control(control)
  if (!is.null(control[["verbose.FUN"]]))
    control[["verbose.FUN"]](control, type = "startup")

  if (control$ncores == 1 && isFALSE(control$future.use.plan)) {
    dat_list <- lapply(
      seq_len(control$nsims),
      function(s) {
        netsim_initialize(x, param, init, control, s)
      }
    )
    dat_list <- Map(netsim_run, dat = dat_list, s = seq_along(dat_list))
  } else {
    if (inherits(control$future.use.plan, c("tweaked", "future"))) {
      with(future::plan(control$future.use.plan), local = TRUE)
    } else if (control$ncores > 1 && isFALSE(control$future.use.plan)) {
      with(future::plan("multisession", workers = control$ncores), local = TRUE)
    } # else if (isTRUE(control$future.use.plan)) - use plan defined by user
    dat_list <- future.apply::future_lapply(
      seq_len(control$nsims),
      function(s) {
        dat <- netsim_initialize(x, param, init, control, s)
        netsim_run(dat, s)
      },
      future.seed = TRUE
    )
  }

  out <- if (control$raw.output) dat_list else process_out.net(dat_list)

  if (!control[[".checkpoint.keep"]])
    netsim_clear_checkpoint(control)

  return(out)
}

# Ensure default values for control.net
#
netsim_validate_control <- function(control) {
  # Controls to be set to TRUE or FALSE if missing
  control_default_bool <- list(
    "TRUE" = c(
      ".checkpoint.compress"
    ),
    "FALSE" = c(
      "resimulate.network",
      "raw.output",
      "verbose",
      ".checkpoint.keep",
      ".traceback.on.error",
      ".dump.frame.on.error",
      "cumulative.edgelist",
      "future.use.plan",
      "tergmLite",
      "save.network",
      "save.diss.stats",
      "save.nwstats",
      "save.transmat",
      "save.network",
      "save.cumulative.edgelist",
      "save.run"
    )
  )

  if (is.null(control$save.other))
    control$save.other <- character(0)

  for (val in names(control_default_bool)) {
    for (flag in control_default_bool[[val]])
      if (is.null(control[[flag]])) control[[flag]] <- as.logical(val)
  }

  # truncate the cumulative edgelists to keep only active partnerships
  if (is.null(control$truncate.el.cuml))
    control$truncate.el.cuml <- 0

  if (is.null(control$start))
    control$start <- 1

  if (control$nsims == 1) {
    control$ncores <- 1
  } else {
    control$ncores <- cap_ncores(control$ncores)
    control$verbose <- FALSE
  }

  control[[".checkpointed"]] <- netsim_is_checkpointed(control)
  if (!control[[".checkpointed"]]) {
    control[[".checkpoint.steps"]] <- Inf
  } else {
    control[[".checkpoint.steps"]] <- as.integer(control[[".checkpoint.steps"]])
  }

  if (control$nsteps < 1 || control$nsteps == Inf) {
    stop("`control$nsteps` must be positive and not infinite.")
  }

  return(control)
}

# Create the `dat` object or load it if checkpointed
#
netsim_initialize <- function(x, param, init, control, s = 1) {
  if (netsim_is_resume_checkpoint(control, s)) {
    dat <- netsim_load_checkpoint(control, s)
  } else {
    param <- generate_random_params(param, verbose = FALSE)
    dat <- control[["initialize.FUN"]](x, param, init, control, s)
    dat <- make_module_list(dat)
    if (get_control(dat, "start") != 1) {
      dat <- set_current_timestep(dat, get_control(dat, "start") - 1L)
    }
    if (get_control(dat, ".checkpointed"))
      netsim_save_checkpoint(dat, s)

    check_end_horizon_control(dat)
  }

  return(dat)
}

# Run a simulation from a initialized dat object
#
netsim_run <- function(dat, s = 1) {
  while (get_current_timestep(dat) < get_control(dat, "nsteps")) {
    steps_to_run <- get_control(dat, "nsteps") - get_current_timestep(dat)
    steps_to_run <- min(steps_to_run, get_control(dat, ".checkpoint.steps"))
    dat <- netsim_run_nsteps(dat, steps_to_run, s)

    if (get_control(dat, ".checkpointed"))
      netsim_save_checkpoint(dat, s)
  }

  return(dat)
}

# Run N simulation steps on a `dat` object
#
netsim_run_nsteps <- function(dat, nsteps, s) {
  for (n in seq_len(nsteps)) {
    dat <- increment_timestep(dat)
    dat <- netsim_run_modules(dat, s)
  }
  return(dat)
}

# Run all the modules of a `dat` object once
#
netsim_run_modules <- function(dat, s) {
  at <- get_current_timestep(dat)

  dat <- withCallingHandlers(
    expr = {
      current_mod <- "epimodel.internal"
      # Applies updaters, if any
      dat <- input_updater(dat)
      dat <- trigger_end_horizon(dat)

      modules <- get_modules(dat)

      for (i in seq_along(modules)) {
        current_mod <- names(modules)[i]
        dat <- modules[[i]](dat, at)
      }

      current_mod <- "epimodel.internal"
      # Run the user-provided trackers, if any
      dat <- epi_trackers(dat)

      ## Verbose module
      if (!is.null(get_control(dat, "verbose.FUN"))) {
        current_mod <- "verbose.FUN"
        do.call(get_control(dat, "verbose.FUN"),
                list(dat, type = "progress", s, at))
      }

      dat
    },
    message = function(e) message(netsim_cond_msg("MESSAGE", current_mod, at)),
    warning = function(e) message(netsim_cond_msg("WARNING", current_mod, at)),
    error = function(e) {
      message(netsim_cond_msg("ERROR", current_mod, at))
      netsim_error_logger(dat, s)
    }
  )

  return(dat)
}

#' @title Primary Infection Module for icm
#'
#' @description This function simulates the main infection process given the
#'              current state of the actors in the system.
#'
#' @inheritParams prevalence.icm
#'
#' @inherit recovery.icm return
#'
#' @export
#' @keywords internal
#'
infection.icm <- function(dat, at) {

  ## Expected acts
  acts <- round(dat$param$act.rate * dat$epi$num[at - 1] / 2)

  ## Edgelist
  p1 <- ssample(which(dat$attr$active == 1), acts, replace = TRUE)
  p2 <- ssample(which(dat$attr$active == 1), acts, replace = TRUE)


  del <- NULL
  if (length(p1) > 0 && length(p2) > 0) {
    del <- data.frame(p1, p2)
    while (any(del$p1 == del$p2)) {
      del$p2 <- ifelse(del$p1 == del$p2,
                       ssample(which(dat$attr$active == 1), 1), del$p2)
    }



    ## Discordant edgelist
    del$p1.stat <- dat$attr$status[del$p1]
    del$p2.stat <- dat$attr$status[del$p2]
    serodis <- (del$p1.stat == "s" & del$p2.stat == "i") |
      (del$p1.stat == "i" & del$p2.stat == "s")
    del <- del[serodis == TRUE, ]


    ## Transmission on edgelist
    if (nrow(del) > 0) {
      del$tprob <- dat$param$inf.prob

      if (!is.null(dat$param$inter.eff) && at >= dat$param$inter.start) {
        del$tprob <- del$tprob * (1 - dat$param$inter.eff)
      }
      del$trans <- rbinom(nrow(del), 1, del$tprob)
      del <- del[del$trans == TRUE, ]
      if (nrow(del) > 0) {
        newIds <- unique(ifelse(del$p1.stat == "s", del$p1, del$p2))
        nInf <- length(newIds)

        dat$attr$status[newIds] <- "i"
        dat$attr$infTime[newIds] <- at
      } else {
        nInf <- 0
      }
    } else {
      nInf <- 0
    }
  } else {
    nInf <- 0
  }


  ## Output
  if (at == 2) {
    dat$epi$si.flow <- c(0, nInf)
  } else {
    dat$epi$si.flow[at] <- nInf
  }

  return(dat)

}


#' @title Recovery: icm Module
#'
#' @description This function simulates recovery from the infected state
#'              either to a distinct recovered state (SIR model type) or back
#'              to a susceptible state (SIS model type), for use in
#'              [icm()].
#'
#' @inheritParams prevalence.icm
#'
#' @return The updated `icm_dat` class main data object.
#'
#' @export
#' @keywords internal
#'
recovery.icm <- function(dat, at) {

  # Conditions --------------------------------------------------------------
  if (!(dat$control$type %in% c("SIR", "SIS"))) {
    return(dat)
  }


  # Variables ---------------------------------------------------------------
  active <- dat$attr$active
  status <- dat$attr$status

  type <- dat$control$type
  recovState <- ifelse(type == "SIR", "r", "s")

  rec.rate <- dat$param$rec.rate

  nRecov <- 0
  idsElig <- which(active == 1 & status == "i")
  nElig <- length(idsElig)

  if (nElig > 0) {
    vecRecov <- which(rbinom(nElig, 1, rec.rate) == 1)
    if (length(vecRecov) > 0) {
      idsRecov <- idsElig[vecRecov]
      nRecov <- length(idsRecov)
      status[idsRecov] <- recovState
    }
  }
  dat$attr$status <- status


  # Output ------------------------------------------------------------------
  outName <- ifelse(type == "SIR", "ir.flow", "is.flow")

  ## Summary statistics
  if (at == 2) {
    dat$epi[[outName[1]]] <- c(0, nRecov)
  } else {
    dat$epi[[outName[1]]][at] <- nRecov
  }

  return(dat)
}


#' @title Primary Infection Module for icm
#'
#' @description This function simulates the main infection process given the
#'              current state of the actors in the system.
#'
#' @inheritParams prevalence.icm
#'
#' @inherit recovery.icm return
#'
#' @export
#' @keywords internal
#'
infection.icm.bip <- function(dat, at) {

  ## Expected acts
  if (dat$param$balance == "g1") {
    acts <- round(dat$param$act.rate *
                    (dat$epi$num[at - 1] + dat$epi$num.g2[at - 1]) / 2)
  }
  if (dat$param$balance == "g2") {
    acts <- round(dat$param$act.rate.g2 *
                    (dat$epi$num[at - 1] + dat$epi$num.g2[at - 1]) / 2)
  }


  ## Edgelist
  p1 <- ssample(which(dat$attr$active == 1 & dat$attr$group == 1),
                acts, replace = TRUE)
  p2 <- ssample(which(dat$attr$active == 1 & dat$attr$group == 2),
                acts, replace = TRUE)

  del <- NULL
  if (length(p1) > 0 && length(p2) > 0) {
    del <- data.frame(p1, p2)

    ## Discordant edgelist
    del$p1.stat <- dat$attr$status[del$p1]
    del$p2.stat <- dat$attr$status[del$p2]
    serodis <- (del$p1.stat == "s" & del$p2.stat == "i") |
      (del$p1.stat == "i" & del$p2.stat == "s")
    del <- del[serodis == TRUE, ]


    ## Transmission on edgelist
    if (nrow(del) > 0) {
      del$tprob <- ifelse(del$p1.stat == "s", dat$param$inf.prob,
                          dat$param$inf.prob.g2)
      if (!is.null(dat$param$inter.eff) && at >= dat$param$inter.start) {
        del$tprob <- del$tprob * (1 - dat$param$inter.eff)
      }
      del$trans <- rbinom(nrow(del), 1, del$tprob)
      del <- del[del$trans == TRUE, ]
      if (nrow(del) > 0) {
        newIdsg1 <- unique(del$p1[del$p1.stat == "s"])
        newIdsg2 <- unique(del$p2[del$p2.stat == "s"])
        nInf <- length(newIdsg1)
        nInfg2 <- length(newIdsg2)
        newIds <- c(newIdsg1, newIdsg2)
        dat$attr$status[newIds] <- "i"
        dat$attr$infTime[newIds] <- at
      } else {
        nInf <- nInfg2 <- 0
      }
    } else {
      nInf <- nInfg2 <- 0
    }
  } else {
    nInf <- nInfg2 <- 0
  }


  ## Output
  if (at == 2) {
    dat$epi$si.flow <- c(0, nInf)
    dat$epi$si.flow.g2 <- c(0, nInfg2)
  } else {
    dat$epi$si.flow[at] <- nInf
    dat$epi$si.flow.g2[at] <- nInfg2
  }

  return(dat)

}


#' @title Recovery: icm Module
#'
#' @description This function simulates recovery from the infected state
#'              either to a distinct recovered state (SIR model type) or back
#'              to a susceptible state (SIS model type), for use in
#'              [icm()].
#'
#' @inheritParams prevalence.icm
#'
#' @inherit recovery.icm return
#'
#' @export
#' @keywords internal
#'
recovery.icm.bip <- function(dat, at) {

  # Conditions --------------------------------------------------------------
  if (!(dat$control$type %in% c("SIR", "SIS"))) {
    return(dat)
  }


  # Variables ---------------------------------------------------------------
  active <- dat$attr$active
  status <- dat$attr$status

  group <- dat$attr$group

  type <- dat$control$type
  recovState <- ifelse(type == "SIR", "r", "s")

  rec.rate <- dat$param$rec.rate
  rec.rate.g2 <- dat$param$rec.rate.g2

  nRecov <- nRecovG2 <- 0
  idsElig <- which(active == 1 & status == "i")
  nElig <- length(idsElig)

  if (nElig > 0) {

    gElig <- group[idsElig]
    rates <- c(rec.rate, rec.rate.g2)
    ratesElig <- rates[gElig]

    vecRecov <- which(rbinom(nElig, 1, ratesElig) == 1)
    if (length(vecRecov) > 0) {
      idsRecov <- idsElig[vecRecov]
      nRecov <- sum(group[idsRecov] == 1)
      nRecovG2 <- sum(group[idsRecov] == 2)
      status[idsRecov] <- recovState
    }
  }
  dat$attr$status <- status


  # Output ------------------------------------------------------------------
  outName <- ifelse(type == "SIR", "ir.flow", "is.flow")
  outName[2] <- paste0(outName, ".g2")

  ## Summary statistics
  if (at == 2) {
    dat$epi[[outName[1]]] <- c(0, nRecov)
  } else {
    dat$epi[[outName[1]]][at] <- nRecov
  }
  if (at == 2) {
    dat$epi[[outName[2]]] <- c(0, nRecovG2)
  } else {
    dat$epi[[outName[2]]][at] <- nRecovG2
  }

  return(dat)
}

#' @title Deterministic Compartmental Model Functions
#'
#' @description These functions parameterize the base deterministic
#'              compartmental models solved using the [dcm()]
#'              function.
#'
#' @param t Time vector, passed into model function internally through
#'        [dcm()] via the control settings in
#'        [control.dcm()].
#' @param t0 Initial conditions for model, passed into model function internally
#'        through [dcm()] via the initial conditions in
#'        [init.dcm()].
#' @param parms Model parameters, passed into model function internally through
#'        [dcm()] via the parameter settings in
#'        [param.dcm()].
#'
#' @details
#' This help page shows the names of all the base deterministic compartmental
#' model functions supported in EpiModel. Base models are those already
#' programmed interally within the software. The model functions may be printed
#' to see their internal structure, either directly on the console or by using
#' the `print.mod` argument in [control.dcm()].
#'
#' The naming convention for the models listed here follows the format:
#' `mod_<disease type>_<number of groups>_<vital dynamics>`. The supported
#' disease types are SI, SIS, and SIR; the number of groups are 1 or 2; and the
#' vital dynamic options are closed (fixed population composition) or open (with
#' arrivals and departures).
#' @name dcm.mods
#'
NULL


# SI, 1 group, closed pop -------------------------------------------------
#' @rdname dcm.mods
#' @keywords internal
#' @export
mod_SI_1g_cl <- function(t, t0, parms) {
  with(as.list(c(t0, parms)), {

    # Derivations
    num <- s.num + i.num

    # Parameters
    lambda <- inf.prob * act.rate * i.num / num
    if (!is.null(parms$inter.eff) && t >= inter.start) {
      lambda <- lambda * (1 - inter.eff)
    }

    # Flows
    si.flow <- lambda * s.num

    # ODEs
    dS <- -si.flow
    dI <- si.flow

    # Output
    list(c(dS, dI,
           si.flow),
         num = num)
  })
}


# SI, 1 group, open pop ---------------------------------------------------
#' @rdname dcm.mods
#' @keywords internal
#' @export
mod_SI_1g_op <- function(t, t0, parms) {
  with(as.list(c(t0, parms)), {

    # Derivations
    num <- s.num + i.num

    # Parameters
    lambda <- inf.prob * act.rate * i.num / num
    if (!is.null(parms$inter.eff) && t >= inter.start) {
      lambda <- lambda * (1 - inter.eff)
    }

    # Flows
    si.flow <- lambda * s.num
    a.flow <- a.rate * num
    ds.flow <- ds.rate * s.num
    di.flow <- di.rate * i.num

    # ODEs
    dS <- -si.flow + a.flow - ds.flow
    dI <- si.flow - di.flow

    # Output
    list(c(dS, dI,
           si.flow, a.flow, ds.flow, di.flow),
         num = num)
  })
}


# SI, 2 group, closed pop -------------------------------------------------
#' @rdname dcm.mods
#' @keywords internal
#' @export
mod_SI_2g_cl <- function(t, t0, parms) {
  with(as.list(c(t0, parms)), {

    # Derivations
    num.g1 <- s.num + i.num
    num.g2 <- s.num.g2 + i.num.g2

    # Act Balancing
    if (balance == "g1") {
      ar.g1 <- act.rate
      ar.g2 <- ar.g1 * num.g1 / num.g2
    }
    if (balance == "g2") {
      ar.g2 <- act.rate.g2
      ar.g1 <- ar.g2 * num.g2 / num.g1
    }

    # Group Lambdas
    lambda.g1 <- inf.prob * ar.g1 * i.num.g2 / num.g2
    lambda.g2 <- inf.prob.g2 * ar.g2 * i.num / num.g1
    if (!is.null(parms$inter.eff) && t >= inter.start) {
      lambda.g1 <- lambda.g1 * (1 - inter.eff)
      lambda.g2 <- lambda.g2 * (1 - inter.eff)
    }

    # Flows
    si.flow <- lambda.g1 * s.num
    si.flow.g2 <- lambda.g2 * s.num.g2

    # ODEs
    dSm1 <- -si.flow
    dIm1 <-  si.flow
    dSm2 <- -si.flow.g2
    dIm2 <-  si.flow.g2

    # Output
    list(c(dSm1, dIm1, dSm2, dIm2,
           si.flow, si.flow.g2),
         num = num.g1, num.g2 = num.g2)
  })
}


# SI, 2 group, open pop ---------------------------------------------------
#' @rdname dcm.mods
#' @keywords internal
#' @export
mod_SI_2g_op <- function(t, t0, parms) {
  with(as.list(c(t0, parms)), {

    # Derivations
    num.g1 <- s.num + i.num
    num.g2 <- s.num.g2 + i.num.g2

    # Act Balancing
    if (balance == "g1") {
      ar.g1 <- act.rate
      ar.g2 <- ar.g1 * num.g1 / num.g2
    }
    if (balance == "g2") {
      ar.g2 <- act.rate.g2
      ar.g1 <- ar.g2 * num.g2 / num.g1
    }

    # Group Lambdas
    lambda.g1 <- inf.prob * ar.g1 * i.num.g2 / num.g2
    lambda.g2 <- inf.prob.g2 * ar.g2 * i.num / num.g1
    if (!is.null(parms$inter.eff) && t >= inter.start) {
      lambda.g1 <- lambda.g1 * (1 - inter.eff)
      lambda.g2 <- lambda.g2 * (1 - inter.eff)
    }

    # Flows
    si.flow <- lambda.g1 * s.num
    si.flow.g2 <- lambda.g2 * s.num.g2
    if (is.na(a.rate.g2)) {
      a.flow <- a.rate * num.g1
      a.flow.g2 <- a.rate * num.g1
    } else {
      a.flow <- a.rate * num.g1
      a.flow.g2 <- a.rate.g2 * num.g2
    }
    ds.flow <- ds.rate * s.num
    ds.flow.g2 <- ds.rate.g2 * s.num.g2
    di.flow <- di.rate * i.num
    di.flow.g2 <- di.rate.g2 * i.num.g2

    # ODEs
    dSm1 <- -si.flow + a.flow - ds.flow
    dIm1 <-  si.flow - di.flow
    dSm2 <- -si.flow.g2 + a.flow.g2 - ds.flow.g2
    dIm2 <-  si.flow.g2 - di.flow.g2

    # Output
    list(c(dSm1, dIm1, dSm2, dIm2,
           si.flow, a.flow, ds.flow, di.flow,
           si.flow.g2, a.flow.g2, ds.flow.g2, di.flow.g2),
         num = num.g1,
         num.g2 = num.g2)
  })
}


# SIR, 1 group, closed pop ------------------------------------------------
#' @rdname dcm.mods
#' @keywords internal
#' @export
mod_SIR_1g_cl <- function(t, t0, parms) {
  with(as.list(c(t0, parms)), {

    # Derivations
    num <- s.num + i.num + r.num

    # Parameters
    lambda <- inf.prob * act.rate * i.num / num
    if (!is.null(parms$inter.eff) && t >= inter.start) {
      lambda <- lambda * (1 - inter.eff)
    }

    # Flows
    si.flow <- lambda * s.num
    ir.flow <- rec.rate * i.num

    # ODEs
    dS <- -si.flow
    dI <- si.flow - ir.flow
    dR <- ir.flow

    # Output
    list(c(dS, dI, dR,
           si.flow, ir.flow),
         num = num)
  })
}


# SIR, 1 group, open pop --------------------------------------------------
#' @rdname dcm.mods
#' @keywords internal
#' @export
mod_SIR_1g_op <- function(t, t0, parms) {
  with(as.list(c(t0, parms)), {

    # Derivations
    num <- s.num + i.num + r.num

    # Parameters
    lambda <- inf.prob * act.rate * i.num / num
    if (!is.null(parms$inter.eff) && t >= inter.start) {
      lambda <- lambda * (1 - inter.eff)
    }

    # Flows
    si.flow <- lambda * s.num
    ir.flow <- rec.rate * i.num
    a.flow <- a.rate * num
    ds.flow <- ds.rate * s.num
    di.flow <- di.rate * i.num
    dr.flow <- dr.rate * r.num

    # ODEs
    dS <- -si.flow + a.flow - ds.flow
    dI <- si.flow - ir.flow - di.flow
    dR <- ir.flow - dr.flow

    # Output
    list(c(dS, dI, dR,
           si.flow, ir.flow, a.flow,
           ds.flow, di.flow, dr.flow),
         num = num)
  })
}


# SIR, 2 group, closed pop ------------------------------------------------
#' @rdname dcm.mods
#' @keywords internal
#' @export
mod_SIR_2g_cl <- function(t, t0, parms) {
  with(as.list(c(t0, parms)), {

    # Derivations
    num.g1 <- s.num + i.num + r.num
    num.g2 <- s.num.g2 + i.num.g2 + r.num.g2

    # Act Balancing
    if (balance == "g1") {
      ar.g1 <- act.rate
      ar.g2 <- ar.g1 * num.g1 / num.g2
    }
    if (balance == "g2") {
      ar.g2 <- act.rate.g2
      ar.g1 <- ar.g2 * num.g2 / num.g1
    }

    # Group Lambdas
    lambda.g1 <- inf.prob * ar.g1 * i.num.g2 / num.g2
    lambda.g2 <- inf.prob.g2 * ar.g2 * i.num / num.g1
    if (!is.null(parms$inter.eff) && t >= inter.start) {
      lambda.g1 <- lambda.g1 * (1 - inter.eff)
      lambda.g2 <- lambda.g2 * (1 - inter.eff)
    }

    # Flows
    si.flow <- lambda.g1 * s.num
    si.flow.g2 <- lambda.g2 * s.num.g2
    ir.flow <- rec.rate * i.num
    ir.flow.g2 <- rec.rate.g2 * i.num.g2

    # ODEs
    dSm1 <- -si.flow
    dIm1 <- si.flow - ir.flow
    dRm1 <- ir.flow
    dSm2 <- -si.flow.g2
    dIm2 <- si.flow.g2 - ir.flow.g2
    dRm2 <- ir.flow.g2

    # Output
    list(c(dSm1, dIm1, dRm1, dSm2, dIm2, dRm2,
           si.flow, ir.flow, si.flow.g2, ir.flow.g2),
         num = num.g1,
         num.g2 = num.g2)
  })
}


# SIR, 2 group, open pop --------------------------------------------------
#' @rdname dcm.mods
#' @keywords internal
#' @export
mod_SIR_2g_op <- function(t, t0, parms) {
  with(as.list(c(t0, parms)), {

    # Derivations
    num.g1 <- s.num + i.num + r.num
    num.g2 <- s.num.g2 + i.num.g2 + r.num.g2

    # Act Balancing
    if (balance == "g1") {
      ar.g1 <- act.rate
      ar.g2 <- ar.g1 * num.g1 / num.g2
    }
    if (balance == "g2") {
      ar.g2 <- act.rate.g2
      ar.g1 <- ar.g2 * num.g2 / num.g1
    }

    # Group Lambdas
    lambda.g1 <- inf.prob * ar.g1 * i.num.g2 / num.g2
    lambda.g2 <- inf.prob.g2 * ar.g2 * i.num / num.g1
    if (!is.null(parms$inter.eff) && t >= inter.start) {
      lambda.g1 <- lambda.g1 * (1 - inter.eff)
      lambda.g2 <- lambda.g2 * (1 - inter.eff)
    }

    # Flows
    si.flow <- lambda.g1 * s.num
    si.flow.g2 <- lambda.g2 * s.num.g2
    ir.flow <- rec.rate * i.num
    ir.flow.g2 <- rec.rate.g2 * i.num.g2
    if (is.na(a.rate.g2)) {
      a.flow <- a.rate * num.g1
      a.flow.g2 <- a.rate * num.g1
    } else {
      a.flow <- a.rate * num.g1
      a.flow.g2 <- a.rate.g2 * num.g2
    }
    ds.flow <- ds.rate * s.num
    ds.flow.g2 <- ds.rate.g2 * s.num.g2
    di.flow <- di.rate * i.num
    di.flow.g2 <- di.rate.g2 * i.num.g2
    dr.flow <- dr.rate * r.num
    dr.flow.g2 <- dr.rate.g2 * r.num.g2

    # ODEs
    dSm1 <- -si.flow + a.flow - ds.flow
    dIm1 <- si.flow - ir.flow - di.flow
    dRm1 <- ir.flow - dr.flow
    dSm2 <- -si.flow.g2 + a.flow.g2 - ds.flow.g2
    dIm2 <- si.flow.g2 - ir.flow.g2 - di.flow.g2
    dRm2 <- ir.flow.g2 - dr.flow.g2

    # Output
    list(c(dSm1, dIm1, dRm1, dSm2, dIm2, dRm2,
           si.flow, ir.flow, a.flow, ds.flow, di.flow, dr.flow,
           si.flow.g2, ir.flow.g2, a.flow.g2, ds.flow.g2,
           di.flow.g2, dr.flow.g2),
         num = num.g1, num.g2 = num.g2)
  })
}


# SIS, 1 group, closed pop ------------------------------------------------
#' @rdname dcm.mods
#' @keywords internal
#' @export
mod_SIS_1g_cl <- function(t, t0, parms) {
  with(as.list(c(t0, parms)), {

    # Derivations
    num <- s.num + i.num

    # Parameters
    lambda <- inf.prob * act.rate * i.num / num
    if (!is.null(parms$inter.eff) && t >= inter.start) {
      lambda <- lambda * (1 - inter.eff)
    }

    # Flows
    si.flow <- lambda * s.num
    is.flow <- rec.rate * i.num

    # ODEs
    dS <- -si.flow + is.flow
    dI <- si.flow - is.flow

    # Output
    list(c(dS, dI, si.flow, is.flow),
         num = num)
  })
}


# SIS, 1 group, open pop --------------------------------------------------
#' @rdname dcm.mods
#' @keywords internal
#' @export
mod_SIS_1g_op <- function(t, t0, parms) {
  with(as.list(c(t0, parms)), {

    # Derivations
    num <- s.num + i.num

    # Parameters
    lambda <- inf.prob * act.rate * i.num / num
    if (!is.null(parms$inter.eff) && t >= inter.start) {
      lambda <- lambda * (1 - inter.eff)
    }

    # Flows
    si.flow <- lambda * s.num
    is.flow <- rec.rate * i.num
    a.flow <- a.rate * num
    ds.flow <- ds.rate * s.num
    di.flow <- di.rate * i.num

    # ODEs
    dS <- -si.flow + is.flow + a.flow - ds.flow
    dI <- si.flow - is.flow - di.flow

    # Output
    list(c(dS, dI, si.flow, is.flow, a.flow, ds.flow, di.flow),
         num = num)
  })
}


# SIS, 2 group, closed pop ------------------------------------------------
#' @rdname dcm.mods
#' @keywords internal
#' @export
mod_SIS_2g_cl <- function(t, t0, parms) {
  with(as.list(c(t0, parms)), {

    # Derivations
    num.g1 <- s.num + i.num
    num.g2 <- s.num.g2 + i.num.g2

    # Act Balancing
    if (balance == "g1") {
      ar.g1 <- act.rate
      ar.g2 <- ar.g1 * num.g1 / num.g2
    }
    if (balance == "g2") {
      ar.g2 <- act.rate.g2
      ar.g1 <- ar.g2 * num.g2 / num.g1
    }

    # Group Lambdas
    lambda.g1 <- inf.prob * ar.g1 * i.num.g2 / num.g2
    lambda.g2 <- inf.prob.g2 * ar.g2 * i.num / num.g1
    if (!is.null(parms$inter.eff) && t >= inter.start) {
      lambda.g1 <- lambda.g1 * (1 - inter.eff)
      lambda.g2 <- lambda.g2 * (1 - inter.eff)
    }

    # Flows
    si.flow <- lambda.g1 * s.num
    is.flow <- rec.rate * i.num
    si.flow.g2 <- lambda.g2 * s.num.g2
    is.flow.g2 <- rec.rate.g2 * i.num.g2

    # ODEs
    dSm1 <- -si.flow + is.flow
    dIm1 <-  si.flow - is.flow
    dSm2 <- -si.flow.g2 + is.flow.g2
    dIm2 <-  si.flow.g2 - is.flow.g2

    # Output
    list(c(dSm1, dIm1, dSm2, dIm2,
           si.flow, is.flow, si.flow.g2, is.flow.g2),
         num = num.g1, num.g2 = num.g2)
  })
}


# SIS, 2 group, open pop --------------------------------------------------
#' @rdname dcm.mods
#' @keywords internal
#' @export
mod_SIS_2g_op <- function(t, t0, parms) {
  with(as.list(c(t0, parms)), {

    # Derivations
    num.g1 <- s.num + i.num
    num.g2 <- s.num.g2 + i.num.g2

    # Act Balancing
    if (balance == "g1") {
      ar.g1 <- act.rate
      ar.g2 <- ar.g1 * num.g1 / num.g2
    }
    if (balance == "g2") {
      ar.g2 <- act.rate.g2
      ar.g1 <- ar.g2 * num.g2 / num.g1
    }

    # Group Lambdas
    lambda.g1 <- inf.prob * ar.g1 * i.num.g2 / num.g2
    lambda.g2 <- inf.prob.g2 * ar.g2 * i.num / num.g1
    if (!is.null(parms$inter.eff) && t >= inter.start) {
      lambda.g1 <- lambda.g1 * (1 - inter.eff)
      lambda.g2 <- lambda.g2 * (1 - inter.eff)
    }

    # Flows
    si.flow <- lambda.g1 * s.num
    si.flow.g2 <- lambda.g2 * s.num.g2
    is.flow <- rec.rate * i.num
    is.flow.g2 <- rec.rate.g2 * i.num.g2
    if (is.na(a.rate.g2)) {
      a.flow <- a.rate * num.g1
      a.flow.g2 <- a.rate * num.g1
    } else {
      a.flow <- a.rate * num.g1
      a.flow.g2 <- a.rate.g2 * num.g2
    }
    ds.flow <- ds.rate * s.num
    ds.flow.g2 <- ds.rate.g2 * s.num.g2
    di.flow <- di.rate * i.num
    di.flow.g2 <- di.rate.g2 * i.num.g2

    # ODEs
    dSm1 <- -si.flow + is.flow + a.flow - ds.flow
    dIm1 <-  si.flow - is.flow - di.flow
    dSm2 <- -si.flow.g2 + is.flow.g2 + a.flow.g2 - ds.flow.g2
    dIm2 <-  si.flow.g2 - is.flow.g2 - di.flow.g2

    # Output
    list(c(dSm1, dIm1, dSm2, dIm2,
           si.flow, is.flow, a.flow, ds.flow, di.flow,
           si.flow.g2, is.flow.g2, a.flow.g2, ds.flow.g2, di.flow.g2),
         num = num.g1, num.g2 = num.g2)
  })
}

#' @title Convert transmat Infection Tree into a phylo Object
#'
#' @description Converts a transmission matrix from the `get_transmat`
#'              function into a `phylo` class object.
#'
#' @param x An object of class `transmat`, the output from
#'        [get_transmat()].
#' @param vertex.exit.times Optional numeric vector providing the time of
#'        departure of vertices, to be used to scale the lengths of branches
#'        reaching to the tips. Index position on vector corresponds to network
#'        id. NA indicates no departure, so branch will extend to the end of the
#'        tree.
#' @param ... Further arguments (unused).
#'
#' @details
#' Converts a [transmat()] object containing information about the
#' history of a simulated infection into a [`ape::phylo`] object
#' representation suitable for plotting as a tree with
#' [ape::plot.phylo()]. Each infection event becomes a 'node'
#' (horizontal branch) in the resulting `phylo` tree, and each network
#' vertex becomes a 'tip' of the tree. The infection events are labeled with the
#' vertex ID of the infector to make it possible to trace the path of infection.
#'
#' The infection timing information is included to position the phylo-nodes,
#' with the lines to the tips drawn to the max time value +1 (unless
#' `vertex.exit.times` are passed in it effectively assumes all vertices
#' are active until the end of the simulation).
#'
#' If the `transmat` contains multiple infection seeds (there are multiple
#' trees with separate root nodes), this function will return a list of class
#' `multiPhylo`, each element of which is a `phylo` object. See
#' [ape::read.tree()].
#'
#' @return A `phylo` class object.
#'
#' @importFrom ape as.phylo
#' @export as.phylo.transmat
#'
#' @examples
#' \donttest{
#' set.seed(13)
#'
#' # Fit a random mixing TERGM with mean degree of 1 and mean edge
#' # duration of 20 time steps
#' nw <- network_initialize(n = 100)
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#' est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' # Parameterize the epidemic model as SI with one infected seed
#' param <- param.net(inf.prob = 0.5)
#' init <- init.net(i.num = 1)
#' control <- control.net(type = "SI", nsteps = 40, nsims = 1, verbose = FALSE)
#'
#' # Simulate the model
#' mod1 <- netsim(est, param, init, control)
#'
#' # Extract the transmission matrix
#' tm <- get_transmat(mod1)
#' head(tm, 15)
#'
#' # Convert to phylo object and plot
#' tmPhylo <- as.phylo.transmat(tm)
#' par(mar = c(1,1,1,1))
#' plot(tmPhylo, show.node.label = TRUE,
#'               root.edge = TRUE,
#'               cex = 0.75)
#' }
#'
as.phylo.transmat <- function(x, vertex.exit.times = NULL, ...) {

  # if not named properly, assume inf, sus at
  if (!all(c("inf", "sus", "at") %in% names(x))) {
    warning("input does not have appropriate column names for transmat,
            assuming first 3 should be 'inf','sus','at'")
    names(x) <- c("inf", "sus", "at")
  }
  tm <- x
  # find roots (infectors that never appear as sus)
  v <- setdiff(unique(tm$inf), unique(tm$sus))
  if (length(v) > 1) {
    message("found multiple trees, returning a list of ", length(v),
            "phylo objects")
    # need to extract the portions of the edgelist and call seperately
    sub_phylos <- lapply(v, function(v_sub) {
      # walk down the list to find elements below v_sub
      sub_rows <- which(tm$inf == v_sub)
      toFind <- v_sub
      while (length(toFind) > 0) {
        i <- toFind[1]
        sub_rows <- unique(c(sub_rows, which(tm$inf == i)))
        toFind <- c(toFind[-1], tm$sus[which(tm$inf == i)])
      }
      # call as.phylo on the subset of the edgelist
      as.phylo.transmat(tm[sub_rows, , drop = FALSE],
                        vertex.exit.times = vertex.exit.times)

    })
    names(sub_phylos) <- paste("seed", v, sep = "_")
    class(sub_phylos) <- c("multiPhylo", class(sub_phylos))
    return(sub_phylos)
  }

  el <- cbind(tm$inf, tm$sus)
  origNodes <- unique(as.vector(el))
  # if vertex.exit.times included check that it is consistant
  if (!is.null(vertex.exit.times)) {
    if (length(origNodes) > length(vertex.exit.times) ||
          any(origNodes > length(vertex.exit.times))) {
      stop("Vertex ids in edgelist imply a larger network size than
           vertex.exit.times")
    }
  }
  # translate ids in el to sequential integers starting from one
  el[, 1] <- match(el[, 1], origNodes)
  el[, 2] <- match(el[, 2], origNodes)

  maxTip <- max(el)  # need to know what phylo node ids will start

  maxTime <- max(x$at) + 1
  if (!is.null(vertex.exit.times)) {
    maxTime <- max(maxTime, vertex.exit.times, na.rm = TRUE)
  }
  # create new ids for phyloNodes
  phyloNodes <- seq(from = maxTip + 1, length.out = length(origNodes) - 1)
  Nnode <- length(phyloNodes)
  # create labels for each phylo node based on infector id
  phylo.label <- tm$inf

  # set default durations
  # since we don't know how long the graph vertices live, assume entire duration
  durations <- rep(NA, length(phyloNodes) * 2)
  tipExitTimes <- rep(maxTime, maxTip)
  if (!is.null(vertex.exit.times)) {
    # replace any NA values with max time
    vertex.exit.times[is.na(vertex.exit.times)] <- maxTime + 1
    # copy the vertex exit times into the appropriate positions in the
    # durations array
    durations[seq_len(maxTip)] <- vertex.exit.times[origNodes]
    # reorder the vertex.exit times to match new ids of tips
    tipExitTimes <- vertex.exit.times[origNodes]
  }

  # create a new edgelist by stepping through the existing edgelist
  # and creating the new links from phylo nodes to graph vertices (tips)
  # and from phylo node to phylo node
  # have to do this as progressive modifications

  # assume at least one xmit has occured
  # create the phylo node linking to the first
  # infector and infectee
  phyloEl <- rbind(cbind(phyloNodes[1], el[1, 1]),
                   cbind(phyloNodes[1], el[1, 2]))

  durations[1] <- tipExitTimes[el[1, 1]] - tm[["at"]][1]
  durations[2] <- tipExitTimes[el[1, 2]] - tm[["at"]][1]

  phyloN <- 1
  # loop over remaining rows
  if (nrow(el) > 1) {
    for (r in 2:nrow(el)) {
      # find id of infector
      infector <- el[r, 1]
      # find the phylo row of phylo node corresponding to the infector
      phyNRow <- which(phyloEl[, 2] == infector)
      # replace the infector with a new phylo node
      phyloEl[phyNRow, 2] <- phyloNodes[phyloN + 1]
      # link the new phylo node to the infector
      phyloEl <- rbind(phyloEl, cbind(phyloNodes[phyloN + 1], infector))
      # link the new phylo node to the infectee (tip)
      phyloEl <- rbind(phyloEl, cbind(phyloNodes[phyloN + 1], el[r, 2]))

      # update the timing on the replaced row that linked to tip
      durations[phyNRow] <-
        durations[phyNRow] - (tipExitTimes[infector] - tm[["at"]][r])
      # add timings for new rows equal to remaining time
      # infector
      durations[nrow(phyloEl) - 1] <- tipExitTimes[infector] - tm[["at"]][r]
      # infectee
      durations[nrow(phyloEl)] <- tipExitTimes[el[r, 2]] - tm[["at"]][r]


      # increment the phylo node counter
      phyloN <- phyloN + 1
    }
  }

  # format the output
  out <- list()
  out[["edge"]] <- phyloEl
  out[["Nnode"]] <- Nnode  # number of non-tip nodes
  out[["tip.label"]] <- origNodes
  out[["node.label"]] <- phylo.label
  out[["root.edge"]] <- x$at[1] # have to assume sim started at 0
  out[["edge.length"]] <- durations

  class(out) <- "phylo"
  return(out)
}

#' @title Convert transmat Infection Tree into a network Object
#'
#' @description Converts a transmission matrix from the `get_transmat`
#'              function into a [`network::network`] class object.
#'
#' @param x An object of class `transmat` to be converted into a
#'          `network` class object.
#' @param ... Unused.
#'
#' @details
#' When converting from a `transmat` to a `network` object, this
#' functions copies the edge attributes within the transmission matrix
#' (`'at'`, `'infDur'`, `'transProb'`, `'actRate'`, and
#' `'finalProb'`) into edge attributes on the network.
#'
#' @return A [`network::network`] object.
#'
#' @method as.network transmat
#' @export
#'
as.network.transmat <- function(x, ...) {
  tm <- x
  ids <- unique(c(tm$sus, tm$inf))

  # remap the ids to a new set
  tm$sus <- match(tm$sus, ids)
  tm$inf <- match(tm$inf, ids)
  net <- network(cbind(tm$inf, tm$sus), matrix.type = "edgelist")
  network.vertex.names(net) <- ids

  # add the other attributes to edges and vertices
  set.edge.attribute(net, "at", tm$at)
  set.edge.attribute(net, "infDur", tm$infDur)
  set.edge.attribute(net, "transProb", tm$transProb)
  set.edge.attribute(net, "actRate", tm$actRate)
  set.edge.attribute(net, "finalProb", tm$finalProb)

  net %v% "at" <- 0
  net <- set_vertex_attribute(net, "at", tm$at, v = tm$sus)

  return(net)
}

#' @title Plot transmat Infection Tree in Three Styles
#'
#' @description Plots the transmission matrix tree from from `get_transmat`
#'              in one of three styles: a phylogram, a directed network, or
#'              a transmission timeline.
#'
#' @param x A [transmat()] object to be plotted.
#' @param style Character name of plot style. One of `"phylo"`,
#'              `"network"`, or `"transmissionTimeline"`.
#' @param ... Additional plot arguments to be passed to lower-level plot
#'            functions (`plot.network`, `plot.phylo`, or
#'            `transmissionTimeline`).
#'
#' @details
#' The `phylo` plot requires the `ape` package. The
#' `transmissionTimeline` plot requires that the `ndtv` package.
#'
#' @export
#' @method plot transmat
#'
#' @seealso [`network::plot.network`], [ape::plot.phylo()],
#'          [ndtv::transmissionTimeline()].
#'
plot.transmat <- function(x,
                          style = c("phylo", "network", "transmissionTimeline"),
                          ...) {

  style <- match.arg(style)

  switch(style,
    "transmissionTimeline" = tm_transsmissionTree_plot(x, ...),
    "network" = plot.network(as.network(x), ...),
    "phylo" = plot(as.phylo.transmat(x),
                   show.node.label = TRUE,
                   root.edge = TRUE,
                   label.offset = 0.1,
                   ...)
  )

}

#' @export
#' @aliases transmat
#' @rdname get_transmat
is.transmat <- function(x) {
  if (inherits(x, "transmat")) {
    return(TRUE)
  } else {
    return(FALSE)
  }
}

# this is a wrapper to load the namespace and call the transmissionTimeline plot
tm_transsmissionTree_plot <- function(x, ...) {
  requireNamespace("ndtv")
  ndtv::transmissionTimeline(x, ...)
}

#' @title Epidemic Parameters for Stochastic Network Models
#'
#' @description Sets the epidemic parameters for stochastic network models
#'              simulated with [netsim()].
#'
#' @param inf.prob Probability of infection per transmissible act between
#'        a susceptible and an infected person. In two-group models, this is the
#'        probability of infection to the group 1 nodes. This may also be a
#'        vector of probabilities, with each element corresponding to the
#'        probability in that time step of infection (see Time-Varying
#'        Parameters below).
#' @param inter.eff Efficacy of an intervention which affects the per-act
#'        probability of infection. Efficacy is defined as 1 - the relative
#'        hazard of infection given exposure to the intervention, compared to no
#'        exposure.
#' @param inter.start Time step at which the intervention starts, between 1 and
#'        the number of time steps specified in the model. This will default to
#'        1 if `inter.eff` is defined but this parameter is not.
#' @param act.rate Average number of transmissible acts *per partnership*
#'        per unit time (see `act.rate` Parameter below). This may also be
#'        a vector of rates, with each element corresponding to the rate in
#'        that time step of infection (see Time-Varying Parameters below).
#' @param rec.rate Average rate of recovery with immunity (in `SIR` models)
#'        or re-susceptibility (in `SIS` models). The recovery rate is the
#'        reciprocal of the disease duration. For two-group models, this is the
#'        recovery rate for group 1 persons only. This parameter is only used
#'        for `SIR` and `SIS` models. This may also be a vector
#'        of rates, with each element corresponding to the rate in that time
#'        step of infection (see Time-Varying Parameters below).
#' @param a.rate Arrival or entry rate. For one-group models, the arrival rate
#'        is the rate of new arrivals per person per unit time. For two-group
#'        models, the arrival rate is parameterized as a rate per group 1
#'        person per unit time, with the `a.rate.g2` rate set as described
#'        below.
#' @param ds.rate Departure or exit rate for susceptible persons. For two-group
#'        models, it is the rate for group 1 susceptible persons only.
#' @param di.rate Departure or exit rate for infected persons. For two-group
#'        models, it is the rate for group 1 infected persons only.
#' @param dr.rate Departure or exit rate for recovered persons. For two-group
#'        models, it is the rate for group 1 recovered persons only. This
#'        parameter is only used for `SIR` models.
#' @param inf.prob.g2 Probability of transmission given a transmissible act
#'        between a susceptible group 2 person and an infected group 1 person.
#'        It is the probability of transmission to group 2 members.
#' @param rec.rate.g2 Average rate of recovery with immunity (in `SIR`
#'        models) or re-susceptibility (in `SIS` models) for group 2
#'        persons. This parameter is only used for two-group `SIR` and
#'        `SIS` models.
#' @param a.rate.g2 Arrival or entry rate for group 2. This may either be
#'        specified numerically as the rate of new arrivals per group 2 person
#'        per unit time, or as `NA`, in which case the group 1 rate,
#'        `a.rate`, governs the group 2 rate. The latter is used when, for
#'        example, the first group is conceptualized as female, and the female
#'        population size determines the arrival rate. Such arrivals are evenly
#'        allocated between the two groups.
#' @param ds.rate.g2 Departure or exit rate for group 2 susceptible persons.
#' @param di.rate.g2 Departure or exit rate for group 2 infected persons.
#' @param dr.rate.g2 Departure or exit rate for group 2 recovered persons. This
#'        parameter is only used for `SIR` model types.
#'
#' @param ... Additional arguments passed to model.
#'
#' @details
#' `param.net` sets the epidemic parameters for the stochastic network
#' models simulated with the [netsim()] function. Models
#' may use the base types, for which these parameters are used, or new process
#' modules which may use these parameters (but not necessarily). A detailed
#' description of network model parameterization for base models is found in
#' the [Network Modeling for Epidemics](https://epimodel.github.io/sismid/)
#' tutorials.
#'
#' For base models, the model specification will be chosen as a result of
#' the model parameters entered here and the control settings in
#' [control.net()]. One-group and two-group models are available,
#' where the latter assumes a heterogeneous mixing between two distinct
#' partitions in the population (e.g., men and women). Specifying any two-group
#' parameters (those with a `.g2`) implies the simulation of a two-group
#' model. All the parameters for a desired model type must be specified, even if
#' they are zero.
#'
#' @section The `act.rate` Parameter:
#' A key difference between these network models and DCM/ICM classes is the
#' treatment of transmission events. With DCM and ICM, contacts or partnerships
#' are mathematically instantaneous events: they have no duration in time, and
#' thus no changes may occur within them over time. In contrast, network models
#' allow for partnership durations defined by the dynamic network model,
#' summarized in the model dissolution coefficients calculated in
#' [dissolution_coefs()]. Therefore, the `act.rate` parameter has
#' a different interpretation here, where it is the number of transmissible acts
#' *per partnership* per unit time.
#'
#' @section Time-Varying Parameters:
#' The `inf.prob`, `act.rate`, `rec.rate` arguments (and their
#' `.g2` companions) may be specified as time-varying parameters by passing
#' in a vector of probabilities or rates, respectively. The value in each
#' position on the vector then corresponds to the probability or rate at that
#' discrete time step for the infected partner. For example, an `inf.prob`
#' of `c(0.5, 0.5, 0.1)` would simulate a 0.5 transmission probability for
#' the first two time steps of a person's infection, followed by a 0.1 for the
#' third time step. If the infected person has not recovered or exited the
#' population by the fourth time step, the third element in the vector will
#' carry forward until one of those events occurs or the simulation ends. For
#' further examples, see the
#' [Network Modeling for Epidemics](https://epimodel.github.io/sismid/) tutorials.
#'
#' @section Random Parameters:
#' In addition to deterministic parameters in either fixed or time-varying
#' varieties above, one may also include a generator for random parameters.
#' These might include a vector of potential parameter values or a statistical
#' distribution definition; in either case, one draw from the generator would
#' be completed per individual simulation. This is possible by passing a list
#' named `random.params` into `param.net`, with each element of
#' `random.params` a named generator function. See the help page and
#' examples in [generate_random_params()]. A simple factory function
#' for sampling is provided with [param_random()] but any function
#' will do.
#'
#' @section Using a Parameter data.frame:
#' It is possible to set input parameters using a specifically formatted
#' `data.frame` object. The first 3 columns of this `data.frame` must
#' be:
#'
#'  * `param`: The name of the parameter. If this is a non-scalar
#'    parameter (a vector of length > 1), end the parameter name with the
#'    position on the vector (e.g., `"p_1"`, `"p_2"`, ...).
#'  * `value`: the value for the parameter (or the value of the
#'    parameter in the Nth position if non-scalar).
#'  * `type`: a character string containing either `"numeric"`,
#'    `"logical"`, or `"character"` to define the parameter object
#'    class.
#'
#'
#' In addition to these 3 columns, the `data.frame` can contain any number
#' of other columns, such as `details` or `source` columns to document
#' parameter meta-data. However, these extra columns will not be used by
#' EpiModel.
#'
#' This data.frame is then passed in to `param.net` under a
#' `data.frame.parameters` argument. Further details and examples are
#' provided in the "Working with Model Parameters in EpiModel" vignette.
#'
#' @section Parameters with New Modules:
#' To build original models outside of the base models, new process modules
#' may be constructed to replace the existing modules or to supplement the
#' existing set. These are passed into the control settings in
#' [control.net()]. New modules may use either the existing model
#' parameters named here, an original set of parameters, or a combination of
#' both. The `...` allows the user to pass an arbitrary set of original
#' model parameters into `param.net`. Whereas there are strict checks with
#' default modules for parameter validity, this becomes a user
#' responsibility when using new modules.
#'
#' @return An `EpiModel` object of class `param.net`.
#'
#' @seealso Use [init.net()] to specify the initial conditions and
#'          [control.net()] to specify the control settings. Run the
#'          parameterized model with [netsim()].
#'
#' @keywords parameterization
#'
#' @export
#'
#' @examples
#' \donttest{
#' ## Example SIR model parameterization with fixed and random parameters
#' # Network model estimation
#' nw <- network_initialize(n = 100)
#' formation <- ~edges
#' target.stats <- 50
#' coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
#' est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
#'
#' # Random epidemic parameter list (here act.rate values are sampled uniformly
#' # with helper function param_random, and inf.prob follows a general Beta
#' # distribution with the parameters shown below)
#' my_randoms <- list(
#'   act.rate = param_random(1:3),
#'   inf.prob = function() rbeta(1, 1, 2)
#' )
#'
#' # Parameters, initial conditions, and control settings
#' param <- param.net(rec.rate = 0.02, random.params = my_randoms)
#'
#' # Printing parameters shows both fixed and and random parameter functions
#' param
#'
#' # Set initial conditions and controls
#' init <- init.net(i.num = 10, r.num = 0)
#' control <- control.net(type = "SIR", nsteps = 10, nsims = 3, verbose = FALSE)
#'
#' # Simulate the model
#' sim <- netsim(est, param, init, control)
#'
#' # Printing the sim object shows the randomly drawn values for each simulation
#' sim
#'
#' # Parameter sets can be extracted with:
#' get_param_set(sim)
#' }
#'
param.net <- function(inf.prob, inter.eff, inter.start, act.rate, rec.rate,
                      a.rate, ds.rate, di.rate, dr.rate, inf.prob.g2,
                      rec.rate.g2, a.rate.g2, ds.rate.g2, di.rate.g2,
                      dr.rate.g2, ...) {


  # Get arguments
  dot.args <- list(...)
  names.dot.args <- names(dot.args)

  # Use "data.frame.params" as default if available
  if ("data.frame.params" %in% names.dot.args) {
    p <- param.net_from_table(dot.args[["data.frame.params"]])
    dot.args[["data.frame.params"]] <- NULL
    names.dot.args <- names(dot.args)
  } else {
    p <- list()
  }

  formal.args <- formals(sys.function())
  formal.args[["..."]] <- NULL
  for (arg in names(formal.args)) {
    if (as.logical(mget(arg) != "")) {
      p[arg] <- list(get(arg))
    }
  }
  if (length(dot.args) > 0) {
    for (i in seq_along(dot.args)) {
      p[[names.dot.args[i]]] <- dot.args[[i]]
    }
  }

  ## random.params checks
  if ("random.params" %in% names.dot.args) {
    for (nm in names(p[["random.params"]])) {
      if (nm %in% names(p)) {
        warning(
          "The parameter `", nm, "` is defined twice, once as fixed",
          " and once as a random parameter.\n Only the random parameter",
          " definition will be used."
        )
      }
    }
  }

  ## Defaults and Checks
  # Check for old .m2 parameter suffix
  m2.flag <- grep(".m2", names(p))
  if (length(m2.flag) > 0) {
    stop("Parameters using the .m2 suffix have been removed. ",
         "Use the .g2 suffix instead (e.g., inf.prob.g2).")
  }
  if (missing(act.rate)) {
    p[["act.rate"]] <- 1
  }
  p[["vital"]] <- ifelse(!missing(a.rate) | !missing(ds.rate) |
                           !missing(di.rate) | !missing(dr.rate), TRUE, FALSE)
  if ("act.rate.g2" %in% names.dot.args) {
    warning("act.rate.g2 parameter was entered. ",
            "If using built-in models, only act.rate parameter will apply.")
  }

  if (!is.null(p[["inter.eff"]]) && is.null(p[["inter.start"]])) {
    p[["inter.start"]] <- 1
  }

  ## Output
  class(p) <- c("param.net", "list")
  return(p)
}

#' @title Update Model Parameters for Stochastic Network Models
#'
#' @description Updates epidemic model parameters originally set with
#'              [param.net()] and adds new parameters.
#'
#' @param param Object of class `param.net`, output from function of same
#'              name.
#' @param new.param.list Named list of new parameters to add to original
#'        parameters.
#'
#' @details
#' This function can update any original parameters specified with
#' [param.net()] and add new parameters. This function would be used
#' if the inputs to [param.net()] were a long list of fixed model
#' parameters that needed supplemental replacements or additions for particular
#' model runs (e.g., changing an intervention efficacy parameter but leaving all
#' other parameters fixed).
#'
#' The `new.param.list` object should be a named list object containing
#' named parameters matching those already in `x` (in which case those
#' original parameter values will be replaced) or not matching (in which case
#' new parameters will be added to `param`).
#'
#' `update_params` modifies a `param.net` object **before** it is passed
#' to [netsim()]; it is the recommended helper for tweaking parameters
#' outside of a running simulation. To modify parameters from *inside* a
#' module function during a simulation, use [set_param()] on the live
#' `dat` object instead. There is no analogous `update_controls()` or
#' `update_inits()` helper: for a `control.net` or `init.net` object,
#' edit the list directly (e.g., `ctrl$nsteps <- 1000`) or rebuild it
#' with a fresh call to the constructor. For *scheduled* mid-simulation
#' changes to parameters or control settings, see the
#' `.param.updater.list` argument to [param.net()] and the
#' `.control.updater.list` argument to [control.net()].
#'
#' @return
#' An updated list object of class `param.net`, which can be passed to the
#' EpiModel function [netsim()].
#'
#' @seealso [set_param()] for editing parameters inside a module function
#'   during a simulation; [param.net()] for the parameter constructor;
#'   [netsim()] for running a simulation.
#'
#' @examples
#' x <- param.net(inf.prob = 0.5, act.rate = 2)
#' y <- list(inf.prob = 0.75, dx.rate = 0.2)
#' z <- update_params(x, y)
#' print(z)
#'
#' @export
#'
update_params <- function(param, new.param.list) {

  if (!inherits(param, "param.net")) {
    stop("x should be object of class param.net")
  }
  if (!inherits(new.param.list, "list")) {
    stop("new.param.list should be object of class list")
  }

  for (ii in seq_along(new.param.list)) {
    param[[names(new.param.list)[ii]]] <- new.param.list[[ii]]
  }

  return(param)
}


#' @title Create a Value Sampler for Random Parameters
#'
#' @description This function returns a 0 argument function that can be used as
#'   a generator function in the `random.params` argument of the
#'   [param.net()] function.
#'
#' @param values A vector of values to sample from.
#' @param prob A vector of weights to use during sampling. If `NULL`,
#'        all values have the same probability of being picked
#'        (default = `NULL`).
#'
#' @return A 0 argument generator function to sample one of the values from the
#' `values` vector.
#'
#' @seealso [param.net()] and [generate_random_params()]
#' @export
#'
#' @examples
#' # Define function with equal sampling probability
#' a <- param_random(1:5)
#' a()
#'
#' # Define function with unequal sampling probability
#' b <- param_random(1:5, prob = c(0.1, 0.1, 0.1, 0.1, 0.6))
#' b()
#'
param_random <- function(values, prob = NULL) {
  if (!is.null(prob) && length(prob) != length(values)) {
    stop("incorrect number of probabilites")
  }

  f <- function() {
    return(sample(x = values, size = 1, prob = prob, replace = TRUE))
  }

  return(f)
}


#' @title Generate Values for Random Parameters
#'
#' @description This function uses the generative functions in the
#'              `random.params` list to create values for the parameters.
#'
#' @param param The `param` argument received by the `netsim`
#'              functions.
#' @param verbose Should the function output the generated values
#'                (default = FALSE)?
#'
#' @return A fully instantiated `param` list.
#'

#' @section `random.params`:
#' The `random.params` argument to the [param.net()] function
#' must be a named list of functions that each return a value that can be used
#' as the argument with the same name. In the example below, `param_random`
#' is a function factory provided by EpiModel for `act.rate` and
#' for `tx.halt.part.prob` we provide bespoke functions. A function factory
#' is a function that returns a new function
#' (see https://adv-r.hadley.nz/function-factories.html).
#'
#' @section Generator Functions:
#' The functions used inside `random_params` must be 0 argument functions
#' returning a valid value for the parameter with the same name.
#'
#' @section `param_random_set`:
#' The `random_params` list can optionally contain a
#' `param_random_set` element. It must be a `data.frame` of possible
#' values to be used as parameters.
#'
#' The column names must correspond either to:
#' the name of one parameter, if this parameter is of size 1; or the name of one
#' parameter with "_1", "_2", etc. appended, with the number representing the
#' position of the value, if this parameter is of size > 1. This means that the
#' parameter names cannot contain any underscores "_" if you intend to use
#' `param_random_set`.
#'
#' The point of the `param.random.set` `data.frame` is to allow the
#' random parameters to be correlated. To achieve this, a whole row of the
#' `data.frame` is selected for each simulation.
#'
#' @export
#'
#' @examples
#' \dontrun{
#'
#' ## Example with only the generator function
#'
#' # Define random parameter list
#' my_randoms <- list(
#'   act.rate = param_random(c(0.25, 0.5, 0.75)),
#'   tx.prob = function() rbeta(1, 1, 2),
#'   stratified.test.rate = function() c(
#'     rnorm(1, 0.05, 0.01),
#'     rnorm(1, 0.15, 0.03),
#'     rnorm(1, 0.25, 0.05)
#'   )
#' )
#'
#' # Parameter model with fixed and random parameters
#' param <- param.net(inf.prob = 0.3, random.params = my_randoms)
#'
#' # Below, `tx.prob` is set first to 0.3 then assigned a random value using
#' # the function from `my_randoms`. A warning notifying of this overwrite is
#' # therefore produced.
#' param <- param.net(tx.prob = 0.3, random.params = my_randoms)
#'
#'
#' # Parameters are drawn automatically in netsim by calling the function
#' # within netsim_loop. Demonstrating draws here but this is not used by
#' # end user.
#' paramDraw <- generate_random_params(param, verbose = TRUE)
#' paramDraw
#'
#'
#' ## Addition of the `param.random.set` `data.frame`
#'
#' # This function will generate sets of correlated parameters
#'  generate_correlated_params <- function() {
#'    param.unique <- runif(1)
#'    param.set.1 <- param.unique + runif(2)
#'    param.set.2 <- param.unique * rnorm(3)
#'
#'    return(list(param.unique, param.set.1, param.set.2))
#'  }
#'
#'  # Data.frame set of random parameters :
#'  correlated_params <- t(replicate(10, unlist(generate_correlated_params())))
#'  correlated_params <- as.data.frame(correlated_params)
#'  colnames(correlated_params) <- c(
#'    "param.unique",
#'    "param.set.1_1", "param.set.1_2",
#'    "param.set.2_1", "param.set.2_2", "param.set.2_3"
#'  )
#'
#' # Define random parameter list with the `param.random.set` element
#' my_randoms <- list(
#'   act.rate = param_random(c(0.25, 0.5, 0.75)),
#'   param.random.set = correlated_params
#' )
#'
#' # Parameter model with fixed and random parameters
#' param <- param.net(inf.prob = 0.3, random.params = my_randoms)
#'
#' # Parameters are drawn automatically in netsim by calling the function
#' # within netsim_loop. Demonstrating draws here but this is not used by
#' # end user.
#' paramDraw <- generate_random_params(param, verbose = TRUE)
#' paramDraw
#'
#' }
generate_random_params <- function(param, verbose = FALSE) {
  if (is.null(param[["random.params"]]) ||
        length(param[["random.params"]]) == 0) {
    return(param)
  } else {
    random.params <- param[["random.params"]]
  }

  if (!is.list(random.params)) {
    stop("`random.params` must be named list of functions")
  }

  rng_names <- names(random.params)
  if (any(rng_names == "")) {
    stop("all elements of `random.params` must be named")
  }

  rng_values <- list()

  if ("param.random.set" %in% rng_names) {
    # Take `param.random.set` out of the `random.params` list
    param.random.set <- random.params[["param.random.set"]]
    random.params[["param.random.set"]] <- NULL
    rng_names <- names(random.params)

    if (!is.data.frame(param.random.set)) {
      stop("`param.random.set` must be a data.frame")
    }

    # Pick one row of the `data.frame`
    sampled.row <- sample.int(nrow(param.random.set), 1)

    # Convert to `param` format
    sampled.set <- unflatten_params(param.random.set[sampled.row, ])

    # Update `rng_values`
    rng_values <- update_list(rng_values, sampled.set)
  }

  if (!all(vapply(random.params, is.function, TRUE))) {
    stop("all elements of `random.params` must be functions \n",
         "(Except 'param.random.set')")
  }

  duplicated.rng <- names(rng_values) %in% rng_names
  if (any(duplicated.rng)) {
    warning("Some parameters are set to be randomly assigned twice: \n",
            paste0(rng_names[duplicated.rng], collapse = ", "), "\n\n",
            "The version from a generator function will be used")
  }

  rng_values[rng_names] <- lapply(random.params, do.call, args = list())
  param <- update_list(param, rng_values)

  param[["random.params.values"]] <- rng_values

  if (verbose == TRUE) {
    msg <-
      "The following values were randomly generated for the given parameters: \n"
    msg <- c(msg, paste0("`", names(rng_values), "`: ", rng_values, "\n"))
    message(msg)
  }

  return(param)
}


#' @title Initial Conditions for Stochastic Network Models
#'
#' @description Sets the initial conditions for stochastic network models
#'              simulated with `netsim`.
#'
#' @param i.num Number of initial infected persons. For two-group models, this
#'        is the number of initial group 1 infected persons.
#' @param r.num Number of initial recovered persons. For two-group models, this
#'        is the number of initial group 1 recovered persons. This parameter is
#'        only used for the `SIR` model type.
#' @param i.num.g2 Number of initial infected persons in group 2. This parameter
#'        is only used for two-group models.
#' @param r.num.g2 Number of initial recovered persons in group 2. This
#'        parameter is only used for two-group `SIR` models.
#' @param status.vector A vector of length equal to the size of the input
#'        network, containing the status of each node. Setting status here
#'        overrides any inputs passed in the `.num` arguments.
#' @param infTime.vector A vector of length equal to the size of the input
#'        network, containing the (historical) time of infection for each of
#'        those nodes with a current status of `"i"`. Can only be used if
#'        `status.vector` is used, and must contain `NA` values for
#'        any nodes whose status is not `"i"`.
#' @param ... Additional initial conditions passed to model.
#'
#' @details
#' The initial conditions for a model solved with [netsim()] should be
#' input into the `init.net` function. This function handles initial
#' conditions for both base models and new modules. For an overview of
#' specifying initial conditions across a variety of base network models,
#' consult the [Network Modeling for Epidemics](https://epimodel.github.io/sismid/)
#' tutorials.
#'
#' @return An `EpiModel` object of class `init.net`.
#'
#' @seealso Use [param.net()] to specify model parameters and
#'          [control.net()] to specify the control settings. Run the
#'          parameterized model with [netsim()].
#'
#' @keywords parameterization
#'
#' @export
#'
#' @examples
#' # Example of using status.vector and infTime.vector together
#' n <- 100
#' status <- sample(c("s", "i"), size = n, replace = TRUE, prob = c(0.8, 0.2))
#' infTime <- rep(NA, n)
#' infTime[which(status == "i")] <- -rgeom(sum(status == "i"), prob = 0.01) + 2
#'
#' init.net(status.vector = status, infTime.vector = infTime)
#'
init.net <- function(i.num, r.num, i.num.g2, r.num.g2,
                     status.vector, infTime.vector, ...) {

  # Get arguments
  p <- list()
  formal.args <- formals(sys.function())
  formal.args[["..."]] <- NULL
  for (arg in names(formal.args)) {
    if (as.logical(mget(arg) != "")) {
      p[arg] <- list(get(arg))
    }
  }
  dot.args <- list(...)
  names.dot.args <- names(dot.args)
  if (length(dot.args) > 0) {
    for (i in seq_along(dot.args)) {
      p[[names.dot.args[i]]] <- dot.args[[i]]
    }
  }

  ## Defaults and checks

  # Check for old .m2 initial condition suffix
  m2.init <- grep("m2", names(p), value = TRUE)
  if (length(m2.init) > 0) {
    stop("Initial conditions using the .m2 suffix have been removed. ",
         "Use the .g2 suffix instead (e.g., i.num.g2).")
  }
  if (!is.null(p[["i.num"]]) && !is.null(p[["status.vector"]])) {
    stop("Use i.num OR status.vector to set initial infected")
  }
  if (!is.null(p[["infTime.vector"]]) && is.null(p[["status.vector"]])) {
    stop("infTime.vector may only be used if status.vector is used")
  }
  if (!is.null(p[["infTime.vector"]]) &&
        length(p[["infTime.vector"]]) != length(p[["status.vector"]])) {
    stop("Length of infTime.vector must match length of status.vector")
  }


  ## Output
  class(p) <- c("init.net", "list")
  return(p)
}


#' @title Control Settings for Stochastic Network Models
#'
#' @description Sets the controls for stochastic network models simulated with
#'              [`netsim`].
#'
#' @param type Disease type to be modeled, with the choice of `"SI"` for Susceptible-Infected
#'        diseases, `"SIR"` for Susceptible-Infected-Recovered diseases, and `"SIS"` for
#'        Susceptible-Infected-Susceptible diseases.
#' @param nsteps Number of time steps to simulate the model over. This must be a positive integer
#'        that is equal to the final step of a simulation. If a simulation is restarted with `start`
#'        argument, this number must be at least one greater than that argument's value.
#' @param nsims The total number of disease simulations.
#' @param ncores Number of processor cores to run multiple simulations on, using
#'        the `future` framework with `multisession`. See `future.use.plan` for
#'        other `future` backends.
#' @param start For models with network resimulation, time point to start up the simulation. For
#'        restarted simulations, this must be one greater than the final time step in the prior
#'        simulation and must be less than the value in `nsteps`.
#' @param resimulate.network If `TRUE`, resimulate the network at each time step. This is required
#'        when the epidemic or demographic processes impact the network structure (e.g., vital
#'        dynamics). This parameter controls whether `resim_nets.FUN` performs actual network
#'        resimulation; it does not affect the order in which modules are executed (see
#'        `module.order`). Setting `tergmLite = TRUE` forces `resimulate.network = TRUE` with a
#'        warning.
#' @param tergmLite Logical indicating usage of either `tergm` (`tergmLite = FALSE`), or `tergmLite`
#'        (`tergmLite = TRUE`). Default of `FALSE`. When `TRUE`, `resimulate.network` is
#'        automatically set to `TRUE` (with a warning if the user explicitly set it to `FALSE`).
#' @param cumulative.edgelist If `TRUE`, calculates a cumulative edgelist within the network
#'        simulation module. This is used when tergmLite is used and the entire networkDynamic
#'        object is not used.
#' @param truncate.el.cuml Number of time steps of the cumulative edgelist to retain. See help for
#'        [`update_cumulative_edgelist`] for options.
#' @param attr.rules A list containing the  rules for setting the attributes of incoming nodes, with
#'        one list element per attribute to be set (see details below).
#' @param epi.by A character vector of length 1 containing a nodal attribute for which subgroup
#'        stratified prevalence summary statistics are calculated. This nodal attribute must be
#'        contained in the network model formation formula, otherwise it is ignored.
#' @param initialize.FUN Module to initialize the model at time 1, with the default function of
#'        [`initialize.net`].
#' @param departures.FUN Module to simulate departure or exit, with the default function of
#'        [`departures.net`].
#' @param arrivals.FUN Module to simulate arrivals or entries, with the default function of
#'        [`arrivals.net`].
#' @param recovery.FUN Module to simulate disease recovery, with the default function of
#'        [`recovery.net`].
#' @param resim_nets.FUN Module to resimulate the network at each time step, with the default
#'        function of [`resim_nets`].
#' @param summary_nets.FUN Module to extract summary statistics of the network
#'        at each time step, with the default function of [`summary_nets`].
#' @param infection.FUN Module to simulate disease infection, with the default function of
#'        [`infection.net`].
#' @param nwupdate.FUN Module to handle updating of network structure and nodal attributes due to
#'        exogenous epidemic model processes, with the default function of [`nwupdate.net`].
#' @param prevalence.FUN Module to calculate disease prevalence at each time step, with the default
#'        function of [`prevalence.net`].
#' @param verbose.FUN Module to print simulation progress to screen, with the default function of
#'        [`verbose.net`].
#' @param module.order A character vector of module names that lists modules in the order in which
#'        they should be evaluated within each time step. If `NULL`, the modules will be evaluated
#'        as follows: first any new modules supplied through `...` in the order in which they are
#'        listed, then the built-in modules in the order in which they are listed as arguments
#'        above. `initialize.FUN` will always be run first and `verbose.FUN` will always be run last.
#'        Module ordering is independent of `resimulate.network`: the specified order is always
#'        respected regardless of whether network resimulation is enabled. In the default ordering,
#'        `resim_nets.FUN` runs before `infection.FUN`, so the network is resimulated before
#'        transmission is evaluated at each time step.
#' @param save.nwstats If `TRUE`, save network statistics in a data frame. The statistics to be
#'        saved are specified in the `nwstats.formula` argument.
#' @param nwstats.formula A right-hand sided ERGM formula that includes network statistics of
#'        interest, with the default to the formation formula terms. Supports [`multilayer`]
#'        specification.
#' @param save.network If `TRUE`, networkDynamic or networkLite object is saved at simulation end.
#' @param save.transmat If `TRUE`, complete transmission matrix is saved at simulation end.
#' @param save.run If `TRUE`, the `run` sublist of `dat` is saved, allowing a
#'   simulation to restart from this output.
#' @param save.cumulative.edgelist If `TRUE`, the `cumulative.edgelist` is saved at simulation end.
#' @param save.other A character vector of elements on the `netsim_dat` main data list to save out
#'        after each simulation. One example for base models is the attribute list, `"attr"`, at
#'        the final time step.
#' @param verbose If `TRUE`, print model progress to the console.
#' @param verbose.int Time step interval for printing progress to console, where `0` prints
#'        completion status of entire simulation and positive integer `x` prints progress after
#'        every `x` time steps. The default is to print progress after each time step.
#' @param skip.check If `TRUE`, skips the default error checking for the structure and consistency
#'        of the parameter values, initial conditions, and control settings before running base
#'        epidemic models. Setting this to `FALSE` is recommended when running models with new
#'        modules specified.
#' @param raw.output If `TRUE`, `netsim` will output a list of raw data (one per simulation) instead
#'        of a cleaned and formatted `netsim` object.
#' @param future.use.plan If `FALSE`, `netsim` will use `multisession` is used with `workers = ncores for its
#'        parallelization. If `TRUE`, `netsim` will use the user defined plan from `globalEnv`. Finally, it can
#'        take the output of a `future::tweak()` call to setup a user defined temporary plan within `netsim`.
#'        Which can be useful for distributed computation (HPC).
#' @param tergmLite.track.duration If `TRUE`, track duration information for models in `tergmLite`
#'        simulations. Supports [`multilayer`] specification.
#' @param set.control.ergm Control arguments passed to `ergm::simulate_formula.network`. In `netsim`,
#'        this is only used when initializing the network with `edapprox = TRUE`. All other
#'        simulations in `netsim` use `tergm`. Supports [`multilayer`] specification.
#' @param set.control.tergm Control arguments passed to `tergm::simulate_formula.network`. See the
#'        help file for [`netdx`] for details and examples on specifying this parameter. Supports
#'        [`multilayer`] specification.
#' @param save.diss.stats If `TRUE`, `netsim` will compute and save duration and dissolution
#'        statistics for plotting and printing, provided `save.network` is `TRUE`, `tergmLite` is
#'        `FALSE`, and the dissolution model is homogeneous.
#' @param dat.updates Either `NULL`, a single function taking arguments `dat`,
#'        `at`, and `network`, or a list of functions of length one greater
#'        than the number of networks being simulated, with each function in
#'        the list taking arguments `dat` and `at`. Here `dat` is the main
#'        `netsim_dat` class object, `at` is the current timestep, and `network`
#'        is an index indicating the current position within the sequence of
#'        network (re)simulations on each time step. If a single function is
#'        passed, it will be called before the first network is simulated and
#'        after each network is simulated, with `network = 0L` before the first
#'        network is simulated and with `network = i` after the `i`th network
#'        is simulated. If a list of functions is passed, the first function
#'        will be called before the first network is simulated, and the
#'        `i + 1`th function will be called after the `i`th network is
#'        simulated. (Note that `at = 0L` is used for initial cross-sectional
#'        simulations in [`sim_nets_t1`].) The function(s) should return the
#'        `netsim_dat` object with any updates needed to correctly represent the
#'        network states for calls to `simulate` and/or `summary`. This can be
#'        useful if nodal attributes appearing in one network model depend on
#'        nodal degrees in a different network.
#' @param ... Additional control settings passed to model.
#'
#' @details
#' `control.net` sets the required control settings for any network model solved with the [`netsim`]
#' function. Controls are required for both base model types and when passing original process
#' modules. For an overview of control settings for base models, consult the
#' [Network Modeling for Epidemics](https://epimodel.github.io/sismid/) course materials For
#' all base models, the `type` argument is a necessary parameter and it has no default.
#'
#' @section The attr.rules Argument:
#' The `attr.rules` parameter is used to specify the rules for how nodal attribute values for
#' incoming nodes should be set. These rules are only necessary for models in which there are
#' incoming nodes (i.e., arrivals). There are three rules available for each attribute value:
#'  * **`current`**: new nodes will be assigned this attribute in proportion to the distribution of
#'    that attribute among existing nodes at that current time step.
#'  * **`t1`**: new nodes will be assigned this attribute in proportion to the distribution of that
#'    attribute among nodes at time 1 (that is, the proportions set in the original network for
#'    [`netest`]).
#'  * **`Value`**: all new nodes will be assigned this specific value, with no variation.
#' For example, the rules list `attr.rules = list(race = "t1", sex = "current", status = "s")`
#' specifies how the race, sex, and status attributes should be set for incoming nodes. By default,
#' the rule is `"current"` for all attributes except status, in which case it is `"s"` (that is, all
#' incoming nodes are susceptible).
#'
#' @section Checkpointing Simulations:
#' `netsim` has a built-in checkpoint system to prevent losing computation work if the function is
#' interrupted (SIGINT, power loss, time limit exceeded on a computation cluster). When enabled,
#' each simulation will be saved every `.checkpoint.steps` time steps. Then, if a checkpoint enabled
#' simulation is launched again with `netsim`, it will restart at the last checkpoint available in
#' the saved data.
#'
#' To enable the checkpoint capabilities of `netsim`, two control arguments have to be set:
#' `.checkpoint.steps`, which is a positive number of time steps to be run between each file save;
#' and `.checkpoint.dir`, which is the path to a directory to save the checkpointed data. If
#' `.checkpoint.dir` directory does not exist, `netsim` will attempt to create it on the first
#' checkpoint save. With these two controls defined, one can simply re-run `netsim` with the same
#' arguments to restart a set of simulations that were interrupted.
#'
#' Simulations are checkpointed individually: for example, if 3 simulations are run on a single core,
#' the first 2 are finished, then the interruption occurs during the third, `netsim` will only
#' restart the third one from the last checkpoint.
#'
#' A `.checkpoint.compress` argument can be set to overwrite the `compress` argument in `saveRDS`
#' used to save the checkpointed data. The current default for `saveRDS` is `gunzip (gz)`, which
#' provides fast compression that usually works well on `netsim` objects.
#'
#' By default, if `netsim` reaches the end of all simulations, the checkpoint data directory and its
#' content are removed before returning the `netsim` object. The `.checkpoint.keep` argument can be
#' set to `TRUE` to prevent this removal to inspect the raw simulation objects.
#'
#' @section New Modules:
#' Base network models use a set of module functions that specify how the individual nodes in the
#' network are subjected to infection, recovery, demographics, and other processes. Core modules are
#' those listed in the `.FUN` arguments. For each module, there is a default function used in
#' the simulation. The default infection module, for example, is contained in the [`infection.net`]
#' function.
#'
#' For original models, one may substitute replacement module functions for any of the default
#' functions. New modules may be added to the workflow at each time step by passing a module function
#' via the `...` argument. Consult the
#' [Extending EpiModel](https://epimodel.github.io/sismid/9_extending/mod9-Intro.html)
#' section of the Network Modeling for Epidemics course materials.
#' One may remove existing modules, such as `arrivals.FUN`, from the workflow by setting
#' the parameter value for that argument to `NULL`.
#'
#' @section End Horizon:
#' `netsim` implements an "End Horizon" mechanism, where a set of modules are
#' removed from the simulation at a specific time step. This is enabled through
#' the `end.horizon` parameter to `control.net`.
#'
#' This parameter must receive a `list` with fields `at`, the time step at which
#' the end horizon occurs, and `modules`, a character vector with the names of
#' the modules to remove. (e.g `list(at = 208, modules = c("arrivals.FUN",
#' "infections.FUN")))
#'
#' @return
#' An EpiModel object of class `control.net`.
#'
#' @seealso
#' Use [`param.net`] to specify model parameters and [`init.net`] to specify the initial conditions.
#' Run the parameterized model with [`netsim`].
#'
#' @keywords parameterization
#'
#' @export
#'
control.net <- function(type,
                        nsteps,
                        start = 1,
                        nsims = 1,
                        ncores = 1,
                        resimulate.network = FALSE,
                        tergmLite = FALSE,
                        cumulative.edgelist = FALSE,
                        truncate.el.cuml = 0,
                        attr.rules,
                        epi.by,
                        initialize.FUN = initialize.net,
                        resim_nets.FUN = resim_nets,
                        summary_nets.FUN = summary_nets,
                        infection.FUN = NULL,
                        recovery.FUN = NULL,
                        departures.FUN = NULL,
                        arrivals.FUN = NULL,
                        nwupdate.FUN = nwupdate.net,
                        prevalence.FUN = prevalence.net,
                        verbose.FUN = verbose.net,
                        module.order = NULL,
                        save.nwstats = TRUE,
                        nwstats.formula = "formation",
                        save.transmat = TRUE,
                        save.network,
                        save.run = FALSE,
                        save.cumulative.edgelist = FALSE,
                        save.other,
                        verbose = TRUE,
                        verbose.int = 1,
                        skip.check = FALSE,
                        raw.output = FALSE,
                        future.use.plan = FALSE,
                        tergmLite.track.duration = FALSE,
                        set.control.ergm = control.simulate.formula(MCMC.burnin = 2e5),
                        set.control.tergm = control.simulate.formula.tergm(MCMC.maxchanges = .Machine$integer.max),
                        save.diss.stats = TRUE,
                        dat.updates = NULL,
                        ...) {

  # Get arguments
  p <- list()
  formal.args <- formals(sys.function())
  formal.args[["..."]] <- NULL
  for (arg in names(formal.args)) {
    if (as.logical(mget(arg) != "")) {
      p[arg] <- list(get(arg))
    }
  }
  dot.args <- list(...)
  names.dot.args <- names(dot.args)
  if (length(dot.args) > 0) {
    for (i in seq_along(dot.args)) {
      p[[names.dot.args[i]]] <- dot.args[[i]]
    }
  }

  if ("births.FUN" %in% names(dot.args)) {
    stop("The births.FUN parameter has been removed. Use arrivals.FUN instead.")
  }
  if ("deaths.FUN" %in% names(dot.args)) {
    stop("The deaths.FUN parameter has been removed. Use departures.FUN instead.")
  }
  if ("depend" %in% names(dot.args)) {
    stop("The depend parameter has been removed. Use resimulate.network instead.")
  }

  ## Module classification
  bi.mods <- grep(".FUN", names(formal.args), value = TRUE)
  p[["bi.mods"]] <- character()
  bi.nms <- bi.mods
  index <- 1
  if (is.null(p[["type"]])) {
    for (i in seq_along(bi.mods)) {
      if (!is.null(p[[bi.mods[i]]])) {
        p[["bi.mods"]][index] <- bi.mods[i]
        index <- index + 1
      }
    }
  } else {
    p[["bi.mods"]] <- bi.mods
  }
  p[["user.mods"]] <- grep(".FUN", names(dot.args), value = TRUE)
  p[["f.names"]] <- c(p[["bi.mods"]], p[["user.mods"]])

  ## Defaults and checks

  #Check whether any base modules have been redefined by user (note: must come
  # after above)
  bi.nms <- bi.nms[-which(bi.nms %in% c("initialize.FUN", "resim_nets.FUN",
                                        "summary_nets.FUN", "verbose.FUN",
                                        "nwupdate.FUN", "prevalence.FUN"))]
  if (length(bi.nms) > 0) {
    flag1 <- logical()
    for (args in seq_along(bi.nms)) {
      if (!(is.null(p[[bi.nms[args]]]))) {
        temp1 <- get(gsub(".FUN", ".net", bi.nms[args]))
        temp2 <- p[[bi.nms[args]]]
        flag1[args] <- identical(temp1, temp2)
      }
    }

    if (!is.null(p[["type"]]) && sum(flag1, na.rm = TRUE) != length(flag1)) {
      stop("Control parameter 'type' must be null if any user defined base
           modules are present")
    }
  }

  if (!is.null(p[["type"]]) && length(p[["user.mods"]]) > 0) {
    stop("Control parameter 'type' must be null if any user specified modules
         are present")
  }

  if (is.null(p[["nsteps"]])) {
    stop("Specify nsteps")
  }

  if (missing(attr.rules)) {
    p[["attr.rules"]] <- list()
  }

  if (!is.null(p[["epi.by"]])) {
    if (length(p[["epi.by"]]) > 1) {
      stop("Length of epi.by currently limited to 1")
    } else {
      p[["epi.by"]] <- epi.by
    }
  }

  if (is.null(p[["save.network"]])) {
    p[["save.network"]] <- TRUE
  }

  if (p[["tergmLite"]] == TRUE && p[["resimulate.network"]] == FALSE) {
    warning("Because tergmLite = TRUE, resetting resimulate.network = TRUE",
            call. = FALSE)
    p[["resimulate.network"]] <- TRUE
  }

  ## Output
  p <- set.control.class("control.net", p)
  return(p)
}


#' @title Cross Checking of Inputs for Stochastic Network Models
#'
#' @description This function checks that the estimation object from
#'              [netest()] and the three parameter lists from
#'              [param.net()], [init.net()], and
#'              [control.net()] are consistent.
#'
#' @param x An `EpiModel` object of class [netest()].
#' @param param An `EpiModel` object of class [param.net()].
#' @param init An `EpiModel` object of class [init.net()].
#' @param control An `EpiModel` object of class [control.net()].
#'
#' @return
#' This function returns no objects.
#'
#' @export
#' @keywords internal
#'
crosscheck.net <- function(x, param, init, control) {
  check.control.class("net", "EpiModel crosscheck.net")

  if (!is.null(control[["type"]]) && length(control[["user.mods"]]) == 0) {

    if (control[["start"]] == 1 && control[["skip.check"]] == FALSE) {

      # Main class check ----------------------------------------------------
      if (inherits(x, "netest")) {
        x <- list(x)
      }
      if (!inherits(x, "list") || length(x) == 0 ||
            !all(vapply(x, inherits, logical(1), "netest"))) {
        stop("x must be either an object of class netest or a list of objects",
             " of class netest when start == 1")
      }
      if (!inherits(param, "param.net")) {
        stop("param must be an object of class param.net")
      }
      if (!inherits(init, "init.net")) {
        stop("init must be an object of class init.net")
      }
      if (!inherits(control, "control.net")) {
        stop("control must be an object of class control.net")
      }

      # Pull network object from netest object
      nw <- x[[1]]$newnetwork

      # Defaults ------------------------------------------------------------

      # Is status in network formation formula?
      statOnNw <- get_vertex_attribute(nw, "status") %in% c("s", "i", "r")
      statOnNw <- ifelse(sum(statOnNw) > 0, TRUE, FALSE)

      nGroups <- length(unique(get_vertex_attribute(nw, "group")))
      nGroups <- ifelse(nGroups == 2, 2, 1)

      if (nGroups == 2 && is.null(control[["pid.prefix"]])) {
        control[["pid.prefix"]] <- c("g1.", "g2.")
      }

      if (statOnNw == TRUE && is.null(control[["attr.rules"]][["status"]])) {
        control[["attr.rules"]][["status"]] <- "s"
      }


      # Checks -------------------------------------------------------------

      # Check that prevalence in NW attr status and initial conditions match
      if (statOnNw == TRUE) {
        nw1 <- sum(get_vertex_attribute(nw, "status") == 1)
        init1 <- sum(unlist(init[grep("i.num", names(init), value = TRUE)]))
        if ("i.num" %in% names(init) && nw1 != init1) {
          warning("Overriding init infected settings with network
                  status attribute", immediate. = TRUE)
        }
      }

      # Check consistency of status vector to network structure
      if (!is.null(init[["status.vector"]])) {
        if (length(init[["status.vector"]]) != network.size(nw)) {
          stop("Length of status.vector is unequal to size of initial network")
        }
        svals <- sort(unique(init[["status.vector"]]))
        if (control[["type"]] == "SIR") {
          if (any(svals %in% c("s", "i", "r") == FALSE)) {
            stop("status.vector contains values other than \"s\", \"i\",
                 and \"r\" ")
          }
        } else {
          if (any(svals %in% c("s", "i") == FALSE)) {
            stop("status.vector contains values other than \"s\" and \"i\" ")
          }
        }
      }

      # Two-group model checks for inital conditions
      if (nGroups == 2 && is.null(init[["i.num.g2"]]) &&
            is.null(init[["status.vector"]]) && statOnNw == FALSE) {
        stop("Specify i.num.g2 for two-group model simulations")
      }

      # Recovery rate and initial recovered checks
      if (control[["type"]] %in% c("SIR", "SIS")) {
        if (is.null(param[["rec.rate"]])) {
          stop("Specify rec.rate in param.net")
        }
        if (nGroups == 2 && is.null(param[["rec.rate.g2"]])) {
          stop("Specify rec.rate.g2 in param.net")
        }
      }
      if (control[["type"]] == "SIR") {
        if (is.null(init[["r.num"]]) && is.null(init[["status.vector"]]) && statOnNw == FALSE) {
          stop("Specify r.num in init.net")
        }
        if (nGroups == 2 && is.null(init[["r.num.g2"]]) &&
              is.null(init[["status.vector"]]) && statOnNw == FALSE) {
          stop("Specify r.num.g2 in init.net")
        }
      }

      # Check demographic parameters for two-group models
      if (nGroups == 2 && param[["vital"]] == TRUE) {
        if (is.null(param[["a.rate.g2"]])) {
          stop("Specify a.rate.g2 in param.net")
        }
        if (is.null(param[["ds.rate.g2"]])) {
          stop("Specify ds.rate.g2 in param.net")
        }
        if (is.null(param[["di.rate.g2"]])) {
          stop("Specify di.rate.g2 in param.net")
        }
        if (control[["type"]] == "SIR") {
          if (is.null(param[["dr.rate.g2"]])) {
            stop("Specify dr.rate.g2 in param.net")
          }
        }
      }
    }

    if (control[["start"]] > 1) {

      control[["resimulate.network"]] <- TRUE

      if (control[["skip.check"]] == FALSE) {
        if (!inherits(x, "netsim")) {
          stop("x must be a netsim object if control setting start > 1")
        }
        if (is.null(x[["run"]])) {
          stop("x must contain `run` to restart simulation, see `save.run` ",
               "control setting")
        }
        if (control[["nsteps"]] < control[["start"]]) {
          stop("control setting nsteps must be > control setting start in ",
               "restarted simulations")
        }
        if (control[["start"]] > x[["control"]][["nsteps"]] + 1) {
          stop("control setting start must be 1 greater than the nsteps in
               the ", "prior simulation")
        }
      }
    }


    ## Assign modules based on group parameter
    if (!is.null(control[["type"]])) {
      def <- grep(".FUN", names(control))
      args <- names(control)[def]
      flag <- length(grep(".g2", names(param)))

      if (flag == 0) {
        for (i in seq_along(args)) {
          if (is.null(control[[args[i]]])) {
            temp <- get(gsub(".FUN", ".net", args[i]))
            control[[args[i]]] <- temp
            control[["f.args"]][i] <- gsub(".FUN", ".net", args[i])
          }
        }
      } else {
        for (i in seq_along(args)) {
          if (is.null(control[[args[i]]])) {
            temp <- get(gsub(".FUN", ".2g.net", args[i]))
            control[[args[i]]] <- temp
            control[["f.args"]][i] <- gsub(".FUN", ".2g.net", args[i])
          }
        }
      }
    }
  }

  if (!is.null(control[["type"]]) && length(control[["user.mods"]]) > 0) {
    stop("Control setting 'type' must be NULL if any user-specified modules
         specified.")
  }

  if (inherits(x, "netest")) {
    nwparam <- list(x)
  } else if (inherits(x, "netsim")) {
    nwparam <- x$nwparam
  } else if (inherits(x, "networkDynamic")) {
    nwparam <- list(x) # relevant to EpiModel gallery example
  } else { # must be list of netest
    nwparam <- x
  }

  num.nw <- length(nwparam)

  # convert relevant control arguments to multilayer if they are not already, and check length
  for (control_arg_name in c("nwstats.formula", "set.control.ergm",
                             "set.control.tergm", "tergmLite.track.duration")) {
    control_arg_value <- control[[control_arg_name]]
    if (!is(control_arg_value, "multilayer")) {
      control[[control_arg_name]] <- do.call(multilayer, rep(list(control_arg_value), length.out = num.nw))
    } else if (length(control_arg_value) != num.nw) {
      stop("multilayer control argument `", control_arg_name, "` has length ",
           length(control_arg_value), " but should have length ", num.nw, ".")
    }
  }

  # convert nwstats.formula = "formation" to actual formation formula
  for (network in seq_len(num.nw)) {
    if (!is.null(control[["nwstats.formula"]][[network]]) &&
          control[["nwstats.formula"]][[network]] == "formation") {
      control[["nwstats.formula"]][[network]] <- nwparam[[network]]$formation
    }
  }

  # convert dat.updates to a function of dat, at, network if it is not already
  dat.updates <- control[["dat.updates"]]
  if (is.list(dat.updates)) {
    if (length(dat.updates) != num.nw + 1L) {
      stop("control argument `dat.updates` is of length ", length(dat.updates),
           " but should be of length ", num.nw + 1L, ".")
    }
    control[["dat.updates"]] <- trim_env(function(dat, at, network) {
      dat.updates[[network + 1L]](dat = dat, at = at)
    }, keep = c("dat.updates"))
  }

  ## In-place assignment to update param and control
  assign("param", param, pos = parent.frame())
  assign("control", control, pos = parent.frame())
}

#' @title Specify Controls by Network
#'
#' @description This utility function allows specification of certain
#'              [netsim()] controls to vary by network.  The
#'              [netsim()] control arguments currently supporting
#'              `multilayer` specifications are `nwstats.formula`,
#'              `set.control.ergm`, `set.control.tergm`, and
#'              `tergmLite.track.duration`.
#'
#' @param ... control arguments to apply to each network, with the index of the
#'        network corresponding to the index of the control argument
#'
#' @return an object of class `multilayer` containing the specified
#'         control arguments
#'
#' @export
#'
multilayer <- function(...) {
  out <- list(...)
  class(out) <- c("multilayer", class(out))
  return(out)
}

context("Netsim Parallel")

test_that("netsim par, 1 sim, 1 core", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  est <- netest(nw, formation = ~edges, target.stats = 24,
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)

  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsims = 1, nsteps = 5, ncores = 1,
                         verbose = FALSE)
  mod <- netsim(est, param, init, control)
  expect_is(mod, "netsim")
  expect_true(mod$control$nsims == 1)
  expect_true(mod$control$ncores == 1)

})


test_that("netsim par, 2 sims, 2 cores", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  est <- netest(nw, formation = ~edges, target.stats = 24,
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)

  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsims = 2, ncores = 2, nsteps = 5,
                         verbose = FALSE, future.use.plan = TRUE)
  mod <- netsim(est, param, init, control)
  expect_is(mod, "netsim")
  expect_true(mod$control$nsims == 2)
  expect_true(mod$control$ncores == 2)
  expect_true(inherits(future::plan(), "sequential"))
})

test_that("netsim par, 2 sims, 2 cores - future.use.plan", {
  skip_on_cran()
  skip_on_os("windows")
  nw <- network_initialize(n = 50)
  est <- netest(nw, formation = ~edges, target.stats = 24,
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)

  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsims = 2, nsteps = 5,
                         verbose = FALSE, future.use.plan = TRUE)
  with(future::plan("multicore", workers = 2), local = TRUE)
  mod <- netsim(est, param, init, control)
  expect_is(mod, "netsim")
  expect_true(mod$control$nsims == 2)

})

test_that("netsim par, 1 sim, 2 cores", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  est <- netest(nw, formation = ~edges, target.stats = 24,
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)

  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsims = 1, nsteps = 5, ncores = 2,
                         verbose = FALSE)
  mod <- netsim(est, param, init, control)
  expect_is(mod, "netsim")
  expect_true(mod$control$nsims == 1)
  expect_true(mod$control$ncores == 1)

})

test_that("netsim par, 2 sims, 1 cores", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  est <- netest(nw, formation = ~edges, target.stats = 24,
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)

  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsims = 2, nsteps = 5, ncores = 1,
                         verbose = FALSE)
  mod <- netsim(est, param, init, control)
  expect_is(mod, "netsim")
  expect_true(mod$control$nsims == 2)
  expect_true(mod$control$ncores == 1)

})

context("ergm.ego (All SOC)")

# Test ergm.ego ---------------------------------------------------

test_that("ergm and ergm.ego produce the same results in EpiModel", {
  skip_on_cran()
  for (trim in list(FALSE, TRUE)) {
    for (ppopnw in list(FALSE, TRUE)) {
      nw <- network_initialize(n = 50)
      nw %v% "race" <- rep(1:2, length.out = 50)
      nw <- san(nw ~ edges, target.stats = c(50))

      nwe <- ergm.ego:::as.egor.network(nw)

      nw[,] <- FALSE

      set.seed(0)
      if (ppopnw == TRUE) {
        netest_ergm.ego <- netest(nwe, formation = ~edges + degree(1) + nodemix("race"),
                                  coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                                  set.control.ergm.ego = ergm.ego::control.ergm.ego(ppopsize = nw))
      } else {
        ## assumes node order is preserved in nw -> ergm.ego:::as.egor.network(nw) -> pseudopopulation in ergm.ego
        netest_ergm.ego <- netest(nwe, formation = ~edges + degree(1) + nodemix("race"),
                                  coef.diss = dissolution_coefs(~offset(edges), 10, 0))
      }

      set.seed(0)
      netest_ergm <- netest(nw, formation = ~edges + degree(1) + nodemix("race"),
                            target.stats = netest_ergm.ego$target.stats,
                            coef.diss = dissolution_coefs(~offset(edges), 10, 0))

      if (trim == TRUE) {
        netest_ergm <- trim_netest(netest_ergm)
        netest_ergm.ego <- trim_netest(netest_ergm.ego)
      }

      expect_identical(netest_ergm$coef.form.crude, netest_ergm.ego$coef.form.crude)

      set.seed(0)
      dx_ergm <- netdx(netest_ergm, nsteps = 5, dynamic = FALSE, verbose = FALSE)
      set.seed(0)
      dx_ergm.ego <- netdx(netest_ergm.ego, nsteps = 5, dynamic = FALSE, verbose = FALSE)

      expect_identical(dx_ergm$stats, dx_ergm.ego$stats)
      expect_identical(dx_ergm$stats.table.formation, dx_ergm.ego$stats.table.formation)

      set.seed(0)
      dxd_ergm <- netdx(netest_ergm, nsteps = 5, dynamic = TRUE, verbose = FALSE)
      set.seed(0)
      dxd_ergm.ego <- netdx(netest_ergm.ego, nsteps = 5, dynamic = TRUE, verbose = FALSE)

      expect_identical(dxd_ergm$stats, dxd_ergm.ego$stats)
      expect_identical(dxd_ergm$stats.table.formation, dxd_ergm.ego$stats.table.formation)
      expect_identical(dxd_ergm$stats.table.dissolution, dxd_ergm.ego$stats.table.dissolution)
      expect_identical(dxd_ergm$pages, dxd_ergm.ego$pages)
      expect_identical(dxd_ergm$pages_imptd, dxd_ergm.ego$pages_imptd)

      param <- param.net(inf.prob = 0.3, act.rate = 0.5)
      init <- init.net(i.num = 10)
      control <- control.net(type = "SI", nsims = 1, nsteps = 5, verbose = FALSE)

      set.seed(0)
      sim_ergm <- netsim(netest_ergm, param, init, control)
      set.seed(0)
      sim_ergm.ego <- netsim(netest_ergm.ego, param, init, control)

      expect_identical(sim_ergm$stats, sim_ergm.ego$stats)
      expect_identical(sim_ergm$epi, sim_ergm.ego$epi)
    }
  }
})

context("Dissolution Diagnostics")

test_that("simulation diagnostics work as expected", {
  skip_on_cran()

  ## direct implementation of dissolution statistics based on tedgelist
  simulate_diss_stats <- function(est, nsteps, dyad_indexer) {
    nws1 <- simulate(est$formula,
                     coef = est$coef.form.crude,
                     basis = est$newnetwork,
                     dynamic = FALSE)

    nws2 <- simulate(~Form(est$formation) + Persist(est$coef.diss$dissolution),
                     coef = c(est$coef.form, est$coef.diss$coef.crude),
                     basis = nws1,
                     time.slices = nsteps,
                     dynamic = TRUE)

    dissolution <- est$coef.diss$dissolution
    durs <- est$coef.diss$duration

    sim.df <- as.data.frame(nws2)

    edgecounts <- matrix(0, nrow = nsteps, ncol = length(durs))
    edgeages <- matrix(0, nrow = nsteps, ncol = length(durs))
    edgediss <- matrix(0, nrow = nsteps, ncol = length(durs))

    dyad_types <- dyad_indexer(sim.df$tail, sim.df$head, nws1)
    for (i in seq_len(NROW(sim.df))) {
      onset <- max(sim.df$onset[i], 1)
      terminus <- sim.df$terminus[i]

      if (terminus > 1) {
        edgecounts[onset - 1 + seq_len(terminus - onset), dyad_types[i]] <-
          edgecounts[onset - 1 + seq_len(terminus - onset), dyad_types[i]] + 1
        edgeages[onset - 1 + seq_len(terminus - onset), dyad_types[i]] <-
          edgeages[onset - 1 + seq_len(terminus - onset), dyad_types[i]] +
          seq_len(terminus - onset) + (sim.df$onset[i] == 0)
      }

      if (terminus <= nsteps) {
        edgediss[terminus, dyad_types[i]] <- edgediss[terminus, dyad_types[i]] + 1
      }
    }

    init.edgecounts <- summary(dissolution, basis = nws2, at = 0)
    if (length(durs) > 1) {
      init.edgecounts[1] <- init.edgecounts[1] - sum(init.edgecounts[-1])
    }
    edgecounts <- rbind(init.edgecounts, edgecounts)

    edgeagesimputed <- edgeages

    wti <- which(sim.df$onset == 0 & sim.df$terminus > 1)
    for (index in wti) {
      edgeagesimputed[seq_len(sim.df$terminus[index] - 1), dyad_types[index]] <-
        edgeagesimputed[seq_len(sim.df$terminus[index] - 1), dyad_types[index]] +
        rgeom(1, 1/durs[dyad_types[index]])
    }

    pages <- array(edgeages/edgecounts[-1,,drop=FALSE],
                   dim = c(nsteps,length(durs),1))
    pages_imptd <- array(edgeagesimputed/edgecounts[-1,,drop=FALSE],
                         dim = c(nsteps,length(durs),1))
    prop.diss <- array(edgediss/edgecounts[-NROW(edgecounts),,drop=FALSE],
                       dim = c(nsteps,length(durs),1))

    list(pages = pages,
         pages_imptd = pages_imptd,
         prop.diss = prop.diss)
  }

  net_size <- 100
  nsteps <- 10
  nsims <- 2
  seed <- 0

  coefs.diss <- list(dissolution_coefs(~offset(edges), 10, 0),
                     dissolution_coefs(~offset(edges)+offset(nodemix("attr", levels = 2:3,
                                                                     levels2 = c(2,1))), c(8,5,7), 0),
                     dissolution_coefs(~offset(edges)+offset(nodematch("attr", diff = TRUE,
                                                                       levels = c(3,1,2))), c(31, 22, 3, 4), 0))

  dyad_indexers <- list(function(tails, heads, nw) rep(1, length(tails)),
                        function(tails, heads, nw) {
                          attr <- nw %v% "attr"
                          tailattr <- attr[tails]
                          headattr <- attr[heads]
                          indices <- rep(1, length(tails))
                          indices[(tailattr == 2 & headattr == 3) | (tailattr == 3 & headattr == 2)] <- 2
                          indices[tailattr == 2 & headattr == 2] <- 3
                          indices
                        },
                        function(tails, heads, nw) {
                          attr <- nw %v% "attr"
                          tailattr <- attr[tails]
                          headattr <- attr[heads]
                          indices <- rep(1, length(tails))
                          indices[tailattr == 3 & headattr == 3] <- 2
                          indices[tailattr == 1 & headattr == 1] <- 3
                          indices[tailattr == 2 & headattr == 2] <- 4
                          indices
                        })

  formations <- list(~edges,
                     ~edges + nodemix("attr", levels = 2:3, levels2 = c(2,1)),
                     ~edges + nodematch("attr", diff = TRUE, levels = c(3,1,2)))

  targets <- lapply(list(c(100),
                         c(100, 2*100/9, 2*100/9),
                         c(100, 100/9, 100/9, 100/9)),
                    as.integer)

  init.edges.list <- list(FALSE, FALSE, TRUE)
  nested.edapprox.list <- list(FALSE, TRUE, FALSE)
  keep.tnetwork.list <- list(TRUE, FALSE, FALSE)

  for (index in seq_along(coefs.diss)) {
    init.edges <- init.edges.list[[index]]
    nested.edapprox <- nested.edapprox.list[[index]]
    keep.tnetwork <- keep.tnetwork.list[[index]]
    coef.diss <- coefs.diss[[index]]
    dyad_indexer <- dyad_indexers[[index]]

    if (nested.edapprox == TRUE) {
      formation <- formations[[index]]
      target.stats <- targets[[index]]
    } else {
      formation <- ~edges
      target.stats <- c(100)
    }

    nw <- network.initialize(net_size, directed = FALSE)
    nw %v% "attr" <- rep(1:3, length.out = net_size)

    est <- netest(nw,
                  formation = formation,
                  coef.diss = coef.diss,
                  target.stats = target.stats,
                  nested.edapprox = nested.edapprox)

    if (init.edges == FALSE) {
      est$newnetwork[,] <- FALSE
    }

    set.seed(seed)
    dx <- netdx(est, nsims = nsims, nsteps = nsteps, keep.tnetwork = keep.tnetwork, verbose = FALSE)

    capture_output(
      print(dx)
    )
    plot(dx)

    set.seed(seed)
    ds <- list()
    for (i in seq_len(nsims)) {
      ds[[i]] <- simulate_diss_stats(est, nsteps, dyad_indexer)
    }

    durs <- coef.diss$duration

    pages <- array(unlist(lapply(ds, `[[`, "pages")),
                   dim = c(nsteps,length(durs),nsims))
    pages_imptd <- array(unlist(lapply(ds, `[[`, "pages_imptd")),
                         dim = c(nsteps,length(durs),nsims))
    prop.diss <- array(unlist(lapply(ds, `[[`, "prop.diss")),
                       dim = c(nsteps,length(durs),nsims))

    expect_equal(pages, dx$pages)
    expect_equal(pages_imptd, dx$pages_imptd)
    expect_equal(prop.diss, dx$prop.diss)
  }
})

test_that("netsim produces a networkDynamic with the expected data.frame when resimulate.network = FALSE", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
  est <- netest(nw, formation = ~edges + nodematch("race"),
                target.stats = c(25, 10),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)
  param <- param.net(inf.prob = 0, act.rate = 0)
  init <- init.net(status.vector = rep("i", 50))
  control <- control.net(type = NULL,
                         nsims = 1,
                         nsteps = 5,
                         resimulate.network = FALSE,
                         set.control.ergm = control.simulate.formula(),
                         verbose = FALSE)

  set.seed(0)
  mod <- netsim(est, param, init, control)
  nwd <- get_network(mod)

  capture_output(
    print(mod)
  )
  plot(mod)

  set.seed(0)
  nws <- simulate(est$formula,
                  coef = est$coef.form.crude,
                  basis = est$newnetwork,
                  dynamic = FALSE)

  nws <- networkDynamic::as.networkDynamic(nws)
  nws <- networkDynamic::activate.vertices(nws, onset = 0, terminus = Inf)
  nws <- networkDynamic::activate.edges(nws, onset = 0, terminus = Inf)

  nws <- simulate(nws ~ Form(est$formation) + Persist(est$coef.diss$dissolution),
                  coef = c(est$coef.form, est$coef.diss$coef.crude),
                  time.start = 0,
                  time.slices = 5,
                  dynamic = TRUE)

  expect_identical(as.data.frame(nws), as.data.frame(nwd))
})

test_that("netsim produces a networkDynamic with the expected data.frame when resimulate.network = TRUE", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
  est <- netest(nw, formation = ~edges + nodematch("race"),
                target.stats = c(25, 10),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)
  param <- param.net(inf.prob = 0, act.rate = 0)
  init <- init.net(status.vector = rep("i", 50))
  control <- control.net(type = NULL,
                         nsims = 1,
                         resimulate.network = TRUE,
                         nsteps = 5,
                         verbose = FALSE)

  set.seed(0)
  mod <- netsim(est, param, init, control)
  nwd <- get_network(mod)

  capture_output(
    print(mod)
  )
  plot(mod)

  set.seed(0)
  nws <- simulate(est$formula,
                  coef = est$coef.form.crude,
                  basis = est$newnetwork,
                  control = list(MCMC.burnin = 2e5),
                  dynamic = FALSE)

  nws <- networkDynamic::as.networkDynamic(nws)
  nws <- networkDynamic::activate.vertices(nws, onset = 0, terminus = Inf)
  nws <- networkDynamic::activate.edges(nws, onset = 0, terminus = Inf)

  nws %n% "net.obs.period" <- list(observations = list(c(0,1)),
                                   mode = "discrete",
                                   time.increment =  1,
                                   time.unit = "step")

  for (i in seq_len(5)) {
    nws <- suppressWarnings(simulate(nws ~ Form(est$formation) + Persist(est$coef.diss$dissolution),
                                     coef = c(est$coef.form, est$coef.diss$coef.crude),
                                     time.start = i,
                                     time.offset = 0,
                                     dynamic = TRUE))
  }

  expect_identical(as.data.frame(nws), as.data.frame(nwd))
})

test_that("tedgelist_to_toggles functions as expected", {
  skip_on_cran()
  logit <- function(p) log(p/(1-p))
  density <- 1/50
  D <- 10

  for (init.edges in list(FALSE, TRUE)) {
    nw <- network.initialize(100, directed = FALSE)
    if (init.edges == TRUE) {
      nw <- san(nw ~ edges, target.stats = c(100))
    }

    set.seed(0)
    nwd <- simulate(nw ~ Form(~edges) + Persist(~edges),
                    coef = c(logit(density) - log(D), log(D - 1)),
                    time.slices = 10,
                    output = "networkDynamic",
                    dynamic = TRUE)
    toggles <- tedgelist_to_toggles(as.data.frame(nwd))

    set.seed(0)
    changes <- simulate(nw ~ Form(~edges) + Persist(~edges),
                        coef = c(logit(density) - log(D), log(D - 1)),
                        time.slices = 10,
                        output = "changes",
                        dynamic = TRUE)

    if (init.edges == TRUE) {
      changes <- rbind(cbind(0L, as.edgelist(nw), 1L),
                       changes)
    }

    toggles2 <- changes[,-4L,drop=FALSE]

    toggles <- toggles[order(toggles[,1], toggles[,2], toggles[,3]),,drop=FALSE]
    toggles2 <- toggles2[order(toggles2[,1], toggles2[,2], toggles2[,3]),,drop=FALSE]
    expect_identical(unname(toggles), unname(toggles2))
  }
})

context("Multi-Layer Network Models")

test_that("netsim runs with multiple networks, with open or closed population", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))

  dc10 <- dissolution_coefs(~offset(edges), 10, 0)
  dc20 <- dissolution_coefs(~offset(edges), 20, 0)
  dc1 <- dissolution_coefs(~offset(edges), 1, 0)
  est <- netest(nw, formation = ~edges + nodematch("race"),
                target.stats = c(25, 10),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)

  est10 <- est
  est1 <- update_dissolution(est, dc1)
  est20 <- update_dissolution(est, dc20)

  est <- list(est10, est1, est20, est1, est20)

  param_open <- param.net(inf.prob = 0.3, act.rate = 0.1, a.rate = 0.03,
                          di.rate = 0.03, ds.rate = 0.03)
  param_closed <- param.net(inf.prob = 0.3, act.rate = 0.1)

  init <- init.net(i.num = 10)

  nsims <- 3L
  nnets <- 5L
  tergm_nets <- c(1,3,5)

  resim_stat_names <- list(c("triangle", "nodefactor.deg.4.1"),
                           c("edges", "nodematch.race"),
                           c("mean.age", "nodefactor.deg.2.2"),
                           c("degree0", "degree1", "degree2", "degree3"),
                           c("edges", "nodematch.race", "nodefactor.deg.5.1"))

  resim_nwstats_formulas <- multilayer(~triangle + nodefactor("deg.4", levels = I(1)),
                                       "formation",
                                       ~mean.age + nodefactor("deg.2", levels = I(2)),
                                       ~degree(0:3),
                                       ~edges + nodematch("race") + nodefactor("deg.5", levels = I(1)))

  stat_names <- list(c("triangle"),
                     c("edges", "nodematch.race"),
                     c("mean.age"),
                     c("degree0", "degree1", "degree2", "degree3"),
                     c("edges", "nodematch.race"))

  nwstats_formulas <- multilayer(~triangle,
                                 "formation",
                                 ~mean.age,
                                 ~degree(0:3),
                                 "formation")

  for (tergmLite in c(FALSE, TRUE)) {
    for (resimulate.network in unique(c(tergmLite, TRUE))) {
      for (open_population in unique(c(FALSE, resimulate.network))) {
        if (tergmLite == TRUE) {
          save.other <- c()
        } else {
          save.other <- c()
        }
        if (open_population == TRUE) {
          param <- param_open
        } else {
          param <- param_closed
        }
        if (resimulate.network == TRUE) {
          sim_stat_names <- resim_stat_names
          sim_nwstats_formulas <- resim_nwstats_formulas
        } else {
          sim_stat_names <- stat_names
          sim_nwstats_formulas <- nwstats_formulas
        }

        for (iteration in 1:2) {
          if (iteration == 1) {
            nsteps <- 5L
            control <- control.net(type = "SI",
                                   nsims = nsims,
                                   nsteps = nsteps,
                                   tergmLite = tergmLite,
                                   resimulate.network = resimulate.network,
                                   tergmLite.track.duration = TRUE,
                                   dat.updates = function (dat, at, network) {
                                     if (network > 0L) {
                                       if (get_control(dat, "tergmLite") == TRUE) {
                                         dat <- set_attr(dat, paste0("deg.", network),
                                                         get_degree(dat$run$el[[network]]))
                                       } else {
                                         deg_attr <- get_degree(as.edgelist(network.collapse(dat$run$nw[[network]],
                                                                                             at = at,
                                                                                             retain.all.vertices = TRUE)))
                                         for (other_net in seq_len(dat$num.nw)) {
                                           dat$run$nw[[other_net]] <- set_vertex_attribute(dat$run$nw[[other_net]],
                                                                                       paste0("deg.", network),
                                                                                       deg_attr)
                                         }
                                       }
                                     }
                                     dat
                                   },
                                   nwstats.formula = sim_nwstats_formulas,
                                   verbose = FALSE,
                                   save.network = TRUE,
                                   save.run = TRUE,
                                   save.other = c(save.other))
            invisible(capture.output(print(control)))
            basis <- est
          } else {
            control$start <- nsteps + 1L
            nsteps <- 11L
            control$nsteps <- nsteps
            basis <- sim
          }
          sim <- netsim(basis, param, init, control)

          invisible(capture.output(print(sim)))
          plot(sim)
          plot(sim, type = "formation")
          if (tergmLite == FALSE) {
            plot(sim, type = "dissolution")
            plot(sim, type = "duration")
          }

          for (network in seq_len(nnets)) {
            invisible(capture.output(print(sim, network = network)))
            plot(sim, network = network)
            plot(sim, type = "formation", network = network)
            if (tergmLite == FALSE) {
              plot(sim, type = "dissolution", network = network)
              plot(sim, type = "duration", network = network)
            }
            expect_equal(sim$nwparam[[network]]$coef.form[1],
                         est[[network]]$coef.form[1] +
                           log(network.size(est[[network]]$newnetwork)) -
                           log(sim$run[[1]]$num),
                         tolerance = 1e-6)
          }

          for(simno in seq_len(nsims)) {
            for(network in seq_len(nnets)) {
              expect_equal(sim$coef.form[[simno]][[network]][1],
                           est[[network]]$coef.form[1] +
                             log(network.size(est[[network]]$newnetwork)) -
                             log(sim$run[[simno]]$num),
                           tolerance = 1e-6)
              if (tergmLite == TRUE) {
                expect_is(sim$network[[simno]][[network]], "networkLite")
                if (open_population == FALSE) {
                  expect_equal(sim$run[[simno]]$el[[network]], as.edgelist(sim$network[[simno]][[network]]))
                  expect_equal(sim$run[[simno]]$attr[[paste0("deg.", network)]], get_degree(sim$run[[simno]]$el[[network]]))
                }
                if (network %in% tergm_nets) {
                  expect_equal(sim$network[[simno]][[network]] %n% "time", nsteps)
                  expect_true(all((sim$network[[simno]][[network]] %n% "lasttoggle")[,3] >= 0))
                  expect_true(all((sim$network[[simno]][[network]] %n% "lasttoggle")[,3] <= nsteps))
                }
              } else {
                expect_is(sim$network[[simno]][[network]], "networkDynamic")
                if (open_population == FALSE) {
                  if (resimulate.network == TRUE || iteration == 2) {
                    expect_equal(sim$network[[simno]][[network]] %v% paste0("deg.", network),
                                 get_degree(as.edgelist(network.collapse(sim$network[[simno]][[network]], at = nsteps))))
                  }
                }
              }

              expect_equal(network.size(est[[network]]$newnetwork), sim$epi$num[[simno]][1])
              if (tergmLite == TRUE) {
                if (open_population == FALSE) {
                  expect_equal(sim$epi$num[nsteps - 1,simno],
                               sim$run[[simno]]$num)
                }
              } else {
                expect_equal(network.size(network.collapse(sim$network[[simno]][[network]], at = nsteps - 1)),
                               sim$run[[simno]]$num)
              }

              stats_matrix <- get_nwstats(sim, network = network, mode = "list")[[simno]]
              expect_equal(NCOL(stats_matrix), length(sim_stat_names[[network]]))
              expect_equal(NROW(stats_matrix), nsteps)
              expect_equal(colnames(stats_matrix), sim_stat_names[[network]])

              if (open_population == FALSE) {
                final_stats <- stats_matrix[nsteps,]
                names(final_stats) <- colnames(stats_matrix)
                if (tergmLite == TRUE) {
                  nwL <- networkLite(sim$run[[simno]]$el[[network]], sim$run[[simno]]$attr)
                  if (network %in% tergm_nets) {
                    nwL %n% "time" <- sim$network[[simno]][[network]] %n% "time"
                    nwL %n% "lasttoggle" <- sim$network[[simno]][[network]] %n% "lasttoggle"
                  }
                  expect_equal(final_stats,
                               summary(get_network_control(sim, network = network, "nwstats.formula"), basis = nwL))
                } else {
                  expect_equal(final_stats,
                               summary(get_network_control(sim, network = network, "nwstats.formula"), at = nsteps, basis = sim$network[[simno]][[network]])[,,drop=TRUE])
                }
              }
            }
          }
        }
      }
    }
  }
})

test_that("multilayer specifications", {
  skip_on_cran()
  nw <- network_initialize(n = 100)

  dc1 <- dissolution_coefs(~offset(edges), 1, 0)
  dc100 <- dissolution_coefs(~offset(edges), 100, 0)
  dc200 <- dissolution_coefs(~offset(edges), 200, 0)

  est1 <- netest(nw, formation = ~edges,
                 target.stats = c(250),
                 coef.diss = dc1,
                 verbose = FALSE)

  est100 <- update_dissolution(est1, dc100)
  est200 <- update_dissolution(est1, dc200)

  param <- param.net(inf.prob = 0.3, act.rate = 0.1)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI",
                         nsims = 2,
                         nsteps = 10,
                         tergmLite = TRUE,
                         resimulate.network = TRUE,
                         tergmLite.track.duration = multilayer(TRUE, FALSE, TRUE, FALSE, FALSE),
                         nwstats.formula = multilayer(~edges + triangle + mean.age,
                                                      ~edges + degree(0:1) + mean.age,
                                                      ~edges + degree(1:2) + mean.age,
                                                      ~edges + concurrent + mean.age,
                                                      "formation"),
                         verbose = FALSE)

  basis <- list(est100, est200, est1, est1, est200)
  sim <- netsim(basis, param, init, control)

  for (network in seq_len(5L)) {
    stats <- get_nwstats(sim, network = network)
    if (network < 5L) {
      expect_true(all(stats$mean.age >= 0))
      if (get_network_control(sim, network, "tergmLite.track.duration") == TRUE) {
        expect_true(all(stats$mean.age <= 11))
      } else {
        expect_true(all(stats$mean.age >= 1000)) # large number indicating tergm defaults are being used
      }
    }
  }
})

context("Network Models with tergmLite (All SOC)")

# tergmLite = TRUE -----

test_that("tergmLite: 1G, Closed", {
  skip_on_cran()

  # SI
  num <- 1000
  nw <- network_initialize(n = num)
  formation <- ~edges
  target.stats <- 400
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 25)
  est <- netest(nw, formation, target.stats, coef.diss)

  param <- param.net(inf.prob = 0.1, act.rate = 5)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsteps = 20, nsims = 1, ncores = 1,
                         resimulate.network = FALSE, tergmLite = TRUE, verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
  plot(sim, qnts = 1)
  capture_output(
    summary(sim, at = 20)
  )

  # SIS
  param <- param.net(inf.prob = 0.5, act.rate = 1, rec.rate = 0.02)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SIS", nsteps = 10, nsims = 1, ncores = 1,
                         resimulate.network = FALSE, tergmLite = TRUE, verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
  plot(sim, qnts = 1)
  capture_output(
    summary(sim, at = 10)
  )

  # SIR
  param <- param.net(inf.prob = 0.5, act.rate = 1, rec.rate = 0.02)
  init <- init.net(i.num = 10, r.num = 5)
  control <- control.net(type = "SIR", nsteps = 10, nsims = 1, ncores = 1,
                         resimulate.network = FALSE, tergmLite = TRUE, verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
  plot(sim, qnts = 1)
  capture_output(
    summary(sim, at = 10)
  )
})

test_that("tergmLite: 2G, Closed", {
  skip_on_cran()

  # SI
  num1 <- num2 <- 500
  nw <- network_initialize(n = num1 + num2)
  nw <- set_vertex_attribute(nw, "group", rep(1:2, each = num1))
  formation <- ~edges + nodematch("group")
  target.stats <- c(400, 0)
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
  est <- netest(nw, formation, target.stats, coef.diss)

  # Parameters
  param <- param.net(inf.prob = 0.4, inf.prob.g2 = 0.2)
  init <- init.net(i.num = 20, i.num.g2 = 20)
  control <- control.net(type = "SI", nsteps = 20, nsims = 1, ncores = 1,
                         resimulate.network = FALSE, tergmLite = TRUE, verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
  plot(sim)
  capture_output(
    summary(sim, at = 20)
  )

  # SIS
  param <- param.net(inf.prob = 0.4, inf.prob.g2 = 0.2,
                     rec.rate = 0.02, rec.rate.g2 = 0.02)
  init <- init.net(i.num = 20, i.num.g2 = 20)
  control <- control.net(type = "SIS", nsteps = 10, nsims = 1, ncores = 1,
                         resimulate.network = FALSE, tergmLite = TRUE, verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
  plot(sim)
  capture_output(
    summary(sim, at = 10)
  )

  # SIR
  param <- param.net(inf.prob = 0.4, inf.prob.g2 = 0.2,
                     rec.rate = 0.02, rec.rate.g2 = 0.02)
  init <- init.net(i.num = 10, i.num.g2 = 10, r.num = 5, r.num.g2 = 5)
  control <- control.net(type = "SIR", nsteps = 10, nsims = 1, ncores = 1,
                         resimulate.network = FALSE, tergmLite = TRUE, verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
  plot(sim, mean.col = c(1, 2, 3, 1, 2, 3))
  capture_output(
    summary(sim, at = 10)
  )

})


test_that("tergmLite: 1G, Open", {
  skip_on_cran()

  # SI
  num <- 1000
  nw <- network_initialize(n = num)
  formation <- ~edges
  target.stats <- 400
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10,
                                 d.rate = 0.005)
  est <- netest(nw, formation, target.stats, coef.diss)

  # Parameters
  param <- param.net(inf.prob = 0.4, act.rate = 1,
                     a.rate = 0.005, ds.rate = 0.005, di.rate = 0.005)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsteps = 10, nsims = 1, ncores = 1,
                         resimulate.network = TRUE, tergmLite = TRUE,
                         verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
  plot(sim, qnts = FALSE, sim.lines = TRUE)
  plot(sim, y = "si.flow", ylim = c(0, 20))
  capture_output(
    summary(sim, at = 10)
  )

  # SIS
  param <- param.net(inf.prob = 0.4, act.rate = 1, rec.rate = 0.02,
                     a.rate = 0.005, ds.rate = 0.005, di.rate = 0.005)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SIS", nsteps = 10, nsims = 1, ncores = 1,
                         resimulate.network = TRUE, tergmLite = TRUE, verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
  plot(sim, qnts = FALSE, sim.lines = TRUE)
  plot(sim, y = c("si.flow", "is.flow"), legend = TRUE)
  capture_output(
    summary(sim, at = 10)
  )

  # SIR
  param <- param.net(inf.prob = 0.4, act.rate = 1, rec.rate = 0.02,
                     a.rate = 0.005, di.rate = 0.005, ds.rate = 0.005,
                     dr.rate = 0.005)
  init <- init.net(i.num = 10, r.num = 0)
  control <- control.net(type = "SIR", nsteps = 10, nsims = 1, ncores = 1,
                         resimulate.network = TRUE, tergmLite = TRUE, verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
  plot(sim, qnts = FALSE, sim.lines = TRUE)
  plot(sim, qnts = 1)
  plot(sim, y = "num", ylim = c(800, 1200))
})


test_that("tergmLite: 2G, Open", {
  skip_on_cran()

  # SI
  num1 <- num2 <- 500
  nw <- network_initialize(n = num1 + num2)
  nw <- set_vertex_attribute(nw, "group", rep(1:2, each = num1))
  formation <- ~edges + nodematch("group")
  target.stats <- c(400, 0)
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 25,
                                 d.rate = 0.005)
  est <- netest(nw, formation, target.stats, coef.diss)

  # Parameters
  param <- param.net(inf.prob = 0.5, inf.prob.g2 = 0.3,
                     act.rate = 1, a.rate = 0.005, a.rate.g2 = NA,
                     di.rate = 0.005, ds.rate = 0.005,
                     di.rate.g2 = 0.005, ds.rate.g2 = 0.005)
  init <- init.net(i.num = 50, i.num.g2 = 50)
  control <- control.net(type = "SI", nsteps = 10, nsims = 1, ncores = 1,
                         resimulate.network = TRUE, tergmLite = TRUE, verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
  plot(sim, qnts = FALSE, sim.lines = TRUE)
  plot(sim, qnts = 1, ylim = c(0, 500))

  # SIS
  param <- param.net(inf.prob = 0.5, inf.prob.g2 = 0.3,
                     act.rate = 1, a.rate = 0.005, a.rate.g2 = NA,
                     di.rate = 0.005, ds.rate = 0.005,
                     di.rate.g2 = 0.005, ds.rate.g2 = 0.005,
                     rec.rate = 0.02, rec.rate.g2 = 0.02)
  init <- init.net(i.num = 50, i.num.g2 = 50)
  control <- control.net(type = "SIS", nsteps = 10, nsims = 1, ncores = 1,
                         resimulate.network = TRUE, tergmLite = TRUE, verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
  plot(sim, qnts = FALSE, sim.lines = TRUE)
  plot(sim, qnts = 1, ylim = c(0, 500))

  # SIR
  param <- param.net(inf.prob = 0.1, inf.prob.g2 = 0.2,
                     act.rate = 5, a.rate = 0.005, a.rate.g2 = 0.005,
                     di.rate = 0.005, ds.rate = 0.005,
                     di.rate.g2 = 0.005, ds.rate.g2 = 0.005,
                     dr.rate = 0.005, dr.rate.g2 = 0.005,
                     rec.rate = 0.005, rec.rate.g2 = 0.005)
  init <- init.net(i.num = 10, i.num.g2 = 10, r.num = 5, r.num.g2 = 5)
  control <- control.net(type = "SIR", nsteps = 10, nsims = 1, ncores = 1,
                         resimulate.network = TRUE, tergmLite = TRUE, verbose = FALSE)

  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
  plot(sim, qnts = FALSE, sim.lines = TRUE)
  plot(sim, qnts = 1, ylim = c(0, 500))
})


test_that("Models with duration = 1", {
  skip_on_cran()

  num <- 1000
  nw <- network_initialize(n = num)
  formation <- ~edges
  target.stats <- 400
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 1)
  est <- netest(nw, formation, target.stats, coef.diss)

  param <- param.net(inf.prob = 0.1, act.rate = 5)
  init <- init.net(i.num = 10)

  control <- control.net(type = "SI", nsteps = 5, nsims = 1, ncores = 1,
                         tergmLite = FALSE, resimulate.network = TRUE, verbose = FALSE)
  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")

  control <- control.net(type = "SI", nsteps = 5, nsims = 1, ncores = 1,
                         tergmLite = TRUE, resimulate.network = TRUE, verbose = FALSE)
  sim <- netsim(est, param, init, control)
  expect_s3_class(sim, "netsim")
})

test_that("edgecov works with tergmLite", {
  nw <- network_initialize(n = 20)
  nw %v% "race" <- rep(c("a", "b"), length.out = 20)

  # create matrix for edgecov term
  ec <- matrix(runif(400), 20, 20)
  ec <- (ec + t(ec))/3
  nw %n% "ec" <- ec

  fit <- netest(nw,
                formation = ~edges + nodefactor("race") + edgecov("ec") ,
                target.stats = c(30, 15, 10),
                coef.diss = dissolution_coefs(dissolution = ~offset(edges), duration = 1),
                edapprox = TRUE)

  coefsign <- sign(fit$coef.form.crude[3])

  param <- param.net(inf.prob = 0.3, act.rate = 1)
  init <- init.net(i.num = 5, r.num = 0)
  control <- control.net(type = NULL, nsteps = 4, nsims = 1, ncores = 1,
                         dat.updates = function(dat, at, network) {
                           if (network == 0L && at > 0L) {
                             n <- dat$run$net_attr[[1]][["n"]]
                             m <- matrix(if(at == 3) +Inf else -Inf, n, n)*coefsign
                             dat$run$net_attr[[1]][["ec"]] <- m
                           }
                           return(dat)
                         }, tergmLite = TRUE, resimulate.network = TRUE,
                         nwstats.formula = ~edges,
                         set.control.ergm = control.simulate.formula(MCMC.burnin = 1e5),
                         verbose = FALSE)
  sim <- netsim(fit, param, init, control)
  expect_s3_class(sim, "netsim")

  expect_equal(unname(sim$stats$nwstats$sim1[[1]][2,1,drop=TRUE]), 0)
  expect_equal(unname(sim$stats$nwstats$sim1[[1]][3,1,drop=TRUE]), network.dyadcount(nw))
  expect_equal(unname(sim$stats$nwstats$sim1[[1]][4,1,drop=TRUE]), 0)
})

test_that("tergm and tergmLite produce consistent durational statistics", {
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
  est <- netest(nw, formation = ~edges + nodematch("race"),
                target.stats = c(25, 10),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)

  nsims <- 3
  nsteps <- 5

  for (trim in list(FALSE, TRUE)) {
    if (trim == TRUE) {
      est2 <- trim_netest(est)
    } else {
      est2 <- est
    }
    param <- param.net(inf.prob = 0, act.rate = 0)
    init <- init.net(i.num = 0)
    control <- control.net(type = NULL,
                           nsims = nsims,
                           nsteps = nsteps,
                           resimulate.network = TRUE,
                           tergmLite = FALSE,
                           nwstats.formula = ~edges + mean.age,
                           verbose = FALSE)

    set.seed(0)
    mod <- netsim(est2, param, init, control)
    stats <- get_nwstats(mod)

    control$tergmLite <- TRUE
    control$tergmLite.track.duration <- TRUE

    set.seed(0)
    mod2 <- netsim(est2, param, init, control)
    stats2 <- get_nwstats(mod2)

    expect_identical(stats, stats2)
    expect_true(all(stats$mean.age >= 0 & stats$mean.age <= nsteps + 1))

    control$tergmLite <- FALSE
    control$resimulate.network <- FALSE

    set.seed(0)
    mod3 <- netsim(est2, param, init, control)
    stats3 <- get_nwstats(mod3)
    expect_true(all(stats3$mean.age >= 0 & stats3$mean.age <= nsteps + 1))
  }
})

test_that("durational monitor with tergmLite", {
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))

  nsims <- 2
  nsteps <- 4

  param <- param.net(inf.prob = 0.3, act.rate = 0.4)
  init <- init.net(i.num = 10)

  for (duration in list(1,10)) {
    est <- netest(nw, formation = ~edges + nodematch("race"),
                  target.stats = c(25, 10),
                  coef.diss = dissolution_coefs(~offset(edges), duration, 0),
                  verbose = FALSE)

    for (trim in list(FALSE, TRUE)) {
      if (trim == TRUE) {
        est2 <- trim_netest(est)
      } else {
        est2 <- est
      }

      for (tergmLite.track.duration in list(FALSE, TRUE)) {
        control <- control.net(type = "SI",
                               nsims = nsims,
                               nsteps = nsteps,
                               resimulate.network = TRUE,
                               tergmLite = TRUE,
                               tergmLite.track.duration = tergmLite.track.duration,
                               nwstats.formula = ~edges + mean.age,
                               verbose = FALSE)

        mod <- netsim(est2, param, init, control)
        stats <- get_nwstats(mod)

        expect_true(all(stats$mean.age >= 0))
        invisible(capture.output(print(stats$mean.age)))
      }
    }
  }
})

context("Network Extended Models (All SOC)")

test_that("edges models", {
  skip_on_cran()

  nw <- network_initialize(n = 100)
  est <- netest(nw, formation = ~edges, target.stats = 25,
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)
  expect_is(est, "netest")

  ## "SI, 1M, CL: 1 sim"
  param <- param.net(inf.prob = 0.5)
  init <- init.net(i.num = 1)
  control <- control.net(type = "SI", nsims = 1, nsteps = 25, verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_true(max(x$epi$i.num) >= 1)
  expect_true(max(x$epi$i.num) <= 100)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network")
  plot(x, type = "network", sims = "mean", col.status = TRUE)
  test_net(x)
  rm(x)

  ## "SI, 1M, CL: 1 sim, inf.prob = 0"
  param <- param.net(inf.prob = 0)
  init <- init.net(i.num = 1)
  control <- control.net(type = "SI", nsims = 1, nsteps = 25, verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_true(max(x$epi$i.num) == 1)
  expect_true(max(x$epi$si.flow, na.rm = TRUE) == 0)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network")
  plot(x, type = "network", sims = "mean", col.status = TRUE)
  test_net(x)
  rm(x)

  ## "SI, 1M, CL: 2 sim"
  param <- param.net(inf.prob = 0.5)
  init <- init.net(i.num = 1)
  control <- control.net(type = "SI", nsims = 2, nsteps = 25, verbose = FALSE,
                         nwstats.formula = ~edges + meandeg + concurrent)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_true(max(x$epi$i.num) >= 1)
  expect_true(max(x$epi$i.num) <= 100)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "formation", plots.joined = FALSE)
  plot(x, type = "formation", stats = "edges")
  plot(x, type = "formation", stats = c("edges", "meandeg"))
  plot(x, type = "formation", sim.lines = TRUE, qnts.smooth = FALSE)
  plot(x, type = "network")
  plot(x, type = "network", sims = "mean", col.status = TRUE)
  test_net(x)
  rm(x)

  ## "SI, 1M, CL: 2 sim, use TEAs"
  param <- param.net(inf.prob = 1)
  init <- init.net(i.num = 1)
  control <- control.net(type = "SI", nsims = 2, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_true(max(x$epi$i.num) >= 1)
  expect_true(max(x$epi$i.num) <= 100)
  expect_true(sum(get.vertex.attribute.active(x$network[[1]][[1]],
                                              prefix = "testatus",
                                              at = 1) == "i") >= 0)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network")
  plot(x, type = "network", sims = "mean", col.status = TRUE)
  test_net(x)
  rm(x)

  ## "SI, 1M, CL: 2 sim, set status.vector"
  param <- param.net(inf.prob = 0.5)
  init <- init.net(status.vector = c(rep("i", 10),
                                     rep("s", 90)))
  control <- control.net(type = "SI", nsims = 2, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  # this tests that starting infected is same across sims
  expect_true(all(x$epi$i.num[1, ] == 10))
  expect_true(max(x$epi$i.num) <= 100)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network", col.status = TRUE)
  test_net(x)
  rm(x)

  ## "SIR, 1M, CL: 1 sim"
  param <- param.net(inf.prob = 0.5, rec.rate = 0.01)
  init <- init.net(i.num = 10, r.num = 0)
  control <- control.net(type = "SIR", nsims = 1, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_true(max(x$epi$i.num) >= 1)
  expect_true(max(x$epi$i.num) <= 100)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network")
  plot(x, type = "network", sims = "mean", col.status = TRUE)
  test_net(x)
  rm(x)

  ## "SIR, 1M, CL: 1 sim, inf.prob=0"
  param <- param.net(inf.prob = 0, rec.rate = 0.01)
  init <- init.net(i.num = 10, r.num = 0)
  control <- control.net(type = "SIR", nsims = 1,
                         nsteps = 25, verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_true(max(x$epi$i.num) == 10)
  expect_true(max(x$epi$si.flow, na.rm = TRUE) == 0)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network")
  plot(x, type = "network", sims = "mean", col.status = TRUE)
  test_net(x)
  rm(x)

  ## "SIR, 1M, CL: 2 sim"
  param <- param.net(inf.prob = 0.5, rec.rate = 0.1)
  init <- init.net(i.num = 1, r.num = 0)
  control <- control.net(type = "SIR", nsims = 2, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_true(max(x$epi$i.num) >= 1)
  expect_true(max(x$epi$i.num) <= 100)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network")
  plot(x, type = "network", sims = "mean", col.status = TRUE)
  test_net(x)
  rm(x)

  ## "SIR, 1M, CL: 2 sim, set status.vector"
  param <- param.net(inf.prob = 0.5, rec.rate = 0.1)
  init <- init.net(status.vector = rep(c("s", "i"), each = 50))
  control <- control.net(type = "SIR", nsims = 2, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  # this tests that starting infected is same across sims
  expect_true(all(x$epi$i.num[1, ] == 50))
  expect_true(max(x$epi$i.num) <= 100)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network")
  plot(x, type = "network", sims = "mean", col.status = TRUE)
  test_net(x)
  rm(x)

  ## "SIS, 1M, CL: 1 sim"
  param <- param.net(inf.prob = 0.9, rec.rate = 0.01)
  init <- init.net(i.num = 1)
  control <- control.net(type = "SIS", nsims = 1, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_true(max(x$epi$i.num) >= 1)
  expect_true(max(x$epi$i.num) <= 100)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network")
  plot(x, type = "network", sims = "mean", col.status = TRUE)
  test_net(x)
  rm(x)

  ## "SIS, 1M, CL: 1 sim, inf.prob=0"
  param <- param.net(inf.prob = 0, rec.rate = 0.01)
  init <- init.net(i.num = 1)
  control <- control.net(type = "SIS", nsims = 1, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_true(max(x$epi$i.num) == 1)
  expect_true(max(x$epi$si.flow, na.rm = TRUE) == 0)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network")
  plot(x, type = "network", sims = "mean", col.status = TRUE)
  test_net(x)
  rm(x)

  ## "SIS, 1M, CL: 2 sim"
  param <- param.net(inf.prob = 0.5, rec.rate = 0.01)
  init <- init.net(i.num = 1)
  control <- control.net(type = "SIS", nsims = 2, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_true(max(x$epi$i.num) >= 1)
  expect_true(max(x$epi$i.num) <= 100)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "SIS, 1M, CL: 2 sim, test TEAs"
  param <- param.net(inf.prob = 1, rec.rate = 0.01)
  init <- init.net(i.num = 1)
  control <- control.net(type = "SIS", nsims = 2, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_true(max(x$epi$i.num) <= 100)
  expect_true(sum(get.vertex.attribute.active(x$network[[1]][[1]],
                                              prefix = "testatus",
                                              at = 1) == "i") >= 0)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network")
  plot(x, type = "network", sims = "mean", col.status = TRUE)
  test_net(x)
  rm(x)

  ## "SIS, 1M, CL: 2 sim, set status.vector"
  param <- param.net(inf.prob = 0.5, rec.rate = 0.01)
  init <- init.net(status.vector = c(rep("i", 10), rep("s", 90)))
  control <- control.net(type = "SIS", nsims = 2, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  # this tests that starting infected is same across sims
  expect_true(all(x$epi$i.num[1, ] == 10))
  expect_true(max(x$epi$i.num) <= 100)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network")
  plot(x, type = "network", sims = "mean", col.status = TRUE)
  test_net(x)
})


################################################################################

test_that("High departure rate models", {
  skip_on_cran()
  ## "netsim: 1M, ds.rate = 0.5"
  nw <- network_initialize(n = 25)
  est <- netest(nw, formation = ~edges, target.stats = 12,
                coef.diss = dissolution_coefs(~offset(edges), 10, 0.01),
                edapprox = TRUE, verbose = FALSE)
  param <- param.net(inf.prob = 0.5, act.rate = 2,
                     a.rate = 0.01, ds.rate = 0.25,
                     di.rate = 0.1)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsteps = 25, nsims = 1,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_equal(unique(sapply(x$epi, nrow)), 25)
  expect_output(summary(x, at = 25), "EpiModel Summary")

  capture_output(
    get_nwstats(x)
  )

  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "netsim: 1M, di.rate = 0.5"
  param <- param.net(inf.prob = 0.1, act.rate = 2, a.rate = 0.01,
                     ds.rate = 0.01, di.rate = 0.25)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsteps = 25, nsims = 1,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_equal(unique(sapply(x$epi, nrow)), 25)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)

})

################################################################################

test_that("erroneous two-group models", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  nw <- set_vertex_attribute(nw, "group", rep(0:1, each = 50))
  est <- netest(nw, formation = ~edges, target.stats = 25,
                 coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                 edapprox = TRUE, verbose = FALSE)

  param <- param.net(inf.prob = 0.5, inf.prob.g2 = 0.1)
  init <- init.net(i.num = 10, i.num.g2 = 0)
  control <- control.net(type = "SI", nsims = 2, nsteps = 25, verbose = FALSE)
  expect_error(netsim(est, param, init, control))
})


################################################################################

test_that("edges two-group models", {
  skip_on_cran()

  nw <- network_initialize(n = 100)
  nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
  est5 <- netest(nw, formation = ~edges, target.stats = 25,
                 coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                 edapprox = TRUE, verbose = FALSE)
  expect_is(est5, "netest")

  ## "SI, 2M, CL: 2 sim"
  param <- param.net(inf.prob = 0.5, inf.prob.g2 = 0.1)
  init <- init.net(i.num = 10, i.num.g2 = 0)
  control <- control.net(type = "SI", nsims = 2, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est5, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$groups, 2)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  plot(x, type = "network")
  plot(x, type = "network", shp.g2 = "triangle")
  #FLAG: Adding "two-group" attribute to network type
  expect_error(plot(x, type = "network", shp.g2 = TRUE))
  test_net(x)
  rm(x)

  ## "SIR, 2M, CL: 2 sim"
  param <- param.net(inf.prob = 0.5, inf.prob.g2 = 0.1,
                     rec.rate = 0.1, rec.rate.g2 = 0.1)
  init <- init.net(i.num = 10, i.num.g2 = 10,
                   r.num = 0, r.num.g2 = 0)
  control <- control.net(type = "SIR", nsims = 2, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est5, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$groups, 2)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "SIR, 2M, CL: rec.rate = 0"
  param <- param.net(inf.prob = 0.5, inf.prob.g2 = 0.25,
                     rec.rate = 0, rec.rate.g2 = 0, act.rate = 2)
  init <- init.net(i.num = 10, i.num.g2 = 10,
                   r.num = 0, r.num.g2 = 0)
  control <- control.net(type = "SIR", nsteps = 10, nsims = 2,
                         verbose = FALSE)
  x <- netsim(est5, param, init, control)
  expect_equal(max(x$epi$ir.flow, na.rm = TRUE), 0)
  expect_equal(max(x$epi$ir.flow.g2, na.rm = TRUE), 0)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "SIS, 2M, CL: 2 sim"
  param <- param.net(inf.prob = 0.5, inf.prob.g2 = 0.25,
                     rec.rate = 0.01, rec.rate.g2 = 0.01)
  init <- init.net(i.num = 10, i.num.g2 = 10)
  control <- control.net(type = "SIS", nsims = 2, nsteps = 25,
                         verbose = FALSE)
  x <- netsim(est5, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$groups, 2)
  expect_output(summary(x, at = 25), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "SIS, 2M, CL: rec.rate = 0"
  param <- param.net(inf.prob = 0.5, inf.prob.g2 = 0.25,
                     rec.rate = 0, rec.rate.g2 = 0, act.rate = 2)
  init <- init.net(i.num = 10, i.num.g2 = 10)
  control <- control.net(type = "SIS", nsteps = 10, nsims = 2,
                         verbose = FALSE)
  x <- netsim(est5, param, init, control)
  expect_equal(max(x$epi$is.flow, na.rm = TRUE), 0)
  expect_equal(max(x$epi$is.flow.g2, na.rm = TRUE), 0)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

})

################################################################################

test_that("Open population 1 group models", {
  skip_on_cran()

  nw <- network_initialize(n = 100)
  est.vit <- netest(nw, formation = ~edges, target.stats = 25,
                    coef.diss = dissolution_coefs(~offset(edges), 10, 0.01),
                    verbose = FALSE)

  ## "SI, 1M, OP: 1 sim"
  param <- param.net(inf.prob = 0.5, act.rate = 2, a.rate = 0.01,
                     ds.rate = 0.01, di.rate = 0.01)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsteps = 10, nsims = 1,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est.vit, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$vital, TRUE)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "SI, 1M, OP: 2 sim"
  param <- param.net(inf.prob = 0.5, act.rate = 2, a.rate = 0.01,
                     ds.rate = 0.01, di.rate = 0.01)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsteps = 10, nsims = 2,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est.vit, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$vital, TRUE)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "SIR, 1M OP: 1 sim"
  param <- param.net(inf.prob = 0.5, rec.rate = 0.1, act.rate = 2,
                     a.rate = 0.01, ds.rate = 0.01, di.rate = 0.01,
                     dr.rate = 0.01)
  init <- init.net(i.num = 10, r.num = 0)
  control <- control.net(type = "SIR", nsteps = 10, nsims = 1,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est.vit, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$vital, TRUE)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "SIR, 1M, OP: 2 sim"
  param <- param.net(inf.prob = 0.5, rec.rate = 0.1, act.rate = 2,
                     a.rate = 0.01, ds.rate = 0.01, di.rate = 0.01,
                     dr.rate = 0.01)
  init <- init.net(i.num = 10, r.num = 0)
  control <- control.net(type = "SIR", nsteps = 10, nsims = 2,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est.vit, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$vital, TRUE)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "SIS, 1M, OP: 1 sim"
  param <- param.net(inf.prob = 0.5, rec.rate = 0.01, act.rate = 2,
                     a.rate = 0.01, ds.rate = 0.01, di.rate = 0.01)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SIS", nsteps = 10, nsims = 1,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est.vit, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$vital, TRUE)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "SIS, 1M, OP: 2 sim"
  control <- control.net(type = "SIS", nsteps = 10, nsims = 2,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est.vit, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$vital, TRUE)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

})

################################################################################

test_that("Open-population two-group models", {
  skip_on_cran()

  nw <- network_initialize(n = 100)
  nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
  est5.vit <- netest(nw, formation = ~edges + nodematch("group"),
                     target.stats = c(25, 0),
                     coef.diss = dissolution_coefs(~offset(edges), 10, 0.01),
                     edapprox = TRUE, verbose = FALSE)

  ## "SI, 2M, OP: 1 sim"
  param <- param.net(inf.prob = 0.5, inf.prob.g2 = 0.1, act.rate = 2,
                     a.rate = 0.01, ds.rate = 0.01, di.rate = 0.01,
                     a.rate.g2 = 0.01, ds.rate.g2 = 0.01, di.rate.g2 = 0.01)
  init <- init.net(i.num = 10, i.num.g2 = 10)
  control <- control.net(type = "SI", nsteps = 10, nsims = 1,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est5.vit, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$vital, TRUE)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)


  ## "SI, 2M, OP: 2 sim"
  param <- param.net(inf.prob = 0.5, inf.prob.g2 = 0.1, act.rate = 2,
                     a.rate = 0.01, ds.rate = 0.01, di.rate = 0.01,
                     a.rate.g2 = 0.01, ds.rate.g2 = 0.01, di.rate.g2 = 0.01)
  init <- init.net(i.num = 10, i.num.g2 = 10)
  control <- control.net(type = "SI", nsteps = 10, nsims = 2,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est5.vit, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$vital, TRUE)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "SIR, 2M, OP: 1 sim"
  param <- param.net(inf.prob = 0.5, inf.prob.g2 = 0.1, rec.rate = 0.1,
                     rec.rate.g2 = 0.1, act.rate = 2, a.rate = 0.01,
                     a.rate.g2 = NA, ds.rate = 0.01, ds.rate.g2 = 0.01,
                     di.rate = 0.01, di.rate.g2 = 0.01, dr.rate = 0.01,
                     dr.rate.g2 = 0.01)
  init <- init.net(i.num = 10, i.num.g2 = 0,
                   r.num = 0, r.num.g2 = 10)
  control <- control.net(type = "SIR", nsteps = 10, nsims = 1,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est5.vit, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$vital, TRUE)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "SIR, 2M, OP: 3 sim"
  control <- control.net(type = "SIR", nsteps = 10, nsims = 2,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est5.vit, param, init, control)
  expect_is(x, "netsim")
  expect_is(as.data.frame(x), "data.frame")
  expect_equal(x$param$vital, TRUE)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

})

################################################################################

test_that("Extinction open-population models", {
  skip_on_cran()

  nw <- network_initialize(n = 25)
  nw <- set_vertex_attribute(nw, "group", rep(1:2, c(15, 10)))
  est <- netest(nw, formation = ~edges + nodematch("group"),
                target.stats = c(15, 0),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0.01),
                edapprox = TRUE, verbose = FALSE)

  ## "netsim: 2M, ds.rate = 0.5"
  param <- param.net(inf.prob = 0.1, inf.prob.g2 = 0.1, act.rate = 2,
                     a.rate = 0.01, ds.rate = 0.5, di.rate = 0.5,
                     a.rate.g2 = 0.01, ds.rate.g2 = 0.01, di.rate.g2 = 0.01)
  init <- init.net(i.num = 5, i.num.g2 = 0)
  control <- control.net(type = "SI", nsteps = 30, nsims = 1,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

  ## "netsim: 2M, ds.rate.g2 = 0.5"
  param <- param.net(inf.prob = 0.1, inf.prob.g2 = 0.1, act.rate = 2,
                     a.rate = 0.01, ds.rate = 0.01, di.rate = 0.01,
                     a.rate.g2 = 0.01, ds.rate.g2 = 0.5, di.rate.g2 = 0.5)
  init <- init.net(i.num = 5, i.num.g2 = 0)
  control <- control.net(type = "SI", nsteps = 30, nsims = 1,
                         resimulate.network = TRUE, verbose = FALSE)
  x <- netsim(est, param, init, control)
  expect_output(summary(x, at = 10), "EpiModel Summary")
  plot(x)
  plot(x, y = "si.flow", mean.smooth = TRUE)
  plot(x, type = "formation")
  test_net(x)
  rm(x)

})


################################################################################

test_that("status.vector and infTime.vector", {
  skip_on_cran()
  n <- 100
  nw <- network_initialize(n = n)
  formation <- ~edges
  target.stats <- 50
  coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
  est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

  # Epidemic model
  param <- param.net(inf.prob = 0.3, rec.rate = 0.01)

  status <- sample(c("s", "i"), size = n, replace = TRUE, prob = c(0.8, 0.2))
  infTime <- rep(NA, n)
  infTime[which(status == "i")] <- -rgeom(sum(status == "i"), prob = 0.01) + 2
  init <- init.net(status.vector = status, infTime.vector = infTime)

  control <- control.net(type = "SIS", nsteps = 100, nsims = 5, verbose = FALSE)
  mod1 <- netsim(est1, param, init, control)
  expect_is(mod1, "netsim")

  control <- control.net(type = "SIR", nsteps = 100, nsims = 5, verbose = FALSE)
  mod2 <- netsim(est1, param, init, control)
  expect_is(mod2, "netsim")

})

context("Network Estimation")


# Test formation models ---------------------------------------------------

test_that("netest works for edges only model", {
  nw <- network_initialize(n = 50)
  est <- netest(nw, formation = ~edges, target.stats = 25,
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)
  expect_is(est, "netest")
})


test_that("netest works for edges + nodematch model", {
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
  est <- netest(nw, formation = ~edges + nodematch("race"),
                target.stats = c(25, 10),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)
  expect_is(est, "netest")
})


test_that("netest works with offset.coef terms", {
  nw <- network_initialize(n = 100)
  nw <- set_vertex_attribute(nw, "role", rep(c("I", "V", "R"), c(10, 80, 10)))
  est <- netest(nw, formation = ~edges + offset(nodematch("role", diff = TRUE,
                                                          levels = 1:2)),
                coef.form = c(-Inf, -Inf), target.stats = c(40),
                coef.diss = dissolution_coefs(~offset(edges), 52 * 2, 0.0009),
                verbose = FALSE)
  expect_is(est, "netest")
})



# Test dissolution models -------------------------------------------------

test_that("netest works for heterogeneous dissolutions", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
  est <- netest(nw, formation = ~edges + nodematch("race"),
                target.stats = c(50, 20),
                coef.diss = dissolution_coefs(~offset(edges) + offset(nodematch("race")),
                                              c(10, 20)),
                verbose = FALSE
  )
  expect_is(est, "netest")
})

test_that("netest diss_check flags bad models", {
  nw <- network_initialize(n = 100)
  nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))

  formation <- ~edges + nodematch("race", diff = TRUE)
  dissolution <- ~offset(edges) + offset(nodematch("race"))
  coef.diss <- dissolution_coefs(dissolution, c(10, 20))
  expect_error(netest(nw, formation, target.stats = c(50, 20), coef.diss,
                      verbose = FALSE),
               "Term options for one or more terms in dissolution model")
})


# Other tests ---------------------------------------------------------------

test_that("Error if incorrect coef.diss parameter", {
  nw <- network_initialize(n = 50)
  coef.diss <- 1
  expect_error(netest(nw, formation = ~edges, target.stats = 25,
                      coef.diss = coef.diss,
                verbose = FALSE))
})

test_that("update_dissolution tests", {
  skip_on_cran()
  nw <- network_initialize(n = 1000)

  diss.300 <- dissolution_coefs(~offset(edges), 300, 0.001)
  diss.200 <- dissolution_coefs(~offset(edges), 200, 0.001)

  est300 <- netest(nw = nw,
                  formation = ~edges,
                  target.stats = 500,
                  coef.diss = diss.300)

  est200 <- netest(nw = nw,
                  formation = ~edges,
                  target.stats = 500,
                  coef.diss = diss.200)

  est200.compare <- update_dissolution(est300, diss.200)
  expect_true(round(as.numeric(est200$coef.form), 3) ==
                round(as.numeric(est200.compare$coef.form), 3))

  expect_error(update_dissolution(1, diss.200),
               "old.netest must be an object of class netest")
  expect_error(update_dissolution(est300, 1),
               "new.coef.diss must be an object of class disscoef")
  est300$edapprox <- FALSE
  expect_error(update_dissolution(est300, diss.200),
               "Edges dissolution approximation must be used")
})

test_that("differing length update_dissolution tests", {
  skip_on_cran()
  nw <- network_initialize(n = 1000)
  nw %v% "race" <- rep(letters[1:5], length.out = 1000)

  diss_het <- dissolution_coefs(~offset(edges) +
                                  offset(nodematch("race", diff = TRUE)),
                                c(300, 200, 100, 50, 150, 225), 0.001)
  diss_hom <- dissolution_coefs(~offset(edges), 200, 0.001)

  est_het <- netest(nw = nw,
                    formation = ~edges + nodematch("race", diff = TRUE),
                    target.stats = c(500, 50, 50, 90, 30, 10),
                    coef.diss = diss_het)

  est_hom <- netest(nw = nw,
                    formation = ~edges + nodematch("race", diff = TRUE),
                    target.stats = c(500, 50, 50, 90, 30, 10),
                    coef.diss = diss_hom)

  est_hom_compare <- update_dissolution(est_het, diss_hom)
  est_het_compare <- update_dissolution(est_hom, diss_het)

  expect_true(all(round(as.numeric(est_hom$coef.form), 3) ==
                round(as.numeric(est_hom_compare$coef.form), 3)))

  expect_true(all(round(as.numeric(est_het$coef.form), 3) ==
                round(as.numeric(est_het_compare$coef.form), 3)))
})

test_that("duration 1 update_dissolution tests", {
  skip_on_cran()
  nw <- network_initialize(n = 1000)
  nw %v% "race" <- rep(letters[1:5], length.out = 1000)

  diss_het <- dissolution_coefs(~offset(edges) +
                                  offset(nodematch("race", diff = TRUE)),
                                c(300, 200, 100, 50, 150, 225), 0.001)

  diss_hom_1 <- dissolution_coefs(~offset(edges), 1, 0)


  est_het <- netest(nw = nw,
                    formation = ~edges + nodemix("race", levels2 = 1:5),
                    target.stats = c(500, 50, 50, 90, 30, 10),
                    coef.diss = diss_het, nested.edapprox = FALSE)

  est_hom_1 <- netest(nw = nw,
                    formation = ~edges + nodemix("race", levels2 = 1:5),
                    target.stats = c(500, 50, 50, 90, 30, 10),
                    coef.diss = diss_hom_1)

  est_hom_1_update <- update_dissolution(est_het, diss_hom_1)
  est_het_update <- update_dissolution(est_hom_1, diss_het, nested.edapprox = FALSE)

  expect_true(all(round(as.numeric(est_hom_1$coef.form), 3) ==
                round(as.numeric(est_hom_1_update$coef.form), 3)))

  expect_true(all(round(as.numeric(est_het$coef.form), 3) ==
                round(as.numeric(est_het_update$coef.form), 3)))

  dx <- netdx(est_het, dynamic = TRUE, nsteps = 5, verbose = FALSE)
  dx <- netdx(est_het, dynamic = FALSE, nsims = 5, verbose = FALSE)

  expect_error(dx <- netdx(est_hom_1, dynamic = TRUE, nsteps = 5),
               "Running dynamic diagnostics on a cross-sectional ERGM")
  dx <- netdx(est_hom_1, dynamic = FALSE, nsims = 5, verbose = FALSE)

  dx <- netdx(est_het_update, dynamic = TRUE, nsteps = 5, verbose = FALSE)
  dx <- netdx(est_het_update, dynamic = FALSE, nsims = 5, verbose = FALSE)

  expect_error(dx <- netdx(est_hom_1_update, dynamic = TRUE, nsteps = 5),
               "Running dynamic diagnostics on a cross-sectional ERGM")
  dx <- netdx(est_hom_1_update, dynamic = FALSE, nsims = 5, verbose = FALSE)

  param <- param.net(inf.prob = 0.3, act.rate = 0.5, rec.rate = 0.05)
  init <- init.net(r.num = 0, status.vector = rep("s", 1000))
  control <- control.net(type = "SIR", nsims = 1, nsteps = 5,
                         resimulate.network = TRUE, verbose = FALSE,
                         nwupdate.FUN = NULL)
  mod <- netsim(est_het, param, init, control)
  mod <- netsim(est_het_update, param, init, control)
  mod <- netsim(est_hom_1, param, init, control)
  mod <- netsim(est_hom_1_update, param, init, control)
})

test_that("differing length non-nested update_dissolution tests", {
  skip_on_cran()
  nw <- network_initialize(n = 1000)
  nw %v% "race" <- rep(letters[1:5], length.out = 1000)
  nw %v% "age" <- rep(1:3, length.out = 1000)

  diss_het <- dissolution_coefs(~offset(edges) +
                                  offset(nodematch("age", diff = TRUE)),
                                c(300, 50, 150, 225), 0.001)
  diss_hom <- dissolution_coefs(~offset(edges), 200, 0.001)

  est_het <- netest(nw = nw,
                    formation = ~edges + nodematch("race", diff = TRUE),
                    target.stats = c(500, 50, 50, 90, 30, 10),
                    coef.diss = diss_het,
                    nested.edapprox = FALSE)

  est_hom <- netest(nw = nw,
                    formation = ~edges + nodematch("race", diff = TRUE),
                    target.stats = c(500, 50, 50, 90, 30, 10),
                    coef.diss = diss_hom,
                    nested.edapprox = FALSE)

  est_hom_compare <- update_dissolution(est_het, diss_hom, nested.edapprox = FALSE)
  est_het_compare <- update_dissolution(est_hom, diss_het, nested.edapprox = FALSE)

  expect_true(all(round(as.numeric(est_hom$coef.form), 3) ==
                round(as.numeric(est_hom_compare$coef.form), 3)))

  expect_true(all(round(as.numeric(est_het$coef.form), 3) ==
                round(as.numeric(est_het_compare$coef.form), 3)))

  expect_equal(est_hom_compare$formation, ~Passthrough(formation) + Passthrough(dissolution))
  expect_equal(eval(quote(formation), envir = environment(est_hom_compare$formation)),
               ~edges + nodematch("race", diff = TRUE))
  expect_equal(eval(quote(dissolution), envir = environment(est_hom_compare$formation)),
               ~offset(edges))

  expect_equal(est_het_compare$formation, ~Passthrough(formation) + Passthrough(dissolution))
  expect_equal(eval(quote(formation), envir = environment(est_het_compare$formation)),
               ~edges + nodematch("race", diff = TRUE))
  expect_equal(eval(quote(dissolution), envir = environment(est_het_compare$formation)),
               ~offset(edges) + offset(nodematch("age", diff = TRUE)))
})

test_that("non-nested EDA", {
  skip_on_cran()
  nw <- network_initialize(n = 1000)
  nw %v% "race" <- rep(letters[1:5], length.out = 1000)
  nw %v% "age" <- rep(1:3, length.out = 1000)

  diss_het_1 <- dissolution_coefs(~offset(edges) +
                                    offset(nodematch("race", diff = TRUE)),
                                  c(300, 200, 100, 50, 150, 225), 0.001)
  diss_het_2 <- dissolution_coefs(~offset(edges) +
                                    offset(nodematch("age", diff = TRUE)),
                                  c(55, 45, 65, 80), 0.001)

  est_nest_1 <- netest(nw = nw,
                       formation = ~edges + nodematch("race", diff = TRUE),
                       target.stats = c(500, 50, 50, 90, 30, 10),
                       coef.diss = diss_het_1)

  est_non_1 <- netest(nw = nw,
                      formation = ~edges + nodematch("race", diff = TRUE),
                      target.stats = c(500, 50, 50, 90, 30, 10),
                      coef.diss = diss_het_1,
                      nested = FALSE)

  expect_error(est_nest_2 <- netest(nw = nw,
                                    formation = ~edges + nodematch("race", diff = TRUE),
                                    target.stats = c(500, 50, 50, 90, 30, 10),
                                    coef.diss = diss_het_2),
                                    "Term options for one or more terms in dissolution model")

  est_non_2 <- netest(nw = nw,
                      formation = ~edges + nodematch("race", diff = TRUE),
                      target.stats = c(500, 50, 50, 90, 30, 10),
                      coef.diss = diss_het_2,
                      nested = FALSE)

  est_nest_1_non_2 <- update_dissolution(est_nest_1, diss_het_2, nested = FALSE)
  est_non_1_non_2 <- update_dissolution(est_non_1, diss_het_2, nested = FALSE)
  expect_error(est_nest_1_nest_2 <- update_dissolution(est_nest_1, diss_het_2, nested = TRUE),
               "Term options for one or more terms in dissolution model")
  expect_error(est_non_1_nest_2 <- update_dissolution(est_non_1, diss_het_2, nested = TRUE),
               "Term options for one or more terms in dissolution model")
  est_non_2_nest_1 <- update_dissolution(est_non_2, diss_het_1, nested = TRUE)
  est_non_2_non_1 <- update_dissolution(est_non_2, diss_het_1, nested = FALSE)


  expect_true(all(round(as.numeric(est_nest_1$coef.form), 3) ==
                round(as.numeric(est_non_2_nest_1$coef.form), 3)))

  expect_true(all(round(as.numeric(est_non_1$coef.form), 3) ==
                round(as.numeric(est_non_2_non_1$coef.form), 3)))

  expect_true(all(round(as.numeric(est_non_2$coef.form), 3) ==
                round(as.numeric(est_nest_1_non_2$coef.form), 3)))

  expect_true(all(round(as.numeric(est_non_2$coef.form), 3) ==
                round(as.numeric(est_non_1_non_2$coef.form), 3)))

  expect_equal(est_nest_1$formation, ~edges + nodematch("race", diff = TRUE))

  expect_equal(est_non_1$formation, ~Passthrough(formation) + Passthrough(dissolution))
  expect_equal(eval(quote(formation), envir = environment(est_non_1$formation)),
               ~edges + nodematch("race", diff = TRUE))
  expect_equal(eval(quote(dissolution), envir = environment(est_non_1$formation)),
               ~offset(edges) + offset(nodematch("race", diff = TRUE)))

  expect_equal(est_non_2$formation, ~Passthrough(formation) + Passthrough(dissolution))
  expect_equal(eval(quote(formation), envir = environment(est_non_2$formation)),
               ~edges + nodematch("race", diff = TRUE))
  expect_equal(eval(quote(dissolution), envir = environment(est_non_2$formation)),
               ~offset(edges) + offset(nodematch("age", diff = TRUE)))

  expect_equal(est_nest_1_non_2$formation, ~Passthrough(formation) + Passthrough(dissolution))
  expect_equal(eval(quote(formation), envir = environment(est_nest_1_non_2$formation)),
               ~edges + nodematch("race", diff = TRUE))
  expect_equal(eval(quote(dissolution), envir = environment(est_nest_1_non_2$formation)),
               ~offset(edges) + offset(nodematch("age", diff = TRUE)))

  expect_equal(est_non_1_non_2$formation, ~Passthrough(formation) + Passthrough(dissolution))
  expect_equal(eval(quote(formation), envir = environment(est_non_1_non_2$formation)),
               ~edges + nodematch("race", diff = TRUE))
  expect_equal(eval(quote(dissolution), envir = environment(est_non_1_non_2$formation)),
               ~offset(edges) + offset(nodematch("age", diff = TRUE)))

  expect_equal(est_non_2_nest_1$formation, ~edges + nodematch("race", diff = TRUE))

  expect_equal(est_non_2_non_1$formation, ~Passthrough(formation) + Passthrough(dissolution))
  expect_equal(eval(quote(formation), envir = environment(est_non_2_non_1$formation)),
               ~edges + nodematch("race", diff = TRUE))
  expect_equal(eval(quote(dissolution), envir = environment(est_non_2_non_1$formation)),
               ~offset(edges) + offset(nodematch("race", diff = TRUE)))
})

test_that("environment handling in non-nested EDA", {
  skip_on_cran()
  nw <- network_initialize(n = 1000)
  nw %v% "race" <- rep(letters[1:5], length.out = 1000)
  nw %v% "age" <- rep(1:3, length.out = 1000)

  make_formula <- function() {
    a <- "age"
    ff <- ~offset(edges) + offset(nodematch(a, diff = TRUE))
    ff
  }

  ff <- make_formula()

  coef_diss <- dissolution_coefs(ff, c(55, 45, 65, 80))

  r <- "race"

  netest_1 <- netest(nw = nw,
                     formation = ~edges + nodematch(r, diff = TRUE),
                     target.stats = c(500, 50, 50, 90, 40, 60),
                     coef.diss = coef_diss,
                     nested.edapprox = FALSE)

  netdx_1 <- netdx(netest_1, nsims = 2, nsteps = 5, dynamic = TRUE, verbose = FALSE)

  make_formula_2 <- function() {
    x <- "race"
    ff <- ~offset(edges) + offset(nodematch(x, diff = TRUE))
    ff
  }

  ff_2 <- make_formula_2()

  coef_diss_2 <- dissolution_coefs(ff_2, c(30, 40, 50, 25, 35, 55))

  expect_error(netest_2 <- update_dissolution(netest_1, coef_diss_2,
                                              nested.edapprox = TRUE),
               "Term options for one or more terms in dissolution model")
  netest_2 <- update_dissolution(netest_1, coef_diss_2, nested.edapprox = FALSE)
  netdx_2 <- netdx(netest_2, nsims = 2, nsteps = 5, dynamic = TRUE, verbose = FALSE)
})

test_that("non-nested EDA produces expected statistic names, with or without trimming", {
  skip_on_cran()
  for (trim in c(FALSE, TRUE)) {
    nw <- network.initialize(10, directed = FALSE)
    nw %v% "race" <- rep(1:3, length.out = 10)
    nw %v% "age" <- rep(1:2, length.out = 10)

    coef_diss <- dissolution_coefs(~offset(edges) + offset(nodematch("age", diff = TRUE)),
                                   c(10, 25, 20))
    est <- netest(nw,
                  formation = ~edges + nodematch("race", diff = TRUE),
                  target.stats = c(10, 3, 2, 1),
                  coef.diss = coef_diss,
                  nested.edapprox = FALSE)

    if (trim == TRUE) {
      est <- trim_netest(est)
    }

    formation_names <- c("edges", "nodematch.race.1", "nodematch.race.2",
                         "nodematch.race.3", "offset(edges)", "offset(nodematch.age.1)",
                         "offset(nodematch.age.2)")

    dxs <- netdx(est, nsims = 100, dynamic = FALSE, verbose = FALSE)
    expect_equal(rownames(dxs$stats.table.formation), formation_names)

    dxd <- netdx(est, nsims = 2, nsteps = 7, dynamic = TRUE, verbose = FALSE)
    expect_equal(rownames(dxd$stats.table.formation), formation_names)

    param <- param.net(inf.prob = 0.3, act.rate = 0.5)
    init <- init.net(i.num = 3)
    control <- control.net(type = "SI", nsims = 1, nsteps = 5, verbose = FALSE)
    sim <- netsim(est, param, init, control)
    expect_equal(colnames(get_nwstats(sim, mode = "list")[[1]]), formation_names)
  }
})

test_that("trimming non-nested EDA fails when it should", {
  skip_on_cran()
  nw <- network.initialize(10, directed = FALSE)
  nw %v% "race" <- rep(1:3, length.out = 10)
  nw %v% "age" <- rep(1:2, length.out = 10)

  coef_diss <- dissolution_coefs(~offset(edges) + offset(nodematch("age", diff = TRUE)),
                                 c(10, 25, 20))

  r <- "race"

  est <- netest(nw,
                formation = ~edges + nodematch(r, diff = TRUE),
                target.stats = c(10, 3, 2, 1),
                coef.diss = coef_diss,
                nested.edapprox = FALSE)

  est <- trim_netest(est)

  expect_error(dxs <- netdx(est, nsims = 100, dynamic = FALSE, verbose = FALSE), "object 'r' not found")
  expect_error(dxd <- netdx(est, nsims = 2, nsteps = 7, dynamic = TRUE, verbose = FALSE), "object 'r' not found")

  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 3)
  control <- control.net(type = "SI", nsims = 1, nsteps = 5, verbose = FALSE)
  expect_error(sim <- netsim(est, param, init, control), "object 'r' not found")

  a <- "age"
  coef_diss <- dissolution_coefs(~offset(edges) + offset(nodematch(a, diff = TRUE)),
                                 c(10, 25, 20))

  est <- netest(nw,
                formation = ~edges + nodematch("race", diff = TRUE),
                target.stats = c(10, 3, 2, 1),
                coef.diss = coef_diss,
                nested.edapprox = FALSE)

  est <- trim_netest(est)

  expect_error(dxs <- netdx(est, nsims = 100, dynamic = FALSE, verbose = FALSE), "object 'a' not found")
  expect_error(dxd <- netdx(est, nsims = 2, nsteps = 7, dynamic = TRUE, verbose = FALSE), "object 'a' not found")

  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 3)
  control <- control.net(type = "SI", nsims = 1, nsteps = 5, verbose = FALSE)
  expect_error(sim <- netsim(est, param, init, control), "object 'a' not found")
})

test_that("non-nested EDA with substitutions", {
  skip_on_cran()
  nw <- network_initialize(n = 100)
  nw %v% "race" <- rep(letters[1:3], length.out = 100)
  nw %v% "age" <- rep(1:2, length.out = 100)

  make_form <- function() {
    a <- "race"
    ff <- ~edges + nodematch(a, diff = TRUE)
    ff
  }

  make_diss_1 <- function() {
    a <- "age"
    ff <- ~offset(edges) + offset(nodematch(a, diff = TRUE))
    ff
  }

  make_diss_2 <- function() {
    a <- "race"
    ff <- ~offset(edges) + offset(nodematch(a, diff = TRUE))
    ff
  }

  make_diss_3 <- function() {
    b <- "age"
    ff <- ~offset(edges) + offset(nodematch(b, diff = TRUE))
    ff
  }

  make_diss_4 <- function() {
    b <- "race"
    ff <- ~offset(edges) + offset(nodematch(b, diff = TRUE))
    ff
  }

  formation_names_r <- c("edges", "nodematch.race.a", "nodematch.race.b",
                         "nodematch.race.c")

  formation_names_ra <- c("edges", "nodematch.race.a", "nodematch.race.b",
                          "nodematch.race.c", "offset(edges)", "offset(nodematch.age.1)",
                          "offset(nodematch.age.2)")

  formation_names_rr <- c("edges", "nodematch.race.a", "nodematch.race.b",
                          "nodematch.race.c", "offset(edges)", "offset(nodematch.race.a)",
                          "offset(nodematch.race.b)", "offset(nodematch.race.c)")

  diss_1 <- dissolution_coefs(make_diss_1(), c(10, 5, 7))
  diss_2 <- dissolution_coefs(make_diss_2(), c(5, 4, 7, 8))
  diss_3 <- dissolution_coefs(make_diss_3(), c(20, 50, 19))
  diss_4 <- dissolution_coefs(make_diss_4(), c(3, 2, 9, 4))

  formation <- make_form()

  est_1 <- netest(nw = nw,
                  formation = formation,
                  target.stats = c(100, 10, 20, 30),
                  coef.diss = diss_1,
                  nested.edapprox = FALSE)

  est_2 <- netest(nw = nw,
                  formation = formation,
                  target.stats = c(100, 10, 20, 30),
                  coef.diss = diss_2)

  run_sims <- function(est, expected_names) {
    dxs <- netdx(est, nsims = 10, dynamic = FALSE, verbose = FALSE)
    dxd <- netdx(est, nsims = 2, nsteps = 5, dynamic = TRUE, verbose = FALSE)
    param <- param.net(inf.prob = 0.3, act.rate = 0.5)
    init <- init.net(i.num = 3)
    control <- control.net(type = "SI", nsims = 1, nsteps = 5, verbose = FALSE)
    sim <- netsim(est, param, init, control)

    expect_equal(rownames(dxs$stats.table.formation), expected_names)
    expect_equal(rownames(dxd$stats.table.formation), expected_names)
    expect_equal(colnames(get_nwstats(sim, mode = "list")[[1]]), expected_names)
  }

  run_sims(est_1, formation_names_ra)
  run_sims(est_2, formation_names_r)

  est_11 <- update_dissolution(est_1, diss_1, nested.edapprox = FALSE)
  est_12 <- update_dissolution(est_1, diss_2, nested.edapprox = TRUE)
  est_13 <- update_dissolution(est_1, diss_3, nested.edapprox = FALSE)
  est_14 <- update_dissolution(est_1, diss_4, nested.edapprox = FALSE)

  run_sims(est_11, formation_names_ra)
  run_sims(est_12, formation_names_r)
  run_sims(est_13, formation_names_ra)
  run_sims(est_14, formation_names_rr)

  est_21 <- update_dissolution(est_2, diss_1, nested.edapprox = FALSE)
  est_22 <- update_dissolution(est_2, diss_2, nested.edapprox = TRUE)
  est_23 <- update_dissolution(est_2, diss_3, nested.edapprox = FALSE)
  est_24 <- update_dissolution(est_2, diss_4, nested.edapprox = FALSE)

  run_sims(est_21, formation_names_ra)
  run_sims(est_22, formation_names_r)
  run_sims(est_23, formation_names_ra)
  run_sims(est_24, formation_names_rr)
})

context("trim_netest Functionality")

test_that("trim_netest keep argument behaves as expected", {
  skip_on_cran()
  nw <- network_initialize(n = 50)
  nw <- set_vertex_attribute(nw, "sex", rbinom(50, 1, 0.5))

  attrname <- "sex"

  est <- netest(nw, formation = ~edges + nodematch(attrname),
                target.stats = c(25, 10),
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                verbose = FALSE)

  dxs <- netdx(est, nsims = 10, dynamic = FALSE, verbose = FALSE)
  dxd <- netdx(est, nsims = 2, nsteps = 10, dynamic = TRUE, verbose = FALSE)

  param <- param.net(inf.prob = 0.3, act.rate = 0.5)
  init <- init.net(i.num = 10)
  control <- control.net(type = "SI", nsims = 1, nsteps = 5, verbose = FALSE)
  sim <- netsim(est, param, init, control)

  expect_is(dxs, "netdx")
  expect_is(dxd, "netdx")
  expect_is(sim, "netsim")

  trim_est <- trim_netest(est)

  expect_error(netdx(trim_est, nsims = 10, dynamic = FALSE, verbose = FALSE),
               "object 'attrname' not found")
  expect_error(netdx(trim_est, nsims = 2, nsteps = 10, dynamic = TRUE, verbose = FALSE),
               "object 'attrname' not found")
  expect_error(netsim(trim_est, param, init, control),
               "object 'attrname' not found")

  trim_est_keep <- trim_netest(est, keep = "attrname")

  trim_dxs_keep <- netdx(trim_est_keep, nsims = 10, dynamic = FALSE, verbose = FALSE)
  trim_dxd_keep <- netdx(trim_est_keep, nsims = 2, nsteps = 10, dynamic = TRUE, verbose = FALSE)
  trim_sim_keep <- netsim(trim_est_keep, param, init, control)

  expect_is(trim_dxs_keep, "netdx")
  expect_is(trim_dxd_keep, "netdx")
  expect_is(trim_sim_keep, "netsim")
})

context("Full STERGM Workflow (All SOC)")

test_that("Full STERGM", {
  skip_on_cran()
  skip_on_os("windows")
  nw <- network_initialize(n = 50)
  est <- netest(nw, formation = ~edges, target.stats = 25,
                coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                edapprox = FALSE, verbose = FALSE)

  for (trim in c(FALSE, TRUE)) {
    if (trim == TRUE) {
      est2 <- trim_netest(est)
    } else {
      est2 <- est
    }

    # one core test
    dx <- netdx(est2, nsims = 1, nsteps = 10, verbose = FALSE)
    expect_is(dx, "netdx")
    expect_true(!dx$edapprox)
    expect_true(colnames(dx$stats[[1]]) == "edges")

    # parallel test
    dx <- netdx(est2, nsims = 2, nsteps = 10, ncores = 2, verbose = FALSE)
    expect_is(dx, "netdx")
    expect_true(dx$nsims == 2)
    expect_is(dx$nw, "network")

    param <- param.net(inf.prob = 0.3)
    init <- init.net(i.num = 10)
    control <- control.net(type = "SI", nsteps = 5, nsims = 1, verbose = FALSE)
    mod <- netsim(est2, param, init, control)
    expect_is(mod, "netsim")
  }
})

context("Extremal Targets")

test_that("extremal targets are handled correctly", {
  skip_on_cran()
  ff <- ~edges + offset(degrange(6)) + nodematch("sex") + nodematch("race") +
          degrange(6) + offset(nodematch("sex")) + degree(5)

  ff_sum <- ~edges + nodematch("sex") + nodematch("race") + degrange(6) +
              degree(5)

  for (egor in list(FALSE, TRUE)) {
    nw <- network_initialize(n = 100)
    nw <- set_vertex_attribute(nw, "race", rbinom(100, 1, 0.5))
    nw <- set_vertex_attribute(nw, "sex", rbinom(100, 1, 0.5))

    target_stats <- c(50, 0, 20, 0, 0)
    if (egor == TRUE) {
      nw <- san(ff, basis = nw, target.stats = target_stats, offset.coef = c(-Inf,-Inf))
      target_stats <- summary(ff_sum, basis = nw)
      nw <- ergm.ego:::as.egor.network(nw)
    }

    target_stats_names <- c("edges", "nodematch.sex", "nodematch.race", "deg6+", "degree5")
    target_stats_names_regexp <- c("edges", "nodematch\\.sex", "nodematch\\.race", "deg6\\+", "degree5")

    est <- netest(nw, formation = ff,
                  target.stats = target_stats,
                  coef.form = c(1, -Inf),
                  coef.diss = dissolution_coefs(~offset(edges), 10, 0),
                  verbose = FALSE)

    expect_equal(est$target.stats, target_stats)
    expect_equal(est$target.stats.names, target_stats_names)

    dxs <- netdx(est, nsims = 10, dynamic = FALSE, verbose = FALSE)
    invisible(capture.output(print(dxs)))
    plot(dxs)

    dxd <- netdx(est, nsims = 10, nsteps = 5, dynamic = TRUE, verbose = FALSE)
    invisible(capture.output(print(dxd)))
    plot(dxd)

    param <- param.net(inf.prob = 0.3, act.rate = 0.5)
    init <- init.net(i.num = 10)
    control <- control.net(type = "SI", nsims = 2, nsteps = 5, verbose = FALSE)
    mod <- netsim(est, param, init, control)
    invisible(capture.output(print(mod)))

    plot(mod)
    plot(mod, type = "formation")
    plot(mod, type = "duration")
    plot(mod, type = "dissolution")
    plot(mod, type = "network")

    for (obj in list(dxs, dxd, mod)) {
      for(i in seq_along(target_stats_names_regexp)) {
        expect_output(print(obj), paste0(target_stats_names_regexp[i], " *", target_stats[i]))
      }
      expect_output(print(obj), paste0("offset\\(deg6\\+\\)", " *", "NA"))
      expect_output(print(obj), paste0("offset\\(nodematch\\.sex\\)", " *", "NA"))
    }
  }
})

context("networkLite (All SOC)")

test_that("network and networkLite work equally in netest, netdx, and netsim", {
  skip_on_cran()
  net_size <- 100
  bip_size <- 40

  ffdir <- ~odegree(1) + idegree(1)
  ffundir <- ~degree(1)

  for(directed in list(FALSE, TRUE)) {
    for(bipartite in list(FALSE, bip_size)) {
      if(directed && bipartite) {
        next
      }
      for (duration in c(1, 10)) {

        nw <- network.initialize(n = 100, directed = directed, bipartite = bipartite)
        nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))

        set.seed(0)
        est <- netest(nw, formation = ~edges + nodematch("race"),
                      target.stats = c(50, 20),
                      coef.diss = dissolution_coefs(~offset(edges), c(duration)),
                      verbose = FALSE
        )
        dxs <- netdx(est, nsims = 20, verbose = FALSE,
                       dynamic = FALSE, nwstats.formula = if(directed) ffdir else ffundir)

        if (duration > 1) {
          dxd <- netdx(est, nsims = 2, nsteps = 10, verbose = FALSE,
                         dynamic = TRUE)
        }

        param <- param.net(inf.prob = 0.3, act.rate = 0.5)
        init <- init.net(i.num = 10)
        control <- control.net(type = "SI", nsims = 2, nsteps = 5, verbose = FALSE)
        sim <- netsim(est, param, init, control)

        nwL <- as.networkLite(nw)
        set.seed(0)
        estL <- netest(nwL, formation = ~edges + nodematch("race"),
                       target.stats = c(50, 20),
                       coef.diss = dissolution_coefs(~offset(edges), c(duration)),
                       verbose = FALSE
        )
        dxsL <- netdx(estL, nsims = 20, verbose = FALSE,
                       dynamic = FALSE, nwstats.formula = if(directed) ffdir else ffundir)

        if (duration > 1) {
          dxdL <- netdx(estL, nsims = 2, nsteps = 10, verbose = FALSE,
                         dynamic = TRUE)
        }

        simL <- netsim(estL, param, init, control)

        # convert networks to networkLites
        dxs$nw <- as.networkLite(dxs$nw)
        if (duration > 1) {
          dxd$nw <- as.networkLite(dxd$nw)
        }

        # the rest should be equal, including coefs, stats, etc.
        expect_equal(trim_netest(est), trim_netest(estL))
        expect_equal(dxs, dxsL)
        if (duration > 1) {
          expect_equal(dxd, dxdL)
        }
        expect_equal(sim, simL)
      }
    }
  }
})

# Setup
n <- 100
nw <- network_initialize(n)

prev <- 0.2
infIds <- sample(1:n, n*prev)
nw <- set_vertex_attribute(nw, "status", "s")
nw <- set_vertex_attribute(nw, "status", "i", infIds)

mean.deg.inf <- 0.3
inedges.inf <- mean.deg.inf * n * prev

mean.deg.sus <- 0.8
inedges.sus <- mean.deg.sus * n * (1 - prev)

edges <- (inedges.inf + inedges.sus)/2
nmatch <- edges * 0.91

formation <- ~edges + nodefactor("status") + nodematch("status")
target.stats <- c(edges, inedges.sus, nmatch)

coef.diss <- dissolution_coefs(dissolution = ~offset(edges), 50)

suppressMessages(
  est <- netest(nw, formation, target.stats, coef.diss)
)

param <- param.net(inf.prob = 0.03)
init <- init.net()

test_that("nw stats in full tergm", {
  skip_on_cran()
  control <- control.net(type = "SI", nsteps = 5, nsims = 1, ncores = 1,
                         resimulate.network = TRUE, tergmLite = FALSE,
                         nwstats.formula = ~edges +
                           meandeg +
                           nodefactor("status", levels = NULL) +
                           nodematch("status"), verbose = FALSE)
  sim <- netsim(est, param, init, control)

  expect_s3_class(sim, "netsim")
  nws <- get_nwstats(sim)
  expect_equal(names(nws), c("time", "sim", "edges", "meandeg",
                             "nodefactor.status.i", "nodefactor.status.s",
                             "nodematch.status"))
  expect_equal(dim(nws), c(5, 7))

  # default = 'formation'
  control <- control.net(type = "SI", nsteps = 5, nsims = 1, ncores = 1,
                         resimulate.network = TRUE, tergmLite = FALSE,
                         verbose = FALSE)
  sim <- netsim(est, param, init, control)

  expect_s3_class(sim, "netsim")
  nws <- get_nwstats(sim)

  expect_equal(names(nws), c("time", "sim", "edges",
                             "nodefactor.status.s", "nodematch.status"))
  expect_equal(dim(nws), c(5, 5))

})

test_that("nw stats in tergmLite", {
  skip_on_cran()
  control <- control.net(type = "SI", nsteps = 5, nsims = 1, ncores = 1,
                         resimulate.network = TRUE, tergmLite = TRUE,
                         nwstats.formula = ~edges +
                           meandeg +
                           nodefactor("status", levels = NULL) +
                           nodematch("status"), verbose = FALSE)
  sim <- netsim(est, param, init, control)

  expect_s3_class(sim, "netsim")
  nws <- get_nwstats(sim)
  expect_equal(names(nws), c("time", "sim", "edges", "meandeg",
                             "nodefactor.status.i", "nodefactor.status.s",
                             "nodematch.status"))
  expect_equal(dim(nws), c(5, 7))

  # default = 'formation'
  control <- control.net(type = "SI", nsteps = 5, nsims = 1, ncores = 1,
                         resimulate.network = TRUE, tergmLite = FALSE,
                         verbose = FALSE)
  sim <- netsim(est, param, init, control)

  expect_s3_class(sim, "netsim")
  nws <- get_nwstats(sim)

  expect_equal(names(nws), c("time", "sim", "edges",
                             "nodefactor.status.s", "nodematch.status"))
  expect_equal(dim(nws), c(5, 5))
})

test_that("tergm and tergmLite produce equal non-durational statistics", {
  skip_on_cran()
  set.seed(0)
  control <- control.net(type = NULL, 
                         nsims = 1, 
                         nsteps = 10, 
                         ncores = 1, 
                         verbose = FALSE, 
                         resimulate.network = TRUE, 
                         nwstats.formula = ~edges + concurrent + isolates + nodemix("status"))  
  mod1 <- netsim(est, param, init, control)
  a <- mod1$stats$nwstats[[1]][[1]]
  
  set.seed(0)
  control <- control.net(type = NULL, 
                         nsims = 1, 
                         nsteps = 10, 
                         ncores = 1, 
                         verbose = FALSE, 
                         resimulate.network = TRUE, 
                         nwstats.formula = ~edges + concurrent + isolates + nodemix("status"), 
                         tergmLite = TRUE)  
  mod2 <- netsim(est, param, init, control)
  b <- mod2$stats$nwstats[[1]][[1]]
    
  expect_equal(a,b)
})

#' @title Get the Forward or Backward Reachable Nodes for a Set of Nodes
#'
#' @description
#' These functions return the Forward or Backward Reachable Nodes of a set of
#' nodes in a network over a time. Warning, these functions ignore nodes without
#' edges in the period of interest. See the `Number of Nodes` section for
#' details It is much faster than iterating
#' `tsna::tPath`. The distance between to each node can be back calculated
#' using the length of the reachable set at each time step and the fact that the
#' reachable sets are ordered by the time to arrival.
#'
#' @param el_cuml a cumulative edgelist object. That is a data.frame with at
#'   least columns: head, tail, start and stop. Start and stop are inclusive.
#' @param from_step the beginning of the time period.
#' @param to_step the end of the time period.
#' @param nodes the subset of nodes to calculate the FRP for. (default = NULL,
#'        all nodes)
#' @param dense_optim pre-process the adjacency list to speed up the
#'        computations on dense networks. "auto" (default), enable the
#'        optimisation when `n_edges > n_nodes`. "yes" always enables and "no"
#'        always disables. The overhead of the optimization is not worth it on
#'        sparse networks.
#'
#' @return
#' A named list containing:
#'  `reached`: the set of reachable nodes for each of the `nodes`.
#'  `lengths`: A matrix of `length(nodes)` rows and one column per timestep + 1
#'      with the length of the reachable set at each step from `from_step - 1`
#'      to `to_step`. The first column is always one as the set of reachables
#'      at the beginning is just the node itself.
#'
#' @section Number of Nodes:
#' To speed up the calculations and lower the memory usage, these functions
#' only take into account nodes with edges in the cumulative edgelist over the
#' period of interest. The nodes are identified in the `reached` and `lengths`
#' sublists by names (e.g. `node_1093`). Nodes without any edges are therefore
#' not calculated as the only node they reach is themselve (length of 1).
#' Take this fact into account when exploring the distribution of Forward
#' Reachable Paths for example. As the nodes with FRP == 1 are not in the
#' output.
#'
#' @section Time and Memory Use:
#' These functions may be used to efficiently calculate multiple sets of
#' reachable nodes. As cumulative edgelists are way smaller than full
#' `networkDynamic` objects, theses functions are suited for large and dense
#' networks.
#' Also, as long as the size of the `nodes` set is greater than 5, theses
#' functions are faster than iterating over `tsna::tPath`.
#'
#' @section Displaying Progress:
#' These functions are using the
#' [progressr package](https://progressr.futureverse.org/articles/progressr-01-intro.html)
#' to display its progression. Use
#' \code{progressr::with_progress({ fwd_reach <- get_forward_reachable(el, from = 1, to = 260) })}
#' to display the progress bar. Or see the
#' [progressr package](https://progressr.futureverse.org/articles/progressr-01-intro.html)
#' for more information and customization.
#'
#' @examples
#' \dontrun{
#'
#' # load a network dynamic object
#' nd <- readRDS("nd_obj.Rds")
#' # convert it to a cumulative edgelist
#' el_cuml <- as_cumulative_edgelist(nd)
#'
#' # sample 100 node indexes
#' nnodes <- max(el_cuml$head, el_cuml$tail)
#' nodes <- sample(nnodes, 100)
#'
#' # `get_forward_reachable` uses steps [from_step, to_step] inclusive
#' el_fwd <- get_forward_reachable(el_cuml, 1, 52, nodes)[["reached"]]
#'
#' # check if the results are consistent with `tsna::tPath`
#' nodes <- strsplit(names(el_fwd), "_")
#' for (i in seq_along(el_fwd)) {
#'   node <- as.integer(nodes[[i]][2])
#'   t_fwd <- tsna::tPath(
#'     nd, v = node,
#'     start = 1, end = 52 + 1, # tPath works from [start, end) right exclusive
#'     direction = "fwd"
#'   )
#'
#'   t_fwd_set <- which(t_fwd$tdist < Inf)
#'   if(!setequal(el_fwd[[i]], t_fwd_set))
#'     stop("Missmatch on node: ", node)
#' }
#'
#' # Backward:
#' el_bkwd <- get_backward_reachable(el_cuml, 1, 52, nodes = 1)[["reached"]]
#' nodes <- strsplit(names(el_bkwd), "_")
#' t_bkwd <- tsna::tPath(
#'   nd, v = nodes[i][2],
#'   start = 1, end = 52 + 1,
#'   direction = "bkwd", type = "latest.depart"
#' )
#' t_bkwd_set <- which(t_bkwd$tdist < Inf)
#' setequal(el_bkwd[[1]], t_bkwd_set)
#'
#' }
#'
#' @name reachable-nodes
NULL

#' @rdname reachable-nodes
#' @export
get_forward_reachable <- function(el_cuml, from_step, to_step, nodes = NULL,
                                  dense_optim = "auto") {
  # Prepare the cumulative edgelist:
  # - set the `stop` time to `Inf` instead of NA
  # - remove the edges that don't exist in the analysis period
  # - set the oldest edges `start` to `to_step`
  #     (QoL to calcuate the `change_times`)
  # nolint start
  el_cuml <- el_cuml |>
    dplyr::mutate(stop = ifelse(is.na(.data$stop), Inf, .data$stop)) |> # current edges never ends
    dplyr::filter(.data$start <= to_step, .data$stop >= from_step) |> # remove edges before and after analysis period
    dplyr::mutate(start = ifelse(.data$start < from_step, from_step, .data$start)) |> # set older edges to start at beginning of analysis
    dplyr::select("start", "stop", "head", "tail")
  # nolint end

  # Make the node indexes continuous
  # (original indexes are reset at the end)
  orig_indexes <- sort(unique(c(el_cuml$head, el_cuml$tail)))
  el_cuml$head <- match(el_cuml$head, orig_indexes)
  el_cuml$tail <- match(el_cuml$tail, orig_indexes)

  if (is.null(nodes)) {
    nodes <- seq_along(orig_indexes)
  } else {
    nodes <- match(nodes, orig_indexes)
    nodes <- nodes[!is.na(nodes)]
  }

  if (length(nodes) == 0) {
    warning("No nodes have edges in the network over this period")
    return(list(reached = list(), lengths = matrix(0)))
  }

  # Only consider steps where new edges occur
  change_times <- sort(unique(el_cuml$start))
  n_steps <- to_step - from_step + 1

  # the initial FRP contains only the vertex itself
  reach_cur <- as.list(nodes)
  reach_lengths <- matrix(0, ncol = n_steps + 1, nrow = length(nodes))
  reach_lengths[, 1] <- 1

  p <- progressr::progressor(length(change_times))

  for (cur_step in change_times) {
    p()

    # IN EL_CUML: duration is [start, stop] (inclusive)
    # all active edges a time `cur_step` are needed (not only start).
    # This is because getting connected to a node X means also indirectly
    # connecting to its connections, even the ones that started earlier.
    el_cur <- dplyr::filter(
      el_cuml,
      .data$start <= cur_step,
      .data$stop >= cur_step
    )

    adj_list <- get_adj_list(el_cur, length(orig_indexes))

    # precompute the network components. Useful for dense networks
    if (dense_optim == "yes") {
      adj_list <- get_subnet_adj_list(adj_list)
    } else if (dense_optim == "auto" && nrow(el_cur) > length(orig_indexes)) {
      adj_list <- get_subnet_adj_list(adj_list)
    }

    # at time T, the REACH(T) of a node is the subnet connected to the REACH(T-1)
    new_reached <- lapply(reach_cur, get_connected_nodes, adj_list = adj_list)
    reach_lengths[, cur_step - from_step + 2] <- vapply(new_reached, length, 0)
    reach_cur <- Map(c, reach_cur, new_reached)
  }

  reach_lengths <- t(apply(reach_lengths, 1, cumsum))
  reach_cur <- lapply(reach_cur, \(x) orig_indexes[x])

  names(reach_cur) <- paste0("node_", orig_indexes[nodes])
  rownames(reach_lengths) <- names(reach_cur)
  colnames(reach_lengths) <- paste0("step_", (from_step - 1):to_step)

  return(
    list(
      reached = reach_cur,
      lengths = reach_lengths
    )
  )
}

#' @rdname reachable-nodes
#' @export
get_backward_reachable <- function(el_cuml, from_step, to_step, nodes = NULL,
                                   dense_optim = "auto") {
  # simply invert the time before calling get_forward_reachable`
  el_cuml$stop <- ifelse(is.na(el_cuml$stop), Inf, el_cuml$stop)
  tmp <- el_cuml$start
  el_cuml$start <- - el_cuml$stop
  el_cuml$stop <- - tmp

  get_forward_reachable(el_cuml, -to_step, -from_step, nodes, dense_optim)
}

#' Returns all the node connected directly or indirectly to a set of nodes
#'
#' @param adj_list The network represented as an adjacency list
#' @param nodes A set of nodes
#'
#' @return A vector of nodes indexes that are connected together with the ones
#'         provided in the `nodes` argument. The `nodes` themselves are not
#'         listed in this output
get_connected_nodes <- function(adj_list, nodes) {
  n_nodes <- length(adj_list)
  new_connections <- numeric(0)
  subnet <- nodes
  while (length(nodes) > 0 && length(subnet) < n_nodes) {
    nodes <- unlist(adj_list[nodes])
    nodes <- setdiff(nodes, subnet)
    subnet <- c(subnet, nodes)
    new_connections <- c(new_connections, nodes)
  }
  new_connections
}

#' Returns an adjacency list from an edge list
#'
#' @param el An edge list as a data.frame with columns `head` and `tail`
#' @param n_nodes The size number of node in the network
#'
#' @return An adjacency list for the network
#'
#' @details
#' The adjacency list is a `list` of length `n_nodes`. The entry for each node
#' is a integer vector containing the index of all the nodes connected to it.
#' This layout makes it directly subsetable in O(1) at the expanse of memory
#' usage.
#' To get all connections to the nodes 10 and 15 : `unlist(adj_list[c(10, 15)]`
get_adj_list <- function(el, n_nodes) {
  head <- el$head
  tail <- el$tail

  adj_list <- vector(mode = "list", length = n_nodes)
  for (i in seq_len(nrow(el))) {
    e_head <- head[i]
    e_tail <- tail[i]
    adj_list[[e_head]] <- c(adj_list[[e_head]], e_tail)
    adj_list[[e_tail]] <- c(adj_list[[e_tail]], e_head)
  }
  adj_list
}

#' Return an adjacency list of subnets
#'
#' @param adj_list A normal adjacency list
#'
#' @return An adjacency list where only the first node of a subnet contains the
#' subnet and all other contain only the first node
#'
#' @details
#' A graph with 4 components: 1, 2, 3, 4, and 5 and 6, 7, 8  would yield a list
#' like so:
#' 1: 2, 3, 4
#' 2: 1
#' 3: 1
#' 4: 1
#' 5: numeric(0)
#' 6: 7, 8
#' 7: 6,
#' 8: 6
#'
#' This format speeds up the construction of reachable sets on dense networks
get_subnet_adj_list <- function(adj_list) {
  for (i in seq_along(adj_list)) {
    # if first elt connected to before himself, already in a subnet
    if (length(adj_list[[i]]) == 0 || adj_list[[i]][1] < i)
      next
    # get current subnet of i
    subnet <- get_connected_nodes(adj_list, i)
    adj_list[[i]] <- subnet
    # all members of subnet point to i
    adj_list[subnet] <- i
  }
  adj_list
}

#' Convert an object to a `cumulative_edgelist`
#'
#' @param x An object to be converted to a cumulative edgelist
#'
#' @return A `cumulative_edgelist` object, a `data.frame` with at least the
#' following columns: `head`, `tail`, `start`, `stop`.
#'
#' @details
#' The edges are active from time `start` to time `stop` included. If stop is
#' `NA`, the edge was not disolved in the simulation that generated the list.
#'
#' @export
as_cumulative_edgelist <- function(x) {
  UseMethod("as_cumulative_edgelist")
}

#' @export
as_cumulative_edgelist.networkDynamic <- function(x) {
  d <- as.data.frame(x)
  d <- d[c("head", "tail", "onset", "terminus")]
  names(d) <- c("head", "tail", "start", "stop")
  d$stop <- d$stop - 1
  class(d) <- c("cumulative_edgelist", class(d))
  return(d)
}

#' Deduplicate a cumulative edgelist by combining overlapping edges
#'
#' @param el A cumulative edgelist with potentially overlapping edges
#'
#' @return A cumulative edgelist with no overlapping edges
#'
#' @export
dedup_cumulative_edgelist <- function(el) {
  el_n <- el |>
    dplyr::group_by(head, tail) |>
    dplyr::mutate(n = dplyr::n()) |>
    dplyr::ungroup()

  e_unique <- el_n |>
    dplyr::filter(.data$n == 1) |>
    dplyr::select(-"n")

  e_dup <- el_n |>
    dplyr::filter(.data$n > 1) |>
    dplyr::select(-"n") |>
    dplyr::arrange(.data$head, .data$tail, .data$start, .data$stop)

  e_dedup <- e_dup |>
    dplyr::group_by(.data$head, .data$tail) |>
    dplyr::mutate(
      lstart = dplyr::lag(.data$start),
      lstop = dplyr::lag(.data$stop),
      overlap =
        !is.na(.data$lstop) &
        !is.na(.data$lstart) &
        .data$start <= .data$lstop,
      stop = ifelse(.data$overlap, max(.data$stop, .data$lstop, na.rm = TRUE), .data$stop),
      start = ifelse(.data$overlap, min(.data$start, .data$lstart, na.rm = TRUE), .data$start)
    ) |>
    dplyr::select(-c("lstart", "lstop", "overlap")) |>
    dplyr::ungroup() |>
    unique()

  dplyr::bind_rows(e_unique, e_dedup)
}

#' @title Recovery: netsim Module
#'
#' @description This function simulates recovery from the infected state
#'              either to a distinct recovered state (SIR model type) or back
#'              to a susceptible state (SIS model type), for use in
#'              [netsim()].
#'
#' @param dat Main `netsim_dat` object containing a `networkDynamic`
#'        object and other initialization information passed from
#'        [netsim()].
#' @param at Current time step.
#'
#' @return The updated `netsim_dat` main list object.
#'
#' @export
#' @keywords internal
#'
recovery.net <- function(dat, at) {

  ## Only run with SIR/SIS
  type <- get_control(dat, "type", override.null.error = TRUE)
  type <- if (is.null(type)) "None" else type

  if (!(type %in% c("SIR", "SIS"))) {
    return(dat)
  }

  # Variables ---------------------------------------------------------------
  active <- get_attr(dat, "active")
  status <- get_attr(dat, "status")
  infTime <- get_attr(dat, "infTime")
  recovState <- ifelse(type == "SIR", "r", "s")

  rec.rate <- get_param(dat, "rec.rate")

  nRecov <- 0
  idsElig <- which(active == 1 & status == "i")
  nElig <- length(idsElig)


  # Time-Varying Recovery Rate ----------------------------------------------
  infDur <- at - infTime[active == 1 & status == "i"]
  infDur[infDur == 0] <- 1
  lrec.rate <- length(rec.rate)
  if (lrec.rate == 1) {
    ratesElig <- rec.rate
  } else {
    ratesElig <- ifelse(infDur <= lrec.rate, rec.rate[infDur],
                        rec.rate[lrec.rate])
  }


  # Process -----------------------------------------------------------------
  if (nElig > 0) {
    vecRecov <- which(rbinom(nElig, 1, ratesElig) == 1)
    if (length(vecRecov) > 0) {
      idsRecov <- idsElig[vecRecov]
      nRecov <- length(idsRecov)
      status[idsRecov] <- recovState
    }
  }
  dat <- set_attr(dat, "status", status)

  # Output ------------------------------------------------------------------
  outName <- ifelse(type == "SIR", "ir.flow", "is.flow")
  dat <- set_epi(dat, outName, at, nRecov)

  return(dat)
}

#' @title Recovery: netsim Module
#'
#' @description This function simulates recovery from the infected state
#'              either to a distinct recovered state (SIR model type) or back
#'              to a susceptible state (SIS model type), for use in
#'              [netsim()].
#'
#' @inheritParams recovery.net
#'
#' @inherit recovery.net return
#'
#' @export
#' @keywords internal
#'
recovery.2g.net <- function(dat, at) {

  ## Only run with SIR/SIS
  type <- get_control(dat, "type", override.null.error = TRUE)
  type <- if (is.null(type)) "None" else type

  if (!(type %in% c("SIR", "SIS"))) {
    return(dat)
  }

  # Variables ---------------------------------------------------------------
  active <- get_attr(dat, "active")
  status <- get_attr(dat, "status")
  infTime <- get_attr(dat, "infTime")
  group <- get_attr(dat, "group")
  recovState <- ifelse(type == "SIR", "r", "s")

  rec.rate <- get_param(dat, "rec.rate")
  rec.rate.g2 <- get_param(dat, "rec.rate.g2")

  nRecov <- nRecovG2 <- 0
  idsElig <- which(active == 1 & status == "i")
  nElig <- length(idsElig)

  # Time-Varying Recovery Rate ----------------------------------------------
  infDur <- at - infTime[active == 1 & status == "i"]
  infDur[infDur == 0] <- 1
  lrec.rate <- length(rec.rate)
  if (lrec.rate == 1) {
    gElig <- group[idsElig]
    rates <- c(rec.rate, rec.rate.g2)
    ratesElig <- rates[gElig]
  } else {
    gElig <- group[idsElig]
    if (is.null(rec.rate.g2)) {
      rates <- ifelse(infDur <= lrec.rate, rec.rate[infDur],
                      rec.rate[lrec.rate])
    } else {
      rates <- rep(NA, length(infDur))
      rates[gElig == 1] <- ifelse(infDur[gElig == 1] <= lrec.rate,
                                  rec.rate[infDur[gElig == 1]],
                                  rec.rate[lrec.rate])
      rates[gElig == 2] <- ifelse(infDur[gElig == 2] <= length(rec.rate.g2),
                                  rec.rate.g2[infDur[gElig == 2]],
                                  rec.rate.g2[length(rec.rate.g2)])
    }
    ratesElig <- rates
  }


  # Process -----------------------------------------------------------------
  if (nElig > 0) {
    vecRecov <- which(rbinom(nElig, 1, ratesElig) == 1)
    if (length(vecRecov) > 0) {
      idsRecov <- idsElig[vecRecov]
      nRecov <- sum(group[idsRecov] == 1)
      nRecovG2 <- sum(group[idsRecov] == 2)
      status[idsRecov] <- recovState
    }
  }
  dat <- set_attr(dat, "status", status)

  # Output ------------------------------------------------------------------
  outName <- ifelse(type == "SIR", "ir.flow", "is.flow")
  outName[2] <- paste0(outName, ".g2")

  dat <- set_epi(dat, outName[1], at, nRecov)
  dat <- set_epi(dat, outName[2], at, nRecovG2)


  return(dat)
}

#' @title Modules for Stochastic Individual Contact Models
#'
#' @description
#' Stochastic individual contact models of infectious disease simulate epidemics
#' in which contacts between individuals are instantaneous events in discrete
#' time. They are intended to be the stochastic microsimulation analogs to
#' deterministic compartmental models.
#'
#' The [icm()] function handles both the simulation tasks. Within this
#' function are a series of modules that initialize the simulation and then
#' simulate new infections, recoveries, and vital dynamics at each time step. A
#' module also handles the basic bookkeeping calculations for disease
#' prevalence.
#'
#' This help page presents a brief overview of the built-in module functions
#' in the order in which they are used within [icm()]. ICMs support only the
#' built-in SI, SIR, and SIS disease types. For custom or extension epidemic
#' models, use the network model class via [netsim()] and [control.net()]
#' instead.
#'
#' @section Initialization Module:
#' This function sets up agent attributes, like disease status, on the network
#' at the starting time step of disease simulation, \eqn{t_1}. For
#' multiple-simulation function calls, these are reset at the beginning of each
#' simulation.
#'
#'  * [initialize.icm()]: sets which agents are initially
#'        infected, through the initial conditions passed in
#'        [init.icm()].
#'
#'
#' @section Disease Status Modification Modules:
#' The main disease simulation occurs at each time step given the current state
#' of the population at that step. Infection of agents is simulated as a
#' function of disease parameters and population composition. Recovery of agents
#' is likewise simulated with respect to infected nodes. These functions also
#' analyze the flows for summary measures such as disease incidence.
#'
#'  * [infection.icm()]: randomly draws an edgelist given the
#'        parameters, subsets the list for discordant pairs, and simulates
#'        transmission on those discordant pairs through a series of draws from
#'        a binomial distribution.
#'  * [recovery.icm()]: simulates recovery from infection either
#'        to a lifelong immune state (for SIR models) or back to the susceptible
#'        state (for SIS models), as a function of the recovery rate specified
#'        in the `rec.rate` parameter. The recovery rate may vary for
#'        two-group models.
#'
#'
#'
#' @section Demographic Modules:
#' Vital dynamics such as arrival and departure processes are simulated at each
#' time step to update entries into and exits from the population. These are
#' used in open-population ICMs.
#'
#'  * [departures.icm()]: randomly simulates departures or exits
#'        for agents given the departure rate specified in the disease-state and
#'        group-specific departure parameters in [param.icm()]. This
#'        involves deactivating agents from the population, but their historical
#'        data is preserved in the simulation.
#'  * [arrivals.icm()]: randomly simulates new arrivals into the
#'        population given the current population size and the arrival rate
#'        parameters. This involves adding new agents into the population.
#'
#'
#'
#' @section Bookkeeping Module:
#' Simulations require bookkeeping at each time step to calculate the
#' summary epidemiological statistics used in the model output analysis.
#'
#'  * [prevalence.icm()]: calculates the number in each disease
#'        state (susceptible, infected, recovered) at each time step for those
#'        active agents in the population.
#'
#'
#'
#' @name modules.icm
#' @aliases modules.icm
#'
NULL

#' @title Progress Print Module for Deterministic Compartmental Models
#'
#' @description This function prints progress from deterministic compartmental
#'              models simulated with [dcm()] to the console.
#'
#' @param x If the `type` is "startup", then an object of class
#'        `control.dcm`, otherwise the main `df` object in `dcm`
#'        runs.
#' @param type Progress type, either of "startup" for starting messages before
#'        all runs, or "progress" for time step specific messages.
#' @param s Current run number, if type is "progress".
#'
#' @export
#' @keywords internal
#'
verbose.dcm <- function(x, type, s = 1) {

  if (type == "startup") {
    if (x$verbose == TRUE && x$nruns > 1) {
      cat("\nStarting DCM Simulation...")
    }
  }

  if (type == "progress") {
    if (x$verbose == TRUE && x$nruns > 1) {
      cat("\nRun = ", s, "/", x$nruns, sep = "")
    }
  }

}

#' @title Progress Print Module for Stochastic Individual Contact Models
#'
#' @description This function prints progress from stochastic individual contact
#'              models simulated with [icm()] to the console.
#'
#' @param x If the `type` is "startup", then an object of class
#'        `control.icm`; otherwise, an object of class `icm_dat`, the
#'        main data object in `icm` simulations.
#' @param type Progress type, either of "startup" for starting messages before
#'        all simulations, or "progress" for time step specific messages.
#' @param s Current simulation number, if type is "progress".
#' @param at Current time step, if type is "progress".
#'
#' @export
#' @keywords internal
#'
verbose.icm <- function(x, type, s = 1, at = 2) {

  if (type == "startup") {
    if (x$verbose == TRUE) {
      cat("\nStarting ICM Simulation...")
    }
  }

  if (type == "progress") {
    if (x$control$verbose == TRUE) {
      if (x$control$verbose.int == 0 && at == x$control$nsteps) {
        cat("\nSim = ", s, "/", x$control$nsims, sep = "")
      }
      if (x$control$verbose.int > 0 && (at %% x$control$verbose.int == 0)) {
        cat("\014")
        cat("\nEpidemic Simulation")
        cat("\n----------------------------")
        cat("\nSimulation: ", s, "/", x$control$nsims, sep = "")
        cat("\nTimestep: ", at, "/", x$control$nsteps, sep = "")
        status <- x$attr$status
        if (inherits(status, "character")) {
          status <- ifelse(status == "i", 1, 0)
        }
        cat("\nPrevalence:", sum(status, na.rm = TRUE))
        cat("\nPopulation Size:", sum(x$attr$active == 1))
        cat("\n----------------------------")
      }
    }
  }

}


#' @title Progress Print Module for Stochastic Network Models
#'
#' @description This function prints progress from stochastic network models
#'              simulated with [netsim()] to the console.
#'
#' @param x If the `type` is "startup", then an object of class
#'        `control.net`; otherwise, an object of class `netsim_dat`,
#'        the main data object in [netsim()] simulations.
#' @param type Progress type, either of "startup" for starting messages before
#'        all simulations, or "progress" for time step specific messages.
#' @param s Current simulation number, if type is "progress".
#' @param at Current time step, if type is "progress".
#'
#' @export
#' @keywords internal
#'
verbose.net <- function(x, type, s = 1, at = 2) {

  if (type == "startup" && x$ncores == 1) {
    if (x$verbose == TRUE) {
      cat("\nStarting Network Simulation...")
    }
  }

  if (type == "progress" && x$control$ncores == 1) {
    if (x$control$verbose == TRUE) {
      if (x$control$verbose.int == 0 && at == x$control$nsteps) {
        cat("\nSim = ", s, "/", x$control$nsims, sep = "")
      }
      if (x$control$verbose.int > 0 && (at %% x$control$verbose.int == 0)) {
        cat("\014")
        cat("\nEpidemic Simulation")
        cat("\n----------------------------")
        cat("\nSimulation: ", s, "/", x$control$nsims, sep = "")
        cat("\nTimestep: ", at, "/", x$control$nsteps, sep = "")
        active <- get_attr(x, "active")
        status <- get_attr(x, "status", posit_ids = which(active == 1))
        if (inherits(status, "character")) {
          status <- ifelse(status == "i", 1, 0)
        }
        cat("\nPrevalence:", sum(status, na.rm = TRUE))
        cat("\nPopulation Size:", sum(active == 1))
        cat("\n----------------------------")
      }
    }
  }

}

#' @title Save dcm Data to Output List Format
#'
#' @description This function transfers the data from the main `df`
#'              object to the output `out` object at the end of each
#'              run in [dcm()].
#'
#' @param df Main object in [dcm()] simulations.
#' @param s Current run number.
#' @param param Param list set in [param.dcm()].
#' @param control Control list set in [control.dcm()].
#' @param out Out list passed back in for updating at runs 2+.
#'
#' @return A list with the following elements:
#'
#'  * **param:** the epidemic parameters passed into the model through
#'        [param.dcm()], with additional parameters added as
#'        necessary.
#'  * **control:** the control settings passed into the model through
#'        [control.dcm()], with additional controls added as
#'        necessary.
#'  * **epi:** a list of data frames, one for each epidemiological
#'        output from the model.
#'
#'
#' @keywords internal
#' @export
#'
saveout.dcm <- function(df, s, param, control, out = NULL) {

  if (s == 1) {
    out <- list()
    out$param <- param
    out$control <- control
    if (length(control$nsteps) == 1) {
      out$control$timesteps <- seq(1, control$nsteps, control$dt)
    } else {
      out$control$timesteps <- control$nsteps
    }

    out$epi <- list()
    for (j in 2:ncol(df)) {
      out$epi[[names(df)[j]]] <- data.frame(df[, j])
    }
  } else {
    for (j in 2:ncol(df)) {
      out$epi[[names(df)[j]]][, s] <- data.frame(df[, j])
    }
  }

  # Remove NA's from flows by setting last value to penultimate value
  ns <- nrow(out$epi[[1]])
  lr.na <- sapply(out$epi, function(x) is.na(x[ns, s]) & !is.na(x[ns - 1, s]))
  wh.lr.na <- as.numeric(which(lr.na == TRUE))
  if (length(wh.lr.na) > 0) {
    for (jj in wh.lr.na) {
      out$epi[[jj]][ns, s] <- out$epi[[jj]][ns - 1, s]
    }
  }


  ## Processing for final run
  if (s == control$nruns) {

    # Set column names for varying list elements
    for (s in as.vector(which(lapply(out$epi, class) == "data.frame"))) {
      colnames(out$epi[[s]]) <- paste0("run", 1:control$nruns)
    }

  }

  return(out)
}


#' @title Save icm Data to Output List Format
#'
#' @description This function transfers the data from the main `icm_dat`
#'              class data object to the output `out` object at the end of
#'              each simulation in [icm()].
#'
#' @inheritParams prevalence.icm
#' @param s Current simulation number.
#' @param out Out list passed back in for updating at simulations 2+.
#'
#' @return
#' A list with the following elements:
#'
#'  * **param:** the epidemic parameters passed into the model through
#'        [param.icm()], with additional parameters added as
#'        necessary.
#'  * **control:** the control settings passed into the model through
#'        [control.icm()], with additional controls added as
#'        necessary.
#'  * **epi:** a list of data frames, one for each epidemiological
#'        output from the model.
#'
#'
#' @keywords internal
#' @export
#'
saveout.icm <- function(dat, s, out = NULL) {

  if (s == 1) {
    out <- list()
    out$param <- dat$param
    out$control <- dat$control
    out$epi <- list()
    for (j in seq_along(dat$epi)) {
      out$epi[[names(dat$epi)[j]]] <- data.frame(dat$epi[j])
    }
  } else {
    for (j in seq_along(dat$epi)) {
      out$epi[[names(dat$epi)[j]]][, s] <- data.frame(dat$epi[j])
    }
  }

  ## Processing for final run
  if (s == dat$control$nsims) {

    # Remove functions from control list
    ftodel <- grep(".FUN", names(out$control), value = TRUE)
    out$control[ftodel] <- NULL

    # Set column names for varying list elements
    for (i in as.vector(which(lapply(out$epi, class) == "data.frame"))) {
      colnames(out$epi[[i]]) <- paste0("sim", 1:dat$control$nsims)
    }

  }

  return(out)
}


#' @title Save netsim Data to Output List Format
#'
#' @description This function transfers the data from the main `netsim_dat`
#'              object to the output `out` object at the end of each
#'              simulation in [netsim()].
#'
#' @inheritParams recovery.net
#' @param s Current simulation number.
#' @param out Out list passed back in for updating at simulations 2+.
#'
#' @return
#' A list with the following elements:
#'
#'  * **param:** the epidemic parameters passed into the model through
#'        [param.net()], with additional parameters added as
#'        necessary.
#'  * **control:** the control settings passed into the model through
#'        [control.net()], with additional controls added as
#'        necessary.
#'  * **epi:** a list of data frames, one for each epidemiological
#'        output from the model.
#'  * **stats:** a list containing two sublists, `nwstats` for any
#'        network statistics saved in the simulation, and `transmat` for
#'        the transmission matrix saved in the simulation.
#'  * **network:** a list of `networkDynamic` objects,
#'         one for each model simulation.
#'
#'
#' @keywords internal
#' @export
#'
saveout.net <- function(dat, s, out = NULL) {

  ## Shared elements: only set up once on first simulation
  if (s == 1) {
    out <- list()
    out$param <- dat$param
    out$control <- dat$control
    out$nwparam <- dat$nwparam
    out$num.nw <- dat$num.nw

    out$epi <- list()
    for (j in seq_along(dat$epi)) {
      out$epi[[names(dat$epi)[j]]] <- data.frame(dat$epi[j])
    }
  }

  ## Shared elements: merge random param values for s > 1
  if (s > 1) {
    if (!is.null(dat$param$random.params.values)) {
      for (nms in names(dat$param$random.params.values)) {
        if (length(dat$param$random.params.values[[nms]]) > 1) {
          if (!is.list(out$param$random.params.values[[nms]])) {
            out$param$random.params.values[[nms]] <- list(
              out$param$random.params.values[[nms]]
            )
          }

          out$param$random.params.values[[nms]] <- c(
            out$param$random.params.values[[nms]],
            list(dat$param$random.params.values[[nms]])
          )

        } else {
          out$param$random.params.values[[nms]] <- c(
            out$param$random.params.values[[nms]],
            dat$param$random.params.values[[nms]]
          )
        }
      }
    }

    for (j in seq_along(dat$epi)) {
      out$epi[[names(dat$epi)[j]]][, s] <- data.frame(dat$epi[j])
    }
  }

  ## Per-simulation elements: use [s] <- list() to preserve NULLs
  out$coef.form[s] <- list(lapply(dat$nwparam, `[[`, "coef.form"))

  if (dat$control$save.run) {
    out$run[s] <- list(dat$run)
  }

  if (dat$control$save.cumulative.edgelist) {
    out$cumulative.edgelist[s] <- list(get_cumulative_edgelists_df(dat))
  }

  out$attr.history[s] <- list(dat$attr.history)
  out$raw.records[s] <- list(dat$raw.records)

  if (dat$control$save.nwstats == TRUE) {
    ## bind rows
    nwstats <- lapply(dat$stats$nwstats, dplyr::bind_rows)
    ## compute ess
    nwstats <- lapply(nwstats, function(y) structure(y, ess = ess(y)))
    out$stats$nwstats[s] <- list(nwstats)
  }

  if (dat$control$save.transmat == TRUE) {
    if (!is.null(dat$stats$transmat)) {
      transmat <- dplyr::bind_rows(dat$stats$transmat)
      row.names(transmat) <- seq_len(nrow(transmat))
      out$stats$transmat[s] <- list(transmat)
    } else {
      out$stats$transmat[s] <- list(dplyr::tibble())
    }
    if (s == 1) {
      class(out$stats$transmat) <- c("transmat", class(out$stats$transmat))
    }
  }

  if (dat$control$save.network == TRUE) {
    ## call get_network to use most up-to-date el and attr in tergmLite case
    out$network[s] <- list(lapply(seq_len(dat$num.nw), get_network, x = dat))
  }

  if (!is.null(dat$control$save.other)) {
    for (i in seq_along(dat$control$save.other)) {
      el.name <- dat$control$save.other[i]
      if (el.name %in% names(dat)) {
        out[[el.name]][s] <- list(dat[[el.name]])
      } else if (el.name %in% names(dat$run)) {
        out[[el.name]][s] <- list(dat$run[[el.name]])
      } else {
        warning("`", el.name, "` is not saved in `dat` or `dat$run`")
      }
    }
  }

  if (dat$control$save.diss.stats == TRUE &&
        dat$control$save.network == TRUE &&
        dat$control$tergmLite == FALSE &&
        !is.null(dat$nwparam)) {

    ## for each simulated network, if dissolution model is edges-only, compute diss stats
    out$diss.stats[s] <- list(lapply(seq_len(dat$num.nw), function(network) {
      if (dat$nwparam[[network]]$coef.diss$diss.model.type == "edgesonly") {
        net <- get_network(dat, network = network)
        toggles_to_diss_stats(
          tedgelist_to_toggles(as.data.frame(net)),
          dat$nwparam[[network]]$coef.diss,
          dat$control$nsteps,
          net
        )
      } else {
        NULL
      }
    }))
  }

  ## Final processing
  if (s == dat$control$nsims) {

    # Set names for out
    simnames <- paste0("sim", seq_len(dat$control$nsims))
    for (i in as.vector(which(lapply(out$epi, class) == "data.frame"))) {
      colnames(out$epi[[i]]) <- simnames
    }

    if (length(out$run) > 0) {
      out$run <- name_saveout_elts(out$run, "run", simnames)
    }

    top_lvl_elts <- c("attr.history", ".records")
    for (elt in top_lvl_elts) {
      out[[elt]] <- name_saveout_elts(out[[elt]], elt, simnames)
    }

    if (dat$control$save.cumulative.edgelist) {
      out$cumulative.edgelist <- name_saveout_elts(
        out$cumulative.edgelist, "cumulative.edgelist", simnames
      )
    }

    out$coef.form <- name_saveout_elts(out$coef.form, "coef.form", simnames)

    if (dat$control$save.nwstats == TRUE) {
      out$stats$nwstats <- name_saveout_elts(out$stats$nwstats,
                                             "stats$nwstats", simnames)
    }

    if (dat$control$save.transmat == TRUE) {
      out$stats$transmat <- name_saveout_elts(out$stats$transmat,
                                              "stats$transmat", simnames)
    }

    if (dat$control$save.network == TRUE) {
      out$network <- name_saveout_elts(out$network, "network", simnames)
    }

    if (dat$control$save.diss.stats == TRUE &&
          dat$control$save.network == TRUE &&
          dat$control$tergmLite == FALSE) {
      out$diss.stats <- name_saveout_elts(out$diss.stats, "diss.stats", simnames)
    }

    # only rename top level elements, not the ones part of `run`
    for (el.name in dat$control$save.other) {
      if (el.name %in% names(out)) {
        out[[el.name]] <- name_saveout_elts(out[[el.name]], el.name, simnames)
      }
    }

    # Remove functions from control list
    ftodel <- grep(".FUN", names(out$control), value = TRUE)
    out$control[ftodel] <- NULL
    out$control$currsim <- NULL
    environment(out$control$nwstats.formula) <- NULL
  }

  return(out)
}


#' @title Save a List of netsim Data to Output List Format
#'
#' @description This function transfers the data from a list of the main
#'              `netsim_dat` objects to the output `out` object at the
#'              end of all simulations in [netsim()].
#'
#' @param dat_list A list of main `netsim_dat` objects in `netsim`
#'        simulations.
#'
#' @return
#' A list of class `netsim` with the following elements:
#'
#'  * **param:** the epidemic parameters passed into the model through
#'        `param`, with additional parameters added as necessary.
#'  * **control:** the control settings passed into the model through
#'        `control`, with additional controls added as necessary.
#'  * **epi:** a list of data frames, one for each epidemiological
#'        output from the model. Outputs for base models always include the
#'        size of each compartment, as well as flows in, out of, and between
#'        compartments.
#'  * **stats:** a list containing two sublists, `nwstats` for any
#'        network statistics saved in the simulation, and `transmat` for
#'        the transmission matrix saved in the simulation. See
#'        [control.net()] for further
#'        details.
#'  * **network:** a list of `networkDynamic` objects,
#'         one for each model simulation.
#'
#'
#' @keywords internal
#' @export
#'
process_out.net <- function(dat_list) {
  for (s in seq_along(dat_list)) {
    # Set output
    if (s == 1) {
      out <- saveout.net(dat_list[[s]], s)
    } else {
      out <- saveout.net(dat_list[[s]], s, out)
    }
  }
  class(out) <- "netsim"

  return(out)
}

# Assign names to the save elements
# Gives a warning if there aren't enough elements
name_saveout_elts <- function(elt, elt_name, simnames) {
  if (length(elt) == length(simnames)) {
    names(elt) <- simnames
  } else if (length(elt) > 0) {
    warning("The number of `", elt_name, "` is not the number of simulations.",
            "Unable to assign each to the correct simulation.")
  }
  return(elt)
}

#' @title Primary Infection Module for netsim
#'
#' @description This function simulates the main infection process given the
#'              current state of the partnerships and disease in the system.
#'
#' @inheritParams recovery.net
#'
#' @details
#' The main steps in this infection module are as follows:
#'
#'  1. Get IDs for current infected and susceptible nodes given the current
#'     disease status.
#'  2. Call [discord_edgelist()] to get the current discordant
#'     edgelist given step 1.
#'  3. Determine the transmission rates (e.g., as a function of group).
#'  4. Pull the number of acts per partnership in a time step from the
#'     `act.rate` parameter.
#'  5. Calculate the final transmission probabilities given the transmission
#'     rates and act rates.
#'  6. Randomly transmit on the discordant edgelist.
#'  7. Conduct bookkeeping for new infections to update status on the nodes
#'     and calculate disease incidence.
#'
#' @inherit recovery.net return
#'
#' @export
#' @keywords netMod internal
#'
#' @seealso [discord_edgelist()] is used within `infection.net`
#' to obtain a discordant edgelist.
#'
infection.net <- function(dat, at) {

  # Variables ---------------------------------------------------------------
  active <- get_attr(dat, "active")
  status <- get_attr(dat, "status")
  infTime <- get_attr(dat, "infTime")

  inf.prob <- get_param(dat, "inf.prob")
  act.rate <- get_param(dat, "act.rate")
  inter.eff <- get_param(dat, "inter.eff", override.null.error = TRUE)
  inter.start <- get_param(dat, "inter.start", override.null.error = TRUE)

  # Vector of infected and susceptible IDs
  idsInf <- which(active == 1 & status == "i")
  nActive <- sum(active == 1)
  nElig <- length(idsInf)

  # Initialize vectors
  nInf <- 0

  # Process -----------------------------------------------------------------
  # If some infected AND some susceptible, then proceed
  if (nElig > 0 && nElig < nActive) {

    # Get discordant edgelist
    del_list <- lapply(seq_len(dat$num.nw), discord_edgelist, dat = dat, at = at, include.network = TRUE)
    del <- dplyr::bind_rows(del_list)

    # If some discordant edges, then proceed
    if (NROW(del) > 0) {

      # Infection duration to at
      del$infDur <- at - infTime[del$inf]
      del$infDur[del$infDur == 0] <- 1

      # Calculate infection-stage transmission rates
      linf.prob <- length(inf.prob)
      del$transProb <- ifelse(del$infDur <= linf.prob,
                              inf.prob[del$infDur],
                              inf.prob[linf.prob])

      # Interventions
      if (!is.null(inter.eff) && at >= inter.start) {
        del$transProb <- del$transProb * (1 - inter.eff)
      }

      # Calculate infection-stage act/contact rates
      lact.rate <- length(act.rate)
      del$actRate <- ifelse(del$infDur <= lact.rate,
                            act.rate[del$infDur],
                            act.rate[lact.rate])

      # Calculate final transmission probability per timestep
      del$finalProb <- 1 - (1 - del$transProb) ^ del$actRate

      # Randomize transmissions and subset df
      transmit <- rbinom(nrow(del), 1, del$finalProb)
      del <- del[which(transmit == 1), ]

      # Set new infections vector
      idsNewInf <- unique(del$sus)
      status <- get_attr(dat, "status")
      status[idsNewInf] <- "i"
      dat <- set_attr(dat, "status", status)
      infTime[idsNewInf] <- at
      dat <- set_attr(dat, "infTime", infTime)
      nInf <- length(idsNewInf)

    } # end some discordant edges condition
  } # end some active discordant nodes condition


  # Output ------------------------------------------------------------------

  # Save transmission matrix

  if (nInf > 0) {
    dat <- set_transmat(dat, del, at)
  }

  ## Save incidence vector
  dat <- set_epi(dat, "si.flow", at, nInf)

  return(dat)
}

#' @title Primary Infection Module for netsim
#'
#' @description This function simulates the main infection process given the
#'              current state of the partnerships and disease in the system.
#'
#' @inheritParams recovery.net
#'
#' @details
#' The main steps in this infection module are as follows:
#'
#'  1. Get IDs for current infected and susceptibles given the current
#'     disease status.
#'  2. Call [discord_edgelist()] to get the current discordant
#'     edgelist given step 1.
#'  3. Determine the transmission rates (e.g., as a function of group).
#'  4. Pull the number of acts per partnership in a time step from the
#'     act.rate` parameter.
#'  5. Calculate the final transmission probabilities given the transmission
#'     rates and act rates.
#'  6. Randomly transmit on the discordant edgelist.
#'  7. Conduct bookkeeping for new infections to update status on the nodes
#'     and calculate disease incidence.
#'
#'
#' @inherit recovery.net return
#'
#' @export
#' @keywords netMod internal
#'
#' @seealso [discord_edgelist()] is used within `infection.net`
#' to obtain a discordant edgelist.
#'
infection.2g.net <- function(dat, at) {

  # Variables ---------------------------------------------------------------

  active <- get_attr(dat, "active")
  infTime <- get_attr(dat, "infTime")
  status <- get_attr(dat, "status")
  group <- get_attr(dat, "group")

  inf.prob <- get_param(dat, "inf.prob")
  inf.prob.g2 <- get_param(dat, "inf.prob.g2")
  act.rate <- get_param(dat, "act.rate")
  inter.eff <- get_param(dat, "inter.eff", override.null.error = TRUE)
  inter.start <- get_param(dat, "inter.start", override.null.error = TRUE)


  # Vector of infected and susceptible IDs
  idsInf <- which(active == 1 & status == "i")
  nActive <- sum(active == 1)
  nElig <- length(idsInf)

  # Initialize vectors
  nInf <- nInfG2 <- totInf <- 0


  # Process -----------------------------------------------------------------
  # If some infected AND some susceptible, then proceed
  if (nElig > 0 && nElig < nActive) {

    # Get discordant edgelist
    del_list <- lapply(seq_len(dat$num.nw), discord_edgelist, dat = dat, at = at, include.network = TRUE)
    del <- dplyr::bind_rows(del_list)

    # If some discordant edges, then proceed
    if (NROW(del) > 0) {

      # Infection duration to at
      del$infDur <- at - infTime[del$inf]
      del$infDur[del$infDur == 0] <- 1

      # Calculate infection-stage transmission rates
      linf.prob <- length(inf.prob)
      if (is.null(inf.prob.g2)) {
        del$transProb <- ifelse(del$infDur <= linf.prob,
                                inf.prob[del$infDur],
                                inf.prob[linf.prob])
      } else {
        #FLAG
        del$transProb <- ifelse(group[del$sus] == 1,
                                ifelse(del$infDur <= linf.prob,
                                       inf.prob[del$infDur],
                                       inf.prob[linf.prob]),
                                ifelse(del$infDur <= linf.prob,
                                       inf.prob.g2[del$infDur],
                                       inf.prob.g2[linf.prob]))
      }

      # Interventions
      if (!is.null(inter.eff) && at >= inter.start) {
        del$transProb <- del$transProb * (1 - inter.eff)
      }

      # Calculate infection-stage act/contact rates
      lact.rate <- length(act.rate)
      del$actRate <- ifelse(del$infDur <= lact.rate,
                            act.rate[del$infDur],
                            act.rate[lact.rate])

      # Calculate final transmission probability per timestep
      del$finalProb <- 1 - (1 - del$transProb) ^ del$actRate

      # Randomize transmissions and subset df
      transmit <- rbinom(nrow(del), 1, del$finalProb)
      del <- del[which(transmit == 1), ]

      # Set new infections vector
      idsNewInf <- unique(del$sus)
      status[idsNewInf] <- "i"
      dat <- set_attr(dat, "status", status)
      infTime[idsNewInf] <- at
      dat <- set_attr(dat, "infTime", infTime)
      nInf <- sum(group[idsNewInf] == 1)
      nInfG2 <- sum(group[idsNewInf] == 2)
      totInf <- nInf + nInfG2

    } # end some discordant edges condition
  } # end some active discordant nodes condition


  # Output ------------------------------------------------------------------

  # Save transmission matrix
  if (totInf > 0) {
    dat <- set_transmat(dat, del, at)
  }

  ## Save incidence vector
  dat <- set_epi(dat, "si.flow", at, nInf)
  dat <- set_epi(dat, "si.flow.g2", at, nInfG2)

  return(dat)
}

#' @title Discordant Edgelist
#'
#' @description This function returns a `data.frame` with a discordant
#'              edgelist, defined as the set of edges in which the status of the
#'              two partners is one susceptible and one infected.
#'
#' @inheritParams recovery.net
#' @param network In case of models with multiple networks, the network to pull
#'        the current edgelist from. Default of `network = 1`.
#' @param infstat Character vector of disease status values that are considered
#'        infectious, defining the SI pairs.
#' @param include.network Should the `network` value be included as the
#'        final column of the discordant edgelist?
#'
#' @details
#' This internal function works within the parent [infection.net()]
#' function to pull the current edgelist from the dynamic network object, look
#' up the disease status of the head and tails on the edge, and subset the list
#' to those edges with one susceptible and one infected node.
#'
#' EpiModel v2.0.3 extended the function by allowing flexibility in the
#' definition what disease status counts as infectious, with the `infstat`
#' parameter. For extension models with multiple infectious states, this can be
#' a vector of length greater than 1: `infstat = c("i", "a")`.
#'
#' @return
#' This function returns a `data.frame` with the following columns:
#'
#'  * **time:** time step queried.
#'  * **sus:** ID number for the susceptible partner.
#'  * **inf:** ID number for the infectious partner.
#'
#' The output from this function is added to the transmission `data.frame`
#' object that is requested as output in `netsim` simulations with
#' the `save.trans=TRUE` argument.
#'
#' @seealso [netsim()], [infection.net()]
#'
#' @export
#' @keywords netMod internal
#'
discord_edgelist <- function(dat, at, network = 1, infstat = "i", include.network = FALSE) {

  status <- get_attr(dat, "status")
  active <- get_attr(dat, "active")

  el <- get_edgelist(dat, network)

  del <- NULL
  if (nrow(el) > 0) {
    el <- el[sample(seq_len(nrow(el))), , drop = FALSE]
    stat <- matrix(status[el], ncol = 2)
    isInf <- matrix(stat %in% infstat, ncol = 2)
    isSus <- matrix(stat %in% "s", ncol = 2)
    SIpairs <- el[isSus[, 1] * isInf[, 2] == 1, , drop = FALSE]
    ISpairs <- el[isSus[, 2] * isInf[, 1] == 1, , drop = FALSE]
    pairs <- rbind(SIpairs, ISpairs[, 2:1])
    if (nrow(pairs) > 0) {
      sus <- pairs[, 1]
      inf <- pairs[, 2]
      del <- data.frame(at, sus, inf)

      # Check for active status
      keep <- rowSums(matrix(c(active[del$sus], active[del$inf]),
                             ncol = 2)) == 2
      del <- del[keep, ]
      if (nrow(del) < 1) {
        del <- NULL
      }
    }
  }

  if (NROW(del) > 0 && include.network == TRUE) {
    del <- dplyr::bind_cols(del, network = network)
  }

  return(del)
}

#' @title Save Transmission Matrix
#'
#' @description This function appends the transmission matrix created during
#'              `infection.net` and `infection.2g.net`.
#'
#' @inheritParams recovery.net
#' @param del Discordant edgelist created within [infection.net()] and
#'        [infection.2g.net()].
#'
#' @details
#' This internal function works within the parent [infection.net()]
#' functions to save the transmission matrix created at time step `at` to
#' the main `netsim_dat` class object `dat`.
#'
#' @inherit recovery.net return
#'
#' @export
#'
set_transmat <- function(dat, del, at) {
  if (is.null(dat$stats$transmat)) {
    dat$stats$transmat <- list()
  }

  if (length(dat$stats$transmat) < at) {
    nsteps <- get_control(dat, "nsteps")
    dat$stats$transmat <- padded_vector(dat$stats$transmat, nsteps)
  }

  del <- del[!duplicated(del$sus), ]
  # Convert the Discordant Edglist indexes to the corresponding unique_ids
  del[["sus"]] <- get_unique_ids(dat, del[["sus"]])
  del[["inf"]] <- get_unique_ids(dat, del[["inf"]])

  if (length(dat$stats$transmat) < at) {
    dat$stats$transmat <- padded_vector(
      dat$stats$transmat,
      get_control(dat, "nsteps")
    )
  }

  dat$stats$transmat[[at]] <- del

  return(dat)
}

---
title: 'EpiModel: Mathematical Modeling of Infectious Disease Dynamics'
author: "Samuel M. Jenness, Steven M. Goodreau, Martina Morris, and Adrien Le Guillou"
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{EpiModel Introduction}
  \usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, echo=FALSE}
vers <- packageVersion("EpiModel")
year <- format(Sys.time(), "%Y")
```

### Introduction

The **EpiModel** package provides tools for simulating mathematical models of infectious disease dynamics. Epidemic model classes include deterministic compartmental models, stochastic individual-contact models, and stochastic network models. Network models use the robust statistical methods of temporal exponential-family random graph models (ERGMs) from the Statnet suite of software packages in R. Standard templates for epidemic modeling include SI, SIR, and SIS disease types. EpiModel features an API for extending these templates to address novel scientific research aims.

### Documentation

This vignette provides a general orientation to the **EpiModel** tutorials and documentation within the package and hosted elsewhere online. Most full-length tutorials may be found at the **EpiModel** website (<https://www.epimodel.org>), which also points to our **Network Modeling for Epidemics (NME)** short-course (<https://epimodel.github.io/sismid/>). 

The current version of EpiModel is v`r vers`. Within the package, you can consult the extensive help documentation and vignettes for each exported function:

```{r, eval=FALSE}
help(package = "EpiModel")
```

To see the latest updates to **EpiModel**, consult the `NEWS` file in the package, which is also summarized on our Github Releases (<https://github.com/EpiModel/EpiModel/releases>).

If you are interested in the stochastic network model class, we suggest learning about using **EpiModel** with the following sequence:

1. The best place to start is our primary main methods paper, published in the *Journal of Statistical Software* at the reference below. 
2. After that, we suggest you work through the Network Modeling for Epidemics course materials to understand some of the background network theory and see how to run parameterize and run built-in network models.
3. Those interested in extending EpiModel for novel research aims should then consult the model extension materials of the NME course, and look at the variety of extension model templates in our **EpiModel Gallery** (<https://github.com/EpiModel/EpiModel-Gallery>).
4. Some of the latest developments in EpiModel are related to working with network model inputs and outputs, which are covered in these advanced topics vignettes within the package:

- Working With Model Parameters in EpiModel
- Working With Custom Attributes and Summary Statistics in EpiModel
- Working With Network Objects in EpiModel

### Getting Help

Any technical coding questions, non-technical conceptual modeling questions, or EpiModel feature requests may be posted as a Github issues at our main Github repository (<https://github.com/EpiModel/EpiModel/issues>).

### EpiModel Citation

If using EpiModel for teaching or research, please include a citation to our primary methods paper:

> Jenness SM, Goodreau SM and Morris M. EpiModel: An R Package for Mathematical Modeling of Infectious Disease over Networks. *Journal of Statistical Software.* 2018; 84(8): 1-47. doi: 10.18637/jss.v084.i08 (<https://doi.org/10.18637/jss.v084.i08>).

---
title: "Working with Custom Attributes and Summary Statistics in EpiModel"
author: "EpiModel v`r packageVersion('EpiModel')`"
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{Working with Custom Attributes and Summary Statistics}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r, echo = FALSE, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, echo = FALSE, include = FALSE}
library(EpiModel)
```

## Introduction

This vignette covers how to work with nodal attributes, attribute histories, and epidemic summary statistics in EpiModel network models with custom extension modules. It assumes familiarity with setting up and running network models with `netsim()` and with the extension API. See the [Network Modeling for Epidemics](https://epimodel.github.io/sismid/) (NME) course materials and the [EpiModel Gallery](https://epimodel.github.io/EpiModel-Gallery/) for background.

For working with network objects and edgelists, see the companion vignette *Working with Network Objects in EpiModel*. For parameter management, see *Working with Model Parameters in EpiModel*.

In network simulations with `netsim`, all simulation state is stored in the *main list object* (referred to as `dat`). This vignette covers three types of data on `dat`:

- **Current Nodal Attributes:** Per-node characteristics at the current time step.
- **Historical Nodal Attributes:** Selective recordings of attribute values over time.
- **Epidemic Trackers:** Population-level summary statistics computed at each time step.


## Current Nodal Attributes

*Attributes* are characteristics of the nodes (e.g., persons) in the model at the current time step. They are stored as vectors in `dat$attr`, all of the same length as the number of nodes.

### Core Attributes

Every node has four core attributes, managed automatically by EpiModel:

- `active`: 1 if the node is currently in the population, 0 if departed.
- `entrTime`: the time step when the node entered the population.
- `exitTime`: the time step when the node departed (`NA` if still active).
- `unique_id`: a globally unique integer identifier (never reused).

You can retrieve the list of core attributes and their default types with `get_core_attributes()`:

```{r}
get_core_attributes()
```

In addition, the built-in epidemic modules in core EpiModel use the `status` attribute (disease status, with values like `"s"`, `"i"`, `"r"`) and `infTime` (time of infection). More complex disease-specific EpiModel implementations (e.g., for HIV or COVID) may handle disease status differently. Custom attributes of any name and scalar value can be added, such as `age`, `race`, or `viral_load`.

### Positional Indexing vs Unique IDs

Each node can be referenced in two ways:

- **By position:** Think of it like a row number in a spreadsheet. `dat$attr$active[3]` accesses the third node's value directly. This is the standard way to look up node information and is very fast.

- **By `unique_id`:** A globally unique integer attribute assigned at creation and never reused. Slower to look up, but allows referencing nodes that have already departed.

Consider a simple example. A model starts with 5 nodes:

| Position (*row number*) | `unique_id` | `active` |
|:-----------------------:|:-----------:|:--------:|
| 1 | 1 | 1 |
| 2 | 2 | 1 |
| 3 | 3 | 1 |
| 4 | 4 | 1 |
| 5 | 5 | 1 |

Node 3 departs at step 10. The vectors still have 5 elements, but `active[3] = 0`:

| Position (*row number*) | `unique_id` | `active` |
|:-----------------------:|:-----------:|:--------:|
| 1 | 1 | 1 |
| 2 | 2 | 1 |
| 3 | 3 | **0** |
| 4 | 4 | 1 |
| 5 | 5 | 1 |

A new node arrives at step 15. In many simulations, departed nodes are dropped from the attribute vectors, so the new node takes position 3 (the freed slot) but receives a new `unique_id = 6`:

| Position (*row number*) | `unique_id` | `active` |
|:-----------------------:|:-----------:|:--------:|
| 1 | 1 | 1 |
| 2 | 2 | 1 |
| 3 | **6** | 1 |
| 4 | 4 | 1 |
| 5 | 5 | 1 |

Positional indices are used by current edgelists and attribute accessors. Unique IDs are used by cumulative edgelists and attribute histories because they can refer to nodes no longer in the model. See `help("unique_id-tools", package = "EpiModel")` for conversion functions.

### Working With Attributes

#### Accessing Attributes

Use `get_attr()` to extract an attribute vector:

```{r accessing-attr, eval = FALSE}
active <- get_attr(dat, "active")
```

This returns a vector of length equal to the current number of nodes. Requesting a nonexistent attribute throws an error.

#### Modifying Attributes

Below is a minimal module that increments the age of all nodes by 1:

```{r modyfing-attr, eval = FALSE}
aging_module <- function(dat, at) {

  # Extract current attribute
  age <- get_attr(dat, "age")

  # Aging process
  new_age <- age + 1

  # Output updated attributes
  dat <- set_attr(dat, "age", new_age)

  return(dat)
}
```

Key points about `set_attr()`:

- It returns a modified copy of `dat`, which must be assigned back. (This does not cause performance issues due to R's shallow copy semantics since R 3.1.)
- The replacement vector must have the same length as the number of nodes, or the function throws an error.

#### Summary

The recommended workflow for attribute manipulation in modules:

1. Extract attributes into local vectors with `get_attr()`.
2. Modify the local vectors with your dynamic process.
3. Update `dat` with the revised vectors using `set_attr()`.
4. Return `dat` at the end of the function.

These functions have additional arguments described in `help("net-accessor", package = "EpiModel")`.


## Historical Nodal Attributes

Current attributes reflect the state of each node at the present time step. Sometimes you need to track what happened to specific nodes over time. Because saving the full history of all attributes would consume too much memory, EpiModel provides a selective recording mechanism: you choose which attributes, for which nodes, at which time steps to record.

This is useful for diagnostics (verifying that a dynamic process is coded correctly) or for tracking a limited set of individual-level outcomes for further analysis.

### Recording Attribute Histories

The following module records the viral load of infected nodes every 10 time steps. It assumes the model defines a `status` attribute (with values `"s"` for susceptible, `"i"` for infected) and a `viral_load` attribute (continuous for infected nodes, `NA` otherwise).

```{r recording_attr_hist, eval = FALSE}
viral_load_logger_module <- function(dat, at) {

  # Run every 10 time steps
  if (at %% 10 == 0) {

    # Attributes
    status <- get_attr(dat, "status")
    viral_load <- get_attr(dat, "viral_load")

    infected <- which(status == "i")

    dat <- record_attr_history(
      dat, at,
      "viral_load",
      infected,
      viral_load[infected]
    )
  }

  return(dat)
}
```

`record_attr_history()` takes five arguments:

1. `dat`: the main list object.
2. `at`: the time step to associate with the recording (usually the current step).
3. A label for the attribute (here `"viral_load"`).
4. A vector of `posit_id`s for the nodes of interest. Internally, these are converted to `unique_id`s so the history remains valid even after nodes depart.
5. The values to record for those IDs. The number of values must match the number of IDs, unless a single value is provided (which is recycled for all IDs, using less memory).

Use attribute histories sparingly---storage can grow large in open-population models with many nodes over many time steps.

### Accessing Attribute Histories

Attribute histories are accessed after the simulation is complete:

```{r access_attr_hist, eval = FALSE}
sim <- netsim(est, param, init, control)
attr_history <- get_attr_history(sim)
```

The result is a named list of `data.frame`s, one for each recorded attribute label. Suppose we recorded `"viral_load"` (as above) and also `"status"` transitions in another module. The output would look like:

```{r get_attr_ex, eval = FALSE}
get_attr_history(sim)

# $viral_load
#    sim step attribute    uids values
# 1    1   10 viral_load   1001   2000
# 2    1   10 viral_load   1002   1878
# 3    1   20 viral_load   1001   1500
# 4    1   20 viral_load   1002    300
# ...
#
# $status
#    sim step attribute  uids  values
# 1    1   22 status     1001       i
# 2    1   64 status     1002       i
# 3    1  110 status     1001       r
# 4    1  220 status     1002       r
# ...
```

Each `data.frame` has columns `sim` (simulation number), `step` (time step), `attribute` (the label), `uids` (unique IDs), and `values` (the recorded values). The list structure allows each history to have different dimensions.


## Epidemic Trackers

*Epidemic trackers* are population-level summary statistics computed and stored at each time step---historical summary statistics about the population, not individual nodes.

For example, the built-in tracker `num` records the total population size at each step, while `s.num`, `i.num`, and `r.num` record compartment sizes.

### Working With Epidemic Trackers in Modules

Inside a module, access and modify epidemic trackers with `get_epi()` and `set_epi()`. Here is an extended aging module that also tracks mean age and its change over time:

```{r epi-in-modules, eval = FALSE}
aging_track_module <- function(dat, at) {

  # Attributes
  age <- get_attr(dat, "age")

  # Aging process
  new_age <- age + 1

  # Calculate summary statistics
  mean_age <- mean(new_age)
  prev_mean_age <- get_epi(dat, at - 1, "mean_age")
  age_change <- mean_age - prev_mean_age

  # Update nodal attributes
  dat <- set_attr(dat, "age", new_age)

  # Update epidemic trackers
  dat <- set_epi(dat, "mean_age", at, mean_age)
  dat <- set_epi(dat, "age_change", at, age_change)

  return(dat)
}
```

Key points:

- `set_epi()` returns a modified `dat` that must be assigned back. The value must be a scalar.
- `get_epi()` retrieves the tracker value at a specific time step. Requesting a nonexistent tracker throws an error.

### Accessing Epidemic Trackers After a Simulation

Epidemic trackers are the main quantities of interest after a simulation. Access them by calling `as.data.frame()` on a `netsim` object, or use `plot()` and `summary()`:

```{r, eval = FALSE}
sim <- netsim(est, param, init, control)

# Raw per-simulation data
as.data.frame(sim)

# Means across simulations
as.data.frame(sim, out = "mean")

# Plot and summary
plot(sim)
summary(sim, at = 50)
```

### Derived Statistics with `mutate_epi()`

After a simulation, you can compute derived epidemic statistics without re-running the model. `mutate_epi()` works like `dplyr::mutate()` but operates on the `epi` list:

```{r, eval = FALSE}
# Add incidence rate and prevalence
sim <- mutate_epi(sim, prev = i.num / num)
sim <- mutate_epi(sim, ir = (si.flow / s.num) * 100)

# These new variables appear in the data.frame output
as.data.frame(sim)
```

### Stratified Prevalence with `epi.by`

The `epi.by` parameter in `control.net()` automatically generates stratified compartment counts by a nodal attribute. For example, to track infection by a `"risk"` attribute:

```{r tracker-epiby, results = "hide", message = FALSE}
set.seed(0)

nw <- network_initialize(n = 100)
nw <- set_vertex_attribute(nw, "risk", rep(0:1, each = 50))

est <- netest(
  nw, formation = ~edges + nodefactor("risk"),
  target.stats = c(40, 20),
  coef.diss = dissolution_coefs(~offset(edges), 20),  # duration = 20
  verbose = FALSE
)

param <- param.net(inf.prob = 0.3, act.rate = 1)
init <- init.net(i.num = 5)
control <- control.net(
  type = "SI", nsims = 1, nsteps = 50,
  epi.by = "risk",
  verbose = FALSE
)

sim <- netsim(est, param, init, control)
```

With `epi.by = "risk"`, the output includes stratified columns automatically:

```{r}
d <- as.data.frame(sim)
names(d)
```

The naming pattern is `{tracker}.{attribute}{value}` (e.g., `i.num.risk0`, `i.num.risk1`, `num.risk0`, `num.risk1`).


### Custom Epidemic Trackers

For summary statistics beyond built-in compartment counts, you can define custom tracker functions and pass them to `control.net()`. This avoids writing a full module for simple calculations.

#### Writing Tracker Functions

A tracker function must take **exactly one argument** (`dat`) and return a **scalar** value (numeric, logical, or character). It is called automatically at the end of each time step, after all modules have run.

```{r tracker-example}
epi_s_num <- function(dat) {
  needed_attributes <- c("status")
  output <- with(get_attr_list(dat, needed_attributes), {
    sum(status == "s", na.rm = TRUE)
  })
  return(output)
}
```

Here is a tracker that computes the proportion of the population infected:

```{r tracker-commented}
epi_prop_infected <- function(dat) {
  needed_attributes <- c("status", "active")
  output <- with(get_attr_list(dat, needed_attributes), {
    pop <- active == 1
    infected <- status == "i"
    sum(infected & pop, na.rm = TRUE) / sum(pop, na.rm = TRUE)
  })
  return(output)
}
```

We recommend this pattern for tracker functions:

1. Define `needed_attributes` as a vector of attribute names.
2. Use `with(get_attr_list(dat, needed_attributes), { ... })` to work in a clean environment.
3. The last expression inside `with` becomes the output. It **must** be a scalar.
4. Return the output.

#### Using Custom Tracker Functions

Pass tracker functions as a named list via `.tracker.list` in `control.net()`. The names become column names in the output `data.frame`:

```{r tracker-list}
some.trackers <- list(
  prop_infected = epi_prop_infected,
  s_num         = epi_s_num
)

control <- control.net(
  type = "SI",
  nsims = 1,
  nsteps = 50,
  verbose = FALSE,
  .tracker.list = some.trackers
)

param <- param.net(
  inf.prob = 0.3,
  act.rate = 0.1
)

nw <- network_initialize(n = 50)
nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
est <- netest(
  nw,
  formation = ~edges,
  target.stats = 25,
  coef.diss = dissolution_coefs(~offset(edges), 10, 0),  # duration = 10, closed population
  verbose = FALSE
)

init <- init.net(i.num = 10)
sim <- netsim(est, param, init, control)

d <- as.data.frame(sim)

knitr::kable(tail(d, n = 15))
```

Note that in the `.tracker.list` we pass the function objects (`epi_prop_infected`, `epi_s_num`) without parentheses---we are storing the functions, not calling them.

Trackers are **always** run at the end of a simulation step. The values they output reflect the state of the simulation *after* all modules have completed their work for that step.


## Recording Arbitrary Objects (Advanced Debugging)

When debugging complex research models, you may want to inspect the state of an object during simulation without stopping it. `record_raw_object()` saves any R object at a given time step for later inspection.

```{r record-raw, eval = FALSE}
introspect_module <- function(dat, at) {
  age <- get_attr(dat, "age")

  if (mean(age, na.rm = TRUE) > 50) {
    obj <- data.frame(
        age = age,
        status = get_attr(dat, "status")
    )
    dat <- record_raw_object(dat, at, "old pop", obj)
  }

  return(dat)
}
```

After simulation, access recorded objects from the `netsim` output:

```{r record-raw-access, eval = FALSE}
sim <- netsim(est, param, init, control)

# Access raw records for simulation 1
# Returns a nested list keyed by label, then time step
raw <- sim[["raw.records"]][[1]]

# Access the "old pop" data.frame recorded at step 75
raw[["old pop"]][[75]]
```

**Use sparingly:** Raw records are not truncated and can consume substantial memory, especially for large attribute vectors recorded at many time steps. This feature is intended for debugging, not routine output collection.

---
title: "Working with Network Objects in EpiModel"
author: "EpiModel v`r packageVersion('EpiModel')`"
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{Working with Network Objects}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r, echo = FALSE, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, echo = FALSE, include = FALSE}
library(EpiModel)
```

## Introduction

This vignette discusses mechanisms usable inside `EpiModel` network models with custom modules. More information about these in the [Extending EpiModel](https://epimodel.github.io/sismid/9_extending/mod9-Intro.html) section of the [Network Modeling for Epidemics](https://epimodel.github.io/sismid/) course materials.

Inside the simulation, the networks themselves are stored under `dat$run$nw`. Ultimately this vignette will describe multiple aspects of working with network
objects.

## Cumulative Edgelist

The cumulative edgelist refers to the historical list of edges in a network with the time step they start and stopped. Such a list allows to query current relationships (contacts, partnerships, etc.) as well as past ones.

### Using the Cumulative Edgelist

The creation and update of the cumulative edgelist is done through the `EpiModel::update_cumulative_edgelist` function.

```{r create-el_cuml, eval = FALSE}
dat <- update_cumulative_edgelist(dat, network, truncate = Inf)
```

This function takes 3 arguments:

1. `dat`: the *Main List Object*.
2. `network`: the number of the network for which to create the cumulative edgelist (for multi-layer network simulations).
3. `truncate`: a number of time steps after which an inactive edge should be removed from the cumulative edgelist (this saves computer memory for large-scale simulations).

The function returns a modified version of `dat` that needs to be assigned back.

The following snippet will update the cumulative edgelist for all the networks used by a model and remove the edges that ended more than 100 steps ago.

```{r el_cuml-snippet, eval = FALSE}
for (n_network in seq_along(dat$run$nw)) {
  dat <- update_cumulative_edgelist(dat, n_network, truncate = 100)
}
```

In a complete model, this code would need to be run at the end of the initialization module and at each time-step after the resimulation of the networks.

### Accessing the Cumulative Edgelist

Cumulative edge-list refers to nodes with their Unique Ids. See `help("unique_id-tools", package = "EpiModel")` for more information.

#### For a Specific Network

Accessing the cumulative edge-list of a given network is done using the `EpiModel::get_cumulative_edgelist` function.

```{r get_el_cuml, eval = FALSE}
el_cuml <- get_cumulative_edgelist(dat, network)
```

The returned `el_cuml` object is a [`tibble`](https://tibble.tidyverse.org/) with four columns:

1. `head`: the `unique_id` first node of the edge.
2. `tail`: the `unique_id` second node of the edge.
3. `start`: the time-step where the edge was created.
4. `stop`: the last time-step the edge was active.

`EpiModel::get_cumulative_edgelist` will **always** return a [`tibble`](https://tibble.tidyverse.org/) with this 4 columns, even if the cumulative edgelist has not been calculated for this particular network or if no edges are present. In these cases, the [`tibble`](https://tibble.tidyverse.org/) will have no rows but keep the correct column structure.

The `stop` column will **always** contain `NA` if an edge is currently active.

Once an edge is not present anymore, the `stop` column for this edge will contains the last step the edge was active. This means that an edge with a `stop` value existed from `start` to `stop` both inclusive. This makes it coherent with how `R` treats the indexes in a vector for instance (from 1 to `length(vector)` inclusive).

#### For Multiple Networks

We often want to get the cumulative edgelist over several networks as one. `EpiModel::get_cumulative_edgelists_df` function provide such functionality.

```{r get_el_cuml_dfs, eval = FALSE}
el_cumls <- get_cumulative_edgelists_df(dat, networks = NULL)
```

The `networks` argument can be a vector of network position or `NULL`. In this latter case, all networks will be selected.

The output of this function is similar to `EpiModel::get_cumulative_edgelist` with the addition of a `network` column, indicating for each edge the networks
it exists on.

### Contact Tracing

A typical use of the cumulative edgelist is the trace the contacts of a node over given number of steps. The `EpiModel::get_partners` function simplifies this process:

```{r get_partners, eval = FALSE}
partner_list <- get_partners(
    dat,
    index_posit_ids,
    networks = NULL,
    truncate = Inf,
    only.active.nodes = FALSE
)
```

Here we call "indexes" the nodes whose partners (contacts) we want to extract. The arguments are:

1. `dat`: as in `get_cumulative_edgelists_df`.
2. `index_posit_ids`: a list of positional Ids for the indexes of interest.
3. `networks`: as in `get_cumulative_edgelists_df`.
4. `truncate`: similar to the `truncate` argument to `update_cumulative_edgelist` this argument filter out partnerships over this age.
5. `only.active.nodes`: if set to `TRUE`, partnership with inactive nodes are removed.

The output is similar to `get_cumulative_edgelists_df` but the first two columns are called `index` and `partner` and contains the Unique Ids of the indexes given in argument in the first column and there partners in the second one.

Note that the we refer to the indexes of interest with their Positional Ids but the `index` and `partners` columns contains Unique Ids as they can refer to
nodes no longer in the network.

---
title: "Working with Model Parameters in EpiModel"
author: "EpiModel v`r packageVersion('EpiModel')`"
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{Working with Model Parameters}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r, echo = FALSE, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r, echo=FALSE, message=FALSE}
library(EpiModel)
```


## Introduction

This vignette assumes familiarity with setting up and running network models in EpiModel. See the [Network Modeling for Epidemics](https://epimodel.github.io/sismid/) (NME) course materials and the [EpiModel Gallery](https://epimodel.github.io/EpiModel-Gallery/) for background.

For working with nodal attributes and epidemic summary statistics, see the companion vignette *Working with Custom Attributes and Summary Statistics in EpiModel*. For network objects and edgelists, see *Working with Network Objects in EpiModel*.

In a model, *parameters* are the input variables that define aspects of system behavior. In a basic SIS (Susceptible-Infected-Susceptible) model, these might include an **infection probability**, an **act rate**, and a **recovery rate**. In simple models, each parameter is a single fixed value that remains constant throughout a simulation. Real-world applications, however, often require more flexibility: parameters that change over time (e.g., to represent an intervention rollout or behavioral change during an epidemic), parameters drawn from distributions (for sensitivity analysis or calibration), or large parameter sets managed via spreadsheets.

This vignette demonstrates how to implement:

- **Scenarios:** Named sets of parameter changes applied at the start of a simulation or at specific time steps---the primary mechanism for time-varying parameters, counterfactual analysis, and intervention modeling.
- **Parameter input via tables:** Passing many parameters at once through a `data.frame`, enabling spreadsheet-based parameter management.
- **Random parameters:** Distributions of parameter values rather than single fixed values, for uncertainty quantification and sensitivity analysis.
- **Parameter and control updaters (advanced):** The low-level mechanism underlying scenarios, exposed directly for cases where the scenario API is not flexible enough.

## Scenarios

Scenarios allow you to define named sets of parameter values that can change at specific time steps during a simulation. This is the recommended approach for implementing time-varying parameters---for example, modeling an intervention that reduces transmission probability partway through an epidemic, or comparing multiple policy alternatives.

Our research paper implementing time-varying parameter scenarios for COVID-19 sexual distancing interventions was published in the *Journal of Infectious Diseases*. The code [can be found here](https://github.com/EpiModel/SexualDistancing).

### Setup

First, we set up a simple SIS model as the base case.

```{r setup, results = "hide"}
set.seed(10)

nw <- network_initialize(n = 200)
est <- netest(nw,
  formation = ~edges, target.stats = 60,
  coef.diss = dissolution_coefs(~offset(edges), 10, 0),  # duration = 10, closed population
  verbose = FALSE
)

param <- param.net(inf.prob = 0.9, rec.rate = 0.01, act.rate = 2)
init <- init.net(i.num = 10)
control <- control.net(type = "SIS", nsims = 1, nsteps = 250, verbose = FALSE)
```

### Scenario Definitions

Scenarios are defined using `create_scenario_list`, which takes a specially formatted `data.frame` as input and outputs a list of scenarios usable by EpiModel. We use `tibble::tribble` here for readability, but any `data.frame` constructor works.

```{r scenarios_setup}
library(dplyr)

scenarios.df <- tribble(
  ~.scenario.id, ~.at, ~inf.prob, ~rec.rate,
  "base", 0, 0.9, 0.01,
  "initial_change", 0, 0.2, 0.01,
  "multiple_changes", 0, 0.1, 0.04,
  "multiple_changes", 100, 0.9, 0.01,
  "multiple_changes", 200, 0.1, 0.1
)

knitr::kable(scenarios.df)
```

The `data.frame` requires two columns:

- `.scenario.id`: an identifier for the scenario.
- `.at`: the time step at which the parameter changes should apply. Use `0` for changes that take effect before the simulation begins.

Remaining columns are parameter names. They must start with a letter and contain only letters, numbers, or `.` (underscores are reserved for vector indexing, described below). If a cell is `NA`, EpiModel sets that parameter to `NA`---it does *not* mean "leave unchanged."

In this example, the `"multiple_changes"` scenario has three rows sharing the same `.scenario.id`, representing a single scenario with parameter changes at three different time points.

To convert from the `data.frame` to a usable list of scenarios:

```{r scenarios_list}
scenarios.list <- create_scenario_list(scenarios.df)
str(scenarios.list, max.level = 2)
```

### Running Scenarios

We loop over all scenarios, using `use_scenario` to create a modified parameter object for each. Under the hood, `use_scenario` generates parameter updaters (described in the Advanced section below) that apply the specified changes at the designated time steps.

```{r simulations}
# List to hold simulation results
d_list <- vector(mode = "list", length = length(scenarios.list))
names(d_list) <- names(scenarios.list)

for (scenario in scenarios.list) {
  print(scenario$id)
  sc.param <- use_scenario(param, scenario)
  sim <- netsim(est, sc.param, init, control)
  d_sim <- as.data.frame(sim)
  d_sim[["scenario"]] <- scenario$id
  d_list[[scenario$id]] <- d_sim
}
```

For the `"multiple_changes"` scenario, messages appear at time steps 100 and 200 indicating that parameters were modified during the simulation. The other scenarios are silent because their changes occur before the simulation begins (`.at = 0`).

Now we can plot the results. We use base R plotting here rather than `plot.netsim()` because we need to overlay results from separate `netsim` runs on a single axis.

```{r plotting, fig.width = 8}
plot(d_list$base$time, d_list$base$i.num,
     type = "l", col = 1, lwd = 2, ylim = c(0, 250),
     xlab = "Time Step", ylab = "Number Infected")
lines(d_list$initial_change$time, d_list$initial_change$i.num,
      type = "l", col = 2, lwd = 2)
lines(d_list$multiple_changes$time, d_list$multiple_changes$i.num,
      type = "l", col = 3, lwd = 2)
abline(v = c(100, 200), lty = 2)
legend("topleft", legend = names(d_list),
       col = 1:3, lwd = 2, cex = 0.9, bty = "n")
```

The `"initial_change"` scenario shows a slower epidemic than the base case because the infection probability is reduced from 0.9 to 0.2 from the start.

The `"multiple_changes"` scenario demonstrates three phases:

- Steps 1--99: Slow growth (low infection probability, higher recovery rate).
- Steps 100--199: Base-case parameters applied, so the epidemic trajectory accelerates.
- Steps 200--250: Infection probability drops and recovery rate increases, driving rapid epidemic decline.

### Scenarios with Parameter Vectors

In full-scale modeling projects, parameters are often vectors. For example, `hiv.test.rate` might be a vector of length 3 containing weekly HIV screening probabilities for Black, Hispanic, and White persons.

To use vectors in scenarios, create a separate column for each element using the naming convention `paramname_N`, where `N` is the position in the vector:

```{r multi_param}
scenarios.df <- tribble(
  ~.scenario.id, ~.at, ~hiv.test.rate_1, ~hiv.test.rate_2, ~hiv.test.rate_3,
  "base", 0, 0.001, 0.001, 0.001,
  "initial_change", 0, 0.002, 0.001, 0.002,
  "multiple_changes", 0, 0.002, 0.001, 0.002,
  "multiple_changes", 100, 0.004, 0.001, 0.004,
  "multiple_changes", 200, 0.008, 0.001, 0.008
)
knitr::kable(scenarios.df)
```

All elements of a vector parameter must be specified in the `data.frame`, even if some values do not change across scenarios (e.g., `hiv.test.rate_2` is constant here but must still be included).

When working with many parameters, we recommend storing the `scenarios.df` as a CSV, RDS or Excel file for easier sharing and editing.

```{r, eval = FALSE}
# CSV
write.csv(scenarios.df, "scenarios.csv", row.names = FALSE)
scenarios.df <- read.csv("scenarios.csv")

# RDS
saveRDS(scenarios.df, "scenarios.rds")
scenarios.df <- readRDS("scenarios.rds")
```

## Parameter Input via Table

The `param.net` function accepts a `data.frame` of parameters through the `data.frame.params` argument. This allows passing many parameters at once or working with a spreadsheet to track and update parameters before use in EpiModel.

The `data.frame` requires three columns:

1. `param`: The parameter name. For vector parameters (length > 1), append the position with an underscore (e.g., `"p_1"`, `"p_2"`).
2. `value`: The parameter value (as a character string).
3. `type`: The type of the parameter value. Only three values are accepted: `"numeric"`, `"logical"`, or `"character"`.

Additional columns (e.g., `details`, `source`) may be included for documentation but are ignored by EpiModel.

```{r param_table}
df_params <- tribble(
  ~param, ~value, ~type,
  "hiv.test.rate_1",  "0.003",        "numeric",
  "hiv.test.rate_2",  "0.102",        "numeric",
  "hiv.test.rate_3",  "0.492",        "numeric",
  "prep.require.lnt", "TRUE",         "logical",
  "group_1",          "first",        "character",
  "group_2",          "second",       "character"
)
knitr::kable(df_params)
```

Pass the table via `data.frame.params`. These parameters can be combined with named parameters for maximum flexibility. In case of conflict, named parameters take priority over those in the `data.frame`:

```{r param_table2}
param <- param.net(data.frame.params = df_params,
                   other.param = c(5, 10), act.rate = 1)
param
```


## Random Parameters

Fixed parameters assume that each input value is known with certainty. In practice, we may want to explore how uncertainty in parameter values propagates to uncertainty in epidemic outcomes. EpiModel supports drawing parameter values from distributions, so that each simulation uses a different realization.

### The Model

We demonstrate with a simple SI model.

```{r model}
nw <- network_initialize(n = 50)

est <- netest(
  nw, formation = ~edges,
  target.stats = 25,
  coef.diss = dissolution_coefs(~offset(edges), 10, 0),  # duration = 10, closed population
  verbose = FALSE
)

param <- param.net(
  inf.prob = 0.3,
  act.rate = 0.5,
  dummy.param = 4,
  dummy.strat.param = c(0, 1)
)

init <- init.net(i.num = 10)
control <- control.net(type = "SI", nsims = 1, nsteps = 5, verbose = FALSE)
mod <- netsim(est, param, init, control)
mod
```

Here we define four parameters: `inf.prob` (which will remain fixed), `act.rate`, `dummy.param`, and `dummy.strat.param` (which we will make random below). The `dummy.strat.param` parameter is a vector of length 2, which could represent a parameter stratified by subpopulation.

### Adding Random Parameters

To draw parameters from distributions, use the `random.params` argument to `param.net`. There are two approaches.

#### Generator Functions

Define a generator function for each random parameter:

```{r generators}
my.randoms <- list(
  act.rate = param_random(c(0.25, 0.5, 0.75)),
  dummy.param = function() rbeta(1, 1, 2),
  dummy.strat.param = function() {
    c(rnorm(1, 0.05, 0.01),
      rnorm(1, 0.15, 0.03))
  }
)

param <- param.net(
  inf.prob = 0.3,
  random.params = my.randoms
)

param
```

The `my.randoms` list contains three elements:

- `act.rate` uses the `param_random` [function factory](https://adv-r.hadley.nz/function-factories.html) provided by EpiModel (see `?param_random`). Each simulation samples one of the three values with equal probability.
- `dummy.param` is a zero-argument function that returns a single draw from a Beta(1, 2) distribution.
- `dummy.strat.param` is a zero-argument function that returns a vector of length 2, each element drawn from a normal distribution with different mean and standard deviation.

Each element must be named after the parameter it fills and must be a function taking no arguments, returning a vector of the correct length for that parameter. When we print the parameter list before running the model, random parameters appear as function definitions since their values have not yet been realized.

```{r generators_run}
control <- control.net(type = "SI", nsims = 3, nsteps = 5, verbose = FALSE)
mod <- netsim(est, param, init, control)

mod
```

After running 3 simulations, `inf.prob` remains under "Fixed Parameters" while the random parameters each have 3 realized values (one per simulation). The vector parameter `dummy.strat.param` shows `<list>` because each realization is itself a vector of length 2.

To inspect the realized values:

``` {r generators_inspect}
all.params <- get_param_set(mod)
all.params
```

These can be merged with epidemic output for analysis of parameter-outcome relationships:

```{r}
epi <- as.data.frame(mod)
left_join(epi, all.params)
```


#### Parameter Sets

Generator functions draw each parameter independently. When parameters need to be correlated---for example, when using Latin hypercube sampling or when one parameter is derived from another---use pre-defined parameter sets instead.

Define a `data.frame` where each row is a complete set of correlated parameter values:

```{r set_df}
n <- 5

related.param <- data.frame(
  dummy.param = rbeta(n, 1, 2)
)

related.param$dummy.strat.param_1 <- related.param$dummy.param + rnorm(n)
related.param$dummy.strat.param_2 <- related.param$dummy.param * 2 + rnorm(n)

related.param
```

Each row contains parameter values that will be used together in a single simulation. Vector parameters use the same `paramname_N` suffix convention as scenarios. This means underscores are reserved and cannot appear in parameter names themselves.

Save the parameter set in the `my.randoms` list under the reserved name `param.random.set`:

```{r set_param}
my.randoms <- list(
  act.rate = param_random(c(0.25, 0.5, 0.75)),
  param.random.set = related.param
)

param <- param.net(
  inf.prob = 0.3,
  random.params = my.randoms
)

param
```

The `inf.prob` parameter remains fixed, `act.rate` remains independently random, and the two remaining parameters are drawn as correlated sets from the `data.frame`.

```{r set_run}
control <- control.net(type = "SI", nsims = 3, nsteps = 5, verbose = FALSE)
mod <- netsim(est, param, init, control)

mod
```

Verify that correlated sets are sampled together:

```{r}
related.param
get_param_set(mod)
```

## Time-Varying Parameters and Control Settings (Advanced)

The *scenarios* system described above uses an internal updater module to implement parameter changes at designated time steps. This section describes the underlying updater mechanism directly. Use this when the scenario API is not flexible enough---for example, when you need relative (function-based) parameter changes or time-varying *control* settings.

### Parameter Updaters

An updater is a `list` with two required elements: `at` (the time step for the change) and `param` (a named list of new parameter values):

```{r updater-example}
list(
  at = 10,
  param = list(
    inf.prob = 0.3,
    act.rate = 0.5
  )
)
```

This updater sets `inf.prob` to 0.3 and `act.rate` to 0.5 at time step 10. Multiple updaters are combined in a list:

```{r updater-list}
list.of.updaters <- list(
  list(
    at = 100,
    param = list(
      inf.prob = 0.3,
      act.rate = 0.3
    )
  ),
  list(
    at = 125,
    param = list(
      inf.prob = 0.01
    )
  )
)
```

### Incorporating Updaters

Pass the updater list to `param.net` via the `.param.updater.list` argument:

```{r updater-module}
param <- param.net(
 inf.prob = 0.1,
 act.rate = 0.1,
 .param.updater.list = list.of.updaters
)
init <- init.net(i.num = 10)
control <- control.net(
 type = "SI",
 nsims = 1,
 nsteps = 200,
 verbose = FALSE
)
```

```{r updater-module2}
nw <- network_initialize(n = 100)
est <- netest(
  nw,
  formation = ~edges,
  target.stats = 50,
  coef.diss = dissolution_coefs(~offset(edges), 10, 0),  # duration = 10, closed population
  verbose = FALSE
)
mod <- netsim(est, param, init, control)
```

The plot shows inflection points at the two parameter change points. At step 100, both `inf.prob` and `act.rate` increase from 0.1 to 0.3, accelerating transmission. At step 125, `inf.prob` drops to 0.01, slowing the epidemic.

```{r}
plot(mod, mean.smooth = FALSE)
abline(v = c(100, 125), lty = 2, col = "grey50")
```


#### Verbosity

Each updater can include an optional `verbose` element. When `TRUE` (the default), EpiModel prints a message describing the changes:

```{r updater-verbose}
list(
  at = 10,
  param = list(
    inf.prob = 0.3,
    act.rate = 0.5
  ),
  verbose = TRUE
)
```

#### Relative Parameter Changes

Instead of setting a parameter to a fixed new value, you can define a *function* that transforms the current value. This is useful for multiplicative changes or logit-scale adjustments.

```{r updater-function}
list(
  at = 10,
  param = list(
    inf.prob = function(x) plogis(qlogis(x) + log(2)),
    act.rate = 0.5
  )
)
```

In this updater, `act.rate` is set to the fixed value 0.5 as before. For `inf.prob`, instead of a value, we provide a function. At time step 10, EpiModel applies this function to the *current* value of `inf.prob`. If `inf.prob` was set to 0.1 by `param.net`, the function computes `plogis(qlogis(0.1) + log(2))` = `r round(plogis(qlogis(0.1) + log(2)), 4)`, which doubles the odds of transmission while keeping the result on the probability scale.

### Time-Varying Control Settings

Control settings can also be changed during a simulation using the same updater mechanism. Each control updater uses a `control` element (instead of `param`):

```{r control-updater-example}
list.of.updaters <- list(
  list(
    at = 100,
    control = list(
      resimulate.network = FALSE
    )
  ),
  list(
    at = 125,
    control = list(
      verbose = FALSE
    )
  )
)
```

This example turns off network resimulation at step 100 and disables model verbosity at step 125.

Pass control updaters to `control.net` via `.control.updater.list`, paralleling how parameter updaters are passed to `param.net` via `.param.updater.list`:

```{r}
control <- control.net(
 type = "SI",
 nsims = 1,
 nsteps = 200,
 verbose = TRUE,
 .control.updater.list = list.of.updaters
)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.helpers.R
\name{create_dat_object}
\alias{create_dat_object}
\title{Create a Minimal netsim_dat Main List Object for a Network Model}
\usage{
create_dat_object(
  param = list(),
  init = list(),
  control = list(),
  run = list()
)
}
\arguments{
\item{param}{An \code{EpiModel} object of class \code{\link[=param.net]{param.net()}}.}

\item{init}{An \code{EpiModel} object of class \code{\link[=init.net]{init.net()}}.}

\item{control}{An \code{EpiModel} object of class \code{\link[=control.net]{control.net()}}.}

\item{run}{A \code{list} that will contains the objects created by
\code{\link[=netsim]{netsim()}} that are required for between step communication. This
list must be preserved for restarting models.}
}
\value{
A \code{netsim_dat} main list object.
}
\description{
This helper function populates a \code{netsim_dat} main list
object with the minimal required elements. All parameters are
optional. When none are given the resulting object is only a
shell list of class \code{netsim_dat} with the different named
elements defined as empty lists.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icm.mod.status.R
\name{infection.icm.bip}
\alias{infection.icm.bip}
\title{Primary Infection Module for icm}
\usage{
infection.icm.bip(dat, at)
}
\arguments{
\item{dat}{Main \code{icm_dat} class data object passed through \code{icm}
simulations.}

\item{at}{Current time step.}
}
\value{
The updated \code{icm_dat} class main data object.
}
\description{
This function simulates the main infection process given the
current state of the actors in the system.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dcm.inputs.R
\name{control.dcm}
\alias{control.dcm}
\title{Control Settings for Deterministic Compartmental Models}
\usage{
control.dcm(
  type,
  nsteps,
  dt = 1,
  odemethod = "lsoda",
  dede = FALSE,
  new.mod = NULL,
  sens.param = TRUE,
  print.mod = FALSE,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{type}{Disease type to be modeled, with the choice of \code{"SI"} for
Susceptible-Infected diseases, \code{"SIR"} for
Susceptible-Infected-Recovered diseases, and \code{"SIS"} for
Susceptible-Infected-Susceptible diseases.}

\item{nsteps}{Number of time steps to solve the model over or vector of times
to solve the model over. If the number of time steps, then this must
be a positive integer of length 1.}

\item{dt}{Time unit for model solutions, with the default of 1. Model
solutions for fractional time steps may be obtained by setting this to
a number between 0 and 1.}

\item{odemethod}{Ordinary differential equation (ODE) integration method,
with the default of \code{"lsoda"} (see \code{\link[deSolve:ode]{deSolve::ode}} for other
options).}

\item{dede}{If \code{TRUE}, use the delayed differential equation solver,
which allows for time-lagged variables.}

\item{new.mod}{If not running a base model type, a function with a new
model to be simulated (see details).}

\item{sens.param}{If \code{TRUE}, evaluate arguments in parameters or initial
conditions with length greater than 1 as sensitivity analyses, with
one model run per value. If \code{FALSE}, one model will be run with
parameters and initial conditions of arbitrary length (the model may
error unless the model function is designed to accommodate vectors).}

\item{print.mod}{If \code{TRUE}, print the model form to the console.}

\item{verbose}{If \code{TRUE}, print model progress to the console.}

\item{...}{additional control settings passed to model.}
}
\value{
An \code{EpiModel} object of class \code{control.dcm}.
}
\description{
Sets the controls for deterministic compartmental models
simulated with \code{\link[=dcm]{dcm()}}.
}
\details{
\code{control.dcm} sets the required control settings for any deterministic
compartmental models solved with the \code{\link[=dcm]{dcm()}} function. Controls are
required for both base model types and original models. For all base models,
the \code{type} argument is a necessary parameter and it has no default.
}
\section{ODE Solver}{

As of EpiModel 2.5.5, the default ODE solver was changed from \code{"rk4"}
(fixed-step Runge-Kutta 4) to \code{"lsoda"} (adaptive step-size). The \code{"lsoda"}
method automatically adjusts its internal step size, providing numerical
stability across a wide range of parameter values. The fixed-step \code{"rk4"}
solver could produce numerical instability (negative compartment sizes, NaN
values) for models with high transmission rates or stiff dynamics. Users who
require the previous behavior can set \code{odemethod = "rk4"} explicitly.
}

\section{New Model Functions}{

The form of the model function for base models may be displayed with the
\code{print.mod} argument set to \code{TRUE}. In this case, the model will
not be run. These model forms may be used as templates to write original
model functions.

These new models may be input and solved with \code{\link[=dcm]{dcm()}} using the
\code{new.mod} argument, which requires as input a model function.
}

\seealso{
Use \code{\link[=param.dcm]{param.dcm()}} to specify model parameters and
\code{\link[=init.dcm]{init.dcm()}} to specify the initial conditions. Run the
parameterized model with \code{\link[=dcm]{dcm()}}.
}
\keyword{parameterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveout.R
\name{saveout.icm}
\alias{saveout.icm}
\title{Save icm Data to Output List Format}
\usage{
saveout.icm(dat, s, out = NULL)
}
\arguments{
\item{dat}{Main \code{icm_dat} class data object passed through \code{icm}
simulations.}

\item{s}{Current simulation number.}

\item{out}{Out list passed back in for updating at simulations 2+.}
}
\value{
A list with the following elements:
\itemize{
\item \strong{param:} the epidemic parameters passed into the model through
\code{\link[=param.icm]{param.icm()}}, with additional parameters added as
necessary.
\item \strong{control:} the control settings passed into the model through
\code{\link[=control.icm]{control.icm()}}, with additional controls added as
necessary.
\item \strong{epi:} a list of data frames, one for each epidemiological
output from the model.
}
}
\description{
This function transfers the data from the main \code{icm_dat}
class data object to the output \code{out} object at the end of
each simulation in \code{\link[=icm]{icm()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{trim_netest}
\alias{trim_netest}
\title{Function to Reduce the Size of a \code{netest} Object}
\usage{
trim_netest(
  object,
  as.networkLite = TRUE,
  keep.fit = FALSE,
  keep = character(0)
)
}
\arguments{
\item{object}{A \code{netest} class object.}

\item{as.networkLite}{If \code{TRUE}, converts \code{object$newnetwork}
to a \code{networkLite}.}

\item{keep.fit}{If \code{FALSE}, removes the \code{object$fit} (if present)
on the \code{netest} object.}

\item{keep}{Character vector of object names to keep in formula environments.
By default, all objects are removed.}
}
\value{
A \code{netest} object with formula environments trimmed, optionally with the
\code{newnetwork} element converted to a \code{networkLite} and the
\code{fit} element removed.
}
\description{
Trims formula environments from the \code{netest} object.
Optionally converts the \code{newnetwork} element of the
\code{netest} object to a \code{networkLite} class, and removes
the \code{fit} element (if present) from the \code{netest}
object.
}
\details{
With larger, more complex network structures with epidemic models, it is
generally useful to reduce the memory footprint of the fitted TERGM model
object (estimated with \code{\link[=netest]{netest()}}). This utility function removes
all but the bare essentials needed for simulating a network model with
\code{\link[=netsim]{netsim()}}.

The function always trims the environments of \code{object$constraints} and
\code{object$coef.diss$dissolution}.

When both \code{edapprox = TRUE} and \code{nested.edapprox = TRUE} in the
\code{netest} call, also trims the environments of \code{object$formula}
and \code{object$formation}.

When both \code{edapprox = TRUE} and \code{nested.edapprox = FALSE} in the
\code{netest} call, also trims the environments of \code{object$formula},
\code{environment(object$formation)$formation}, and
\code{environment(object$formation)$dissolution}.

When \code{edapprox = FALSE} in the \code{netest} call, also trims the
environments of \code{object$formation},
\code{environment(object$formula)$formation} and
\code{environment(object$formula)$dissolution}.

By default all objects are removed from these trimmed environments. Specific
objects may be retained by passing their names as the \code{keep} argument.
For the output of \code{trim_netest} to be usable in \code{\link[=netsim]{netsim()}}
simulation, any objects referenced in the formulas should be included in the
\code{keep} argument.

If \code{as.networkLite = TRUE}, converts \code{object$newnetwork} to a
\code{networkLite} object. If \code{keep.fit = FALSE}, removes \code{fit} (if
present) from \code{object}.
}
\examples{
\donttest{
nw <- network_initialize(n = 100)
formation <- ~edges + concurrent
target.stats <- c(50, 25)
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)
est <- netest(nw, formation, target.stats, coef.diss,
              set.control.ergm = control.ergm(MCMC.burnin = 1e5,
                                              MCMC.interval = 1000))
print(object.size(est), units = "KB")

est.small <- trim_netest(est)
print(object.size(est.small), units = "KB")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgelists.R
\name{get_edgelist}
\alias{get_edgelist}
\title{Get an Edgelist From the Specified Network}
\usage{
get_edgelist(dat, network)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{network}{Numerical index of the network from which the edgelist should
be extracted. (May be > 1 for models with multiple overlapping
networks.)}
}
\value{
An edgelist in matrix form with two columns. Each column contains the
posit_ids (see \code{get_posit_ids}) of the nodes in each edge.
}
\description{
This function outputs an edgelist from the specified network,
selecting the method depending on the stored network type.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.R
\name{as.epi.data.frame}
\alias{as.epi.data.frame}
\title{Validate and Convert to epi.data.frame}
\usage{
as.epi.data.frame(df)
}
\arguments{
\item{df}{A \code{data.frame} to convert into an \code{epi.data.frame}}
}
\description{
This methods ensures that the \code{data.frame} is correctly formatted as an
\code{epi.data.frame}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendo.R
\name{as.phylo.transmat}
\alias{as.phylo.transmat}
\title{Convert transmat Infection Tree into a phylo Object}
\usage{
\method{as.phylo}{transmat}(x, vertex.exit.times = NULL, ...)
}
\arguments{
\item{x}{An object of class \code{transmat}, the output from
\code{\link[=get_transmat]{get_transmat()}}.}

\item{vertex.exit.times}{Optional numeric vector providing the time of
departure of vertices, to be used to scale the lengths of branches
reaching to the tips. Index position on vector corresponds to network
id. NA indicates no departure, so branch will extend to the end of the
tree.}

\item{...}{Further arguments (unused).}
}
\value{
A \code{phylo} class object.
}
\description{
Converts a transmission matrix from the \code{get_transmat}
function into a \code{phylo} class object.
}
\details{
Converts a \code{\link[=transmat]{transmat()}} object containing information about the
history of a simulated infection into a \code{\link[ape:phylo]{ape::phylo}} object
representation suitable for plotting as a tree with
\code{\link[ape:plot.phylo]{ape::plot.phylo()}}. Each infection event becomes a 'node'
(horizontal branch) in the resulting \code{phylo} tree, and each network
vertex becomes a 'tip' of the tree. The infection events are labeled with the
vertex ID of the infector to make it possible to trace the path of infection.

The infection timing information is included to position the phylo-nodes,
with the lines to the tips drawn to the max time value +1 (unless
\code{vertex.exit.times} are passed in it effectively assumes all vertices
are active until the end of the simulation).

If the \code{transmat} contains multiple infection seeds (there are multiple
trees with separate root nodes), this function will return a list of class
\code{multiPhylo}, each element of which is a \code{phylo} object. See
\code{\link[ape:read.tree]{ape::read.tree()}}.
}
\examples{
\donttest{
set.seed(13)

# Fit a random mixing TERGM with mean degree of 1 and mean edge
# duration of 20 time steps
nw <- network_initialize(n = 100)
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

# Parameterize the epidemic model as SI with one infected seed
param <- param.net(inf.prob = 0.5)
init <- init.net(i.num = 1)
control <- control.net(type = "SI", nsteps = 40, nsims = 1, verbose = FALSE)

# Simulate the model
mod1 <- netsim(est, param, init, control)

# Extract the transmission matrix
tm <- get_transmat(mod1)
head(tm, 15)

# Convert to phylo object and plot
tmPhylo <- as.phylo.transmat(tm)
par(mar = c(1,1,1,1))
plot(tmPhylo, show.node.label = TRUE,
              root.edge = TRUE,
              cex = 0.75)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.accessor.R
\name{update_unique_ids}
\alias{update_unique_ids}
\title{Create the Unique Identifiers for New Nodes}
\usage{
update_unique_ids(dat, n.new)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{n.new}{The number of new nodes to give \code{unique_ids} to.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function is called by \code{\link[=append_core_attr]{append_core_attr()}} and
appends new \code{unique_ids} to the created nodes. It also keeps track of
the already used \code{unique_ids} with the \code{dat$run$last_unique_id}
variable.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.icm.R
\name{plot.icm}
\alias{plot.icm}
\title{Plot Data from a Stochastic Individual Contact Epidemic Model}
\usage{
\method{plot}{icm}(
  x,
  y = NULL,
  popfrac = FALSE,
  sim.lines = FALSE,
  sims = NULL,
  sim.col = NULL,
  sim.lwd = NULL,
  sim.alpha = NULL,
  mean.line = TRUE,
  mean.smooth = TRUE,
  mean.col = NULL,
  mean.lwd = 2,
  mean.lty = 1,
  qnts = 0.5,
  qnts.col = NULL,
  qnts.alpha = 0.5,
  qnts.smooth = TRUE,
  legend = NULL,
  leg.cex = 0.8,
  grid = FALSE,
  add = FALSE,
  xlim = NULL,
  ylim = NULL,
  main = "",
  xlab = "Time",
  ylab = NULL,
  ...
)
}
\arguments{
\item{x}{An \code{EpiModel} model object of class \code{icm}.}

\item{y}{Output compartments or flows from \code{icm} object to plot. -------}

\item{popfrac}{If \code{TRUE}, plot prevalence of values rather than numbers
(see details).}

\item{sim.lines}{If \code{TRUE}, plot individual simulation lines. Default is
to plot lines for one-group models but not for two-group models.}

\item{sims}{A vector of simulation numbers to plot.}

\item{sim.col}{Vector of any standard R color format for simulation lines.}

\item{sim.lwd}{Line width for simulation lines.}

\item{sim.alpha}{Transparency level for simulation lines, where
0 = transparent and 1 = opaque (see \code{adjustcolor} function).}

\item{mean.line}{If \code{TRUE}, plot mean of simulations across time.}

\item{mean.smooth}{If \code{TRUE}, use a loess smoother on the mean line.}

\item{mean.col}{Vector of any standard R color format for mean lines.}

\item{mean.lwd}{Line width for mean lines.}

\item{mean.lty}{Line type for mean lines.}

\item{qnts}{If numeric, plot polygon of simulation quantiles based on the
range implied by the argument (see details). If \code{FALSE}, suppress
polygon from plot.}

\item{qnts.col}{Vector of any standard R color format for polygons.}

\item{qnts.alpha}{Transparency level for quantile polygons, where 0 =
transparent and 1 = opaque (see \code{adjustcolor} function).}

\item{qnts.smooth}{If \code{TRUE}, use a loess smoother on quantile polygons.}

\item{legend}{If \code{TRUE}, plot default legend.}

\item{leg.cex}{Legend scale size.}

\item{grid}{If \code{TRUE}, a grid is added to the background of plot
(see \code{\link[=grid]{grid()}} for details), with default of nx by ny.}

\item{add}{If \code{TRUE}, new plot window is not called and lines are added
to existing plot window.}

\item{xlim}{the x limits (x1, x2) of the plot.  Note that \code{x1 > x2}
    is allowed and leads to a \sQuote{reversed axis}.

    The default value, \code{NULL}, indicates that the range of the
    \link{finite} values to be plotted should be used.}

\item{ylim}{the y limits of the plot.}

\item{main}{a main title for the plot, see also \code{\link[graphics]{title}}.}

\item{xlab}{a label for the x axis, defaults to a description of \code{x}.}

\item{ylab}{a label for the y axis, defaults to a description of \code{y}.}

\item{...}{Additional arguments to pass.}
}
\description{
Plots epidemiological data from a stochastic individual contact
model simulated with \code{\link[=icm]{icm()}}.
}
\details{
This plotting function will extract the epidemiological output from a model
object of class \code{icm} and plot the time series data of disease
prevalence and other results. The summary statistics that the function
calculates and plots are individual simulation lines, means of the individual
simulation lines, and quantiles of those individual simulation lines. The
mean line, toggled on with \code{mean.line=TRUE}, is calculated as the row
mean across simulations at each time step.

Compartment prevalences are the size of a compartment over some denominator.
To plot the raw numbers from any compartment, use \code{popfrac=FALSE}; this
is the default for any plots of flows. The \code{popfrac} parameter
calculates and plots the denominators of all specified compartments using
these rules: 1) for one-group models, the prevalence of any compartment is
the compartment size divided by the total population size; 2) for two-group
models, the prevalence of any compartment is the compartment size divided by
the group population size. For any prevalences that are not automatically
calculated, the \code{\link[=mutate_epi]{mutate_epi()}} function may be used to add new
variables to the \code{icm} object to plot or analyze.

The quantiles show the range of outcome values within a certain specified
quantile range. By default, the interquartile range is shown: that is the
middle 50\\% of the data. This is specified by \code{qnts=0.5}. To show the
middle 95\\% of the data, specify \code{qnts=0.95}. To toggle off the polygons
where they are plotted by default, specify \code{qnts=FALSE}.
}
\examples{
\donttest{
## Example 1: Plotting multiple compartment values from SIR model
param <- param.icm(inf.prob = 0.5, act.rate = 0.5, rec.rate = 0.02)
init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
control <- control.icm(type = "SIR", nsteps = 100,
                       nsims = 3, verbose = FALSE)
mod <- icm(param, init, control)
plot(mod, grid = TRUE)

## Example 2: Plot only infected with specific output from SI model
param <- param.icm(inf.prob = 0.25, act.rate = 0.25)
init <- init.icm(s.num = 500, i.num = 10)
control <- control.icm(type = "SI", nsteps = 100,
                       nsims = 3, verbose = FALSE)
mod2 <- icm(param, init, control)

# Plot prevalence
plot(mod2, y = "i.num", mean.line = FALSE, sim.lines = TRUE)

# Plot incidence
par(mfrow = c(1, 2))
plot(mod2, y = "si.flow", mean.smooth = TRUE, grid = TRUE)
plot(mod2, y = "si.flow", qnts.smooth = FALSE, qnts = 1)
}

}
\seealso{
\code{\link[=icm]{icm()}}
}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icm.mod.init.R
\name{init_status.icm}
\alias{init_status.icm}
\title{Disease Status Initialization Module for icm}
\usage{
init_status.icm(dat)
}
\arguments{
\item{dat}{Main \code{icm_dat} class data object passed through \code{icm}
simulations.}
}
\value{
The updated \code{icm_dat} class main data object.
}
\description{
This function sets the initial disease status on the
network given the specified initial conditions.
}
\seealso{
This is an initialization module for \code{\link[=icm]{icm()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.R
\name{get_network}
\alias{get_network}
\alias{get_network.netdx}
\alias{get_network.netsim}
\alias{get_network.netsim_dat}
\title{Extract Network Objects from Network Simulations}
\usage{
get_network(x, ...)

\method{get_network}{netdx}(x, sim = 1, collapse = FALSE, at = NULL, ...)

\method{get_network}{netsim}(x, sim = 1, network = 1, collapse = FALSE, at = NULL, ...)

\method{get_network}{netsim_dat}(x, network = 1L, ...)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{\link[=netsim]{netsim()}},
\code{\link[=netdx]{netdx()}}, or \code{netsim_dat}.}

\item{...}{Additional arguments.}

\item{sim}{Simulation number of extracted network, for \code{netdx} and
\code{netsim}.}

\item{collapse}{If \code{TRUE}, collapse the \code{networkDynamic} object to
a static \code{network} object at a specified time step. Applicable to
\code{netdx} objects and \code{netsim} objects with
\code{tergmLite == FALSE}.}

\item{at}{If \code{collapse} is \code{TRUE}, the time step at which the
extracted network should be collapsed. Applicable to \code{netdx}
objects and \code{netsim} objects with \code{tergmLite == FALSE}.}

\item{network}{Network number, for \code{netsim} or \code{netsim_dat} objects
with multiple overlapping networks (advanced use, and not applicable
to \code{netdx} objects).}
}
\value{
For \code{netdx} or \code{netsim} with \code{tergmLite == FALSE}, a
\code{networkDynamic} object (if \code{collapse = FALSE}) or a
static \code{network} object (if \code{collapse = TRUE}). For
\code{netsim} with \code{tergmLite == TRUE} or \code{netsim_dat} with
\code{tergmLite == TRUE}, a \code{networkLite} object. For
\code{netsim_dat} with \code{tergmLite == FALSE}, a
\code{networkDynamic} object.
}
\description{
Extracts the network object from either a network epidemic model
object generated with \code{netsim}, a network diagnostic
simulation generated with \code{netdx}, or a \code{netsim_dat}
object used internally in \code{netsim}. For \code{netdx} or
\code{netsim} with \code{tergmLite == FALSE}, the extracted
network object is a \code{networkDynamic}, which can be
collapsed down to a static \code{network} object with the
\code{collapse} and \code{at} arguments. For \code{netsim} with
\code{tergmLite == TRUE}, the extracted network object is the
final \code{networkLite}, the \code{collapse} argument should be
\code{FALSE}, and the \code{at} argument should be missing. For
\code{netsim_dat}, the \code{collapse} and \code{at} arguments
are not supported, and the network object is either the current
\code{networkLite} (if \code{tergmLite == TRUE}) or the current
\code{networkDynamic} (if \code{tergmLite == FALSE}).
}
\details{
This function requires that the network object is saved during the network
simulation while running either \code{\link[=netsim]{netsim()}} or \code{\link[=netdx]{netdx()}}.
For the former, that is specified by setting the \code{save.network}
parameter in \code{\link[=control.net]{control.net()}} to \code{TRUE}. For the latter, that
is specified with the \code{keep.tnetwork} parameter directly in
\code{\link[=netdx]{netdx()}}.
}
\examples{
\donttest{
# Set up network and TERGM formula
nw <- network_initialize(n = 100)
nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)

# Estimate the model
est <- netest(nw, formation, target.stats, coef.diss)

# Run diagnostics, saving the networkDynamic objects
dx <- netdx(est, nsteps = 10, nsims = 3, keep.tnetwork = TRUE,
            verbose = FALSE)

# Extract the network for simulation 2 from dx object
get_network(dx, sim = 2)

# Extract and collapse the network from simulation 1 at time step 5
get_network(dx, collapse = TRUE, at = 5)

# Parameterize the epidemic model, and simulate it
param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
init <- init.net(i.num = 10, i.num.g2 = 10)
control <- control.net(type = "SI", nsteps = 10, nsims = 3, verbose = FALSE)
mod <- netsim(est, param, init, control)

# Extract the network for simulation 2 from mod object
get_network(mod, sim = 2)

## Extract and collapse the network from simulation 1 at time step 5
get_network(mod, collapse = TRUE, at = 5)
}

}
\keyword{extract}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.infection.R
\name{set_transmat}
\alias{set_transmat}
\title{Save Transmission Matrix}
\usage{
set_transmat(dat, del, at)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{del}{Discordant edgelist created within \code{\link[=infection.net]{infection.net()}} and
\code{\link[=infection.2g.net]{infection.2g.net()}}.}

\item{at}{Current time step.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function appends the transmission matrix created during
\code{infection.net} and \code{infection.2g.net}.
}
\details{
This internal function works within the parent \code{\link[=infection.net]{infection.net()}}
functions to save the transmission matrix created at time step \code{at} to
the main \code{netsim_dat} class object \code{dat}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reachable.R
\name{dedup_cumulative_edgelist}
\alias{dedup_cumulative_edgelist}
\title{Deduplicate a cumulative edgelist by combining overlapping edges}
\usage{
dedup_cumulative_edgelist(el)
}
\arguments{
\item{el}{A cumulative edgelist with potentially overlapping edges}
}
\value{
A cumulative edgelist with no overlapping edges
}
\description{
Deduplicate a cumulative edgelist by combining overlapping edges
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbose.R
\name{verbose.icm}
\alias{verbose.icm}
\title{Progress Print Module for Stochastic Individual Contact Models}
\usage{
verbose.icm(x, type, s = 1, at = 2)
}
\arguments{
\item{x}{If the \code{type} is "startup", then an object of class
\code{control.icm}; otherwise, an object of class \code{icm_dat}, the
main data object in \code{icm} simulations.}

\item{type}{Progress type, either of "startup" for starting messages before
all simulations, or "progress" for time step specific messages.}

\item{s}{Current simulation number, if type is "progress".}

\item{at}{Current time step, if type is "progress".}
}
\description{
This function prints progress from stochastic individual contact
models simulated with \code{\link[=icm]{icm()}} to the console.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveout.R
\name{saveout.net}
\alias{saveout.net}
\title{Save netsim Data to Output List Format}
\usage{
saveout.net(dat, s, out = NULL)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{s}{Current simulation number.}

\item{out}{Out list passed back in for updating at simulations 2+.}
}
\value{
A list with the following elements:
\itemize{
\item \strong{param:} the epidemic parameters passed into the model through
\code{\link[=param.net]{param.net()}}, with additional parameters added as
necessary.
\item \strong{control:} the control settings passed into the model through
\code{\link[=control.net]{control.net()}}, with additional controls added as
necessary.
\item \strong{epi:} a list of data frames, one for each epidemiological
output from the model.
\item \strong{stats:} a list containing two sublists, \code{nwstats} for any
network statistics saved in the simulation, and \code{transmat} for
the transmission matrix saved in the simulation.
\item \strong{network:} a list of \code{networkDynamic} objects,
one for each model simulation.
}
}
\description{
This function transfers the data from the main \code{netsim_dat}
object to the output \code{out} object at the end of each
simulation in \code{\link[=netsim]{netsim()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.scenarios.R
\name{use_scenario}
\alias{use_scenario}
\title{Apply a scenario object to a param.net object}
\usage{
use_scenario(param, scenario)
}
\arguments{
\item{param}{Object of class \code{param.net}, output from function of same
name.}

\item{scenario}{a scenario object usually created from a \code{data.frame} of
scenarios using the \code{create_scenario_list} function. See the vignette
"network-model-scenarios".}
}
\value{
An updated list object of class \code{param.net}, which can be passed to the
EpiModel function \code{\link[=netsim]{netsim()}}.
}
\description{
Apply a scenario object to a param.net object
}
\section{scenario}{

A scenario is a list containing an "id" field, the name of the scenario and
a ".param.updater.list" containing a list of updaters that modifies the
parameters of the model at given time steps. If a scenario contains a
parameter not defined in the \code{param} object, an error will be produced.
See the vignette "model-parameters" for the technical detail of their
implementation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{get_formula_term_attr}
\alias{get_formula_term_attr}
\title{Output ERGM Formula Attributes into a Character Vector}
\usage{
get_formula_term_attr(form, nw)
}
\arguments{
\item{form}{An ERGM model formula.}

\item{nw}{A network object.}
}
\value{
A character vector of vertex attributes.
}
\description{
Given a formation formula for a network model, outputs a
character vector of vertex attributes to be used in
\code{\link[=netsim]{netsim()}} simulations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ErgmTerms.R
\name{InitErgmTerm.absdiffby}
\alias{InitErgmTerm.absdiffby}
\alias{absdiffby}
\title{Definition for absdiffby ERGM Term}
\usage{
InitErgmTerm.absdiffby(nw, arglist, ...)
}
\arguments{
\item{nw}{An object of class \code{network}.}

\item{arglist}{A list of arguments as specified in the \code{ergm.userterms}
package framework.}

\item{...}{Additional data passed into the function as specified in the
\code{ergm.userterms} package framework.}
}
\description{
This function defines and initializes the absdiffby ERGM term
that allows for representing homophily with respect to a
non-binary attribute (e.g., age) differentially by a binary
attribute (e.g., sex).
}
\details{
This ERGM user term was written to allow for age-based homophily in
partnership formation that is asymmetric by sex. The \code{absdiff} component
targets age-based homophily while the \code{by} component allows that to be
structured by a binary attribute such as "male", in order to enforce an
offset in the average difference. This allows, for example, a average age
difference in partnerships, but with males (on average) older than females.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.helpers.R
\name{set_current_timestep}
\alias{set_current_timestep}
\title{Set the Current Timestep}
\usage{
set_current_timestep(dat, timestep)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{timestep}{The new value for the timestep.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Changes the current timestep in the \code{netsim_dat} object.
Use with caution. This function exists to work around unforeseen
corner cases. In most situation, \code{increment_timestep} is
preferred.
}
\section{Mutability}{

This DOES NOT modify the \code{netsim_dat} object in place. The result must
be assigned back to \code{dat} in order to be registered:
\code{dat <- increment_timestep(dat)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.inputs.R
\name{crosscheck.net}
\alias{crosscheck.net}
\title{Cross Checking of Inputs for Stochastic Network Models}
\usage{
crosscheck.net(x, param, init, control)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{\link[=netest]{netest()}}.}

\item{param}{An \code{EpiModel} object of class \code{\link[=param.net]{param.net()}}.}

\item{init}{An \code{EpiModel} object of class \code{\link[=init.net]{init.net()}}.}

\item{control}{An \code{EpiModel} object of class \code{\link[=control.net]{control.net()}}.}
}
\value{
This function returns no objects.
}
\description{
This function checks that the estimation object from
\code{\link[=netest]{netest()}} and the three parameter lists from
\code{\link[=param.net]{param.net()}}, \code{\link[=init.net]{init.net()}}, and
\code{\link[=control.net]{control.net()}} are consistent.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netdx.R
\name{tedgelist_to_toggles}
\alias{tedgelist_to_toggles}
\title{Convert Timed Edgelist to Matrix of Toggles}
\usage{
tedgelist_to_toggles(tedgelist)
}
\arguments{
\item{tedgelist}{A timed edgelist, as produced by
\code{\link[networkDynamic:as.data.frame.networkDynamic]{networkDynamic::as.data.frame.networkDynamic}}.}
}
\value{
The matrix of toggles corresponding to \code{tedgelist}.
}
\description{
Convert Timed Edgelist to Matrix of Toggles
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.record.R
\name{record_raw_object}
\alias{record_raw_object}
\title{Record an Arbitrary Object During a Simulation}
\usage{
record_raw_object(dat, at, label, object)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{The time where the recording happens.}

\item{label}{The name to give to the recorded object.}

\item{object}{The object to be recorded.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function records any object during a simulation to allow its
inspection afterward. The records are stored in \code{dat[["raw.records"]]}
during the simulation, where \code{dat} is the main \code{netsim_dat} class
object, and in the \code{netsim} object under the \code{raw.records}
\code{collections::queue} object.
}
\details{
See the "Time-Varying Parameters" section of the "Working With Model
Parameters" vignette.
}
\examples{
\dontrun{

dat <- record_raw_object(dat, at, "a.df", data.frame(x = 2:200))
dat <- record_raw_object(dat, at, "a.message", "I recorded something")

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.netdx}
\alias{print.netdx}
\title{Utility Function for Printing netdx Object}
\usage{
\method{print}{netdx}(x, digits = 3, ...)
}
\arguments{
\item{x}{an object of class \code{netdx}}

\item{digits}{number of digits to print in statistics tables}

\item{...}{additional arguments (currently ignored)}
}
\description{
Prints basic information and statistics from a \code{netdx}
object.
}
\details{
Given a \code{netdx} object, \code{print.netdx} prints the diagnostic method
(static/dynamic), number of simulations, and (if dynamic) the number of time
steps per simulation used in generating the \code{netdx} object, as well as
printing the formation statistics table and (if present) the duration and
dissolution statistics tables.  The statistics tables are interpreted as
follows.

Each row has the name of a particular network statistic.  In the formation
table, these correspond to actual network statistics in the obvious way.
In the duration and dissolution tables, these correspond to dissolution
model dyad types: in a homogeneous dissolution model, all dyads are of the
\code{edges} type; in a heterogeneous dissolution model, a dyad with a
nonzero \code{nodematch} or \code{nodemix} change statistic in the
dissolution model has type equal to that statistic, and has type equal to
\code{edges} otherwise.  The statistics of interest for the duration and
dissolution tables are, respectively, the mean age of extant edges and the
edge dissolution rate, broken down by dissolution model dyad type.  (The
current convention is to treat the mean age and dissolution rate for a
particular dissolution dyad type as 0 on time steps with no edges of that
type; this behavior may be changed in the future.)

The columns are named \code{Target}, \verb{Sim Mean}, \verb{Pct Diff},
\verb{Sim SE}, \verb{Z Score}, \verb{SD(Sim Means)}, and
\code{SD(Statistic)}.  The \verb{Sim Mean} column refers to the mean
statistic value, across all time steps in all simulations in the dynamic
case, and across all sampled networks in all simulations in the static case.
The \verb{Sim SE} column refers to the standard error in the mean, estimated
using \code{\link[coda:effectiveSize]{coda::effectiveSize}}.  The \code{Target}
column indicates the target value (if present) for the statistic, and the
\verb{Pct Diff} column gives \verb{(Sim Mean - Target)/Target} when
\code{Target} is present.  The \verb{Z Score} column gives
\verb{(Sim Mean - Target)/(Sim SE)}.  The \verb{SD(Sim Means)} column gives
the empirical standard deviation across simulations of the mean statistic
value within simulation, and \code{SD(Statistic)} gives the empirical
standard deviation of the statistic value across all the simulated data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{edgelist_censor}
\alias{edgelist_censor}
\title{Table of Edge Censoring}
\usage{
edgelist_censor(el)
}
\arguments{
\item{el}{A timed edgelist with start and end times extracted from a
\code{networkDynamic} object using the
\code{as.data.frame.networkDynamic} function.}
}
\value{
A 4 x 2 table containing the number and percent of edges in \code{el}
that are left-censored, right-censored, both-censored, or uncensored.
}
\description{
Outputs a table of the number and percent of edges that are
left-censored, right-censored, both-censored, or uncensored for
a \code{networkDynamic} object.
}
\details{
Given a STERGM simulation over a specified number of time steps, the edges
within that simulation may be left-censored (started before the first step),
right-censored (continued after the last step), right and left-censored, or
uncensored. The amount of censoring will increase when the average edge
duration approaches the length of the simulation.
}
\examples{
\donttest{
# Initialize and parameterize network model
nw <- network_initialize(n = 100)
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)

# Model estimation
est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

# Simulate the network and extract a timed edgelist
dx <- netdx(est, nsims = 1, nsteps = 100, keep.tedgelist = TRUE,
      verbose = FALSE)
el <- as.data.frame(dx)

# Calculate censoring
edgelist_censor(el)
}

}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netdx.R
\name{make_stats_table}
\alias{make_stats_table}
\title{Create a Summary Table of Simulation Statistics}
\usage{
make_stats_table(stats, targets)
}
\arguments{
\item{stats}{A list of simulated statistics matrices, of length equal to the
number of simulations performed.  Each matrix should have one row for each
simulated network if \code{dynamic == FALSE}, one row for each time step
if \code{dynamic == TRUE}, and one column for each statistic.  The columns
should be named for the statistics they correspond to, with all matrices
having the same statistics, in the same order.  Each matrix may have an
\code{attr}-style attribute named \code{"ess"} attached, giving the
effective sample sizes for the columns of the matrix; if this attribute is
\code{NULL}, then the effective sample sizes will be computed within the
call to \code{make_stats_table}.}

\item{targets}{A vector of target values for the statistics in \code{stats}.
May be named (in which case targets will be matched to statistics based on
column names in matrices in \code{stats}) or unnamed (in which case
targets will be matched to statistics based on position, and the number of
targets must equal the number of columns).}
}
\value{
A \code{data.frame} summarizing the simulated statistics.
}
\description{
Create a Summary Table of Simulation Statistics
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.R
\name{get_nwstats}
\alias{get_nwstats}
\title{Extract Network Statistics from netsim or netdx Object}
\usage{
get_nwstats(x, sim = NULL, network = 1, mode = c("data.frame", "list"))
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{\link[=netsim]{netsim()}} or
\code{\link[=netdx]{netdx()}}.}

\item{sim}{A vector of simulation numbers from the extracted object.
(Default = NULL, all simulations are included)}

\item{network}{Network number, for \code{netsim} objects with multiple
overlapping networks (advanced use, and not applicable to \code{netdx}
objects).}

\item{mode}{Either \code{"data.frame"} or \code{"list"}, indicating the
desired output.}
}
\value{
A data frame or list of matrices containing the network statistics.
}
\description{
Extracts network statistics from a network epidemic model
simulated with \code{netsim} or a network diagnostics object
simulated with \code{netdx}. Statistics can be returned either
as a single data frame or as a list of matrices (one matrix
for each simulation).
}
\examples{
\donttest{
# Two-group Bernoulli random graph TERGM
nw <- network_initialize(n = 100)
nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

dx <- netdx(est, nsim = 3, nsteps = 10, verbose = FALSE,
            nwstats.formula = ~edges + isolates)
get_nwstats(dx)
get_nwstats(dx, sim = 1)

# SI epidemic model
param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
init <- init.net(i.num = 10, i.num.g2 = 10)
control <- control.net(type = "SI", nsteps = 10, nsims = 3,
                       nwstats.formula = ~edges + meandeg + degree(0:5),
                       verbose = FALSE)
mod <- netsim(est, param, init, control)

# Extract the network statistics from all or sets of simulations
get_nwstats(mod)
get_nwstats(mod, sim = 2)
get_nwstats(mod, sim = c(1, 3))

# On the fly summary stats
summary(get_nwstats(mod))
colMeans(get_nwstats(mod))
}

}
\keyword{extract}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.helpers.R
\name{increment_timestep}
\alias{increment_timestep}
\title{Increment the Current Timestep}
\usage{
increment_timestep(dat)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function adds 1 to the timestep counter stored in the
\code{netsim_dat} main list object.
}
\section{Mutability}{

This DOES NOT modify the \code{netsim_dat} object in place. The result must
be assigned back to \code{dat} in order to be registered:
\code{dat <- increment_timestep(dat)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.summary.R
\name{summary_nets}
\alias{summary_nets}
\title{Extract Summary Statistics of Networks Used in netsim}
\usage{
summary_nets(dat, at)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{Current time step.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function calls \code{summary} on each network being
simulated in \code{netsim}, provided \code{save.nwstats} and
\code{resimulate.network} are both \code{TRUE}. It records the
statistics represented by \code{nwstats.formula} in
\code{dat$stats$nwstats}, where \code{dat} is the main
\code{netsim_dat} class object.
}
\keyword{internal}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icm.mod.vital.R
\name{arrivals.icm}
\alias{arrivals.icm}
\title{Arrivals: icm Module}
\usage{
arrivals.icm(dat, at)
}
\arguments{
\item{dat}{Main \code{icm_dat} class data object passed through \code{icm}
simulations.}

\item{at}{Current time step.}
}
\value{
The updated \code{icm_dat} class main data object.
}
\description{
This function simulates arrival for use in \code{\link[=icm]{icm()}}
simulations.
}
\seealso{
\code{\link[=icm]{icm()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ErgmTerms.R
\name{InitErgmTerm.fuzzynodematch}
\alias{InitErgmTerm.fuzzynodematch}
\alias{fuzzynodematch}
\title{Definition for fuzzynodematch ERGM Term}
\usage{
InitErgmTerm.fuzzynodematch(nw, arglist, ...)
}
\arguments{
\item{nw}{An object of class \code{network}.}

\item{arglist}{A list of arguments as specified in the \code{ergm.userterms}
package framework.}

\item{...}{Additional data passed into the function as specified in the
\code{ergm.userterms} package framework.}
}
\description{
This function defines and initializes the fuzzynodematch ERGM
term that allows for generalized homophily.
}
\details{
This ERGM user term was written to allow for generalized homophily.The
\code{attr} term argument should specify a character vertex attribute
encoding the "venues" associated to each node.  The \code{split} argument
should specify a string that separates different "venues" in the attribute
value for each node, as handled by \code{strsplit} with \code{fixed = TRUE}.
For example, if \code{split} is \code{"|"} (the default), and the attribute
value for a given node is \code{"a12|b476"}, then the associated venues for
this node are \code{"a12"} and \code{"b476"}.  The empty string \code{""} is
interpreted as "no venues".

If the \code{binary} term argument is \code{FALSE} (the default), the change
statistic for an on-toggle is the number of unique venues associated to both
nodes (informally speaking, this could be described as the number of venues
on which the two nodes "match"); if \code{binary} is \code{TRUE}, the change
statistic for an on-toggle is \code{1} if any venue is associated to both
nodes, and \code{0} otherwise.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dcm.mods.R
\name{dcm.mods}
\alias{dcm.mods}
\alias{mod_SI_1g_cl}
\alias{mod_SI_1g_op}
\alias{mod_SI_2g_cl}
\alias{mod_SI_2g_op}
\alias{mod_SIR_1g_cl}
\alias{mod_SIR_1g_op}
\alias{mod_SIR_2g_cl}
\alias{mod_SIR_2g_op}
\alias{mod_SIS_1g_cl}
\alias{mod_SIS_1g_op}
\alias{mod_SIS_2g_cl}
\alias{mod_SIS_2g_op}
\title{Deterministic Compartmental Model Functions}
\usage{
mod_SI_1g_cl(t, t0, parms)

mod_SI_1g_op(t, t0, parms)

mod_SI_2g_cl(t, t0, parms)

mod_SI_2g_op(t, t0, parms)

mod_SIR_1g_cl(t, t0, parms)

mod_SIR_1g_op(t, t0, parms)

mod_SIR_2g_cl(t, t0, parms)

mod_SIR_2g_op(t, t0, parms)

mod_SIS_1g_cl(t, t0, parms)

mod_SIS_1g_op(t, t0, parms)

mod_SIS_2g_cl(t, t0, parms)

mod_SIS_2g_op(t, t0, parms)
}
\arguments{
\item{t}{Time vector, passed into model function internally through
\code{\link[=dcm]{dcm()}} via the control settings in
\code{\link[=control.dcm]{control.dcm()}}.}

\item{t0}{Initial conditions for model, passed into model function internally
through \code{\link[=dcm]{dcm()}} via the initial conditions in
\code{\link[=init.dcm]{init.dcm()}}.}

\item{parms}{Model parameters, passed into model function internally through
\code{\link[=dcm]{dcm()}} via the parameter settings in
\code{\link[=param.dcm]{param.dcm()}}.}
}
\description{
These functions parameterize the base deterministic
compartmental models solved using the \code{\link[=dcm]{dcm()}}
function.
}
\details{
This help page shows the names of all the base deterministic compartmental
model functions supported in EpiModel. Base models are those already
programmed interally within the software. The model functions may be printed
to see their internal structure, either directly on the console or by using
the \code{print.mod} argument in \code{\link[=control.dcm]{control.dcm()}}.

The naming convention for the models listed here follows the format:
\verb{mod_<disease type>_<number of groups>_<vital dynamics>}. The supported
disease types are SI, SIS, and SIR; the number of groups are 1 or 2; and the
vital dynamic options are closed (fixed population composition) or open (with
arrivals and departures).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.inputs.R
\name{multilayer}
\alias{multilayer}
\title{Specify Controls by Network}
\usage{
multilayer(...)
}
\arguments{
\item{...}{control arguments to apply to each network, with the index of the
network corresponding to the index of the control argument}
}
\value{
an object of class \code{multilayer} containing the specified
control arguments
}
\description{
This utility function allows specification of certain
\code{\link[=netsim]{netsim()}} controls to vary by network.  The
\code{\link[=netsim]{netsim()}} control arguments currently supporting
\code{multilayer} specifications are \code{nwstats.formula},
\code{set.control.ergm}, \code{set.control.tergm}, and
\code{tergmLite.track.duration}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netest.R
\name{update_dissolution}
\alias{update_dissolution}
\title{Adjust Dissolution Component of Network Model Fit}
\usage{
update_dissolution(old.netest, new.coef.diss, nested.edapprox = TRUE)
}
\arguments{
\item{old.netest}{An object of class \code{netest}, from the
\code{\link[=netest]{netest()}} function.}

\item{new.coef.diss}{An object of class \code{disscoef}, from the
\code{\link[=dissolution_coefs]{dissolution_coefs()}} function.}

\item{nested.edapprox}{Logical. If \code{edapprox = TRUE} the dissolution
model is an initial segment of the formation model (see details in
\code{\link[=netest]{netest()}}).}
}
\value{
An updated network model object of class \code{netest}.
}
\description{
Adjusts the dissolution component of a dynamic ERGM fit using
the \code{\link[=netest]{netest()}} function with the edges dissolution
approximation method.
}
\details{
Fitting an ERGM is a computationally intensive process when the model
includes dyad dependent terms. With the edges dissolution approximation
method of Carnegie et al, the coefficients for a temporal ERGM are
approximated by fitting a static ERGM and adjusting the formation
coefficients to account for edge dissolution. This function provides a very
efficient method to adjust the coefficients of that model when one wants to
use a different dissolution model; a typical use case may be to fit several
different models with different average edge durations as targets. The
example below exhibits that case.
}
\examples{
\dontrun{
nw <- network_initialize(n = 1000)

# Two dissolutions: an average duration of 300 versus 200
diss.300 <- dissolution_coefs(~offset(edges), 300, 0.001)
diss.200 <- dissolution_coefs(~offset(edges), 200, 0.001)

# Fit the two reference models
est300 <- netest(nw = nw,
                formation = ~edges,
                target.stats = c(500),
                coef.diss = diss.300)

est200 <- netest(nw = nw,
                formation = ~edges,
                target.stats = c(500),
                coef.diss = diss.200)

# Alternatively, update the 300 model with the 200 coefficients
est200.compare <- update_dissolution(est300, diss.200)

identical(est200$coef.form, est200.compare$coef.form)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.netsim.R
\name{plot.netsim}
\alias{plot.netsim}
\title{Plot Data from a Stochastic Network Epidemic Model}
\usage{
\method{plot}{netsim}(
  x,
  type = "epi",
  y = NULL,
  popfrac = FALSE,
  sim.lines = FALSE,
  sims = NULL,
  sim.col = NULL,
  sim.lwd = NULL,
  sim.alpha = NULL,
  mean.line = TRUE,
  mean.smooth = TRUE,
  mean.col = NULL,
  mean.lwd = 2,
  mean.lty = 1,
  qnts = 0.5,
  qnts.col = NULL,
  qnts.alpha = 0.5,
  qnts.smooth = TRUE,
  legend = NULL,
  leg.cex = 0.8,
  grid = FALSE,
  add = FALSE,
  network = 1,
  at = 1,
  col.status = FALSE,
  shp.g2 = NULL,
  vertex.cex = NULL,
  stats = NULL,
  targ.line = TRUE,
  targ.col = NULL,
  targ.lwd = 2,
  targ.lty = 2,
  plots.joined = NULL,
  duration.imputed = TRUE,
  method = "l",
  main = NULL,
  xlim = NULL,
  xlab = NULL,
  ylim = NULL,
  ylab = NULL,
  ...
)
}
\arguments{
\item{x}{An \code{EpiModel} model object of class \code{netsim}.}

\item{type}{Type of plot: \code{"epi"} for epidemic model results,
\code{"network"} for a static network plot (\code{plot.network}),
or \code{"formation"}, \code{"duration"}, or \code{"dissolution"} for
network formation, duration, or dissolution statistics.}

\item{y}{Output compartments or flows from \code{netsim} object to plot.}

\item{popfrac}{If \code{TRUE}, plot prevalence of values rather than numbers
(see details).}

\item{sim.lines}{If \code{TRUE}, plot individual simulation lines. Default is
to plot lines for one-group models but not for two-group models.}

\item{sims}{If \code{type="epi"} or \code{"formation"}, a vector of
simulation numbers to plot. If \code{type="network"}, a single
simulation number for which to plot the network, or else \code{"min"}
to plot the simulation number with the lowest disease prevalence,
\code{"max"} for the simulation with the highest disease prevalence,
or \code{"mean"} for the simulation with the prevalence closest to the
mean across simulations at the specified time step.}

\item{sim.col}{Vector of any standard R color format for simulation lines.}

\item{sim.lwd}{Line width for simulation lines.}

\item{sim.alpha}{Transparency level for simulation lines, where
0 = transparent and 1 = opaque (see \code{adjustcolor} function).}

\item{mean.line}{If \code{TRUE}, plot mean of simulations across time.}

\item{mean.smooth}{If \code{TRUE}, use a loess smoother on the mean line.}

\item{mean.col}{Vector of any standard R color format for mean lines.}

\item{mean.lwd}{Line width for mean lines.}

\item{mean.lty}{Line type for mean lines.}

\item{qnts}{If numeric, plot polygon of simulation quantiles based on the
range implied by the argument (see details). If \code{FALSE}, suppress
polygon from plot.}

\item{qnts.col}{Vector of any standard R color format for polygons.}

\item{qnts.alpha}{Transparency level for quantile polygons, where 0 =
transparent and 1 = opaque (see \code{adjustcolor} function).}

\item{qnts.smooth}{If \code{TRUE}, use a loess smoother on quantile polygons.}

\item{legend}{If \code{TRUE}, plot default legend.}

\item{leg.cex}{Legend scale size.}

\item{grid}{If \code{TRUE}, a grid is added to the background of plot
(see \code{\link[=grid]{grid()}} for details), with default of nx by ny.}

\item{add}{If \code{TRUE}, new plot window is not called and lines are added
to existing plot window.}

\item{network}{Network number, for simulations with multiple networks
representing the population.}

\item{at}{If \code{type = "network"}, time step for network graph.}

\item{col.status}{If \code{TRUE} and \code{type="network"}, automatic disease
status colors (blue = susceptible, red = infected, green = recovered).}

\item{shp.g2}{If \code{type = "network"} and \code{x} is for a two-group model,
shapes for the Group 2 vertices, with acceptable inputs of "triangle"
and "square". Group 1 vertices will remain circles.}

\item{vertex.cex}{Relative size of plotted vertices if \code{type="network"},
with implicit default of 1.}

\item{stats}{If \verb{type="formation","duration","dissolution"}, statistics
to plot. For \code{type = "formation"}, \code{stats} are among those
specified in \code{nwstats.formula} of \code{\link[=control.net]{control.net()}}; for
\verb{type = "duration", "dissolution"}, \code{stats} are among those
of the dissolution model (without \code{offset()}). The default is
to plot all statistics.}

\item{targ.line}{If \code{TRUE}, plot target or expected value line for
the statistic of interest.}

\item{targ.col}{Vector of standard R colors for target statistic lines, with
default colors based on \code{RColorBrewer} color palettes.}

\item{targ.lwd}{Line width for the line showing the target statistic values.}

\item{targ.lty}{Line type for the line showing the target statistic values.}

\item{plots.joined}{If \code{TRUE} and
\verb{type="formation","duration","dissolution"}, combine all
statistics in one plot, versus one plot per statistic if
\code{FALSE}.}

\item{duration.imputed}{If \code{type = "duration"}, a logical indicating
whether or not to impute starting times for relationships extant at
the start of the simulation. Defaults to \code{TRUE} when
\code{type = "duration"}.}

\item{method}{Plot method for \verb{type="formation", "duration", "dissolution"},
with options of \code{"l"} for line plots and \code{"b"} for box plots.}

\item{main}{a main title for the plot, see also \code{\link[graphics]{title}}.}

\item{xlim}{the x limits (x1, x2) of the plot.  Note that \code{x1 > x2}
    is allowed and leads to a \sQuote{reversed axis}.

    The default value, \code{NULL}, indicates that the range of the
    \link{finite} values to be plotted should be used.}

\item{xlab}{a label for the x axis, defaults to a description of \code{x}.}

\item{ylim}{the y limits of the plot.}

\item{ylab}{a label for the y axis, defaults to a description of \code{y}.}

\item{...}{Additional arguments to pass.}
}
\description{
Plots epidemiological and network data from a stochastic network
model simulated with \code{\link[=netsim]{netsim()}}.
}
\details{
This plot function can produce three types of plots with a stochastic network
model simulated through \code{\link[=netsim]{netsim()}}:
\enumerate{
\item \strong{\code{type="epi"}}: epidemic model results (e.g., disease
prevalence and incidence) may be plotted.
\item \strong{\code{type="network"}}: a static network plot will be
generated. A static network plot of a dynamic network is a
cross-sectional extraction of that dynamic network at a specific
time point. This plotting function wraps the
\code{\link[network:plot.network]{network::plot.network}} function in the \code{network} package.
Consult the help page for \code{plot.network} for all of the plotting
parameters. In addition, four plotting parameters specific to
\code{netsim} plots are available: \code{sim}, \code{at},
\code{col.status}, and \code{shp.g2}.
\item \strong{\code{type="formation"}}: summary network statistics related
to the network model formation are plotted. These plots are similar
to the formation plots for \code{netdx} objects. When running a
\code{netsim} simulation, one must specify there that
\code{save.nwstats=TRUE}; the plot here will then show the network
statistics requested explicitly in \code{nwstats.formula}, or will use
the formation formula set in \code{netest} otherwise.
\item \strong{\verb{type="duration","dissolution"}}: as in
\code{\link[=plot.netdx]{plot.netdx()}}; supported in \code{plot.netsim} only when
the dissolution model is \code{~offset(edges)}, \code{tergmLite} is
\code{FALSE}, and \code{save.network} is \code{TRUE}.
}

When \code{type="epi"}, this plotting function will extract the
epidemiological output from a model object of class \code{netsim} and plot
the time series data of disease prevalence and other results. The summary
statistics that the function calculates and plots are individual simulation
lines, means of the individual simulation lines, and quantiles of those
individual simulation lines. The mean line, toggled on with
\code{mean.line=TRUE}, is calculated as the row mean across simulations at
each time step.

Compartment prevalences are the size of a compartment over some denominator.
To plot the raw numbers from any compartment, use \code{popfrac=FALSE}; this
is the default for any plots of flows. The \code{popfrac} parameter
calculates and plots the denominators of all specified compartments using
these rules: 1) for one-group models, the prevalence of any compartment is
the compartment size divided by the total population size; 2) for two-group
models, the prevalence of any compartment is the compartment size divided by
the group population size. For any prevalences that are not automatically
calculated, the \code{\link[=mutate_epi]{mutate_epi()}} function may be used to add new
variables to the \code{netsim} object to plot or analyze.

The quantiles show the range of outcome values within a certain specified
quantile range. By default, the interquartile range is shown: that is the
middle 50\% of the data. This is specified by \code{qnts=0.5}. To show the
middle 95\% of the data, specify \code{qnts=0.95}. To toggle off the polygons
where they are plotted by default, specify \code{qnts=FALSE}.

When \code{type="network"}, this function will plot cross sections of the
simulated networks at specified time steps. Because it is only possible to
plot one time step from one simulation at a time, it is necessary to enter
these in the \code{at} and \code{sims} parameters. To aid in visualizing
representative and extreme simulations at specific time steps, the
\code{sims} parameter may be set to \code{"mean"} to plot the simulation in
which the disease prevalence is closest to the average across all
simulations, \code{"min"} to plot the simulation in which the prevalence is
lowest, and \code{"max"} to plot the simulation in which the prevalence is
highest.
}
\examples{
\donttest{
## SI Model without Network Feedback
# Initialize network and set network model parameters
nw <- network_initialize(n = 100)
nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)

# Estimate the network model
est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

# Simulate the epidemic model
param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
init <- init.net(i.num = 10, i.num.g2 = 10)
control <- control.net(type = "SI", nsteps = 20, nsims = 3,
                       verbose = FALSE, save.nwstats = TRUE,
                       nwstats.formula = ~edges + meandeg + concurrent)
mod <- netsim(est, param, init, control)

# Plot epidemic trajectory
plot(mod)
plot(mod, type = "epi", grid = TRUE)
plot(mod, type = "epi", popfrac = TRUE)
plot(mod, type = "epi", y = "si.flow", qnts = 1, ylim = c(0, 4))

# Plot static networks
par(mar = c(0, 0, 0, 0))
plot(mod, type = "network", vertex.cex = 1.5)

# Automatic coloring of infected nodes as red
par(mfrow = c(1, 2), mar = c(0, 0, 2, 0))
plot(mod, type = "network", main = "Min Prev | Time 50",
     col.status = TRUE, at = 20, sims = "min", vertex.cex = 1.25)
plot(mod, type = "network", main = "Max Prev | Time 50",
     col.status = TRUE, at = 20, sims = "max", vertex.cex = 1.25)

# Automatic shape by group number (circle = group 1)
par(mar = c(0, 0, 0, 0))
plot(mod, type = "network", at = 20, col.status = TRUE,
     shp.g2 = "square")
plot(mod, type = "network", at = 20, col.status = TRUE,
     shp.g2 = "triangle", vertex.cex = 2)

# Plot formation statistics
par(mfrow = c(1,1), mar = c(3,3,1,1), mgp = c(2,1,0))
plot(mod, type = "formation", grid = TRUE)
plot(mod, type = "formation", plots.joined = FALSE)
plot(mod, type = "formation", sims = 2:3)
plot(mod, type = "formation", plots.joined = FALSE,
     stats = c("edges", "concurrent"))
plot(mod, type = "formation", stats = "meandeg",
     mean.lwd = 1, qnts.col = "seagreen", mean.col = "black")
}

}
\seealso{
\code{\link[network:plot.network]{network::plot.network}}, \code{\link[=mutate_epi]{mutate_epi()}}
}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.end.horizon.R
\name{trigger_end_horizon}
\alias{trigger_end_horizon}
\title{Function to Trigger the End Horizon}
\usage{
trigger_end_horizon(dat)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Function to Trigger the End Horizon
}
\details{
This function triggers the end horizon if a control \code{end.horizon} exists and
its \code{at} value is equal to the current timestep. The end horizon consists on
the removal of a set of modules from the module list.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{mutate_epi}
\alias{mutate_epi}
\title{Add New Epidemiology Variables}
\usage{
mutate_epi(x, ...)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{dcm}, \code{icm}, or
\code{netsim}.}

\item{...}{Name-value pairs of expressions (see examples below).}
}
\value{
The updated \code{EpiModel} object of class \code{dcm}, \code{icm},
or \code{netsim}.
}
\description{
Inspired by \code{dplyr::mutate}, \code{mutate_epi} adds new
variables to the epidemiological and related variables within
simulated model objects of any class in \code{EpiModel}.
}
\examples{
# DCM example
param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
init <- init.dcm(s.num = 500, i.num = 1)
control <- control.dcm(type = "SI", nsteps = 500)
mod1 <- dcm(param, init, control)
mod1 <- mutate_epi(mod1, prev = i.num/num)
plot(mod1, y = "prev")

\donttest{
# Network model example
nw <- network_initialize(n = 100)
nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
init <- init.net(i.num = 1, i.num.g2 = 0)
control <- control.net(type = "SI", nsteps = 10, nsims = 3,
                       verbose = FALSE)
mod1 <- netsim(est1, param, init, control)
mod1

# Add the prevalences to the dataset
mod1 <- mutate_epi(mod1, i.prev = i.num / num,
                         i.prev.g2 = i.num.g2 / num.g2)
plot(mod1, y = c("i.prev", "i.prev.g2"), qnts = 0.5, legend = TRUE)

# Add incidence rate per 100 person years (assume time step = 1 week)
mod1 <- mutate_epi(mod1, ir100 = 5200*(si.flow + si.flow.g2) /
                                      (s.num + s.num.g2))
as.data.frame(mod1)
as.data.frame(mod1, out = "mean")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update.R
\name{add_vertices}
\alias{add_vertices}
\title{Fast Version of network::add.vertices for Edgelist-formatted Network}
\usage{
add_vertices(el, nv)
}
\arguments{
\item{el}{A two-column matrix of current edges (edgelist) with an attribute
variable \code{n} containing the total current network size.}

\item{nv}{A integer equal to the number of nodes to add to the network
size at the given time step.}
}
\value{
Returns the matrix of current edges, \code{el}, with the population size
attribute updated based on the number of new vertices specified in \code{nv}.
}
\description{
This function performs a simple operation of updating the
edgelist attribute \code{n} that tracks the total network
size implicit in an edgelist representation of the network.
}
\details{
This function is used in \code{EpiModel} modules to add vertices (nodes) to
the edgelist object to account for entries into the population (e.g., births
and in-migration).
}
\examples{
\dontrun{
library("EpiModel")
nw <- network_initialize(100)
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
x <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

param <- param.net(inf.prob = 0.3)
init <- init.net(i.num = 10)
control <- control.net(type = "SI", nsteps = 100, nsims = 5,
                       tergmLite = TRUE)

# networkLite representation after initialization
dat <- crosscheck.net(x, param, init, control)
dat <- initialize.net(x, param, init, control)

# Check current network size
attributes(dat$el[[1]])$n

# Add 10 vertices
dat$el[[1]] <- add_vertices(dat$el[[1]], 10)

# Check new network size
attributes(dat$el[[1]])$n
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.accessor.R
\name{unique_id-tools}
\alias{unique_id-tools}
\alias{get_unique_ids}
\alias{get_posit_ids}
\title{Convert Unique Identifiers to/from Positional Identifiers}
\usage{
get_unique_ids(dat, posit_ids = NULL)

get_posit_ids(dat, unique_ids = NULL)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{posit_ids}{A vector of node positional identifiers (default = NULL).}

\item{unique_ids}{A vector of node unique identifiers (default = NULL).}
}
\value{
A vector of unique or positional identifiers.
}
\description{
EpiModel refers to its nodes either by positional identifiers
(\code{posit_ids}), which describe the position of a node in the
\code{attr} vector, or by unique identifiers
(\code{unique_ids}), which allow references to nodes even after
they are deactivated.
}
\section{All elements}{

When \code{unique_ids} or \code{posit_ids} is NULL (default)
the full list of positional IDs or unique IDs is returned.
}

\section{Deactivated nodes}{

When providing \code{unique_ids} of deactivated nodes to
\code{get_posit_ids}, \code{NA}s are returned instead and a warning is
produced.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.icm}
\alias{print.icm}
\title{Print Method for ICM Objects}
\usage{
\method{print}{icm}(x, ...)
}
\arguments{
\item{x}{An object of class \code{icm}, from \code{\link[=icm]{icm()}}.}

\item{...}{Additional arguments (currently ignored).}
}
\description{
Prints a summary of a stochastic individual contact model
object, including the model type, number of simulations, time steps, model
parameters, and output variable names.
}
\details{
Given an \code{icm} object, \code{print.icm} displays:
\itemize{
\item \strong{Simulation summary}: model class, model type (e.g., SI, SIR, SIS),
number of simulations, number of time steps, and number of groups.
\item \strong{Model parameters}: all parameters passed via \code{\link[=param.icm]{param.icm()}}, excluding
internal bookkeeping parameters (\code{groups}, \code{vital}).
\item \strong{Model output}: the names of all epidemic output variables stored in the
model.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dcm.R
\name{dcm}
\alias{dcm}
\title{Deterministic Compartmental Models}
\usage{
dcm(param, init, control)
}
\arguments{
\item{param}{Model parameters, as an object of class \code{\link[=param.dcm]{param.dcm()}}.}

\item{init}{Initial conditions, as an object of class \code{\link[=init.dcm]{init.dcm()}}.}

\item{control}{Control settings, as an object of class
\code{\link[=control.dcm]{control.dcm()}}.}
}
\value{
A list of class \code{dcm} with the following elements:
\itemize{
\item \strong{param:} the epidemic parameters passed into the model through
\code{param}, with additional parameters added as necessary.
\item \strong{control:} the control settings passed into the model through
\code{control}, with additional controls added as necessary.
\item \strong{epi:} a list of data frames, one for each epidemiological
output from the model. Outputs for base models always include the
size of each compartment, as well as flows in, out of, and between
compartments.
}
}
\description{
Solves deterministic compartmental epidemic models for
infectious disease.
}
\details{
The \code{dcm} function uses the ordinary differential equation solver in
the \code{deSolve} package to model disease as a deterministic compartmental
system. The parameterization for these models follows the standard approach
in \code{EpiModel}, with epidemic parameters, initial conditions, and control
settings.

The \code{dcm} function performs modeling of both base model types and
original models with new structures. Base model types include one-group
and two-group models with disease types for Susceptible-Infected (SI),
Susceptible-Infected-Recovered (SIR), and Susceptible-Infected-Susceptible
(SIS). Both base and original models require the \code{param},
\code{init}, and \code{control} inputs.
}
\examples{
## Example 1: SI Model (One-Group)
# Set parameters
param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
init <- init.dcm(s.num = 500, i.num = 1)
control <- control.dcm(type = "SI", nsteps = 500)
mod1 <- dcm(param, init, control)
mod1
plot(mod1)

## Example 2: SIR Model with Vital Dynamics (One-Group)
param <- param.dcm(inf.prob = 0.2, act.rate = 5,
                   rec.rate = 1/3, a.rate = 1/90, ds.rate = 1/100,
                   di.rate = 1/35, dr.rate = 1/100)
init <- init.dcm(s.num = 500, i.num = 1, r.num = 0)
control <- control.dcm(type = "SIR", nsteps = 500)
mod2 <- dcm(param, init, control)
mod2
plot(mod2)

## Example 3: SIS Model with act.rate Sensitivity Parameter
param <- param.dcm(inf.prob = 0.2, act.rate = seq(0.1, 0.5, 0.1),
                   rec.rate = 1/50)
init <- init.dcm(s.num = 500, i.num = 1)
control <- control.dcm(type = "SIS", nsteps = 500)
mod3 <- dcm(param, init, control)
mod3
plot(mod3)

## Example 4: SI Model with Vital Dynamics (Two-Group)
param <- param.dcm(inf.prob = 0.4,  inf.prob.g2 = 0.1,
                   act.rate = 0.25, balance = "g1",
                   a.rate = 1/100, a.rate.g2 = NA,
                   ds.rate = 1/100, ds.rate.g2 = 1/100,
                   di.rate = 1/50, di.rate.g2 = 1/50)
init <- init.dcm(s.num = 500, i.num = 1,
                 s.num.g2 = 500, i.num.g2 = 0)
control <- control.dcm(type = "SI", nsteps = 500)
mod4 <- dcm(param, init, control)
mod4
plot(mod4)

## Example 5: SI Model with Varying Initial Conditions
param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
init <- init.dcm(s.num = 500, i.num = c(1, 5, 25))
control <- control.dcm(type = "SI", nsteps = 500)
mod5 <- dcm(param, init, control)
mod5
plot(mod5)

}
\references{
Soetaert K, Petzoldt T, Setzer W. Solving Differential Equations in
R: Package deSolve. Journal of Statistical Software. 2010; 33(9): 1-25.
\doi{10.18637/jss.v033.i09}.
}
\seealso{
Extract the model results with \code{\link[=as.data.frame.dcm]{as.data.frame.dcm()}}.
Summarize the time-specific model results with \code{\link[=summary.dcm]{summary.dcm()}}.
Plot the model results with \code{\link[=plot.dcm]{plot.dcm()}}. Plot a compartment flow
diagram with \code{\link[=comp_plot]{comp_plot()}}.
}
\keyword{model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge.R
\name{merge.icm}
\alias{merge.icm}
\title{Merge Data across Stochastic Individual Contact Model Simulations}
\usage{
\method{merge}{icm}(x, y, ...)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{\link[=icm]{icm()}}.}

\item{y}{Another \code{EpiModel} object of class \code{\link[=icm]{icm()}}, with the
identical model parameterization as \code{x}.}

\item{...}{Additional merge arguments (not used).}
}
\value{
An \code{EpiModel} object of class \code{\link[=icm]{icm()}} containing the
data from both \code{x} and \code{y}.
}
\description{
Merges epidemiological data from two independent simulations of
stochastic individual contact models from \code{\link[=icm]{icm()}}.
}
\details{
This merge function combines the results of two independent simulations of
\code{\link[=icm]{icm()}} class models, simulated under separate function calls. The
model parameterization between the two calls must be exactly the same, except
for the number of simulations in each call. This allows for manual
parallelization of model simulations.

This merge function does not work the same as the default merge, which allows
for a combined object where the structure differs between the input elements.
Instead, the function checks that objects are identical in model
parameterization in every respect (except number of simulations) and binds
the results.
}
\examples{
param <- param.icm(inf.prob = 0.2, act.rate = 0.8)
init <- init.icm(s.num = 1000, i.num = 100)
control <- control.icm(type = "SI", nsteps = 10,
                       nsims = 3, verbose = FALSE)
x <- icm(param, init, control)

control <- control.icm(type = "SI", nsteps = 10,
                       nsims = 1, verbose = FALSE)
y <- icm(param, init, control)

z <- merge(x, y)

# Examine separate and merged data
as.data.frame(x)
as.data.frame(y)
as.data.frame(z)

}
\keyword{extract}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.R
\name{get_sims}
\alias{get_sims}
\title{Extract Network Simulations}
\usage{
get_sims(x, sims = NULL, var = NULL)
}
\arguments{
\item{x}{An object of class \code{netsim}.}

\item{sims}{Either a numeric vector of simulation numbers to retain in the
output object, or \code{"mean"}, which selects the one simulation with
the value of the variable specified in \code{var} closest to the mean
of \code{var} across all simulations at the final step.}

\item{var}{A character vector of variables to retain from \code{x} if
\code{sims} is a numeric vector, or a single variable name for
selecting the average simulation from the set if \code{sims = "mean"}.}
}
\value{
An updated object of class \code{netsim} containing only the
simulations specified in \code{sims} and the variables specified in
\code{var}.
}
\description{
Subsets the entire \code{netsim} object to a subset of
simulations, essentially functioning like a reverse of
\code{merge}.
}
\examples{
\donttest{
# Network model estimation
nw <- network_initialize(n = 100)
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

# Epidemic model
param <- param.net(inf.prob = 0.3)
init <- init.net(i.num = 10)
control <- control.net(type = "SI", nsteps = 10, nsims = 3, verbose.int = 0)
mod1 <- netsim(est1, param, init, control)

# Get sim 2
s.g2 <- get_sims(mod1, sims = 2)

# Get sims 2 and 3 and keep only a subset of variables
s.g2.small <- get_sims(mod1, sims = 2:3, var = c("i.num", "si.flow"))

# Extract the mean simulation for the variable i.num
sim.mean <- get_sims(mod1, sims = "mean", var = "i.num")
}

}
\keyword{extract}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.prevalence.R
\name{prevalence.net}
\alias{prevalence.net}
\title{Get Epidemic Output from netsim Model}
\usage{
prevalence.net(dat, at)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{Current time step.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Provides all active model state sizes from the network at the
specified time step, output to a list of vectors.
}
\details{
This network utility is used during the \code{\link[=netsim]{netsim()}} simulation
process to efficiently query the current size of each state or compartment
in the model at any given timestep. For a two-group network, the current
state size for each group and overall is provided.
}
\keyword{internal}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reachable.R
\name{reachable-nodes}
\alias{reachable-nodes}
\alias{get_forward_reachable}
\alias{get_backward_reachable}
\title{Get the Forward or Backward Reachable Nodes for a Set of Nodes}
\usage{
get_forward_reachable(
  el_cuml,
  from_step,
  to_step,
  nodes = NULL,
  dense_optim = "auto"
)

get_backward_reachable(
  el_cuml,
  from_step,
  to_step,
  nodes = NULL,
  dense_optim = "auto"
)
}
\arguments{
\item{el_cuml}{a cumulative edgelist object. That is a data.frame with at
least columns: head, tail, start and stop. Start and stop are inclusive.}

\item{from_step}{the beginning of the time period.}

\item{to_step}{the end of the time period.}

\item{nodes}{the subset of nodes to calculate the FRP for. (default = NULL,
all nodes)}

\item{dense_optim}{pre-process the adjacency list to speed up the
computations on dense networks. "auto" (default), enable the
optimisation when \code{n_edges > n_nodes}. "yes" always enables and "no"
always disables. The overhead of the optimization is not worth it on
sparse networks.}
}
\value{
A named list containing:
\code{reached}: the set of reachable nodes for each of the \code{nodes}.
\code{lengths}: A matrix of \code{length(nodes)} rows and one column per timestep + 1
with the length of the reachable set at each step from \code{from_step - 1}
to \code{to_step}. The first column is always one as the set of reachables
at the beginning is just the node itself.
}
\description{
These functions return the Forward or Backward Reachable Nodes of a set of
nodes in a network over a time. Warning, these functions ignore nodes without
edges in the period of interest. See the \verb{Number of Nodes} section for
details It is much faster than iterating
\code{tsna::tPath}. The distance between to each node can be back calculated
using the length of the reachable set at each time step and the fact that the
reachable sets are ordered by the time to arrival.
}
\section{Number of Nodes}{

To speed up the calculations and lower the memory usage, these functions
only take into account nodes with edges in the cumulative edgelist over the
period of interest. The nodes are identified in the \code{reached} and \code{lengths}
sublists by names (e.g. \code{node_1093}). Nodes without any edges are therefore
not calculated as the only node they reach is themselve (length of 1).
Take this fact into account when exploring the distribution of Forward
Reachable Paths for example. As the nodes with FRP == 1 are not in the
output.
}

\section{Time and Memory Use}{

These functions may be used to efficiently calculate multiple sets of
reachable nodes. As cumulative edgelists are way smaller than full
\code{networkDynamic} objects, theses functions are suited for large and dense
networks.
Also, as long as the size of the \code{nodes} set is greater than 5, theses
functions are faster than iterating over \code{tsna::tPath}.
}

\section{Displaying Progress}{

These functions are using the
\href{https://progressr.futureverse.org/articles/progressr-01-intro.html}{progressr package}
to display its progression. Use
\code{progressr::with_progress({ fwd_reach <- get_forward_reachable(el, from = 1, to = 260) })}
to display the progress bar. Or see the
\href{https://progressr.futureverse.org/articles/progressr-01-intro.html}{progressr package}
for more information and customization.
}

\examples{
\dontrun{

# load a network dynamic object
nd <- readRDS("nd_obj.Rds")
# convert it to a cumulative edgelist
el_cuml <- as_cumulative_edgelist(nd)

# sample 100 node indexes
nnodes <- max(el_cuml$head, el_cuml$tail)
nodes <- sample(nnodes, 100)

# `get_forward_reachable` uses steps [from_step, to_step] inclusive
el_fwd <- get_forward_reachable(el_cuml, 1, 52, nodes)[["reached"]]

# check if the results are consistent with `tsna::tPath`
nodes <- strsplit(names(el_fwd), "_")
for (i in seq_along(el_fwd)) {
  node <- as.integer(nodes[[i]][2])
  t_fwd <- tsna::tPath(
    nd, v = node,
    start = 1, end = 52 + 1, # tPath works from [start, end) right exclusive
    direction = "fwd"
  )

  t_fwd_set <- which(t_fwd$tdist < Inf)
  if(!setequal(el_fwd[[i]], t_fwd_set))
    stop("Missmatch on node: ", node)
}

# Backward:
el_bkwd <- get_backward_reachable(el_cuml, 1, 52, nodes = 1)[["reached"]]
nodes <- strsplit(names(el_bkwd), "_")
t_bkwd <- tsna::tPath(
  nd, v = nodes[i][2],
  start = 1, end = 52 + 1,
  direction = "bkwd", type = "latest.depart"
)
t_bkwd_set <- which(t_bkwd$tdist < Inf)
setequal(el_bkwd[[1]], t_bkwd_set)

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icm.inputs.R
\name{param.icm}
\alias{param.icm}
\title{Epidemic Parameters for Stochastic Individual Contact Models}
\usage{
param.icm(
  inf.prob,
  inter.eff,
  inter.start,
  act.rate,
  rec.rate,
  a.rate,
  ds.rate,
  di.rate,
  dr.rate,
  inf.prob.g2,
  act.rate.g2,
  rec.rate.g2,
  a.rate.g2,
  ds.rate.g2,
  di.rate.g2,
  dr.rate.g2,
  balance,
  ...
)
}
\arguments{
\item{inf.prob}{Probability of infection per transmissible act between
a susceptible and an infected person. In two-group models, this is the
probability of infection for the group 1 members.}

\item{inter.eff}{Efficacy of an intervention which affects the per-act
probability of infection. Efficacy is defined as 1 - the relative
hazard of infection given exposure to the intervention, compared to no
exposure.}

\item{inter.start}{Time step at which the intervention starts, between 1 and
the number of time steps specified in the model. This will default to
1 if \code{inter.eff} is defined but this parameter is not.}

\item{act.rate}{Average number of transmissible acts per person per unit
time. For two-group models, this is the number of acts per group 1
person per unit time; a balance between the acts in groups 1 and 2 is
necessary, and set using the \code{balance} parameter (see details).}

\item{rec.rate}{Average rate of recovery with immunity (in \code{SIR} models)
or re-susceptibility (in \code{SIS} models). The recovery rate is the
reciprocal of the disease duration. For two-group models, this is the
recovery rate for group 1 persons only. This parameter is only used
for \code{SIR} and \code{SIS} models.}

\item{a.rate}{Arrival or entry rate. For one-group models, the arrival rate
is the rate of new arrivals per person per unit time. For two-group
models, the arrival rate is parameterized as a rate per group 1
person per unit time, with the \code{a.rate.g2} rate set as described
below.}

\item{ds.rate}{Departure or exit rate for susceptible persons. For two-group
models, it is the rate for the group 1 susceptible persons only.}

\item{di.rate}{Departure or exit rate for infected persons. For two-group
models, it is the rate for the group 1 infected persons only.}

\item{dr.rate}{Departure or exit rate for recovered persons. For two-group
models, it is the rate for the group 1 recovered persons only. This
parameter is only used for \code{SIR} models.}

\item{inf.prob.g2}{Probability of infection per transmissible act
between a susceptible group 2 person and an infected group 1 person.
It is the probability of infection to group 2 members.}

\item{act.rate.g2}{Average number of transmissible acts per group 2 person
per unit time; a balance between the acts in groups 1 and 2 is
necessary, and set using the \code{balance} parameter (see details).}

\item{rec.rate.g2}{Average rate of recovery with immunity (in \code{SIR}
models) or re-susceptibility (in \code{SIS} models) for group 2
persons. This parameter is only used for two-group \code{SIR} and
\code{SIS} models.}

\item{a.rate.g2}{Arrival or entry rate for group 2. This may either be
specified numerically as the rate of new arrivals per group 2 persons
per unit time, or as \code{NA} in which case the group 1 rate,
\code{a.rate}, governs the group 2 rate. The latter is used when,
for example, the first group is conceptualized as female, and the
female population size determines the arrival rate. Such arrivals are
evenly allocated between the two groups.}

\item{ds.rate.g2}{Departure or exit rate for group 2 susceptible persons.}

\item{di.rate.g2}{Departure or exit rate for group 2 infected persons.}

\item{dr.rate.g2}{Departure or exit rate for group 2 recovered persons. This
parameter is only used for \code{SIR} model types.}

\item{balance}{For two-group models, balance the \code{act.rate} to the rate
set for group 1 (with \code{balance="g1"}) or group 2 (with
\code{balance="g2"}). See details.}

\item{...}{Additional arguments passed to model.}
}
\value{
An \code{EpiModel} object of class \code{param.icm}.
}
\description{
Sets the epidemic parameters for stochastic individual contact
models simulated with \code{icm}.
}
\details{
\code{param.icm} sets the epidemic parameters for the stochastic individual
contact models simulated with the \code{\link[=icm]{icm()}} function. Models
may use the base types, for which these parameters are used, or new process
modules which may use these parameters (but not necessarily).

For base models, the model specification will be chosen as a result of
the model parameters entered here and the control settings in
\code{\link[=control.icm]{control.icm()}}. One-group and two-group models are available,
where the former assumes a homogeneous mixing in the population and the
latter assumes some form of heterogeneous mixing between two distinct
partitions in the population (e.g., men and women). Specifying any group two
parameters (those with a \code{.g2}) implies the simulation of a two-group
model. All the parameters for a desired model type must be specified, even if
they are zero.
}
\section{Act Balancing}{

In two-group models, a balance between the number of acts for group 1 members
and those for group 2 members must be maintained. With purely heterogeneous
mixing, the product of one group size and act rate must equal the product of
the other group size and act rate: \eqn{N_1 \alpha_1 = N_2 \alpha_2}, where
\eqn{N_i} is the group size and \eqn{\alpha_i} the group-specific act rate
at time \eqn{t}. The \code{balance} parameter here specifies which group's
act rate should control the others with respect to balancing.
}

\seealso{
Use \code{\link[=init.icm]{init.icm()}} to specify the initial conditions and
\code{\link[=control.icm]{control.icm()}} to specify the control settings. Run the
parameterized model with \code{\link[=icm]{icm()}}.
}
\keyword{parameterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{copy_nwattr_to_datattr}
\alias{copy_nwattr_to_datattr}
\title{Copy Vertex Attributes From Network to \code{netsim_dat} List}
\usage{
copy_nwattr_to_datattr(dat, nw)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{nw}{Network from which to copy vertex attributes.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Copies the vertex attributes stored on the network object to the
main \code{attr} list in the \code{netsim_dat} data object.
}
\seealso{
\code{\link[=get_formula_term_attr]{get_formula_term_attr()}}, \code{\link[=get_attr_prop]{get_attr_prop()}},
\code{\link[=auto_update_attr]{auto_update_attr()}}, and
\code{\link[=copy_datattr_to_nwattr]{copy_datattr_to_nwattr()}}.
}
\keyword{internal}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icm.inputs.R
\name{control.icm}
\alias{control.icm}
\title{Control Settings for Stochastic Individual Contact Models}
\usage{
control.icm(type, nsteps, nsims = 1, verbose = FALSE, verbose.int = 0)
}
\arguments{
\item{type}{Disease type to be modeled, with the choice of \code{"SI"} for
Susceptible-Infected diseases, \code{"SIR"} for
Susceptible-Infected-Recovered diseases, and \code{"SIS"} for
Susceptible-Infected-Susceptible diseases.}

\item{nsteps}{Number of time steps to solve the model over. This must be a
positive integer.}

\item{nsims}{Number of simulations to run.}

\item{verbose}{If \code{TRUE}, print model progress to the console.}

\item{verbose.int}{Time step interval for printing progress to console, where
0 (the default) prints completion status of entire simulation and
positive integer \code{x} prints progress after every \code{x} time
steps.}
}
\value{
An \code{EpiModel} object of class \code{control.icm}.
}
\description{
Sets the controls for stochastic individual contact models
simulated with \code{\link[=icm]{icm()}}.
}
\details{
\code{control.icm} sets the required control settings for any stochastic
individual contact model solved with the \code{\link[=icm]{icm()}} function. ICM simulations
use the built-in SI, SIR, and SIS disease types only. The \code{type} argument
is required and has no default. For custom or extension epidemic models, use
the network model class via \code{\link[=control.net]{control.net()}} instead.
}
\seealso{
Use \code{\link[=param.icm]{param.icm()}} to specify model parameters and
\code{\link[=init.icm]{init.icm()}} to specify the initial conditions. Run the
parameterized model with \code{\link[=icm]{icm()}}.
}
\keyword{parameterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgelists.R
\name{get_cumulative_degree}
\alias{get_cumulative_degree}
\title{Return the Cumulative Degree of a Set of Index Nodes}
\usage{
get_cumulative_degree(
  dat,
  index_posit_ids,
  networks = NULL,
  truncate = Inf,
  only.active.nodes = FALSE
)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{index_posit_ids}{The positional IDs of the indexes of interest.}

\item{networks}{Numerical indexes of the networks to extract the partnerships from. (May be > 1
for models with multi-layer networks.) If \code{NULL}, extract from all networks.}

\item{truncate}{After how many time steps a partnership that is no longer
active should be removed from the output.}

\item{only.active.nodes}{If \code{TRUE}, then inactive (e.g., deceased) partners will be removed from
the output.}
}
\value{
A \code{data.frame} with 2 columns:
\itemize{
\item \code{index_pid}: the positional ID (see \code{get_posit_ids}) of the
indexes.
\item \code{degree}: the cumulative degree of the index.
}
}
\description{
Return the Cumulative Degree of a Set of Index Nodes
}
\section{Cumulative Degree}{

The cumulative degree of a node is the number of edges connected to this
node at during the time window. The time window is by default all the steps
stored in the \code{cumulative_edgelist} or set by the \code{truncate} parameter.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dcm.inputs.R
\name{crosscheck.dcm}
\alias{crosscheck.dcm}
\title{Cross Checking of Inputs for Deterministic Compartmental Models}
\usage{
crosscheck.dcm(param, init, control)
}
\arguments{
\item{param}{An \code{EpiModel} object of class \code{\link[=param.dcm]{param.dcm()}}.}

\item{init}{An \code{EpiModel} object of class \code{\link[=init.dcm]{init.dcm()}}.}

\item{control}{An \code{EpiModel} object of class \code{\link[=control.dcm]{control.dcm()}}.}
}
\value{
This function returns no objects.
}
\description{
This function checks that the three parameter lists from
\code{\link[=param.dcm]{param.dcm()}}, \code{\link[=init.dcm]{init.dcm()}}, and
\code{\link[=control.dcm]{control.dcm()}} are consistent.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.R
\name{get_args}
\alias{get_args}
\title{Get Arguments from EpiModel Parameterization Functions}
\usage{
get_args(formal.args, dot.args)
}
\arguments{
\item{formal.args}{The output of \code{formals(sys.function())}.}

\item{dot.args}{The output of \code{list(...)}.}
}
\value{
A list of argument names and values.
}
\description{
Returns a list of argument names and values for use for
parameter processing functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgelists.R
\name{get_partners}
\alias{get_partners}
\title{Return the Historical Contacts (Partners) of a Set of Index Nodes}
\usage{
get_partners(
  dat,
  index_posit_ids,
  networks = NULL,
  truncate = Inf,
  only.active.nodes = FALSE
)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{index_posit_ids}{The positional IDs of the indexes of interest.}

\item{networks}{Numerical indexes of the networks to extract the partnerships from. (May be > 1
for models with multi-layer networks.) If \code{NULL}, extract from all networks.}

\item{truncate}{After how many time steps a partnership that is no longer
active should be removed from the output.}

\item{only.active.nodes}{If \code{TRUE}, then inactive (e.g., deceased) partners will be removed from
the output.}
}
\value{
A \code{data.frame} with 5 columns:
\itemize{
\item \code{index}: the unique IDs of the indexes.
\item \code{partner}: the unique IDs of the partners/contacts.
\item \code{start}: the time step at which the edge started.
\item \code{stop}: the time step in which the edge stopped; if ongoing, then \code{NA} is returned.
\item \code{network}: the numerical index for the network on which the partnership/contact is located.
}
}
\description{
From a full cumulative edgelist that contains the history of contacts (both persistent and
one-time), this function returns a data frame containing details of the index (head) and partner
(tail) nodes, along with start and stop time steps for the partnership and the network location.
}
\details{
Note that \code{get_partners} takes as input the positional IDs of the indexes of interest but returns
the unique IDs. That is by design, because while \code{get_partners} would be expected to be called
for active nodes, some partners (contacts) of nodes may be inactive in the network history.
Therefore, both index and partner IDs are returned as unique IDs for consistency. To convert
between a positional to a unique ID, you may use \code{\link{get_unique_ids}}; to convert between a
unique ID to a positional ID, you may use \code{\link{get_posit_ids}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgelists.R
\name{get_discordant_edgelist}
\alias{get_discordant_edgelist}
\title{Get Discordant Edgelist Based on Specified Status Variable}
\usage{
get_discordant_edgelist(
  dat,
  status.attr,
  head.status,
  tail.status,
  networks = NULL
)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{status.attr}{The name of the status attribute of interest.}

\item{head.status}{The value(s) of \code{status.attr} for which to look for the head of the edge.
Can be a single value or a vector.}

\item{tail.status}{The value(s) of \code{status.attr} for which to look for the tail of the edge.
Can be a single value or a vector.}

\item{networks}{Numerical indexes of the networks to extract the partnerships
from. (May be > 1 for models with multiple overlapping
networks.) If \code{NULL}, extract from all networks.}
}
\value{
A \code{data.frame} with the following columns:
\itemize{
\item \code{head}: Positional ID of the head node.
\item \code{tail}: Positional ID of the tail node.
\item \code{head_status}: Status of the head node.
\item \code{tail_status}: Status of the tail node.
\item \code{network}: The numerical index of the network on which the partnership is located.
}
}
\description{
This function returns a \code{data.frame} with a discordant
edgelist, defined as the set of edges for which the status attribute
of interest is discordant between the two partners.
}
\details{
This is a generalized version of the \code{discord_edgelist} function.
It creates an edgelist of current partnerships in which the status attribute
of interest (as specified by the parameter \code{status.attr}) of one partner matches
the value (or one of the values) of the \code{head.status} parameter while the
corresponding status attribute of the other partner matches the value (or
one of the values) of the \code{tail.status} parameter.
}
\seealso{
\code{\link[=discord_edgelist]{discord_edgelist()}}
}
\keyword{internal}
\keyword{netMod}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.simnet.R
\name{resim_nets}
\alias{resim_nets}
\title{Resimulate Dynamic Network at Time 2+}
\usage{
resim_nets(dat, at)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{Current time step.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function resimulates the dynamic network in stochastic
network models simulated in \code{\link[=netsim]{netsim()}} with dependence
between the epidemic and demographic processes and the network
structure.
}
\keyword{internal}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.init.R
\name{init_status.net}
\alias{init_status.net}
\title{Disease Status Initialization Module for netsim}
\usage{
init_status.net(dat)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function sets the initial disease status on the
network given the specified initial conditions.
}
\details{
This internal function sets, either randomly or deterministically, the nodes
that are infected at \eqn{t_1}, the starting time of network simulations. If
the number to be initially infected is passed, this function sets the initial
number infected based on the number specified, either as a set of random
draws from a binomial distribution or as the exact number specified. In
either case, the specific nodes infected are a random sample from the
network. In contrast, a set of specific nodes may be infected by passing a
vector containing the status of each node to \code{\link[=netsim]{netsim()}}.

For the initially infected nodes, this module sets the time of infection as
\eqn{t_1}, the starting time of network simulations. For models with vital
dynamics, the infection time for those initially infected nodes is a random
draw from an exponential distribution with the rate parameter defined by the
\code{di.rate} argument. For models without vital dynamics, the infection
time is a random draw from a uniform distribution of integers with a minimum
of 1 and a maximum of the number of time steps in the model. In both cases,
to set the infection times to be in the past, these times are multiplied by
-1, and 2 is added to allow for possible infection times up until step 2,
when the disease simulation time loop starts.
}
\seealso{
This is an initialization module for \code{\link[=netsim]{netsim()}}.
}
\keyword{internal}
\keyword{netMod}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.record.R
\name{record_attr_history}
\alias{record_attr_history}
\title{Record Attribute History}
\usage{
record_attr_history(dat, at, attribute, posit_ids, values)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{The time where the recording happens.}

\item{attribute}{The name of the value to record.}

\item{posit_ids}{A numeric vector of posit_ids to which the measure applies.
(see \code{get_posit_ids}).}

\item{values}{The values to be recorded.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function records values specific to a time-step and a group of nodes.
In the records, the \code{posit_ids} are converted to \code{unique_ids} which
allows the recording of data for nodes that are no longer in the network by
the end of the run. The records are stored in \code{dat[["attr.history"]]}
where \code{dat} is the main \code{netsim_dat} class object, and can be
accessed from the \code{netsim} object with \code{get_attr_history}.
}
\details{
See the "Time-Varying Parameters" section of the "Working With Model
Parameters" vignette.
}
\examples{
\dontrun{
# This function must be used inside a custom module
dat <- record_attr_history(dat, at, "attr_1", get_posit_ids(dat), 5)
some_nodes <- get_posit_ids(dat)
some_nodes <- some_nodes[runif(length(some_nodes)) < 0.2]
dat <- record_attr_history(
  dat, at,
  "attr_2",
  some_nodes,
  rnorm(length(some_nodes))
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.inputs.R
\name{param.net}
\alias{param.net}
\title{Epidemic Parameters for Stochastic Network Models}
\usage{
param.net(
  inf.prob,
  inter.eff,
  inter.start,
  act.rate,
  rec.rate,
  a.rate,
  ds.rate,
  di.rate,
  dr.rate,
  inf.prob.g2,
  rec.rate.g2,
  a.rate.g2,
  ds.rate.g2,
  di.rate.g2,
  dr.rate.g2,
  ...
)
}
\arguments{
\item{inf.prob}{Probability of infection per transmissible act between
a susceptible and an infected person. In two-group models, this is the
probability of infection to the group 1 nodes. This may also be a
vector of probabilities, with each element corresponding to the
probability in that time step of infection (see Time-Varying
Parameters below).}

\item{inter.eff}{Efficacy of an intervention which affects the per-act
probability of infection. Efficacy is defined as 1 - the relative
hazard of infection given exposure to the intervention, compared to no
exposure.}

\item{inter.start}{Time step at which the intervention starts, between 1 and
the number of time steps specified in the model. This will default to
1 if \code{inter.eff} is defined but this parameter is not.}

\item{act.rate}{Average number of transmissible acts \emph{per partnership}
per unit time (see \code{act.rate} Parameter below). This may also be
a vector of rates, with each element corresponding to the rate in
that time step of infection (see Time-Varying Parameters below).}

\item{rec.rate}{Average rate of recovery with immunity (in \code{SIR} models)
or re-susceptibility (in \code{SIS} models). The recovery rate is the
reciprocal of the disease duration. For two-group models, this is the
recovery rate for group 1 persons only. This parameter is only used
for \code{SIR} and \code{SIS} models. This may also be a vector
of rates, with each element corresponding to the rate in that time
step of infection (see Time-Varying Parameters below).}

\item{a.rate}{Arrival or entry rate. For one-group models, the arrival rate
is the rate of new arrivals per person per unit time. For two-group
models, the arrival rate is parameterized as a rate per group 1
person per unit time, with the \code{a.rate.g2} rate set as described
below.}

\item{ds.rate}{Departure or exit rate for susceptible persons. For two-group
models, it is the rate for group 1 susceptible persons only.}

\item{di.rate}{Departure or exit rate for infected persons. For two-group
models, it is the rate for group 1 infected persons only.}

\item{dr.rate}{Departure or exit rate for recovered persons. For two-group
models, it is the rate for group 1 recovered persons only. This
parameter is only used for \code{SIR} models.}

\item{inf.prob.g2}{Probability of transmission given a transmissible act
between a susceptible group 2 person and an infected group 1 person.
It is the probability of transmission to group 2 members.}

\item{rec.rate.g2}{Average rate of recovery with immunity (in \code{SIR}
models) or re-susceptibility (in \code{SIS} models) for group 2
persons. This parameter is only used for two-group \code{SIR} and
\code{SIS} models.}

\item{a.rate.g2}{Arrival or entry rate for group 2. This may either be
specified numerically as the rate of new arrivals per group 2 person
per unit time, or as \code{NA}, in which case the group 1 rate,
\code{a.rate}, governs the group 2 rate. The latter is used when, for
example, the first group is conceptualized as female, and the female
population size determines the arrival rate. Such arrivals are evenly
allocated between the two groups.}

\item{ds.rate.g2}{Departure or exit rate for group 2 susceptible persons.}

\item{di.rate.g2}{Departure or exit rate for group 2 infected persons.}

\item{dr.rate.g2}{Departure or exit rate for group 2 recovered persons. This
parameter is only used for \code{SIR} model types.}

\item{...}{Additional arguments passed to model.}
}
\value{
An \code{EpiModel} object of class \code{param.net}.
}
\description{
Sets the epidemic parameters for stochastic network models
simulated with \code{\link[=netsim]{netsim()}}.
}
\details{
\code{param.net} sets the epidemic parameters for the stochastic network
models simulated with the \code{\link[=netsim]{netsim()}} function. Models
may use the base types, for which these parameters are used, or new process
modules which may use these parameters (but not necessarily). A detailed
description of network model parameterization for base models is found in
the \href{https://epimodel.github.io/sismid/}{Network Modeling for Epidemics}
tutorials.

For base models, the model specification will be chosen as a result of
the model parameters entered here and the control settings in
\code{\link[=control.net]{control.net()}}. One-group and two-group models are available,
where the latter assumes a heterogeneous mixing between two distinct
partitions in the population (e.g., men and women). Specifying any two-group
parameters (those with a \code{.g2}) implies the simulation of a two-group
model. All the parameters for a desired model type must be specified, even if
they are zero.
}
\section{The \code{act.rate} Parameter}{

A key difference between these network models and DCM/ICM classes is the
treatment of transmission events. With DCM and ICM, contacts or partnerships
are mathematically instantaneous events: they have no duration in time, and
thus no changes may occur within them over time. In contrast, network models
allow for partnership durations defined by the dynamic network model,
summarized in the model dissolution coefficients calculated in
\code{\link[=dissolution_coefs]{dissolution_coefs()}}. Therefore, the \code{act.rate} parameter has
a different interpretation here, where it is the number of transmissible acts
\emph{per partnership} per unit time.
}

\section{Time-Varying Parameters}{

The \code{inf.prob}, \code{act.rate}, \code{rec.rate} arguments (and their
\code{.g2} companions) may be specified as time-varying parameters by passing
in a vector of probabilities or rates, respectively. The value in each
position on the vector then corresponds to the probability or rate at that
discrete time step for the infected partner. For example, an \code{inf.prob}
of \code{c(0.5, 0.5, 0.1)} would simulate a 0.5 transmission probability for
the first two time steps of a person's infection, followed by a 0.1 for the
third time step. If the infected person has not recovered or exited the
population by the fourth time step, the third element in the vector will
carry forward until one of those events occurs or the simulation ends. For
further examples, see the
\href{https://epimodel.github.io/sismid/}{Network Modeling for Epidemics} tutorials.
}

\section{Random Parameters}{

In addition to deterministic parameters in either fixed or time-varying
varieties above, one may also include a generator for random parameters.
These might include a vector of potential parameter values or a statistical
distribution definition; in either case, one draw from the generator would
be completed per individual simulation. This is possible by passing a list
named \code{random.params} into \code{param.net}, with each element of
\code{random.params} a named generator function. See the help page and
examples in \code{\link[=generate_random_params]{generate_random_params()}}. A simple factory function
for sampling is provided with \code{\link[=param_random]{param_random()}} but any function
will do.
}

\section{Using a Parameter data.frame}{

It is possible to set input parameters using a specifically formatted
\code{data.frame} object. The first 3 columns of this \code{data.frame} must
be:
\itemize{
\item \code{param}: The name of the parameter. If this is a non-scalar
parameter (a vector of length > 1), end the parameter name with the
position on the vector (e.g., \code{"p_1"}, \code{"p_2"}, ...).
\item \code{value}: the value for the parameter (or the value of the
parameter in the Nth position if non-scalar).
\item \code{type}: a character string containing either \code{"numeric"},
\code{"logical"}, or \code{"character"} to define the parameter object
class.
}

In addition to these 3 columns, the \code{data.frame} can contain any number
of other columns, such as \code{details} or \code{source} columns to document
parameter meta-data. However, these extra columns will not be used by
EpiModel.

This data.frame is then passed in to \code{param.net} under a
\code{data.frame.parameters} argument. Further details and examples are
provided in the "Working with Model Parameters in EpiModel" vignette.
}

\section{Parameters with New Modules}{

To build original models outside of the base models, new process modules
may be constructed to replace the existing modules or to supplement the
existing set. These are passed into the control settings in
\code{\link[=control.net]{control.net()}}. New modules may use either the existing model
parameters named here, an original set of parameters, or a combination of
both. The \code{...} allows the user to pass an arbitrary set of original
model parameters into \code{param.net}. Whereas there are strict checks with
default modules for parameter validity, this becomes a user
responsibility when using new modules.
}

\examples{
\donttest{
## Example SIR model parameterization with fixed and random parameters
# Network model estimation
nw <- network_initialize(n = 100)
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

# Random epidemic parameter list (here act.rate values are sampled uniformly
# with helper function param_random, and inf.prob follows a general Beta
# distribution with the parameters shown below)
my_randoms <- list(
  act.rate = param_random(1:3),
  inf.prob = function() rbeta(1, 1, 2)
)

# Parameters, initial conditions, and control settings
param <- param.net(rec.rate = 0.02, random.params = my_randoms)

# Printing parameters shows both fixed and and random parameter functions
param

# Set initial conditions and controls
init <- init.net(i.num = 10, r.num = 0)
control <- control.net(type = "SIR", nsteps = 10, nsims = 3, verbose = FALSE)

# Simulate the model
sim <- netsim(est, param, init, control)

# Printing the sim object shows the randomly drawn values for each simulation
sim

# Parameter sets can be extracted with:
get_param_set(sim)
}

}
\seealso{
Use \code{\link[=init.net]{init.net()}} to specify the initial conditions and
\code{\link[=control.net]{control.net()}} to specify the control settings. Run the
parameterized model with \code{\link[=netsim]{netsim()}}.
}
\keyword{parameterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.modules.R
\name{remove_modules}
\alias{remove_modules}
\title{Remove a Set of Modules From the Module List}
\usage{
remove_modules(dat, names.to.remove)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{names.to.remove}{a character vector containing the name of the modules
to remove.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Remove a Set of Modules From the Module List
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{format_param}
\alias{format_param}
\title{Format One Parameter for Printing with the \code{print.param.xxx}
Functions}
\usage{
format_param(param_name, param_value)
}
\arguments{
\item{param_name}{The name of the parameter to print.}

\item{param_value}{The value of the parameter to print.}
}
\description{
Format One Parameter for Printing with the \code{print.param.xxx}
Functions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.updater.R
\name{common_updater}
\alias{common_updater}
\title{Update Either the "param" or "control" List}
\usage{
common_updater(dat, type)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{type}{Either \code{"param"} or \code{"control"}.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Update Either the "param" or "control" List
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.netsim}
\alias{print.netsim}
\title{Print Method for Network Model Simulations}
\usage{
\method{print}{netsim}(x, nwstats = TRUE, digits = 3, network = 1, ...)
}
\arguments{
\item{x}{An object of class \code{netsim}, from \code{\link[=netsim]{netsim()}}.}

\item{nwstats}{If \code{TRUE} (the default), print network statistics tables
(formation, duration, and dissolution) when available.}

\item{digits}{Number of digits to print in the network statistics tables.}

\item{network}{Integer index of the network for which to display statistics,
for multi-network models. Default is \code{1}.}

\item{...}{Additional arguments (currently ignored).}
}
\description{
Prints a detailed summary of a stochastic network model
simulation object, including simulation metadata, model parameters,
output variables, and (optionally) network formation, duration, and
dissolution statistics with target comparisons.
}
\details{
Given a \code{netsim} object, \code{print.netsim} displays the following sections:

\strong{Simulation summary}: model class, model type (e.g., SI, SIR, SIS),
number of simulations, number of time steps, and number of network groups.

\strong{Model parameters}: printed via \code{print.param.net()}, showing fixed and
(if applicable) random parameters.

\strong{Model functions}: for extension models (\code{type = NULL}), lists the names
of all custom module functions.

\strong{Model output}: the names of all epidemic output variables, saved network
objects, transmission matrices, and any other saved elements.

\strong{Network statistics} (when \code{nwstats = TRUE} and network statistics were
saved during the simulation):

The formation, duration, and dissolution statistics tables are computed and
displayed in the same way as for \code{\link[=print.netdx]{print.netdx()}}.  Each table contains the
columns \code{Target}, \verb{Sim Mean}, \verb{Pct Diff}, \verb{Sim SE}, \verb{Z Score},
\verb{SD(Sim Means)}, and \code{SD(Statistic)}.  The \verb{Sim Mean} column is the mean
statistic value across all time steps and simulations.  \verb{Sim SE} is the
standard error estimated using \link[coda:effectiveSize]{coda::effectiveSize}.  \verb{Pct Diff} gives
\verb{(Sim Mean - Target) / Target} and \verb{Z Score} gives
\verb{(Sim Mean - Target) / Sim SE} when a target is available.  \verb{SD(Sim Means)}
is the standard deviation of per-simulation means, and \code{SD(Statistic)} is
the overall standard deviation of the statistic.

\emph{Formation statistics}: each row corresponds to a network statistic from the
formation formula (e.g., \code{edges}, \code{nodematch}), compared against the target
statistics from network estimation.  These statistics assess whether the
network structure is maintained at target levels during the epidemic
simulation (which is particularly important in open-population models where
demographic turnover can shift network structure).

\emph{Duration statistics}: each row corresponds to a dissolution model dyad
type.  In a homogeneous dissolution model (\code{~offset(edges)}), all dyads are
of the \code{edges} type.  The statistic of interest is the mean age of extant
edges, compared against the target duration from \code{\link[=dissolution_coefs]{dissolution_coefs()}}.

\emph{Dissolution statistics}: same row structure as the duration table.  The
statistic of interest is the edge dissolution rate (proportion of edges
dissolving per time step), compared against \verb{1 / target duration}.

Duration and dissolution tables are only available when
\code{control$save.diss.stats = TRUE}, \code{control$save.network = TRUE},
\code{control$tergmLite = FALSE}, and the dissolution formula is
\code{~offset(edges)}.  When these conditions are not met, a note listing the
requirements is printed instead.
}
\seealso{
\code{\link[=netsim]{netsim()}}, \code{\link[=print.netdx]{print.netdx()}}, \code{\link[=summary.netsim]{summary.netsim()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netest.R
\name{netest}
\alias{netest}
\title{Dynamic Network Model Estimation}
\usage{
netest(
  nw,
  formation,
  target.stats,
  coef.diss,
  constraints = NULL,
  coef.form = NULL,
  edapprox = TRUE,
  set.control.ergm = control.ergm(),
  set.control.tergm = control.tergm(MCMC.maxchanges = .Machine$integer.max),
  set.control.ergm.ego = NULL,
  verbose = FALSE,
  nested.edapprox = TRUE,
  ...
)
}
\arguments{
\item{nw}{An object of class \code{network} or \code{egor}, with the latter
indicating an \code{ergm.ego} fit.}

\item{formation}{Right-hand sided STERGM formation formula in the form
\code{~edges + ...}, where \code{...} are additional network statistics
(ERGM terms). This formula specifies which structural features of
the network should be reproduced by the model. Common terms include
\code{edges} (overall connectivity), \code{nodematch} (homophily by attribute),
\code{concurrent} (overlapping partnerships), and \code{degree} (degree
distribution constraints). See \code{\link[ergm:ergm-terms]{ergm::ergm-terms}} for the full
list of available terms.}

\item{target.stats}{Vector of target statistics for the formation model, with
one number for each network statistic in the model. These are the
observed (or desired) values for each term in the formation formula.
For example, if \code{formation = ~edges + concurrent}, then
\code{target.stats = c(175, 40)} means the model should produce
approximately 175 edges and 40 nodes with 2 or more partners. For
an \code{edges}-only model, a useful starting value is
\code{mean_degree * network_size / 2}. Ignored if fitting via \code{ergm.ego}.}

\item{coef.diss}{An object of class \code{disscoef} output from the
\code{\link{dissolution_coefs}} function. This encodes the average partnership
duration(s) and the corresponding dissolution model coefficients.
For models with vital dynamics (arrivals and departures), the
\code{d.rate} argument in \code{\link{dissolution_coefs}} should be set to adjust
for the competing risk of node departure.}

\item{constraints}{Right-hand sided formula specifying constraints for the
modeled network, in the form \code{~...}, where \code{...} are
constraint terms. By default, no constraints are set.}

\item{coef.form}{Vector of coefficients for the offset terms in the formation
formula.}

\item{edapprox}{If \code{TRUE}, use the indirect edges dissolution
approximation method for the dynamic model fit, otherwise use the
more time-intensive full STERGM estimation (see details). The
approximation is recommended for most use cases, especially when
average partnership durations are moderate to long (> 25 time steps).
Direct STERGM estimation (\code{edapprox = FALSE}) is slower but may be
preferred for very short durations or when inferential quantities
(standard errors, p-values) on formation coefficients are needed.
For \code{nw} of class \code{egor}, only \code{edapprox = TRUE} is supported.}

\item{set.control.ergm}{Control arguments passed to \code{ergm} (see details).}

\item{set.control.tergm}{Control arguments passed to \code{tergm} (see details).}

\item{set.control.ergm.ego}{Control arguments passed to \code{ergm.ego} (see
details).}

\item{verbose}{If \code{TRUE}, print model fitting progress to console.}

\item{nested.edapprox}{Logical. If \code{edapprox = TRUE} the dissolution
model is an initial segment of the formation model (see details).}

\item{...}{Additional arguments passed to other functions.}
}
\value{
A fitted network model object of class \code{netest}. This object is
passed to \code{\link{netdx}} for diagnostics and to \code{\link{netsim}} for epidemic
simulation. Use \code{print()} to view the model form, including the
formation formula, target statistics, and dissolution model. Use
\code{summary()} to view the estimated model coefficients and goodness-of-fit
statistics from the underlying ERGM or STERGM fit.
}
\description{
Estimates statistical network models using the exponential
random graph modeling (ERGM) framework with extensions for
dynamic/temporal models (STERGM). This is typically the first
step in the network modeling pipeline, followed by \code{\link{netdx}}
for model diagnostics and \code{\link{netsim}} for epidemic simulation.
}
\details{
\code{netest} is a wrapper function for the \code{ergm}, \code{ergm.ego}, and \code{tergm}
functions that estimate static and dynamic network models. Network model
estimation is the first step in simulating a stochastic network epidemic model
in \code{EpiModel}. The output from \code{netest} is a necessary input for running the
epidemic simulations in \code{\link{netsim}}. With a fitted network model, one should
always first proceed to model diagnostics, available through the \code{\link{netdx}}
function, to check model fit. A detailed description of fitting these
models, along with examples, may be found in the
\href{https://epimodel.github.io/sismid/}{Network Modeling for Epidemics}
tutorials.
}
\section{Edges Dissolution Approximation}{

The edges dissolution approximation method is described in Carnegie et al.
This approximation requires that the dissolution coefficients are known, that
the formation model is being fit to cross-sectional data conditional on those
dissolution coefficients, and that the terms in the dissolution model are a
subset of those in the formation model. Under certain additional conditions,
the formation coefficients of a STERGM model are approximately equal to the
coefficients of that same model fit to the observed cross-sectional data as
an ERGM, minus the corresponding coefficients in the dissolution model. The
approximation thus estimates this ERGM (which is typically much faster than
estimating a STERGM) and subtracts the dissolution coefficients.

The conditions under which this approximation best hold are when there are
few relational changes from one time step to another; i.e. when either
average relational durations are long, or density is low, or both.
Conveniently, these are the same conditions under which STERGM estimation is
slowest. Note that the same approximation is also used to obtain starting
values for the STERGM estimate when the latter is being conducted. The
estimation does not allow for calculation of standard errors, p-values, or
likelihood for the formation model; thus, this approach is of most use when
the main goal of estimation is to drive dynamic network simulations rather
than to conduct inference on the formation model. The user is strongly
encouraged to examine the behavior of the resulting simulations to confirm
that the approximation is adequate for their purposes. For an example, see
the vignette for the package \code{tergm}.

It has recently been found that subtracting a modified version of the
dissolution coefficients from the formation coefficients provides a more
principled approximation, and this is now the form of the approximation
applied by \code{netest}. The modified values subtracted from the formation
coefficients are equivalent to the (crude) dissolution coefficients with
their target durations increased by 1. The \code{nested.edapprox} argument
toggles whether to implement this modified version by appending the
dissolution terms to the formation model and appending the relevant values to
the vector of formation model coefficients (value = \code{FALSE}), whereas
the standard version subtracts the relevant values from the initial formation
model coefficients (value = \code{TRUE}).
}

\section{Control Arguments}{

The \code{ergm}, \code{ergm.ego}, and \code{tergm} functions allow control settings for the
model fitting process. When fitting a STERGM directly (setting
\code{edapprox} to \code{FALSE}), control parameters may be passed to the
\code{tergm} function with the \code{set.control.tergm} argument in \code{netest}.
The controls should be input through the \code{control.tergm()} function,
with the available parameters listed in the \code{\link[tergm:control.tergm]{tergm::control.tergm}} help
page in the \code{tergm} package.

When fitting a STERGM indirectly (setting \code{edapprox} to \code{TRUE}), control
settings may be passed to the \code{ergm} function using \code{set.control.ergm},
or to the \code{ergm.ego} function using \code{set.control.ergm.ego}. The controls
should be input through the \code{control.ergm()} and \code{control.ergm.ego()}
functions, respectively, with the available parameters listed in the
\code{\link[ergm:control.ergm]{ergm::control.ergm}} help page in the \code{ergm} package and the
\code{\link[ergm.ego:control.ergm.ego]{ergm.ego::control.ergm.ego}} help page in the \code{ergm.ego}
package. An example is below.
}

\section{Typical Workflow}{

The network modeling pipeline in EpiModel typically follows these steps:
\enumerate{
\item Initialize a network: \code{\link{network_initialize}}
\item Specify formation and dissolution: a formation formula (e.g.,
\code{~edges + concurrent}) with target statistics, and dissolution
coefficients via \code{\link{dissolution_coefs}}
\item Estimate the network model: \code{netest()}
\item Diagnose model fit: \code{\link{netdx}}
\item Simulate the epidemic: \code{\link{netsim}} with \code{\link{param.net}}, \code{\link{init.net}},
and \code{\link{control.net}}
}
}

\examples{
# Initialize a network of 100 nodes
nw <- network_initialize(n = 100)

# Set formation formula
formation <- ~edges + concurrent

# Set target statistics for formation
target.stats <- c(50, 25)

# Obtain the offset coefficients
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)

# Estimate the STERGM using the edges dissolution approximation
est <- netest(nw, formation, target.stats, coef.diss,
              set.control.ergm = control.ergm(MCMC.burnin = 1e5,
                                              MCMC.interval = 1000))
# View the model form (formation, targets, dissolution)
est

# View the estimated coefficients
summary(est)

\dontrun{
# Model with homophily on a nodal attribute
nw2 <- network_initialize(n = 500)
nw2 <- set_vertex_attribute(nw2, "risk", rep(0:1, each = 250))
formation2 <- ~edges + nodematch("risk")
target.stats2 <- c(175, 110)
coef.diss2 <- dissolution_coefs(dissolution = ~offset(edges), duration = 50)
est2 <- netest(nw2, formation2, target.stats2, coef.diss2)
est2

# Direct STERGM estimation (slower, for short durations or inference)
est3 <- netest(nw, formation, target.stats, coef.diss, edapprox = FALSE)
}

}
\references{
Krivitsky PN, Handcock MS. "A Separable Model for Dynamic Networks." JRSS(B).
2014; 76.1: 29-46.

Carnegie NB, Krivitsky PN, Hunter DR, Goodreau SM. An Approximation Method
for Improving Dynamic Network Model Fitting. Journal of Computational and
Graphical Statistics. 2014; 24(2): 502-519.

Jenness SM, Goodreau SM and Morris M. EpiModel: An R Package for Mathematical
Modeling of Infectious Disease over Networks. Journal of Statistical
Software. 2018; 84(8): 1-47.
}
\seealso{
Use \code{\link{dissolution_coefs}} to compute dissolution coefficients
before estimation. Use \code{\link{netdx}} to diagnose the fitted network
model, and \code{\link{netsim}} to simulate epidemic spread over a simulated
dynamic network consistent with the model fit. Parameterize the
epidemic simulation with \code{\link{param.net}}, \code{\link{init.net}}, and
\code{\link{control.net}}.
}
\keyword{model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.helpers.R
\name{get_current_timestep}
\alias{get_current_timestep}
\title{Return the Current Timestep}
\usage{
get_current_timestep(dat)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}
}
\value{
The current timestep.
}
\description{
Return the Current Timestep
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.netdx.R
\name{plot.netdx}
\alias{plot.netdx}
\title{Plot Dynamic Network Model Diagnostics}
\usage{
\method{plot}{netdx}(
  x,
  type = "formation",
  method = "l",
  sims = NULL,
  stats = NULL,
  duration.imputed = TRUE,
  sim.lines = FALSE,
  sim.col = NULL,
  sim.lwd = NULL,
  mean.line = TRUE,
  mean.smooth = TRUE,
  mean.col = NULL,
  mean.lwd = 2,
  mean.lty = 1,
  qnts = 0.5,
  qnts.col = NULL,
  qnts.alpha = 0.5,
  qnts.smooth = TRUE,
  targ.line = TRUE,
  targ.col = NULL,
  targ.lwd = 2,
  targ.lty = 2,
  plots.joined = NULL,
  legend = NULL,
  grid = FALSE,
  xlim = NULL,
  xlab = NULL,
  ylim = NULL,
  ylab = NULL,
  ...
)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{netdx}.}

\item{type}{Plot type, with options of \code{"formation"} for network
model formation statistics, \code{"duration"} for dissolution model
statistics for average edge duration, or \code{"dissolution"} for
dissolution model statistics for proportion of ties dissolved per time
step.}

\item{method}{Plot method, with options of \code{"l"} for line plots and
\code{"b"} for box plots.}

\item{sims}{A vector of simulation numbers to plot.}

\item{stats}{Statistics to plot. For \code{type = "formation"}, \code{stats}
are among those specified in the call to \code{\link[=netdx]{netdx()}};
for \verb{type = "duration", "dissolution"}, \code{stats} are among
those of the dissolution model (without \code{offset()}). The default
is to plot all statistics.}

\item{duration.imputed}{If \code{type = "duration"}, a logical indicating
whether or not to impute starting times for relationships extant at
the start of the simulation. Defaults to \code{TRUE} when
\code{type = "duration"}.}

\item{sim.lines}{If \code{TRUE}, plot individual simulation lines. Default is
to plot lines for one-group models but not for two-group models.}

\item{sim.col}{Vector of any standard R color format for simulation lines.}

\item{sim.lwd}{Line width for simulation lines.}

\item{mean.line}{If \code{TRUE}, plot mean of simulations across time.}

\item{mean.smooth}{If \code{TRUE}, use a loess smoother on the mean line.}

\item{mean.col}{Vector of any standard R color format for mean lines.}

\item{mean.lwd}{Line width for mean lines.}

\item{mean.lty}{Line type for mean lines.}

\item{qnts}{If numeric, plot polygon of simulation quantiles based on the
range implied by the argument (see details). If \code{FALSE}, suppress
polygon from plot.}

\item{qnts.col}{Vector of any standard R color format for polygons.}

\item{qnts.alpha}{Transparency level for quantile polygons, where 0 =
transparent and 1 = opaque (see \code{adjustcolor} function).}

\item{qnts.smooth}{If \code{TRUE}, use a loess smoother on quantile polygons.}

\item{targ.line}{If \code{TRUE}, plot target or expected value line for
the statistic of interest.}

\item{targ.col}{Vector of standard R colors for target statistic lines, with
default colors based on \code{RColorBrewer} color palettes.}

\item{targ.lwd}{Line width for the line showing the target statistic values.}

\item{targ.lty}{Line type for the line showing the target statistic values.}

\item{plots.joined}{If \code{TRUE}, combine all statistics in one plot,
versus one plot per statistic if \code{FALSE}.}

\item{legend}{If \code{TRUE}, plot default legend.}

\item{grid}{If \code{TRUE}, a grid is added to the background of plot
(see \code{\link[=grid]{grid()}} for details), with default of nx by ny.}

\item{xlim}{the x limits (x1, x2) of the plot.  Note that \code{x1 > x2}
    is allowed and leads to a \sQuote{reversed axis}.

    The default value, \code{NULL}, indicates that the range of the
    \link{finite} values to be plotted should be used.}

\item{xlab}{a label for the x axis, defaults to a description of \code{x}.}

\item{ylim}{the y limits of the plot.}

\item{ylab}{a label for the y axis, defaults to a description of \code{y}.}

\item{...}{Additional arguments to pass.}
}
\description{
Plots dynamic network model diagnostics calculated in
\code{\link[=netdx]{netdx()}}.
}
\details{
The plot function for \code{netdx} objects will generate plots of two types
of model diagnostic statistics that run as part of the diagnostic tools
within that function. The \code{formation} plot shows the summary statistics
requested in \code{nwstats.formula}, where the default includes those
statistics in the network model formation formula specified in the original
call to \code{\link[=netest]{netest()}}.

The \code{duration} plot shows the average age of existing edges at each time
step, up until the maximum time step requested. The age is used as an
estimator of the average duration of edges in the equilibrium state. When
\code{duration.imputed = FALSE}, edges that exist at the beginning of the
simulation are assumed to start with an age of 1, yielding a burn-in period
before the observed mean approaches its target.  When
\code{duration.imputed = TRUE}, expected ages prior to the start of the
simulation are calculated from the dissolution model, typically eliminating
the need for a burn-in period.

The \code{dissolution} plot shows the proportion of the extant ties that are
dissolved at each time step, up until the maximum time step requested.
Typically, the proportion of ties that are dissolved is the reciprocal of the
mean relational duration. This plot thus contains similar information to that
in the duration plot, but should reach its expected value more quickly, since
it is not subject to censoring.

The \code{plots.joined} argument will control whether the statistics
are joined in one plot or plotted separately, assuming there are multiple
statistics in the model. The default is based on the number of network
statistics requested. The layout of the separate plots within the larger plot
window is also based on the number of statistics.
}
\examples{
\dontrun{
# Network initialization and model parameterization
nw <- network_initialize(n = 500)
nw <- set_vertex_attribute(nw, "sex", rbinom(500, 1, 0.5))
formation <- ~edges + nodematch("sex")
target.stats <- c(500, 300)
coef.diss <- dissolution_coefs(dissolution = ~offset(edges) +
                  offset(nodematch("sex")), duration = c(50, 40))

# Estimate the model
est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

# Static diagnostics
dx1 <- netdx(est, nsims = 1e4, dynamic = FALSE,
             nwstats.formula = ~edges + meandeg + concurrent +
                                nodefactor("sex", levels = NULL) +
                                nodematch("sex"))
dx1

# Plot diagnostics
plot(dx1)
plot(dx1, stats = c("edges", "concurrent"), mean.col = "black",
     sim.lines = TRUE, plots.joined = FALSE)
plot(dx1, stats = "edges", method = "b",
     col = "seagreen3", grid = TRUE)

# Dynamic diagnostics
dx2 <- netdx(est, nsims = 10, nsteps = 500,
             nwstats.formula = ~edges + meandeg + concurrent +
                                nodefactor("sex", levels = NULL) +
                                nodematch("sex"))
dx2

# Formation statistics plots, joined and separate
plot(dx2, grid = TRUE)
plot(dx2, type = "formation", plots.joined = TRUE)
plot(dx2, type = "formation", sims = 1, plots.joined = TRUE,
     qnts = FALSE, sim.lines = TRUE, mean.line = FALSE)
plot(dx2, type = "formation", plots.joined = FALSE,
     stats = c("edges", "concurrent"), grid = TRUE)

plot(dx2, method = "b", col = "bisque", grid = TRUE)
plot(dx2, method = "b", stats = "meandeg", col = "dodgerblue")

# Duration statistics plot
par(mfrow = c(1, 2))
# With duration imputed
plot(dx2, type = "duration", sim.line = TRUE, sim.lwd = 0.3,
     targ.lty = 1, targ.lwd = 0.5)
# Without duration imputed
plot(dx2, type = "duration", sim.line = TRUE, sim.lwd = 0.3,
     targ.lty = 1, targ.lwd = 0.5, duration.imputed = FALSE)

# Dissolution statistics plot
plot(dx2, type = "dissolution", qnts = 0.25, grid = TRUE)
plot(dx2, type = "dissolution", method = "b", col = "pink1")
}

}
\seealso{
\code{\link[=netdx]{netdx()}}
}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.R
\name{set_network}
\alias{set_network}
\alias{set_network.netsim_dat}
\title{Set Network State During netsim Simulation}
\usage{
set_network(x, ...)

\method{set_network}{netsim_dat}(x, network = 1L, nw, ...)
}
\arguments{
\item{x}{a \code{netsim_dat} object}

\item{network}{the index of the network to set on \code{x}}

\item{nw}{the value of the network to set on \code{x}}
}
\value{
the \code{netsim_dat} object with the network state updated
}
\description{
This function updates the \code{netsim_dat} object given a
network representing the current state of the simulation.
}
\details{
If running \code{tergmLite} simulation, this function updates
\code{x$el[[network]]} and (if \code{tergmLite.track.duration} is \code{TRUE}
for the network index \code{network}) the network attributes \code{"time"}
and \code{"lasttoggle"} in \code{x$net_attr[[network]]}. If not running
\code{tergmLite} simulation, this function updates the \code{networkDynamic}
object stored in \code{x$nw[[network]]}. The input \code{nw} should be of
class \code{networkLite} when running \code{tergmLite} simulation, and of
class \code{networkDynamic} when not running \code{tergmLite} simulation.
}
\keyword{internal}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.R
\name{as.data.frame.dcm}
\alias{as.data.frame.dcm}
\title{Extract Model Data for Deterministic Compartmental Models}
\usage{
\method{as.data.frame}{dcm}(x, row.names = NULL, optional = FALSE, run = NULL, ...)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{\link[=dcm]{dcm()}}.}

\item{row.names}{See \code{\link[=as.data.frame.default]{as.data.frame.default()}}.}

\item{optional}{See \code{\link[=as.data.frame.default]{as.data.frame.default()}}.}

\item{run}{Run number for model; used for multiple-run sensitivity models. If
\code{NULL} (default), will output data from all runs in a stacked data frame.}

\item{...}{See \code{\link[=as.data.frame.default]{as.data.frame.default()}}.}
}
\value{
A data frame containing the data from \code{x}.
}
\description{
This function extracts a model run from an object of class
\code{dcm} into a data frame using the generic
\code{as.data.frame} function.
}
\details{
Model output from \code{\link[=dcm]{dcm()}} simulations are available as a data
frame with this helper function. The output data frame will include
columns for time, the size of each compartment, the overall population
size (the sum of compartment sizes), and the size of each flow.

For models with multiple runs (i.e., varying parameters - see example below),
the default with the \code{run} parameter not specified will output all runs
in a single stacked data frame.
}
\examples{
## Example 1: One-group SIS model with varying act.rate
param <- param.dcm(inf.prob = 0.2, act.rate = seq(0.05, 0.5, 0.05),
                   rec.rate = 1/50)
init <- init.dcm(s.num = 500, i.num = 1)
control <- control.dcm(type = "SIS", nsteps = 10)
mod1 <- dcm(param, init, control)
as.data.frame(mod1)
as.data.frame(mod1, run = 1)
as.data.frame(mod1, run = 10)

## Example 2: Two-group SIR model with vital dynamics
param <- param.dcm(inf.prob = 0.2, inf.prob.g2 = 0.1,
                   act.rate = 3, balance = "g1",
                   rec.rate = 1/50, rec.rate.g2 = 1/50,
                   a.rate = 1/100, a.rate.g2 = NA,
                   ds.rate = 1/100, ds.rate.g2 = 1/100,
                   di.rate = 1/90, di.rate.g2 = 1/90,
                   dr.rate = 1/100, dr.rate.g2 = 1/100)
init <- init.dcm(s.num = 500, i.num = 1, r.num = 0,
                 s.num.g2 = 500, i.num.g2 = 1, r.num.g2 = 0)
control <- control.dcm(type = "SIR", nsteps = 10)
mod2 <- dcm(param, init, control)
as.data.frame(mod2)

}
\keyword{extract}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{get_attr_prop}
\alias{get_attr_prop}
\title{Proportional Table of Vertex Attributes}
\usage{
get_attr_prop(dat, nwterms)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{nwterms}{Vector of attributes on the network object, usually as
output of \code{\link[=get_formula_term_attr]{get_formula_term_attr()}}.}
}
\value{
A table containing the proportional distribution of each attribute in
\code{nwterms}.
}
\description{
Calculates the proportional distribution of each vertex
attribute contained in a network.
}
\seealso{
\code{\link[=get_formula_term_attr]{get_formula_term_attr()}},
\code{\link[=copy_nwattr_to_datattr]{copy_nwattr_to_datattr()}},
\code{\link[=auto_update_attr]{auto_update_attr()}}.
}
\keyword{internal}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.dcm}
\alias{summary.dcm}
\title{Summary Model Statistics}
\usage{
\method{summary}{dcm}(object, at, run = 1, digits = 3, ...)
}
\arguments{
\item{object}{An \code{EpiModel} object of class \code{dcm}.}

\item{at}{Time step for model statistics.}

\item{run}{Model run number, for \code{dcm} class models with multiple runs
(sensitivity analyses).}

\item{digits}{Number of significant digits to print.}

\item{...}{Additional summary function arguments (not used).}
}
\description{
Extracts and prints model statistics solved with \code{dcm}.
}
\details{
This function provides summary statistics for the main epidemiological
outcomes (state and transition size and prevalence) from a \code{dcm} model.
Time-specific summary measures are provided, so it is necessary to input a
time of interest. For multiple-run models (sensitivity analyses), input a
model run number. See examples below.
}
\examples{
## Deterministic SIR model with varying act.rate
param <- param.dcm(inf.prob = 0.2, act.rate = 2:4, rec.rate = 1/3,
                   a.rate = 0.011, ds.rate = 0.01,
                   di.rate = 0.03, dr.rate = 0.01)
init <- init.dcm(s.num = 1000, i.num = 1, r.num = 0)
control <- control.dcm(type = "SIR", nsteps = 50)
mod <- dcm(param, init, control)
summary(mod, at = 25, run = 1)
summary(mod, at = 25, run = 3)
summary(mod, at = 26, run = 3)

}
\seealso{
\code{\link[=dcm]{dcm()}}
}
\keyword{extract}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{apportion_lr}
\alias{apportion_lr}
\title{Apportion Using the Largest Remainder Method}
\usage{
apportion_lr(vector.length, values, proportions, shuffled = FALSE)
}
\arguments{
\item{vector.length}{Length for the output vector.}

\item{values}{Values for the output vector.}

\item{proportions}{Proportion distribution with one number for each value.
This must sum to 1.}

\item{shuffled}{If \code{TRUE}, randomly shuffle the order of the vector.}
}
\value{
A vector of length \code{vector.length} containing the apportioned
values from \code{values}.
}
\description{
Apportions a vector of values given a specified frequency
distribution of those values such that the length of the output
is robust to rounding and other instabilities.
}
\examples{
\dontrun{
## Example 1: Without rounding
apportioned_vec_1 <- apportion_lr(4, c(1, 2, 3, 4, 5),
                                     c(0.25, 0, 0.25, 0.25, 0.25))

## Example 2: With rounding
apportioned_vec_2 <- apportion_lr(5, c(1, 2, 3, 4, 5),
                                     c(0.21, 0, 0.29, 0.25, 0.25))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icm.mod.init.R
\name{initialize.icm}
\alias{initialize.icm}
\title{Initialization: icm Module}
\usage{
initialize.icm(param, init, control)
}
\arguments{
\item{param}{An \code{EpiModel} object of class \code{\link[=param.icm]{param.icm()}}.}

\item{init}{An \code{EpiModel} object of class \code{\link[=init.icm]{init.icm()}}.}

\item{control}{An \code{EpiModel} object of class \code{\link[=control.icm]{control.icm()}}.}
}
\value{
The updated \code{icm_dat} class main data object.
}
\description{
This function initializes the main \code{icm_dat} class data
object, and simulates disease status and other attributes.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny.R
\name{epiweb}
\alias{epiweb}
\title{EpiModel Web}
\usage{
epiweb(class, ...)
}
\arguments{
\item{class}{Model class, with options of \code{"dcm"} and \code{"icm"}.}

\item{...}{Additional arguments passed to \link[shiny:runApp]{shiny::runApp}, such as
\code{port}, \code{host}, or \code{launch.browser}.}
}
\description{
Launches interactive Shiny applications for exploring
deterministic compartmental models (DCMs) and stochastic
individual contact models (ICMs) of infectious disease
transmission.
}
\details{
\code{epiweb} launches a web-based graphical interface for configuring, running,
and analyzing epidemic models built on EpiModel. Each application provides
interactive controls for model parameters, real-time (or on-demand)
visualization with plotly, narrative model summaries, and downloadable
data tables. Both applications include a \strong{Guide} tab with a comprehensive
user manual covering the underlying theory, parameter definitions, and
instructions for interpreting output.
}
\section{DCM Application (\code{class = "dcm"})}{

The DCM app solves systems of ordinary differential equations for one-group
SI, SIR, and SIS models. Models run instantly and the interface is fully
reactive (no run button). Features include:
\itemize{
\item \strong{Scenario presets:} Flu-like (SIR), STI-like (SIS), and
Measles-like (SIR) configurations that set all parameters to
epidemiologically plausible values.
\item \strong{Interventions:} An optional mid-epidemic intervention that
reduces the transmission probability by a user-specified efficacy,
shown as a vertical dashed line on the plot.
\item \strong{Sensitivity analysis:} Vary any epidemic parameter across a
range of values to compare trajectories side by side with a
blue-to-red color ramp.
\item \strong{Vital dynamics:} Optional births and deaths to model endemic
equilibria over longer time horizons.
\item \strong{Summary tab:} Displays the basic reproduction number
(R\ifelse{html}{\out{<sub>0</sub>}}{0}), peak timing, cumulative
infections, attack rate, and contextual interpretation.
\item \strong{Data tab:} Searchable table of model output with CSV download.
}
}

\section{ICM Application (\code{class = "icm"})}{

The ICM app runs agent-based stochastic microsimulations for one-group
SI, SIR, and SIS models. Because ICMs are computationally more expensive,
a \strong{Run Model} button controls when simulations execute. Features include:
\itemize{
\item \strong{Stochastic visualization:} Multiple simulations are summarized
with mean trajectory lines and interquartile range (IQR) ribbons
showing the middle 50\\% of outcomes. Single simulations display
individual trajectories.
\item \strong{Scenario presets:} The same Flu-like, STI-like, and Measles-like
configurations as the DCM app, adapted for smaller populations suitable
for agent-based simulation.
\item \strong{Interventions:} Same intervention system as DCM, natively
supported by \link{param.icm}.
\item \strong{Vital dynamics:} Optional births and deaths, as in DCM.
\item \strong{Summary tab:} Displays R\ifelse{html}{\out{<sub>0</sub>}}{0},
epidemic timeline from the mean trajectory, and a stochastic variation
section showing the range and standard deviation of peak infections and
final prevalence across simulations.
\item \strong{Data tab:} View simulation output as means, standard deviations,
or individual simulation values, with CSV download.
}
}

\section{Required Packages}{

Both applications require the \code{shiny}, \code{bslib}, \code{DT}, and \code{plotly} packages.
Install any missing packages with
\code{install.packages(c("shiny", "bslib", "DT", "plotly"))}.
}

\examples{
\dontrun{
## Deterministic compartmental models
epiweb(class = "dcm")

## Stochastic individual contact models
epiweb(class = "icm")
}

}
\seealso{
\link{dcm} and \link{param.dcm} for the DCM modeling API;
\link{icm} and \link{param.icm} for the ICM modeling API.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.end.horizon.R
\name{check_end_horizon_control}
\alias{check_end_horizon_control}
\title{Check the format of the end.horizon control}
\usage{
check_end_horizon_control(dat)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}
}
\value{
TRUE invisibly on succes, errors otherwise
}
\description{
Check the format of the end.horizon control
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.trackers.R
\name{epi_trackers}
\alias{epi_trackers}
\title{Function to run the user-provided epi trackers}
\usage{
epi_trackers(dat)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
see the "Working with Custom Attributes and Summary Statistics in EpiModel"
vignette.
}
\section{The \code{tracker.list} list}{

\code{.tracker.list} is a list of NAMED functions stored in the
\code{control} list of the main \code{netsim_dat} class object.
}

\section{Tracker Functions}{

This function will apply the tracker functions present in the control list
\code{.tracker.list}. Each tracker must be a function with EXACTLY one
argument: the \code{netsim_dat} main list object. They must return a VALUE of
length one (numeric, logical or character).
}

\examples{
\dontrun{

# Create some trackers
epi_prop_infected <- function(dat) {
  # we need two attributes for our calculation: `status` and `active`
  needed_attributes <- c("status", "active")
  # we use `with` to simplify code
  output <- with(EpiModel::get_attr_list(dat, needed_attributes), {
    pop <- active == 1             # we only look at active nodes
    cond <- status == "i"   # which are infected
    # how many are `infected` among the `active`
    sum(cond & pop, na.rm = TRUE) / sum(pop, na.rm = TRUE)
  })
  return(output)
}

epi_s_num <- function(dat) {
  needed_attributes <- c("status")
  output <- with(get_attr_list(dat, needed_attributes), {
    sum(status == "s", na.rm = TRUE)
  })
  return(output)
}

# Store the trackers in a named list. The names will be used as column names
# for in the `epi` list
some.trackers <- list(
  prop_infected = epi_prop_infected,
  s_num         = epi_s_num
)

# Make a simple SI model with custom trackers
control <- EpiModel::control.net(
  type = "SI",
  nsims = 1,
  nsteps = 50,
  verbose = FALSE,
  .tracker.list = some.trackers
)

param <- EpiModel::param.net(
  inf.prob = 0.3,
  act.rate = 0.1
)

nw <- network_initialize(n = 50)
nw <- set_vertex_attribute(nw, "race", rbinom(50, 1, 0.5))
est <- EpiModel::netest(
  nw,
  formation = ~edges,
  target.stats = 25,
  coef.diss = dissolution_coefs(~offset(edges), 10, 0),
  verbose = FALSE
)

init <- EpiModel::init.net(i.num = 10)
sim <- EpiModel::netsim(est, param, init, control)

d <- as.data.frame(sim)
d
}

}
\seealso{
\code{\link[=netsim]{netsim()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.param_df.R
\name{param.net_from_table}
\alias{param.net_from_table}
\title{Parameters List for Stochastic Network Models from a Formatted
Data Frame}
\usage{
param.net_from_table(long.param.df)
}
\arguments{
\item{long.param.df}{A \code{data.frame} of parameters. See details for the
expected format.}
}
\value{
A list object of class \code{param.net}, which can be passed to
\code{\link[=netsim]{netsim()}}.
}
\description{
Sets the epidemic parameters for stochastic network models with
\code{\link[=netsim]{netsim()}} using a specially formatted data frame of
parameters.
}
\section{long.param.df}{

It is possible to set input parameters using a specifically formatted
\code{data.frame} object. The first 3 columns of this \code{data.frame} must
be:
\itemize{
\item \code{param}: The name of the parameter. If this is a non-scalar
parameter (a vector of length > 1), end the parameter name with the
position on the vector (e.g., \code{"p_1"}, \code{"p_2"}, ...).
\item \code{value}: the value for the parameter (or the value of the
parameter in the Nth position if non-scalar).
\item \code{type}: a character string containing either \code{"numeric"},
\code{"logical"}, or \code{"character"} to define the parameter object
class.
}

In addition to these 3 columns, the \code{data.frame} can contain any number
of other columns, such as \code{details} or \code{source} columns to document
parameter meta-data. However, these extra columns will not be used by
EpiModel.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.accessor.R
\name{check_attr_lengths}
\alias{check_attr_lengths}
\title{Check that All Attributes in the Main \code{netsim_dat} Object are of Equal
Length}
\usage{
check_attr_lengths(dat)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}
}
\value{
invisible(TRUE) if everything is correct; an error if not.
}
\description{
Check that All Attributes in the Main \code{netsim_dat} Object are of Equal
Length
}
\keyword{internal}
\keyword{not_used}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modules.icm.R
\name{modules.icm}
\alias{modules.icm}
\title{Modules for Stochastic Individual Contact Models}
\description{
Stochastic individual contact models of infectious disease simulate epidemics
in which contacts between individuals are instantaneous events in discrete
time. They are intended to be the stochastic microsimulation analogs to
deterministic compartmental models.

The \code{\link[=icm]{icm()}} function handles both the simulation tasks. Within this
function are a series of modules that initialize the simulation and then
simulate new infections, recoveries, and vital dynamics at each time step. A
module also handles the basic bookkeeping calculations for disease
prevalence.

This help page presents a brief overview of the built-in module functions
in the order in which they are used within \code{\link[=icm]{icm()}}. ICMs support only the
built-in SI, SIR, and SIS disease types. For custom or extension epidemic
models, use the network model class via \code{\link[=netsim]{netsim()}} and \code{\link[=control.net]{control.net()}}
instead.
}
\section{Initialization Module}{

This function sets up agent attributes, like disease status, on the network
at the starting time step of disease simulation, \eqn{t_1}. For
multiple-simulation function calls, these are reset at the beginning of each
simulation.
\itemize{
\item \code{\link[=initialize.icm]{initialize.icm()}}: sets which agents are initially
infected, through the initial conditions passed in
\code{\link[=init.icm]{init.icm()}}.
}
}

\section{Disease Status Modification Modules}{

The main disease simulation occurs at each time step given the current state
of the population at that step. Infection of agents is simulated as a
function of disease parameters and population composition. Recovery of agents
is likewise simulated with respect to infected nodes. These functions also
analyze the flows for summary measures such as disease incidence.
\itemize{
\item \code{\link[=infection.icm]{infection.icm()}}: randomly draws an edgelist given the
parameters, subsets the list for discordant pairs, and simulates
transmission on those discordant pairs through a series of draws from
a binomial distribution.
\item \code{\link[=recovery.icm]{recovery.icm()}}: simulates recovery from infection either
to a lifelong immune state (for SIR models) or back to the susceptible
state (for SIS models), as a function of the recovery rate specified
in the \code{rec.rate} parameter. The recovery rate may vary for
two-group models.
}
}

\section{Demographic Modules}{

Vital dynamics such as arrival and departure processes are simulated at each
time step to update entries into and exits from the population. These are
used in open-population ICMs.
\itemize{
\item \code{\link[=departures.icm]{departures.icm()}}: randomly simulates departures or exits
for agents given the departure rate specified in the disease-state and
group-specific departure parameters in \code{\link[=param.icm]{param.icm()}}. This
involves deactivating agents from the population, but their historical
data is preserved in the simulation.
\item \code{\link[=arrivals.icm]{arrivals.icm()}}: randomly simulates new arrivals into the
population given the current population size and the arrival rate
parameters. This involves adding new agents into the population.
}
}

\section{Bookkeeping Module}{

Simulations require bookkeeping at each time step to calculate the
summary epidemiological statistics used in the model output analysis.
\itemize{
\item \code{\link[=prevalence.icm]{prevalence.icm()}}: calculates the number in each disease
state (susceptible, infected, recovered) at each time step for those
active agents in the population.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icm.inputs.R
\name{crosscheck.icm}
\alias{crosscheck.icm}
\title{Cross Checking of Inputs for Stochastic Individual Contact Models}
\usage{
crosscheck.icm(param, init, control)
}
\arguments{
\item{param}{An \code{EpiModel} object of class \code{\link[=param.icm]{param.icm()}}.}

\item{init}{An \code{EpiModel} object of class \code{\link[=init.icm]{init.icm()}}.}

\item{control}{An \code{EpiModel} object of class \code{\link[=control.icm]{control.icm()}}.}
}
\value{
This function returns no objects.
}
\description{
This function checks that the three parameter lists from
\code{\link[=param.icm]{param.icm()}}, \code{\link[=init.icm]{init.icm()}}, and
\code{\link[=control.icm]{control.icm()}} are consistent.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgelists.R
\name{update_cumulative_edgelist}
\alias{update_cumulative_edgelist}
\title{Update a Cumulative Edgelist of the Specified Network}
\usage{
update_cumulative_edgelist(dat, network, truncate = 0)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{network}{Numerical index of the network for which the cumulative
edgelist will be updated. (May be > 1 for models with
multiple overlapping networks.)}

\item{truncate}{After how many time steps a partnership that is no longer
active should be removed from the output.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Update a Cumulative Edgelist of the Specified Network
}
\section{Truncation}{

To avoid storing a cumulative edgelist too long, the \code{truncate}
parameter defines a number of steps after which an edge that is no longer
active is truncated out of the cumulative edgelist.
When \code{truncate = Inf}, no edges are ever removed. When
\code{truncate = 0}, only the active edges are kept. You may want this
behavior to keep track of the active edges' start step.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{get_vertex_attribute}
\alias{get_vertex_attribute}
\title{Get Vertex Attribute on Network Object}
\usage{
get_vertex_attribute(x, attrname)
}
\arguments{
\item{x}{An object of class network.}

\item{attrname}{The name of the attribute to get.}
}
\value{
Returns a vector of vertex attribute values for the attribute specified
by \code{attrname}.
}
\description{
Gets a vertex attribute from an object of class \code{network}.
This function simplifies the related function in the
\code{network} package.
}
\details{
This function is used in \code{EpiModel} workflows to query vertex
attributes on an initialized empty network object (see
\code{\link[=network_initialize]{network_initialize()}}).
}
\examples{
nw <- network_initialize(100)
nw <- set_vertex_attribute(nw, "age", runif(100, 15, 65))
get_vertex_attribute(nw, "age")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveout.R
\name{saveout.dcm}
\alias{saveout.dcm}
\title{Save dcm Data to Output List Format}
\usage{
saveout.dcm(df, s, param, control, out = NULL)
}
\arguments{
\item{df}{Main object in \code{\link[=dcm]{dcm()}} simulations.}

\item{s}{Current run number.}

\item{param}{Param list set in \code{\link[=param.dcm]{param.dcm()}}.}

\item{control}{Control list set in \code{\link[=control.dcm]{control.dcm()}}.}

\item{out}{Out list passed back in for updating at runs 2+.}
}
\value{
A list with the following elements:
\itemize{
\item \strong{param:} the epidemic parameters passed into the model through
\code{\link[=param.dcm]{param.dcm()}}, with additional parameters added as
necessary.
\item \strong{control:} the control settings passed into the model through
\code{\link[=control.dcm]{control.dcm()}}, with additional controls added as
necessary.
\item \strong{epi:} a list of data frames, one for each epidemiological
output from the model.
}
}
\description{
This function transfers the data from the main \code{df}
object to the output \code{out} object at the end of each
run in \code{\link[=dcm]{dcm()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.R
\name{get_nwparam}
\alias{get_nwparam}
\title{Extract Network Model Parameters}
\usage{
get_nwparam(x, network = 1)
}
\arguments{
\item{x}{Main \code{netsim_dat} data object used in \code{netsim} simulations.}

\item{network}{Network number, for simulations with multiple networks
representing the population.}
}
\description{
Extracts a list of network model parameters saved in the
initialization module.
}
\keyword{extract}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dcm.R
\name{plot.dcm}
\alias{plot.dcm}
\title{Plot Data from a Deterministic Compartmental Epidemic Model}
\usage{
\method{plot}{dcm}(
  x,
  y = NULL,
  popfrac = FALSE,
  run = NULL,
  col = NULL,
  lwd = NULL,
  lty = NULL,
  alpha = 0.9,
  legend = NULL,
  leg.name = NULL,
  leg.cex = 0.8,
  grid = FALSE,
  add = FALSE,
  main = "",
  xlim = NULL,
  ylim = NULL,
  xlab = "Time",
  ylab = NULL,
  ...
)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{dcm}.}

\item{y}{Output compartments or flows from \code{dcm} object to plot.}

\item{popfrac}{If \code{TRUE}, plot prevalence of values rather than numbers
(see details).}

\item{run}{Run number to plot, for models with multiple runs
(default is run 1).}

\item{col}{Color for lines, either specified as a single color in a standard
R color format, or alternatively as a color palette from
\link[RColorBrewer:RColorBrewer]{RColorBrewer::RColorBrewer} (see details).}

\item{lwd}{Line width for output lines.}

\item{lty}{Line type for output lines.}

\item{alpha}{Transparency level for lines, where 0 = transparent and
1 = opaque (see \code{adjustcolor} function).}

\item{legend}{Type of legend to plot. Values are \code{"n"} for no legend,
\code{"full"} for full legend, and \code{"lim"} for limited legend
(see details).}

\item{leg.name}{Character string to use for legend, with the default
determined automatically based on the \code{y} input.}

\item{leg.cex}{Legend scale size.}

\item{grid}{If \code{TRUE}, a grid is added to the background of plot
(see \code{\link[=grid]{grid()}} for details), with default of nx by ny.}

\item{add}{If \code{TRUE}, new plot window is not called and lines are added
to existing plot window.}

\item{main}{a main title for the plot, see also \code{\link[graphics]{title}}.}

\item{xlim}{the x limits (x1, x2) of the plot.  Note that \code{x1 > x2}
    is allowed and leads to a \sQuote{reversed axis}.

    The default value, \code{NULL}, indicates that the range of the
    \link{finite} values to be plotted should be used.}

\item{ylim}{the y limits of the plot.}

\item{xlab}{a label for the x axis, defaults to a description of \code{x}.}

\item{ylab}{a label for the y axis, defaults to a description of \code{y}.}

\item{...}{Additional arguments to pass to main plot window (see
\code{\link[=plot.default]{plot.default()}}).}
}
\description{
Plots epidemiological data from a deterministic compartment
epidemic model solved with \code{\link[=dcm]{dcm()}}.
}
\details{
This function plots epidemiological outcomes from a deterministic
compartmental model solved with \code{\link[=dcm]{dcm()}}. Depending on the number of
model runs (sensitivity analyses) and number of groups, the default plot is
the fractional proportion of each compartment in the model over time. The
specific compartments or flows to plot may be set using the \code{y}
parameter, and in multiple run models the specific run may also be specified.
}
\section{The \code{popfrac} Argument}{

Compartment prevalence is the size of a compartment over some denominator.
To plot the raw numbers from any compartment, use \code{popfrac=FALSE}; this
is the default. The \code{popfrac} parameter calculates
and plots the denominators of all specified compartments using these rules:
\enumerate{
\item for one-group models, the prevalence of any compartment is the compartment
size divided by the total population size; 2) for two-group models, the
prevalence of any compartment is the compartment size divided by the group
size.
}
}

\section{Color Palettes}{

Since \code{\link[=dcm]{dcm()}} supports multiple run sensitivity models, plotting
the results of such models uses a complex color scheme for distinguishing
runs. This is accomplished using the \code{\link[RColorBrewer:RColorBrewer]{RColorBrewer::RColorBrewer}} color
palettes, which include a range of linked colors using named palettes. For
\code{plot.dcm}, one may either specify a brewer color palette listed in
\code{\link[RColorBrewer:brewer.pal.info]{RColorBrewer::brewer.pal.info}}, or, alternatively, a vector of standard R
colors (named, hexidecimal, or positive integers; see \code{\link[=col2rgb]{col2rgb()}}).
}

\section{Plot Legends}{

There are three automatic legend types available, and the legend is
added by default for plots. To turn off the legend, use \code{legend="n"}. To
plot a legend with values for every line in a sensitivity analysis, use
\code{legend="full"}. With models with many runs, this may be visually
overwhelming. In those cases, use \code{legend="lim"} to plot a legend
limited to the highest and lowest values of the varying parameter in the
model. In cases where the default legend names are not helpful, one may
override those names with the \code{leg.name} argument.
}

\examples{
# Deterministic SIR model with varying act rate
param <- param.dcm(inf.prob = 0.2, act.rate = 1:10,
                   rec.rate = 1/3, a.rate = 0.011, ds.rate = 0.01,
                   di.rate = 0.03, dr.rate = 0.01)
init <- init.dcm(s.num = 1000, i.num = 1, r.num = 0)
control <- control.dcm(type = "SIR", nsteps = 100, dt = 0.25)
mod <- dcm(param, init, control)

# Plot disease prevalence by default
plot(mod)

# Plot prevalence of susceptibles
plot(mod, y = "s.num", popfrac = TRUE, col = "Greys")

# Plot number of susceptibles
plot(mod, y = "s.num", popfrac = FALSE, col = "Greys", grid = TRUE)

# Plot multiple runs of multiple compartments together
plot(mod, y = c("s.num", "i.num"),
     run = 5, xlim = c(0, 50), grid = TRUE)
plot(mod, y = c("s.num", "i.num"),
     run = 10, lty = 2, legend = "n", add = TRUE)

}
\seealso{
\code{\link[=dcm]{dcm()}}, \code{\link[RColorBrewer:brewer.pal.info]{RColorBrewer::brewer.pal.info}}
}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icm.utils.R
\name{prevalence.icm.bip}
\alias{prevalence.icm.bip}
\title{Get Epidemic Output from icm Model}
\usage{
prevalence.icm.bip(dat, at)
}
\arguments{
\item{dat}{Main \code{icm_dat} class data object passed through \code{icm}
simulations.}

\item{at}{Current time step.}
}
\value{
The updated \code{icm_dat} class main data object.
}
\description{
This function provides all active model state sizes from
the network at the specified time step, output to a list of
vectors.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{brewer_ramp}
\alias{brewer_ramp}
\title{RColorBrewer Color Ramp for EpiModel Plots}
\usage{
brewer_ramp(n, plt, delete.lights = TRUE)
}
\arguments{
\item{n}{Number of colors to return.}

\item{plt}{\code{RColorBrewer} palette from \code{\link[RColorBrewer:brewer.pal]{RColorBrewer::brewer.pal}}.}

\item{delete.lights}{If TRUE, delete the lightest colors from the color
palette; this helps with plotting in many high-contrast palettes.}
}
\value{
A vector of length equal to \code{n} with a range of color values consistent
with an RColorBrewer color palette.
}
\description{
Returns a vector of colors consistent with a high-brightness set
of colors from an \code{RColorBrewer} palette.
}
\details{
\code{RColorBrewer} provides easy access to helpful color palettes, but the
built-in palettes are limited to the set of colors in the existing palette.
This function expands the palette size to any number of colors by filling
in the gaps. Also, colors within the "div" and "seq" set of palettes whose
colors are very light (close to white) are deleted by default for better
visualization of plots.
}
\examples{
# Shows a 100-color ramp for 4 RColorBrewer palettes
par(mfrow = c(2, 2), mar=c(1, 1, 2, 1))
pals <- c("Spectral", "Greys", "Blues", "Set1")
for (i in seq_along(pals)) {
 plot(1:100, 1:100, type = "n", axes = FALSE, main = pals[i])
 abline(v = 1:100, lwd = 6, col = brewer_ramp(100, pals[i]))
}

}
\seealso{
\link[RColorBrewer:RColorBrewer]{RColorBrewer::RColorBrewer}
}
\keyword{colorUtils}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.accessor.R
\name{padded_vector}
\alias{padded_vector}
\title{Grow a Vector to a Given Size, Padding it With Empty Elements}
\usage{
padded_vector(orig, size)
}
\arguments{
\item{orig}{A vector to grow.}

\item{size}{The final size of the vector.}
}
\value{
A vector of size \code{size} padded with \code{NULL}s or \code{NA}s at the end.
}
\description{
Grow a vector to a given size, padding it with \code{NULL} if \code{orig} is a \code{list}
and with \code{NA} otherwise
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.netest}
\alias{summary.netest}
\title{Summary for Network Model Fit}
\usage{
\method{summary}{netest}(object, ...)
}
\arguments{
\item{object}{An \code{EpiModel} object of class \code{netest}.}

\item{...}{Additional summary function arguments.}
}
\description{
Prints the summary model fit statistics for an ERGM or STERGM
fit.
}
\details{
This function is simply a wrapper function for \code{summary.ergm}.
Additionally, if the edges dissolution approximation was used to fit the
temporal ERGM, then the dissolution coefficient information will be printed.

If the \code{fit} object is attached to the \code{netest} object, then
\code{summary.netest} will call \code{summary} on \code{fit} using the
\code{...} passed to \code{summary.netest}.  Otherwise,
\code{summary.netest} will print the stored summary of the fit generated
in the original \code{netest} call, using the \code{...} passed to
\code{netest}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{network_initialize}
\alias{network_initialize}
\title{Initialize Network Object}
\usage{
network_initialize(n)
}
\arguments{
\item{n}{Network size.}
}
\value{
Returns an object of class \code{network}.
}
\description{
Initialize an undirected network object for use in EpiModel
workflows.
}
\details{
This function is used in \code{EpiModel} workflows to initialize an empty
network object.  The network attributes \code{directed}, \code{bipartite},
\code{hyper}, \code{loops}, and \code{multiple} are set to \code{FALSE}.
}
\examples{
nw <- network_initialize(100)
nw

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netsim.R
\name{netsim}
\alias{netsim}
\title{Stochastic Network Models}
\usage{
netsim(x, param, init, control)
}
\arguments{
\item{x}{If \code{control$start == 1}, either a fitted network model object
of class \code{netest} or a list of such objects. If
\code{control$start > 1}, an object of class \code{netsim}. When
multiple networks are used (multi-layer models), pass a list of
\code{netest} objects, one per network layer; the node sets (including
network size and nodal attributes) are assumed to be the same for
all networks.}

\item{param}{Model parameters, as an object of class \code{\link{param.net}}.
Includes transmission probability (\code{inf.prob}), act rate
(\code{act.rate}), recovery rate (\code{rec.rate}), and demographic rates
for models with vital dynamics. Custom parameters for extended
models may also be passed through \code{param.net}.}

\item{init}{Initial conditions, as an object of class \code{\link{init.net}}.
Specifies the initial number of infected nodes (\code{i.num}) and,
for SIR models, recovered nodes (\code{r.num}). For two-group models,
the corresponding \code{.g2} parameters are also required.}

\item{control}{Control settings, as an object of class \code{\link{control.net}}.
Key settings include \code{type} (disease model: \code{"SI"}, \code{"SIR"}, or
\code{"SIS"}), \code{nsteps} (number of time steps), \code{nsims} (number of
simulations), \code{tergmLite} (lightweight mode for performance), and
\code{resimulate.network} (required for models with vital dynamics).
For extended models, custom module functions are also passed here.}
}
\value{
A list of class \code{netsim} with the following elements:
\itemize{
\item \strong{param:} the epidemic parameters passed into the model through
\code{param}, with additional parameters added as necessary.
\item \strong{control:} the control settings passed into the model through
\code{control}, with additional controls added as necessary.
\item \strong{epi:} a list of data frames, one for each epidemiological
output from the model. Outputs for base models always include the
size of each compartment, as well as flows in, out of, and between
compartments.
\item \strong{stats:} a list containing two sublists, \code{nwstats} for any
network statistics saved in the simulation, and \code{transmat} for
the transmission matrix saved in the simulation. See
\code{\link[=control.net]{control.net()}} for further
details.
\item \strong{network:} a list of lists of \code{networkDynamic} or
\code{networkLite} objects, with one list of objects for each model
simulation.
}

If \code{control$raw.output == TRUE}: A list of the raw (pre-processed)
\code{netsim_dat} objects, for use in simulation continuation.

The \code{epi} data can be extracted as a data frame with
\code{\link[=as.data.frame.netsim]{as.data.frame.netsim()}}, with options for per-simulation values
(\code{out = "vals"}), means (\code{out = "mean"}), standard deviations
(\code{out = "sd"}), or quantiles (\code{out = "qnt"}).
}
\description{
Simulates stochastic network epidemic models for infectious
disease. This is typically the final step in the network
modeling pipeline, after network estimation with \code{\link{netest}}
and model diagnostics with \code{\link{netdx}}.
}
\details{
Stochastic network models explicitly represent phenomena within and across
edges (pairs of nodes that remain connected) over time. This enables edges to
have duration, allowing for repeated transmission-related acts within the
same dyad, specification of edge formation and dissolution rates, control
over the temporal sequencing of multiple edges, and specification of
network-level features. A detailed description of these models, along with
examples, is found in the \href{https://epimodel.github.io/sismid/}{Network Modeling for Epidemics}
course materials.

The \code{netsim} function performs modeling of both the base model types
and original models. Base model types include one-group and two-group models
with disease types for Susceptible-Infected (SI), Susceptible-Infected-Recovered (SIR),
and Susceptible-Infected-Susceptible (SIS).

Original models may be parameterized by writing new process modules that
either take the place of existing modules (for example, disease recovery), or
supplement the set of existing processes with a new one contained in a new
module. This functionality is documented in the
\href{https://epimodel.github.io/sismid/9_extending/mod9-Intro.html}{Extending EpiModel}
section of the \href{https://epimodel.github.io/sismid/}{Network Modeling for Epidemics}
course materials. The list of modules within \code{netsim} available for
modification is listed in \code{\link[=modules.net]{modules.net()}}.
}
\section{Module Pipeline}{

At each time step, \code{netsim} executes a series of modules in sequence.
For base models, the default pipeline is:
\enumerate{
\item \strong{Network resimulation} (\code{\link{resim_nets}}): updates the network
structure (if \code{resimulate.network = TRUE}).
\item \strong{Infection} (\code{\link{infection.net}}): simulates disease transmission
across discordant edges (where one partner is susceptible and the
other is infected).
\item \strong{Recovery} (\code{\link{recovery.net}}): simulates recovery from infection
(SIR and SIS models only).
\item \strong{Departures} (\code{\link{departures.net}}): simulates node exits from the
network (if vital dynamics are enabled).
\item \strong{Arrivals} (\code{\link{arrivals.net}}): simulates new node entries into the
network (if vital dynamics are enabled).
\item \strong{Prevalence} (\code{\link{prevalence.net}}): calculates summary statistics.
}

See \code{\link[=modules.net]{modules.net()}} for full details on each module.
}

\section{Performance and tergmLite}{

Setting \code{tergmLite = TRUE} in \code{\link{control.net}} uses a lightweight network
representation (\code{networkLite}) that is substantially faster than the full
\code{networkDynamic} representation, often by a factor of 20--50x. This is
recommended for large networks or when running many simulations. The
trade-off is that full \code{networkDynamic} objects are not available for
post-hoc analysis; use the cumulative edgelist
(\code{cumulative.edgelist = TRUE}) instead to track partnership histories.

The \code{resimulate.network} control must be set to \code{TRUE} when demographic
processes (arrivals and departures) change the network composition over
time. Without it, the network structure evolves independently of the
epidemic and demographic dynamics.
}

\section{Multi-Network Models}{

For models with multiple overlapping network layers (e.g., sexual and
needle-sharing networks), pass a list of \code{\link{netest}} objects to the \code{x}
argument, one per network layer. Each layer has its own
formation/dissolution dynamics but shares the same node set. See the
\code{multilayer} documentation and \code{test-multinets.R} for examples.
}

\section{Restarting and Checkpointing}{

Simulations can be checkpointed and restarted if interrupted. Set
\code{.checkpoint.steps} and \code{.checkpoint.dir} in \code{\link{control.net}} to enable
automatic checkpointing. To restart a simulation from a prior \code{netsim}
output, pass the \code{netsim} object as \code{x} and set \code{control$start} to one
greater than the final time step of the prior simulation. See the
Checkpointing Simulations section of \code{\link{control.net}} for full details.
}

\examples{
\dontrun{
## Example 1: SI Model without Network Feedback
# Network model estimation
nw <- network_initialize(n = 100)
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

# Epidemic model
param <- param.net(inf.prob = 0.3)
init <- init.net(i.num = 10)
control <- control.net(type = "SI", nsteps = 100, nsims = 5, verbose.int = 0)
mod1 <- netsim(est1, param, init, control)

# Print, plot, and summarize the results
mod1
plot(mod1)
summary(mod1, at = 50)

## Example 2: SIR Model with Network Feedback (Two-Group)
nw <- network_initialize(n = 100)
nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
formation <- ~edges
target.stats <- 50

# Recalculate dissolution coefficient with departure rate
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20,
                               d.rate = 0.0021)

# Reestimate the model with new coefficient
est2 <- netest(nw, formation, target.stats, coef.diss)

# Set parameters to include demographic rates
param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15,
                   rec.rate = 0.02, rec.rate.g2 = 0.02,
                   a.rate = 0.002, a.rate.g2 = NA,
                   ds.rate = 0.001, ds.rate.g2 = 0.001,
                   di.rate = 0.001, di.rate.g2 = 0.001,
                   dr.rate = 0.001, dr.rate.g2 = 0.001)
init <- init.net(i.num = 10, i.num.g2 = 10,
                 r.num = 0, r.num.g2 = 0)
control <- control.net(type = "SIR", nsteps = 100, nsims = 5,
                       resimulate.network = TRUE, tergmLite = TRUE)

# Simulate the model with new network fit
mod2 <- netsim(est2, param, init, control)

# Print, plot, and summarize the results
mod2
plot(mod2)
summary(mod2, at = 40)

## Example 3: Post-Simulation Analysis
# Extract epi data as a data frame
as.data.frame(mod1)
as.data.frame(mod1, out = "mean")

# Extract the transmission matrix from simulation 1
get_transmat(mod1, sim = 1)

# Derive new epi statistics and plot them
mod1 <- mutate_epi(mod1, prev = i.num / num)
plot(mod1, y = "prev")
}

}
\references{
Jenness SM, Goodreau SM and Morris M. EpiModel: An R Package for Mathematical
Modeling of Infectious Disease over Networks. Journal of Statistical
Software. 2018; 84(8): 1-47.
}
\seealso{
Estimate the network model with \code{\link{netest}} and diagnose model
fit with \code{\link{netdx}} before running simulations. Extract model
results with \code{\link[=as.data.frame.netsim]{as.data.frame.netsim()}}. Summarize the
time-specific model results with \code{\link[=summary.netsim]{summary.netsim()}}. Plot the
model results with \code{\link[=plot.netsim]{plot.netsim()}}. Extract the network with
\code{\link{get_network}}, the transmission matrix with \code{\link{get_transmat}},
and derive new epi statistics with \code{\link{mutate_epi}}.
}
\keyword{model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reachable.R
\name{get_subnet_adj_list}
\alias{get_subnet_adj_list}
\title{Return an adjacency list of subnets}
\usage{
get_subnet_adj_list(adj_list)
}
\arguments{
\item{adj_list}{A normal adjacency list}
}
\value{
An adjacency list where only the first node of a subnet contains the
subnet and all other contain only the first node
}
\description{
Return an adjacency list of subnets
}
\details{
A graph with 4 components: 1, 2, 3, 4, and 5 and 6, 7, 8  would yield a list
like so:
1: 2, 3, 4
2: 1
3: 1
4: 1
5: numeric(0)
6: 7, 8
7: 6,
8: 6

This format speeds up the construction of reachable sets on dense networks
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dcm.inputs.R
\name{init.dcm}
\alias{init.dcm}
\title{Initial Conditions for Deterministic Compartmental Models}
\usage{
init.dcm(s.num, i.num, r.num, s.num.g2, i.num.g2, r.num.g2, ...)
}
\arguments{
\item{s.num}{Number of initial susceptible persons. For two-group models,
this is the number of initial group 1 susceptible persons.}

\item{i.num}{Number of initial infected persons. For two-group models, this
is the number of initial group 1 infected persons.}

\item{r.num}{Number of initial recovered persons. For two-group models, this
is the number of initial group 1 recovered persons. This parameter is
only used for the \code{SIR} model type.}

\item{s.num.g2}{Number of initial susceptible persons in group 2. This
parameter is only used for two-group models.}

\item{i.num.g2}{Number of initial infected persons in group 2. This parameter
is only used for two-group models.}

\item{r.num.g2}{Number of initial recovered persons in group 2. This
parameter is only used for two-group \code{SIR} models.}

\item{...}{Additional initial conditions passed to model.}
}
\value{
An \code{EpiModel} object of class \code{init.dcm}.
}
\description{
Sets the initial conditions for deterministic compartmental
models simulated with \code{dcm}.
}
\details{
The initial conditions for a model solved with \code{\link[=dcm]{dcm()}} should be
input into the \code{init.dcm} function. This function handles initial
conditions for both base model types and original models.

Original models may use the parameter names listed as arguments here, a new
set of names, or a combination of both. With new models, initial conditions
must be input in the same order that the solved derivatives from the model
are output.
}
\section{Sensitivity Analyses}{

Like \code{\link[=param.dcm]{param.dcm()}}, initial conditions may be specified as vectors of length
greater than one to run sensitivity analyses over initial conditions. When
\code{sens.param = TRUE} in \code{\link[=control.dcm]{control.dcm()}} (the default), each element of the
vector produces a separate model run. If both parameters and initial
conditions have vector values, all vectors must have the same length.
}

\examples{
# SI model initial conditions
init <- init.dcm(s.num = 500, i.num = 1)

# Sensitivity analysis over initial infected count
init <- init.dcm(s.num = 500, i.num = c(1, 5, 25))

}
\seealso{
Use \code{\link[=param.dcm]{param.dcm()}} to specify model parameters and
\code{\link[=control.dcm]{control.dcm()}} to specify the control settings. Run the
parameterized model with \code{\link[=dcm]{dcm()}}.
}
\keyword{parameterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update.R
\name{depart_nodes}
\alias{depart_nodes}
\title{Depart Nodes from the netsim_dat Object}
\usage{
depart_nodes(dat, departures)
}
\arguments{
\item{dat}{the \code{netsim_dat} object}

\item{departures}{the vertex ids of nodes to depart}
}
\value{
the updated \code{netsim_dat} object with the nodes in
\code{departures} departed
}
\description{
Depart Nodes from the netsim_dat Object
}
\details{
If \code{tergmLite} is \code{FALSE}, the vertex ids
\code{departures} are deactivated (from the current timestep onward) in each
\code{networkDynamic} stored in \code{dat$nw}. If \code{tergmLite} is
\code{TRUE}, the vertex ids \code{departures} are deleted from \code{dat$el},
\code{dat$attr}, and \code{dat$net_attr}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{color_tea}
\alias{color_tea}
\title{Create a TEA Variable for Infection Status for \code{ndtv} Animations}
\usage{
color_tea(
  nd,
  old.var = "testatus",
  old.sus = "s",
  old.inf = "i",
  old.rec = "r",
  new.var = "ndtvcol",
  new.sus = NULL,
  new.inf = NULL,
  new.rec = NULL,
  verbose = TRUE
)
}
\arguments{
\item{nd}{An object of class \code{networkDynamic}.}

\item{old.var}{Old TEA variable name.}

\item{old.sus}{Status value for susceptible in old TEA variable.}

\item{old.inf}{Status value for infected in old TEA variable.}

\item{old.rec}{Status value for recovered in old TEA variable.}

\item{new.var}{New TEA variable name to be stored in \code{networkDynamic}
object.}

\item{new.sus}{Status value for susceptible in new TEA variable.}

\item{new.inf}{Status value for infected in new TEA variable.}

\item{new.rec}{Status value for recovered in new TEA variable.}

\item{verbose}{If \code{TRUE}, print progress to console.}
}
\value{
The updated object of class \code{networkDynamic}.
}
\description{
Creates a new color-named temporally-extended attribute (TEA)
variable in a \code{networkDynamic} object containing a disease
status TEA in numeric format.
}
\details{
The \code{ndtv} package (\url{https://cran.r-project.org/package=ndtv})
produces animated visuals for dynamic networks with evolving edge structures
and nodal attributes. Nodal attribute dynamics in \code{ndtv} movies require
a temporally extended attribute (TEA) containing a standard R color for each
node at each time step. By default, the \code{EpiModel} package uses TEAs to
store disease status history in network model simulations run in
\code{\link[=netsim]{netsim()}}. But that status TEA is in numeric format (0, 1, 2).
The \code{color_tea} function transforms those numeric values of that disease
status TEA into a TEA with color values in order to visualize status changes
in \code{ndtv}.

The convention in \code{\link[=plot.netsim]{plot.netsim()}} is to color the susceptible
nodes as blue, infected nodes as red, and recovered nodes as green. Alternate
colors may be specified using the \code{new.sus}, \code{new.inf}, and
\code{new.rec} parameters, respectively.

Using the \code{color_tea} function with a \code{netsim} object requires that
TEAs for disease status be used and that the \code{networkDynamic} object be
saved in the output: \code{tergmListe} must be  set to \code{FALSE} in
\code{\link[=control.net]{control.net()}}.
}
\seealso{
\code{\link[=netsim]{netsim()}} and the \code{ndtv} package documentation.
}
\keyword{colorUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgelists.R
\name{as_tibble_edgelist}
\alias{as_tibble_edgelist}
\title{Convert an Edgelist into a Tibble}
\usage{
as_tibble_edgelist(el)
}
\arguments{
\item{el}{An edgelist in matrix or data frame form.}
}
\value{
The edgelist in tibble form with two columns named \code{head} and \code{tail}.
}
\description{
Convert an Edgelist into a Tibble
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netdx.R
\name{netdx}
\alias{netdx}
\title{Dynamic Network Model Diagnostics}
\usage{
netdx(
  x,
  nsims = 1,
  dynamic = TRUE,
  nsteps = NULL,
  nwstats.formula = "formation",
  set.control.ergm = control.simulate.formula(),
  set.control.tergm = control.simulate.formula.tergm(MCMC.maxchanges =
    .Machine$integer.max),
  sequential = TRUE,
  keep.tedgelist = FALSE,
  keep.tnetwork = FALSE,
  verbose = TRUE,
  ncores = 1,
  skip.dissolution = FALSE,
  future.use.plan = FALSE
)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{netest}.}

\item{nsims}{Number of simulations to run. For dynamic diagnostics, 5--10
simulations are usually sufficient to assess model fit. For static
diagnostics, use 10,000+ draws to obtain stable estimates.}

\item{dynamic}{If \code{TRUE}, runs dynamic diagnostics that simulate the
temporal network forward in time, checking both formation targets
and partnership duration/dissolution. If \code{FALSE}, draws from the
static ERGM fit to check cross-sectional network structure only
(faster, but does not verify dissolution dynamics). Static
diagnostics are only available when the model was fit with the
edges dissolution approximation (\code{edapprox = TRUE} in \code{\link{netest}}).}

\item{nsteps}{Number of time steps per simulation (dynamic simulations
only). Should be at least several multiples of the longest target
partnership duration to allow the duration and dissolution statistics
to stabilize. For example, if the target duration is 50, running for
500 time steps is a reasonable starting point.}

\item{nwstats.formula}{A right-hand sided ERGM formula with the network
statistics of interest. The default is the formation formula of the
network model contained in \code{x}. You may track additional network
statistics beyond the formation terms by specifying them here, such
as \code{~ edges + meandeg + concurrent + degree(0:4)}. This is useful
for verifying that the model produces reasonable values for network
features that were not directly targeted in the formation model.}

\item{set.control.ergm}{Control arguments passed to \code{ergm}'s
\code{simulate_formula.network} (see details).}

\item{set.control.tergm}{Control arguments passed to \code{tergm}'s
\code{simulate_formula.network} (see details).}

\item{sequential}{For static diagnostics (\code{dynamic=FALSE}): if
\code{FALSE}, each of the \code{nsims} simulated Markov chains begins
at the initial network; if \code{TRUE}, the end of one simulation is
used as the start of the next.}

\item{keep.tedgelist}{If \code{TRUE}, keep the timed edgelist generated from
the dynamic simulations. Returned in the form of a list of matrices,
with one entry per simulation. Accessible at \verb{$edgelist}.}

\item{keep.tnetwork}{If \code{TRUE}, keep the full networkDynamic objects
from the dynamic simulations. Returned in the form of a list of nD
objects, with one entry per simulation. Accessible at \verb{$network}.}

\item{verbose}{If \code{TRUE}, print progress to the console.}

\item{ncores}{Number of processor cores to run multiple simulations
on, using the \code{future} framework.}

\item{skip.dissolution}{If \code{TRUE}, skip over the calculations of
duration and dissolution stats in \code{netdx}.}

\item{future.use.plan}{If \code{FALSE}, \code{netdx} will use \code{multisession} is used with \verb{workers = ncores for its parallelization. If }TRUE\verb{, }netdx\verb{will use the user defined plan from}globalEnv\verb{. Finally, it can take the output of a }future::tweak()\verb{call to setup a user defined temporary plan within}netdx`.
Which can be useful for distributed computation (HPC).}
}
\value{
A list of class \code{netdx}. Use \code{print()} to view summary tables of
formation statistics, duration, and dissolution diagnostics. Use
\code{\link{plot.netdx}} to visualize these diagnostics over time. Use
\code{\link[=as.data.frame.netdx]{as.data.frame.netdx()}} to extract timed edgelists (if
\code{keep.tedgelist = TRUE}).
}
\description{
Runs diagnostic simulations on an ERGM/STERGM estimated with
\code{\link{netest}} to assess whether the fitted model reproduces the
intended network features. Both static (cross-sectional) and
dynamic (temporal) diagnostics are supported. This is the
recommended second step in the network modeling pipeline, after
estimation with \code{\link{netest}} and before epidemic simulation with
\code{\link{netsim}}.
}
\details{
The \code{netdx} function handles dynamic network diagnostics for network
models fit with the \code{\link{netest}} function. Given the fitted model,
\code{netdx} simulates a specified number of dynamic networks for a specified
number of time steps per simulation. The network statistics in
\code{nwstats.formula} are saved for each time step. Summary statistics for
the formation model terms, as well as dissolution model and relational
duration statistics, are then calculated and can be accessed when printing or
plotting the \code{netdx} object. See \code{\link{print.netdx}} and \code{\link{plot.netdx}}
for details on printing and plotting.
}
\section{Control Arguments}{

Models fit with the full STERGM method in \code{netest} (setting the
\code{edapprox} argument to \code{FALSE}) require only a call to
\code{tergm}'s \code{simulate_formula.network}. Control parameters for those
simulations may be set using \code{set.control.tergm} in \code{netdx}.
The parameters should be input through the \code{control.simulate.formula.tergm}
function, with the available parameters listed in the
\code{\link[tergm:control.simulate.formula.tergm]{tergm::control.simulate.formula.tergm}} help page in the \code{tergm}
package.

Models fit with the ERGM method with the edges dissolution approximation
(setting \code{edapprox} to \code{TRUE}) require a call first to
\code{ergm}'s \code{simulate_formula.network} for simulating an initial
network, and second to \code{tergm}'s \code{simulate_formula.network} for
simulating that static network forward through time. Control parameters may
be set for both processes in \code{netdx}. For the first, the parameters
should be input through the \code{control.simulate.formula()} function, with
the available parameters listed in the
\code{\link[ergm:control.simulate.formula]{ergm::control.simulate.formula}} help
page in the \code{ergm} package. For the second, parameters should be input
through the \code{control.simulate.formula.tergm()} function, with the
available parameters listed in the \code{\link[tergm:control.simulate.formula.tergm]{tergm::control.simulate.formula.tergm}}
help page in the \code{tergm} package. An example is shown below.
}

\section{Static vs. Dynamic Diagnostics}{

Static diagnostics (\code{dynamic = FALSE}) draw many independent networks from
the fitted ERGM and compare the resulting statistics to the target values.
This is fast and checks whether the cross-sectional structure is correct,
but it does not verify partnership durations or dissolution rates. Dynamic
diagnostics (\code{dynamic = TRUE}) simulate the full temporal network forward
in time, checking both formation targets and dissolution/duration dynamics.
Dynamic diagnostics are slower but more comprehensive, and are required to
verify models that will be used with vital dynamics (arrivals/departures).
}

\section{Interpreting Diagnostics}{

After running \code{netdx}, use \code{print()} and \code{\link{plot.netdx}} to inspect the
results. Key indicators of a good model fit include:
\itemize{
\item \strong{Formation statistics:} The "Sim Mean" should be close to the
"Target" value. A small "Pct Diff" (< 5\\%) and a "Z Score" near 0
indicate good fit.
\item \strong{Duration statistics} (dynamic only): The simulated mean edge
durations should match the values passed to \code{\link{dissolution_coefs}}.
\item \strong{Dissolution statistics} (dynamic only): The simulated dissolution
rates should be approximately \code{1 / duration}.
}

Common problems: If formation statistics are off, the ERGM may need
increased burn-in (via \code{set.control.ergm}), or the target statistics
may be incompatible (e.g., specifying more edges than the network can
support). If durations are off but formation is correct, verify that
\code{d.rate} was correctly specified in \code{\link{dissolution_coefs}} for models
with vital dynamics.
}

\examples{
\donttest{
# Static diagnostics on a simple model
nw <- network_initialize(n = 100)
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 25)
est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
dx <- netdx(est, nsims = 1e4, dynamic = FALSE, verbose = FALSE)
dx
plot(dx)
}

\dontrun{
# Static diagnostics with additional network statistics
dx1 <- netdx(est,
  nsims = 1e4, dynamic = FALSE,
  nwstats.formula = ~ edges + meandeg + concurrent
)
dx1
plot(dx1, method = "b", stats = c("edges", "concurrent"))

# Dynamic diagnostics on the STERGM approximation
dx2 <- netdx(est,
  nsims = 5, nsteps = 500,
  nwstats.formula = ~ edges + meandeg + concurrent,
  set.control.ergm = control.simulate.formula(MCMC.burnin = 1e6)
)
dx2
plot(dx2, stats = c("edges", "meandeg"), plots.joined = FALSE)
plot(dx2, type = "duration")
plot(dx2, type = "dissolution", qnts.col = "orange2")
plot(dx2, type = "dissolution", method = "b", col = "bisque")

# Dynamic diagnostics on a more complex model
nw <- network_initialize(n = 1000)
nw <- set_vertex_attribute(nw, "neighborhood", rep(1:10, 100))
formation <- ~edges + nodematch("neighborhood", diff = TRUE)
target.stats <- c(800, 45, 81, 24, 16, 32, 19, 42, 21, 24, 31)
coef.diss <- dissolution_coefs(dissolution = ~offset(edges) +
                    offset(nodematch("neighborhood", diff = TRUE)),
                    duration = c(52, 58, 61, 55, 81, 62, 52, 64, 52, 68, 58))
est2 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
dx3 <- netdx(est2, nsims = 5, nsteps = 100)
print(dx3)
plot(dx3)
plot(dx3, type = "duration", plots.joined = TRUE, qnts = 0.2, legend = TRUE)
plot(dx3, type = "dissolution", mean.smooth = FALSE, mean.col = "red")
}

}
\seealso{
Estimate the network model with \code{\link{netest}} before running diagnostics.
Plot diagnostics with \code{\link{plot.netdx}} and print summary tables with
\code{\link{print.netdx}}. After diagnostics confirm a good fit, simulate the
epidemic with \code{\link{netsim}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.comp_plot.R
\name{comp_plot}
\alias{comp_plot}
\alias{comp_plot.netsim}
\alias{comp_plot.icm}
\alias{comp_plot.dcm}
\title{Plot Compartment Diagram for Epidemic Models}
\usage{
comp_plot(x, at, digits, ...)

\method{comp_plot}{netsim}(x, at = 1, digits = 3, ...)

\method{comp_plot}{icm}(x, at = 1, digits = 3, ...)

\method{comp_plot}{dcm}(x, at = 1, digits = 3, run = 1, ...)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{dcm}, \code{icm}, or
\code{netsim}.}

\item{at}{Time step for model statistics.}

\item{digits}{Number of significant digits to print.}

\item{...}{Additional arguments passed to plot (not currently used).}

\item{run}{Model run number, for \code{dcm} class models with multiple runs
(sensitivity analyses).}
}
\description{
Plots a compartment flow diagram for deterministic compartmental
models, stochastic individual contact models, and stochastic
network models.
}
\details{
The \code{comp_plot} function provides a visual summary of an epidemic model
at a specific time step. The information contained in \code{comp_plot} is the
same as in the \code{summary} functions for a model, but presented
graphically as a compartment flow diagram.

For \code{dcm} class plots, specify the model run number if the model
contains multiple runs, as in a sensitivity analysis. For \code{icm} and
\code{netsim} class plots, the \code{run} argument is not used; the plots
show the means and standard deviations across simulations at the specified
time step.

These plots are currently limited to one-group models for each of the three
model classes. That functionality may be expanded in future software
releases.
}
\examples{
## Example 1: DCM SIR model with varying act.rate
param <- param.dcm(inf.prob = 0.2, act.rate = 5:7,
                   rec.rate = 1/3, a.rate = 1/90, ds.rate = 1/100,
                   di.rate = 1/35, dr.rate = 1/100)
init <- init.dcm(s.num = 1000, i.num = 1, r.num = 0)
control <- control.dcm(type = "SIR", nsteps = 25, verbose = FALSE)
mod1 <- dcm(param, init, control)
comp_plot(mod1, at = 25, run = 3)

## Example 2: ICM SIR model with 3 simulations
param <- param.icm(inf.prob = 0.2, act.rate = 3, rec.rate = 1/50,
                   a.rate = 1/100, ds.rate = 1/100,
                   di.rate = 1/90, dr.rate = 1/100)
init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
control <- control.icm(type = "SIR", nsteps = 25,
                       nsims = 3, verbose = FALSE)
mod2 <- icm(param, init, control)
comp_plot(mod2, at = 25, digits = 1)

}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.inputs.R
\name{control.net}
\alias{control.net}
\title{Control Settings for Stochastic Network Models}
\usage{
control.net(
  type,
  nsteps,
  start = 1,
  nsims = 1,
  ncores = 1,
  resimulate.network = FALSE,
  tergmLite = FALSE,
  cumulative.edgelist = FALSE,
  truncate.el.cuml = 0,
  attr.rules,
  epi.by,
  initialize.FUN = initialize.net,
  resim_nets.FUN = resim_nets,
  summary_nets.FUN = summary_nets,
  infection.FUN = NULL,
  recovery.FUN = NULL,
  departures.FUN = NULL,
  arrivals.FUN = NULL,
  nwupdate.FUN = nwupdate.net,
  prevalence.FUN = prevalence.net,
  verbose.FUN = verbose.net,
  module.order = NULL,
  save.nwstats = TRUE,
  nwstats.formula = "formation",
  save.transmat = TRUE,
  save.network,
  save.run = FALSE,
  save.cumulative.edgelist = FALSE,
  save.other,
  verbose = TRUE,
  verbose.int = 1,
  skip.check = FALSE,
  raw.output = FALSE,
  future.use.plan = FALSE,
  tergmLite.track.duration = FALSE,
  set.control.ergm = control.simulate.formula(MCMC.burnin = 2e+05),
  set.control.tergm = control.simulate.formula.tergm(MCMC.maxchanges =
    .Machine$integer.max),
  save.diss.stats = TRUE,
  dat.updates = NULL,
  ...
)
}
\arguments{
\item{type}{Disease type to be modeled, with the choice of \code{"SI"} for Susceptible-Infected
diseases, \code{"SIR"} for Susceptible-Infected-Recovered diseases, and \code{"SIS"} for
Susceptible-Infected-Susceptible diseases.}

\item{nsteps}{Number of time steps to simulate the model over. This must be a positive integer
that is equal to the final step of a simulation. If a simulation is restarted with \code{start}
argument, this number must be at least one greater than that argument's value.}

\item{start}{For models with network resimulation, time point to start up the simulation. For
restarted simulations, this must be one greater than the final time step in the prior
simulation and must be less than the value in \code{nsteps}.}

\item{nsims}{The total number of disease simulations.}

\item{ncores}{Number of processor cores to run multiple simulations on, using
the \code{future} framework with \code{multisession}. See \code{future.use.plan} for
other \code{future} backends.}

\item{resimulate.network}{If \code{TRUE}, resimulate the network at each time step. This is required
when the epidemic or demographic processes impact the network structure (e.g., vital
dynamics). This parameter controls whether \code{resim_nets.FUN} performs actual network
resimulation; it does not affect the order in which modules are executed (see
\code{module.order}). Setting \code{tergmLite = TRUE} forces \code{resimulate.network = TRUE} with a
warning.}

\item{tergmLite}{Logical indicating usage of either \code{tergm} (\code{tergmLite = FALSE}), or \code{tergmLite}
(\code{tergmLite = TRUE}). Default of \code{FALSE}. When \code{TRUE}, \code{resimulate.network} is
automatically set to \code{TRUE} (with a warning if the user explicitly set it to \code{FALSE}).}

\item{cumulative.edgelist}{If \code{TRUE}, calculates a cumulative edgelist within the network
simulation module. This is used when tergmLite is used and the entire networkDynamic
object is not used.}

\item{truncate.el.cuml}{Number of time steps of the cumulative edgelist to retain. See help for
\code{\link{update_cumulative_edgelist}} for options.}

\item{attr.rules}{A list containing the  rules for setting the attributes of incoming nodes, with
one list element per attribute to be set (see details below).}

\item{epi.by}{A character vector of length 1 containing a nodal attribute for which subgroup
stratified prevalence summary statistics are calculated. This nodal attribute must be
contained in the network model formation formula, otherwise it is ignored.}

\item{initialize.FUN}{Module to initialize the model at time 1, with the default function of
\code{\link{initialize.net}}.}

\item{resim_nets.FUN}{Module to resimulate the network at each time step, with the default
function of \code{\link{resim_nets}}.}

\item{summary_nets.FUN}{Module to extract summary statistics of the network
at each time step, with the default function of \code{\link{summary_nets}}.}

\item{infection.FUN}{Module to simulate disease infection, with the default function of
\code{\link{infection.net}}.}

\item{recovery.FUN}{Module to simulate disease recovery, with the default function of
\code{\link{recovery.net}}.}

\item{departures.FUN}{Module to simulate departure or exit, with the default function of
\code{\link{departures.net}}.}

\item{arrivals.FUN}{Module to simulate arrivals or entries, with the default function of
\code{\link{arrivals.net}}.}

\item{nwupdate.FUN}{Module to handle updating of network structure and nodal attributes due to
exogenous epidemic model processes, with the default function of \code{\link{nwupdate.net}}.}

\item{prevalence.FUN}{Module to calculate disease prevalence at each time step, with the default
function of \code{\link{prevalence.net}}.}

\item{verbose.FUN}{Module to print simulation progress to screen, with the default function of
\code{\link{verbose.net}}.}

\item{module.order}{A character vector of module names that lists modules in the order in which
they should be evaluated within each time step. If \code{NULL}, the modules will be evaluated
as follows: first any new modules supplied through \code{...} in the order in which they are
listed, then the built-in modules in the order in which they are listed as arguments
above. \code{initialize.FUN} will always be run first and \code{verbose.FUN} will always be run last.
Module ordering is independent of \code{resimulate.network}: the specified order is always
respected regardless of whether network resimulation is enabled. In the default ordering,
\code{resim_nets.FUN} runs before \code{infection.FUN}, so the network is resimulated before
transmission is evaluated at each time step.}

\item{save.nwstats}{If \code{TRUE}, save network statistics in a data frame. The statistics to be
saved are specified in the \code{nwstats.formula} argument.}

\item{nwstats.formula}{A right-hand sided ERGM formula that includes network statistics of
interest, with the default to the formation formula terms. Supports \code{\link{multilayer}}
specification.}

\item{save.transmat}{If \code{TRUE}, complete transmission matrix is saved at simulation end.}

\item{save.network}{If \code{TRUE}, networkDynamic or networkLite object is saved at simulation end.}

\item{save.run}{If \code{TRUE}, the \code{run} sublist of \code{dat} is saved, allowing a
simulation to restart from this output.}

\item{save.cumulative.edgelist}{If \code{TRUE}, the \code{cumulative.edgelist} is saved at simulation end.}

\item{save.other}{A character vector of elements on the \code{netsim_dat} main data list to save out
after each simulation. One example for base models is the attribute list, \code{"attr"}, at
the final time step.}

\item{verbose}{If \code{TRUE}, print model progress to the console.}

\item{verbose.int}{Time step interval for printing progress to console, where \code{0} prints
completion status of entire simulation and positive integer \code{x} prints progress after
every \code{x} time steps. The default is to print progress after each time step.}

\item{skip.check}{If \code{TRUE}, skips the default error checking for the structure and consistency
of the parameter values, initial conditions, and control settings before running base
epidemic models. Setting this to \code{FALSE} is recommended when running models with new
modules specified.}

\item{raw.output}{If \code{TRUE}, \code{netsim} will output a list of raw data (one per simulation) instead
of a cleaned and formatted \code{netsim} object.}

\item{future.use.plan}{If \code{FALSE}, \code{netsim} will use \code{multisession} is used with \verb{workers = ncores for its parallelization. If }TRUE\verb{, }netsim\verb{will use the user defined plan from}globalEnv\verb{. Finally, it can take the output of a }future::tweak()\verb{call to setup a user defined temporary plan within}netsim`.
Which can be useful for distributed computation (HPC).}

\item{tergmLite.track.duration}{If \code{TRUE}, track duration information for models in \code{tergmLite}
simulations. Supports \code{\link{multilayer}} specification.}

\item{set.control.ergm}{Control arguments passed to \code{ergm::simulate_formula.network}. In \code{netsim},
this is only used when initializing the network with \code{edapprox = TRUE}. All other
simulations in \code{netsim} use \code{tergm}. Supports \code{\link{multilayer}} specification.}

\item{set.control.tergm}{Control arguments passed to \code{tergm::simulate_formula.network}. See the
help file for \code{\link{netdx}} for details and examples on specifying this parameter. Supports
\code{\link{multilayer}} specification.}

\item{save.diss.stats}{If \code{TRUE}, \code{netsim} will compute and save duration and dissolution
statistics for plotting and printing, provided \code{save.network} is \code{TRUE}, \code{tergmLite} is
\code{FALSE}, and the dissolution model is homogeneous.}

\item{dat.updates}{Either \code{NULL}, a single function taking arguments \code{dat},
\code{at}, and \code{network}, or a list of functions of length one greater
than the number of networks being simulated, with each function in
the list taking arguments \code{dat} and \code{at}. Here \code{dat} is the main
\code{netsim_dat} class object, \code{at} is the current timestep, and \code{network}
is an index indicating the current position within the sequence of
network (re)simulations on each time step. If a single function is
passed, it will be called before the first network is simulated and
after each network is simulated, with \code{network = 0L} before the first
network is simulated and with \code{network = i} after the \code{i}th network
is simulated. If a list of functions is passed, the first function
will be called before the first network is simulated, and the
\code{i + 1}th function will be called after the \code{i}th network is
simulated. (Note that \code{at = 0L} is used for initial cross-sectional
simulations in \code{\link{sim_nets_t1}}.) The function(s) should return the
\code{netsim_dat} object with any updates needed to correctly represent the
network states for calls to \code{simulate} and/or \code{summary}. This can be
useful if nodal attributes appearing in one network model depend on
nodal degrees in a different network.}

\item{...}{Additional control settings passed to model.}
}
\value{
An EpiModel object of class \code{control.net}.
}
\description{
Sets the controls for stochastic network models simulated with
\code{\link{netsim}}.
}
\details{
\code{control.net} sets the required control settings for any network model solved with the \code{\link{netsim}}
function. Controls are required for both base model types and when passing original process
modules. For an overview of control settings for base models, consult the
\href{https://epimodel.github.io/sismid/}{Network Modeling for Epidemics} course materials For
all base models, the \code{type} argument is a necessary parameter and it has no default.
}
\section{The attr.rules Argument}{

The \code{attr.rules} parameter is used to specify the rules for how nodal attribute values for
incoming nodes should be set. These rules are only necessary for models in which there are
incoming nodes (i.e., arrivals). There are three rules available for each attribute value:
\itemize{
\item \strong{\code{current}}: new nodes will be assigned this attribute in proportion to the distribution of
that attribute among existing nodes at that current time step.
\item \strong{\code{t1}}: new nodes will be assigned this attribute in proportion to the distribution of that
attribute among nodes at time 1 (that is, the proportions set in the original network for
\code{\link{netest}}).
\item \strong{\code{Value}}: all new nodes will be assigned this specific value, with no variation.
For example, the rules list \code{attr.rules = list(race = "t1", sex = "current", status = "s")}
specifies how the race, sex, and status attributes should be set for incoming nodes. By default,
the rule is \code{"current"} for all attributes except status, in which case it is \code{"s"} (that is, all
incoming nodes are susceptible).
}
}

\section{Checkpointing Simulations}{

\code{netsim} has a built-in checkpoint system to prevent losing computation work if the function is
interrupted (SIGINT, power loss, time limit exceeded on a computation cluster). When enabled,
each simulation will be saved every \code{.checkpoint.steps} time steps. Then, if a checkpoint enabled
simulation is launched again with \code{netsim}, it will restart at the last checkpoint available in
the saved data.

To enable the checkpoint capabilities of \code{netsim}, two control arguments have to be set:
\code{.checkpoint.steps}, which is a positive number of time steps to be run between each file save;
and \code{.checkpoint.dir}, which is the path to a directory to save the checkpointed data. If
\code{.checkpoint.dir} directory does not exist, \code{netsim} will attempt to create it on the first
checkpoint save. With these two controls defined, one can simply re-run \code{netsim} with the same
arguments to restart a set of simulations that were interrupted.

Simulations are checkpointed individually: for example, if 3 simulations are run on a single core,
the first 2 are finished, then the interruption occurs during the third, \code{netsim} will only
restart the third one from the last checkpoint.

A \code{.checkpoint.compress} argument can be set to overwrite the \code{compress} argument in \code{saveRDS}
used to save the checkpointed data. The current default for \code{saveRDS} is \code{gunzip (gz)}, which
provides fast compression that usually works well on \code{netsim} objects.

By default, if \code{netsim} reaches the end of all simulations, the checkpoint data directory and its
content are removed before returning the \code{netsim} object. The \code{.checkpoint.keep} argument can be
set to \code{TRUE} to prevent this removal to inspect the raw simulation objects.
}

\section{New Modules}{

Base network models use a set of module functions that specify how the individual nodes in the
network are subjected to infection, recovery, demographics, and other processes. Core modules are
those listed in the \code{.FUN} arguments. For each module, there is a default function used in
the simulation. The default infection module, for example, is contained in the \code{\link{infection.net}}
function.

For original models, one may substitute replacement module functions for any of the default
functions. New modules may be added to the workflow at each time step by passing a module function
via the \code{...} argument. Consult the
\href{https://epimodel.github.io/sismid/9_extending/mod9-Intro.html}{Extending EpiModel}
section of the Network Modeling for Epidemics course materials.
One may remove existing modules, such as \code{arrivals.FUN}, from the workflow by setting
the parameter value for that argument to \code{NULL}.
}

\section{End Horizon}{

\code{netsim} implements an "End Horizon" mechanism, where a set of modules are
removed from the simulation at a specific time step. This is enabled through
the \code{end.horizon} parameter to \code{control.net}.

This parameter must receive a \code{list} with fields \code{at}, the time step at which
the end horizon occurs, and \code{modules}, a character vector with the names of
the modules to remove. (e.g `list(at = 208, modules = c("arrivals.FUN",
"infections.FUN")))
}

\seealso{
Use \code{\link{param.net}} to specify model parameters and \code{\link{init.net}} to specify the initial conditions.
Run the parameterized model with \code{\link{netsim}}.
}
\keyword{parameterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{ssample}
\alias{ssample}
\title{Stable Sampling Function}
\usage{
ssample(x, size, replace = FALSE, prob = NULL)
}
\arguments{
\item{x}{Either a vector of one or more elements from which to choose, or a
positive integer.}

\item{size}{Non-negative integer giving the number of items to choose.}

\item{replace}{Should sampling be with replacement?}

\item{prob}{Vector of probability weights for obtaining the elements of the
vector being sampled.}
}
\value{
A vector containing the sampled value(s).
}
\description{
Provides a sampling function useful for dynamic simulations, in
which the length of the input vector may be multiple lengths and
the size of the sample may be 0.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendo.R
\name{plot.transmat}
\alias{plot.transmat}
\title{Plot transmat Infection Tree in Three Styles}
\usage{
\method{plot}{transmat}(x, style = c("phylo", "network", "transmissionTimeline"), ...)
}
\arguments{
\item{x}{A \code{\link[=transmat]{transmat()}} object to be plotted.}

\item{style}{Character name of plot style. One of \code{"phylo"},
\code{"network"}, or \code{"transmissionTimeline"}.}

\item{...}{Additional plot arguments to be passed to lower-level plot
functions (\code{plot.network}, \code{plot.phylo}, or
\code{transmissionTimeline}).}
}
\description{
Plots the transmission matrix tree from from \code{get_transmat}
in one of three styles: a phylogram, a directed network, or
a transmission timeline.
}
\details{
The \code{phylo} plot requires the \code{ape} package. The
\code{transmissionTimeline} plot requires that the \code{ndtv} package.
}
\seealso{
\code{\link[network:plot.network]{network::plot.network}}, \code{\link[ape:plot.phylo]{ape::plot.phylo()}},
\code{\link[ndtv:transmissionTimeline]{ndtv::transmissionTimeline()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendo.R, R/get.R
\name{is.transmat}
\alias{is.transmat}
\alias{transmat}
\alias{get_transmat}
\title{Extract Transmissions Matrix from Network Epidemic Model}
\usage{
is.transmat(x)

get_transmat(x, sim = 1, deduplicate = TRUE)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{\link[=netsim]{netsim()}}.}

\item{sim}{Simulation number of extracted network.}

\item{deduplicate}{If \code{TRUE}, randomly select one transmission event in
the case that multiple events occur per newly infected agent within
a time step.}
}
\value{
A data frame with the following standard columns:
\itemize{
\item \strong{at:} the time step at which the transmission occurred.
\item \strong{sus:} the ID number of the susceptible (newly infected) node.
\item \strong{inf:} the ID number of the infecting node.
\item \strong{infDur:} the duration of the infecting node's disease at the
time of the transmission.
\item \strong{transProb:} the probability of transmission per act.
\item \strong{actRate:} the rate of acts per unit time.
\item \strong{finalProb:} the final transmission probability for the
transmission event.
}
}
\description{
Extracts the matrix of transmission data for each transmission
event that occurred within a network epidemic model.
}
\examples{
\donttest{
## Simulate SI epidemic on two-group Bernoulli random graph
nw <- network_initialize(n = 100)
nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)
param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
init <- init.net(i.num = 10, i.num.g2 = 10)
control <- control.net(type = "SI", nsteps = 10, nsims = 3, verbose = FALSE)
mod <- netsim(est, param, init, control)

## Extract the transmission matrix from simulation 2
get_transmat(mod, sim = 2)
}

}
\keyword{extract}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.simnet.R
\name{simulate_dat}
\alias{simulate_dat}
\title{Simulate a Network for a Specified Number of Time Steps}
\usage{
simulate_dat(dat, at, network = 1L, nsteps = 1L)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{Current time step.}

\item{network}{index of the network to simulate}

\item{nsteps}{number of time steps to simulate}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function simulates a dynamic network over one or multiple
time steps for TERGMs or one or multiple cross-sectional network
panels for ERGMs, for use in \code{\link[=netsim]{netsim()}} modeling.
Network statistics are also extracted and saved if
\code{save.nwstats == TRUE} and
\code{resimulate.network == FALSE}.
}
\keyword{internal}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{mcat}
\alias{mcat}
\title{Write Out Test Progress to Console}
\usage{
mcat(test)
}
\arguments{
\item{test}{Character string with the name of a test.}
}
\description{
Writes the name of a test and \code{...} to console to show test
progress.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.logs.R
\name{netsim_error_logger}
\alias{netsim_error_logger}
\title{Handle the Logging of Traceback and Dumping of Frames on Error}
\usage{
netsim_error_logger(dat, s)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{s}{The number of the simulation that failed}
}
\value{
Nothing, after logging and dumping frames, the function gives the
control back to the general error handler
}
\description{
If \code{control$.traceback.on.error == TRUE}, this function prints the traceback
of the current simulation to STDIN. This is useful when \code{ncores > 1} or in
HPC settings.
If \code{control$.dump.frames.on.error == TRUE}, this function saves a debugging
dump for "postmortem debugging". The dumps are named
"dump_\%Y\%m\%d_\%H\%M\%S_s.rda" and stored at the root of the working directory.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.init.R
\name{overwrite_attrs}
\alias{overwrite_attrs}
\title{Helper to use a \code{data.frame} to initialize some attributes}
\usage{
overwrite_attrs(dat)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Uses \code{dat$init$init_attr} to overwrite some attributes of the
nodes at initialization
}
\details{
If an \code{init_attr} \code{data.frame} is present in \code{dat$init}, use it to overwrite
the attributes it contains.
\code{init_attr} must have a number of rows equal to the number of nodes in the
model as the attributes will be overwritten one to one, ensuring the correct
ordering.
\code{init_attr} columns MUST have a corresponding attribute already initialized.
See "R/default_attributes.R" for adding new attributes to the model.
\code{init_attr} is removed from \code{dat$init} at the end of the function to free up
its memory.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.R
\name{get_attr_history}
\alias{get_attr_history}
\title{Extract the Attributes History from Network Simulations}
\usage{
get_attr_history(sims)
}
\arguments{
\item{sims}{An \code{EpiModel} object of class \code{netsim}.}
}
\value{
A list of \code{data.frame}s, one for each "measure" recorded in the
simulation by the \code{record_attr_history} function.
}
\description{
Extract the Attributes History from Network Simulations
}
\examples{
\dontrun{

# With `sims` the result of a `netsim` call
get_attr_history(sims)

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.accessor.R
\name{net-accessor}
\alias{net-accessor}
\alias{get_attr_list}
\alias{get_attr}
\alias{add_attr}
\alias{set_attr}
\alias{append_attr}
\alias{remove_node_attr}
\alias{get_epi_list}
\alias{get_epi}
\alias{add_epi}
\alias{set_epi}
\alias{get_param_list}
\alias{get_param}
\alias{add_param}
\alias{set_param}
\alias{get_control_list}
\alias{get_control}
\alias{get_network_control}
\alias{add_control}
\alias{set_control}
\alias{get_init_list}
\alias{get_init}
\alias{add_init}
\alias{set_init}
\alias{get_core_attributes}
\alias{append_core_attr}
\title{Functions to Access and Edit the Main netsim_dat Object in Network Models}
\usage{
get_attr_list(dat, item = NULL)

get_attr(dat, item, posit_ids = NULL, override.null.error = FALSE)

add_attr(dat, item)

set_attr(dat, item, value, posit_ids = NULL, override.length.check = FALSE)

append_attr(dat, item, value, n.new)

remove_node_attr(dat, posit_ids)

get_epi_list(dat, item = NULL)

get_epi(dat, item, at = NULL, override.null.error = FALSE)

add_epi(dat, item)

set_epi(dat, item, at, value)

get_param_list(dat, item = NULL)

get_param(dat, item, override.null.error = FALSE)

add_param(dat, item)

set_param(dat, item, value)

get_control_list(dat, item = NULL)

get_control(dat, item, override.null.error = FALSE)

get_network_control(dat, network, item, override.null.error = FALSE)

add_control(dat, item)

set_control(dat, item, value)

get_init_list(dat, item = NULL)

get_init(dat, item, override.null.error = FALSE)

add_init(dat, item)

set_init(dat, item, value)

get_core_attributes()

append_core_attr(dat, at, n.new)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{item}{A character vector containing the name of the element to access
(for \code{get_} functions), create (for \code{add_} functions), or
edit (for \code{set_} and \code{append_} functions). Can be of length greater
than 1 for \verb{get_*_list} functions.}

\item{posit_ids}{For \code{set_attr} and \code{get_attr}, a numeric vector of
posit_ids to subset the desired \code{item}.}

\item{override.null.error}{If TRUE, \code{get_} will return NULL if the
\code{item} does not exist instead of throwing an error.
(default = FALSE).}

\item{value}{New value to be attributed in the \code{set_} and \code{append_}
functions.}

\item{override.length.check}{If TRUE, \code{set_attr} allows the modification
of the \code{item} size. (default = FALSE).}

\item{n.new}{For \code{append_core_attr}, the number of new nodes to initiate
with core attributes; for \code{append_attr}, the number of new
elements to append at the end of \code{item}.}

\item{at}{For \code{get_epi}, the timestep at which to access the specified
\code{item}; for \code{set_epi}, the timestep at which to add the new
value for the epi output \code{item}; for \code{append_core_attr}, the
current time step.}

\item{network}{index of network for which to get control}
}
\value{
A vector or a list of vectors for \code{get_} functions; the main
list object for \code{set_}, \code{append_}, and \code{add_}
functions.
}
\description{
These \code{get_}, \code{set_}, \code{append_}, and \code{add_}
functions allow a safe and efficient way to retrieve and mutate
the main \code{netsim_dat} class object of network models
(typical variable name \code{dat}). They are intended for use
\emph{inside module functions} that run during a \code{\link[=netsim]{netsim()}}
simulation, not for editing the user-facing \code{param.net},
\code{init.net}, or \code{control.net} inputs before a run or the
output object returned by \code{\link[=netsim]{netsim()}}. See the \strong{Intended
Usage} section below for details and alternatives.

This function returns an exhaustive named list of the attributes managed by
EpiModel itself. It can be used to check the validity of an attributes list
and of its types.
}
\section{Intended Usage}{

These accessors operate on the live \code{netsim_dat} object that EpiModel
constructs internally when a simulation starts and passes through each
module at every time step. The expected calling context is \strong{inside a
user-supplied module function} (e.g., a custom \code{infection.FUN},
\code{recovery.FUN}, or arrivals/departures module) registered with
\code{\link[=control.net]{control.net()}}.

They are \emph{not} a general-purpose tool for editing the user-facing input
or output objects of \code{\link[=netsim]{netsim()}}:
\itemize{
\item Calling them on a \code{\link[=param.net]{param.net()}}, \code{\link[=init.net]{init.net()}}, or \code{\link[=control.net]{control.net()}}
object before a run will appear to succeed (those objects are also
list-like) but produces an object that is not a valid simulation
input.
\item Calling them on the object returned by \code{\link[=netsim]{netsim()}} modifies only the
saved input slot, not anything that would change the result of a
re-run. Note also that \code{\link[=netsim]{netsim()}} strips the \verb{*.FUN} entries from
its returned \code{control} slot, so feeding that slot back into a new
simulation will fail.
}

For modifications outside a running simulation, use the following
instead:
\itemize{
\item \strong{Editing parameters before a run:} \code{\link[=update_params]{update_params()}} for a
\code{param.net} object, or direct list assignment
(e.g., \code{p$inf.prob <- 0.5}).
\item \strong{Editing init or control before a run:} direct list assignment
(e.g., \code{ctrl$nsteps <- 1000}), or rebuild with a fresh call to the
\code{\link[=init.net]{init.net()}} / \code{\link[=control.net]{control.net()}} constructor.
\item \strong{Scheduled mid-simulation changes:} the \code{.param.updater.list}
argument to \code{\link[=param.net]{param.net()}} and the \code{.control.updater.list} argument
to \code{\link[=control.net]{control.net()}}. See the "model-parameters" vignette for the
underlying updater module and the scenario API built on top of it.
}
}

\section{Core Attribute}{

The \code{append_core_attr} function initializes the attributes necessary for
EpiModel to work (the four core attributes are: "active", "unique_id",
"entrTime", and "exitTime"). These attributes are used in the initialization
phase of the simulation, to create the nodes (see
\code{\link[=initialize.net]{initialize.net()}}); and also used when adding nodes during the
simulation (see \code{\link[=arrivals.net]{arrivals.net()}}).
}

\section{Mutability}{

The \code{set_}, \code{append_}, and \code{add_} functions DO NOT modify the
\code{netsim_dat} object in place. The result must be assigned back to
\code{dat} in order to be registered: \verb{dat <- set_*(dat, item, value)}.
}

\section{\code{set_} and \code{append_} vs \code{add_}}{

The \code{set_} and \code{append_} functions edit a pre-existing element or
create a new one if it does not exist already by calling the \code{add_}
functions internally.
}

\examples{
dat <- create_dat_object(control = list(nsteps = 150))
dat <- append_core_attr(dat, 1, 100)

dat <- add_attr(dat, "age")
dat <- set_attr(dat, "age", runif(100))
dat <- set_attr(dat, "status", rbinom(100, 1, 0.9))
dat <- append_attr(dat, "status", 1, 10)
dat <- append_attr(dat, "age", NA, 10)
get_attr_list(dat)
get_attr_list(dat, c("age", "active"))
get_attr(dat, "status")
get_attr(dat, "status", c(1, 4))

dat <- add_epi(dat, "i.num")
dat <- set_epi(dat, "i.num", 150, 10)
dat <- set_epi(dat, "s.num", 150, 90)
get_epi_list(dat)
get_epi_list(dat, c("i.num", "s.num"))
get_epi(dat, "i.num")
get_epi(dat, "i.num", c(1, 4))

dat <- add_param(dat, "x")
dat <- set_param(dat, "x", 0.4)
dat <- set_param(dat, "y", 0.8)
get_param_list(dat)
get_param_list(dat, c("x", "y"))
get_param(dat, "x")

dat <- add_init(dat, "x")
dat <- set_init(dat, "x", 0.4)
dat <- set_init(dat, "y", 0.8)
get_init_list(dat)
get_init_list(dat, c("x", "y"))
get_init(dat, "x")

dat <- add_control(dat, "x")
dat <- set_control(dat, "x", 0.4)
dat <- set_control(dat, "y", 0.8)
get_control_list(dat)
get_control_list(dat, c("x", "y"))
get_control(dat, "x")

}
\seealso{
\code{\link[=update_params]{update_params()}} for editing a \code{param.net} object outside a
simulation; \code{\link[=param.net]{param.net()}}, \code{\link[=init.net]{init.net()}}, \code{\link[=control.net]{control.net()}} for input
constructors; \code{\link[=netsim]{netsim()}} for running a simulation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.infection.R
\name{infection.net}
\alias{infection.net}
\title{Primary Infection Module for netsim}
\usage{
infection.net(dat, at)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{Current time step.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function simulates the main infection process given the
current state of the partnerships and disease in the system.
}
\details{
The main steps in this infection module are as follows:
\enumerate{
\item Get IDs for current infected and susceptible nodes given the current
disease status.
\item Call \code{\link[=discord_edgelist]{discord_edgelist()}} to get the current discordant
edgelist given step 1.
\item Determine the transmission rates (e.g., as a function of group).
\item Pull the number of acts per partnership in a time step from the
\code{act.rate} parameter.
\item Calculate the final transmission probabilities given the transmission
rates and act rates.
\item Randomly transmit on the discordant edgelist.
\item Conduct bookkeeping for new infections to update status on the nodes
and calculate disease incidence.
}
}
\seealso{
\code{\link[=discord_edgelist]{discord_edgelist()}} is used within \code{infection.net}
to obtain a discordant edgelist.
}
\keyword{internal}
\keyword{netMod}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.init.R
\name{init_nets}
\alias{init_nets}
\title{Network Data and Stats Initialization}
\usage{
init_nets(dat, x)
}
\arguments{
\item{dat}{A main data object of class \code{netsim_dat} obtained from
\code{\link[=create_dat_object]{create_dat_object()}}, including the \code{control}
argument.}

\item{x}{Either a fitted network model object of class \code{netest}, or a
list of such objects.}
}
\value{
A \code{netsim_dat} class main data object with network data and
stats initialized.
}
\description{
This function initializes the network data and stats on the main
\code{netsim_dat} class data object.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update.R
\name{delete_vertices}
\alias{delete_vertices}
\title{Fast Version of network::delete.vertices for Edgelist-formatted
Network}
\usage{
delete_vertices(el, vid)
}
\arguments{
\item{el}{A two-column matrix of current edges (edgelist) with an attribute
variable \code{n} containing the total current network size.}

\item{vid}{A vector of IDs to delete from the edgelist.}
}
\value{
Returns an updated edgelist object, \code{el}, with the edges of deleted
vertices removed from the edgelist and the ID numbers of the remaining edges
permuted downward.
}
\description{
Given a current two-column matrix of edges and a vector of IDs
to delete from the matrix, this function first removes any rows
of the edgelist in which the IDs are present and then permutes
downward the index of IDs on the edgelist that were numerically
larger than the IDs deleted.
}
\details{
This function is used in \code{EpiModel} modules to remove vertices (nodes)
from the edgelist object to account for exits from the population (e.g.,
deaths and out-migration).
}
\examples{
\dontrun{
library("EpiModel")
set.seed(12345)
nw <- network_initialize(100)
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
x <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

param <- param.net(inf.prob = 0.3)
init <- init.net(i.num = 10)
control <- control.net(type = "SI", nsteps = 100, nsims = 5,
                       tergmLite = TRUE)

# Set seed for reproducibility
set.seed(123456)

# networkLite representation structure after initialization
dat <- crosscheck.net(x, param, init, control)
dat <- initialize.net(x, param, init, control)

# Current edges
head(dat$el[[1]], 20)

# Remove nodes 1 and 2
nodes.to.delete <- 1:2
dat$el[[1]] <- delete_vertices(dat$el[[1]], nodes.to.delete)

# Newly permuted edges
head(dat$el[[1]], 20)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icm.inputs.R
\name{init.icm}
\alias{init.icm}
\title{Initial Conditions for Stochastic Individual Contact Models}
\usage{
init.icm(s.num, i.num, r.num, s.num.g2, i.num.g2, r.num.g2, ...)
}
\arguments{
\item{s.num}{Number of initial susceptible persons. For two-group models,
this is the number of initial group 1 susceptible persons.}

\item{i.num}{Number of initial infected persons. For two-group models, this
is the number of initial group 1 infected persons.}

\item{r.num}{Number of initial recovered persons. For two-group models, this
is the number of initial group 1 recovered persons. This parameter is
only used for the \code{SIR} model type.}

\item{s.num.g2}{Number of initial susceptible persons in group 2. This
parameter is only used for two-group models.}

\item{i.num.g2}{Number of initial infected persons in group 2. This parameter
is only used for two-group models.}

\item{r.num.g2}{Number of initial recovered persons in group 2. This
parameter is only used for two-group \code{SIR} models.}

\item{...}{Additional initial conditions passed to model.}
}
\value{
An \code{EpiModel} object of class \code{init.icm}.
}
\description{
Sets the initial conditions for stochastic individual contact
models simulated with \code{icm}.
}
\details{
The initial conditions for a model solved with \code{\link[=icm]{icm()}} should be
input into the \code{init.icm} function. This function handles initial
conditions for both base models and original models using new modules.
}
\seealso{
Use \code{\link[=param.icm]{param.icm()}} to specify model parameters and
\code{\link[=control.icm]{control.icm()}} to specify the control settings. Run the
parameterized model with \code{\link[=icm]{icm()}}.
}
\keyword{parameterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.R
\name{as.data.frame.icm}
\alias{as.data.frame.icm}
\alias{as.data.frame.netsim}
\title{Extract Model Data for Stochastic Models}
\usage{
\method{as.data.frame}{icm}(
  x,
  row.names = NULL,
  optional = FALSE,
  out = "vals",
  sim = NULL,
  qval = NULL,
  repair = "drop",
  ...
)

\method{as.data.frame}{netsim}(
  x,
  row.names = NULL,
  optional = FALSE,
  out = "vals",
  sim = NULL,
  repair = "drop",
  ...
)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{icm} or \code{netsim}.}

\item{row.names}{See \code{\link[=as.data.frame.default]{as.data.frame.default()}}.}

\item{optional}{See \code{\link[=as.data.frame.default]{as.data.frame.default()}}.}

\item{out}{Data output to data frame: \code{"mean"} for row means across
simulations, \code{"sd"} for row standard deviations across
simulations, \code{"qnt"} for row quantiles at the level specified in
\code{qval}, or \code{"vals"} for values from individual simulations.}

\item{sim}{If \code{out="vals"}, the simulation number to output. If not
specified, then data from all simulations will be output.}

\item{qval}{Quantile value required when \code{out="qnt"}.}

\item{repair}{What to do with epi trackers that are too short. "drop" will
remove them from the output, "pad" will add \code{NA} rows to get to the
right size(Default = "drop").}

\item{...}{See \code{\link[=as.data.frame.default]{as.data.frame.default()}}.}
}
\value{
A data frame containing the data from \code{x}.
}
\description{
This function extracts model simulations for objects of classes
\code{icm} and \code{netsim} into a data frame using
the generic \code{as.data.frame} function.
}
\details{
These methods work for both \code{icm} and \code{netsim} class models. The
available output includes time-specific means, standard deviations,
quantiles, and simulation values (compartment and flow sizes) from these
stochastic model classes. Means, standard deviations, and quantiles are
calculated by taking the row summary (i.e., each row of data corresponds
to a time step) across all simulations in the model output.
}
\examples{
## Stochastic ICM SIS model
param <- param.icm(inf.prob = 0.8, act.rate = 2, rec.rate = 0.1)
init <- init.icm(s.num = 500, i.num = 1)
control <- control.icm(type = "SIS", nsteps = 10,
                       nsims = 3, verbose = FALSE)
mod <- icm(param, init, control)

# Default output all simulation runs, default to all in stacked data.frame
as.data.frame(mod)
as.data.frame(mod, sim = 2)

# Time-specific means across simulations
as.data.frame(mod, out = "mean")

# Time-specific standard deviations across simulations
as.data.frame(mod, out = "sd")

# Time-specific quantile values across simulations
as.data.frame(mod, out = "qnt", qval = 0.25)
as.data.frame(mod, out = "qnt", qval = 0.75)

\dontrun{
## Stochastic SI Network Model
nw <- network_initialize(n = 100)
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)
est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

param <- param.net(inf.prob = 0.5)
init <- init.net(i.num = 10)
control <- control.net(type = "SI", nsteps = 10, nsims = 3, verbose = FALSE)
mod <- netsim(est, param, init, control)

# Same data extraction methods as with ICMs
as.data.frame(mod)
as.data.frame(mod, sim = 2)
as.data.frame(mod, out = "mean")
as.data.frame(mod, out = "sd")
as.data.frame(mod, out = "qnt", qval = 0.25)
as.data.frame(mod, out = "qnt", qval = 0.75)
}

}
\keyword{extract}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{make_restart_point}
\alias{make_restart_point}
\title{Make a Lightweight Restart Point From a \code{netsim} Object with tergmLite}
\usage{
make_restart_point(sim_obj, time_attrs, sim_num = 1, keep_steps = 1)
}
\arguments{
\item{sim_obj}{a \code{netsim} object from an ended \code{netsim} call.}

\item{time_attrs}{a \code{character} vector containing the names of the attributes
that are expressed in time-steps. These will be offsetted so the last
step in the original simulation become the step 1 (default) in the new
ones. If no such attributes exist, pass \code{c()}.}

\item{sim_num}{the number of the simulation to extract from the \code{netsim}
object (default = 1).}

\item{keep_steps}{The number of simulation steps to keep from the previous
run. By default only keep one but more is possible if some
back-history is wanted.}
}
\value{
a trimed \code{netsim} object with only one simulation that is ready to be
used as a restart point.
}
\description{
Extract the elements required for re-initializing a \code{netsim} simulation from
a completed simulation. This function also resets the Unique IDs and Time
values to reduce the size of the simulation. This function only works for
simulations where \code{control$tergmLite = TRUE}
}
\details{
The restart point created always contains a single simulation and drops the
\code{attr.history}, the \code{raw.records} and \code{stats} from the initial simulation.

The epi trackers, cumulative edgelists, transmission matrix and \code{nwstats} are
truncated to only contain the last \code{keep_steps} entries.

Warning: the \code{time_attrs} argument is mandatory. Almost all simulation worth
restarting have such attributes (e.g. time.of.hiv.infection). If no such
argument exists, passing \code{c()} will allow the function to run while ensuring
that this was done on purpose.

When restarting from the output of this function, it is suggested to express
the time steps in a relative maner in \code{control.net}:

\if{html}{\out{<div class="sourceCode">}}\preformatted{control.net(
  start = restart_point$control$nsteps + 1),
  nsteps = restart_point$control$nsteps + 1 + 104)
)
}\if{html}{\out{</div>}}
}
\examples{
\dontrun{
# With  pre-existing `sim`, `param` and `init` object (see `netsim`)

# List all attributes that store a time step
time_attrs <- c(
  "inf.time",
  "stage.time",
  "aids.time",
  "prep.start.last"
)
# Make a restart point a re-run for 10 more timesteps
x <- make_restart_point(sim, time_attrs, sim_num = 1, keep_steps = 1)
control <- control_msm(
  start = x$control$nsteps + 1,
  nsteps = x$control$nsteps + 1 + 10
)
sim <- netsim(x, param, init, control)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EpiModel-package.R
\docType{package}
\name{EpiModel-package}
\alias{EpiModel-package}
\alias{EpiModel}
\title{Mathematical Modeling of Infectious Disease Dynamics}
\description{
\tabular{ll}{
Package: \tab EpiModel\cr
Type: \tab Package\cr
Version: \tab 2.6.1\cr
Date: \tab 2026-05-12\cr
License: \tab GPL-3\cr
LazyLoad: \tab yes\cr
}
}
\details{
The EpiModel software package provides tools for building, solving, and
visualizing mathematical models of infectious disease dynamics. These tools
allow users to simulate epidemic models in multiple frameworks for both
pedagogical purposes ("base models") and novel research purposes
("extension models").
}
\section{Model Classes and Infectious Disease Types}{

EpiModel provides functionality for three classes of epidemic models:
\itemize{
\item \strong{Deterministic Compartmental Models:} these continuous-time
models are solved using ordinary differential equations. EpiModel
allows for easy specification of sensitivity analyses to compare
multiple scenarios of the same model across different parameter
values.
\item \strong{Stochastic Individual Contact Models:} a novel class of
individual-based, microsimulation models that were developed to add
random variation in all components of the transmission system, from
infection to recovery to vital dynamics (arrivals and departures).
\item \strong{Stochastic Network Models:} with the underlying statistical
framework of temporal exponential random graph models (ERGMs) recently
developed in the \strong{Statnet} suite of software in R, network
models over epidemics simulate edge (e.g., partnership) formation and
dissolution stochastically according to a specified statistical model,
with disease spread across that network.
}

EpiModel supports three infectious disease types to be run across all of the
three classes.
\itemize{
\item \strong{Susceptible-Infectious (SI):} a two-state disease in which
there is life-long infection without recovery. HIV/AIDS is one
example, although for this case it is common to model infection
stages as separate compartments.
\item \strong{Susceptible-Infectious-Recovered (SIR):} a three-stage
disease in which one has life-long recovery with immunity after
infection. Measles is one example, but modern models for the disease
also require consideration of vaccination patterns in the population.
\item \strong{Susceptible-Infectious-Susceptible (SIS):} a two-stage
disease in which one may transition back and forth from the
susceptible to infected states throughout life. Examples include
bacterial sexually transmitted diseases like gonorrhea.
}

These basic disease types may be extended in any arbitrarily complex way to
simulate specific diseases for research questions.
}

\section{Model Parameterization and Simulation}{

EpiModel uses three model setup functions for each model class to input the
necessary parameters, initial conditions, and control settings:
\itemize{
\item \code{\link[=param.dcm]{param.dcm()}}, \code{\link[=param.icm]{param.icm()}}, and
\code{\link[=param.net]{param.net()}} are used to input epidemic parameters for each
of the three model classes. Parameters include the rate of contacts or
acts between actors, the probability of transmission per contact, and
recovery and demographic rates for models that include those
transitions.
\item \code{\link[=init.dcm]{init.dcm()}}, \code{\link[=init.icm]{init.icm()}}, and
\code{\link[=init.net]{init.net()}} are used to input the initial conditions for
each class. The main conditions are limited to the numbers or, if
applicable, the specific agents in the population who are infected or
recovered at the simulation outset.
\item \code{\link[=control.dcm]{control.dcm()}}, \code{\link[=control.icm]{control.icm()}}, and
\code{\link[=control.net]{control.net()}} are used to specify the remaining control
settings for each simulation. The core controls for base model
types include the disease type, number of time steps, and number of
simulations. Controls are also used to input new model functions (for
DCMs) and new model modules (for ICMs and network models) to allow the
user to simulate fully original epidemic models in EpiModel. See the
documentation for the specific control functions help pages.
}

With the models parameterized, the functions for simulating epidemic models
are:
\itemize{
\item \code{\link[=dcm]{dcm()}} for deterministic compartmental models.
\item \code{\link[=icm]{icm()}} for individual contact models.
\item Network models are simulated in a three-step process:
\enumerate{
\item \code{\link[=netest]{netest()}} estimates the statistical model for the network
structure itself (i.e., how partnerships form and dissolve over time
given the parameterization of those processes). This function is a
wrapper around the \code{ergm} and \code{tergm} functions in the
\code{ergm} and \code{tergm} packages. The current statistical
framework for model simulation is called "egocentric inference":
target statistics summarizing these formation and dissolution
processes collected from an egocentric sample of the population.
\item \code{\link[=netdx]{netdx()}} runs diagnostics on the dynamic model fit by
simulating the base network over time to ensure the model fits the
targets for formation and dissolution.
\item \code{\link[=netsim]{netsim()}} simulates the stochastic network epidemic
models, with a given network model fit in \code{\link[=netest]{netest()}}. Here
the function requires this model fit object along with the
parameters, initial conditions, and control settings as defined
above.
}
}
}

\references{
The EpiModel website is at \url{https://www.epimodel.org/}, and the source
code is at \url{https://github.com/EpiModel/EpiModel}. Bug reports and
feature requests are welcome.

Our primary methods paper on EpiModel is published in the \strong{Journal of
Statistical Software}. If you use EpiModel for any research or teaching
purposes, please cite this reference:

Jenness SM, Goodreau SM, and Morris M. EpiModel: An R Package for
Mathematical Modeling of Infectious Disease over Networks. Journal of
Statistical Software. 2018; 84(8): 1-47. \doi{10.18637/jss.v084.i08}.

We have also developed two extension packages for modeling specific disease
dynamics. For HIV and bacterial sexually transmitted infections, we have
developed \code{EpiModelHIV}, which is available on Github at
\url{https://github.com/EpiModel/EpiModelHIV}. For COVID-19, we have
developed \code{EpiModelCOVID}, which is available at
\url{https://github.com/EpiModel/EpiModelCOVID}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://www.epimodel.org/}
  \item \url{https://epimodel.github.io/EpiModel/}
  \item Report bugs at \url{https://github.com/EpiModel/EpiModel/issues/}
}

}
\author{
\strong{Maintainer}: Samuel Jenness \email{samuel.m.jenness@emory.edu}

Authors:
\itemize{
  \item Samuel Jenness \email{samuel.m.jenness@emory.edu}
  \item Steven M. Goodreau \email{goodreau@uw.edu}
  \item Martina Morris \email{morrism@uw.edu}
  \item Adrien Le Guillou \email{contact@aleguillou.org}
  \item Chad Klumb \email{cklumb@gmail.com}
}

Other contributors:
\itemize{
  \item Skye Bender-deMoll \email{skyebend@uw.edu} [contributor]
}

}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modules.net.R
\name{modules.net}
\alias{modules.net}
\title{Modules for Stochastic Network Models}
\description{
Stochastic network models of infectious disease in EpiModel require
statistical modeling of networks, simulation of those networks forward
through time, and simulation of epidemic dynamics on top of those evolving
networks. The \code{\link[=netsim]{netsim()}} function handles both the network and
epidemic simulation tasks. Within this function are a series of modules that
initialize the simulation and then simulate new infections, recoveries, and
demographics on the network. Modules also handle the resimulation of the
network and some bookkeeping calculations for disease prevalence.

Writing original network models that expand upon our "base" model set will
require modifying the existing modules or adding new modules to the workflow
in \code{\link[=netsim]{netsim()}}. The existing modules may be used as a template for
replacement or new modules.

This help page provides an orientation to these module functions, in the
order in which they are used within \code{\link[=netsim]{netsim()}}, to help guide users
in writing their own functions. These module functions are not shown
on the help index since they are not called directly by the end-user. To
understand these functions in more detail, review the separate help pages
listed below.
}
\section{Initialization Module}{

This function sets up nodal attributes, like disease status, on the network
at the starting time step of disease simulation, \eqn{t_1}. For
multiple-simulation function calls, these are reset at the beginning of each
individual simulation.
\itemize{
\item \code{\link[=initialize.net]{initialize.net()}}: sets up the main \code{netsim_dat} data
structure used in the simulation, initializes which nodes are infected
(via the initial conditions passed in \code{\link[=init.net]{init.net()}}), and
simulates a first time step of the networks given the network model
fit from \code{\link[=netest]{netest()}}.
}
}

\section{Disease Status Modification Modules}{

The main disease simulation occurs at each time step given the current state
of the network at that step. Infection of nodes is simulated as a function of
attributes of the nodes and the edges. Recovery of nodes is likewise
simulated as a function of nodal attributes of those infected nodes. These
functions also calculate summary flow measures such as disease incidence.
\itemize{
\item \code{\link[=infection.net]{infection.net()}}: simulates disease transmission given an
edgelist of discordant partnerships by calculating the relevant
transmission and act rates for each edge, and then updating the nodal
attributes and summary statistics.
\item \code{\link[=recovery.net]{recovery.net()}}: simulates recovery from infection either
to a lifelong immune state (for SIR models) or back to the susceptible
state (for SIS models), as a function of the recovery rate parameters
specified in \code{\link[=param.net]{param.net()}}.
}
}

\section{Demographic Modules}{

Demographics such as arrival and departure processes are simulated at each
time step to update entries into and exits from the network. These are used
in epidemic models with network feedback, in which the network is resimulated
at each time step to account for the nodal changes affecting the edges.
\itemize{
\item \code{\link[=departures.net]{departures.net()}}: randomly simulates departure for nodes
given their disease status (susceptible, infected, recovered), and
their group-specific departure rates specified in
\code{\link[=param.net]{param.net()}}. Departures involve deactivating nodes.
\item \code{\link[=arrivals.net]{arrivals.net()}}: randomly simulates new arrivals into the
network given the current population size and the arrival rate
specified in the \code{a.rate} parameters. This involves adding new
nodes into the network.
}
}

\section{Network Resimulation Module}{

In dependent network models, the network object is resimulated at each time
step to account for changes in the size of the network (changed through
entries and exits), and the disease status of the nodes.
\itemize{
\item \code{\link[=resim_nets]{resim_nets()}}: resimulates the network object one time step
forward given the set of formation and dissolution coefficients
estimated in \code{\link[=netest]{netest()}}.
}
}

\section{Bookkeeping Module}{

Network simulations require bookkeeping at each time step to calculate the
summary epidemiological statistics used in the model output analysis.
\itemize{
\item \code{\link[=prevalence.net]{prevalence.net()}}: calculates the number in each disease
state (susceptible, infected, recovered) at each time step for those
active nodes in the network. If the \code{epi.by} control is used, it
calculates these statistics by a set of specified nodal attributes.
\item \code{\link[=verbose.net]{verbose.net()}}: summarizes the current state of the
simulation and prints this to the console.
}
}

\section{One- & Two-Group Modules}{

If epidemic \code{type} is supplied within \code{\link[=control.net]{control.net()}},
EpiModel defaults each of the base epidemic and demographic modules described
above (arrivals.FUN, departures.FUN, infection.FUN, recovery.FUN) to the
correct .net function based on variables passed to \code{\link[=param.net]{param.net()}}
(e.g. num.g2, denoting population size of group two, would select the
two-group variants of the aforementioned modules). Two-group modules are
denoted by a .2g affix (e.g., recovery.2g.net)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.simnet.R
\name{edges_correct}
\alias{edges_correct}
\title{Adjustment for the Edges Coefficient with Changing Network Size}
\usage{
edges_correct(dat, at)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{Current time step.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Adjusts the edges coefficient in a dynamic network model
simulated in \code{\link[=netsim]{netsim()}} to preserve the mean
degree of nodes in the network. Requires \code{at >= 2}.
Maintains the \code{num(.g2)} epi fields (initialized in
\code{\link[=sim_nets_t1]{sim_nets_t1()}}) for computing the coefficient
adjustment.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.netsim}
\alias{summary.netsim}
\title{Summary Model Statistics}
\usage{
\method{summary}{netsim}(object, at, digits = 3, ...)
}
\arguments{
\item{object}{An \code{EpiModel} object of class \code{netsim}.}

\item{at}{Time step for model statistics.}

\item{digits}{Number of significant digits to print.}

\item{...}{Additional summary function arguments.}
}
\description{
Extracts and prints model statistics simulated with
\code{netsim}.
}
\details{
This function provides summary statistics for the main epidemiological
outcomes (state and transition size and prevalence) from a \code{netsim}
model. Time-specific summary measures are provided, so it is necessary to
input a time of interest.
}
\examples{
\dontrun{
## SI Model without Network Feedback
# Initialize network and set network model parameters
nw <- network_initialize(n = 100)
nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 50))
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)

# Estimate the ERGM models (see help for netest)
est1 <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

# Parameters, initial conditions, and controls for model
param <- param.net(inf.prob = 0.3, inf.prob.g2 = 0.15)
init <- init.net(i.num = 10, i.num.g2 = 10)
control <- control.net(type = "SI", nsteps = 100, nsims = 5, verbose.int = 0)

# Run the model simulation
mod <- netsim(est1, param, init, control)

summary(mod, at = 1)
summary(mod, at = 50)
summary(mod, at = 100)
}

}
\seealso{
\code{\link[=netsim]{netsim()}}
}
\keyword{extract}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.vital.R
\name{arrivals.2g.net}
\alias{arrivals.2g.net}
\title{Arrivals: netsim Module}
\usage{
arrivals.2g.net(dat, at)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{Current time step.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function simulates new arrivals into the network
for use in \code{\link[=netsim]{netsim()}} simulations.
}
\seealso{
\code{\link[=netsim]{netsim()}}
}
\keyword{internal}
\keyword{netMod}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{dissolution_coefs}
\alias{dissolution_coefs}
\title{Dissolution Coefficients for Stochastic Network Models}
\usage{
dissolution_coefs(dissolution, duration, d.rate = 0)
}
\arguments{
\item{dissolution}{Right-hand sided STERGM dissolution formula
(see \code{\link[=netest]{netest()}}). See below for list of supported
dissolution models.}

\item{duration}{A vector of mean edge durations in arbitrary time units.}

\item{d.rate}{Departure or exit rate from the population, as a single
homogeneous rate that applies to the entire population.}
}
\value{
A list of class \code{disscoef} with the following elements:
\itemize{
\item \strong{dissolution:} right-hand sided STERGM dissolution formula
passed in the function call.
\item \strong{duration:} mean edge durations passed into the function.
\item \strong{coef.crude:} mean durations transformed into logit
coefficients.
\item \strong{coef.adj:} crude coefficients adjusted for the risk of
departure on edge persistence, if the \code{d.rate} argument is
supplied.
\item \strong{coef.form.corr:} corrections to be subtracted from formation
coefficients.
\item \strong{d.rate:} the departure rate.
\item \strong{diss.model.type:} the form of the dissolution model; options
include \code{edgesonly}, \code{nodematch}, and \code{nodemix}.
}
}
\description{
Calculates dissolution coefficients, given a dissolution model
and average edge duration, to pass as offsets to an ERGM/TERGM
model fit in \code{netest}.
}
\details{
This function performs two calculations for dissolution coefficients
used in a network model estimated with \code{\link[=netest]{netest()}}:
\enumerate{
\item \strong{Transformation:} the mean durations of edges in a network are
mathematically transformed to logit coefficients.
\item \strong{Adjustment:} in a dynamic network simulation in an open
population (in which there are departures), it is further necessary to
adjust these coefficients; this upward adjustment accounts for
departure as a competing risk to edge dissolution.
}

The current dissolution models supported by this function and in network
model estimation in \code{\link[=netest]{netest()}} are as follows:
\itemize{
\item \code{~offset(edges)}: a homogeneous dissolution model in which the
edge duration is the same for all partnerships. This requires
specifying one duration value.
\item \code{~offset(edges) + offset(nodematch("<attr>"))}: a heterogeneous
model in which the edge duration varies by whether the nodes in the
dyad have similar values of a specified attribute. The duration
vector should now contain two values: the first is the mean edge
duration of non-matched dyads, and the second is the duration of the
matched dyads.
\item \code{~offset(edges) + offset(nodemix("<attr>"))}: a heterogeneous
model that extends the nodematch model to include non-binary
attributes for homophily. The duration vector should first contain
the base value, then the values for every other possible combination
in the term.
}
}
\examples{
## Homogeneous dissolution model with no departures
dissolution_coefs(dissolution = ~offset(edges), duration = 25)

## Homogeneous dissolution model with departures
dissolution_coefs(dissolution = ~offset(edges), duration = 25,
                  d.rate = 0.001)

## Heterogeneous dissolution model in which same-race edges have
## shorter duration compared to mixed-race edges, with no departures
dissolution_coefs(dissolution = ~offset(edges) + offset(nodematch("race")),
                  duration = c(20, 10))

## Heterogeneous dissolution model in which same-race edges have
## shorter duration compared to mixed-race edges, with departures
dissolution_coefs(dissolution = ~offset(edges) + offset(nodematch("race")),
                  duration = c(20, 10), d.rate = 0.001)

\dontrun{
## Extended example for differential homophily by age group
# Set up the network with nodes categorized into 5 age groups
nw <- network_initialize(n = 1000)
age.grp <- sample(1:5, 1000, TRUE)
nw <- set_vertex_attribute(nw, "age.grp", age.grp)

# durations = non-matched, age.grp1 & age.grp1, age.grp2 & age.grp2, ...
# TERGM will include differential homophily by age group with nodematch term
# Target stats for the formation model are overall edges, and then the number
# matched within age.grp 1, age.grp 2, ..., age.grp 5
form <- ~edges + nodematch("age.grp", diff = TRUE)
target.stats <- c(450, 100, 125, 40, 80, 100)

# Target stats for the dissolution model are duration of non-matched edges,
# then duration of edges matched within age.grp 1, age.grp 2, ..., age.grp 5
durs <- c(60, 30, 80, 100, 125, 160)
diss <- dissolution_coefs(~offset(edges) +
                            offset(nodematch("age.grp", diff = TRUE)),
                          duration = durs)

# Fit the TERGM
fit <- netest(nw, form, target.stats, diss)

# Full diagnostics to evaluate model fit
dx <- netdx(fit, nsims = 10, ncores = 4, nsteps = 300)
print(dx)

# Simulate one long time series to examine timed edgelist
dx <- netdx(fit, nsims = 1, nsteps = 5000, keep.tedgelist = TRUE)

# Extract timed-edgelist
te <- as.data.frame(dx)
head(te)

# Limit to non-censored edges
te <- te[which(te$onset.censored == FALSE & te$terminus.censored == FALSE),
         c("head", "tail", "duration")]
head(te)

# Look up the age group of head and tail nodes
te$ag.head <- age.grp[te$head]
te$ag.tail <- age.grp[te$tail]
head(te)

# Recover average edge durations for age-group pairing
mean(te$duration[te$ag.head != te$ag.tail])
mean(te$duration[te$ag.head == 1 & te$ag.tail == 1])
mean(te$duration[te$ag.head == 2 & te$ag.tail == 2])
mean(te$duration[te$ag.head == 3 & te$ag.tail == 3])
mean(te$duration[te$ag.head == 4 & te$ag.tail == 4])
mean(te$duration[te$ag.head == 5 & te$ag.tail == 5])
durs
}

}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{get_network_attributes}
\alias{get_network_attributes}
\title{Get Network Attributes from a Network Object}
\usage{
get_network_attributes(x)
}
\arguments{
\item{x}{An object of class \code{network} or \code{networkLite}.}
}
\value{
Returns the named list of network attributes.
}
\description{
Gets all network attributes except \code{"mnext"} from its
network argument.
}
\details{
This function is used in \code{EpiModel} workflows to copy relevant network
attributes from the network object to the \code{netsim_dat} object when
initializing \code{netsim} runs.
}
\examples{
nw <- network_initialize(100)
get_network_attributes(nw)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgelists.R
\name{get_cumulative_edgelists_df}
\alias{get_cumulative_edgelists_df}
\title{Get the Cumulative Edgelists of a Model}
\usage{
get_cumulative_edgelists_df(dat, networks = NULL)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{networks}{Numerical indexes of the networks to extract the partnerships
from. (May be > 1 for models with multiple overlapping
networks.) If \code{NULL}, extract from all networks.}
}
\value{
A \code{data.frame} with 5 columns:
\itemize{
\item \code{index}: the unique ID (see \code{get_unique_ids}) of the
indexes.
\item \code{partner}: the unique ID (see \code{get_unique_ids}) of the
partners/contacts.
\item \code{start}: the time step in which the edge started.
\item \code{stop}: the time step in which the edge stopped; if ongoing,
then \code{NA} is returned.
\item \code{network}: the numerical index for the network on which the
partnership/contact is located.
}
}
\description{
Get the Cumulative Edgelists of a Model
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbose.R
\name{verbose.net}
\alias{verbose.net}
\title{Progress Print Module for Stochastic Network Models}
\usage{
verbose.net(x, type, s = 1, at = 2)
}
\arguments{
\item{x}{If the \code{type} is "startup", then an object of class
\code{control.net}; otherwise, an object of class \code{netsim_dat},
the main data object in \code{\link[=netsim]{netsim()}} simulations.}

\item{type}{Progress type, either of "startup" for starting messages before
all simulations, or "progress" for time step specific messages.}

\item{s}{Current simulation number, if type is "progress".}

\item{at}{Current time step, if type is "progress".}
}
\description{
This function prints progress from stochastic network models
simulated with \code{\link[=netsim]{netsim()}} to the console.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reachable.R
\name{get_connected_nodes}
\alias{get_connected_nodes}
\title{Returns all the node connected directly or indirectly to a set of nodes}
\usage{
get_connected_nodes(adj_list, nodes)
}
\arguments{
\item{adj_list}{The network represented as an adjacency list}

\item{nodes}{A set of nodes}
}
\value{
A vector of nodes indexes that are connected together with the ones
provided in the \code{nodes} argument. The \code{nodes} themselves are not
listed in this output
}
\description{
Returns all the node connected directly or indirectly to a set of nodes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.scenarios.R
\name{create_scenario_list}
\alias{create_scenario_list}
\title{Make a list of EpiModel scenarios from a data.frame of scenarios}
\usage{
create_scenario_list(scenarios.df)
}
\arguments{
\item{scenarios.df}{a \code{data.frame}}
}
\value{
a list of EpiModel scenarios
}
\description{
An EpiModel scenario allows one or multiple set of parameters to be applied
to a model a predefined timesteps. They are usually used by a researcher who
wants to model counterfactuals using a pre calibrated model.
}
\section{scenarios.df}{

The \code{scenarios.df} is a \code{data.frame} of values to be used as
parameters.

It must contain a ".at" column, specifying when the changes should occur.
It requires the "updater" module of EpiModel. \emph{See, vignette}. If the ".at"
value of a row is less than two, the changes will be applied to the
parameter list iteself. The second mandatory column is ".scenario.id". It
is used to distinguish the different scenarios. If multiple rows share the
same ".scenario.id", the resulting scenario will contain one updater per row.
This permits modifying parameters at multiple points in time. (e.g. an
intervention limited in time).

The other column names must correspond either to:
the name of one parameter if this parameter is of size 1 or the name of the
parameter with "_1", "_2", "\emph{N" with the second part being the position of
the value for a parameter of size > 1. This means that the parameter names
cannot contain any underscore "}". (e.g "a.rate", "d.rate_1", "d.rate_2")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.infection.R
\name{discord_edgelist}
\alias{discord_edgelist}
\title{Discordant Edgelist}
\usage{
discord_edgelist(dat, at, network = 1, infstat = "i", include.network = FALSE)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{Current time step.}

\item{network}{In case of models with multiple networks, the network to pull
the current edgelist from. Default of \code{network = 1}.}

\item{infstat}{Character vector of disease status values that are considered
infectious, defining the SI pairs.}

\item{include.network}{Should the \code{network} value be included as the
final column of the discordant edgelist?}
}
\value{
This function returns a \code{data.frame} with the following columns:
\itemize{
\item \strong{time:} time step queried.
\item \strong{sus:} ID number for the susceptible partner.
\item \strong{inf:} ID number for the infectious partner.
}

The output from this function is added to the transmission \code{data.frame}
object that is requested as output in \code{netsim} simulations with
the \code{save.trans=TRUE} argument.
}
\description{
This function returns a \code{data.frame} with a discordant
edgelist, defined as the set of edges in which the status of the
two partners is one susceptible and one infected.
}
\details{
This internal function works within the parent \code{\link[=infection.net]{infection.net()}}
function to pull the current edgelist from the dynamic network object, look
up the disease status of the head and tails on the edge, and subset the list
to those edges with one susceptible and one infected node.

EpiModel v2.0.3 extended the function by allowing flexibility in the
definition what disease status counts as infectious, with the \code{infstat}
parameter. For extension models with multiple infectious states, this can be
a vector of length greater than 1: \code{infstat = c("i", "a")}.
}
\seealso{
\code{\link[=netsim]{netsim()}}, \code{\link[=infection.net]{infection.net()}}
}
\keyword{internal}
\keyword{netMod}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{delete_attr}
\alias{delete_attr}
\title{Delete Elements from Attribute List}
\usage{
delete_attr(dat, ids)
}
\arguments{
\item{dat}{Either the \code{netsim_dat} class object passed through
\code{netsim} simulations, or the main data object passed through
\code{icm} simulations.}

\item{ids}{ID numbers to delete from the list.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Deletes elements from the main attribute list.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveout.R
\name{process_out.net}
\alias{process_out.net}
\title{Save a List of netsim Data to Output List Format}
\usage{
process_out.net(dat_list)
}
\arguments{
\item{dat_list}{A list of main \code{netsim_dat} objects in \code{netsim}
simulations.}
}
\value{
A list of class \code{netsim} with the following elements:
\itemize{
\item \strong{param:} the epidemic parameters passed into the model through
\code{param}, with additional parameters added as necessary.
\item \strong{control:} the control settings passed into the model through
\code{control}, with additional controls added as necessary.
\item \strong{epi:} a list of data frames, one for each epidemiological
output from the model. Outputs for base models always include the
size of each compartment, as well as flows in, out of, and between
compartments.
\item \strong{stats:} a list containing two sublists, \code{nwstats} for any
network statistics saved in the simulation, and \code{transmat} for
the transmission matrix saved in the simulation. See
\code{\link[=control.net]{control.net()}} for further
details.
\item \strong{network:} a list of \code{networkDynamic} objects,
one for each model simulation.
}
}
\description{
This function transfers the data from a list of the main
\code{netsim_dat} objects to the output \code{out} object at the
end of all simulations in \code{\link[=netsim]{netsim()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.inputs.R
\name{update_params}
\alias{update_params}
\title{Update Model Parameters for Stochastic Network Models}
\usage{
update_params(param, new.param.list)
}
\arguments{
\item{param}{Object of class \code{param.net}, output from function of same
name.}

\item{new.param.list}{Named list of new parameters to add to original
parameters.}
}
\value{
An updated list object of class \code{param.net}, which can be passed to the
EpiModel function \code{\link[=netsim]{netsim()}}.
}
\description{
Updates epidemic model parameters originally set with
\code{\link[=param.net]{param.net()}} and adds new parameters.
}
\details{
This function can update any original parameters specified with
\code{\link[=param.net]{param.net()}} and add new parameters. This function would be used
if the inputs to \code{\link[=param.net]{param.net()}} were a long list of fixed model
parameters that needed supplemental replacements or additions for particular
model runs (e.g., changing an intervention efficacy parameter but leaving all
other parameters fixed).

The \code{new.param.list} object should be a named list object containing
named parameters matching those already in \code{x} (in which case those
original parameter values will be replaced) or not matching (in which case
new parameters will be added to \code{param}).

\code{update_params} modifies a \code{param.net} object \strong{before} it is passed
to \code{\link[=netsim]{netsim()}}; it is the recommended helper for tweaking parameters
outside of a running simulation. To modify parameters from \emph{inside} a
module function during a simulation, use \code{\link[=set_param]{set_param()}} on the live
\code{dat} object instead. There is no analogous \code{update_controls()} or
\code{update_inits()} helper: for a \code{control.net} or \code{init.net} object,
edit the list directly (e.g., \code{ctrl$nsteps <- 1000}) or rebuild it
with a fresh call to the constructor. For \emph{scheduled} mid-simulation
changes to parameters or control settings, see the
\code{.param.updater.list} argument to \code{\link[=param.net]{param.net()}} and the
\code{.control.updater.list} argument to \code{\link[=control.net]{control.net()}}.
}
\examples{
x <- param.net(inf.prob = 0.5, act.rate = 2)
y <- list(inf.prob = 0.75, dx.rate = 0.2)
z <- update_params(x, y)
print(z)

}
\seealso{
\code{\link[=set_param]{set_param()}} for editing parameters inside a module function
during a simulation; \code{\link[=param.net]{param.net()}} for the parameter constructor;
\code{\link[=netsim]{netsim()}} for running a simulation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update.R
\name{arrive_nodes}
\alias{arrive_nodes}
\title{Arrive New Nodes to the netsim_dat Object}
\usage{
arrive_nodes(dat, nArrivals)
}
\arguments{
\item{dat}{the \code{netsim_dat} object}

\item{nArrivals}{number of new nodes to arrive}
}
\value{
the updated \code{netsim_dat} object with \code{nArrivals} new nodes
added
}
\description{
Arrive New Nodes to the netsim_dat Object
}
\details{
\code{nArrivals} new nodes are added to the network data stored on
the \code{netsim_dat} object. If \code{tergmLite} is \code{FALSE}, these
nodes are activated from the current timestep onward. Attributes for the new
nodes must be set separately.

Note that this function only supports arriving new nodes; returning to an
active state nodes that were previously active in the network is not
supported.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print_nwstats_table}
\alias{print_nwstats_table}
\title{Print Helper For Network Stats Tables}
\usage{
print_nwstats_table(nwtable, digits)
}
\arguments{
\item{nwtable}{A formation or dissolution statistics \code{data.frame}.}

\item{digits}{Argument to be passed to \code{round}.}
}
\description{
Print Helper For Network Stats Tables
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.init.R
\name{initialize.net}
\alias{initialize.net}
\title{Initialization: netsim Module}
\usage{
initialize.net(x, param, init, control, s)
}
\arguments{
\item{x}{If \code{control$start == 1}, either a fitted network model object
of class \code{netest} or a list of such objects. If
\code{control$start > 1}, an object of class \code{netsim}. When
multiple networks are used, the node sets (including network size
and nodal attributes) are assumed to be the same for all networks.}

\item{param}{An \code{EpiModel} object of class \code{\link[=param.net]{param.net()}}.}

\item{init}{An \code{EpiModel} object of class \code{\link[=init.net]{init.net()}}.}

\item{control}{An \code{EpiModel} object of class \code{\link[=control.net]{control.net()}}.}

\item{s}{Simulation number, used for restarting dependent simulations.}
}
\value{
A \code{netsim_dat} class main data object.
}
\description{
This function initializes the main \code{netsim_dat} class data
object on which data are stored, simulates the initial state of
the networks, and simulates disease status and other attributes.
}
\details{
When re-initializing a simulation, the \code{netsim} object passed
to \code{initialize.net} must contain the elements \code{param},
\code{nwparam}, \code{epi}, \code{coef.form}, and \code{num.nw}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.param_df.R
\name{param.net_to_table}
\alias{param.net_to_table}
\title{Coerce a list of parameters to a \code{long.param.df}}
\usage{
param.net_to_table(params)
}
\arguments{
\item{params}{A list of parameters to be formatted into a \code{long.param.df}}
}
\value{
A \code{data.frame} of parameters.
}
\description{
Coerce a list of parameters to a \code{long.param.df}
}
\section{long.param.df}{

It is possible to set input parameters using a specifically formatted
\code{data.frame} object. The first 3 columns of this \code{data.frame} must
be:
\itemize{
\item \code{param}: The name of the parameter. If this is a non-scalar
parameter (a vector of length > 1), end the parameter name with the
position on the vector (e.g., \code{"p_1"}, \code{"p_2"}, ...).
\item \code{value}: the value for the parameter (or the value of the
parameter in the Nth position if non-scalar).
\item \code{type}: a character string containing either \code{"numeric"},
\code{"logical"}, or \code{"character"} to define the parameter object
class.
}

In addition to these 3 columns, the \code{data.frame} can contain any number
of other columns, such as \code{details} or \code{source} columns to document
parameter meta-data. However, these extra columns will not be used by
EpiModel.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reachable.R
\name{as_cumulative_edgelist}
\alias{as_cumulative_edgelist}
\title{Convert an object to a \code{cumulative_edgelist}}
\usage{
as_cumulative_edgelist(x)
}
\arguments{
\item{x}{An object to be converted to a cumulative edgelist}
}
\value{
A \code{cumulative_edgelist} object, a \code{data.frame} with at least the
following columns: \code{head}, \code{tail}, \code{start}, \code{stop}.
}
\description{
Convert an object to a \code{cumulative_edgelist}
}
\details{
The edges are active from time \code{start} to time \code{stop} included. If stop is
\code{NA}, the edge was not disolved in the simulation that generated the list.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test_icm}
\alias{test_icm}
\title{Test the Model Output from a Stochastic Individual Contact Model}
\usage{
test_icm(x)
}
\arguments{
\item{x}{An object of class \code{icm}.}
}
\description{
Tests whether the model output from an individual contact model
is consistent with key balancing equations for compartment and
flow sizes for each simulation for each time step.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.accessor.R
\name{is_active_posit_ids}
\alias{is_active_posit_ids}
\title{Are These Nodes Active (Positional IDs)}
\usage{
is_active_posit_ids(dat, posit_ids)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{posit_ids}{A vector of node positional identifiers.}
}
\value{
A logical vector with TRUE if the node is still active and FALSE
otherwise.
}
\description{
Are These Nodes Active (Positional IDs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.inputs.R
\name{init.net}
\alias{init.net}
\title{Initial Conditions for Stochastic Network Models}
\usage{
init.net(i.num, r.num, i.num.g2, r.num.g2, status.vector, infTime.vector, ...)
}
\arguments{
\item{i.num}{Number of initial infected persons. For two-group models, this
is the number of initial group 1 infected persons.}

\item{r.num}{Number of initial recovered persons. For two-group models, this
is the number of initial group 1 recovered persons. This parameter is
only used for the \code{SIR} model type.}

\item{i.num.g2}{Number of initial infected persons in group 2. This parameter
is only used for two-group models.}

\item{r.num.g2}{Number of initial recovered persons in group 2. This
parameter is only used for two-group \code{SIR} models.}

\item{status.vector}{A vector of length equal to the size of the input
network, containing the status of each node. Setting status here
overrides any inputs passed in the \code{.num} arguments.}

\item{infTime.vector}{A vector of length equal to the size of the input
network, containing the (historical) time of infection for each of
those nodes with a current status of \code{"i"}. Can only be used if
\code{status.vector} is used, and must contain \code{NA} values for
any nodes whose status is not \code{"i"}.}

\item{...}{Additional initial conditions passed to model.}
}
\value{
An \code{EpiModel} object of class \code{init.net}.
}
\description{
Sets the initial conditions for stochastic network models
simulated with \code{netsim}.
}
\details{
The initial conditions for a model solved with \code{\link[=netsim]{netsim()}} should be
input into the \code{init.net} function. This function handles initial
conditions for both base models and new modules. For an overview of
specifying initial conditions across a variety of base network models,
consult the \href{https://epimodel.github.io/sismid/}{Network Modeling for Epidemics}
tutorials.
}
\examples{
# Example of using status.vector and infTime.vector together
n <- 100
status <- sample(c("s", "i"), size = n, replace = TRUE, prob = c(0.8, 0.2))
infTime <- rep(NA, n)
infTime[which(status == "i")] <- -rgeom(sum(status == "i"), prob = 0.01) + 2

init.net(status.vector = status, infTime.vector = infTime)

}
\seealso{
Use \code{\link[=param.net]{param.net()}} to specify model parameters and
\code{\link[=control.net]{control.net()}} to specify the control settings. Run the
parameterized model with \code{\link[=netsim]{netsim()}}.
}
\keyword{parameterization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.updater.R
\name{update_list}
\alias{update_list}
\title{Update List \code{x} Using the Elements of List \code{new.x}.}
\usage{
update_list(x, new.x)
}
\arguments{
\item{x}{A list.}

\item{new.x}{A list.}
}
\value{
The full \code{x} list with the modifications added by \code{new.x}.
}
\description{
Update List \code{x} Using the Elements of List \code{new.x}.
}
\details{
This function updates list \code{x} by name. If \code{x} and \code{new.x}
elements are not named, the function will not work properly. If a function is
provided to replace an element that was originally not a function, this
function will be applied to the original value.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.inputs.R
\name{param_random}
\alias{param_random}
\title{Create a Value Sampler for Random Parameters}
\usage{
param_random(values, prob = NULL)
}
\arguments{
\item{values}{A vector of values to sample from.}

\item{prob}{A vector of weights to use during sampling. If \code{NULL},
all values have the same probability of being picked
(default = \code{NULL}).}
}
\value{
A 0 argument generator function to sample one of the values from the
\code{values} vector.
}
\description{
This function returns a 0 argument function that can be used as
a generator function in the \code{random.params} argument of the
\code{\link[=param.net]{param.net()}} function.
}
\examples{
# Define function with equal sampling probability
a <- param_random(1:5)
a()

# Define function with unequal sampling probability
b <- param_random(1:5, prob = c(0.1, 0.1, 0.1, 0.1, 0.6))
b()

}
\seealso{
\code{\link[=param.net]{param.net()}} and \code{\link[=generate_random_params]{generate_random_params()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.inputs.R
\name{generate_random_params}
\alias{generate_random_params}
\title{Generate Values for Random Parameters}
\usage{
generate_random_params(param, verbose = FALSE)
}
\arguments{
\item{param}{The \code{param} argument received by the \code{netsim}
functions.}

\item{verbose}{Should the function output the generated values
(default = FALSE)?}
}
\value{
A fully instantiated \code{param} list.
}
\description{
This function uses the generative functions in the
\code{random.params} list to create values for the parameters.
}
\section{\code{random.params}}{

The \code{random.params} argument to the \code{\link[=param.net]{param.net()}} function
must be a named list of functions that each return a value that can be used
as the argument with the same name. In the example below, \code{param_random}
is a function factory provided by EpiModel for \code{act.rate} and
for \code{tx.halt.part.prob} we provide bespoke functions. A function factory
is a function that returns a new function
(see https://adv-r.hadley.nz/function-factories.html).
}

\section{Generator Functions}{

The functions used inside \code{random_params} must be 0 argument functions
returning a valid value for the parameter with the same name.
}

\section{\code{param_random_set}}{

The \code{random_params} list can optionally contain a
\code{param_random_set} element. It must be a \code{data.frame} of possible
values to be used as parameters.

The column names must correspond either to:
the name of one parameter, if this parameter is of size 1; or the name of one
parameter with "_1", "\emph{2", etc. appended, with the number representing the
position of the value, if this parameter is of size > 1. This means that the
parameter names cannot contain any underscores "}" if you intend to use
\code{param_random_set}.

The point of the \code{param.random.set} \code{data.frame} is to allow the
random parameters to be correlated. To achieve this, a whole row of the
\code{data.frame} is selected for each simulation.
}

\examples{
\dontrun{

## Example with only the generator function

# Define random parameter list
my_randoms <- list(
  act.rate = param_random(c(0.25, 0.5, 0.75)),
  tx.prob = function() rbeta(1, 1, 2),
  stratified.test.rate = function() c(
    rnorm(1, 0.05, 0.01),
    rnorm(1, 0.15, 0.03),
    rnorm(1, 0.25, 0.05)
  )
)

# Parameter model with fixed and random parameters
param <- param.net(inf.prob = 0.3, random.params = my_randoms)

# Below, `tx.prob` is set first to 0.3 then assigned a random value using
# the function from `my_randoms`. A warning notifying of this overwrite is
# therefore produced.
param <- param.net(tx.prob = 0.3, random.params = my_randoms)


# Parameters are drawn automatically in netsim by calling the function
# within netsim_loop. Demonstrating draws here but this is not used by
# end user.
paramDraw <- generate_random_params(param, verbose = TRUE)
paramDraw


## Addition of the `param.random.set` `data.frame`

# This function will generate sets of correlated parameters
 generate_correlated_params <- function() {
   param.unique <- runif(1)
   param.set.1 <- param.unique + runif(2)
   param.set.2 <- param.unique * rnorm(3)

   return(list(param.unique, param.set.1, param.set.2))
 }

 # Data.frame set of random parameters :
 correlated_params <- t(replicate(10, unlist(generate_correlated_params())))
 correlated_params <- as.data.frame(correlated_params)
 colnames(correlated_params) <- c(
   "param.unique",
   "param.set.1_1", "param.set.1_2",
   "param.set.2_1", "param.set.2_2", "param.set.2_3"
 )

# Define random parameter list with the `param.random.set` element
my_randoms <- list(
  act.rate = param_random(c(0.25, 0.5, 0.75)),
  param.random.set = correlated_params
)

# Parameter model with fixed and random parameters
param <- param.net(inf.prob = 0.3, random.params = my_randoms)

# Parameters are drawn automatically in netsim by calling the function
# within netsim_loop. Demonstrating draws here but this is not used by
# end user.
paramDraw <- generate_random_params(param, verbose = TRUE)
paramDraw

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.R
\name{get_param_set}
\alias{get_param_set}
\title{Extract the Parameter Set from Network Simulations}
\usage{
get_param_set(sims)
}
\arguments{
\item{sims}{An \code{EpiModel} object of class \code{netsim}.}
}
\value{
A \code{data.frame} with one row per simulation and one column per
parameter or parameter element where the parameters are of size > 1.
}
\description{
Extract the Parameter Set from Network Simulations
}
\section{Output Format}{

The outputted \code{data.frame} has one row per simulation and the columns
correspond to the parameters used in this simulation.

The column name will match the parameter name if it is a size 1 parameter or
if the parameter is of size > 1, there will be N columns (with N being the
size of the parameter) named \code{parameter.name_1},
\code{parameter.name_2}, ..., \code{parameter.name_N}.
}

\examples{
\donttest{
# Setup network
nw <- network_initialize(n = 50)

est <- netest(
  nw, formation = ~edges,
  target.stats = c(25),
  coef.diss = dissolution_coefs(~offset(edges), 10, 0),
  verbose = FALSE
)

init <- init.net(i.num = 10)

n <- 5

related.param <- data.frame(
  dummy.param = rbeta(n, 1, 2)
)

 my.randoms <- list(
   act.rate = param_random(c(0.25, 0.5, 0.75)),
   dummy.param = function() rbeta(1, 1, 2),
   dummy.strat.param = function() c(
     rnorm(1, 0, 10),
     rnorm(1, 10, 1)
   )
 )

param <- param.net(
  inf.prob = 0.3,
  dummy = c(0, 1, 2),
  random.params = my.randoms
)

control <- control.net(type = "SI", nsims = 3, nsteps = 5, verbose = FALSE)
mod <- netsim(est, param, init, control)

get_param_set(mod)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{check_degdist_bal}
\alias{check_degdist_bal}
\title{Check Degree Distribution for Balance in Target Statistics}
\usage{
check_degdist_bal(num.g1, num.g2, deg.dist.g1, deg.dist.g2)
}
\arguments{
\item{num.g1}{Number of nodes in group 1.}

\item{num.g2}{Number of nodes in group 2.}

\item{deg.dist.g1}{Vector with fractional degree distribution for group 1.}

\item{deg.dist.g2}{Vector with fractional degree distribution for group 2.}
}
\description{
Checks for consistency in the implied network statistics
of a two-group network in which the group size and
group-specific degree distributions are specified.
}
\details{
This function outputs the number of nodes of degree 0 to g, where g is the
length of a fractional degree distribution vector, given that vector and the
size of the group. This utility is used to check for balance in implied
degree given that fractional distribution within two-group network
simulations, in which the degree-constrained counts must be equal across
groups.
}
\examples{
# An unbalanced distribution
check_degdist_bal(num.g1 = 500, num.g2 = 500,
                  deg.dist.g2 = c(0.40, 0.55, 0.03, 0.02),
                  deg.dist.g1 = c(0.48, 0.41, 0.08, 0.03))

# A balanced distribution
check_degdist_bal(num.g1 = 500, num.g2 = 500,
                  deg.dist.g1 = c(0.40, 0.55, 0.04, 0.01),
                  deg.dist.g2 = c(0.48, 0.41, 0.08, 0.03))

}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgelists.R
\name{get_edgelists_df}
\alias{get_edgelists_df}
\title{Get the Edgelist(s) from the Specified Network(s)}
\usage{
get_edgelists_df(dat, networks = NULL)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{networks}{Numerical indexes of the networks to extract the partnerships
from. (May be > 1 for models with multiple overlapping
networks.) If \code{NULL}, extract from all networks.}
}
\value{
A \code{data.frame} with the following columns:
\itemize{
\item \code{head}: Positional ID of the head node.
\item \code{tail}: Positional ID of the tail node.
\item \code{network}: The numerical index of the network on which the edge is located.
}
}
\description{
Get the Edgelist(s) from the Specified Network(s)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{auto_update_attr}
\alias{auto_update_attr}
\title{Update Vertex Attributes for Incoming Vertices}
\usage{
auto_update_attr(dat, newNodes, curr.tab)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{newNodes}{Vector of nodal IDs for incoming nodes at the current time
step.}

\item{curr.tab}{Current proportional distribution of all vertex attributes.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Updates the vertex attributes on a network for new nodes
incoming into that network, based on a set of rules for each
attribute that the user specifies in \code{\link[=control.net]{control.net()}}.
}
\seealso{
\code{\link[=copy_nwattr_to_datattr]{copy_nwattr_to_datattr()}}, \code{\link[=get_attr_prop]{get_attr_prop()}},
\code{\link[=auto_update_attr]{auto_update_attr()}}.
}
\keyword{internal}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.logs.R
\name{netsim_cond_msg}
\alias{netsim_cond_msg}
\title{Message to Find in Which Module a \code{condition} Occurred}
\usage{
netsim_cond_msg(cond, module, at, msg)
}
\arguments{
\item{cond}{The type of \code{condition} handled (message, warning, error).}

\item{module}{The name of the module where the \code{condition} occurred.}

\item{at}{The time step the \code{condition} occurred.}

\item{msg}{The \code{condition}'s message.}
}
\value{
A formatted string describing where and when the \code{condition}
occurred as well as the \code{condition}'s message.
}
\description{
This function returns a formatted string describing when, where,
and why an error, message, or warning occurred.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{truncate_sim}
\alias{truncate_sim}
\alias{truncate_sim.dcm}
\alias{truncate_sim.icm}
\alias{truncate_sim.netsim}
\title{Truncate Simulation Time Series}
\usage{
truncate_sim(x, at, reset.time)

\method{truncate_sim}{dcm}(x, at, reset.time = TRUE)

\method{truncate_sim}{icm}(x, at, reset.time = TRUE)

\method{truncate_sim}{netsim}(x, at, reset.time = TRUE)
}
\arguments{
\item{x}{Object of class \code{dcm}, \code{netsim}, or \code{icm}.}

\item{at}{Time step at which to left-truncate the time series.}

\item{reset.time}{If \code{TRUE}, the time step sequence in the truncated model
will be reset to start at 1. If \code{FALSE}, the original time step
values will be preserved. Default is \code{TRUE}.}
}
\value{
The updated object of class \code{dcm}, \code{netsim}, or \code{icm}.
}
\description{
Left-truncates simulation epidemiological summary statistics
and network statistics at a specified time step.
}
\details{
This function would be used when running a follow-up simulation from time
steps \code{b} to \code{c} after a burn-in period from time \code{a} to
\code{b}, where the final time window of interest for data analysis is
\code{b} to \code{c} only.
}
\examples{
# DCM examples
param <- param.dcm(inf.prob = 0.2, act.rate = 0.25)
init <- init.dcm(s.num = 500, i.num = 1)
control <- control.dcm(type = "SI", nsteps = 200)
mod1 <- dcm(param, init, control)
plot(mod1)

# Reset time
mod2a <- truncate_sim(mod1, at = 150)
plot(mod2a)
head(as.data.frame(mod2a))

# Do not reset time
mod2b <- truncate_sim(mod1, at = 150, reset.time = FALSE)
plot(mod2b)
head(as.data.frame(mod2b))

\donttest{
# ICM example
param <- param.icm(inf.prob = 0.2, act.rate = 0.25)
init <- init.icm(s.num = 500, i.num = 1)
control <- control.icm(type = "SI", nsteps = 200, nsims = 1)
mod1 <- icm(param, init, control)

# Reset time
mod2a <- truncate_sim(mod1, at = 150)
plot(mod2a)
head(as.data.frame(mod2a))

# Do not reset time
mod2b <- truncate_sim(mod1, at = 150, reset.time = FALSE)
plot(mod2b)
head(as.data.frame(mod2b))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update.R
\name{delete_edges}
\alias{delete_edges}
\title{Remove Edges That Include Specified Vertices}
\usage{
delete_edges(el, vid)
}
\arguments{
\item{el}{A two-column matrix of current edges (edgelist).}

\item{vid}{A vector of vertex IDs whose edges are to be deleted from the
edgelist.}
}
\value{
Returns an updated edgelist object, with any edges including the specified
vertices removed.
}
\description{
Given a current two-column matrix of edges and a vector of
vertex IDs, this function removes any rows of the edgelist in
which the IDs are present.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.epi.data.frame.R
\name{plot.epi.data.frame}
\alias{plot.epi.data.frame}
\title{Plot Epidemic Model Results From a Netsim Data.Frame}
\usage{
\method{plot}{epi.data.frame}(
  x,
  y = NULL,
  sims = NULL,
  legend = NULL,
  mean.col = NULL,
  qnts.col = NULL,
  sim.lwd = NULL,
  sim.col = NULL,
  sim.alpha = NULL,
  popfrac = FALSE,
  qnts = 0.5,
  qnts.alpha = 0.5,
  qnts.smooth = TRUE,
  mean.line = TRUE,
  mean.smooth = TRUE,
  add = FALSE,
  mean.lwd = 2,
  mean.lty = 1,
  xlim = NULL,
  ylim = NULL,
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  sim.lines = FALSE,
  grid = FALSE,
  leg.cex = 0.8,
  ...
)
}
\arguments{
\item{x}{A \code{data.frame} obtain with \code{as.data.frame(netsim_object)}.}

\item{y}{Output compartments or flows from \code{netsim} object to plot.}

\item{sims}{If \code{type="epi"} or \code{"formation"}, a vector of
simulation numbers to plot. If \code{type="network"}, a single
simulation number for which to plot the network, or else \code{"min"}
to plot the simulation number with the lowest disease prevalence,
\code{"max"} for the simulation with the highest disease prevalence,
or \code{"mean"} for the simulation with the prevalence closest to the
mean across simulations at the specified time step.}

\item{legend}{If \code{TRUE}, plot default legend.}

\item{mean.col}{Vector of any standard R color format for mean lines.}

\item{qnts.col}{Vector of any standard R color format for polygons.}

\item{sim.lwd}{Line width for simulation lines.}

\item{sim.col}{Vector of any standard R color format for simulation lines.}

\item{sim.alpha}{Transparency level for simulation lines, where
0 = transparent and 1 = opaque (see \code{adjustcolor} function).}

\item{popfrac}{If \code{TRUE}, plot prevalence of values rather than numbers
(see details).}

\item{qnts}{If numeric, plot polygon of simulation quantiles based on the
range implied by the argument (see details). If \code{FALSE}, suppress
polygon from plot.}

\item{qnts.alpha}{Transparency level for quantile polygons, where 0 =
transparent and 1 = opaque (see \code{adjustcolor} function).}

\item{qnts.smooth}{If \code{TRUE}, use a loess smoother on quantile polygons.}

\item{mean.line}{If \code{TRUE}, plot mean of simulations across time.}

\item{mean.smooth}{If \code{TRUE}, use a loess smoother on the mean line.}

\item{add}{If \code{TRUE}, new plot window is not called and lines are added
to existing plot window.}

\item{mean.lwd}{Line width for mean lines.}

\item{mean.lty}{Line type for mean lines.}

\item{xlim}{the x limits (x1, x2) of the plot.  Note that \code{x1 > x2}
    is allowed and leads to a \sQuote{reversed axis}.

    The default value, \code{NULL}, indicates that the range of the
    \link{finite} values to be plotted should be used.}

\item{ylim}{the y limits of the plot.}

\item{main}{a main title for the plot, see also \code{\link[graphics]{title}}.}

\item{xlab}{a label for the x axis, defaults to a description of \code{x}.}

\item{ylab}{a label for the y axis, defaults to a description of \code{y}.}

\item{sim.lines}{If \code{TRUE}, plot individual simulation lines. Default is
to plot lines for one-group models but not for two-group models.}

\item{grid}{If \code{TRUE}, a grid is added to the background of plot
(see \code{\link[=grid]{grid()}} for details), with default of nx by ny.}

\item{leg.cex}{Legend scale size.}

\item{...}{Additional arguments to pass.}
}
\description{
This function is a wrapper around \code{plot.netsim} accepting a
\code{data.frame} obtain with \code{as.data.frame(netsim_object)}.
}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.vital.R
\name{departures.2g.net}
\alias{departures.2g.net}
\title{Departures: netsim Module}
\usage{
departures.2g.net(dat, at)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{Current time step.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function simulates departure for use in \link{netsim}
simulations.
}
\seealso{
\code{\link[=netsim]{netsim()}}
}
\keyword{internal}
\keyword{netMod}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{get_network_term_attr}
\alias{get_network_term_attr}
\title{Output Network Attributes into a Character Vector}
\usage{
get_network_term_attr(nw)
}
\arguments{
\item{nw}{A network object.}
}
\value{
A character vector of vertex attributes.
}
\description{
Given a simulated network, outputs a character vector of vertex
attributes to be used in \code{\link[=netsim]{netsim()}} simulations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendo.R
\name{as.network.transmat}
\alias{as.network.transmat}
\title{Convert transmat Infection Tree into a network Object}
\usage{
\method{as.network}{transmat}(x, ...)
}
\arguments{
\item{x}{An object of class \code{transmat} to be converted into a
\code{network} class object.}

\item{...}{Unused.}
}
\value{
A \code{\link[network:network]{network::network}} object.
}
\description{
Converts a transmission matrix from the \code{get_transmat}
function into a \code{\link[network:network]{network::network}} class object.
}
\details{
When converting from a \code{transmat} to a \code{network} object, this
functions copies the edge attributes within the transmission matrix
(\code{'at'}, \code{'infDur'}, \code{'transProb'}, \code{'actRate'}, and
\code{'finalProb'}) into edge attributes on the network.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.R
\name{as.data.frame.netdx}
\alias{as.data.frame.netdx}
\title{Extract Timed Edgelists for netdx Objects}
\usage{
\method{as.data.frame}{netdx}(x, row.names = NULL, optional = FALSE, sim = NULL, ...)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{netdx}.}

\item{row.names}{See \code{\link[=as.data.frame.default]{as.data.frame.default()}}.}

\item{optional}{See \code{\link[=as.data.frame.default]{as.data.frame.default()}}.}

\item{sim}{The simulation number to output. If \code{NULL}, then data from
all simulations will be output.}

\item{...}{See \code{\link[=as.data.frame.default]{as.data.frame.default()}}.}
}
\value{
A data frame containing the data from \code{x}.
}
\description{
This function extracts timed edgelists for objects of class
\code{netdx} into a data frame using the generic
\code{as.data.frame} function.
}
\examples{
\donttest{
# Initialize and parameterize the network model
nw <- network_initialize(n = 100)
formation <- ~edges
target.stats <- 50
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 20)

# Model estimation
est <- netest(nw, formation, target.stats, coef.diss, verbose = FALSE)

# Simulate the network with netdx
dx <- netdx(est, nsims = 3, nsteps = 10, keep.tedgelist = TRUE,
            verbose = FALSE)

# Extract data from the first simulation
as.data.frame(dx, sim = 1)

# Extract data from all simulations
as.data.frame(dx)
}

}
\keyword{extract}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgelists.R
\name{get_cumulative_edgelist}
\alias{get_cumulative_edgelist}
\title{Get a Cumulative Edgelist From a Specified Network}
\usage{
get_cumulative_edgelist(dat, network)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{network}{Numerical index of the network from which the cumulative
edgelist should be extracted. (May be > 1 for models with
multiple overlapping networks.)}
}
\value{
A cumulative edgelist in \code{data.frame} form with 4 columns:
\itemize{
\item \code{head}: the unique ID (see \code{get_unique_ids}) of the
head node on the edge.
\item \code{tail}: the unique ID (see \code{get_unique_ids}) of the
tail node on the edge.
\item \code{start}: the time step in which the edge started.
\item \code{stop}: the time step in which the edge stopped; if ongoing,
then \code{NA} is returned.
}
}
\description{
Get a Cumulative Edgelist From a Specified Network
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge.R
\name{merge.netsim}
\alias{merge.netsim}
\title{Merge Model Simulations across netsim Objects}
\usage{
\method{merge}{netsim}(
  x,
  y,
  keep.transmat = TRUE,
  keep.network = TRUE,
  keep.nwstats = TRUE,
  keep.other = TRUE,
  param.error = TRUE,
  keep.diss.stats = TRUE,
  ...
)
}
\arguments{
\item{x}{An \code{EpiModel} object of class \code{\link[=netsim]{netsim()}}.}

\item{y}{Another \code{EpiModel} object of class \code{\link[=netsim]{netsim()}},
with the identical model parameterization as \code{x}.}

\item{keep.transmat}{If \code{TRUE}, keep the transmission matrices from the
original \code{x} and \code{y} elements. Note: transmission matrices
only saved when (\code{save.transmat == TRUE}).}

\item{keep.network}{If \code{TRUE}, keep the \code{networkDynamic} objects
from the original \code{x} and \code{y} elements. Note: network
only saved when (\code{tergmLite == FALSE}).}

\item{keep.nwstats}{If \code{TRUE}, keep the network statistics (as set by
the \code{nwstats.formula} parameter in \code{control.net}) from
the original \code{x} and \code{y} elements.}

\item{keep.other}{If \code{TRUE}, keep the other simulation elements (as set
by the \code{save.other} parameter in \code{control.net}) from the
original \code{x} and \code{y} elements.}

\item{param.error}{If \code{TRUE}, if \code{x} and \code{y} have different
params (in \code{\link[=param.net]{param.net()}}) or controls (passed in
\code{\link[=control.net]{control.net()}}) an error will prevent the merge. Use
\code{FALSE} to override that check.}

\item{keep.diss.stats}{If \code{TRUE}, keep \code{diss.stats} from the
original \code{x} and \code{y} objects.}

\item{...}{Additional merge arguments (not currently used).}
}
\value{
An \code{EpiModel} object of class \code{\link[=netsim]{netsim()}} containing
the data from both \code{x} and \code{y}.
}
\description{
Merges epidemiological data from two independent simulations of
stochastic network models from \code{netsim}.
}
\details{
This merge function combines the results of two independent simulations of
\code{\link[=netsim]{netsim()}} class models, simulated under separate function calls.
The model parameterization between the two calls must be exactly the same,
except for the number of simulations in each call. This allows for manual
parallelization of model simulations.

This merge function does not work the same as the default merge, which allows
for a combined object where the structure differs between the input elements.
Instead, the function checks that objects are identical in model
parameterization in every respect (except number of simulations) and binds
the results.
}
\examples{
\donttest{
# Network model
nw <- network_initialize(n = 100)
coef.diss <- dissolution_coefs(dissolution = ~offset(edges), duration = 10)
est <- netest(nw, formation = ~edges, target.stats = 25,
              coef.diss = coef.diss, verbose = FALSE)

# Epidemic models
param <- param.net(inf.prob = 1)
init <- init.net(i.num = 1)
control <- control.net(type = "SI", nsteps = 20, nsims = 2,
                       save.nwstats = TRUE,
                       nwstats.formula = ~edges + degree(0),
                       verbose = FALSE)
x <- netsim(est, param, init, control)
y <- netsim(est, param, init, control)

# Merging
z <- merge(x, y)

# Examine separate and merged data
as.data.frame(x)
as.data.frame(y)
as.data.frame(z)
}

}
\keyword{extract}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_bands.R
\name{geom_bands}
\alias{geom_bands}
\title{ggplot2 Geom for Quantile Bands}
\usage{
geom_bands(mapping, lower = 0.25, upper = 0.75, alpha = 0.25, ...)
}
\arguments{
\item{mapping}{Standard aesthetic mapping \code{aes()} input for ggplot2.}

\item{lower}{Lower quantile for the time series.}

\item{upper}{Upper quantile for the time series.}

\item{alpha}{Transparency of the ribbon fill.}

\item{...}{Additional arguments passed to \code{stat_summary}.}
}
\description{
Plots quantile bands given a data.frame with stochastic model
results from \code{\link[=icm]{icm()}} or \code{\link[=netsim]{netsim()}}.
}
\details{
This is a wrapper around \code{ggplot::stat_summary} with a ribbon geom as
aesthetic output.
}
\examples{
\donttest{
param <- param.icm(inf.prob = 0.2, act.rate = 0.25)
init <- init.icm(s.num = 500, i.num = 1)
control <- control.icm(type = "SI", nsteps = 250, nsims = 5)
mod1 <- icm(param, init, control)
df <- as.data.frame(mod1)
df.mean <- as.data.frame(mod1, out = "mean")

library(ggplot2)
ggplot() +
   geom_line(data = df, mapping = aes(time, i.num, group = sim),
   alpha = 0.25, lwd = 0.25, color = "firebrick") +
   geom_bands(data = df, mapping = aes(time, i.num),
              lower = 0.1, upper = 0.9, fill = "firebrick") +
   geom_line(data = df.mean, mapping = aes(time, i.num)) +
   theme_minimal()
}

}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{idgroup}
\alias{idgroup}
\title{Group Numbers for Two-Group Network}
\usage{
idgroup(nw, ids = NULL)
}
\arguments{
\item{nw}{Object of class \code{network} or \code{networkDynamic}.}

\item{ids}{Vector of ID numbers for which the group number
should be returned. If \code{NULL} (default), return all IDs.}
}
\value{
A vector containing the group number for each of the specified nodes.
}
\description{
Outputs group numbers given ID numbers for a two-group network.
}
\examples{
nw <- network_initialize(n = 10)
nw <- set_vertex_attribute(nw, "group", rep(1:2, each = 5))
idgroup(nw)
idgroup(nw, ids = c(3, 6))

}
\keyword{internal}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.utils.R
\name{get_degree}
\alias{get_degree}
\title{Get Individual Degree from Network or Edgelist}
\usage{
get_degree(x)
}
\arguments{
\item{x}{Either an object of class \code{network} or \code{edgelist}
generated from a network. If \code{x} is an edgelist, then it must
contain an attribute for the total network size, \code{n}.}
}
\value{
A vector of length equal to the total network size, containing the
current degree of each node in the network.
}
\description{
A fast method for querying the current degree of all individuals
within a network.
}
\details{
Individual-level data on the current degree of nodes within a network is
often useful for summary statistics. Given a \code{network} class object,
\code{net}, one way to look up the current degree is to get a summary of the
ERGM term, \code{sociality}, as in:
\code{summary(net ~ sociality(nodes = NULL))}. But that is computationally
inefficient for a number of reasons. This function provides a fast method for
generating the vector of degrees using a query of the edgelist. It is even
faster if the parameter \code{x} is already transformed into an edgelist.
}
\examples{
\donttest{
nw <- network_initialize(n = 500)

set.seed(1)
fit <- ergm(nw ~ edges, target.stats = 250)
sim <- simulate(fit)

# Slow ERGM-based method
ergm.method <- unname(summary(sim ~ sociality(nodes = NULL)))
ergm.method

# Fast tabulate method with network object
deg.net <- get_degree(sim)
deg.net

# Even faster if network already transformed into an edgelist
el <- as.edgelist(sim)
deg.el <- get_degree(el)
deg.el

identical(as.integer(ergm.method), deg.net, deg.el)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reachable.R
\name{get_adj_list}
\alias{get_adj_list}
\title{Returns an adjacency list from an edge list}
\usage{
get_adj_list(el, n_nodes)
}
\arguments{
\item{el}{An edge list as a data.frame with columns \code{head} and \code{tail}}

\item{n_nodes}{The size number of node in the network}
}
\value{
An adjacency list for the network
}
\description{
Returns an adjacency list from an edge list
}
\details{
The adjacency list is a \code{list} of length \code{n_nodes}. The entry for each node
is a integer vector containing the index of all the nodes connected to it.
This layout makes it directly subsetable in O(1) at the expanse of memory
usage.
To get all connections to the nodes 10 and 15 : \verb{unlist(adj_list[c(10, 15)]}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icm.R
\name{icm}
\alias{icm}
\title{Stochastic Individual Contact Models}
\usage{
icm(param, init, control)
}
\arguments{
\item{param}{Model parameters, as an object of class \code{\link[=param.icm]{param.icm()}}.}

\item{init}{Initial conditions, as an object of class \code{\link[=init.icm]{init.icm()}}.}

\item{control}{Control settings, as an object of class
\code{\link[=control.icm]{control.icm()}}.}
}
\value{
A list of class \code{icm} with the following elements:
\itemize{
\item \strong{param:} the epidemic parameters passed into the model through
\code{param}, with additional parameters added as necessary.
\item \strong{control:} the control settings passed into the model through
\code{control}, with additional controls added as necessary.
\item \strong{epi:} a list of data frames, one for each epidemiological
output from the model. Outputs for base models always include the
size of each compartment, as well as flows in, out of, and between
compartments.
}
}
\description{
Simulates stochastic individual contact epidemic models for
infectious disease.
}
\details{
Individual contact models are intended to be the stochastic microsimulation
analogs to deterministic compartmental models. ICMs simulate disease spread
on individual agents in discrete time as a function of processes with
stochastic variation. The stochasticity is inherent in all transition
processes: infection, recovery, and demographics.

The \code{icm} function performs modeling of both the base model types
and original models. Base model types include one-group and two-group
models with disease types for Susceptible-Infected (SI),
Susceptible-Infected-Recovered (SIR), and Susceptible-Infected-Susceptible
(SIS). Original models may be built by writing new process modules that
either take the place of existing modules (for example, disease recovery),
or supplement the set of existing processes with a new one contained in an
original module.
}
\examples{
\dontrun{
## Example 1: SI Model
param <- param.icm(inf.prob = 0.2, act.rate = 0.25)
init <- init.icm(s.num = 500, i.num = 1)
control <- control.icm(type = "SI", nsteps = 500, nsims = 10)
mod1 <- icm(param, init, control)
mod1
plot(mod1)

## Example 2: SIR Model
param <- param.icm(inf.prob = 0.2, act.rate = 0.25, rec.rate = 1/50)
init <- init.icm(s.num = 500, i.num = 1, r.num = 0)
control <- control.icm(type = "SIR", nsteps = 500, nsims = 10)
mod2 <- icm(param, init, control)
mod2
plot(mod2)

## Example 3: SIS Model
param <- param.icm(inf.prob = 0.2, act.rate = 0.25, rec.rate = 1/50)
init <- init.icm(s.num = 500, i.num = 1)
control <- control.icm(type = "SIS", nsteps = 500, nsims = 10)
mod3 <- icm(param, init, control)
mod3
plot(mod3)

## Example 4: SI Model with Vital Dynamics (Two-Group)
param <- param.icm(inf.prob = 0.4,  inf.prob.g2 = 0.1,
                   act.rate = 0.25, balance = "g1",
                   a.rate = 1/100, a.rate.g2 = NA,
                   ds.rate = 1/100, ds.rate.g2 = 1/100,
                   di.rate = 1/50, di.rate.g2 = 1/50)
init <- init.icm(s.num = 500, i.num = 1,
                 s.num.g2 = 500, i.num.g2 = 0)
control <- control.icm(type = "SI", nsteps = 500, nsims = 10)
mod4 <- icm(param, init, control)
mod4
plot(mod4)
}

}
\seealso{
Extract the model results with \code{\link[=as.data.frame.icm]{as.data.frame.icm()}}.
Summarize the time-specific model results with \code{\link[=summary.icm]{summary.icm()}}.
Plot the model results with \code{\link[=plot.icm]{plot.icm()}}. Plot a compartment flow
diagram with \code{\link[=comp_plot]{comp_plot()}}.
}
\keyword{model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbose.R
\name{verbose.dcm}
\alias{verbose.dcm}
\title{Progress Print Module for Deterministic Compartmental Models}
\usage{
verbose.dcm(x, type, s = 1)
}
\arguments{
\item{x}{If the \code{type} is "startup", then an object of class
\code{control.dcm}, otherwise the main \code{df} object in \code{dcm}
runs.}

\item{type}{Progress type, either of "startup" for starting messages before
all runs, or "progress" for time step specific messages.}

\item{s}{Current run number, if type is "progress".}
}
\description{
This function prints progress from deterministic compartmental
models simulated with \code{\link[=dcm]{dcm()}} to the console.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.icm}
\alias{summary.icm}
\title{Summary Model Statistics}
\usage{
\method{summary}{icm}(object, at, digits = 3, ...)
}
\arguments{
\item{object}{An \code{EpiModel} object of class \code{icm}.}

\item{at}{Time step for model statistics.}

\item{digits}{Number of significant digits to print.}

\item{...}{Additional summary function arguments.}
}
\description{
Extracts and prints model statistics simulated with \code{icm}.
}
\details{
This function provides summary statistics for the main epidemiological
outcomes (state and transition size and prevalence) from an \code{icm} model.
Time-specific summary measures are provided, so it is necessary to input a
time of interest.
}
\examples{
\donttest{
## Stochastic ICM SI model with 3 simulations
param <- param.icm(inf.prob = 0.2, act.rate = 1)
init <- init.icm(s.num = 500, i.num = 1)
control <- control.icm(type = "SI", nsteps = 50,
                       nsims = 5, verbose = FALSE)
mod <- icm(param, init, control)
summary(mod, at = 25)
summary(mod, at = 50)
}

}
\seealso{
\code{\link[=icm]{icm()}}
}
\keyword{extract}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{set_vertex_attribute}
\alias{set_vertex_attribute}
\title{Set Vertex Attribute on Network Object}
\usage{
set_vertex_attribute(x, attrname, value, v = NULL)
}
\arguments{
\item{x}{An object of class network.}

\item{attrname}{The name of the attribute to set.}

\item{value}{A vector of values of the attribute to be set.}

\item{v}{IDs for the vertices whose attributes are to be altered.}
}
\value{
Returns an object of class \code{network}.
}
\description{
Sets a vertex attribute on an object of class \code{network}.
This function simplifies the related function in the
\code{network} package.
}
\details{
This function is used in \code{EpiModel} workflows to set vertex attributes
on an initialized empty network object (see \code{\link[=network_initialize]{network_initialize()}}).
}
\examples{
nw <- network_initialize(100)
nw <- set_vertex_attribute(nw, "age", runif(100, 15, 65))
nw

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.modules.R
\name{set_modules}
\alias{set_modules}
\title{Set the List of Modules}
\usage{
set_modules(dat, modules)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{modules}{A named list of modules to be run by the model}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Set the List of Modules
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netdx.R
\name{toggles_to_diss_stats}
\alias{toggles_to_diss_stats}
\title{Convert Matrix of Toggles to Dissolution and Duration Statistics}
\usage{
toggles_to_diss_stats(toggles, coef.diss, nsteps, nw, time.start = 0L)
}
\arguments{
\item{toggles}{A matrix of toggles, as produced by
\code{\link[=tedgelist_to_toggles]{tedgelist_to_toggles()}}.}

\item{coef.diss}{Dissolution coefficients used in the simulation.}

\item{nsteps}{Number of time steps in the simulation.}

\item{nw}{Network used in the simulation.}

\item{time.start}{Starting time for the simulation.}
}
\value{
Named list containing dissolution and duration statistics matrices
and other related information.
}
\description{
Convert Matrix of Toggles to Dissolution and Duration Statistics
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.nwupdate.R
\name{nwupdate.net}
\alias{nwupdate.net}
\title{Dynamic Network Updates}
\usage{
nwupdate.net(dat, at)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{Current time step.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function handles all calls to the network object contained
on the main \code{netsim_dat} object handled in \code{netsim}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.fn.modules.R
\name{make_module_list}
\alias{make_module_list}
\title{Populate the Module List After the Initialization Module is run}
\usage{
make_module_list(dat)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
Populate the Module List After the Initialization Module is run
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.simnet.R
\name{sim_nets_t1}
\alias{sim_nets_t1}
\title{Initialize Networks Used in netsim}
\usage{
sim_nets_t1(dat)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function initializes the networks used in
\code{\link[=netsim]{netsim()}}. The initial edge set for a given network
is obtained either from simulating the cross-sectional model
(if \code{edapprox == TRUE}) or from the \code{newnetwork}
element of the \code{netest} object (if
\code{edapprox == FALSE}). Once the initial edge sets are
determined, the first time step is simulated if
\code{resimulate.network == TRUE}, and all time steps are
simulated if \code{resimulate.network == FALSE}. Initializes the
\code{num(.g2)} epi fields used in
\code{\link[=edges_correct]{edges_correct()}} for computing edge coefficient
adjustments.
}
\keyword{internal}
\keyword{netUtils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ErgmTerms.R
\name{InitErgmTerm.absdiffnodemix}
\alias{InitErgmTerm.absdiffnodemix}
\alias{absdiffnodemix}
\title{Definition for absdiffnodemix ERGM Term}
\usage{
InitErgmTerm.absdiffnodemix(nw, arglist, ...)
}
\arguments{
\item{nw}{An object of class \code{network}.}

\item{arglist}{A list of arguments as specified in the \code{ergm.userterms}
package framework.}

\item{...}{Additional data passed into the function as specified in the
\code{ergm.userterms} package framework.}
}
\description{
This function defines and initializes the absdiffnodemix ERGM
term that allows for targeting homophily based on a non-binary
attribute (e.g., age) by combinations of a binary attribute
(e.g., race).
}
\details{
This ERGM user term was written to allow for age-based homophily in
partnership formation that is heterogeneous by race. The \code{absdiff}
component targets the distribution of age mixing on that continuous
variable, and the \code{nodemix} component differentiates this for
black-black, black-white, and white-white couples.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.dcm}
\alias{print.dcm}
\title{Print Method for DCM Objects}
\usage{
\method{print}{dcm}(x, ...)
}
\arguments{
\item{x}{An object of class \code{dcm}, from \code{\link[=dcm]{dcm()}}.}

\item{...}{Additional arguments (currently ignored).}
}
\description{
Prints a summary of a deterministic compartmental model object,
including the model type, number of runs, time steps, model parameters, and
output variable names.
}
\details{
Given a \code{dcm} object, \code{print.dcm} displays:
\itemize{
\item \strong{Simulation summary}: model class, model type (e.g., SI, SIR, SIS;
omitted for new/custom models), number of runs, number of time steps, and
number of groups.
\item \strong{Model parameters}: all parameters passed via \code{\link[=param.dcm]{param.dcm()}}, excluding
internal bookkeeping parameters (\code{groups}, \code{vital}).
\item \strong{Model output}: the names of all epidemic output variables stored in the
model.
}
}
